/*****************************************************************************************/

Prompt == FUNCTION SP_I_ACT ==
CREATE OR REPLACE FUNCTION SP_I_ACT
(
     p_name VARCHAR2
   , p_tsk_assignment_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_project_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_week_of DATE;
   h_count PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_POP_DT IS
         SELECT WEEK_OF
         FROM SUMMARY_TIMESHEETS
         WHERE PROJECT_ID = h_project_id
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         ORDER BY WEEK_OF;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_ACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_ACT'), '-- ' || SYSTIMESTAMP, ':r := SP_I_ACT(' || 
       '''' || p_name || ''''
          || ', ' || '''' || p_tsk_assignment_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_ACT', 32+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_ACT(33) : activities.sql(36) : ' || SYSTIMESTAMP);
   END IF; 

   If p_tsk_assignment_id IS NULL Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_I_ACT', 37+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_I_ACT(38) : activities.sql(41) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ELEMENT_ID_REQUIRED);
   End If;
   
   h_tsk_assignment_id := p_tsk_assignment_id;
   
   BEGIN
      SELECT PROJECT_ID INTO h_project_id
      FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ACT', 53+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ACT(54) : activities.sql(57) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_I_ACT', 60+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_I_ACT(61) : activities.sql(64) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_project_id);
   End If;
   
   BEGIN
      SELECT COUNT(ELEMENT_ID) INTO h_count
      FROM ACTIVITIES
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ACT', 75+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ACT(76) : activities.sql(79) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_uniqueID := SYS_GUID();
   BEGIN
      INSERT INTO ACTIVITIES(
      ELEMENT_ID,
      TSK_ASSIGNMENT_ID,
      ELEMENT_NAME,
      REC_USER)
      VALUES( h_uniqueID,
      h_tsk_assignment_id,
      p_name,
      p_rec_user );
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ACT', 101+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ACT(102) : activities.sql(105) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --Populate detail timesheet for all weeks (With effort)
   
   OPEN C_POP_DT;
   
   LOOP
      FETCH C_POP_DT INTO h_week_of;
      IF C_POP_DT%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      If Coalesce(h_count, 0) = 0 Then
         BEGIN
            INSERT INTO DETAIL_TIMESHEETS (
            ACTIVITY_ID, WEEK_OF,
            MONDAY_REGULAR, MONDAY_SPECIAL,
            TUESDAY_REGULAR, TUESDAY_SPECIAL,
            WEDNESDAY_REGULAR, WEDNESDAY_SPECIAL,
            THURSDAY_REGULAR, THURSDAY_SPECIAL,
            FRIDAY_REGULAR, FRIDAY_SPECIAL,
            SATURDAY_REGULAR, SATURDAY_SPECIAL,
            SUNDAY_REGULAR, SUNDAY_SPECIAL,
            BILLABLE, REC_USER, REC_DATETIME )
            SELECT  h_uniqueID, ST.WEEK_OF,
            ST.MONDAY_REGULAR, ST.MONDAY_SPECIAL,
            ST.TUESDAY_REGULAR, ST.TUESDAY_SPECIAL,
            ST.WEDNESDAY_REGULAR, ST.WEDNESDAY_SPECIAL,
            ST.THURSDAY_REGULAR, ST.THURSDAY_SPECIAL,
            ST.FRIDAY_REGULAR, ST.FRIDAY_SPECIAL,
            ST.SATURDAY_REGULAR, ST.SATURDAY_SPECIAL,
            ST.SUNDAY_REGULAR, ST.SUNDAY_SPECIAL,
            TA.BILLABLE, p_rec_user, SYSTIMESTAMP
            FROM SUMMARY_TIMESHEETS ST
            JOIN TASK_ASSIGNMENTS TA
            ON TA.PROJECT_ID = ST.PROJECT_ID
            AND TA.TSK_ASSIGNMENT_ID = ST.TSK_ASSIGNMENT_ID
            WHERE ST.PROJECT_ID = h_project_id
            AND ST.TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND WEEK_OF = h_week_of;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ACT', 159+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ACT(160) : activities.sql(163) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            INSERT INTO DETAIL_TIMESHEETS (
            ACTIVITY_ID, WEEK_OF,
            BILLABLE, REC_USER, REC_DATETIME )
            SELECT  h_uniqueID, ST.WEEK_OF,
            TA.BILLABLE, p_rec_user, SYSTIMESTAMP
            FROM SUMMARY_TIMESHEETS ST
            JOIN TASK_ASSIGNMENTS TA
            ON TA.PROJECT_ID = ST.PROJECT_ID
            AND TA.TSK_ASSIGNMENT_ID = ST.TSK_ASSIGNMENT_ID
            WHERE ST.PROJECT_ID = h_project_id
            AND ST.TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND WEEK_OF = h_week_of;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ACT', 187+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ACT(188) : activities.sql(191) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_POP_DT ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ACT', 202+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ACT(203) : activities.sql(206) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_ACT', 208+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_ACT(209) : activities.sql(212) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ACT', 215+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ACT(216) : activities.sql(219) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ACT', 227+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ACT(228) : activities.sql(231) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_ACT;
ALTER FUNCTION SP_I_ACT COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION SP_U_ACT ==
CREATE OR REPLACE FUNCTION SP_U_ACT
(
     p_id VARCHAR2
   , p_tsk_assignment_id VARCHAR2
   , p_name VARCHAR2
   , p_billable PLS_INTEGER
   , p_timecode_1_id VARCHAR2
   , p_timecode_1_name VARCHAR2
   , p_timecode_2_id VARCHAR2
   , p_timecode_2_name VARCHAR2
   , p_sched_work PLS_INTEGER
   , p_etc PLS_INTEGER
   , p_milestone_percent PLS_INTEGER
   , p_edpercent PLS_INTEGER
   , p_week_of VARCHAR2
   , p_monday_regular PLS_INTEGER
   , p_monday_special PLS_INTEGER
   , p_tuesday_regular PLS_INTEGER
   , p_tuesday_special PLS_INTEGER
   , p_wednesday_regular PLS_INTEGER
   , p_wednesday_special PLS_INTEGER
   , p_thursday_regular PLS_INTEGER
   , p_thursday_special PLS_INTEGER
   , p_friday_regular PLS_INTEGER
   , p_friday_special PLS_INTEGER
   , p_saturday_regular PLS_INTEGER
   , p_saturday_special PLS_INTEGER
   , p_sunday_regular PLS_INTEGER
   , p_sunday_special PLS_INTEGER
   , p_notes VARCHAR2
   , p_pda_flag VARCHAR2
   , p_etc_modified VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_work_todate PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_ACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_ACT'), '-- ' || SYSTIMESTAMP, ':r := SP_U_ACT(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_tsk_assignment_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || COALESCE(TO_CHAR(p_billable), 'NULL')
          || ', ' || '''' || p_timecode_1_id || ''''
          || ', ' || '''' || p_timecode_1_name || ''''
          || ', ' || '''' || p_timecode_2_id || ''''
          || ', ' || '''' || p_timecode_2_name || ''''
          || ', ' || COALESCE(TO_CHAR(p_sched_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_etc), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_milestone_percent), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_edpercent), 'NULL')
          || ', ' || '''' || p_week_of || ''''
          || ', ' || COALESCE(TO_CHAR(p_monday_regular), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_monday_special), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_tuesday_regular), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_tuesday_special), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_wednesday_regular), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_wednesday_special), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_thursday_regular), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_thursday_special), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_friday_regular), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_friday_special), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_saturday_regular), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_saturday_special), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_sunday_regular), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_sunday_special), 'NULL')
          || ', ' || '''' || p_notes || ''''
          || ', ' || '''' || p_pda_flag || ''''
          || ', ' || '''' || p_etc_modified || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_ACT', 77+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_ACT(78) : activities.sql(325) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) AND (p_week_of IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_ACT', 82+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_ACT(83) : activities.sql(330) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ELEMENT_ID_REQUIRED);
   End If;
   
   h_id := p_id;
   
   --check existence
   BEGIN
      SELECT REC_STATUS
      INTO h_rec_status
      FROM DETAIL_TIMESHEETS
      WHERE ACTIVITY_ID = p_id
      AND WEEK_OF = TO_DATE(SubStr(p_week_of, 1, 10));
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT', 102+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT(103) : activities.sql(350) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      --go insert
      BEGIN
         INSERT INTO DETAIL_TIMESHEETS (
         ACTIVITY_ID,
         WEEK_OF,
         BILLABLE,
         MONDAY_REGULAR,
         MONDAY_SPECIAL,
         TUESDAY_REGULAR,
         TUESDAY_SPECIAL,
         WEDNESDAY_REGULAR,
         WEDNESDAY_SPECIAL,
         THURSDAY_REGULAR,
         THURSDAY_SPECIAL,
         FRIDAY_REGULAR,
         FRIDAY_SPECIAL,
         SATURDAY_REGULAR,
         SATURDAY_SPECIAL,
         SUNDAY_REGULAR,
         SUNDAY_SPECIAL,
         PDA_FLAG,
         REC_USER,
         NOTES)
         VALUES (
         p_id,
         p_week_of,
         p_billable,
         p_monday_regular,
         p_monday_special,
         p_tuesday_regular,
         p_tuesday_special,
         p_wednesday_regular,
         p_wednesday_special,
         p_thursday_regular,
         p_thursday_special,
         p_friday_regular,
         p_friday_special,
         p_saturday_regular,
         p_saturday_special,
         p_sunday_regular,
         p_sunday_special,
         CASE WHEN p_pda_flag IS NOT NULL THEN p_pda_flag ELSE 'P' END,
         p_rec_user,
         p_notes );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT', 162+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT(163) : activities.sql(410) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         UPDATE DETAIL_TIMESHEETS SET
         BILLABLE = p_billable,
         MONDAY_REGULAR = p_monday_regular,
         MONDAY_SPECIAL = p_monday_special,
         TUESDAY_REGULAR = p_tuesday_regular,
         TUESDAY_SPECIAL = p_tuesday_special,
         WEDNESDAY_REGULAR = p_wednesday_regular,
         WEDNESDAY_SPECIAL = p_wednesday_special,
         THURSDAY_REGULAR = p_thursday_regular,
         THURSDAY_SPECIAL = p_thursday_special,
         FRIDAY_REGULAR = p_friday_regular,
         FRIDAY_SPECIAL = p_friday_special,
         SATURDAY_REGULAR = p_saturday_regular,
         SATURDAY_SPECIAL = p_saturday_special,
         SUNDAY_REGULAR = p_sunday_regular,
         SUNDAY_SPECIAL = p_sunday_special,
         PDA_FLAG = CASE WHEN p_pda_flag IS NOT NULL THEN p_pda_flag ELSE 'P' END,
         REC_USER = p_rec_user,
         REC_STATUS = 'U',
         NOTES = p_notes
         WHERE ACTIVITY_ID = p_id
         AND WEEK_OF = TO_DATE(SubStr(p_week_of, 1, 10));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT', 200+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT(201) : activities.sql(448) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   BEGIN
      SELECT
      COALESCE( SUM(MONDAY_REGULAR + MONDAY_SPECIAL + TUESDAY_REGULAR + TUESDAY_SPECIAL + WEDNESDAY_REGULAR + WEDNESDAY_SPECIAL + THURSDAY_REGULAR + THURSDAY_SPECIAL + FRIDAY_REGULAR + FRIDAY_SPECIAL + SATURDAY_REGULAR + SATURDAY_SPECIAL + SUNDAY_REGULAR + SUNDAY_SPECIAL), 0)
      INTO
      h_work_todate
      FROM DETAIL_TIMESHEETS
      WHERE ACTIVITY_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT', 219+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT(220) : activities.sql(467) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE ACTIVITIES SET
      ELEMENT_NAME = p_name,
      BILLABLE = p_billable,
      TIMECODE_1_ID = p_timecode_1_id,
      TIMECODE_1_NAME = p_timecode_1_name,
      TIMECODE_2_ID = p_timecode_2_id,
      TIMECODE_2_NAME = p_timecode_2_name,
      EFFORT_ASSIGNMENT = p_sched_work,
      WORK_TODATE = h_work_todate,
      ETC = p_etc,
      ETC_MODIFIED = p_etc_modified,
      MILESTONE_PERCENT = p_milestone_percent,
      EDPERCENT = p_edpercent,
      REC_USER = p_rec_user,
      REC_STATUS = 'U'
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT', 251+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT(252) : activities.sql(499) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      BEGIN
         INSERT INTO ACTIVITIES(
         ELEMENT_ID,
         TSK_ASSIGNMENT_ID,
         ELEMENT_NAME,
         BILLABLE,
         TIMECODE_1_ID,
         TIMECODE_1_NAME,
         TIMECODE_2_ID,
         TIMECODE_2_NAME,
         EFFORT_ASSIGNMENT,
         WORK_TODATE,
         ETC,
         ETC_MODIFIED,
         MILESTONE_PERCENT,
         EDPERCENT,
         REC_USER,
         REC_STATUS )
         VALUES(
         p_id,
         p_tsk_assignment_id,
         p_name,
         p_billable,
         p_timecode_1_id,
         p_timecode_1_name,
         p_timecode_2_id,
         p_timecode_2_name,
         p_sched_work,
         h_work_todate,
         p_etc,
         p_etc_modified,
         p_milestone_percent,
         p_edpercent,
         p_rec_user,
         'I');
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT', 302+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT(303) : activities.sql(550) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_ACT', 309+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_ACT(310) : activities.sql(557) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT', 316+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT(317) : activities.sql(564) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT', 328+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT(329) : activities.sql(576) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_ACT;
ALTER FUNCTION SP_U_ACT COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION SP_D_ACT ==
CREATE OR REPLACE FUNCTION SP_D_ACT
(
     p_id VARCHAR2
   , p_week_of VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_ACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_ACT'), '-- ' || SYSTIMESTAMP, ':r := SP_D_ACT(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_week_of || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_ACT', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_ACT(21) : activities.sql(613) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      DELETE FROM DETAIL_TIMESHEETS
      WHERE ACTIVITY_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ACT', 36+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ACT(37) : activities.sql(629) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      DELETE FROM ACTIVITIES
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ACT', 54+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ACT(55) : activities.sql(647) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --The broker assume there is always one default activity record in the database
   --Which is only false when the WorkToDate is greater than 0
   --Hide the problem for now!
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_ACT', 67+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_ACT(68) : activities.sql(660) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(0, p_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ACT', 74+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ACT(75) : activities.sql(667) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ACT', 86+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ACT(87) : activities.sql(679) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_ACT;
ALTER FUNCTION SP_D_ACT COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION SP_S_ACT ==
CREATE OR REPLACE FUNCTION SP_S_ACT
(
     p_tsk_assignment_id VARCHAR2
   , p_week_of VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
      -- Resultset Cursor Declaration
      SELECT_READ1 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SELECT_READ2 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_ACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_ACT'), '-- ' || SYSTIMESTAMP, ':r := SP_S_ACT(' || 
       '''' || p_tsk_assignment_id || ''''
          || ', ' || '''' || p_week_of || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ACT', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_ACT(25) : activities.sql(720) : ' || SYSTIMESTAMP);
   END IF; 

   If Coalesce(p_bitflag, 0) = 0 Then
      OPEN SELECT_READ1 FOR SELECT
      0 as LEVEL_ID, 0 as TYPE_ID, 0 as SCHED_WORK, 0 as MILESTONE_PERCENT,
      p_tsk_assignment_id || 'SAF' as ELEMENT_ID, '' as ELEMENT_NAME,
      0 as CHILD_COUNT, p_tsk_assignment_id as TSK_ASSIGNMENT_ID,
      0 as DELETED_COUNT, 0 as EDPERCENT, 0 as ETC, 0 as REMAINING,
      '' as TIMECODE_1_ID, '' as TIMECODE_1_NAME, '' as TIMECODE_2_ID, '' as TIMECODE_2_NAME,
      0 as WORK_TODATE, 0 as EDPCOMPLETED, 0 as PERCENT_SPENT,
      TO_DATE(SubStr(p_week_of, 1, 10)) as WEEK_OF,
      0 as MONDAY_REGULAR, 0 as MONDAY_SPECIAL, 0 as TUESDAY_REGULAR, 0 as TUESDAY_SPECIAL,
      0 as WEDNESDAY_REGULAR, 0 as WEDNESDAY_SPECIAL, 0 as THURSDAY_REGULAR, 0 as THURSDAY_SPECIAL,
      0 as FRIDAY_REGULAR, 0 AS FRIDAY_SPECIAL, 0 as SATURDAY_REGULAR, 0 as SATURDAY_SPECIAL,
      0 as SUNDAY_REGULAR, 0 as SUNDAY_SPECIAL, '' as REC_USER, 'U' as REC_STATUS,
      TO_TIMESTAMP('1899-12-30 8:00:00') as REC_DATETIME,
      '' as NOTES,
      TO_TIMESTAMP('1899-12-30 8:00:00') as CREATED_DT,
      1 as BILLABLE, 'P' as PDA_FLAG, 'N' as ETC_MODIFIED
      FROM DUAL
      UNION
      SELECT
      1 AS LEVEL_ID, 212 AS TYPE_ID, ACT.EFFORT_ASSIGNMENT AS SCHED_WORK,
      ACT.MILESTONE_PERCENT as MILESTONE_PERCENT , ACT.ELEMENT_ID as ELEMENT_ID, ACT.ELEMENT_NAME  as ELEMENT_NAME,
      0 AS CHILD_COUNT, ACT.TSK_ASSIGNMENT_ID  as TSK_ASSIGNMENT_ID, 0 AS DELETED_COUNT, ACT.EDPERCENT as EDPERCENT,
      ACT.ETC  as ETC, ACT.EFFORT_ASSIGNMENT - ACT.WORK_TODATE AS REMAINING, ACT.TIMECODE_1_ID AS TIMECODE_1_ID,
      ACT.TIMECODE_1_NAME AS TIMECODE_1_NAME, ACT.TIMECODE_2_ID AS TIMECODE_2_ID, ACT.TIMECODE_2_NAME AS TIMECODE_2_NAME,
      ACT.WORK_TODATE AS WORK_TODATE, CASE WHEN ACT.MILESTONE_PERCENT = 100 THEN 0 ELSE 1 END AS EDPCOMPLETED,
      0 AS PERCENT_SPENT,COALESCE(DT.WEEK_OF,TO_DATE(SubStr(p_week_of, 1, 10))) AS WEEK_OF,
      DT.MONDAY_REGULAR AS MONDAY_REGULAR, DT.MONDAY_SPECIAL AS MONDAY_SPECIAL,
      DT.TUESDAY_REGULAR AS TUESDAY_REGULAR, DT.TUESDAY_SPECIAL AS TUESDAY_SPECIAL, DT.WEDNESDAY_REGULAR AS WEDNESDAY_REGULAR,
      DT.WEDNESDAY_SPECIAL AS WEDNESDAY_SPECIAL, DT.THURSDAY_REGULAR AS THURSDAY_REGULAR, DT.THURSDAY_SPECIAL AS THURSDAY_SPECIAL,
      DT.FRIDAY_REGULAR AS FRIDAY_REGULAR, DT.FRIDAY_SPECIAL AS FRIDAY_SPECIAL, DT.SATURDAY_REGULAR AS SATURDAY_REGULAR,
      DT.SATURDAY_SPECIAL AS SATURDAY_SPECIAL, DT.SUNDAY_REGULAR AS SUNDAY_REGULAR,
      DT.SUNDAY_SPECIAL AS SUNDAY_SPECIAL, DT.REC_USER AS REC_USER, DT.REC_STATUS AS REC_STATUS,
      TO_DATE(SubStr(DT.REC_DATETIME, 1, 10)) AS REC_DATETIME,
      DT.NOTES AS NOTES,
      ACT.REC_DATETIME AS CREATED_DT,
      ACT.BILLABLE AS BILLABLE,
      DT.PDA_FLAG AS PDA_FLAG, ACT.ETC_MODIFIED AS ETC_MODIFIED
      FROM ACTIVITIES ACT
      LEFT OUTER JOIN DETAIL_TIMESHEETS DT ON DT.ACTIVITY_ID = ACT.ELEMENT_ID
      AND DT.WEEK_OF = TO_DATE(SubStr(p_week_of, 1, 10))
      WHERE ACT.TSK_ASSIGNMENT_ID = p_tsk_assignment_id 
      ORDER BY LEVEL_ID, CREATED_DT;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ACT', 71+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_ACT(72) : activities.sql(767) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_READ1;
   ElsIf Coalesce(p_bitflag, 0) = 1 Then
      OPEN SELECT_READ2 FOR SELECT 1 AS LEVEL_ID, 212 AS TYPE_ID, ACT.EFFORT_ASSIGNMENT AS SCHED_WORK,
      ACT.MILESTONE_PERCENT, ACT.ELEMENT_ID, ACT.ELEMENT_NAME,
      0 AS CHILD_COUNT, ACT.TSK_ASSIGNMENT_ID, 0 AS DELETED_COUNT, ACT.EDPERCENT,
      ACT.ETC, ACT.EFFORT_ASSIGNMENT - ACT.WORK_TODATE AS REMAINING, ACT.TIMECODE_1_ID,
      ACT.TIMECODE_1_NAME, ACT.TIMECODE_2_ID, ACT.TIMECODE_2_NAME, ACT.WORK_TODATE,
      CASE WHEN ACT.MILESTONE_PERCENT = 100 THEN 0 ELSE 1 END AS EDPCOMPLETED,
      0 AS PERCENT_SPENT,
      COALESCE(DT.WEEK_OF, TO_DATE(SubStr(p_week_of, 1, 10))) AS WEEK_OF,
      DT.MONDAY_REGULAR,
      DT.MONDAY_SPECIAL, DT.TUESDAY_REGULAR, DT.TUESDAY_SPECIAL, DT.WEDNESDAY_REGULAR,
      DT.WEDNESDAY_SPECIAL, DT.THURSDAY_REGULAR, DT.THURSDAY_SPECIAL, DT.FRIDAY_REGULAR,
      DT.FRIDAY_SPECIAL, DT.SATURDAY_REGULAR, DT.SATURDAY_SPECIAL, DT.SUNDAY_REGULAR,
      DT.SUNDAY_SPECIAL, DT.REC_USER, DT.REC_STATUS, DT.REC_DATETIME, DT.NOTES,
      ACT.REC_DATETIME AS CREATED_DT, ACT.BILLABLE, DT.PDA_FLAG, ACT.ETC_MODIFIED
      FROM ACTIVITIES ACT
      LEFT JOIN DETAIL_TIMESHEETS DT
      ON DT.ACTIVITY_ID = ACT.ELEMENT_ID
      AND DT.WEEK_OF = TO_DATE(SubStr(p_week_of, 1, 10))
      WHERE ACT.ELEMENT_ID = p_tsk_assignment_id
      ORDER BY LEVEL_ID, CREATED_DT;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ACT', 96+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_ACT(97) : activities.sql(792) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_READ2;
   End If;
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ACT', 104+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ACT(105) : activities.sql(800) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ACT', 116+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ACT(117) : activities.sql(812) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_ACT;
ALTER FUNCTION SP_S_ACT COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION SP_U_ACT_ETC ==
CREATE OR REPLACE FUNCTION SP_U_ACT_ETC
(
     p_tsk_assignment_id VARCHAR2
   , p_etc_diff PLS_INTEGER
   , p_sched_work PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_ea NUMBER := 0;
   h_ratio NUMBER := 0;
-- Declare Cursors
      CURSOR C_ETC IS
         SELECT ELEMENT_ID, EFFORT_ASSIGNMENT
         FROM ACTIVITIES
         WHERE TSK_ASSIGNMENT_ID = p_tsk_assignment_id
         FOR UPDATE OF ETC;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_ACT_ETC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_ACT_ETC'), '-- ' || SYSTIMESTAMP, ':r := SP_U_ACT_ETC(' || 
       '''' || p_tsk_assignment_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_etc_diff), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_sched_work), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_ACT_ETC', 29+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_ACT_ETC(30) : activities.sql(858) : ' || SYSTIMESTAMP);
   END IF; 

   OPEN C_ETC;
   
   LOOP
      FETCH C_ETC INTO h_id, h_ea;
      IF C_ETC%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      h_ratio := Coalesce(h_ea, 0) / Coalesce(p_sched_work, 0);
      
      BEGIN
         UPDATE ACTIVITIES
         SET ETC = ETC + (p_etc_diff * h_ratio)
         WHERE CURRENT OF C_ETC;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT_ETC', 61+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT_ETC(62) : activities.sql(890) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE C_ETC ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT_ETC', 75+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT_ETC(76) : activities.sql(904) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_ACT_ETC', 81+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_ACT_ETC(82) : activities.sql(910) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_tsk_assignment_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_tsk_assignment_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT_ETC', 88+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT_ETC(89) : activities.sql(917) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT_ETC', 100+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT_ETC(101) : activities.sql(929) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_ACT_ETC;
ALTER FUNCTION SP_U_ACT_ETC COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION SP_U_ACT_MP ==
CREATE OR REPLACE FUNCTION SP_U_ACT_MP
(
     p_tsk_assignment_id VARCHAR2
   , p_milestone_percent_diff PLS_INTEGER
   , p_sched_work PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_ea NUMBER := 0;
   h_ratio NUMBER := 0;
-- Declare Cursors
      CURSOR C_MP IS
         SELECT ELEMENT_ID, EFFORT_ASSIGNMENT
         FROM ACTIVITIES
         WHERE TSK_ASSIGNMENT_ID = p_tsk_assignment_id
         FOR UPDATE OF MILESTONE_PERCENT;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_ACT_MP') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_ACT_MP'), '-- ' || SYSTIMESTAMP, ':r := SP_U_ACT_MP(' || 
       '''' || p_tsk_assignment_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_milestone_percent_diff), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_sched_work), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_ACT_MP', 29+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_ACT_MP(30) : activities.sql(975) : ' || SYSTIMESTAMP);
   END IF; 

   OPEN C_MP;
   
   LOOP
      FETCH C_MP INTO h_id, h_ea;
      IF C_MP%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      h_ratio := Coalesce(h_ea, 0) / Coalesce(p_sched_work, 0);
      
      BEGIN
         UPDATE ACTIVITIES
         SET MILESTONE_PERCENT = MILESTONE_PERCENT + (p_milestone_percent_diff * h_ratio)
         WHERE CURRENT OF C_MP;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT_MP', 61+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT_MP(62) : activities.sql(1007) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE C_MP ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT_MP', 75+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT_MP(76) : activities.sql(1021) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_ACT_MP', 81+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_ACT_MP(82) : activities.sql(1027) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_tsk_assignment_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_tsk_assignment_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT_MP', 88+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT_MP(89) : activities.sql(1034) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ACT_MP', 100+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ACT_MP(101) : activities.sql(1046) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_ACT_MP;
ALTER FUNCTION SP_U_ACT_MP COMPILE DEBUG;

