/****************************************************************************************/

Prompt == FUNCTION FC_I_CONTACT ==
CREATE OR REPLACE FUNCTION FC_I_CONTACT
(
     h_name IN OUT NOCOPY VARCHAR2
   , h_uniqueID IN OUT NOCOPY VARCHAR2
   , h_parent_id IN OUT NOCOPY VARCHAR2
   , sh_h_type_id PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := sh_h_type_id;
   h_max_rank PLS_INTEGER := 0;
   h_rankBase64 VARCHAR2(8);
   h_rec_status VARCHAR2(1);
   h_rank VARCHAR2(200);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_I_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_I_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := FC_I_CONTACT(' || 
       '''' || h_name || ''''
          || ', ' || '''' || h_uniqueID || ''''
          || ', ' || '''' || h_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_type_id), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_CONTACT', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_I_CONTACT(26) : contacts.sql(29) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If h_uniqueID IS NULL Then
      h_uniqueID := SYS_GUID();
   End If;
   
   -- Validate parent_id
   If h_parent_id IS NULL Then
      -- Get the max sibling level from the root
      PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_CONTACTS', h_max_rank);
      
      --Get Base 64
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      -- insert-select
      BEGIN
         INSERT INTO TMT_CONTACTS(RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER, ASSIGN_NAMES)
         SELECT
         RTRIM(COALESCE(RANK, '')) || h_rankBase64,
         h_parent_id,
         h_uniqueID,
         h_name,
         h_type_id,
         LEVEL_ID + 1,
         h_rec_user,
         h_rec_user || ';'
         FROM TMT_CONTACTS WHERE ELEMENT_ID IS NuLL;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_CONTACT', 65+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_CONTACT(66) : contacts.sql(69) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      -- Validate parent_id
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'FC_I_CONTACT', 72+1) THEN 
         LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: FC_I_CONTACT(73) : contacts.sql(76) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_CONTACTS' || '''' || ' ID: ' || '''' || h_parent_id || '''');
      END IF; 
      PMODEFINES.PMO_VALIDATE_RECORD('TMT_CONTACTS', h_parent_id, h_rec_status);
      
      -- Get next sibling count
      PMODEFINES.PMO_GET_MAX_RANK('TMT_CONTACTS', h_parent_id, h_max_rank);
      
      --Get Base 64
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      -- insert-select
      BEGIN
         INSERT INTO TMT_CONTACTS(RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER, ASSIGN_NAMES)
         SELECT
         RTRIM(Rank) || h_rankBase64,
         h_parent_id,
         h_uniqueID,
         h_name,
         h_type_id,
         LEVEL_ID + 1,
         h_rec_user,
         h_rec_user || ';'
         FROM TMT_CONTACTS WHERE ELEMENT_ID = h_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_CONTACT', 105+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_CONTACT(106) : contacts.sql(109) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_CONTACT', 112+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_I_CONTACT(113) : contacts.sql(116) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_I_CONTACT;
ALTER FUNCTION FC_I_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_I_CONTACT ==
CREATE OR REPLACE FUNCTION SP_I_CONTACT
(
     p_name VARCHAR2
   , p_parent_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_name VARCHAR2(255);
   h_parent_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
   h_uniqueID VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_I_CONTACT(' || 
       '''' || p_name || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_CONTACT', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_CONTACT(26) : contacts.sql(152) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_name := p_name;
   h_parent_id := p_parent_id;
   h_type_id := p_type_id;
   h_rec_user := p_rec_user;
   h_uniqueID := NULL;
   
   PMO_DB2M.SC_STATE := FC_I_CONTACT(h_name, h_uniqueID, h_parent_id, h_type_id, h_rec_user);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_CONTACT', 39+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_CONTACT(40) : contacts.sql(166) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTACT', 46+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTACT(47) : contacts.sql(173) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTACT', 58+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTACT(59) : contacts.sql(185) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_CONTACT;
ALTER FUNCTION SP_I_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_CONTACT ==
CREATE OR REPLACE FUNCTION SP_U_CONTACT
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_name VARCHAR2
   , p_description VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_U_CONTACT(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || '''' || p_description || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTACT', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_CONTACT(23) : contacts.sql(224) : ' || SYSTIMESTAMP);
   END IF; 

   -- Validate id
   If p_id IS NULL Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_CONTACT', 28+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_CONTACT(29) : contacts.sql(230) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ELEMENT_ID_REQUIRED);
   End If;
   
   -- Execute Update Statement
   If p_parent_id IS NULL Then
      BEGIN
         UPDATE TMT_CONTACTS
         SET ELEMENT_NAME = p_name,
         DESCRIPTION  = p_description,
         REC_STATUS   = 'U',
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER     = p_rec_user
         WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTACT', 53+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTACT(54) : contacts.sql(255) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         UPDATE TMT_CONTACTS
         SET ELEMENT_NAME = p_name,
         DESCRIPTION  = p_description,
         REC_STATUS   = 'U',
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER     = p_rec_user
         WHERE ELEMENT_ID = p_id AND PARENT_ID = p_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTACT', 76+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTACT(77) : contacts.sql(278) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_CONTACT', 83+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_CONTACT(84) : contacts.sql(285) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTACT', 90+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTACT(91) : contacts.sql(292) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTACT', 102+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTACT(103) : contacts.sql(304) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_CONTACT;
ALTER FUNCTION SP_U_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_D_CONTACT ==
CREATE OR REPLACE FUNCTION SP_D_CONTACT
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_exists VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_rank VARCHAR2(200);
   h_private_count PLS_INTEGER := 0;
   h_dependency_count PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CC1 IS
         SELECT ELEMENT_ID FROM TMT_CONTACTS
         WHERE  ROWNUM <= 1 AND  RANK LIKE h_rank || '%' AND PRIVATE_FLAG = 'Y';

      CURSOR CC2 IS
         SELECT ELEMENT_ID FROM TMT_CONTACTS
         WHERE  ROWNUM <= 1 AND  RANK LIKE h_rank || '%' AND TYPE_ID = PMO_CNST.SC_RESOURCE;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_D_CONTACT(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTACT', 35+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_CONTACT(36) : contacts.sql(356) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate id
   h_id := p_id;
   h_rec_status := p_del_flag;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(256, 'SP_D_CONTACT', 44+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_DELETE: SP_D_CONTACT(45) : contacts.sql(365) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_CONTACTS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_VALIDATE_DELETE('TMT_CONTACTS', h_id, h_rec_status);
   
   -- Validate parent id
   If (p_parent_id IS NULL) Then
      -- Find Necessary Info
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_CONTACTS WHERE ELEMENT_ID = p_id AND PARENT_ID IS NuLL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 59+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(60) : contacts.sql(380) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --strcpy(h_rec_status, "");
      h_rec_status := '';
   Else
      h_parent_id := p_parent_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_D_CONTACT', 68+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_D_CONTACT(69) : contacts.sql(389) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_CONTACTS' || '''' || ' ID: ' || '''' || h_parent_id || '''');
      END IF; 
      PMODEFINES.PMO_VALIDATE_EXISTENCE('TMT_CONTACTS', h_parent_id, h_rec_status);
      
      If SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_CONTACT', 74+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_CONTACT(75) : contacts.sql(395) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.PARENT_MARKED_AS_DELETED);
      End If;
      
      -- Find Necessary Info
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_CONTACTS WHERE ELEMENT_ID = p_id AND PARENT_ID = p_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 88+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(89) : contacts.sql(409) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   OPEN CC1;
   
   FETCH CC1 INTO h_exists;
   IF CC1%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      h_private_count := 1;
   Else
      h_private_count := 0;
   End If;
   
   BEGIN
      CLOSE CC1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 117+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(118) : contacts.sql(438) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN CC2;
   
   FETCH CC2 INTO h_exists;
   IF CC2%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      h_dependency_count := 1;
   Else
      h_dependency_count := 0;
   End If;
   
   BEGIN
      CLOSE CC2 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 145+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(146) : contacts.sql(466) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If SubStr(p_del_flag, 1, 1) = '?' Then
      If (Coalesce(h_private_count, 0) <> 0) OR (Coalesce(h_dependency_count, 0) <> 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_CONTACT', 153+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_CONTACT(154) : contacts.sql(474) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_DELETE_RECORD), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.CANNOT_DELETE_RECORD, p_id);
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_CONTACT', 158+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_CONTACT(159) : contacts.sql(479) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR(0, p_id);
      End If;
   End If;
   
   -- Execute Delete Statement
   If SubStr(p_del_flag, 1, 1) = 'X' Then
      If Coalesce(h_private_count, 0) = 0 Then
         -- Delete Tree
         PMODEFINES.PMO_PMO_VERIFY_RANK(h_rank, 0);
         BEGIN
            DELETE FROM TMT_CONTACTS WHERE RANK LIKE h_rank || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 181+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(182) : contacts.sql(502) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         -- Update Parent's Hierarchy Info
         If SubStr(h_rec_status, 1, 1) = 'D' Then
            If p_parent_id IS NULL Then
               BEGIN
                  UPDATE TMT_CONTACTS SET DELETED_COUNT = DELETED_COUNT - 1
                  WHERE ELEMENT_ID IS NuLL;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 202+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(203) : contacts.sql(523) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TMT_CONTACTS SET DELETED_COUNT = DELETED_COUNT - 1
                  WHERE ELEMENT_ID = h_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 220+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(221) : contacts.sql(541) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            If p_parent_id IS NULL Then
               BEGIN
                  UPDATE TMT_CONTACTS SET CHILD_COUNT = CHILD_COUNT - 1
                  WHERE ELEMENT_ID IS NuLL;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 240+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(241) : contacts.sql(561) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TMT_CONTACTS SET CHILD_COUNT = CHILD_COUNT - 1
                  WHERE ELEMENT_ID = h_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 258+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(259) : contacts.sql(579) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_CONTACT', 266+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_CONTACT(267) : contacts.sql(587) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PRIVATE_CHILD_EXISTS), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.PRIVATE_CHILD_EXISTS, p_id);
      End If;
   Else
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_CONTACT', 273+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_CONTACT(274) : contacts.sql(594) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.PARENT_MARKED_AS_DELETED);
      End If;
      
      If Coalesce(h_private_count, 0) = 0 Then
         -- Update Parent's Hierarchy Info
         If p_parent_id IS NULL Then
            BEGIN
               UPDATE TMT_CONTACTS SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
               WHERE ELEMENT_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 294+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(295) : contacts.sql(615) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_CONTACTS SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 312+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(313) : contacts.sql(633) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         -- Lock Tree
         PMODEFINES.PMO_PMO_VERIFY_RANK(h_rank, 0);
         BEGIN
            UPDATE TMT_CONTACTS SET
            REC_STATUS   = p_del_flag,
            REC_DATETIME = SYSTIMESTAMP,
            REC_USER     = p_rec_user
            WHERE RANK LIKE h_rank || '%' AND REC_STATUS IN ('I','U');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 336+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(337) : contacts.sql(657) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_CONTACT', 342+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_CONTACT(343) : contacts.sql(663) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PRIVATE_CHILD_EXISTS), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.PRIVATE_CHILD_EXISTS, p_id);
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_CONTACT', 349+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_CONTACT(350) : contacts.sql(670) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 356+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(357) : contacts.sql(677) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 368+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(369) : contacts.sql(689) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_CONTACT;
ALTER FUNCTION SP_D_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_CONTACT ==
CREATE OR REPLACE FUNCTION SP_S_CONTACT
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
   h_sql VARCHAR2(2048);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_S_CONTACT(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTACT', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_CONTACT(21) : contacts.sql(726) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If (p_bitflag IS NOT NULL) AND (Coalesce(p_bitflag, 0) = 2) Then
      h_sql := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, ';
      h_sql := h_sql || 
                  'CASE WHEN TYPE_ID = 13 THEN (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE T.ELEMENT_NAME END AS ELEMENT_NAME, ';
      h_sql := h_sql || 'T.CHILD_COUNT, ';
      h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.PRIVATE_FLAG, T.DESCRIPTION, ';
      h_sql := h_sql || 
                  'CASE WHEN TYPE_ID = 13 THEN (SELECT TITLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE '''' END AS TITLE, ';
      h_sql := h_sql || 'CASE WHEN T.TYPE_ID = 13 THEN ';
      h_sql := h_sql || '((SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID = ';
      h_sql := h_sql || '(SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE ELEMENT_ID = ';
      h_sql := h_sql || '(SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = T.ELEMENT_ID AND ';
      h_sql := h_sql || 
                  'PARENT_ID = (SELECT PRIMARY_ROLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)))) || '', '' || ';
      h_sql := h_sql || '(SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID = ';
      h_sql := h_sql || '(SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = T.ELEMENT_ID AND ';
      h_sql := h_sql || 
                  'PARENT_ID = (SELECT PRIMARY_ROLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)))) ';
      h_sql := h_sql || 'ELSE '''' END AS COMPETENCY ';
      h_sql := h_sql || 'FROM TMT_CONTACTS T ';
      
      -- Validate id
      If (p_id IS NULL) Then
         h_sql := h_sql || 
                  'WHERE (PMO_ORASTD.LOCATE(T.RANK, (SELECT RANK FROM TMT_CONTACTS WHERE PRIVATE_FLAG = ''Y'')) = 1 OR T.PARENT_ID IS NULL) ';
         h_sql := h_sql || 'AND T.TYPE_ID <> 0 ';
      Else
         h_id := p_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_CONTACT', 54+1) THEN 
            LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_CONTACT(55) : contacts.sql(760) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_CONTACTS' || '''' || ' ID: ' || '''' || h_id || '''');
         END IF; 
         PMODEFINES.PMO_VALIDATE_EXISTENCE('TMT_CONTACTS', h_id, h_rec_status);
         
         h_sql := h_sql || 'WHERE (T.PARENT_ID = ''';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ''' OR T.ELEMENT_ID = ''';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ''') AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
      End If;
   ElsIf Coalesce(p_bitflag, 0) = 1 Then
      h_sql := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, ';
      h_sql := h_sql || 
                  'T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_CONTACTS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT, ';
      h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.PRIVATE_FLAG, T.DESCRIPTION ';
      h_sql := h_sql || 'FROM TMT_CONTACTS T ';
      
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_CONTACTS
         WHERE ELEMENT_ID = p_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_CONTACT', 80+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_CONTACT(81) : contacts.sql(786) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_sql := h_sql || 'WHERE ((T.RANK LIKE ''0000%%'') OR (T.RANK LIKE ''';
      h_sql := h_sql || h_rank;
      h_sql := h_sql || '%%'')) AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') AND T.TYPE_ID IN (';
      h_sql := h_sql || TO_CHAR(PMO_CNST.SC_CONTACT_LIST);
      h_sql := h_sql || ',';
      h_sql := h_sql || TO_CHAR(PMO_CNST.SC_FOLDER);
      h_sql := h_sql || ') ';
   Else
      h_sql := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, ';
      h_sql := h_sql || 
                  'T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_CONTACTS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT, ';
      h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.PRIVATE_FLAG, T.DESCRIPTION ';
      h_sql := h_sql || 'FROM TMT_CONTACTS T ';
      
      -- Validate id
      If (p_id IS NULL) Then
         h_sql := h_sql || 
                  'WHERE (PMO_ORASTD.LOCATE(T.RANK, (SELECT RANK FROM TMT_CONTACTS WHERE PRIVATE_FLAG = ''Y'')) = 1 OR T.PARENT_ID IS NULL) ';
         h_sql := h_sql || 'AND T.TYPE_ID <> 0 AND T.RANK NOT LIKE ''qqqq%''';
      Else
         h_id := p_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_CONTACT', 107+1) THEN 
            LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_CONTACT(108) : contacts.sql(813) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_CONTACTS' || '''' || ' ID: ' || '''' || h_id || '''');
         END IF; 
         PMODEFINES.PMO_VALIDATE_EXISTENCE('TMT_CONTACTS', h_id, h_rec_status);
         
         If Coalesce(p_bitflag, 0) = PMO_CNST.SC_REFRESH Then
            h_sql := h_sql || 'WHERE T.ELEMENT_ID = ''';
            h_sql := h_sql || h_id;
            h_sql := h_sql || ''' AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') AND T.TYPE_ID = 12 ';
         Else
            h_sql := h_sql || 'WHERE (T.PARENT_ID = ''';
            h_sql := h_sql || h_id;
            h_sql := h_sql || ''' OR T.ELEMENT_ID = ''';
            h_sql := h_sql || h_id;
            h_sql := h_sql || ''') AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') AND T.TYPE_ID = 12 ';
         End If;
      End If;
      h_sql := h_sql || 'AND T.TYPE_ID <> 13 ';
   End If;
   
   h_sql := h_sql || ' ORDER BY RANK ';
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTACT', 129+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_CONTACT(130) : contacts.sql(835) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTACT', 132+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_CONTACT(133) : contacts.sql(838) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(h_sql);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_CONTACT', 140+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_CONTACT(141) : contacts.sql(846) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_CONTACT', 152+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_CONTACT(153) : contacts.sql(858) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_CONTACT;
ALTER FUNCTION SP_S_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_A_RES_CONTACT ==
CREATE OR REPLACE FUNCTION SP_A_RES_CONTACT
(
     p_resource_id VARCHAR2
   , p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_exists VARCHAR2(32);
   h_name VARCHAR2(255);
   h_rec_status VARCHAR2(1);
   h_max_rank PLS_INTEGER := 0;
   h_rankBase64 VARCHAR2(8);
   h_rank VARCHAR2(200);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_RES_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_RES_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_A_RES_CONTACT(' || 
       '''' || p_resource_id || ''''
          || ', ' || '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_CONTACT', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_RES_CONTACT(27) : contacts.sql(901) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If (p_id IS NULL) OR (p_resource_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RES_CONTACT', 33+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RES_CONTACT(34) : contacts.sql(908) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_PARAMS, p_id);
   End If;
   
   -- Validate id
   h_id := p_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_A_RES_CONTACT', 41+1) THEN 
      LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_A_RES_CONTACT(42) : contacts.sql(916) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_CONTACTS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_VALIDATE_EXISTENCE('TMT_CONTACTS', h_id, h_rec_status);
   
   BEGIN
      SELECT NICK_NAME INTO h_name
      FROM RESOURCES WHERE RESOURCE_ID  = p_resource_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_CONTACT', 54+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_CONTACT(55) : contacts.sql(929) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Validate resource id
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RES_CONTACT', 62+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RES_CONTACT(63) : contacts.sql(937) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_resource_id || '''');
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, p_resource_id);
   End If;
   
   BEGIN
      SELECT ELEMENT_ID INTO h_exists FROM TMT_CONTACTS
      WHERE ELEMENT_ID = p_resource_id AND PARENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_CONTACT', 76+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_CONTACT(77) : contacts.sql(951) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_RES_CONTACT', 83+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_RES_CONTACT(84) : contacts.sql(958) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ASSIGNMENT_ALREADY_EXISTS), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ASSIGNMENT_ALREADY_EXISTS);
   End If;
   
   -- Get next sibling count
   PMODEFINES.PMO_GET_MAX_RANK('TMT_CONTACTS', h_id, h_max_rank);
   
   --Get Base 64
   PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
   
   -- insert-select
   BEGIN
      INSERT INTO TMT_CONTACTS(RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER)
      SELECT RTRIM(Rank) || h_rankBase64, p_id, p_resource_id, h_name, PMO_CNST.SC_RESOURCE, LEVEL_ID + 1,
      p_rec_user FROM TMT_CONTACTS WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_CONTACT', 109+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_CONTACT(110) : contacts.sql(984) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_CONTACTS
      SET ASSIGN_NAMES = RTRIM(ASSIGN_NAMES) || h_name || ';'
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_CONTACT', 128+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_CONTACT(129) : contacts.sql(1003) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_RES_CONTACT', 134+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_RES_CONTACT(135) : contacts.sql(1009) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_CONTACT', 141+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_CONTACT(142) : contacts.sql(1016) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_CONTACT', 153+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_CONTACT(154) : contacts.sql(1028) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_RES_CONTACT;
ALTER FUNCTION SP_A_RES_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_UA_RES_CONTACT ==
CREATE OR REPLACE FUNCTION SP_UA_RES_CONTACT
(
     p_resource_id VARCHAR2
   , p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_full_name VARCHAR2(99);
   h_length PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_UA_RES_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_UA_RES_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_UA_RES_CONTACT(' || 
       '''' || p_resource_id || ''''
          || ', ' || '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_UA_RES_CONTACT', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_UA_RES_CONTACT(21) : contacts.sql(1065) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Execute Delete Statement
   BEGIN
      DELETE FROM TMT_CONTACTS WHERE PARENT_ID = p_id AND ELEMENT_ID = p_resource_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_CONTACT', 38+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_CONTACT(39) : contacts.sql(1083) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         SELECT NICK_NAME || ';', LENGTH(NICK_NAME) + 1
         INTO h_full_name, h_length
         FROM RESOURCES
         WHERE RESOURCE_ID  = p_resource_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_CONTACT', 55+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_CONTACT(56) : contacts.sql(1100) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Update the parent CHILD_COUNT field
      BEGIN
         UPDATE TMT_CONTACTS
         SET CHILD_COUNT  = CHILD_COUNT - 1,
         ASSIGN_NAMES = SUBSTR(ASSIGN_NAMES , 1,InStr(ASSIGN_NAMES,h_full_name)-1) || SUBSTR(ASSIGN_NAMES , InStr(ASSIGN_NAMES,h_full_name) + h_length)
         WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_CONTACT', 76+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_CONTACT(77) : contacts.sql(1121) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_UA_RES_CONTACT', 83+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_UA_RES_CONTACT(84) : contacts.sql(1128) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_CONTACT', 90+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_CONTACT(91) : contacts.sql(1135) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_CONTACT', 102+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_CONTACT(103) : contacts.sql(1147) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_UA_RES_CONTACT;
ALTER FUNCTION SP_UA_RES_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_RES_A_CONTACT ==
CREATE OR REPLACE FUNCTION SP_S_RES_A_CONTACT
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rank VARCHAR2(200);
-- Declare Cursors
      -- Resultset Cursor Declaration
      SELECT_READ1 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SELECT_READ2 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_RES_A_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_RES_A_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_S_RES_A_CONTACT(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_CONTACT', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_RES_A_CONTACT(27) : contacts.sql(1190) : ' || SYSTIMESTAMP);
   END IF; 

   -- Execute Select Statement
   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_RES_A_CONTACT', 32+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_RES_A_CONTACT(33) : contacts.sql(1196) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ELEMENT_ID_REQUIRED);
   End If;
   
   h_id := p_id;
   
   If (p_bitflag IS NOT NULL) AND (Coalesce(p_bitflag, 0) = 2) Then
      BEGIN
         SELECT RANK
         INTO h_rank
         FROM TMT_CONTACTS
         WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_CONTACT', 51+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_CONTACT(52) : contacts.sql(1215) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN SELECT_READ1 FOR SELECT T1.LEVEL_ID, T1.TYPE_ID, T1.RANK, T1.PARENT_ID, T1.ELEMENT_ID, T1.ELEMENT_NAME,
      CASE WHEN T1.PARENT_ID IS NuLL THEN 0 ELSE 1 END AS CHILD_COUNT,
      T1.DELETED_COUNT, T1.REC_STATUS, T1.DESCRIPTION
      FROM TMT_CONTACTS T1
      WHERE (SUBSTR(h_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
      AND (RANK LIKE SUBSTR(h_rank, 1, 8) || '%'))
      AND T1.TYPE_ID NOT in (0, 13)
      AND T1.REC_STATUS in ('I', 'U', 'O')
      ORDER BY T1.RANK DESC;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_CONTACT', 66+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_CONTACT(67) : contacts.sql(1230) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_READ1;
   Else
      OPEN SELECT_READ2 FOR SELECT T1.LEVEL_ID, T1.TYPE_ID, T1.RANK, T1.PARENT_ID, T1.ELEMENT_ID, T1.ELEMENT_NAME,
      CASE WHEN T1.PARENT_ID IS NuLL THEN 0 ELSE 1 END AS CHILD_COUNT,
      T1.DELETED_COUNT, T1.REC_STATUS, T1.DESCRIPTION
      FROM TMT_CONTACTS T1
      WHERE T1.ELEMENT_ID IN (SELECT PARENT_ID FROM TMT_CONTACTS
      WHERE ELEMENT_ID = p_id)
      AND T1.TYPE_ID < 5000
      AND T1.TYPE_ID <> 0
      AND T1.REC_STATUS in ('I', 'U', 'O')
      ORDER BY T1.RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_CONTACT', 81+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_CONTACT(82) : contacts.sql(1245) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_READ2;
   End If;
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_CONTACT', 89+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_CONTACT(90) : contacts.sql(1253) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_CONTACT', 101+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_CONTACT(102) : contacts.sql(1265) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_RES_A_CONTACT;
ALTER FUNCTION SP_S_RES_A_CONTACT COMPILE DEBUG;

