/****************************************************************************************/

Prompt == FUNCTION SP_I_DATAFIELD ==
CREATE OR REPLACE FUNCTION SP_I_DATAFIELD
(
     p_name VARCHAR2
   , p_description VARCHAR2
   , p_parent_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_uniqueID PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_rankBase64 VARCHAR2(8);
   h_rank VARCHAR2(200);
   h_parent_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_parent_id_int PLS_INTEGER := 0;
   h_id_count PLS_INTEGER := 0;
   h_level PLS_INTEGER := 0;
   h_broker_type PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_DATAFIELD') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_DATAFIELD'), '-- ' || SYSTIMESTAMP, ':r := SP_I_DATAFIELD(' || 
       '''' || p_name || ''''
          || ', ' || '''' || p_description || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_DATAFIELD', 32+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_DATAFIELD(33) : datafields.sql(36) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_broker_type := 90;
   
   BEGIN
      SELECT COUNT(ELEMENT_ID) INTO h_id_count FROM TMT_DATAFIELDS;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 47+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(48) : datafields.sql(51) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_id_count, 0) = 0 Then
      h_uniqueID := 1;
   Else
      BEGIN
         SELECT MAX(ELEMENT_ID)+1 INTO h_uniqueID FROM TMT_DATAFIELDS;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 63+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(64) : datafields.sql(67) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   -- Validate parent_id
   If p_parent_id IS NULL Then
      -- Get the max sibling level from the root
      PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_DATAFIELDS', h_max_rank);
      
      --Get Base 64
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      --Execute Insert Statement
      BEGIN
         SELECT RTRIM(COALESCE(RANK, '')) || h_rankBase64, LEVEL_ID + 1 INTO h_rank, h_level
         FROM TMT_DATAFIELDS WHERE ELEMENT_ID IS NuLL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 87+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(88) : datafields.sql(91) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO TMT_DATAFIELDS(RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER, DESCRIPTION)
         VALUES(h_rank, -1, h_uniqueID, p_name, p_type_id, h_level, p_rec_user, p_description);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 105+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(106) : datafields.sql(109) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      -- Validate parent_id
      h_parent_id := p_parent_id;
      h_parent_id_int := TO_NUMBER(h_parent_id);
      
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_I_DATAFIELD', 115+1) THEN 
         LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_I_DATAFIELD(116) : datafields.sql(119) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_DATAFIELDS' || '''' || ' ID: ' || COALESCE(TO_CHAR(h_parent_id_int), 'NULL'));
      END IF; 
      PMODEFINES.PMO_VALIDATE_RECORD('TMT_DATAFIELDS', h_parent_id_int, h_rec_status);
      
      -- Get next sibling count
      PMODEFINES.PMO_GET_MAX_RANK('TMT_DATAFIELDS', h_parent_id_int, h_max_rank);
      
      --Get Base 64
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      --Execute Insert Statement
      BEGIN
         SELECT RTRIM(COALESCE(RANK, '')) || h_rankBase64, LEVEL_ID + 1 INTO h_rank, h_level
         FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_parent_id_int;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 135+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(136) : datafields.sql(139) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO TMT_DATAFIELDS(RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER, DESCRIPTION)
         VALUES(h_rank, h_parent_id_int, h_uniqueID, p_name, p_type_id, h_level, p_rec_user, p_description);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 153+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(154) : datafields.sql(157) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         BEGIN
            UPDATE CHANGE_LOG SET MODIF_DATE_TIME = SYSTIMESTAMP WHERE BROKER_TYPE = h_broker_type;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 173+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(174) : datafields.sql(177) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         PMO_DB2M.SC_STATE := 0;
      END;
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         PMO_DB2M.SC_STATE := 0;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_DATAFIELD', 185+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_DATAFIELD(186) : datafields.sql(189) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || COALESCE(TO_CHAR(h_uniqueID), 'NULL'));
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 192+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(193) : datafields.sql(196) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 204+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(205) : datafields.sql(208) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_DATAFIELD;
ALTER FUNCTION SP_I_DATAFIELD COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_DATAFIELD ==
CREATE OR REPLACE FUNCTION SP_U_DATAFIELD
(
     p_id VARCHAR2
   , p_name VARCHAR2
   , p_description VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_id_int PLS_INTEGER := 0;
   h_rec_status VARCHAR2(1);
   h_broker_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_DATAFIELD') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_DATAFIELD'), '-- ' || SYSTIMESTAMP, ':r := SP_U_DATAFIELD(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || '''' || p_description || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_DATAFIELD', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_DATAFIELD(26) : datafields.sql(250) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If p_id IS NULL Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_DATAFIELD', 32+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_DATAFIELD(33) : datafields.sql(257) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ELEMENT_ID_REQUIRED);
   End If;
   
   h_broker_type := 90;
   
   h_id := p_id;
   h_id_int := TO_NUMBER(h_id);
   
   BEGIN
      SELECT TYPE_ID, REC_STATUS INTO h_type_id, h_rec_status
      FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_id_int;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_DATAFIELD', 51+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_DATAFIELD(52) : datafields.sql(276) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_DATAFIELD', 57+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_DATAFIELD(58) : datafields.sql(282) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, p_id);
   End If;
   
   BEGIN
      UPDATE TMT_DATAFIELDS SET
      ELEMENT_NAME = p_name,
      DESCRIPTION  = p_description,
      REC_STATUS   = 'U',
      REC_DATETIME = SYSTIMESTAMP,
      REC_USER     = p_rec_user
      WHERE ELEMENT_ID = h_id_int;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_DATAFIELD', 80+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_DATAFIELD(81) : datafields.sql(305) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      If Coalesce(h_type_id, 0) = 7 Then
         -- Add rank like '0000' || '%' to force the query using index
         BEGIN
            UPDATE TMT_COMPETENCIES SET
            ELEMENT_NAME = p_description
            WHERE PROFICIENCY_LEVEL_ID = h_id_int
            AND RANK LIKE '0000' || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_DATAFIELD', 102+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_DATAFIELD(103) : datafields.sql(327) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         BEGIN
            UPDATE CHANGE_LOG SET MODIF_DATE_TIME = SYSTIMESTAMP WHERE BROKER_TYPE = h_broker_type;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_DATAFIELD', 121+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_DATAFIELD(122) : datafields.sql(346) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         PMO_DB2M.SC_STATE := 0;
      END;
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         PMO_DB2M.SC_STATE := 0;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_DATAFIELD', 133+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_DATAFIELD(134) : datafields.sql(358) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || COALESCE(TO_CHAR(h_id_int), 'NULL'));
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id_int);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_DATAFIELD', 140+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_DATAFIELD(141) : datafields.sql(365) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_DATAFIELD', 152+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_DATAFIELD(153) : datafields.sql(377) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_DATAFIELD;
ALTER FUNCTION SP_U_DATAFIELD COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_DATAFIELD ==
CREATE OR REPLACE FUNCTION SP_S_DATAFIELD
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_id_int PLS_INTEGER := 0;
   h_fs_flag PLS_INTEGER := 0;
   h_sql VARCHAR2(2048);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_DATAFIELD') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_DATAFIELD'), '-- ' || SYSTIMESTAMP, ':r := SP_S_DATAFIELD(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_DATAFIELD', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_DATAFIELD(22) : datafields.sql(415) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   BEGIN
      SELECT SETTING_VALUE INTO h_fs_flag
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'REPOSITORY_TYPE';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_DATAFIELD', 36+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_DATAFIELD(37) : datafields.sql(430) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_sql := 
         'SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, CHILD_COUNT, DELETED_COUNT, REC_STATUS, POSITION, DESCRIPTION ';
   h_sql := h_sql || 'FROM TMT_DATAFIELDS ';
   
   If (p_bitflag IS NOT NULL) AND (Coalesce(p_bitflag, 0) = 2) Then
      h_sql := h_sql || 'WHERE ELEMENT_ID IS NOT NULL ';
   Else
      -- Validate id
      If (p_id IS NULL) Then
         h_sql := h_sql || 'WHERE PARENT_ID IS NULL ';
         h_sql := h_sql || 'AND TYPE_ID <> 0 ';
      Else
         h_id := p_id;
         h_id_int := TO_NUMBER(h_id);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_DATAFIELD', 56+1) THEN 
            LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_DATAFIELD(57) : datafields.sql(450) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_DATAFIELDS' || '''' || ' ID: ' || COALESCE(TO_CHAR(h_id_int), 'NULL'));
         END IF; 
         PMODEFINES.PMO_VALIDATE_EXISTENCE('TMT_DATAFIELDS', h_id_int, h_rec_status);
         
         h_sql := h_sql || 'WHERE ( PARENT_ID = ';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ' OR ELEMENT_ID = ';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ' ) ';
      End If;
   End If;
   If Coalesce(h_fs_flag, 0) <> 0 Then
      h_sql := h_sql || 'AND REC_STATUS <> ''D'' ';
   End If;
   h_sql := h_sql || ' ORDER BY RANK ';
   
   --Return a resultset
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_DATAFIELD', 74+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_DATAFIELD(75) : datafields.sql(468) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_DATAFIELD', 77+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_DATAFIELD(78) : datafields.sql(471) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(h_sql);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_DATAFIELD', 84+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_DATAFIELD(85) : datafields.sql(478) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_DATAFIELD', 96+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_DATAFIELD(97) : datafields.sql(490) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_DATAFIELD;
ALTER FUNCTION SP_S_DATAFIELD COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_STATE_DEF ==
CREATE OR REPLACE FUNCTION SP_S_STATE_DEF
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
   h_child_type PLS_INTEGER := 0;
   h_name VARCHAR2(100);
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      CSDEF0 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CSDEF2 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CSDEF1 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_STATE_DEF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_STATE_DEF'), '-- ' || SYSTIMESTAMP, ':r := SP_S_STATE_DEF(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STATE_DEF', 31+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_STATE_DEF(32) : datafields.sql(538) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_bitflag := p_bitflag;
   
   h_child_type := 243;
   
   CASE h_bitflag
      WHEN PMO_CNST.SC_PROJECT THEN
         h_type_id := 12;
         h_name := 'Project';
         
      
      WHEN PMO_CNST.SC_PROPOSAL THEN
         h_type_id := 12;
         h_name := 'Project';
         
      
      WHEN PMO_CNST.SC_PROJ_TYPE1 THEN
         h_type_id := 12;
         h_name := 'Project';
         
      
      WHEN PMO_CNST.SC_PROJ_TYPE2 THEN
         h_type_id := 12;
         h_name := 'Project';
         
      
      WHEN PMO_CNST.SC_PROJ_TYPE3 THEN
         h_type_id := 12;
         h_name := 'Project';
         
      
      WHEN PMO_CNST.SC_DELIVERABLE THEN
         h_type_id := 119;
         h_name := 'Deliverable';
         
      
      WHEN PMO_CNST.SC_TASK THEN
         h_type_id := 120;
         h_name := 'Task';
         
      
      WHEN PMO_CNST.SC_CRI_CHANGE_REQUEST THEN
         h_type_id := 19;
         h_name := 'Change Request';
         
      
      WHEN PMO_CNST.SC_CRI_RISK THEN
         h_type_id := 99;
         h_name := 'Risk';
         
      
      WHEN PMO_CNST.SC_CRI_ISSUE THEN
         h_type_id := 106;
         h_name := 'Issue';
         
      
      WHEN PMO_CNST.SC_DOCUMENT THEN
         h_type_id := 108;
         h_name := 'Document';
         
      
      WHEN PMO_CNST.SC_NOTE_MINUTES THEN
         h_type_id := 118;
         h_name := 'Note/Minutes';
         
      
      WHEN PMO_CNST.SC_CRI_REQUIREMENT THEN
         h_type_id := 105;
         h_name := 'Requirement';
         
      
      WHEN PMO_CNST.SC_CRI_DEFECT THEN
         h_type_id := 115;
         h_name := 'Defects';
         
      
      WHEN PMO_CNST.SC_CRI_SERVICE_REQUEST THEN
         h_type_id := 116;
         h_name := 'Service Requests';
         
      
      WHEN PMO_CNST.SC_SCORECARD THEN
         h_type_id := 121;
         h_name := 'Scorecard';
         
      
      WHEN PMO_CNST.SC_CRI_ACTION_ITEM THEN
         h_type_id := 136;
         h_name := 'Actions';
         
      
      WHEN PMO_CNST.SC_ALERT_GROUP THEN
         h_type_id := PMO_CNST.SC_ALERT_GROUP;
         h_name := 'Alert';
         
      
      ELSE
         h_type_id := 0;
         h_name := '';
   END CASE;
   
   h_id := p_id;
   h_parent_id := TO_CHAR(Coalesce(h_bitflag, 0));
   
   If Coalesce(h_bitflag, 0) = 0 Then
      OPEN CSDEF0 FOR SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, TO_CHAR(TYPE_ID) AS ELEMENT_ID, ELEMENT_NAME, 1 AS CHILD_COUNT, 0 AS POSITION, 0 AS STAGE_ID, h_id AS PROJECT_ID,
      '0' AS ASSIGN_FLAG, '' AS GROUP_IDS, '' AS GROUP_NAMES, '' AS PARTICIPANT_IDS, '' AS PARTICIPANT_NAMES, '' AS SECURITY_IDS, '' AS SECURITY_NAMES, '' AS SUBJECT, '' AS MESSAGE, '0' AS FLAG, 0 AS VARIANCE
      FROM TMT_WORKFLOW_CYCLES
      WHERE LEVEL_ID = 0
      AND REC_STATUS<>'D'
      AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3, PMO_CNST.SC_DELIVERABLE, PMO_CNST.SC_TASK, PMO_CNST.SC_CHANGE_REQUEST, PMO_CNST.SC_RISK, PMO_CNST.SC_ISSUE, PMO_CNST.SC_DOCUMENT, PMO_CNST.SC_NOTE_MINUTES, PMO_CNST.SC_REQUIREMENT, PMO_CNST.SC_SERVICE_REQUEST, PMO_CNST.SC_DEFECT, PMO_CNST.SC_SCORECARD, PMO_CNST.SC_CRI_ACTION_ITEM, PMO_CNST.SC_ALERT_GROUP)
      ORDER BY RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STATE_DEF', 148+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_STATE_DEF(149) : datafields.sql(655) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN CSDEF0;
   Else
      If Coalesce(h_bitflag, 0) = PMO_CNST.SC_ALERT_GROUP Then
         --  for alert messages we use alert type_id as stage_id
         
         OPEN CSDEF2 FOR SELECT DISTINCT 2 AS LEVEL_ID, h_child_type AS TYPE_ID, '0000' || TO_CHAR(msg.TYPE_ID) AS RANK,
         h_parent_id AS PARENT_ID, h_parent_id || '.' || TO_CHAR(msg.TYPE_ID) AS ELEMENT_ID,
         msg.ALERT_NAME AS ELEMENT_NAME, 0 AS CHILD_COUNT, 0 AS POSITION, msg.TYPE_ID AS STAGE_ID, h_id AS PROJECT_ID,
         COALESCE(ASSIGN_FLAG,'0') AS ASSIGN_FLAG, GROUP_IDS, GROUP_NAMES, PARTICIPANT_IDS, PARTICIPANT_NAMES,
         SD.SECURITY_IDS, SD.SECURITY_NAMES, SUBJECT, RTRIM(MESSAGE) AS MESSAGE,
         CASE WHEN SD.STAGE_ID IS NuLL THEN '0' ELSE '1' END AS FLAG, SD.VARIANCE
         FROM ALERT_MESSAGES msg
         LEFT JOIN PROJECT_STATE_DEF SD
         ON SD.PROJECT_ID = h_id
         AND SD.STAGE_ID = msg.TYPE_ID
         WHERE msg.ALERT_TYPE = 1
         AND msg.ALERT_BIT = 1
         UNION ALL
         SELECT 1 AS LEVEL_ID, h_type_id AS TYPE_ID, '0000' AS RANK, '' AS PARENT_ID,
         h_parent_id AS ELEMENT_ID, h_name AS ELEMENT_NAME, 1 AS CHILD_COUNT, 0 AS POSITION, 0 AS STAGE_ID,
         h_id AS PROJECT_ID, '0' AS ASSIGN_FLAG, '' AS GROUP_IDS, '' AS GROUP_NAMES, '' AS PARTICIPANT_IDS,
         '' AS PARTICIPANT_NAMES, '' AS SECURITY_IDS, '' AS SECURITY_NAMES, '' AS SUBJECT, '' AS MESSAGE, '0' AS FLAG, 0 AS VARIANCE
         FROM TMT_WORKFLOW_CYCLES
         WHERE TYPE_ID = h_type_id
         ORDER BY RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STATE_DEF', 176+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_STATE_DEF(177) : datafields.sql(683) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN CSDEF2;
      Else
         OPEN CSDEF1 FOR SELECT DF.LEVEL_ID, h_child_type AS TYPE_ID, DF.RANK, h_parent_id AS PARENT_ID, h_parent_id || '.' || TO_CHAR(ELEMENT_ID) AS ELEMENT_ID,
         DF.DESCRIPTION AS ELEMENT_NAME, DF.CHILD_COUNT, DF.POSITION, DF.ELEMENT_ID AS STAGE_ID, h_id AS PROJECT_ID,
         COALESCE(ASSIGN_FLAG,'0') AS ASSIGN_FLAG, GROUP_IDS, GROUP_NAMES, PARTICIPANT_IDS, PARTICIPANT_NAMES, SD.SECURITY_IDS, SD.SECURITY_NAMES, SUBJECT, RTRIM(MESSAGE) AS MESSAGE,
         CASE WHEN SD.STAGE_ID IS NuLL THEN '0' ELSE '1' END AS FLAG, SD.VARIANCE
         FROM TMT_DATAFIELDS DF
         LEFT JOIN PROJECT_STATE_DEF SD
         ON SD.PROJECT_ID = h_id
         AND SD.ELEMENT_TYPE = p_bitflag
         AND SD.STAGE_ID = DF.ELEMENT_ID
         WHERE DF.TYPE_ID = h_type_id
         AND DF.LEVEL_ID = 2
         UNION ALL
         SELECT DF.LEVEL_ID, p_bitflag AS TYPE_ID, DF.RANK, '' AS PARENT_ID,
         h_parent_id AS ELEMENT_ID, h_name AS ELEMENT_NAME, DF.CHILD_COUNT, DF.POSITION, DF.ELEMENT_ID AS STAGE_ID, h_id AS PROJECT_ID,
         '0' AS ASSIGN_FLAG, '' AS GROUP_IDS, '' AS GROUP_NAMES, '' AS PARTICIPANT_IDS, '' AS PARTICIPANT_NAMES, '' AS SECURITY_IDS, '' AS SECURITY_NAMES, '' AS SUBJECT, '' AS MESSAGE, '0' AS FLAG, 0 AS VARIANCE
         FROM TMT_DATAFIELDS DF
         WHERE DF.TYPE_ID = h_type_id
         AND DF.LEVEL_ID = 1
         ORDER BY RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STATE_DEF', 200+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_STATE_DEF(201) : datafields.sql(707) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN CSDEF1;
      End If;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_STATE_DEF', 210+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_STATE_DEF(211) : datafields.sql(717) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_STATE_DEF', 222+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_STATE_DEF(223) : datafields.sql(729) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_STATE_DEF;
ALTER FUNCTION SP_S_STATE_DEF COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_D_DATAFIELD ==
CREATE OR REPLACE FUNCTION SP_D_DATAFIELD
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_rank VARCHAR2(200);
   h_parent_id_int PLS_INTEGER := 0;
   h_id_int PLS_INTEGER := 0;
   h_broker_type PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_DATAFIELD') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_DATAFIELD'), '-- ' || SYSTIMESTAMP, ':r := SP_D_DATAFIELD(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_DATAFIELD', 27+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_DATAFIELD(28) : datafields.sql(773) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_broker_type := 90;
   
   -- Validate id
   h_id := p_id;
   h_id_int := TO_NUMBER(h_id);
   
   h_rec_status := p_del_flag;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(256, 'SP_D_DATAFIELD', 40+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_DELETE: SP_D_DATAFIELD(41) : datafields.sql(786) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_DATAFIELDS' || '''' || ' ID: ' || COALESCE(TO_CHAR(h_id_int), 'NULL'));
   END IF; 
   PMODEFINES.PMO_VALIDATE_DELETE('TMT_DATAFIELDS', h_id_int, h_rec_status);
   
   -- Validate parent id
   If (p_parent_id IS NULL) Then
      -- Find Necessary Info
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_id_int AND PARENT_ID IS NuLL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 55+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(56) : datafields.sql(801) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_rec_status := '';
   Else
      h_parent_id := p_parent_id;
      h_parent_id_int := TO_NUMBER(h_parent_id);
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_D_DATAFIELD', 65+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_D_DATAFIELD(66) : datafields.sql(811) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_DATAFIELDS' || '''' || ' ID: ' || COALESCE(TO_CHAR(h_parent_id_int), 'NULL'));
      END IF; 
      PMODEFINES.PMO_VALIDATE_EXISTENCE('TMT_DATAFIELDS', h_parent_id_int, h_rec_status);
      
      If SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_DATAFIELD', 71+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_DATAFIELD(72) : datafields.sql(817) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.PARENT_MARKED_AS_DELETED);
      End If;
      
      -- Find Necessary Info
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_id_int AND PARENT_ID = h_parent_id_int;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 85+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(86) : datafields.sql(831) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If SubStr(p_del_flag, 1, 1) = '?' Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_DATAFIELD', 93+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_DATAFIELD(94) : datafields.sql(839) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || COALESCE(TO_CHAR(h_id_int), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR(0, h_id_int);
   End If;
   
   -- Execute Delete Statement
   If SubStr(p_del_flag, 1, 1) = 'X' Then
      -- Delete Tree
      PMODEFINES.PMO_PMO_VERIFY_RANK(h_rank, 0);
      BEGIN
         DELETE FROM TMT_DATAFIELDS WHERE RANK LIKE h_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 114+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(115) : datafields.sql(860) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Update Parent's Hierarchy Info
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         If p_parent_id IS NULL Then
            BEGIN
               UPDATE TMT_DATAFIELDS SET DELETED_COUNT = DELETED_COUNT - 1
               WHERE ELEMENT_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 135+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(136) : datafields.sql(881) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_DATAFIELDS SET DELETED_COUNT = DELETED_COUNT - 1
               WHERE ELEMENT_ID = h_parent_id_int;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 153+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(154) : datafields.sql(899) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If p_parent_id IS NULL Then
            BEGIN
               UPDATE TMT_DATAFIELDS SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 173+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(174) : datafields.sql(919) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_DATAFIELDS SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID = h_parent_id_int;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 191+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(192) : datafields.sql(937) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   Else
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_DATAFIELD', 200+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_DATAFIELD(201) : datafields.sql(946) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.PARENT_MARKED_AS_DELETED);
      End If;
      
      -- Update Parent's Hierarchy Info
      If p_parent_id IS NULL Then
         BEGIN
            UPDATE TMT_DATAFIELDS SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
            WHERE ELEMENT_ID IS NuLL;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 220+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(221) : datafields.sql(966) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TMT_DATAFIELDS SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
            WHERE ELEMENT_ID = h_parent_id_int;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 238+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(239) : datafields.sql(984) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      PMODEFINES.PMO_PMO_VERIFY_RANK(h_rank, 0);
      BEGIN
         UPDATE TMT_DATAFIELDS SET
         REC_STATUS   = p_del_flag,
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER     = p_rec_user
         WHERE RANK LIKE h_rank || '%' AND REC_STATUS IN ('I','U');
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 261+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(262) : datafields.sql(1007) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         BEGIN
            UPDATE CHANGE_LOG SET MODIF_DATE_TIME = SYSTIMESTAMP WHERE BROKER_TYPE = h_broker_type;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 281+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(282) : datafields.sql(1027) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         PMO_DB2M.SC_STATE := 0;
      END;
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         PMO_DB2M.SC_STATE := 0;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_DATAFIELD', 293+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_DATAFIELD(294) : datafields.sql(1039) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || COALESCE(TO_CHAR(h_id_int), 'NULL'));
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id_int);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 300+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(301) : datafields.sql(1046) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 312+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(313) : datafields.sql(1058) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_DATAFIELD;
ALTER FUNCTION SP_D_DATAFIELD COMPILE DEBUG;

