/****************************************************************************************/

Prompt == FUNCTION SP_I_FAC ==
CREATE OR REPLACE FUNCTION SP_I_FAC
(
     p_name VARCHAR2
   , p_parent_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_current_datetime TIMESTAMP;
   h_parent_id VARCHAR2(32);
   h_parent_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
   h_uniqueID VARCHAR2(32);
   h_max_rank PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_FAC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_FAC'), '-- ' || SYSTIMESTAMP, ':r := SP_I_FAC(' || 
       '''' || p_name || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_FAC', 29+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_FAC(30) : factors.sql(33) : ' || SYSTIMESTAMP);
   END IF; 

   If p_parent_id IS NULL Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_I_FAC', 34+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_I_FAC(35) : factors.sql(38) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ELEMENT_ID_REQUIRED);
   End If;
   
   --Declare local variables
   
   h_uniqueID := SYS_GUID();
   
   BEGIN
      SELECT (SYSTIMESTAMP)  INTO  h_current_datetime FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_FAC', 51+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_FAC(52) : factors.sql(55) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_parent_id := p_parent_id;
   
   BEGIN
      SELECT RTRIM(RANK), RTRIM(RANK), LEVEL_ID + 1, REC_STATUS
      INTO h_parent_rank, h_rank, h_level_id, h_rec_status
      FROM TMT_CRI WHERE ELEMENT_ID = h_parent_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_FAC', 68+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_FAC(69) : factors.sql(72) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         SELECT RTRIM(PARENT_RANK), RTRIM(RANK), LEVEL_ID + 1, REC_STATUS
         INTO h_parent_rank, h_rank, h_level_id, h_rec_status
         FROM TMT_FACTORS WHERE ELEMENT_ID = h_parent_id AND REC_STATUS IN ('I', 'U', 'O');
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_FAC', 84+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_FAC(85) : factors.sql(88) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_I_FAC', 90+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_I_FAC(91) : factors.sql(94) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_parent_id || '''');
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, p_parent_id);
      End If;
      If SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_I_FAC', 96+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_I_FAC(97) : factors.sql(100) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.MARKED_AS_UNAVAILABLE), 'NULL'));
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.MARKED_AS_UNAVAILABLE);
      End If;
      
      PMODEFINES.PMO_GET_MAX_RANK('TMT_FACTORS', h_parent_id, h_max_rank);
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      BEGIN
         INSERT INTO TMT_FACTORS(PARENT_RANK, RANK, PARENT_ID, ELEMENT_ID, TYPE_ID, LEVEL_ID, ELEMENT_NAME, REC_USER)
         VALUES(h_parent_rank, RTRIM(h_rank) || h_rankBase64, h_parent_id, h_uniqueID, p_type_id, h_level_id, p_name, p_rec_user);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_FAC', 117+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_FAC(118) : factors.sql(121) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      If SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_I_FAC', 124+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_I_FAC(125) : factors.sql(128) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.MARKED_AS_UNAVAILABLE), 'NULL'));
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.MARKED_AS_UNAVAILABLE);
      End If;
      
      PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_FACTORS', h_max_rank);
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      BEGIN
         INSERT INTO TMT_FACTORS(PARENT_RANK, RANK, PARENT_ID, ELEMENT_ID, TYPE_ID, LEVEL_ID, ELEMENT_NAME, REC_USER)
         VALUES(h_parent_rank, '0000' || h_rankBase64, h_parent_id, h_uniqueID, p_type_id, 0, p_name, p_rec_user);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_FAC', 145+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_FAC(146) : factors.sql(149) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_FAC', 152+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_FAC(153) : factors.sql(156) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_FAC', 159+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_FAC(160) : factors.sql(163) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_FAC', 171+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_FAC(172) : factors.sql(175) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_FAC;
ALTER FUNCTION SP_I_FAC COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_FAC ==
CREATE OR REPLACE FUNCTION SP_U_FAC
(
     p_id VARCHAR2
   , p_name VARCHAR2
   , p_description VARCHAR2
   , p_migration_cost NUMBER
   , p_priority PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
   h_old_migration_cost NUMBER := 0.00;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_FAC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_FAC'), '-- ' || SYSTIMESTAMP, ':r := SP_U_FAC(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || '''' || p_description || ''''
          || ', ' || COALESCE(TO_CHAR(p_migration_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_priority), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_FAC', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_FAC(29) : factors.sql(220) : ' || SYSTIMESTAMP);
   END IF; 

   If p_id IS NULL Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_FAC', 33+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_FAC(34) : factors.sql(225) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ELEMENT_ID_REQUIRED);
   End If;
   
   -- Declare host variables
   
   BEGIN
      SELECT TYPE_ID, RANK, COALESCE(MITIGATION_COST, 0.00), REC_STATUS
      INTO h_type_id, h_rank, h_old_migration_cost, h_rec_status
      FROM TMT_FACTORS WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FAC', 50+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FAC(51) : factors.sql(242) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_FAC', 56+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_FAC(57) : factors.sql(248) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, p_id);
   End If;
   If SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_FAC', 62+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_FAC(63) : factors.sql(254) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.MARKED_AS_UNAVAILABLE), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.MARKED_AS_UNAVAILABLE);
   End If;
   
   If Coalesce(h_type_id, 0) = PMO_CNST.SC_FACTOR Then
      BEGIN
         UPDATE TMT_FACTORS SET
         ELEMENT_NAME = p_name,
         DESCRIPTION = p_description,
         MITIGATION_COST = p_migration_cost,
         PRIORITY = p_priority,
         REC_STATUS = 'U',
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER = p_rec_user
         WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FAC', 88+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FAC(89) : factors.sql(280) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE TMT_CRI T
         SET T.MITIGATION_COST = (SELECT COALESCE(SUM(MITIGATION_COST), 0.00)
         FROM TMT_FACTORS
         WHERE PARENT_ID = T.ELEMENT_ID
         AND TYPE_ID = PMO_CNST.SC_FACTOR
         AND REC_STATUS IN ('I', 'O', 'U')),
         T.REC_DATETIME = SYSTIMESTAMP
         WHERE RANK = (SELECT RTRIM(PARENT_RANK) FROM TMT_FACTORS WHERE ELEMENT_ID = p_id );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FAC', 112+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FAC(113) : factors.sql(304) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         UPDATE TMT_FACTORS SET
         ELEMENT_NAME = p_name,
         DESCRIPTION = p_description,
         PRIORITY = p_priority,
         REC_STATUS = 'U',
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER = p_rec_user
         WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FAC', 136+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FAC(137) : factors.sql(328) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_FAC', 143+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_FAC(144) : factors.sql(335) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FAC', 150+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FAC(151) : factors.sql(342) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FAC', 162+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FAC(163) : factors.sql(354) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_FAC;
ALTER FUNCTION SP_U_FAC COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_DELETE_FAC ==
CREATE OR REPLACE FUNCTION FC_DELETE_FAC
(
     h_id IN OUT NOCOPY VARCHAR2
   , h_parent_id IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
   h_xsql VARCHAR2(8000);
-- Declare Cursors
      -- Dynamic Cursor Declaration
      C_FAC PMO_DB2M.REF_CURSOR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_DELETE_FAC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_DELETE_FAC'), '-- ' || SYSTIMESTAMP, ':r := FC_DELETE_FAC(' || 
       '''' || h_id || ''''
          || ', ' || '''' || h_parent_id || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DELETE_FAC', 23+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_DELETE_FAC(24) : factors.sql(394) : ' || SYSTIMESTAMP);
   END IF; 

   --Declare local variables
   
   If h_parent_id IS NULL Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DELETE_FAC', 30+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_DELETE_FAC(31) : factors.sql(401) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.ELEMENT_ID_REQUIRED;
   End If;
   
   If h_id IS NULL Then
      h_xsql := 'SELECT RANK, REC_STATUS FROM TMT_FACTORS WHERE PARENT_ID = ''' || h_parent_id || ''' ';
   Else
      h_xsql := 'SELECT RANK, REC_STATUS FROM TMT_FACTORS WHERE ELEMENT_ID = ''' || h_id 
            || ''' AND PARENT_ID = ''' || h_parent_id || ''' ';
   End If;
   
   OPEN C_FAC FOR h_xsql;
   
   LOOP
      FETCH C_FAC INTO h_rank, h_rec_status;
      IF C_FAC%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      PMODEFINES.PMO_PMO_VERIFY_RANK(h_rank, 0);
      BEGIN
         DELETE FROM TMT_FACTORS WHERE RANK LIKE h_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_FAC', 69+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_FAC(70) : factors.sql(440) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         If Coalesce(Length(h_rank), 0) = 8 Then
            BEGIN
               UPDATE TMT_FACTORS
               SET DELETED_COUNT = DELETED_COUNT - 1
               WHERE ELEMENT_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_FAC', 90+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_FAC(91) : factors.sql(461) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_FACTORS
               SET DELETED_COUNT = DELETED_COUNT - 1
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_FAC', 109+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_FAC(110) : factors.sql(480) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If Coalesce(Length(h_rank), 0) = 8 Then
            BEGIN
               UPDATE TMT_FACTORS
               SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_FAC', 130+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_FAC(131) : factors.sql(501) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT PARENT_ID INTO h_parent_id
               FROM TMT_FACTORS
               WHERE RANK = SUBSTR(h_rank, 1, 8);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_FAC', 145+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_FAC(146) : factors.sql(516) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_FACTORS
               SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_FAC', 164+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_FAC(165) : factors.sql(535) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         --Update risk
         BEGIN
            SELECT RANK INTO h_rank
            FROM TMT_CRI
            WHERE ELEMENT_ID=h_parent_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_FAC', 181+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_FAC(182) : factors.sql(552) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         -- factor type is always the first level in tmt_factors, factor can't have
         -- another factor as its child
         BEGIN
            UPDATE TMT_CRI T
            SET T.MITIGATION_COST = (SELECT COALESCE(SUM(MITIGATION_COST), 0.00)
            FROM TMT_FACTORS
            WHERE PARENT_ID = T.ELEMENT_ID
            AND TYPE_ID = PMO_CNST.SC_FACTOR
            AND REC_STATUS IN ('I', 'O', 'U')),
            T.REC_DATETIME = SYSTIMESTAMP,
            T.REC_USER = h_rec_user
            WHERE (SUBSTR(h_rank, 1, CASE WHEN LENGTH(T.RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(T.RANK) END) = T.RANK 
            AND (T.RANK LIKE SUBSTR(h_rank, 1, 8) || '%'))
            AND T.TYPE_ID = PMO_CNST.SC_RISK
            AND T.REC_STATUS IN ('I', 'O', 'U');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_FAC', 211+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_FAC(212) : factors.sql(582) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_CRI T
            SET T.ESTIMATED_BUDGET = (SELECT COALESCE(SUM(MITIGATION_COST), 0.00)
            FROM TMT_FACTORS
            WHERE PARENT_ID = T.ELEMENT_ID
            AND TYPE_ID = PMO_CNST.SC_FACTOR
            AND REC_STATUS IN ('I', 'O', 'U'))
            WHERE T.ELEMENT_ID=h_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_FAC', 234+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_FAC(235) : factors.sql(605) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_FAC ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_FAC', 249+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_FAC(250) : factors.sql(620) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DELETE_FAC', 256+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_DELETE_FAC(257) : factors.sql(627) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_DELETE_FAC;
ALTER FUNCTION FC_DELETE_FAC COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_COPY_FAC ==
CREATE OR REPLACE FUNCTION FC_COPY_FAC
(
     h_source_id IN OUT NOCOPY VARCHAR2
   , h_target_id IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_description VARCHAR2(255);
   h_element_name VARCHAR2(255);
   h_parent_id VARCHAR2(32);
   h_parent_rank VARCHAR2(200);
   h_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_uniqueID VARCHAR2(32);
   h_mitigation_cost NUMBER := 0.00;
   h_child_count PLS_INTEGER := 0;
   h_deleted_count PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   prev_level_id PLS_INTEGER;
   t_max_rank PLS_INTEGER;
-- Declare Cursors
      CURSOR C_FACTOR IS
         SELECT RTRIM(RANK)
         FROM TMT_FACTORS
         WHERE PARENT_ID = h_source_id AND LEVEL_ID = 0
         ORDER BY RANK;

      CURSOR C_FACTOR1 IS
         SELECT ELEMENT_NAME, LEVEL_ID, TYPE_ID, CHILD_COUNT, MAX_RANK,
         DELETED_COUNT, MITIGATION_COST, DESCRIPTION, RANK
         FROM TMT_FACTORS
         WHERE RANK LIKE h_rank || '%'
         ORDER BY RANK;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_COPY_FAC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_COPY_FAC'), '-- ' || SYSTIMESTAMP, ':r := FC_COPY_FAC(' || 
       '''' || h_source_id || ''''
          || ', ' || '''' || h_target_id || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_COPY_FAC', 45+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_COPY_FAC(46) : factors.sql(683) : ' || SYSTIMESTAMP);
   END IF; 

   --Declare local variables
   
   If h_source_id IS NULL OR h_target_id IS NULL Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_COPY_FAC', 52+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_COPY_FAC(53) : factors.sql(690) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.ELEMENT_ID_REQUIRED;
   End If;
   
   BEGIN
      SELECT RTRIM(RANK), RTRIM(PARENT_RANK) INTO h_rank, h_parent_rank
      FROM TMT_FACTORS
      WHERE ELEMENT_ID = h_source_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_COPY_FAC', 67+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_COPY_FAC(68) : factors.sql(705) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      BEGIN
         SELECT RTRIM(RANK), RTRIM(PARENT_RANK) INTO h_rank, h_parent_rank
         FROM TMT_FACTORS
         WHERE ELEMENT_ID = h_target_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_COPY_FAC', 83+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_COPY_FAC(84) : factors.sql(721) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         OPEN C_FACTOR;
         
         LOOP
            FETCH C_FACTOR INTO h_rank;
            IF C_FACTOR%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            OPEN C_FACTOR1;
            
            LOOP
               FETCH C_FACTOR1
               INTO h_element_name, h_level_id, h_type_id, h_child_count, h_max_rank,
               h_deleted_count, h_mitigation_cost,
               h_description, h_rank;
               IF C_FACTOR1
               %FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
               
               If Coalesce(h_level_id, 0) = 0 Then
                  t_max_rank := CASE WHEN Coalesce(h_max_rank, 0) > 32767 THEN BITAND(Coalesce(
                                                                             h_max_rank, 0), 65535) - 65536 ELSE BITAND(Coalesce(
                                                                             h_max_rank, 0), 65535) END;
                  PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_FACTORS', h_max_rank);
                  PMO_STPU.toBase64Long(Coalesce(h_max_rank, 0), h_rankBase64);
                  
                  BEGIN
                     SELECT RTRIM(RANK) INTO h_parent_rank
                     FROM TMT_CRI WHERE ELEMENT_ID = h_target_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_COPY_FAC', 136+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_COPY_FAC(137) : factors.sql(774) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  --strcpy(h_rank, h_rankBase64);
                  h_rank := h_rankBase64;
                  --strcpy(h_parent_id, h_target_id);
                  h_parent_id := h_target_id;
                  h_max_rank := t_max_rank;
                  prev_level_id := CASE WHEN Coalesce(h_level_id, 0) > 32767 THEN BITAND(Coalesce(
                                                                                h_level_id, 0), 65535) - 65536 ELSE BITAND(Coalesce(
                                                                                h_level_id, 0), 65535) END;
               ElsIf Coalesce(h_level_id, 0) > prev_level_id Then
                  h_rank := SubStr(h_rankBase64, 1, Coalesce(Length(h_rankBase64), 0));
                  --strcpy(h_parent_id, h_uniqueID);
                  h_parent_id := h_uniqueID;
                  prev_level_id := CASE WHEN Coalesce(h_level_id, 0) > 32767 THEN BITAND(Coalesce(
                                                                                h_level_id, 0), 65535) - 65536 ELSE BITAND(Coalesce(
                                                                                h_level_id, 0), 65535) END;
               ElsIf Coalesce(h_level_id, 0) < prev_level_id Then
                  h_rank := SubStr(h_rankBase64, 1, Coalesce(Length(h_rankBase64), 0));
                  
                  BEGIN
                     SELECT ELEMENT_ID, RANK INTO h_parent_id, h_parent_rank
                     FROM TMT_CRI
                     WHERE RANK = SUBSTR(h_rank, 1, LENGTH(h_rank) - 4);
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_COPY_FAC', 169+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_COPY_FAC(170) : factors.sql(807) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  prev_level_id := CASE WHEN Coalesce(h_level_id, 0) > 32767 THEN BITAND(Coalesce(
                                                                                h_level_id, 0), 65535) - 65536 ELSE BITAND(Coalesce(
                                                                                h_level_id, 0), 65535) END;
               Else
                  h_rank := SubStr(h_rankBase64, 1, Coalesce(Length(h_rankBase64), 0));
                  NULL;
               End If;
               
               h_uniqueID := SYS_GUID();
               
               BEGIN
                  INSERT INTO TMT_FACTORS(PARENT_RANK, RANK, PARENT_ID, ELEMENT_ID, TYPE_ID, LEVEL_ID, MAX_RANK,
                  ELEMENT_NAME, DELETED_COUNT, MITIGATION_COST,
                  DESCRIPTION, CHILD_COUNT, REC_USER)
                  VALUES(h_parent_rank, h_rank, h_parent_id, h_uniqueID, h_type_id, h_level_id, h_max_rank,
                  h_element_name, h_deleted_count, h_mitigation_cost,
                  h_description, h_child_count, h_rec_user);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_COPY_FAC', 200+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_COPY_FAC(201) : factors.sql(838) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            END LOOP;
            
            BEGIN
               CLOSE C_FACTOR1 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_COPY_FAC', 214+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_COPY_FAC(215) : factors.sql(852) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         END LOOP;
         
         BEGIN
            CLOSE C_FACTOR ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_COPY_FAC', 228+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_COPY_FAC(229) : factors.sql(866) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_COPY_FAC', 237+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_COPY_FAC(238) : factors.sql(875) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_COPY_FAC;
ALTER FUNCTION FC_COPY_FAC COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_D_FAC ==
CREATE OR REPLACE FUNCTION SP_D_FAC
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_current_datetime TIMESTAMP;
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
   h_rec_user VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_FAC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_FAC'), '-- ' || SYSTIMESTAMP, ':r := SP_D_FAC(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_FAC', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_FAC(27) : factors.sql(912) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate id
   h_id := p_id;
   h_parent_id := p_parent_id;
   h_rec_user := p_rec_user;
   h_rec_status := p_del_flag;
   
   If (p_id IS NULL) OR (p_parent_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_FAC', 39+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_FAC(40) : factors.sql(925) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ELEMENT_ID_REQUIRED);
   End If;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(256, 'SP_D_FAC', 44+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_DELETE: SP_D_FAC(45) : factors.sql(930) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_FACTORS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_VALIDATE_DELETE('TMT_FACTORS', h_id, h_rec_status);
   
   If SubStr(p_del_flag, 1, 1) = '?' Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_FAC', 50+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_FAC(51) : factors.sql(936) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR(0, p_id);
   End If;
   
   -- Execute Delete Statement
   If SubStr(p_del_flag, 1, 1) = 'X' Then
      PMO_DB2M.SC_STATE := FC_DELETE_FAC(h_id, h_parent_id, h_rec_user);
   Else
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_FAC', 61+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_FAC(62) : factors.sql(947) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.PARENT_MARKED_AS_DELETED);
      End If;
      
      BEGIN
         SELECT RANK INTO h_rank
         FROM TMT_FACTORS
         WHERE ELEMENT_ID = h_id AND PARENT_ID = h_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_FAC', 76+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_FAC(77) : factors.sql(962) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Update Parent's Hierarchy Info
      If Coalesce(Length(h_rank), 0) = 8 Then
         BEGIN
            UPDATE TMT_FACTORS
            SET CHILD_COUNT = CHILD_COUNT - 1,
            DELETED_COUNT = DELETED_COUNT + 1
            WHERE ELEMENT_ID IS NuLL;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_FAC', 98+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_FAC(99) : factors.sql(984) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TMT_FACTORS
            SET CHILD_COUNT = CHILD_COUNT - 1,
            DELETED_COUNT = DELETED_COUNT + 1
            WHERE ELEMENT_ID = h_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_FAC', 118+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_FAC(119) : factors.sql(1004) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         SELECT (SYSTIMESTAMP)  INTO  h_current_datetime FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_FAC', 132+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_FAC(133) : factors.sql(1018) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Lock Tree
      PMODEFINES.PMO_PMO_VERIFY_RANK(h_rank, 0);
      BEGIN
         UPDATE TMT_FACTORS
         SET REC_STATUS = p_del_flag/**del_flag*/,
         REC_DATETIME = h_current_datetime,
         REC_USER = p_rec_user/**rec_user:ind_d_fac->rec_user*/
         WHERE RANK LIKE h_rank || '%' AND REC_STATUS IN ('I','U');
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_FAC', 155+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_FAC(156) : factors.sql(1041) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --Update risk
      If Coalesce(Length(h_rank), 0) > 8 Then
         BEGIN
            SELECT PARENT_ID INTO h_parent_id
            FROM TMT_FACTORS
            WHERE RANK = SUBSTR(h_rank, 1, 8);
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_FAC', 172+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_FAC(173) : factors.sql(1058) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         SELECT RANK INTO h_rank
         FROM TMT_CRI
         WHERE ELEMENT_ID=h_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_FAC', 188+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_FAC(189) : factors.sql(1074) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- factor type is always the first level in tmt_factors, factor can't have
      -- another factor as its child
      BEGIN
         UPDATE TMT_CRI T
         SET T.MITIGATION_COST = (SELECT SUM(MITIGATION_COST)
         FROM TMT_FACTORS
         WHERE PARENT_ID = T.ELEMENT_ID
         AND TYPE_ID = PMO_CNST.SC_FACTOR
         AND REC_STATUS IN ('I', 'O', 'U')),
         T.REC_DATETIME = SYSTIMESTAMP,
         T.REC_USER = p_rec_user/**rec_user:ind_d_fac->rec_user*/
         WHERE (SUBSTR(h_rank, 1, CASE WHEN LENGTH(T.RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(T.RANK) END) = T.RANK 
         AND (T.RANK LIKE SUBSTR(h_rank, 1, 8) || '%'))
         AND T.TYPE_ID = PMO_CNST.SC_RISK
         AND T.REC_STATUS IN ('I', 'O', 'U');
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_FAC', 218+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_FAC(219) : factors.sql(1104) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE TMT_CRI T
         SET T.ESTIMATED_BUDGET = (SELECT SUM(MITIGATION_COST)
         FROM TMT_FACTORS
         WHERE PARENT_ID = T.ELEMENT_ID
         AND TYPE_ID = PMO_CNST.SC_FACTOR
         AND REC_STATUS IN ('I', 'O', 'U'))
         WHERE T.ELEMENT_ID=h_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_FAC', 241+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_FAC(242) : factors.sql(1127) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_FAC', 247+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_FAC(248) : factors.sql(1133) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_FAC', 254+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_FAC(255) : factors.sql(1140) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_FAC', 266+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_FAC(267) : factors.sql(1152) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_FAC;
ALTER FUNCTION SP_D_FAC COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_FAC ==
CREATE OR REPLACE FUNCTION SP_S_FAC
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
      -- Resultset Cursor Declaration
      C66674A PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C66674B PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_FAC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_FAC'), '-- ' || SYSTIMESTAMP, ':r := SP_S_FAC(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_FAC', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_FAC(23) : factors.sql(1191) : ' || SYSTIMESTAMP);
   END IF; 

   If p_id IS NULL Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_FAC', 27+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_FAC(28) : factors.sql(1196) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ELEMENT_ID_REQUIRED);
   End If;
   
   If (NOT (p_bitflag IS NULL)) AND (Coalesce(p_bitflag, 0) = 1) Then
      OPEN C66674A FOR SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
      T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_FACTORS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
      T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION, T.MITIGATION_COST,
      CASE WHEN T.TYPE_ID = PMO_CNST.SC_TRIGGER THEN NuLL ELSE T.PRIORITY END AS PRIORITY
      FROM TMT_FACTORS T
      WHERE (T.PARENT_ID = p_id OR T.ELEMENT_ID = p_id)
      AND T.REC_STATUS NOT IN ('D', 'L', 'A')
      ORDER BY RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_FAC', 42+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_FAC(43) : factors.sql(1211) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C66674A;
   Else
      OPEN C66674B FOR SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
      T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_FACTORS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
      T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION, T.MITIGATION_COST,
      CASE WHEN T.TYPE_ID = PMO_CNST.SC_TRIGGER THEN NuLL ELSE T.PRIORITY END AS PRIORITY
      FROM TMT_FACTORS T
      WHERE (T.PARENT_ID = p_id/*HOST(h_id)*/ OR T.ELEMENT_ID = p_id/*HOST(h_id)*/)
      ORDER BY RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_FAC', 54+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_FAC(55) : factors.sql(1223) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C66674B;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_FAC', 63+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_FAC(64) : factors.sql(1232) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_FAC', 75+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_FAC(76) : factors.sql(1244) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_FAC;
ALTER FUNCTION SP_S_FAC COMPILE DEBUG;

