-- IMPORTANT: this cpp contains all pure c functions that are used in many other CPPs,

-- please if you add a new function to this cpp put your function prototype in globalfunction.h

-- already in pmo_sqlprep.h : #include "pmodefines.h"

/********************************************************************************************************/

Prompt == FUNCTION GET_DAY_OF_WEEK ==
CREATE OR REPLACE FUNCTION GET_DAY_OF_WEEK
(
     h_date IN OUT NOCOPY VARCHAR2
   , h_day_name IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_day_of_week PLS_INTEGER := 0;
   null_return LONG;
BEGIN
   --DB2 WEEKOFDAY( SUN: 1, MON: 2, TUE: 3, WED: 4, THU: 5, FRI: 6, SAT: 7 )
   --PMO WEEKOFDAY( SUN: 0, MON: 1, TUE: 2, WED: 3, THU: 4, FRI: 5, SAT: 6 )
   
   BEGIN
      SELECT PMO_ORASTD.DAYOFWEEK(TO_DATE(SubStr(h_date, 1, 10)))
      INTO h_day_of_week
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_DAY_OF_WEEK', 22+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: GET_DAY_OF_WEEK(23) : globalfunction.sql(32) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   null_return := GetDayName(Coalesce(h_day_of_week, 0) - 1, h_day_name);
   h_day_name := h_day_name;
   
   RETURN Coalesce(h_day_of_week, 0) - 1;
   
END;
/
SHOW ERRORS FUNCTION GET_DAY_OF_WEEK;
ALTER FUNCTION GET_DAY_OF_WEEK COMPILE DEBUG;

/********************************************************************************************************/

Prompt == FUNCTION GetDayName ==
CREATE OR REPLACE FUNCTION GetDayName
(
     sh_day_of_week PLS_INTEGER
   , day_name IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   day_of_week PLS_INTEGER := Coalesce(sh_day_of_week, 0);
BEGIN
   If day_of_week = 6 Then
      day_name := 'Saturday';
   ElsIf day_of_week = 5 Then
      day_name := 'Friday';
   ElsIf day_of_week = 4 Then
      day_name := 'Thursday';
   ElsIf day_of_week = 3 Then
      day_name := 'Wednesday';
   ElsIf day_of_week = 2 Then
      day_name := 'Tuesday';
   ElsIf day_of_week = 1 Then
      day_name := 'Monday';
   ElsIf day_of_week = 0 Then
      day_name := 'Sunday';
   Else
      NULL;
   End If;
   
   RETURN day_of_week;
END;
/
SHOW ERRORS FUNCTION GetDayName;
ALTER FUNCTION GetDayName COMPILE DEBUG;

/********************************************************************************************************/

Prompt == FUNCTION GetDayOfWeekName ==
CREATE OR REPLACE FUNCTION GetDayOfWeekName
(
     sh_week_starting_day PLS_INTEGER
   , sh_day_of_week PLS_INTEGER
   , sh_day_offset PLS_INTEGER
   , day_name IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   week_starting_day PLS_INTEGER := Coalesce(sh_week_starting_day, 0);
   day_of_week PLS_INTEGER := Coalesce(sh_day_of_week, 0);
   day_offset PLS_INTEGER := Coalesce(sh_day_offset, 0);
   offset PLS_INTEGER NOT NULL := 0;
   result PLS_INTEGER NOT NULL := 0;
   target_day_of_year PLS_INTEGER NOT NULL := 8;
   null_return LONG;
BEGIN
   --trace_printf("week_starting_day: %d\n", week_starting_day);
   --trace_printf("day_of_week: %d\n", day_of_week);
   --trace_printf("day_offset: %d\n", day_offset);
   --trace_printf("day_name: %s\n", day_name);
   If day_offset = PMO_CNST.SC_SIX_DAYS_BEFORE_TODAY Then
      offset := -6;
   End If;
   If day_offset = PMO_CNST.SC_FIVE_DAYS_BEFORE_TODAY Then
      offset := -5;
   End If;
   If day_offset = PMO_CNST.SC_FOUR_DAYS_BEFORE_TODAY Then
      offset := -4;
   End If;
   If day_offset = PMO_CNST.SC_THREE_DAYS_BEFORE_TODAY Then
      offset := -3;
   End If;
   If day_offset = PMO_CNST.SC_TWO_DAYS_BEFORE_TODAY Then
      offset := -2;
   End If;
   
   If day_offset = PMO_CNST.SC_TWO_DAYS_AFTER_TODAY Then
      offset := 2;
   End If;
   If day_offset = PMO_CNST.SC_THREE_DAYS_AFTER_TODAY Then
      offset := 3;
   End If;
   If day_offset = PMO_CNST.SC_FOUR_DAYS_AFTER_TODAY Then
      offset := 4;
   End If;
   If day_offset = PMO_CNST.SC_FIVE_DAYS_AFTER_TODAY Then
      offset := 5;
   End If;
   If day_offset = PMO_CNST.SC_SIX_DAYS_AFTER_TODAY Then
      offset := 6;
   End If;
   
   --trace_printf("offset: %d\n", offset);
   If week_starting_day = day_of_week Then
      If offset < 0 Then
         result := 0;
      Else
         If offset < 7 Then
            target_day_of_year := (day_of_week + offset) MOD 7;
         Else
            result := 0;
         End If;
      End If;
   ElsIf week_starting_day > day_of_week Then
      If offset < 0 Then
         If (-1 * offset) <= (7 - (week_starting_day - day_of_week)) Then
            If (day_of_week + offset) >= 0 Then
               target_day_of_year := (day_of_week + offset) MOD 7;
            Else
               target_day_of_year := 7 + (day_of_week + offset);
            End If;
         Else
            result := 0;
         End If;
      Else
         If offset < (week_starting_day - day_of_week) Then
            target_day_of_year := (day_of_week + offset) MOD 7;
         Else
            result := 0;
         End If;
      End If;
   Else
      If offset < 0 Then
         --          if (((-1) * offset) < (7 - (day_of_week - week_starting_day)))
         If (-1 * offset) <= (day_of_week - week_starting_day) Then
            If (day_of_week + offset) >= 0 Then
               target_day_of_year := (day_of_week + offset) MOD 7;
            Else
               target_day_of_year := 7 + (day_of_week + offset);
            End If;
         Else
            result := 0;
         End If;
      Else
         If offset < (7 - (day_of_week - week_starting_day)) Then
            target_day_of_year := (day_of_week + offset) MOD 7;
         Else
            result := 0;
         End If;
      End If;
   End If;
   
   If (target_day_of_year >= 0) AND (target_day_of_year < 7) Then
      result := 1;
      null_return := GetDayName(target_day_of_year, day_name);
   Else
      result := 0;
      NULL;
   End If;
   
   --trace_printf("AAAAAAAAAAAAAAAAweek_starting_day: %d\n", week_starting_day);
   --trace_printf("day_of_week: %d\n", day_of_week);
   --trace_printf("day_offset: %d\n", day_offset);
   --trace_printf("day_name: %s\n", day_name);
   --trace_printf("result: %d\n", result);
   
   RETURN result;
END;
/
SHOW ERRORS FUNCTION GetDayOfWeekName;
ALTER FUNCTION GetDayOfWeekName COMPILE DEBUG;

/********************************************************************************************************/

-- project types in wbs view

Prompt == FUNCTION isProjectType ==
CREATE OR REPLACE FUNCTION isProjectType
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN ((type001 = PMO_CNST.SC_PROJECT) OR (type001 = PMO_CNST.SC_PROPOSAL) 
                 OR (type001 = PMO_CNST.SC_PROJ_TYPE1) OR (type001 = PMO_CNST.SC_PROJ_TYPE2) 
                 OR (type001 = PMO_CNST.SC_PROJ_TYPE3)) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isProjectType;
ALTER FUNCTION isProjectType COMPILE DEBUG;

/********************************************************************************************************/

-- not project types

Prompt == FUNCTION isNotProjectType ==
CREATE OR REPLACE FUNCTION isNotProjectType
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN (NOT (isProjectType(type001)) <> 0) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isNotProjectType;
ALTER FUNCTION isNotProjectType COMPILE DEBUG;

/********************************************************************************************************/

-- task types directly created in wbs view

Prompt == FUNCTION isPureWBSTask ==
CREATE OR REPLACE FUNCTION isPureWBSTask
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN ((type001 = PMO_CNST.SC_TASK) OR (type001 = PMO_CNST.SC_MILESTONE) 
                 OR (type001 = PMO_CNST.SC_TASK_TYPE1) OR (type001 = PMO_CNST.SC_TASK_TYPE2) 
                 OR (type001 = PMO_CNST.SC_TASK_TYPE3)) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isPureWBSTask;
ALTER FUNCTION isPureWBSTask COMPILE DEBUG;

/********************************************************************************************************/

-- task types transferred from cri or change to from transferred cri in wbs view

Prompt == FUNCTION isWBSTransferedTask ==
CREATE OR REPLACE FUNCTION isWBSTransferedTask
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN ((type001 = PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK) 
                 OR (type001 = PMO_CNST.SC_WBS_RISK_TASK) OR (type001 = PMO_CNST.SC_WBS_ISSUE_TASK) 
                 OR (type001 = PMO_CNST.SC_WBS_REQUIREMENT_TASK) 
                 OR (type001 = PMO_CNST.SC_WBS_DEFECT_TASK) 
                 OR (type001 = PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) 
                 OR (type001 = PMO_CNST.SC_WBS_ACTION_ITEM_TASK)) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isWBSTransferedTask;
ALTER FUNCTION isWBSTransferedTask COMPILE DEBUG;

/********************************************************************************************************/

-- all wbs task types

Prompt == FUNCTION isWBSTask ==
CREATE OR REPLACE FUNCTION isWBSTask
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN (isPureWBSTask(type001) <> 0 OR isWBSTransferedTask(type001) <> 0 
                 OR (type001 = PMO_CNST.SC_PERSONAL_TASK) 
                 OR (type001 = PMO_CNST.SC_ACTIVE_ADM_TASK)) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isWBSTask;
ALTER FUNCTION isWBSTask COMPILE DEBUG;

/********************************************************************************************************/

-- deliverable types directly created in wbs view

Prompt == FUNCTION isPureWBSDeliv ==
CREATE OR REPLACE FUNCTION isPureWBSDeliv
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN ((type001 = PMO_CNST.SC_DELIVERABLE) OR (type001 = PMO_CNST.SC_WORK_PRODUCT) 
                 OR (type001 = PMO_CNST.SC_SUMMARY_TASK) 
                 OR (type001 = PMO_CNST.SC_DELIVERABLE_TYPE1) 
                 OR (type001 = PMO_CNST.SC_DELIVERABLE_TYPE2) 
                 OR (type001 = PMO_CNST.SC_DELIVERABLE_TYPE3)) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isPureWBSDeliv;
ALTER FUNCTION isPureWBSDeliv COMPILE DEBUG;

/********************************************************************************************************/

-- deliverable types transferred from cri or change to from transferred cri in wbs view

Prompt == FUNCTION isWBSTransferedDeliv ==
CREATE OR REPLACE FUNCTION isWBSTransferedDeliv
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN ((type001 = PMO_CNST.SC_WBS_CHANGE_REQUEST) OR (type001 = PMO_CNST.SC_WBS_RISK) 
                 OR (type001 = PMO_CNST.SC_WBS_ISSUE) OR (type001 = PMO_CNST.SC_WBS_REQUIREMENT) 
                 OR (type001 = PMO_CNST.SC_WBS_DEFECT) 
                 OR (type001 = PMO_CNST.SC_WBS_SERVICE_REQUEST) 
                 OR (type001 = PMO_CNST.SC_WBS_ACTION_ITEM)) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isWBSTransferedDeliv;
ALTER FUNCTION isWBSTransferedDeliv COMPILE DEBUG;

/********************************************************************************************************/

-- all wbs deliverable types

Prompt == FUNCTION isWBSDeliv ==
CREATE OR REPLACE FUNCTION isWBSDeliv
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN (isPureWBSDeliv(type001) <> 0 OR isWBSTransferedDeliv(
                 type001) <> 0) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isWBSDeliv;
ALTER FUNCTION isWBSDeliv COMPILE DEBUG;

/********************************************************************************************************/

-- all wbs types

Prompt == FUNCTION isWBSType ==
CREATE OR REPLACE FUNCTION isWBSType
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN (isProjectType(type001) <> 0 OR isWBSDeliv(type001) <> 0 OR isWBSTask(
                 type001) <> 0) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isWBSType;
ALTER FUNCTION isWBSType COMPILE DEBUG;

/********************************************************************************************************/

-- reponsible task types directly created in cri view

Prompt == FUNCTION isCRIRespTask ==
CREATE OR REPLACE FUNCTION isCRIRespTask
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN ((type001 = PMO_CNST.SC_CRI_MILESTONE) 
                 OR (type001 = PMO_CNST.SC_CRI_TASK)) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isCRIRespTask;
ALTER FUNCTION isCRIRespTask COMPILE DEBUG;

/********************************************************************************************************/

-- task types change to from cri type in cri view

Prompt == FUNCTION isCRIConvertTask ==
CREATE OR REPLACE FUNCTION isCRIConvertTask
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN ((type001 = PMO_CNST.SC_CRI_CHANGE_REQUEST_TASK) 
                 OR (type001 = PMO_CNST.SC_CRI_RISK_TASK) OR (type001 = PMO_CNST.SC_CRI_ISSUE_TASK) 
                 OR (type001 = PMO_CNST.SC_CRI_REQUIREMENT_TASK) 
                 OR (type001 = PMO_CNST.SC_CRI_DEFECT_TASK) 
                 OR (type001 = PMO_CNST.SC_CRI_SERVICE_REQUEST_TASK) 
                 OR (type001 = PMO_CNST.SC_CRI_ACTION_ITEM_TASK)) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isCRIConvertTask;
ALTER FUNCTION isCRIConvertTask COMPILE DEBUG;

/********************************************************************************************************/

-- task types change to from cri type in cri view

Prompt == FUNCTION isCRITask ==
CREATE OR REPLACE FUNCTION isCRITask
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN (isCRIRespTask(type001) <> 0 OR isCRIConvertTask(type001) <> 0) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isCRITask;
ALTER FUNCTION isCRITask COMPILE DEBUG;

/********************************************************************************************************/

-- reponsible deliverable types directly created in cri view

Prompt == FUNCTION isCRIRespDeliv ==
CREATE OR REPLACE FUNCTION isCRIRespDeliv
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN ((type001 = PMO_CNST.SC_CRI_DELIVERABLE) 
                 OR (type001 = PMO_CNST.SC_CRI_SUMMARY_TASK)) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isCRIRespDeliv;
ALTER FUNCTION isCRIRespDeliv COMPILE DEBUG;

/********************************************************************************************************/

-- real cri types created in cri view

Prompt == FUNCTION isPureCRIDeliv ==
CREATE OR REPLACE FUNCTION isPureCRIDeliv
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN ((type001 = PMO_CNST.SC_CRI_CHANGE_REQUEST) OR (type001 = PMO_CNST.SC_CRI_RISK) 
                 OR (type001 = PMO_CNST.SC_CRI_ISSUE) OR (type001 = PMO_CNST.SC_CRI_REQUIREMENT) 
                 OR (type001 = PMO_CNST.SC_CRI_SERVICE_REQUEST) 
                 OR (type001 = PMO_CNST.SC_CRI_DEFECT) 
                 OR (type001 = PMO_CNST.SC_CRI_ACTION_ITEM)) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isPureCRIDeliv;
ALTER FUNCTION isPureCRIDeliv COMPILE DEBUG;

/********************************************************************************************************/

-- all cri deliverable types

Prompt == FUNCTION isCRIDeliv ==
CREATE OR REPLACE FUNCTION isCRIDeliv
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN (isPureCRIDeliv(type001) <> 0 OR isCRIRespDeliv(type001) <> 0) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isCRIDeliv;
ALTER FUNCTION isCRIDeliv COMPILE DEBUG;

/********************************************************************************************************/

-- all responsible type in cri view

--short isCRIRespElement(SC_SQL_LONG type)

Prompt == FUNCTION isCRIwbsElement ==
CREATE OR REPLACE FUNCTION isCRIwbsElement
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN (isCRIRespDeliv(type001) <> 0 OR isCRIRespTask(type001) <> 0) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isCRIwbsElement;
ALTER FUNCTION isCRIwbsElement COMPILE DEBUG;

/********************************************************************************************************/

-- all types in cri views

Prompt == FUNCTION isCRIType ==
CREATE OR REPLACE FUNCTION isCRIType
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN (isCRITask(type001) <> 0 OR isCRIDeliv(type001) <> 0 OR isCRIwbsElement(
                 type001) <> 0) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isCRIType;
ALTER FUNCTION isCRIType COMPILE DEBUG;

/********************************************************************************************************/

-- all task types

Prompt == FUNCTION isTask ==
CREATE OR REPLACE FUNCTION isTask
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN (isWBSTask(type001) <> 0 OR isCRITask(type001) <> 0) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isTask;
ALTER FUNCTION isTask COMPILE DEBUG;

/********************************************************************************************************/

-- all deliverable types

Prompt == FUNCTION isDeliv ==
CREATE OR REPLACE FUNCTION isDeliv
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN (isWBSDeliv(type001) <> 0 OR isCRIDeliv(type001) <> 0) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isDeliv;
ALTER FUNCTION isDeliv COMPILE DEBUG;

/********************************************************************************************************/

Prompt == FUNCTION isChangeToTask ==
CREATE OR REPLACE FUNCTION isChangeToTask
(
     sh_type001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   RETURN CASE WHEN (isWBSTransferedDeliv(type001) <> 0 OR isPureCRIDeliv(
                 type001) <> 0) THEN 1 ELSE 0 END;
END;
/
SHOW ERRORS FUNCTION isChangeToTask;
ALTER FUNCTION isChangeToTask COMPILE DEBUG;

/*******************************************************************************************************/

Prompt == FUNCTION getDayStatus ==
CREATE OR REPLACE FUNCTION getDayStatus
(
     sh_db_dayStatus PLS_INTEGER
) RETURN PLS_INTEGER
AS
   db_dayStatus PLS_INTEGER := Coalesce(sh_db_dayStatus, 0);
BEGIN
   If BITAND(db_dayStatus, 255) = 53 Then
      RETURN 51;
   ElsIf BITAND(db_dayStatus, 255) = 54 Then
      RETURN 52;
   Else
      RETURN db_dayStatus;
   End If;
END;
/
SHOW ERRORS FUNCTION getDayStatus;
ALTER FUNCTION getDayStatus COMPILE DEBUG;

/*******************************************************************************************************/

Prompt == FUNCTION GetTable ==
CREATE OR REPLACE FUNCTION GetTable
(
     sh_h_type_id PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := Coalesce(sh_h_type_id, 0);
BEGIN
   If h_type_id = PMO_CNST.SC_DOCUMENT OR h_type_id = PMO_CNST.SC_WP_LINK 
         OR h_type_id = PMO_CNST.SC_WORK_INPUT OR h_type_id = PMO_CNST.SC_SOFTWARE 
         OR h_type_id = PMO_CNST.SC_LINK OR h_type_id = PMO_CNST.SC_NOTE_MINUTES Then
      RETURN PMO_CNST.SC_TABLE_DOC;
   End If;
   
   If h_type_id = PMO_CNST.SC_POOLS Then
      RETURN PMO_CNST.SC_TABLE_POL;
   End If;
   
   If isWBSType(h_type_id) <> 0 Then
      RETURN PMO_CNST.SC_TABLE_WBS;
   End If;
   
   If isCRIType(h_type_id) <> 0 Then
      RETURN PMO_CNST.SC_TABLE_CRI;
   End If;
   
   RETURN 0;
END;
/
SHOW ERRORS FUNCTION GetTable;
ALTER FUNCTION GetTable COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION GetAssnType ==
CREATE OR REPLACE FUNCTION GetAssnType
(
     sh_h_type_id PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := Coalesce(sh_h_type_id, 0);
BEGIN
   If h_type_id = PMO_CNST.SC_WBS_CHANGE_REQUEST_ASSN Then
      RETURN PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_TASK_ASSN Then
      RETURN PMO_CNST.SC_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_WBS_CHANGE_REQUEST_ASSN Then
      RETURN PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_WBS_CHANGE_REQUEST_ASSN Then
      RETURN PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_WBS_SERVICE_REQUEST_ASSN Then
      RETURN PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_WBS_ISSUE_ASSN Then
      RETURN PMO_CNST.SC_WBS_ISSUE_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_WBS_RISK_ASSN Then
      RETURN PMO_CNST.SC_WBS_RISK_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_WBS_DEFECT_ASSN Then
      RETURN PMO_CNST.SC_WBS_DEFECT_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_WBS_ACTION_ITEM_ASSN Then
      RETURN PMO_CNST.SC_WBS_ACTION_ITEM_TASK;
   End If;
   
   RETURN 0;
END;
/
SHOW ERRORS FUNCTION GetAssnType;
ALTER FUNCTION GetAssnType COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION GetTaskType ==
CREATE OR REPLACE FUNCTION GetTaskType
(
     sh_h_type_id PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := Coalesce(sh_h_type_id, 0);
BEGIN
   If h_type_id = PMO_CNST.SC_CRI_CHANGE_REQUEST Then
      RETURN PMO_CNST.SC_CRI_CHANGE_REQUEST_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_RISK Then
      RETURN PMO_CNST.SC_CRI_RISK_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_ISSUE Then
      RETURN PMO_CNST.SC_CRI_ISSUE_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_REQUIREMENT Then
      RETURN PMO_CNST.SC_CRI_REQUIREMENT_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_DEFECT Then
      RETURN PMO_CNST.SC_CRI_DEFECT_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_SERVICE_REQUEST Then
      RETURN PMO_CNST.SC_CRI_SERVICE_REQUEST_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_ACTION_ITEM Then
      RETURN PMO_CNST.SC_CRI_ACTION_ITEM_TASK;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_CHANGE_REQUEST_TASK Then
      RETURN PMO_CNST.SC_CRI_CHANGE_REQUEST;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_RISK_TASK Then
      RETURN PMO_CNST.SC_CRI_RISK;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_ISSUE_TASK Then
      RETURN PMO_CNST.SC_CRI_ISSUE;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_REQUIREMENT_TASK Then
      RETURN PMO_CNST.SC_CRI_REQUIREMENT;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_DEFECT_TASK Then
      RETURN PMO_CNST.SC_CRI_DEFECT;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_SERVICE_REQUEST_TASK Then
      RETURN PMO_CNST.SC_CRI_SERVICE_REQUEST;
   End If;
   
   If h_type_id = PMO_CNST.SC_CRI_ACTION_ITEM_TASK Then
      RETURN PMO_CNST.SC_CRI_ACTION_ITEM;
   End If;
   
   RETURN 0;
END;
/
SHOW ERRORS FUNCTION GetTaskType;
ALTER FUNCTION GetTaskType COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION groupType ==
CREATE OR REPLACE FUNCTION groupType
(
     sh_type001 PLS_INTEGER
   , grouptype IN OUT NOCOPY PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type001 PLS_INTEGER := Coalesce(sh_type001, 0);
BEGIN
   If isProjectType(type001) <> 0 Then
      grouptype := PMO_CNST.SC_PROJECT;
   ElsIf isDeliv(type001) <> 0 Then
      grouptype := PMO_CNST.SC_DELIVERABLE;
   ElsIf isTask(type001) <> 0 Then
      grouptype := PMO_CNST.SC_TASK;
   Else
      grouptype := type001;
   End If;
   
   RETURN 0;
END;
/
SHOW ERRORS FUNCTION groupType;
ALTER FUNCTION groupType COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION GetSecurityName ==
CREATE OR REPLACE FUNCTION GetSecurityName
(
     sh_security_id PLS_INTEGER
   , security_name IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   security_id PLS_INTEGER := Coalesce(sh_security_id, 0);
BEGIN
   CASE security_id
      WHEN PMO_CNST.SC_MANAGER THEN
         security_name := 'SCPROJECTMANAGER';
      
      WHEN PMO_CNST.SC_CO_MANAGER THEN
         security_name := 'SCCOMANAGER';
      
      WHEN PMO_CNST.SC_DLV_MANAGER THEN
         security_name := 'SCDELIVERABLEMANAGER';
      
      WHEN PMO_CNST.SC_DLV_RESPONSIBLE THEN
         security_name := 'SCDELIVERABLERESPONSIBLE';
      
      WHEN PMO_CNST.SC_TEAMMEMBER THEN
         security_name := 'SCTEAMMEMBER';
      
      WHEN PMO_CNST.SC_TASK_RESPONSIBLE THEN
         security_name := 'SCTASKRESPONSIBLE';
      
      WHEN PMO_CNST.SC_FINANCE THEN
         security_name := 'SCFINANCE';
      
      WHEN PMO_CNST.SC_CO_FINANCE THEN
         security_name := 'SCCOFINANCE';
      
      WHEN PMO_CNST.SC_RESOURCE_MANAGER THEN
         security_name := 'SCRESOURCEMANAGER';
      
      WHEN PMO_CNST.SC_METHOD_AUTHOR THEN
         security_name := 'SCMETHODAUTHOR';
      
      WHEN PMO_CNST.SC_PROJECT_OFFICE THEN
         security_name := 'SCPROJECTOFFICE';
      
      WHEN PMO_CNST.SC_PARTICIPANT THEN
         security_name := 'SCPARTICIPANTS';
      
      WHEN PMO_CNST.SC_AFFECTED THEN
         security_name := 'SCPROJECTASSIGNMENT';
      
      WHEN PMO_CNST.SC_REQUIREMENT_RESPONSIBLE THEN
         security_name := 'SCREQUIREMENTRESPONSIBLE';
      
      WHEN PMO_CNST.SC_CHANGE_REQUEST_RESPONSIBLE THEN
         security_name := 'SCCHANGEREQUESTRESPONSIBLE';
      
      WHEN PMO_CNST.SC_DEFECT_RESPONSIBLE THEN
         security_name := 'SCDEFECTRESPONSIBLE';
      
      WHEN PMO_CNST.SC_SERVICE_REQUEST_RESPONSIBLE THEN
         security_name := 'SCSERVICEREQUESTRESPONSIBLE';
      
      WHEN PMO_CNST.SC_ISSUE_RESPONSIBLE THEN
         security_name := 'SCISSUERESPONSIBLE';
      
      WHEN PMO_CNST.SC_RISK_RESPONSIBLE THEN
         security_name := 'SCRISKRESPONSIBLE';
      
      WHEN PMO_CNST.SC_DOCUMENT_RESPONSIBLE THEN
         security_name := 'SCDOCUMENTRESPONSIBLE';
      
      WHEN PMO_CNST.SC_SCORECARD_RESPONSIBLE THEN
         security_name := 'SCSCORECARDRESPONSIBLE';
      ELSE
         NULL;
   END CASE;
   
   RETURN 0;
END;
/
SHOW ERRORS FUNCTION GetSecurityName;
ALTER FUNCTION GetSecurityName COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION GetPrefixName ==
CREATE OR REPLACE FUNCTION GetPrefixName
(
     sh_table_type PLS_INTEGER
   , prefix IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   table_type PLS_INTEGER := Coalesce(sh_table_type, 0);
BEGIN
   CASE table_type
      WHEN PMO_CNST.SC_TABLE_WBS THEN
         prefix := 'WBS#';
      
      WHEN PMO_CNST.SC_TABLE_CRI THEN
         prefix := 'CRI#';
      
      WHEN PMO_CNST.SC_TABLE_DOC THEN
         prefix := 'DOC#';
         --      case SC_TABLE_SRQ:
         --              strcpy(prefix, "SRQ#");
         --              break;
         --      case SC_TABLE_DFC:
         --              strcpy(prefix, "DFC#");
         --              break;
      
      WHEN PMO_CNST.SC_TABLE_RES THEN
         prefix := 'RES#';
      
      WHEN PMO_CNST.SC_TABLE_POL THEN
         prefix := 'POL#';
      
      WHEN PMO_CNST.SC_TABLE_AST THEN
         prefix := 'AST#';
      
      WHEN PMO_CNST.SC_TABLE_SCD THEN
         prefix := 'SCD#';
      
      WHEN PMO_CNST.SC_TABLE_CLN THEN
         prefix := 'CLN#';
      
      WHEN PMO_CNST.SC_TABLE_VDR THEN
         prefix := 'VDR#';
      ELSE
         NULL;
   END CASE;
   RETURN 0;
END;
/
SHOW ERRORS FUNCTION GetPrefixName;
ALTER FUNCTION GetPrefixName COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION GetSecurityID ==
CREATE OR REPLACE FUNCTION GetSecurityID
(
     sh_table_type PLS_INTEGER
   , sh_element_type PLS_INTEGER
   , sh_security_group PLS_INTEGER
   , security_id001 IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   table_type PLS_INTEGER := Coalesce(sh_table_type, 0);
   element_type PLS_INTEGER := Coalesce(sh_element_type, 0);
   security_group PLS_INTEGER := Coalesce(sh_security_group, 0);
BEGIN
   CASE security_group
      WHEN PMO_CNST.SC_CO_RESPONSIBLE_GROUP THEN
         -- valid just for project and proposal
         If isProjectType(element_type) <> 0 Then
            security_id001 := 'SCCOMANAGER';
         Else
            -- for folder and all other undefined ids
            security_id001 := NULL;
         End If;
         
      
      WHEN PMO_CNST.SC_RESPONSIBLE_GROUP THEN
         -- default security_id for RESPONSIBLE_GROUP
         If isProjectType(element_type) <> 0 Then
            security_id001 := 'SCPROJECTMANAGER';
         ElsIf isWBSTask(element_type) <> 0 OR isCRIRespTask(element_type) <> 0 Then
            security_id001 := 'SCTASKRESPONSIBLE';
         ElsIf isWBSDeliv(element_type) <> 0 OR isCRIRespDeliv(element_type) <> 0 Then
            security_id001 := 'SCDELIVERABLEMANAGER';
         ElsIf (element_type = PMO_CNST.SC_CRI_CHANGE_REQUEST) 
               OR (element_type = PMO_CNST.SC_CRI_CHANGE_REQUEST_TASK) Then
            security_id001 := 'SCCHANGEREQUESTRESPONSIBLE';
         ElsIf (element_type = PMO_CNST.SC_CRI_RISK) OR (element_type = PMO_CNST.SC_CRI_RISK_TASK) Then
            security_id001 := 'SCRISKRESPONSIBLE';
         ElsIf (element_type = PMO_CNST.SC_CRI_ISSUE) OR (element_type = PMO_CNST.SC_CRI_ISSUE_TASK) Then
            security_id001 := 'SCISSUERESPONSIBLE';
         ElsIf (element_type = PMO_CNST.SC_CRI_REQUIREMENT) 
               OR (element_type = PMO_CNST.SC_CRI_REQUIREMENT_TASK) Then
            security_id001 := 'SCREQUIREMENTRESPONSIBLE';
         ElsIf (element_type = PMO_CNST.SC_CRI_DEFECT) OR (element_type = PMO_CNST.SC_CRI_DEFECT_TASK) Then
            security_id001 := 'SCDEFECTRESPONSIBLE';
         ElsIf (element_type = PMO_CNST.SC_CRI_SERVICE_REQUEST) 
               OR (element_type = PMO_CNST.SC_CRI_SERVICE_REQUEST_TASK) Then
            security_id001 := 'SCSERVICEREQUESTRESPONSIBLE';
         ElsIf (element_type = PMO_CNST.SC_CRI_ACTION_ITEM) 
               OR (element_type = PMO_CNST.SC_CRI_ACTION_ITEM_TASK) Then
            security_id001 := 'SCACTIONITEMRESPONSIBLE';
         ElsIf (element_type = PMO_CNST.SC_WORK_INPUT) OR (element_type = PMO_CNST.SC_LINK) 
               OR (element_type = PMO_CNST.SC_SOFTWARE) OR (element_type = PMO_CNST.SC_WP_LINK) 
               OR (element_type = PMO_CNST.SC_DOCUMENT) 
               OR (element_type = PMO_CNST.SC_LAYOUT_SETTINGS) 
               OR (element_type = PMO_CNST.SC_PRINT_SETTINGS) 
               OR (element_type = PMO_CNST.SC_REPORT_SETTINGS) Then
            security_id001 := 'SCDOCUMENTRESPONSIBLE';
         ElsIf element_type = PMO_CNST.SC_NOTE_MINUTES Then
            security_id001 := 'SCNOTEMINUTESRESPONSIBLE';
         ElsIf element_type = PMO_CNST.SC_POOLS Then
            security_id001 := 'SCRESOURCEMANAGER';
         ElsIf element_type = PMO_CNST.SC_SCORECARD Then
            security_id001 := 'SCSCORECARDRESPONSIBLE';
         ElsIf element_type = PMO_CNST.SC_FOLDER Then
            security_id001 := 'SCFOLDERMANAGER';
         Else
            -- for folder and all other undefined ids
            security_id001 := NULL;
         End If;
         
         
      
      WHEN PMO_CNST.SC_PARTICIPANT_GROUP THEN
         -- default security_id for PARTICIPANT_GROUP
         If isProjectType(element_type) <> 0 Then
            security_id001 := 'SCPROJECTPARTICIPANT';
         ElsIf isWBSTask(element_type) <> 0 OR isCRIRespTask(element_type) <> 0 Then
            security_id001 := 'SCTASKPARTICIPANT';
         ElsIf isWBSDeliv(element_type) <> 0 OR isCRIRespDeliv(element_type) <> 0 Then
            security_id001 := 'SCDELIVERABLEPARTICIPANT';
         ElsIf (element_type = PMO_CNST.SC_CRI_CHANGE_REQUEST) 
               OR (element_type = PMO_CNST.SC_CRI_CHANGE_REQUEST_TASK) Then
            security_id001 := 'SCCHANGEREQUESTPARTICIPANT';
         ElsIf (element_type = PMO_CNST.SC_CRI_RISK) OR (element_type = PMO_CNST.SC_CRI_RISK_TASK) Then
            security_id001 := 'SCRISKPARTICIPANT';
         ElsIf (element_type = PMO_CNST.SC_CRI_ISSUE) OR (element_type = PMO_CNST.SC_CRI_ISSUE_TASK) Then
            security_id001 := 'SCISSUEPARTICIPANT';
         ElsIf (element_type = PMO_CNST.SC_CRI_REQUIREMENT) 
               OR (element_type = PMO_CNST.SC_CRI_REQUIREMENT_TASK) Then
            security_id001 := 'SCREQUIREMENTPARTICIPANT';
         ElsIf (element_type = PMO_CNST.SC_CRI_DEFECT) OR (element_type = PMO_CNST.SC_CRI_DEFECT_TASK) Then
            security_id001 := 'SCDEFECTPARTICIPANT';
         ElsIf (element_type = PMO_CNST.SC_CRI_SERVICE_REQUEST) 
               OR (element_type = PMO_CNST.SC_CRI_SERVICE_REQUEST_TASK) Then
            security_id001 := 'SCSERVICEREQUESTPARTICIPANT';
         ElsIf (element_type = PMO_CNST.SC_CRI_ACTION_ITEM) 
               OR (element_type = PMO_CNST.SC_CRI_ACTION_ITEM_TASK) Then
            security_id001 := 'SCACTIONITEMPARTICIPANT';
         ElsIf (element_type = PMO_CNST.SC_WORK_INPUT) OR (element_type = PMO_CNST.SC_LINK) 
               OR (element_type = PMO_CNST.SC_SOFTWARE) OR (element_type = PMO_CNST.SC_WP_LINK) 
               OR (element_type = PMO_CNST.SC_DOCUMENT) Then
            security_id001 := 'SCDOCUMENTPARTICIPANT';
         ElsIf element_type = PMO_CNST.SC_NOTE_MINUTES Then
            security_id001 := 'SCNOTEMINUTESPARTICIPANT';
         ElsIf element_type = PMO_CNST.SC_POOLS Then
            security_id001 := 'SCPOOLPARTICIPANT';
         ElsIf element_type = PMO_CNST.SC_SCORECARD Then
            security_id001 := 'SCSCORECARDPARTICIPANT';
         ElsIf element_type = PMO_CNST.SC_FOLDER Then
            security_id001 := 'SCFOLDERPARTICIPANT';
         Else
            -- for folder and all other undefined ids
            security_id001 := NULL;
         End If;
         
         
      
      WHEN PMO_CNST.SC_TEAMMEMBER_GROUP THEN
         If isProjectType(element_type) <> 0 Then
            security_id001 := 'SCPROJECTASSIGNMENT';
         ElsIf isWBSTask(element_type) <> 0 OR isCRIRespTask(element_type) <> 0 OR isWBSTransferedDeliv(
               element_type) <> 0 Then
            security_id001 := 'SCTASKTEAMMEMBER';
         ElsIf (element_type = PMO_CNST.SC_CRI_CHANGE_REQUEST) 
               OR (element_type = PMO_CNST.SC_CRI_CHANGE_REQUEST_TASK) 
               OR (element_type = PMO_CNST.SC_WBS_CHANGE_REQUEST) Then
            security_id001 := 'SCCHANGEREQUESTTEAMMEMBER';
         ElsIf (element_type = PMO_CNST.SC_CRI_RISK) OR (element_type = PMO_CNST.SC_CRI_RISK_TASK) 
               OR (element_type = PMO_CNST.SC_WBS_RISK) Then
            security_id001 := 'SCRISKTEAMMEMBER';
         ElsIf (element_type = PMO_CNST.SC_CRI_ISSUE) OR (element_type = PMO_CNST.SC_CRI_ISSUE_TASK) 
               OR (element_type = PMO_CNST.SC_WBS_ISSUE) Then
            security_id001 := 'SCISSUETEAMMEMBER';
         ElsIf (element_type = PMO_CNST.SC_CRI_REQUIREMENT) 
               OR (element_type = PMO_CNST.SC_CRI_REQUIREMENT_TASK) 
               OR (element_type = PMO_CNST.SC_WBS_REQUIREMENT) Then
            security_id001 := 'SCREQUIREMENTTEAMMEMBER';
         ElsIf (element_type = PMO_CNST.SC_CRI_DEFECT) OR (element_type = PMO_CNST.SC_CRI_DEFECT_TASK) 
               OR (element_type = PMO_CNST.SC_WBS_DEFECT) Then
            security_id001 := 'SCDEFECTTEAMMEMBER';
         ElsIf (element_type = PMO_CNST.SC_CRI_SERVICE_REQUEST) 
               OR (element_type = PMO_CNST.SC_CRI_SERVICE_REQUEST_TASK) 
               OR (element_type = PMO_CNST.SC_WBS_SERVICE_REQUEST) Then
            security_id001 := 'SCSERVICEREQUESTTEAMMEMBER';
         ElsIf (element_type = PMO_CNST.SC_CRI_ACTION_ITEM) 
               OR (element_type = PMO_CNST.SC_CRI_ACTION_ITEM_TASK) 
               OR (element_type = PMO_CNST.SC_WBS_ACTION_ITEM) Then
            security_id001 := 'SCACTIONITEMTEAMMEMBER';
         Else
            -- for folder and all other undefined ids
            security_id001 := NULL;
         End If;
         
         
      
      WHEN PMO_CNST.SC_WKF_VISITOR_GROUP THEN
         If isProjectType(element_type) <> 0 Then
            security_id001 := 'SCPROJECTWORKFLOWVISITOR';
         ElsIf isWBSTask(element_type) <> 0 OR isCRIRespTask(element_type) <> 0 Then
            security_id001 := 'SCTASKWORKFLOWVISITOR';
         ElsIf isWBSDeliv(element_type) <> 0 OR isCRIRespDeliv(element_type) <> 0 Then
            security_id001 := 'SCDELIVERABLEWORKFLOWVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_CHANGE_REQUEST) 
               OR (element_type = PMO_CNST.SC_CRI_CHANGE_REQUEST_TASK) Then
            security_id001 := 'SCCHANGEREQUESTWORKFLOWVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_RISK) OR (element_type = PMO_CNST.SC_CRI_RISK_TASK) Then
            security_id001 := 'SCRISKWORKFLOWVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_ISSUE) OR (element_type = PMO_CNST.SC_CRI_ISSUE_TASK) Then
            security_id001 := 'SCISSUEWORKFLOWVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_REQUIREMENT) 
               OR (element_type = PMO_CNST.SC_CRI_REQUIREMENT_TASK) Then
            security_id001 := 'SCREQUIREMENTWORKFLOWVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_DEFECT) OR (element_type = PMO_CNST.SC_CRI_DEFECT_TASK) Then
            security_id001 := 'SCDEFECTWORKFLOWVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_SERVICE_REQUEST) 
               OR (element_type = PMO_CNST.SC_CRI_SERVICE_REQUEST_TASK) Then
            security_id001 := 'SCSERVICEREQUESTWORKFLOWVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_ACTION_ITEM) 
               OR (element_type = PMO_CNST.SC_CRI_ACTION_ITEM_TASK) Then
            security_id001 := 'SCACTIONITEMWORKFLOWVISITOR';
         ElsIf (element_type = PMO_CNST.SC_WORK_INPUT) OR (element_type = PMO_CNST.SC_LINK) 
               OR (element_type = PMO_CNST.SC_SOFTWARE) OR (element_type = PMO_CNST.SC_WP_LINK) 
               OR (element_type = PMO_CNST.SC_DOCUMENT) Then
            security_id001 := 'SCDOCUMENTWORKFLOWVISITOR';
         ElsIf element_type = PMO_CNST.SC_NOTE_MINUTES Then
            security_id001 := 'SCNOTEMINUTESWORKFLOWVISITOR';
         ElsIf element_type = PMO_CNST.SC_SCORECARD Then
            security_id001 := 'SCSCORECARDWORKFLOWVISITOR';
         Else
            security_id001 := 'SCPROJECTWORKFLOWVISITOR';
         End If;
         -- for folder and all other undefined ids
         
         
      
      WHEN PMO_CNST.SC_VISITOR_GROUP THEN
         If (element_type = PMO_CNST.SC_CRI_CHANGE_REQUEST) 
               OR (element_type = PMO_CNST.SC_CRI_CHANGE_REQUEST_TASK) Then
            security_id001 := 'SCCHANGEREQUESTVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_RISK) OR (element_type = PMO_CNST.SC_CRI_RISK_TASK) Then
            security_id001 := 'SCRISKVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_ISSUE) OR (element_type = PMO_CNST.SC_CRI_ISSUE_TASK) Then
            security_id001 := 'SCISSUEVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_REQUIREMENT) 
               OR (element_type = PMO_CNST.SC_CRI_REQUIREMENT_TASK) Then
            security_id001 := 'SCREQUIREMENTVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_DEFECT) OR (element_type = PMO_CNST.SC_CRI_DEFECT_TASK) Then
            security_id001 := 'SCDEFECTVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_SERVICE_REQUEST) 
               OR (element_type = PMO_CNST.SC_CRI_SERVICE_REQUEST_TASK) Then
            security_id001 := 'SCSERVICEREQUESTVISITOR';
         ElsIf (element_type = PMO_CNST.SC_CRI_ACTION_ITEM) 
               OR (element_type = PMO_CNST.SC_CRI_ACTION_ITEM_TASK) Then
            security_id001 := 'SCACTIONITEMVISITOR';
         ElsIf (element_type = PMO_CNST.SC_WORK_INPUT) OR (element_type = PMO_CNST.SC_LINK) 
               OR (element_type = PMO_CNST.SC_SOFTWARE) OR (element_type = PMO_CNST.SC_WP_LINK) 
               OR (element_type = PMO_CNST.SC_DOCUMENT) Then
            security_id001 := 'SCDOCUMENTVISITOR';
         ElsIf element_type = PMO_CNST.SC_NOTE_MINUTES Then
            security_id001 := 'SCNOTEMINUTESVISITOR';
         ElsIf element_type = PMO_CNST.SC_SCORECARD Then
            security_id001 := 'SCSCORECARDVISITOR';
         ElsIf element_type = PMO_CNST.SC_FOLDER Then
            security_id001 := 'SCFOLDERVISITOR';
         Else
            -- for folder and all other undefined ids
            security_id001 := NULL;
         End If;
         
      ELSE
         NULL;
   END CASE;
   RETURN 0;
END;
/
SHOW ERRORS FUNCTION GetSecurityID;
ALTER FUNCTION GetSecurityID COMPILE DEBUG;

/*********************************************************************************************/

-- this function returns the security_group_id related to a security_id

Prompt == FUNCTION GetSecurityGroup ==
CREATE OR REPLACE FUNCTION GetSecurityGroup
(
     h_security_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
BEGIN
   If PMO_ORASTD.strncmp(h_security_id, 'SCCOMANAGER', 11) = 0 Then
      RETURN PMO_CNST.SC_CO_RESPONSIBLE_GROUP;
   ElsIf (PMO_ORASTD.strncmp(h_security_id, 'SCPROJECTMANAGER', 16) = 0) OR (PMO_ORASTD.strncmp(
         h_security_id, 'SCTASKRESPONSIBLE', 17) = 0) OR (PMO_ORASTD.strncmp(h_security_id
         , 'SCDELIVERABLEMANAGER', 20) = 0) OR (PMO_ORASTD.strncmp(h_security_id
         , 'SCCHANGEREQUESTRESPONSIBLE', 26) = 0) OR (PMO_ORASTD.strncmp(h_security_id
         , 'SCRISKRESPONSIBLE', 17) = 0) OR (PMO_ORASTD.strncmp(h_security_id, 'SCISSUERESPONSIBLE'
         , 18) = 0) OR (PMO_ORASTD.strncmp(h_security_id, 'SCREQUIREMENTRESPONSIBLE', 24) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCDEFECTRESPONSIBLE', 19) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCSERVICEREQUESTRESPONSIBLE', 27) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCACTIONITEMRESPONSIBLE', 23) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCDOCUMENTRESPONSIBLE', 21) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCNOTEMINUTESRESPONSIBLE', 24) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCRESOURCEMANAGER', 17) = 0) OR (PMO_ORASTD.strncmp(
         h_security_id, 'SCSCORECARDRESPONSIBLE', 22) = 0) Then
      RETURN PMO_CNST.SC_RESPONSIBLE_GROUP;
   ElsIf (PMO_ORASTD.strncmp(h_security_id, 'SCPROJECTPARTICIPANT', 20) = 0) OR (PMO_ORASTD.strncmp(
         h_security_id, 'SCTASKPARTICIPANT', 17) = 0) OR (PMO_ORASTD.strncmp(h_security_id
         , 'SCDELIVERABLEPARTICIPANT', 24) = 0) OR (PMO_ORASTD.strncmp(h_security_id
         , 'SCCHANGEREQUESTPARTICIPANT', 26) = 0) OR (PMO_ORASTD.strncmp(h_security_id
         , 'SCRISKPARTICIPANT', 17) = 0) OR (PMO_ORASTD.strncmp(h_security_id, 'SCISSUEPARTICIPANT'
         , 18) = 0) OR (PMO_ORASTD.strncmp(h_security_id, 'SCREQUIREMENTPARTICIPANT', 24) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCDEFECTPARTICIPANT', 19) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCSERVICEREQUESTPARTICIPANT', 27) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCACTIONITEMPARTICIPANT', 23) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCDOCUMENTPARTICIPANT', 21) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCNOTEMINUTESPARTICIPANT', 24) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCPOOLPARTICIPANT', 17) = 0) OR (PMO_ORASTD.strncmp(
         h_security_id, 'SCSCORECARDSPARTICIPANT', 23) = 0) Then
      RETURN PMO_CNST.SC_PARTICIPANT_GROUP;
   ElsIf (PMO_ORASTD.strncmp(h_security_id, 'SCPROJECTASSIGNMENT', 19) = 0) OR (PMO_ORASTD.strncmp(
         h_security_id, 'SCTASKTEAMMEMBER', 16) = 0) OR (PMO_ORASTD.strncmp(h_security_id
         , 'SCCHANGEREQUESTTEAMMEMBER', 25) = 0) OR (PMO_ORASTD.strncmp(h_security_id
         , 'SCRISKTEAMMEMBER', 16) = 0) OR (PMO_ORASTD.strncmp(h_security_id, 'SCISSUETEAMMEMBER'
         , 17) = 0) OR (PMO_ORASTD.strncmp(h_security_id, 'SCREQUIREMENTTEAMMEMBER', 23) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCDEFECTTEAMMEMBER', 18) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCSERVICEREQUESTTEAMMEMBER', 26) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCACTIONITEMTEAMMEMBER', 22) = 0) Then
      RETURN PMO_CNST.SC_TEAMMEMBER_GROUP;
   ElsIf (PMO_ORASTD.strncmp(h_security_id, 'SCPROJECTWORKFLOWVISITOR', 24) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCTASKWORKFLOWVISITOR', 21) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCDELIVERABLEWORKFLOWVISITOR', 28) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCCHANGEREQUESTWORKFLOWVISITOR', 30) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCRISKWORKFLOWVISITOR', 21) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCISSUEWORKFLOWVISITOR', 22) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCREQUIREMENTWORKFLOWVISITOR', 28) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCDEFECTWORKFLOWVISITOR', 23) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCSERVICEREQUESTWORKFLOWVISITOR', 31) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCACTIONITEMWORKFLOWVISITOR', 27) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCDOCUMENTWORKFLOWVISITOR', 25) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCNOTEMINUTESWORKFLOWVISITOR', 28) = 0) 
         OR (PMO_ORASTD.strncmp(h_security_id, 'SCSCORECARDWORKFLOWVISITOR', 26) = 0) Then
      RETURN PMO_CNST.SC_WKF_VISITOR_GROUP;
   End If;
   
   RETURN 0;
END;
/
SHOW ERRORS FUNCTION GetSecurityGroup;
ALTER FUNCTION GetSecurityGroup COMPILE DEBUG;

/***********************************************************************/

Prompt == FUNCTION getCloseStage ==
CREATE OR REPLACE FUNCTION getCloseStage
(
     sh_type_id PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type_id PLS_INTEGER := Coalesce(sh_type_id, 0);
   stage_id PLS_INTEGER;
BEGIN
   If isPureWBSTask(type_id) <> 0 Then
      stage_id := 5521;
   ElsIf isPureWBSDeliv(type_id) <> 0 Then
      stage_id := 5491;
   ElsIf (type_id = PMO_CNST.SC_CRI_CHANGE_REQUEST) OR (type_id = PMO_CNST.SC_CRI_CHANGE_REQUEST_TASK) 
         OR (type_id = PMO_CNST.SC_WBS_CHANGE_REQUEST) 
         OR (type_id = PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK) Then
      stage_id := 4910;
   ElsIf (type_id = PMO_CNST.SC_CRI_RISK) OR (type_id = PMO_CNST.SC_CRI_RISK_TASK) 
         OR (type_id = PMO_CNST.SC_WBS_RISK) OR (type_id = PMO_CNST.SC_WBS_RISK_TASK) Then
      stage_id := 4920;
   ElsIf (type_id = PMO_CNST.SC_CRI_REQUIREMENT) OR (type_id = PMO_CNST.SC_CRI_REQUIREMENT_TASK) 
         OR (type_id = PMO_CNST.SC_WBS_REQUIREMENT) 
         OR (type_id = PMO_CNST.SC_WBS_REQUIREMENT_TASK) Then
      stage_id := 4930;
   ElsIf (type_id = PMO_CNST.SC_CRI_ISSUE) OR (type_id = PMO_CNST.SC_CRI_ISSUE_TASK) 
         OR (type_id = PMO_CNST.SC_WBS_ISSUE) OR (type_id = PMO_CNST.SC_WBS_ISSUE_TASK) Then
      stage_id := 4939;
   ElsIf (type_id = PMO_CNST.SC_CRI_DEFECT) OR (type_id = PMO_CNST.SC_CRI_DEFECT_TASK) 
         OR (type_id = PMO_CNST.SC_WBS_DEFECT) OR (type_id = PMO_CNST.SC_WBS_DEFECT_TASK) Then
      stage_id := 5386;
   ElsIf (type_id = PMO_CNST.SC_CRI_ACTION_ITEM) OR (type_id = PMO_CNST.SC_CRI_ACTION_ITEM_TASK) 
         OR (type_id = PMO_CNST.SC_WBS_ACTION_ITEM) 
         OR (type_id = PMO_CNST.SC_WBS_ACTION_ITEM_TASK) Then
      stage_id := 6117;
   ElsIf (type_id = PMO_CNST.SC_CRI_SERVICE_REQUEST) 
         OR (type_id = PMO_CNST.SC_CRI_SERVICE_REQUEST_TASK) 
         OR (type_id = PMO_CNST.SC_WBS_SERVICE_REQUEST) 
         OR (type_id = PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) Then
      stage_id := 5417;
   End If;
   
   RETURN stage_id;
END;
/
SHOW ERRORS FUNCTION getCloseStage;
ALTER FUNCTION getCloseStage COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION getTaskDefaultStage ==
CREATE OR REPLACE FUNCTION getTaskDefaultStage
(
     sh_type_id PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type_id PLS_INTEGER := Coalesce(sh_type_id, 0);
   stage_id PLS_INTEGER NOT NULL := 0;
BEGIN
   If type_id = PMO_CNST.SC_TASK Then
      stage_id := 5500;
   ElsIf type_id = PMO_CNST.SC_WBS_REQUIREMENT_TASK Then
      stage_id := 4871;
   ElsIf type_id = PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK Then
      stage_id := 4724;
   ElsIf type_id = PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK Then
      stage_id := 5415;
   ElsIf type_id = PMO_CNST.SC_WBS_ACTION_ITEM_TASK Then
      stage_id := 6082;
   ElsIf type_id = PMO_CNST.SC_WBS_DEFECT_TASK Then
      stage_id := 5384;
   ElsIf type_id = PMO_CNST.SC_WBS_ISSUE_TASK Then
      stage_id := 4828;
   ElsIf type_id = PMO_CNST.SC_WBS_RISK_TASK Then
      stage_id := 4849;
   End If;
   
   RETURN stage_id;
END;
/
SHOW ERRORS FUNCTION getTaskDefaultStage;
ALTER FUNCTION getTaskDefaultStage COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION getTaskCompletedStage ==
CREATE OR REPLACE FUNCTION getTaskCompletedStage
(
     sh_type_id PLS_INTEGER
) RETURN PLS_INTEGER
AS
   type_id PLS_INTEGER := Coalesce(sh_type_id, 0);
   stage_id PLS_INTEGER NOT NULL := 0;
BEGIN
   If type_id = PMO_CNST.SC_TASK Then
      stage_id := 5520;
   ElsIf type_id = PMO_CNST.SC_WBS_REQUIREMENT_TASK Then
      stage_id := 4929;
   ElsIf type_id = PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK Then
      stage_id := 4909;
   ElsIf type_id = PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK Then
      stage_id := 5416;
   ElsIf type_id = PMO_CNST.SC_WBS_ACTION_ITEM_TASK Then
      stage_id := 6116;
   ElsIf type_id = PMO_CNST.SC_WBS_DEFECT_TASK Then
      stage_id := 5385;
   ElsIf type_id = PMO_CNST.SC_WBS_ISSUE_TASK Then
      stage_id := 4938;
   ElsIf type_id = PMO_CNST.SC_WBS_RISK_TASK Then
      stage_id := 4919;
   End If;
   
   RETURN stage_id;
END;
/
SHOW ERRORS FUNCTION getTaskCompletedStage;
ALTER FUNCTION getTaskCompletedStage COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION ConversionType ==
CREATE OR REPLACE FUNCTION ConversionType
(
     sh_h_current_type PLS_INTEGER
   , sh_h_new_type PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_current_type PLS_INTEGER := Coalesce(sh_h_current_type, 0);
   h_new_type PLS_INTEGER := Coalesce(sh_h_new_type, 0);
   m_lConversionType PLS_INTEGER NOT NULL := 0;
   m_current_type_group PLS_INTEGER;
   m_new_type_group PLS_INTEGER;
BEGIN
   If isProjectType(h_current_type) <> 0 AND isProjectType(h_new_type) <> 0 Then
      m_lConversionType := 1;
      -- SC_PRJ_2_PRJ
   ElsIf isPureWBSTask(h_current_type) <> 0 AND isPureWBSTask(h_new_type) <> 0 Then
      m_lConversionType := 2;
      -- SC_PURE_WBSTASK_2_PURE_WBSTASK
   ElsIf isPureWBSDeliv(h_current_type) <> 0 AND isPureWBSDeliv(h_new_type) <> 0 Then
      m_lConversionType := 3;
      -- SC_PURE_WBSDEL_2_PURE_WBSDEL
   ElsIf isWBSTransferedDeliv(h_current_type) <> 0 AND isWBSTransferedDeliv(h_new_type) <> 0 Then
      m_lConversionType := 4;
      -- SC_TRAN_WBSDEL_2_TRAN_WBSDEL
   ElsIf isWBSTransferedDeliv(h_current_type) <> 0 AND isWBSTransferedTask(h_new_type) <> 0 Then
      m_lConversionType := 5;
      -- SC_TRAN_WBSDEL_2_TRAN_WBSTASK
   ElsIf isWBSTransferedTask(h_current_type) <> 0 AND isWBSTransferedTask(h_new_type) <> 0 Then
      m_lConversionType := 6;
      -- SC_TRAN_WBSTASK_2_TRAN_WBSTASK
   ElsIf isWBSTransferedTask(h_current_type) <> 0 AND isWBSTransferedDeliv(h_new_type) <> 0 Then
      m_lConversionType := 7;
      -- SC_TRAN_WBSTASK_2_TRAN_WBSDEL
   ElsIf isCRIConvertTask(h_current_type) <> 0 AND isCRIConvertTask(h_new_type) <> 0 Then
      m_lConversionType := 8;
      -- SC_CONV_CRITASK_2_CONV_CRITASK
   ElsIf isCRIConvertTask(h_current_type) <> 0 AND isPureCRIDeliv(h_new_type) <> 0 Then
      m_lConversionType := 9;
      -- SC_CONV_CRITASK_2_PURE_CRIDEL
   ElsIf isPureCRIDeliv(h_current_type) <> 0 AND isCRIConvertTask(h_new_type) <> 0 Then
      m_lConversionType := 10;
      -- SC_PURE_CRIDEL_2_CONV_CRITASK
   ElsIf isPureCRIDeliv(h_current_type) <> 0 AND isPureCRIDeliv(h_new_type) <> 0 Then
      m_lConversionType := 11;
      -- SC_PURE_CRIDEL_2_PURE_CRIDEL
   End If;
   
   RETURN m_lConversionType;
END;
/
SHOW ERRORS FUNCTION ConversionType;
ALTER FUNCTION ConversionType COMPILE DEBUG;

/*************************************************************************/

Prompt == FUNCTION IsCRISystemFolder ==
CREATE OR REPLACE FUNCTION IsCRISystemFolder
(
     sh_h_type_id PLS_INTEGER
   , sh_h_stage_id PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := Coalesce(sh_h_type_id, 0);
   h_stage_id PLS_INTEGER := Coalesce(sh_h_stage_id, 0);
BEGIN
   If h_type_id = PMO_CNST.SC_FOLDER Then
      If (h_stage_id = PMO_CNST.SC_PROJECT) OR (h_stage_id = PMO_CNST.SC_CRI_REQUIREMENT) 
            OR (h_stage_id = PMO_CNST.SC_CRI_CHANGE_REQUEST) 
            OR (h_stage_id = PMO_CNST.SC_CRI_SERVICE_REQUEST) 
            OR (h_stage_id = PMO_CNST.SC_CRI_ACTION_ITEM) OR (h_stage_id = PMO_CNST.SC_CRI_DEFECT) 
            OR (h_stage_id = PMO_CNST.SC_CRI_ISSUE) OR (h_stage_id = PMO_CNST.SC_CRI_RISK) Then
         RETURN 1;
      End If;
   End If;
   RETURN 0;
END;
/
SHOW ERRORS FUNCTION IsCRISystemFolder;
ALTER FUNCTION IsCRISystemFolder COMPILE DEBUG;

/*************************************************************************/

Prompt == FUNCTION IsDOCSystemFolder ==
CREATE OR REPLACE FUNCTION IsDOCSystemFolder
(
     sh_h_type_id PLS_INTEGER
   , sh_h_stage_id PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := Coalesce(sh_h_type_id, 0);
   h_stage_id PLS_INTEGER := Coalesce(sh_h_stage_id, 0);
BEGIN
   If h_type_id = PMO_CNST.SC_FOLDER Then
      If h_stage_id = PMO_CNST.SC_PROJECT Then
         RETURN 1;
      End If;
   End If;
   RETURN 0;
END;
/
SHOW ERRORS FUNCTION IsDOCSystemFolder;
ALTER FUNCTION IsDOCSystemFolder COMPILE DEBUG;

