/********************************************************************************************************************************/

-- EXTEND CALENDAR

/********************************************************************************************************************************/

Prompt == FUNCTION EXTEND_PRJCAL ==
CREATE OR REPLACE FUNCTION EXTEND_PRJCAL
(
     h_id IN OUT NOCOPY VARCHAR2
   , h_dayofyear IN OUT NOCOPY DATE
) RETURN PLS_INTEGER
AS
   h_project_id VARCHAR2(32);
   h_status_date DATE;
   h_total PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('EXTEND_PRJCAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('EXTEND_PRJCAL'), '-- ' || SYSTIMESTAMP, ':r := EXTEND_PRJCAL(' || 
       '''' || h_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_dayofyear || '''',''''''), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_PRJCAL', 19+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: EXTEND_PRJCAL(20) : impexwbs.sql(27) : ' || SYSTIMESTAMP);
   END IF; 

   --SC_CHAR(h_dayofyear, SC_H_DATE);
   
   h_project_id := h_id;
   h_type := PMO_CNST.SC_PROJECT;
   
   -- Check for project calendar
   BEGIN
      SELECT
      TO_DATE(SubStr(h_dayofyear, 1, 10)) + 1     ,
      CASE WHEN MAX(DAY_YEAR) > TO_DATE(SubStr(h_dayofyear, 1, 10)) THEN 0 ELSE 1 END,
      TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1     
      INTO
      h_dayofyear,
      h_total,
      h_status_date
      FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRJCAL', 44+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRJCAL(45) : impexwbs.sql(52) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      If Coalesce(h_total, 0) = 1 Then
         h_temp_var := 0;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_PRJCAL', 53+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: EXTEND_PRJCAL(54) : impexwbs.sql(61) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN POPULATE_CALENDAR(h_project_id, h_type, h_status_date, h_dayofyear, h_temp_var);
      Else
         PMO_DB2M.SC_STATE := 0;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_PRJCAL', 62+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: EXTEND_PRJCAL(63) : impexwbs.sql(70) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION EXTEND_PRJCAL;
ALTER FUNCTION EXTEND_PRJCAL COMPILE DEBUG;

/********************************************************************************************************************************/

Prompt == FUNCTION FC_PORTF_PRJ ==
CREATE OR REPLACE FUNCTION FC_PORTF_PRJ
(
     h_resource_id IN OUT NOCOPY VARCHAR2
   , h_id IN OUT NOCOPY VARCHAR2
   , h_ranklike IN OUT NOCOPY VARCHAR2
   , sh_h_type PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_type PLS_INTEGER := sh_h_type;
   h_rank VARCHAR2(200);
   h_prj_level_id PLS_INTEGER := 0;
   h_prj_element_name VARCHAR2(255);
   h_prj_element_id VARCHAR2(32);
   h_prj_rank VARCHAR2(200);
   h_subproject_count PLS_INTEGER := 0;
   myTeam PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR myteamprj IS
         SELECT T.LEVEL_ID, T.ELEMENT_ID, T.ELEMENT_NAME, T.RANK, SUBPROJECT_COUNT
         FROM PROJECT_ASSIGNMENTS A, TMT_WBS T
         WHERE A.RESOURCE_ID = h_resource_id
         AND T.PROJECT_ID = A.PROJECT_ID
         AND T.ELEMENT_ID = A.PROJECT_ID
         AND T.REC_STATUS <> 'D'
         AND BITAND(A.PARTICIPANT, 1219) > 0
         AND T.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
         ORDER BY T.RANK;

      CURSOR prjrecset IS
         SELECT T.LEVEL_ID, T.ELEMENT_ID, T.ELEMENT_NAME, T.RANK, SUBPROJECT_COUNT
         FROM PROJECT_ASSIGNMENTS A, TMT_WBS T
         WHERE A.RESOURCE_ID = h_resource_id
         AND T.PROJECT_ID = A.PROJECT_ID
         AND T.ELEMENT_ID = A.PROJECT_ID
         AND T.REC_STATUS <> 'D'
         AND (BITAND(A.PARTICIPANT, 3) > 0 OR (BITAND(A.PARTICIPANT, 3) = 0 AND T.PUBLISHED ='1' ))
         AND T.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
         ORDER BY T.RANK;

      CURSOR C_PORT IS
         SELECT RANK FROM PMO_TMP_PROJECTS_01;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_PORTF_PRJ') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_PORTF_PRJ'), '-- ' || SYSTIMESTAMP, ':r := FC_PORTF_PRJ(' || 
       '''' || h_resource_id || ''''
          || ', ' || '''' || h_id || ''''
          || ', ' || '''' || h_ranklike || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_PORTF_PRJ', 52+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_PORTF_PRJ(53) : impexwbs.sql(133) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_resource_id: %d:%s, h_ranklike: %d:%s\n",h_resource_id_Ind,h_resource_id,h_ranklike_Ind,h_ranklike);
   --trace_printf("h_id: %d:%s, h_type: %d\n",h_id_Ind,h_id,h_type);
   
   If Coalesce(h_type, 0) >= 10000 Then
      myTeam := 1;
      h_type := Coalesce(h_type, 0) - 10000;
   End If;
   
   If (h_ranklike IS NULL) OR (Coalesce(Length(h_ranklike), 0) = 0) Then
      If Coalesce(h_type, 0) = 0 Then
         BEGIN
            SELECT TYPE_ID, RANK INTO h_type, h_rank
            FROM TMT_WBS WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 74+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(75) : impexwbs.sql(155) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               SELECT RANK, SUBSTR(RANK,1,8), TYPE_ID
               INTO h_rank, h_ranklike, h_type
               FROM TMT_PORTFOLIOS WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 90+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(91) : impexwbs.sql(171) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_type := PMO_CNST.SC_FOLDER;
               
               BEGIN
                  SELECT ELEMENT_ID INTO h_resource_id
                  FROM TMT_PORTFOLIOS WHERE RANK = h_ranklike AND TYPE_ID = 13;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 107+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(108) : impexwbs.sql(188) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_PORTF_PRJ', 113+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_PORTF_PRJ(114) : impexwbs.sql(194) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RECORD_NOT_FOUND;
               NULL;
            End If;
         Else
            h_type := PMO_CNST.SC_PROJECT;
         End If;
      ElsIf (Coalesce(h_type, 0) = PMO_CNST.SC_FOLDER) OR (Coalesce(
                h_type, 0) = PMO_CNST.SC_TABLE_PORTFOLIO) Then
         BEGIN
            SELECT RANK, SUBSTR(RANK,1,8)
            INTO h_rank, h_ranklike
            FROM TMT_PORTFOLIOS WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 133+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(134) : impexwbs.sql(214) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_PORTF_PRJ', 140+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_PORTF_PRJ(141) : impexwbs.sql(221) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         
         If (h_resource_id IS NULL) OR (Coalesce(Length(h_resource_id), 0) = 0) Then
            BEGIN
               SELECT ELEMENT_ID INTO h_resource_id
               FROM TMT_PORTFOLIOS WHERE RANK = h_ranklike AND TYPE_ID = 13;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 155+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(156) : impexwbs.sql(236) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      ElsIf Coalesce(h_type, 0) = PMO_CNST.SC_PROJECT Then
         BEGIN
            SELECT RANK INTO h_rank FROM TMT_WBS WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 169+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(170) : impexwbs.sql(250) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_PORTF_PRJ', 176+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_PORTF_PRJ(177) : impexwbs.sql(257) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
      End If;
      
      h_ranklike := h_rank;
   End If;
   h_ranklike := h_ranklike || '%%';
   --trace_printf("*** h_ranklike: %d:%s\n",h_ranklike_Ind,h_ranklike);
   
   If (h_id IS NULL) AND (Coalesce(Length(h_ranklike), 0) < 8) Then
      -- select all projects that user assigned to them
      If myTeam = 1 Then
         OPEN myteamprj;
      Else
         OPEN prjrecset;
      End If;
      h_rank := 'QQ';
      LOOP
         If myTeam = 1 Then
            FETCH myteamprj
            INTO h_prj_level_id, h_prj_element_id, h_prj_element_name,
            h_prj_rank, h_subproject_count;
            IF myteamprj
            %FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         Else
            FETCH prjrecset
            INTO h_prj_level_id, h_prj_element_id, h_prj_element_name,
            h_prj_rank, h_subproject_count;
            IF prjrecset
            %FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         End If;
         --timeLog(file, "TTTTTTTT01");
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         If (Coalesce(Length(h_prj_rank), 0) <= Coalesce(Length(h_rank), 0)) OR (PMO_ORASTD.strncmp(h_rank
               , h_prj_rank, Coalesce(Length(h_rank), 0)) <> 0) Then
            h_rank := h_prj_rank;
            
            If Coalesce(h_subproject_count, 0) > 0 Then
               -- insert sub projects
               BEGIN
                  INSERT INTO PMO_TMP_PROJECTS_01 (LEVEL_ID, ELEMENT_ID, ELEMENT_NAME, RANK)
                  SELECT T.LEVEL_ID, T.ELEMENT_ID, T.ELEMENT_NAME, T.RANK
                  FROM TMT_WBS T
                  WHERE T.RANK LIKE h_prj_rank || '%'
                  AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
                  AND REC_STATUS <> 'D';
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 245+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(246) : impexwbs.sql(326) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               -- insert sub projects
               BEGIN
                  INSERT INTO PMO_TMP_PROJECTS_01 (LEVEL_ID, ELEMENT_ID, ELEMENT_NAME, RANK)
                  VALUES(h_prj_level_id, h_prj_element_id, h_prj_element_name, h_prj_rank);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 264+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(265) : impexwbs.sql(345) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      END LOOP;
      
      If myTeam = 1 Then
         BEGIN
            CLOSE myteamprj ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 281+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(282) : impexwbs.sql(362) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            CLOSE prjrecset ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 294+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(295) : impexwbs.sql(375) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   ElsIf (Coalesce(h_type, 0) = PMO_CNST.SC_FOLDER) OR (Coalesce(
                h_type, 0) = PMO_CNST.SC_TABLE_PORTFOLIO) Then
      If Coalesce(h_type, 0) = PMO_CNST.SC_FOLDER Then
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_PROJECTS_COPY';
         PMO_DB2M.SC_STATE := 0;
         
         BEGIN
            INSERT INTO PMO_TMP_PROJECTS_COPY (LEVEL_ID, ELEMENT_ID, ELEMENT_NAME, RANK, PROJECT_FLAG)
            SELECT T.LEVEL_ID, T.ELEMENT_ID, T.ELEMENT_NAME, T.RANK,  0 AS PROJECT_FLAG
            FROM PROJECT_ASSIGNMENTS A JOIN TMT_WBS T
            ON T.PROJECT_ID = A.PROJECT_ID AND T.ELEMENT_ID = A.PROJECT_ID AND T.REC_STATUS <> 'D'
            WHERE A.RESOURCE_ID = h_resource_id
            AND T.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
            AND (BITAND(A.PARTICIPANT, 3) > 0 OR (BITAND(A.PARTICIPANT, 3) = 0 AND T.PUBLISHED ='1' ));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 324+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(325) : impexwbs.sql(405) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         INSERT INTO PMO_TMP_PROJECTS_01 (LEVEL_ID, ELEMENT_ID, ELEMENT_NAME, RANK)
         SELECT W.LEVEL_ID, W.ELEMENT_ID, W.ELEMENT_NAME, W.RANK
         FROM TMT_PORTFOLIOS P JOIN PMO_TMP_PROJECTS_COPY W ON W.ELEMENT_ID = P.ELEMENT_ID
         WHERE P.RANK LIKE h_ranklike AND P.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND P.REC_STATUS <> 'D';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 345+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(346) : impexwbs.sql(426) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM PMO_TMP_PROJECTS_01
         WHERE ELEMENT_ID IN (SELECT B.ELEMENT_ID
         FROM PMO_TMP_PROJECTS_COPY A, PMO_TMP_PROJECTS_COPY B
         WHERE A.RANK < B.RANK AND ((A.RANK >= SUBSTR(B.RANK, 1, 8) AND A.RANK <= B.RANK) 
         AND SUBSTR(B.RANK,1, CASE WHEN LENGTH(A.RANK) > LENGTH(B.RANK) THEN 1 ELSE LENGTH(A.RANK) END ) = A.RANK));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 366+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(367) : impexwbs.sql(447) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- insert sub-projects into the temporary table
      
      OPEN C_PORT;
      
      LOOP
         FETCH C_PORT INTO h_rank;
         IF C_PORT%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         --trace_printf("RANK: %s\n",h_rank);
         
         BEGIN
            INSERT INTO PMO_TMP_PROJECTS_01 (LEVEL_ID, ELEMENT_ID, ELEMENT_NAME, RANK)
            SELECT T.LEVEL_ID, T.ELEMENT_ID, T.ELEMENT_NAME, T.RANK
            FROM TMT_WBS T
            WHERE T.RANK LIKE h_rank || '%' AND T.RANK > h_rank AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND REC_STATUS <> 'D';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 403+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(404) : impexwbs.sql(484) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      END LOOP;
      
      BEGIN
         CLOSE C_PORT ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 417+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(418) : impexwbs.sql(498) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         INSERT INTO PMO_TMP_PROJECTS_01 (LEVEL_ID, ELEMENT_ID, ELEMENT_NAME, RANK)
         SELECT LEVEL_ID, ELEMENT_ID, ELEMENT_NAME, RANK
         FROM TMT_WBS
         WHERE RANK LIKE h_ranklike AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND REC_STATUS <> 'D';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PORTF_PRJ', 437+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PORTF_PRJ(438) : impexwbs.sql(518) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --trace_printf("===== SC_STATE: %d\n",SC_STATE);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_PORTF_PRJ', 445+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_PORTF_PRJ(446) : impexwbs.sql(526) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_PORTF_PRJ;
ALTER FUNCTION FC_PORTF_PRJ COMPILE DEBUG;

/*****************************************************************************************/

-- Team Member Select *** for Import Export  *** TESTED ***

/*****************************************************************************************/

/*****************************************************************************************/

/******************************************************************************************/

-- h_bitflag = 1 update ACTUAL  MILESTONE_PERCENT

-- h_bitflag = 2 update BASELINE start/finish dates  duration

/*****************************************************************************************/

Prompt == FUNCTION SP_ISLOCKED_BYME ==
CREATE OR REPLACE FUNCTION SP_ISLOCKED_BYME
(
     p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   i PLS_INTEGER;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_ISLOCKED_BYME') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_ISLOCKED_BYME'), '-- ' || SYSTIMESTAMP, ':r := SP_ISLOCKED_BYME(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ISLOCKED_BYME', 17+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_ISLOCKED_BYME(18) : impexwbs.sql(568) : ' || SYSTIMESTAMP);
   END IF; 

   i := 0;
   PMO_DB2M.SC_STATE := 0;
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ISLOCKED_BYME', 26+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ISLOCKED_BYME(27) : impexwbs.sql(577) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ISLOCKED_BYME', 38+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ISLOCKED_BYME(39) : impexwbs.sql(589) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_ISLOCKED_BYME;
ALTER FUNCTION SP_ISLOCKED_BYME COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION SP_S_IMPEX_WBS ==
CREATE OR REPLACE FUNCTION SP_S_IMPEX_WBS
(
     p_id VARCHAR2
   , p_bigflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_calendar_id VARCHAR2(32);
   h_calendar_name VARCHAR2(254);
   h_resource_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_ranklike VARCHAR2(200);
   h_rec_status VARCHAR2(1);
   h_type PLS_INTEGER := 0;
   stmt VARCHAR2(29999);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_IMPEX_WBS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_IMPEX_WBS'), '-- ' || SYSTIMESTAMP, ':r := SP_S_IMPEX_WBS(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bigflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_IMPEX_WBS', 27+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_IMPEX_WBS(28) : impexwbs.sql(633) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_PROJECTS_01';
   PMO_DB2M.SC_STATE := 0;
   
   h_calendar_id := p_id;
   h_resource_id := p_rec_user;
   
   BEGIN
      SELECT RANK, TYPE_ID, PROJECT_ID
      INTO h_rank, h_type, h_project_id
      FROM TMT_WBS WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_WBS', 49+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_WBS(50) : impexwbs.sql(655) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d,h_rank: %s\n", SC_STATE,h_rank);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      BEGIN
         SELECT RTRIM(RANK), REPLACE( RTRIM(ELEMENT_NAME), '''', ''''''), TYPE_ID INTO h_rank, h_calendar_name, h_type
         FROM TMT_PORTFOLIOS WHERE ELEMENT_ID = h_calendar_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_WBS', 65+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_WBS(66) : impexwbs.sql(671) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_IMPEX_WBS', 73+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_IMPEX_WBS(74) : impexwbs.sql(679) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, p_id);
   End If;
   
   PMO_DB2M.SC_STATE := FC_PORTF_PRJ(h_resource_id, h_calendar_id, h_rank, h_type);
   
   h_ranklike := h_rank;
   h_ranklike := h_ranklike || '%';
   BEGIN
      INSERT INTO PMO_TMP_WBS_01 (LEVEL_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_NAME, RANK)
      SELECT LEVEL_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_NAME, RANK
      FROM TMT_WBS 
      WHERE PROJECT_ID IN (SELECT DISTINCT ELEMENT_ID FROM PMO_TMP_PROJECTS_01);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_WBS', 97+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_WBS(98) : impexwbs.sql(703) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("h_type: %d,h_ranklike: %s\n", h_type,h_ranklike); //trace_flush();
   
   If Coalesce(h_type, 0) = PMO_CNST.SC_FOLDER Then
      stmt := 'SELECT ''1'' AS ROW_TYPE, ''0000'' AS RANK, 0 AS LEVEL_ID, '''' AS PARENT_ID, ''';
      stmt := stmt || h_calendar_id;
      stmt := stmt || ''' AS ELEMENT_ID, ''';
      stmt := stmt || h_calendar_name;
      stmt := stmt || ''' AS NAME, 64 AS TYPE_ID, 64 AS ELEMENT_TYPE, ''FOLDER'' AS EXTERNAL_ID, ''';
      stmt := stmt || h_calendar_id;
      stmt := stmt || ''' AS CALENDAR_ID, ''';
      stmt := stmt || h_calendar_name;
      stmt := stmt || 
                ''' AS CALENDAR_NAME, ''N'' AS CRITICAL, 500 AS PRIORITY, 480 AS MINUTE_PER_DAY, 2400 AS MINUTE_PER_WEEK, '
             || ''''' AS RESOURCE_ID, '''' AS RESPONSIBLE, '''' AS E_MAIL_ADDRESS, 0 AS CONSTRAINT_TYPE, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS CONSTRAINT_DT, 1 AS CHILD_COUNT, 0 AS ASSIGN_COUNT, '
             || '0 AS SD, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS START_DT, 0 AS FD, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS FINISH_DT, 0 AS DURATION, '
             || 'TO_TIMESTAMP(''1899-12-31 00:00:00'') AS BASELINE1_START, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS BASELINE1_FINISH, 0 AS BASELINE1_DURATION, 0 AS BASELINE1_EFFORT, '
             || 'TO_TIMESTAMP(''1899-12-31 00:00:00'') AS BASELINE2_START, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS BASELINE2_FINISH, 0 AS BASELINE2_DURATION, 0 AS BASELINE2_EFFORT, '
             || 'TO_TIMESTAMP(''1899-12-31 00:00:00'') AS BASELINE3_START, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS BASELINE3_FINISH, 0 AS BASELINE3_DURATION, 0 AS BASELINE3_EFFORT, '
             || 'TO_TIMESTAMP(''1899-12-31 00:00:00'') AS PLANNED_START, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS PLANNED_FINISH, 0 AS PLANNED_DURATION, 0 AS PLANNED_EFFORT, '
             || 'TO_TIMESTAMP(''1899-12-31 00:00:00'') AS PROPOSED_START, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS PROPOSED_FINISH, 0 AS PROPOSED_DURATION, 0 AS PROPOSED_EFFORT, '
             || 'TO_TIMESTAMP(''1899-12-31 00:00:00'') AS FORECAST_START, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS FORECAST_FINISH, 0 AS FORECAST_DURATION, 0 AS FORECAST_EFFORT, '
             || 'TO_TIMESTAMP(''1899-12-31 00:00:00'') AS SCHED_START, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS SCHED_FINISH, 0 AS SCHED_DURATION, 0 AS SCHED_EFFORT, '
             || 'TO_TIMESTAMP(''1899-12-31 00:00:00'') AS ACTUAL_START, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS ACTUAL_FINISH, 0 AS ACTUAL_DURATION, 0 AS ACTUAL_EFFORT, '
             || '0 AS UNRESOLVED_DURATION, '''' AS DEPEND_ID, '''' AS IS_EXTERNAL, '''' AS IS_PRED, 0 AS DEPENDENCY_TYPE, 0 AS LEAD_OR_LAG_TYPE, 0 AS AMOUNT, 0 AS LAG_DISPLAY_UNIT, '
             || ''''' AS PROJECT_ID, '''' AS PROJECT_NAME, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS PROJECT_START_DT, '''' AS PROJECT_FINISH_DT, '
             || ''''' AS CREATOR_ID, '''' AS CREATOR_NAME, '''' AS CREATOR_E_MAIL_ADDRESS, '
             || '''1899-12-31 1899-12-31'' AS START_FINISH_DATE, ''0'' AS START_FINISH_FLAG, 0 AS TOTAL_SLACK, 0 AS FREE_SLACK, '
             || ''''' AS RES_ASSIGNMENT_ID, '''' AS RESOURCE_TYPE, 0 AS MILESTONE_PERCENT, 0 AS PERCENT_COMPLETE, '
             || '0 AS REMAINING_WORK, 0 AS WORK_TODATE, 0 AS CP_EFFORT, 0 AS CP_UNIT, '
             || ''''' AS ASSIGN_CONTOUR, 0 AS BEST, 0 AS ESTIMATE, 0 AS ETC, 0.00 AS REGULAR_COST, 0.00 AS SPECIAL_COST, 0.00 AS REGULAR_RATE, 0.00 AS SPECIAL_RATE, 0 AS MSP_ID, '''' AS LAST_SUBMITTED, 0 AS ACTUAL_TODATE, '''' AS WBS_CODE, 0 AS TASK_TYPE, 0 AS UNITS_MSP, 0 AS CONTOUR_TYPE '
             || 'FROM DUAL UNION ALL '
             || 'SELECT ''1'' AS ROW_TYPE, TW.RANK, TW.LEVEL_ID+1 AS LEVEL_ID, CASE WHEN TW.TYPE_ID IN (';
      stmt := stmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
      stmt := stmt || ') THEN ''';
      stmt := stmt || h_calendar_id;
      stmt := stmt || ''' ELSE TW.PARENT_ID END AS PARENT_ID, TW.ELEMENT_ID, TW.ELEMENT_NAME AS NAME, '
             || 'TW.TYPE_ID, TW.TYPE_ID AS ELEMENT_TYPE, '''' AS EXTERNAL_ID, '
             || 'TW.CALENDAR_ID, TC.ELEMENT_NAME AS CALENDAR_NAME, CASE WHEN TW.EFFORT_DRIVEN = 1 THEN ''Y'' ELSE ''N'' END AS CRITICAL, TW.PRIORITY, TW.MINUTE_PER_DAY, TW.MINUTE_PER_WEEK, '
             || 'RA.RESOURCE_ID, RC.FULL_NAME AS RESPONSIBLE, RC.E_MAIL_ADDRESS, TW.CONSTRAINT_TYPE, TW.CONSTRAINT_DT, TW.CHILD_COUNT, TW.ASSIGN_COUNT, '
             || 'TW.SD, TW.START_DT, TW.FD, TW.FINISH_DT, TW.DURATION, '
             || 'TW.BASELINE1_START, TW.BASELINE1_FINISH, TW.BASELINE1_DURATION, TW.BASELINE1_EFFORT, '
             || 'TW.BASELINE2_START, TW.BASELINE2_FINISH, TW.BASELINE2_DURATION, TW.BASELINE2_EFFORT, '
             || 'TW.BASELINE3_START, TW.BASELINE3_FINISH, TW.BASELINE3_DURATION, TW.BASELINE3_EFFORT, '
             || 'TW.PLANNED_START, TW.PLANNED_FINISH, TW.PLANNED_DURATION, TW.PLANNED_EFFORT, '
             || 'TW.PROPOSED_START, TW.PROPOSED_FINISH, TW.PROPOSED_DURATION, TW.PROPOSED_EFFORT, '
             || 'TW.FORECAST_START, TW.FORECAST_FINISH, TW.FORECAST_DURATION, TW.FORECAST_EFFORT, '
             || 'TW.SCHED_START, TW.SCHED_FINISH, TW.SCHED_DURATION, TW.SCHED_EFFORT, '
             || 'TW.ACTUAL_START, TW.ACTUAL_FINISH, TW.ACTUAL_DURATION, TW.ACTUAL_EFFORT, '
             || 'TW.UNRESOLVED_DURATION, '
             || ''''' AS DEPEND_ID, CASE WHEN TW.TYPE_ID IN (';
      stmt := stmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
      stmt := stmt || 
                ') AND TW.FS_CODE IS NOT NULL THEN ''F'' ELSE '''' END AS IS_EXTERNAL, '''' AS IS_PRED, 0 AS DEPENDENCY_TYPE, 0 AS LEAD_OR_LAG_TYPE, 0 AS AMOUNT, 0 AS LAG_DISPLAY_UNIT, '
             || ''''' AS PROJECT_ID, '''' AS PROJECT_NAME, TW.REC_DATETIME AS PROJECT_START_DT, '''' AS PROJECT_FINISH_DT, '
             || ''''' AS CREATOR_ID, '''' AS CREATOR_NAME, '''' AS CREATOR_E_MAIL_ADDRESS, '
             || 'TO_CHAR( TO_DATE(SubStr(TW.START_FINISH_DATE, 1, 10))) || '' '' || TO_CHAR( TO_DATE(SubStr(TW.START_FINISH_DATE, 1, 10))) AS START_FINISH_DATE, '
             || 'TW.START_FINISH_FLAG, TW.TOTAL_SLACK, TW.FREE_SLACK, '
             || ''''' AS RES_ASSIGNMENT_ID, '''' AS RESOURCE_TYPE, Trunc( TW.MILESTONE_PERCENT) AS MILESTONE_PERCENT, CASE WHEN TW.SCHED_EFFORT > 0 THEN ( (TW.WORK_TODATE / TW.SCHED_EFFORT) * 100) ELSE 0 END AS PERCENT_COMPLETE, '
             || '( TW.PLANNED_EFFORT + TW.SCHED_EFFORT - TW.WORK_TODATE) AS REMAINING_WORK, TW.WORK_TODATE AS WORK_TODATE, 0 AS CP_EFFORT, 0 AS CP_UNIT, '
             || ''''' AS ASSIGN_CONTOUR, 0 AS BEST, 0 AS ESTIMATE, TW.ETC AS ETC, 0.00 AS REGULAR_COST, 0.00 AS SPECIAL_COST, 0.00 AS REGULAR_RATE, 0.00 AS SPECIAL_RATE, TW.MSP_ID, '''' AS LAST_SUBMITTED, 0 AS ACTUAL_TODATE, TW.WBS_CODE, TW.TASK_TYPE, 0 AS UNITS_MSP, 0 AS CONTOUR_TYPE '
             || 'FROM PMO_TMP_PROJECTS_01 TMP, (TMT_WBS TW JOIN RESOURCE_ASSIGNMENTS RA ON RA.ELEMENT_ID = TW.ELEMENT_ID AND BITAND(RA.PARTICIPANT, ';
      stmt := stmt || TO_CHAR(PMO_CNST.SC_MANAGER_RESPONSIBLE);
      stmt := stmt || ') > 0) '
             || 'JOIN RESOURCES RC ON RC.RESOURCE_ID = RA.RESOURCE_ID '
             || 'LEFT OUTER JOIN TMT_CALENDARS TC ON TC.ELEMENT_ID = TW.CALENDAR_ID '
             || 'WHERE TW.PROJECT_ID = TMP.ELEMENT_ID';
   Else
      stmt := 
            'SELECT ''1'' AS ROW_TYPE, TW.RANK, TW.LEVEL_ID, TW.PARENT_ID, TW.ELEMENT_ID, TW.ELEMENT_NAME AS NAME, '
             || 'TW.TYPE_ID, TW.TYPE_ID AS ELEMENT_TYPE, '''' AS EXTERNAL_ID, '
             || 'TW.CALENDAR_ID, TC.ELEMENT_NAME AS CALENDAR_NAME, CASE WHEN TW.EFFORT_DRIVEN = 1 THEN ''Y'' ELSE ''N'' END AS CRITICAL, TW.PRIORITY, TW.MINUTE_PER_DAY, TW.MINUTE_PER_WEEK, '
             || 'RA.RESOURCE_ID, RC.FULL_NAME AS RESPONSIBLE, RC.E_MAIL_ADDRESS, TW.CONSTRAINT_TYPE, TW.CONSTRAINT_DT, TW.CHILD_COUNT, TW.ASSIGN_COUNT, '
             || 'TW.SD, TW.START_DT, TW.FD, TW.FINISH_DT, TW.DURATION, '
             || 'TW.BASELINE1_START, TW.BASELINE1_FINISH, TW.BASELINE1_DURATION, TW.BASELINE1_EFFORT, '
             || 'TW.BASELINE2_START, TW.BASELINE2_FINISH, TW.BASELINE2_DURATION, TW.BASELINE2_EFFORT, '
             || 'TW.BASELINE3_START, TW.BASELINE3_FINISH, TW.BASELINE3_DURATION, TW.BASELINE3_EFFORT, '
             || 'TW.PLANNED_START, TW.PLANNED_FINISH, TW.PLANNED_DURATION, TW.PLANNED_EFFORT, '
             || 'TW.PROPOSED_START, TW.PROPOSED_FINISH, TW.PROPOSED_DURATION, TW.PROPOSED_EFFORT, '
             || 'TW.FORECAST_START, TW.FORECAST_FINISH, TW.FORECAST_DURATION, TW.FORECAST_EFFORT, '
             || 'TW.SCHED_START, TW.SCHED_FINISH, TW.SCHED_DURATION, TW.SCHED_EFFORT, '
             || 'TW.ACTUAL_START, TW.ACTUAL_FINISH, TW.ACTUAL_DURATION, TW.ACTUAL_EFFORT, '
             || 'TW.UNRESOLVED_DURATION, '
             || ''''' AS DEPEND_ID, CASE WHEN TW.TYPE_ID IN (';
      stmt := stmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
      stmt := stmt || 
                ') AND TW.FS_CODE IS NOT NULL THEN ''F'' ELSE '''' END AS IS_EXTERNAL, '''' AS IS_PRED, 0 AS DEPENDENCY_TYPE, 0 AS LEAD_OR_LAG_TYPE, 0 AS AMOUNT, 0 AS LAG_DISPLAY_UNIT, '
             || ''''' AS PROJECT_ID, '''' AS PROJECT_NAME, TW.REC_DATETIME AS PROJECT_START_DT, '''' AS PROJECT_FINISH_DT, '
             || ''''' AS CREATOR_ID, '''' AS CREATOR_NAME, '''' AS CREATOR_E_MAIL_ADDRESS, '
             || 'TO_CHAR( TO_DATE(SubStr(TW.START_FINISH_DATE, 1, 10))) || '' '' || TO_CHAR( TO_DATE(SubStr(TW.START_FINISH_DATE, 1, 10))) AS START_FINISH_DATE, '
             || 'TW.START_FINISH_FLAG, TW.TOTAL_SLACK, TW.FREE_SLACK, '
             || ''''' AS RES_ASSIGNMENT_ID, '''' AS RESOURCE_TYPE, Trunc( TW.MILESTONE_PERCENT) AS MILESTONE_PERCENT, CASE WHEN TW.SCHED_EFFORT > 0 THEN ( (TW.WORK_TODATE / TW.SCHED_EFFORT) * 100) ELSE 0 END AS PERCENT_COMPLETE, '
             || '( TW.PLANNED_EFFORT + TW.SCHED_EFFORT - TW.WORK_TODATE) AS REMAINING_WORK, TW.WORK_TODATE AS WORK_TODATE, 0 AS CP_EFFORT, 0 AS CP_UNIT, '
             || ''''' AS ASSIGN_CONTOUR, 0 AS BEST, 0 AS ESTIMATE, TW.ETC AS ETC, 0.00 AS REGULAR_COST, 0.00 AS SPECIAL_COST, 0.00 AS REGULAR_RATE, 0.00 AS SPECIAL_RATE, TW.MSP_ID, '''' AS LAST_SUBMITTED, 0 AS ACTUAL_TODATE, TW.WBS_CODE, TW.TASK_TYPE, 0 AS UNITS_MSP, 0 AS CONTOUR_TYPE '
             || 'FROM (TMT_WBS TW JOIN RESOURCE_ASSIGNMENTS RA ON RA.ELEMENT_ID = TW.ELEMENT_ID AND BITAND(RA.PARTICIPANT, ';
      stmt := stmt || TO_CHAR(PMO_CNST.SC_MANAGER_RESPONSIBLE);
      stmt := stmt || ') > 0) '
             || 'JOIN RESOURCES RC ON RC.RESOURCE_ID = RA.RESOURCE_ID '
             || 'LEFT OUTER JOIN TMT_CALENDARS TC ON TC.ELEMENT_ID = TW.CALENDAR_ID '
             || 'WHERE TW.RANK LIKE ''';
      stmt := stmt || h_ranklike;
      stmt := stmt || ''' AND TW.REC_STATUS <> ''D''';
   End If;
   
   If Coalesce(p_bigflag, 0) = 3 Then
      stmt := stmt || ' AND TW.TYPE_ID IN (';
      stmt := stmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
      stmt := stmt || ') ORDER BY RANK ';
   Else
      If (Coalesce(h_type, 0) = PMO_CNST.SC_FOLDER) OR (isProjectType(Coalesce(h_type, 0))) <> 0 Then
         stmt := stmt || ' UNION ALL '
                || 'SELECT ''2'' AS ROW_TYPE, SUCC.RANK AS RANK, SUCC.LEVEL_ID + 1 AS LEVEL_ID, PRED.PARENT_ID, PRED.ELEMENT_ID, PRED.ELEMENT_NAME AS NAME, 205 AS TYPE_ID, '
                || 'PRED.TYPE_ID AS ELEMENT_TYPE, '''' AS EXTERNAL_ID, '
                || 'PRED.CALENDAR_ID, '''' AS CALENDAR_NAME, CASE WHEN PRED.EFFORT_DRIVEN = 1 THEN ''Y'' ELSE ''N'' END AS CRITICAL, PRED.PRIORITY, PRED.MINUTE_PER_DAY, PRED.MINUTE_PER_WEEK, '
                || 'RA.RESOURCE_ID AS RESOURCE_ID, RC.FULL_NAME AS RESPONSIBLE, RC.E_MAIL_ADDRESS AS E_MAIL_ADDRESS, PRED.CONSTRAINT_TYPE, PRED.CONSTRAINT_DT, PRED.CHILD_COUNT, PRED.ASSIGN_COUNT, '
                || 'PRED.SD, PRED.START_DT, PRED.FD, PRED.FINISH_DT, PRED.DURATION, '
                || 'PRED.BASELINE1_START, PRED.BASELINE1_FINISH, PRED.BASELINE1_DURATION, PRED.BASELINE1_EFFORT, '
                || 'PRED.BASELINE2_START, PRED.BASELINE2_FINISH, PRED.BASELINE2_DURATION, PRED.BASELINE2_EFFORT, '
                || 'PRED.BASELINE3_START, PRED.BASELINE3_FINISH, PRED.BASELINE3_DURATION, PRED.BASELINE3_EFFORT, '
                || 'PRED.PLANNED_START, PRED.PLANNED_FINISH, PRED.PLANNED_DURATION, PRED.PLANNED_EFFORT, '
                || 'PRED.PROPOSED_START, PRED.PROPOSED_FINISH, PRED.PROPOSED_DURATION, PRED.PROPOSED_EFFORT, '
                || 'PRED.FORECAST_START, PRED.FORECAST_FINISH, PRED.FORECAST_DURATION, PRED.FORECAST_EFFORT, '
                || 'PRED.SCHED_START, PRED.SCHED_FINISH, PRED.SCHED_DURATION, PRED.SCHED_EFFORT, '
                || 'PRED.ACTUAL_START AS ACTUAL_START, PRED.ACTUAL_FINISH AS ACTUAL_FINISH, PRED.ACTUAL_DURATION, PRED.ACTUAL_EFFORT, '
                || 'PRED.UNRESOLVED_DURATION, '
                || 'DEP.DEPEND_ID, case WHEN DEP.SUCC_PROJECT_ID = DEP.PRED_PROJECT_ID THEN ''0'' ELSE ''1'' END AS IS_EXTERNAL, ''1'' AS IS_PRED, DEP.DEP_TYPE AS DEPENDENCY_TYPE, DEP.LEAD_OR_LAG_TYPE, DEP.AMOUNT, DEP.LAG_DISPLAY_UNIT, '
                || 'PRED.PROJECT_ID, PRED_PRJ.ELEMENT_NAME AS PROJECT_NAME, '
                || 'PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(PRED_PRJ.START_DT, 1, 10)), PMO_ORASTD.TIME( PRED_PRJ.START_DT)) AS PROJECT_START_DT, '
                || 'TO_CHAR( TO_DATE(SubStr(PRED_PRJ.FINISH_DT, 1, 10))) || TO_CHAR( PMO_ORASTD.TIME( PRED_PRJ.FINISH_DT)) AS PROJECT_FINISH_DT, '
                || 'PRED_PRJ.CREATED_BY AS CREATOR_ID, PRJ_RC.FULL_NAME AS CREATOR_NAME, PRJ_RC.E_MAIL_ADDRESS AS CREATOR_E_MAIL_ADDRESS, '
                || ''''' AS START_FINISH_DATE, '''' AS START_FINISH_FLAG, 0 AS TOTAL_SLACK, 0 AS FREE_SLACK, '
                || ''''' AS RES_ASSIGNMENT_ID, '''' AS RESOURCE_TYPE, 0 AS MILESTONE_PERCENT, 0 AS PERCENT_COMPLETE, 0 AS REMAINING_WORK, 0 AS WORK_TODATE, 0 AS CP_EFFORT, 0 AS CP_UNIT, '
                || ''''' AS ASSIGN_CONTOUR, 0 AS BEST, 0 AS ESTIMATE, 0 AS ETC, 0 AS REGULAR_COST, 0 AS SPECIAL_COST, 0 AS REGULAR_RATE, 0 AS SPECIAL_RATE, PRED.MSP_ID, '''' AS LAST_SUBMITTED, 0 AS ACTUAL_TODATE, '''' AS WBS_CODE, 0 AS TASK_TYPE, 0 AS UNITS_MSP, 0 AS CONTOUR_TYPE '
                || 'FROM PMO_TMP_PROJECTS_01 TMP JOIN PMO_TMP_WBS_01 SUCC ON SUCC.PROJECT_ID = TMP.ELEMENT_ID '
                || 'JOIN DEPENDENCIES DEP ON DEP.SUCC_ID = SUCC.ELEMENT_ID '
                || 'JOIN TMT_WBS PRED ON PRED.ELEMENT_ID = DEP.PRED_ID '
                || 'JOIN RESOURCE_ASSIGNMENTS RA ON RA.ELEMENT_ID = PRED.ELEMENT_ID AND BITAND(RA.PARTICIPANT, ';
         stmt := stmt || TO_CHAR(PMO_CNST.SC_MANAGER_RESPONSIBLE);
         stmt := stmt || ') > 0 '
                || 'JOIN RESOURCES RC ON RC.RESOURCE_ID = RA.RESOURCE_ID '
                || 'JOIN TMT_WBS PRED_PRJ ON PRED_PRJ.ELEMENT_ID = PRED.PROJECT_ID '
                || 'JOIN RESOURCES PRJ_RC ON PRJ_RC.RESOURCE_ID = PRED_PRJ.CREATED_BY ';
         stmt := stmt || ' UNION ALL '
                || 'SELECT ''2'' AS ROW_TYPE, PRED.RANK AS RANK, PRED.LEVEL_ID + 1 AS LEVEL_ID, SUCC.PARENT_ID, SUCC.ELEMENT_ID, SUCC.ELEMENT_NAME AS NAME, 206 AS TYPE_ID, '
                || 'SUCC.TYPE_ID AS ELEMENT_TYPE, '''' AS EXTERNAL_ID, '
                || 'SUCC.CALENDAR_ID, '''' AS CALENDAR_NAME, CASE WHEN SUCC.EFFORT_DRIVEN = 1 THEN ''Y'' ELSE ''N'' END AS CRITICAL, SUCC.PRIORITY, SUCC.MINUTE_PER_DAY, SUCC.MINUTE_PER_WEEK, '
                || 'RA.RESOURCE_ID AS RESOURCE_ID, RC.FULL_NAME AS RESPONSIBLE, RC.E_MAIL_ADDRESS AS E_MAIL_ADDRESS, SUCC.CONSTRAINT_TYPE, SUCC.CONSTRAINT_DT, SUCC.CHILD_COUNT, SUCC.ASSIGN_COUNT, '
                || 'SUCC.SD, SUCC.START_DT, SUCC.FD, SUCC.FINISH_DT, SUCC.DURATION, '
                || 'SUCC.BASELINE1_START, SUCC.BASELINE1_FINISH, SUCC.BASELINE1_DURATION, SUCC.BASELINE1_EFFORT, '
                || 'SUCC.BASELINE2_START, SUCC.BASELINE2_FINISH, SUCC.BASELINE2_DURATION, SUCC.BASELINE2_EFFORT, '
                || 'SUCC.BASELINE3_START, SUCC.BASELINE3_FINISH, SUCC.BASELINE3_DURATION, SUCC.BASELINE3_EFFORT, '
                || 'SUCC.PLANNED_START, SUCC.PLANNED_FINISH, SUCC.PLANNED_DURATION, SUCC.PLANNED_EFFORT, '
                || 'SUCC.PROPOSED_START, SUCC.PROPOSED_FINISH, SUCC.PROPOSED_DURATION, SUCC.PROPOSED_EFFORT, '
                || 'SUCC.FORECAST_START, SUCC.FORECAST_FINISH, SUCC.FORECAST_DURATION, SUCC.FORECAST_EFFORT, '
                || 'SUCC.SCHED_START, SUCC.SCHED_FINISH, SUCC.SCHED_DURATION, SUCC.SCHED_EFFORT, '
                || 'SUCC.ACTUAL_START, SUCC.ACTUAL_FINISH, SUCC.ACTUAL_DURATION, SUCC.ACTUAL_EFFORT, '
                || 'SUCC.UNRESOLVED_DURATION, '
                || 'DEP.DEPEND_ID, case WHEN DEP.SUCC_PROJECT_ID = DEP.PRED_PROJECT_ID THEN ''0'' ELSE ''1'' END AS IS_EXTERNAL, ''0'' AS IS_PRED, DEP.DEP_TYPE AS DEPENDENCY_TYPE, DEP.LEAD_OR_LAG_TYPE, DEP.AMOUNT, DEP.LAG_DISPLAY_UNIT, '
                || 'SUCC.PROJECT_ID, SUCC_PRJ.ELEMENT_NAME AS PROJECT_NAME, '
                || 'PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(SUCC_PRJ.START_DT, 1, 10)), PMO_ORASTD.TIME( SUCC_PRJ.START_DT)) AS PROJECT_START_DT, '
                || 'TO_CHAR( TO_DATE(SubStr(SUCC_PRJ.FINISH_DT, 1, 10))) || TO_CHAR( PMO_ORASTD.TIME( SUCC_PRJ.FINISH_DT)) AS PROJECT_FINISH_DT, '
                || 'SUCC_PRJ.CREATED_BY AS CREATOR_ID, PRJ_RC.FULL_NAME AS CREATOR_NAME, PRJ_RC.E_MAIL_ADDRESS AS CREATOR_E_MAIL_ADDRESS, '
                || ''''' AS START_FINISH_DATE, '''' AS START_FINISH_FLAG, 0 AS TOTAL_SLACK, 0 AS FREE_SLACK, '
                || ''''' AS RES_ASSIGNMENT_ID, '''' AS RESOURCE_TYPE, 0 AS MILESTONE_PERCENT, 0 AS PERCENT_COMPLETE, 0 AS REMAINING_WORK, 0 AS WORK_TODATE, 0 AS CP_EFFORT, 0 AS CP_UNIT, '
                || ''''' AS ASSIGN_CONTOUR, 0 AS BEST, 0 AS ESTIMATE, 0 AS ETC, 0 AS REGULAR_COST, 0 AS SPECIAL_COST, 0 AS REGULAR_RATE, 0 AS SPECIAL_RATE, SUCC.MSP_ID, '''' AS LAST_SUBMITTED, 0 AS ACTUAL_TODATE, '''' AS WBS_CODE, 0 AS TASK_TYPE, 0 AS UNITS_MSP, 0 AS CONTOUR_TYPE '
                || 'FROM PMO_TMP_PROJECTS_01 TMP JOIN PMO_TMP_WBS_01 PRED ON PRED.PROJECT_ID = TMP.ELEMENT_ID '
                || 'JOIN DEPENDENCIES DEP ON DEP.PRED_ID = PRED.ELEMENT_ID '
                || 'JOIN TMT_WBS SUCC ON SUCC.ELEMENT_ID = DEP.SUCC_ID '
                || 'JOIN RESOURCE_ASSIGNMENTS RA ON RA.ELEMENT_ID = SUCC.ELEMENT_ID AND BITAND(RA.PARTICIPANT, ';
         stmt := stmt || TO_CHAR(PMO_CNST.SC_MANAGER_RESPONSIBLE);
         stmt := stmt || ') > 0 '
                || 'JOIN RESOURCES RC ON RC.RESOURCE_ID = RA.RESOURCE_ID '
                || 'JOIN TMT_WBS SUCC_PRJ ON SUCC_PRJ.ELEMENT_ID = SUCC.PROJECT_ID '
                || 'JOIN RESOURCES PRJ_RC ON PRJ_RC.RESOURCE_ID = SUCC_PRJ.CREATED_BY ';
         stmt := stmt || ' UNION ALL '
                || 'SELECT ''3'' AS ROW_TYPE, TW.RANK, TW.LEVEL_ID + 1 AS LEVEL_ID, '
                || 'TW.ELEMENT_ID AS PARENT_ID, TA.TSK_ASSIGNMENT_ID AS ELEMENT_ID, ''Resource Assignment'' AS NAME, '
                || '213 AS TYPE_ID, '
                || 'TW.TYPE_ID AS ELEMENT_TYPE, TW.PROJECT_ID AS EXTERNAL_ID, '
                || 'TW.CALENDAR_ID, '''' AS CALENDAR_NAME, CASE WHEN TW.EFFORT_DRIVEN = 1 THEN ''Y'' ELSE ''N'' END AS CRITICAL, TW.PRIORITY, TW.MINUTE_PER_DAY, TW.MINUTE_PER_WEEK, '
                || 'RA.RESOURCE_ID, RC.FULL_NAME AS RESPONSIBLE, RC.E_MAIL_ADDRESS, TW.CONSTRAINT_TYPE, TW.CONSTRAINT_DT, TW.CHILD_COUNT, TW.ASSIGN_COUNT, '
                || 'TW.SD, TW.START_DT, TW.FD, TW.FINISH_DT, TW.DURATION, '
                || 'TA.BASELINE1_START, TA.BASELINE1_FINISH, TA.BASELINE1_DURATION, TA.BASELINE1_WORK AS BASELINE1_EFFORT, '
                || 'TA.BASELINE2_START, TA.BASELINE2_FINISH, TA.BASELINE2_DURATION, TA.BASELINE2_WORK AS BASELINE2_EFFORT, '
                || 'TA.BASELINE3_START, TA.BASELINE3_FINISH, TA.BASELINE3_DURATION, TA.BASELINE3_WORK AS BASELINE3_EFFORT, '
                || 'TA.PLANNED_START, TA.PLANNED_FINISH, TA.PLANNED_DURATION, TA.PLANNED_WORK AS PLANNED_EFFORT, '
                || 'TA.PROPOSED_START, TA.PROPOSED_FINISH, TA.PROPOSED_DURATION, TA.PROPOSED_WORK AS PROPOSED_EFFORT, '
                || 'TA.FORECAST_START, TA.FORECAST_FINISH, TA.FORECAST_DURATION, TA.ETC AS FORECAST_EFFORT, '
                || 'TA.SCHED_START, TA.SCHED_FINISH, TA.SCHED_DURATION, TA.SCHED_WORK AS SCHED_EFFORT, '
                || 'TA.ACTUAL_START, TA.ACTUAL_FINISH, TA.ACTUAL_DURATION, TA.WORK_TODATE AS ACTUAL_EFFORT, '
                || '0 AS UNRESOLVED_DURATION, '
                || ''''' AS DEPEND_ID, '''' AS IS_EXTERNAL, '''' AS IS_PRED, 0 AS DEPENDENCY_TYPE, 0 AS LEAD_OR_LAG_TYPE, 0 AS AMOUNT, 0 AS LAG_DISPLAY_UNIT, '
                || ''''' AS PROJECT_ID, '''' AS PROJECT_NAME, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS PROJECT_START_DT, '''' AS PROJECT_FINISH_DT, '
                || ''''' AS CREATOR_ID, '''' AS CREATOR_NAME, '''' AS CREATOR_E_MAIL_ADDRESS, '
                || ''''' AS START_FINISH_DATE, '''' AS START_FINISH_FLAG, 0 AS TOTAL_SLACK, 0 AS FREE_SLACK, '
                || 'RA.RES_ASSIGNMENT_ID, RA.RESOURCE_TYPE, Trunc( TA.MILESTONE_PERCENT) AS MILESTONE_PERCENT, case WHEN TA.SCHED_WORK > 0 THEN ( (TA.WORK_TODATE / TA.SCHED_WORK) * 100) ELSE 0 END AS PERCENT_COMPLETE, ( TA.PLANNED_WORK + TA.SCHED_WORK - TA.WORK_TODATE) AS REMAINING_WORK, '
                || 'TA.WORK_TODATE, TA.C_EFFORT AS CP_EFFORT, TA.C_UNIT AS CP_UNIT, '
                || 'TA.ASSIGN_STRING AS ASSIGN_CONTOUR, TA.BEST, TA.ESTIMATE, TA.ETC, TA.REGULAR_COST, TA.SPECIAL_COST, TA.REGULAR_RATE, TA.SPECIAL_RATE, 0 AS MSP_ID, '
                || 'TO_CHAR(TA.LAST_SUBMITTED) AS LAST_SUBMITTED, '
                || '0 AS ACTUAL_TODATE, '''' AS WBS_CODE, TW.TASK_TYPE, TA.UNITS_MSP, CASE WHEN TA.C_EFFORT = 0 AND TA.CONTOUR_TYPE = 1 AND TA.UNITS_MSP <> 0 AND TA.UNITS_MSP <> 10000 THEN 0 ELSE TA.CONTOUR_TYPE END AS CONTOUR_TYPE '
                || 'FROM PMO_TMP_PROJECTS_01 TMP JOIN TMT_WBS TW ON TW.PROJECT_ID = TMP.ELEMENT_ID AND TW.TYPE_ID IN (';
         stmt := stmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         stmt := stmt || 
                ') JOIN RESOURCE_ASSIGNMENTS RA ON RA.ELEMENT_ID = TW.ELEMENT_ID AND MOD( RA.PARTICIPANT, 32) > 15 AND RA.RESOURCE_TYPE = ''0'' '
                || 'JOIN TASK_ASSIGNMENTS TA ON TA.RES_ASSIGNMENT_ID = RA.RES_ASSIGNMENT_ID '
                || 'JOIN RESOURCES RC ON RC.RESOURCE_ID = RA.RESOURCE_ID ';
         stmt := stmt || ' UNION ALL '
                || 'SELECT ''3'' AS ROW_TYPE, TW.RANK, TW.LEVEL_ID + 1 AS LEVEL_ID, TW.ELEMENT_ID AS PARENT_ID, TA.TSK_ASSIGNMENT_ID AS ELEMENT_ID, ''Profile Assignment'' AS NAME, 541 AS TYPE_ID, '
                || 'TW.TYPE_ID AS ELEMENT_TYPE, TW.PROJECT_ID AS EXTERNAL_ID, '
                || 'TW.CALENDAR_ID, '''' AS CALENDAR_NAME, CASE WHEN TW.EFFORT_DRIVEN = 1 THEN ''Y'' ELSE ''N'' END AS CRITICAL, TW.PRIORITY, TW.MINUTE_PER_DAY, TW.MINUTE_PER_WEEK, '
                || 'RA.PROFILE_ID AS RESOURCE_ID, P.PROFILE_NAME AS RESPONSIBLE, '''' AS E_MAIL_ADDRESS, TW.CONSTRAINT_TYPE, TW.CONSTRAINT_DT, TW.CHILD_COUNT, TW.ASSIGN_COUNT, '
                || 'TW.SD, TW.START_DT, TW.FD, TW.FINISH_DT, TW.DURATION, '
                || 'TA.BASELINE1_START, TA.BASELINE1_FINISH, TA.BASELINE1_DURATION, TA.BASELINE1_WORK AS BASELINE1_EFFORT, '
                || 'TA.BASELINE2_START, TA.BASELINE2_FINISH, TA.BASELINE2_DURATION, TA.BASELINE2_WORK AS BASELINE2_EFFORT, '
                || 'TA.BASELINE3_START, TA.BASELINE3_FINISH, TA.BASELINE3_DURATION, TA.BASELINE3_WORK AS BASELINE3_EFFORT, '
                || 'TA.PLANNED_START, TA.PLANNED_FINISH, TA.PLANNED_DURATION, TA.PLANNED_WORK AS PLANNED_EFFORT, '
                || 'TA.PROPOSED_START, TA.PROPOSED_FINISH, TA.PROPOSED_DURATION, TA.PROPOSED_WORK AS PROPOSED_EFFORT, '
                || 'TA.FORECAST_START, TA.FORECAST_FINISH, TA.FORECAST_DURATION, TA.ETC AS FORECAST_EFFORT, '
                || 'TA.SCHED_START, TA.SCHED_FINISH, TA.SCHED_DURATION, TA.SCHED_WORK AS SCHED_EFFORT, '
                || 'TA.ACTUAL_START AS ACTUAL_START, TA.ACTUAL_FINISH AS ACTUAL_FINISH, TA.ACTUAL_DURATION AS ACTUAL_DURATION, TA.WORK_TODATE AS ACTUAL_EFFORT, '
                || '0 AS UNRESOLVED_DURATION, '
                || ''''' AS DEPEND_ID, '''' AS IS_EXTERNAL, '''' AS IS_PRED, 0 AS DEPENDENCY_TYPE, 0 AS LEAD_OR_LAG_TYPE, 0 AS AMOUNT, 0 AS LAG_DISPLAY_UNIT, '
                || ''''' AS PROJECT_ID, '''' AS PROJECT_NAME, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS PROJECT_START_DT, '''' AS PROJECT_FINISH_DT, '
                || ''''' AS CREATOR_ID, '''' AS CREATOR_NAME, '''' AS CREATOR_E_MAIL_ADDRESS, '
                || ''''' AS START_FINISH_DATE, '''' AS START_FINISH_FLAG, 0 AS TOTAL_SLACK, 0 AS FREE_SLACK, '
                || 'RA.RES_ASSIGNMENT_ID, RA.RESOURCE_TYPE, Trunc( TA.MILESTONE_PERCENT) AS MILESTONE_PERCENT, case WHEN TA.SCHED_WORK > 0 THEN ( (TA.WORK_TODATE / TA.SCHED_WORK) * 100) ELSE 0 END AS PERCENT_COMPLETE, ( TA.PLANNED_WORK + TA.SCHED_WORK - TA.WORK_TODATE) AS REMAINING_WORK, '
                || 'TA.WORK_TODATE, TA.C_EFFORT AS CP_EFFORT, TA.C_UNIT AS CP_UNIT, '
                || 'TA.ASSIGN_STRING AS ASSIGN_CONTOUR, TA.BEST, TA.ESTIMATE, TA.ETC, TA.REGULAR_COST, TA.SPECIAL_COST, TA.REGULAR_RATE, TA.SPECIAL_RATE, 0 AS MSP_ID, '''' AS LAST_SUBMITTED, 0 AS ACTUAL_TODATE, '''' AS WBS_CODE, TW.TASK_TYPE, TA.UNITS_MSP, CASE WHEN TA.C_EFFORT = 0 AND TA.CONTOUR_TYPE = 1 AND TA.UNITS_MSP <> 0 AND TA.UNITS_MSP <> 10000 THEN 0 ELSE TA.CONTOUR_TYPE END AS CONTOUR_TYPE '
                || 'FROM PMO_TMP_PROJECTS_01 TMP JOIN TMT_WBS TW ON TW.PROJECT_ID = TMP.ELEMENT_ID '
                || 'JOIN RESOURCE_ASSIGNMENTS RA ON RA.ELEMENT_ID = TW.ELEMENT_ID AND MOD( RA.PARTICIPANT, 32) > 15  AND RA.RESOURCE_TYPE = ''1'' '
                || 'JOIN TASK_ASSIGNMENTS TA ON TA.RES_ASSIGNMENT_ID = RA.RES_ASSIGNMENT_ID '
                || 'JOIN PROFILES P ON P.PROFILE_ID = RA.PROFILE_ID ';
         stmt := stmt || ' ORDER BY RANK, ROW_TYPE ';
      Else
         stmt := stmt || ''' UNION ALL '
                || 'SELECT ''2'' AS ROW_TYPE, SUCC.RANK AS RANK, SUCC.LEVEL_ID + 1 AS LEVEL_ID, PRED.PARENT_ID, PRED.ELEMENT_ID, PRED.ELEMENT_NAME AS NAME, 205 AS TYPE_ID, '
                || 'PRED.TYPE_ID AS ELEMENT_TYPE, '''' AS EXTERNAL_ID, '
                || 'PRED.CALENDAR_ID, '''' AS CALENDAR_NAME, CASE WHEN PRED.EFFORT_DRIVEN = 1 THEN ''Y'' ELSE ''N'' END AS CRITICAL, PRED.PRIORITY, PRED.MINUTE_PER_DAY, PRED.MINUTE_PER_WEEK, '
                || 'RA.RESOURCE_ID AS RESOURCE_ID, RC.FULL_NAME AS RESPONSIBLE, RC.E_MAIL_ADDRESS AS E_MAIL_ADDRESS, PRED.CONSTRAINT_TYPE, PRED.CONSTRAINT_DT, PRED.CHILD_COUNT, PRED.ASSIGN_COUNT, '
                || 'PRED.SD, PRED.START_DT, PRED.FD, PRED.FINISH_DT, PRED.DURATION, '
                || 'PRED.BASELINE1_START, PRED.BASELINE1_FINISH, PRED.BASELINE1_DURATION, PRED.BASELINE1_EFFORT, '
                || 'PRED.BASELINE2_START, PRED.BASELINE2_FINISH, PRED.BASELINE2_DURATION, PRED.BASELINE2_EFFORT, '
                || 'PRED.BASELINE3_START, PRED.BASELINE3_FINISH, PRED.BASELINE3_DURATION, PRED.BASELINE3_EFFORT, '
                || 'PRED.PLANNED_START, PRED.PLANNED_FINISH, PRED.PLANNED_DURATION, PRED.PLANNED_EFFORT, '
                || 'PRED.PROPOSED_START, PRED.PROPOSED_FINISH, PRED.PROPOSED_DURATION, PRED.PROPOSED_EFFORT, '
                || 'PRED.FORECAST_START, PRED.FORECAST_FINISH, PRED.FORECAST_DURATION, PRED.FORECAST_EFFORT, '
                || 'PRED.SCHED_START, PRED.SCHED_FINISH, PRED.SCHED_DURATION, PRED.SCHED_EFFORT, '
                || 'PRED.ACTUAL_START AS ACTUAL_START, PRED.ACTUAL_FINISH AS ACTUAL_FINISH, PRED.ACTUAL_DURATION, PRED.ACTUAL_EFFORT, '
                || 'PRED.UNRESOLVED_DURATION, '
                || 'DEP.DEPEND_ID, case WHEN DEP.SUCC_PROJECT_ID = DEP.PRED_PROJECT_ID THEN ''0'' ELSE ''1'' END AS IS_EXTERNAL, ''1'' AS IS_PRED, DEP.DEP_TYPE AS DEPENDENCY_TYPE, DEP.LEAD_OR_LAG_TYPE, DEP.AMOUNT, DEP.LAG_DISPLAY_UNIT, '
                || 'PRED.PROJECT_ID, PRED_PRJ.ELEMENT_NAME AS PROJECT_NAME, '
                || 'PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(PRED_PRJ.START_DT, 1, 10)), PMO_ORASTD.TIME( PRED_PRJ.START_DT)) AS PROJECT_START_DT, '
                || 'TO_CHAR( TO_DATE(SubStr(PRED_PRJ.FINISH_DT, 1, 10))) || TO_CHAR( PMO_ORASTD.TIME( PRED_PRJ.FINISH_DT)) AS PROJECT_FINISH_DT, '
                || 'PRED_PRJ.CREATED_BY AS CREATOR_ID, PRJ_RC.FULL_NAME AS CREATOR_NAME, PRJ_RC.E_MAIL_ADDRESS AS CREATOR_E_MAIL_ADDRESS, '
                || ''''' AS START_FINISH_DATE, '''' AS START_FINISH_FLAG, 0 AS TOTAL_SLACK, 0 AS FREE_SLACK, '
                || ''''' AS RES_ASSIGNMENT_ID, '''' AS RESOURCE_TYPE, 0 AS MILESTONE_PERCENT, 0 AS PERCENT_COMPLETE, 0 AS REMAINING_WORK, 0 AS WORK_TODATE, 0 AS CP_EFFORT, 0 AS CP_UNIT, '
                || ''''' AS ASSIGN_CONTOUR, 0 AS BEST, 0 AS ESTIMATE, 0 AS ETC, 0 AS REGULAR_COST, 0 AS SPECIAL_COST, 0 AS REGULAR_RATE, 0 AS SPECIAL_RATE, PRED.MSP_ID, '''' AS LAST_SUBMITTED, 0 AS ACTUAL_TODATE, '''' AS WBS_CODE, 0 AS TASK_TYPE, 0 AS UNITS_MSP, 0 AS CONTOUR_TYPE '
                || 'FROM TMT_WBS SUCC JOIN DEPENDENCIES DEP ON DEP.SUCC_ID = SUCC.ELEMENT_ID '
                || 'JOIN TMT_WBS PRED ON PRED.ELEMENT_ID = DEP.PRED_ID '
                || 'JOIN RESOURCE_ASSIGNMENTS RA ON RA.ELEMENT_ID = PRED.ELEMENT_ID AND BITAND(RA.PARTICIPANT, ';
         stmt := stmt || TO_CHAR(PMO_CNST.SC_MANAGER_RESPONSIBLE);
         stmt := stmt || ') > 0 '
                || 'JOIN RESOURCES RC ON RC.RESOURCE_ID = RA.RESOURCE_ID '
                || 'JOIN TMT_WBS PRED_PRJ ON PRED_PRJ.ELEMENT_ID = PRED.PROJECT_ID '
                || 'JOIN RESOURCES PRJ_RC ON PRJ_RC.RESOURCE_ID = PRED_PRJ.CREATED_BY '
                || 'WHERE SUCC.RANK LIKE ''';
         stmt := stmt || h_ranklike;
         stmt := stmt || ''' UNION ALL '
                || 'SELECT ''2'' AS ROW_TYPE, PRED.RANK AS RANK, PRED.LEVEL_ID + 1 AS LEVEL_ID, SUCC.PARENT_ID, SUCC.ELEMENT_ID, SUCC.ELEMENT_NAME AS NAME, 206 AS TYPE_ID, '
                || 'SUCC.TYPE_ID AS ELEMENT_TYPE, '''' AS EXTERNAL_ID, '
                || 'SUCC.CALENDAR_ID, '''' AS CALENDAR_NAME, CASE WHEN SUCC.EFFORT_DRIVEN = 1 THEN ''Y'' ELSE ''N'' END AS CRITICAL, SUCC.PRIORITY, SUCC.MINUTE_PER_DAY, SUCC.MINUTE_PER_WEEK, '
                || 'RA.RESOURCE_ID AS RESOURCE_ID, RC.FULL_NAME AS RESPONSIBLE, RC.E_MAIL_ADDRESS AS E_MAIL_ADDRESS, SUCC.CONSTRAINT_TYPE, SUCC.CONSTRAINT_DT, SUCC.CHILD_COUNT, SUCC.ASSIGN_COUNT, '
                || 'SUCC.SD, SUCC.START_DT, SUCC.FD, SUCC.FINISH_DT, SUCC.DURATION, '
                || 'SUCC.BASELINE1_START, SUCC.BASELINE1_FINISH, SUCC.BASELINE1_DURATION, SUCC.BASELINE1_EFFORT, '
                || 'SUCC.BASELINE2_START, SUCC.BASELINE2_FINISH, SUCC.BASELINE2_DURATION, SUCC.BASELINE2_EFFORT, '
                || 'SUCC.BASELINE3_START, SUCC.BASELINE3_FINISH, SUCC.BASELINE3_DURATION, SUCC.BASELINE3_EFFORT, '
                || 'SUCC.PLANNED_START, SUCC.PLANNED_FINISH, SUCC.PLANNED_DURATION, SUCC.PLANNED_EFFORT, '
                || 'SUCC.PROPOSED_START, SUCC.PROPOSED_FINISH, SUCC.PROPOSED_DURATION, SUCC.PROPOSED_EFFORT, '
                || 'SUCC.FORECAST_START, SUCC.FORECAST_FINISH, SUCC.FORECAST_DURATION, SUCC.FORECAST_EFFORT, '
                || 'SUCC.SCHED_START, SUCC.SCHED_FINISH, SUCC.SCHED_DURATION, SUCC.SCHED_EFFORT, '
                || 'SUCC.ACTUAL_START, SUCC.ACTUAL_FINISH, SUCC.ACTUAL_DURATION, SUCC.ACTUAL_EFFORT, '
                || 'SUCC.UNRESOLVED_DURATION, '
                || 'DEP.DEPEND_ID, case WHEN DEP.SUCC_PROJECT_ID = DEP.PRED_PROJECT_ID THEN ''0'' ELSE ''1'' END AS IS_EXTERNAL, ''0'' AS IS_PRED, DEP.DEP_TYPE AS DEPENDENCY_TYPE, DEP.LEAD_OR_LAG_TYPE, DEP.AMOUNT, DEP.LAG_DISPLAY_UNIT, '
                || 'SUCC.PROJECT_ID, SUCC_PRJ.ELEMENT_NAME AS PROJECT_NAME, '
                || 'PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(SUCC_PRJ.START_DT, 1, 10)), PMO_ORASTD.TIME( SUCC_PRJ.START_DT)) AS PROJECT_START_DT, '
                || 'TO_CHAR( TO_DATE(SubStr(SUCC_PRJ.FINISH_DT, 1, 10))) || TO_CHAR( PMO_ORASTD.TIME( SUCC_PRJ.FINISH_DT)) AS PROJECT_FINISH_DT, '
                || 'SUCC_PRJ.CREATED_BY AS CREATOR_ID, PRJ_RC.FULL_NAME AS CREATOR_NAME, PRJ_RC.E_MAIL_ADDRESS AS CREATOR_E_MAIL_ADDRESS, '
                || ''''' AS START_FINISH_DATE, '''' AS START_FINISH_FLAG, 0 AS TOTAL_SLACK, 0 AS FREE_SLACK, '
                || ''''' AS RES_ASSIGNMENT_ID, '''' AS RESOURCE_TYPE, 0 AS MILESTONE_PERCENT, 0 AS PERCENT_COMPLETE, 0 AS REMAINING_WORK, 0 AS WORK_TODATE, 0 AS CP_EFFORT, 0 AS CP_UNIT, '
                || ''''' AS ASSIGN_CONTOUR, 0 AS BEST, 0 AS ESTIMATE, 0 AS ETC, 0 AS REGULAR_COST, 0 AS SPECIAL_COST, 0 AS REGULAR_RATE, 0 AS SPECIAL_RATE, SUCC.MSP_ID, '''' AS LAST_SUBMITTED, 0 AS ACTUAL_TODATE, '''' AS WBS_CODE, 0 AS TASK_TYPE, 0 AS UNITS_MSP, 0 AS CONTOUR_TYPE '
                || 'FROM TMT_WBS PRED JOIN DEPENDENCIES DEP ON DEP.PRED_ID = PRED.ELEMENT_ID '
                || 'JOIN TMT_WBS SUCC ON SUCC.ELEMENT_ID = DEP.SUCC_ID '
                || 'JOIN RESOURCE_ASSIGNMENTS RA ON RA.ELEMENT_ID = SUCC.ELEMENT_ID AND BITAND(RA.PARTICIPANT, ';
         stmt := stmt || TO_CHAR(PMO_CNST.SC_MANAGER_RESPONSIBLE);
         stmt := stmt || ') > 0 '
                || 'JOIN RESOURCES RC ON RC.RESOURCE_ID = RA.RESOURCE_ID '
                || 'JOIN TMT_WBS SUCC_PRJ ON SUCC_PRJ.ELEMENT_ID = SUCC.PROJECT_ID '
                || 'JOIN RESOURCES PRJ_RC ON PRJ_RC.RESOURCE_ID = SUCC_PRJ.CREATED_BY '
                || 'WHERE PRED.RANK LIKE ''';
         stmt := stmt || h_ranklike;
         stmt := stmt || ''' UNION ALL '
                || 'SELECT ''3'' AS ROW_TYPE, TW.RANK, TW.LEVEL_ID + 1 AS LEVEL_ID, '
                || 'TW.ELEMENT_ID AS PARENT_ID, TA.TSK_ASSIGNMENT_ID AS ELEMENT_ID, ''Resource Assignment'' AS NAME, '
                || '213 AS TYPE_ID, '
                || 'TW.TYPE_ID AS ELEMENT_TYPE, TW.PROJECT_ID AS EXTERNAL_ID, '
                || 'TW.CALENDAR_ID, '''' AS CALENDAR_NAME, CASE WHEN TW.EFFORT_DRIVEN = 1 THEN ''Y'' ELSE ''N'' END AS CRITICAL, TW.PRIORITY, TW.MINUTE_PER_DAY, TW.MINUTE_PER_WEEK, '
                || 'RA.RESOURCE_ID, RC.FULL_NAME AS RESPONSIBLE, RC.E_MAIL_ADDRESS, TW.CONSTRAINT_TYPE, TW.CONSTRAINT_DT, TW.CHILD_COUNT, TW.ASSIGN_COUNT, '
                || 'TW.SD, TW.START_DT, TW.FD, TW.FINISH_DT, TW.DURATION, '
                || 'TA.BASELINE1_START, TA.BASELINE1_FINISH, TA.BASELINE1_DURATION, TA.BASELINE1_WORK AS BASELINE1_EFFORT, '
                || 'TA.BASELINE2_START, TA.BASELINE2_FINISH, TA.BASELINE2_DURATION, TA.BASELINE2_WORK AS BASELINE2_EFFORT, '
                || 'TA.BASELINE3_START, TA.BASELINE3_FINISH, TA.BASELINE3_DURATION, TA.BASELINE3_WORK AS BASELINE3_EFFORT, '
                || 'TA.PLANNED_START, TA.PLANNED_FINISH, TA.PLANNED_DURATION, TA.PLANNED_WORK AS PLANNED_EFFORT, '
                || 'TA.PROPOSED_START, TA.PROPOSED_FINISH, TA.PROPOSED_DURATION, TA.PROPOSED_WORK AS PROPOSED_EFFORT, '
                || 'TA.FORECAST_START, TA.FORECAST_FINISH, TA.FORECAST_DURATION, TA.ETC AS FORECAST_EFFORT, '
                || 'TA.SCHED_START, TA.SCHED_FINISH, TA.SCHED_DURATION, TA.SCHED_WORK AS SCHED_EFFORT, '
                || 'TA.ACTUAL_START, TA.ACTUAL_FINISH, TA.ACTUAL_DURATION, TA.WORK_TODATE AS ACTUAL_EFFORT, '
                || '0 AS UNRESOLVED_DURATION, '
                || ''''' AS DEPEND_ID, '''' AS IS_EXTERNAL, '''' AS IS_PRED, 0 AS DEPENDENCY_TYPE, 0 AS LEAD_OR_LAG_TYPE, 0 AS AMOUNT, 0 AS LAG_DISPLAY_UNIT, '
                || ''''' AS PROJECT_ID, '''' AS PROJECT_NAME, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS PROJECT_START_DT, '''' AS PROJECT_FINISH_DT, '
                || ''''' AS CREATOR_ID, '''' AS CREATOR_NAME, '''' AS CREATOR_E_MAIL_ADDRESS, '
                || ''''' AS START_FINISH_DATE, '''' AS START_FINISH_FLAG, 0 AS TOTAL_SLACK, 0 AS FREE_SLACK, '
                || 'RA.RES_ASSIGNMENT_ID, RA.RESOURCE_TYPE, Trunc( TA.MILESTONE_PERCENT) AS MILESTONE_PERCENT, case WHEN TA.SCHED_WORK > 0 THEN ( (TA.WORK_TODATE / TA.SCHED_WORK) * 100) ELSE 0 END AS PERCENT_COMPLETE, ( TA.SCHED_WORK - TA.WORK_TODATE) AS REMAINING_WORK, '
                || 'TA.WORK_TODATE, TA.C_EFFORT AS CP_EFFORT, TA.C_UNIT AS CP_UNIT, '
                || 'TA.ASSIGN_STRING AS ASSIGN_CONTOUR, TA.BEST, TA.ESTIMATE, TA.ETC, TA.REGULAR_COST, TA.SPECIAL_COST, TA.REGULAR_RATE, TA.SPECIAL_RATE, 0 AS MSP_ID, '
                || 'TO_CHAR(TA.LAST_SUBMITTED) AS LAST_SUBMITTED, '
                || '0 AS ACTUAL_TODATE, '''' AS WBS_CODE, TW.TASK_TYPE, TA.UNITS_MSP, CASE WHEN TA.C_EFFORT = 0 AND TA.CONTOUR_TYPE = 1 AND TA.UNITS_MSP <> 0 AND TA.UNITS_MSP <> 10000 THEN 0 ELSE TA.CONTOUR_TYPE END AS CONTOUR_TYPE '
                || 'FROM TMT_WBS TW JOIN RESOURCE_ASSIGNMENTS RA ON RA.ELEMENT_ID = TW.ELEMENT_ID AND MOD( RA.PARTICIPANT, 32) > 15 AND RA.RESOURCE_TYPE = ''0'' '
                || 'JOIN TASK_ASSIGNMENTS TA ON TA.RES_ASSIGNMENT_ID = RA.RES_ASSIGNMENT_ID '
                || 'JOIN RESOURCES RC ON RC.RESOURCE_ID = RA.RESOURCE_ID '
                || 'WHERE TW.RANK LIKE ''';
         stmt := stmt || h_ranklike;
         stmt := stmt || ''' AND TW.TYPE_ID IN (';
         stmt := stmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         stmt := stmt || ') UNION ALL '
                || 'SELECT ''3'' AS ROW_TYPE, TW.RANK, TW.LEVEL_ID + 1 AS LEVEL_ID, TW.ELEMENT_ID AS PARENT_ID, TA.TSK_ASSIGNMENT_ID AS ELEMENT_ID, ''Profile Assignment'' AS NAME, 541 AS TYPE_ID, '
                || 'TW.TYPE_ID AS ELEMENT_TYPE, TW.PROJECT_ID AS EXTERNAL_ID, '
                || 'TW.CALENDAR_ID, '''' AS CALENDAR_NAME, CASE WHEN TW.EFFORT_DRIVEN = 1 THEN ''Y'' ELSE ''N'' END AS CRITICAL, TW.PRIORITY, TW.MINUTE_PER_DAY, TW.MINUTE_PER_WEEK, '
                || 'RA.PROFILE_ID AS RESOURCE_ID, P.PROFILE_NAME AS RESPONSIBLE, '''' AS E_MAIL_ADDRESS, TW.CONSTRAINT_TYPE, TW.CONSTRAINT_DT, TW.CHILD_COUNT, TW.ASSIGN_COUNT, '
                || 'TW.SD, TW.START_DT, TW.FD, TW.FINISH_DT, TW.DURATION, '
                || 'TA.BASELINE1_START, TA.BASELINE1_FINISH, TA.BASELINE1_DURATION, TA.BASELINE1_WORK AS BASELINE1_EFFORT, '
                || 'TA.BASELINE2_START, TA.BASELINE2_FINISH, TA.BASELINE2_DURATION, TA.BASELINE2_WORK AS BASELINE2_EFFORT, '
                || 'TA.BASELINE3_START, TA.BASELINE3_FINISH, TA.BASELINE3_DURATION, TA.BASELINE3_WORK AS BASELINE3_EFFORT, '
                || 'TA.PLANNED_START, TA.PLANNED_FINISH, TA.PLANNED_DURATION, TA.PLANNED_WORK AS PLANNED_EFFORT, '
                || 'TA.PROPOSED_START, TA.PROPOSED_FINISH, TA.PROPOSED_DURATION, TA.PROPOSED_WORK AS PROPOSED_EFFORT, '
                || 'TA.FORECAST_START, TA.FORECAST_FINISH, TA.FORECAST_DURATION, TA.ETC AS FORECAST_EFFORT, '
                || 'TA.SCHED_START, TA.SCHED_FINISH, TA.SCHED_DURATION, TA.SCHED_WORK AS SCHED_EFFORT, '
                || 'TA.ACTUAL_START AS ACTUAL_START, TA.ACTUAL_FINISH AS ACTUAL_FINISH, TA.ACTUAL_DURATION AS ACTUAL_DURATION, TA.WORK_TODATE AS ACTUAL_EFFORT, '
                || '0 AS UNRESOLVED_DURATION, '
                || ''''' AS DEPEND_ID, '''' AS IS_EXTERNAL, '''' AS IS_PRED, 0 AS DEPENDENCY_TYPE, 0 AS LEAD_OR_LAG_TYPE, 0 AS AMOUNT, 0 AS LAG_DISPLAY_UNIT, '
                || ''''' AS PROJECT_ID, '''' AS PROJECT_NAME, TO_TIMESTAMP(''1899-12-31 00:00:00'') AS PROJECT_START_DT, '''' AS PROJECT_FINISH_DT, '
                || ''''' AS CREATOR_ID, '''' AS CREATOR_NAME, '''' AS CREATOR_E_MAIL_ADDRESS, '
                || ''''' AS START_FINISH_DATE, '''' AS START_FINISH_FLAG, 0 AS TOTAL_SLACK, 0 AS FREE_SLACK, '
                || 'RA.RES_ASSIGNMENT_ID, RA.RESOURCE_TYPE, Trunc( TA.MILESTONE_PERCENT) AS MILESTONE_PERCENT, case WHEN TA.SCHED_WORK > 0 THEN ( (TA.WORK_TODATE / TA.SCHED_WORK) * 100) ELSE 0 END AS PERCENT_COMPLETE, ( TA.SCHED_WORK - TA.WORK_TODATE) AS REMAINING_WORK, '
                || 'TA.WORK_TODATE, TA.C_EFFORT AS CP_EFFORT, TA.C_UNIT AS CP_UNIT, '
                || 'TA.ASSIGN_STRING AS ASSIGN_CONTOUR, TA.BEST, TA.ESTIMATE, TA.ETC, TA.REGULAR_COST, TA.SPECIAL_COST, TA.REGULAR_RATE, TA.SPECIAL_RATE, 0 AS MSP_ID, '''' AS LAST_SUBMITTED, 0 AS ACTUAL_TODATE, '''' AS WBS_CODE, TW.TASK_TYPE, TA.UNITS_MSP, CASE WHEN TA.C_EFFORT = 0 AND TA.CONTOUR_TYPE = 1 AND TA.UNITS_MSP <> 0 AND TA.UNITS_MSP <> 10000 THEN 0 ELSE TA.CONTOUR_TYPE END AS CONTOUR_TYPE '
                || 'FROM TMT_WBS TW JOIN RESOURCE_ASSIGNMENTS RA ON RA.ELEMENT_ID = TW.ELEMENT_ID AND MOD( RA.PARTICIPANT, 32) > 15  AND RA.RESOURCE_TYPE = ''1'' '
                || 'JOIN TASK_ASSIGNMENTS TA ON TA.RES_ASSIGNMENT_ID = RA.RES_ASSIGNMENT_ID '
                || 'JOIN PROFILES P ON P.PROFILE_ID = RA.PROFILE_ID '
                || ' WHERE TW.RANK LIKE ''';
         stmt := stmt || h_ranklike;
         stmt := stmt || ''' ORDER BY RANK, ROW_TYPE ';
      End If;
   End If;
   --trace_printf("%s\n",(char *) stmt);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_IMPEX_WBS', 467+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_IMPEX_WBS(468) : impexwbs.sql(1073) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_IMPEX_WBS', 470+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_IMPEX_WBS(471) : impexwbs.sql(1076) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(stmt);
   --  //return PROC_STATE;
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_WBS', 478+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_WBS(479) : impexwbs.sql(1084) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_WBS', 490+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_WBS(491) : impexwbs.sql(1096) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_IMPEX_WBS;
ALTER FUNCTION SP_S_IMPEX_WBS COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION SP_S_IMPEX_A_NEW ==
CREATE OR REPLACE FUNCTION SP_S_IMPEX_A_NEW
(
     p_id VARCHAR2
   , p_bigflag PLS_INTEGER
   , p_rec_status VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_calendar_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_type PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      C_AAA PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_IMPEX_A_NEW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_IMPEX_A_NEW'), '-- ' || SYSTIMESTAMP, ':r := SP_S_IMPEX_A_NEW(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bigflag), 'NULL')
          || ', ' || '''' || p_rec_status || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_IMPEX_A_NEW', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_IMPEX_A_NEW(26) : impexwbs.sql(1138) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --  SC_CHAR(h_calendar_name, 255);
   --  SC_CHAR(h_ranklike, SC_H_FULL_RANK);
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_PROJECTS_01';
   PMO_DB2M.SC_STATE := 0;
   
   h_type := 0;
   h_calendar_id := p_id;
   
   h_resource_id := NULL;
   h_rank := NULL;
   
   PMO_DB2M.SC_STATE := FC_PORTF_PRJ(h_resource_id, h_calendar_id, h_rank, h_type);
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0001';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0001
      SELECT TSK_ASSIGNMENT_ID AS TA_ID
      FROM TASK_ASSIGNMENTS T JOIN PMO_TMP_PROJECTS_01 TMP ON TMP.ELEMENT_ID = T.PROJECT_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_A_NEW', 62+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_A_NEW(63) : impexwbs.sql(1175) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_00PA';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_00PA
      SELECT RESOURCE_ID, MIN(PROPOSED_START), MAX(PROPOSED_FINISH)
      FROM TASK_ASSIGNMENTS T JOIN PMO_TMP_PROJECTS_01 TMP
      ON TMP.ELEMENT_ID = T.PROJECT_ID AND T.PROJECT_ID = T.TASK_ID
      GROUP BY RESOURCE_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_A_NEW', 87+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_A_NEW(88) : impexwbs.sql(1200) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0002';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0002
      SELECT TW.ELEMENT_ID, CAL.ELEMENT_ID AS CALENDAR_ID, TW.ELEMENT_NAME AS CALENDAR_NAME, CAL.RANK AS CALENDAR_RANK
      FROM TMT_WBS TW JOIN TMT_CALENDARS CAL ON TW.ELEMENT_ID = CAL.ELEMENT_ID
      JOIN PMO_TMP_PROJECTS_01 TMP ON TMP.ELEMENT_ID = TW.ELEMENT_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_A_NEW', 111+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_A_NEW(112) : impexwbs.sql(1224) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0003';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0003
      SELECT DISTINCT A.RESOURCE_ID, CAL.ELEMENT_ID AS CALENDAR_ID, CAL.ELEMENT_NAME AS CALENDAR_NAME, CAL.RANK AS CALENDAR_RANK
      FROM PMO_TMP_IMXWBS_0001 JOIN TASK_ASSIGNMENTS A
      ON A.TSK_ASSIGNMENT_ID = PMO_TMP_IMXWBS_0001.TA_ID AND A.RESOURCE_TYPE = '0'
      JOIN TMT_CALENDARS CAL ON A.RESOURCE_ID = CAL.ELEMENT_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_A_NEW', 136+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_A_NEW(137) : impexwbs.sql(1249) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0010';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0010
      SELECT DISTINCT RESOURCE_ID, CAL.ELEMENT_ID, CAL.ELEMENT_NAME
      FROM PMO_TMP_IMXWBS_0001 JOIN TASK_ASSIGNMENTS A
      ON A.TSK_ASSIGNMENT_ID = PMO_TMP_IMXWBS_0001.TA_ID AND A.RESOURCE_TYPE = '1'
      JOIN TMT_CALENDARS CAL ON A.PROJECT_ID = CAL.ELEMENT_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_A_NEW', 161+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_A_NEW(162) : impexwbs.sql(1274) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_AAA FOR SELECT
   0 AS LEVEL_ID, TYPE_ID, '' AS RANK, '' AS PARENT_ID, T.ELEMENT_ID, ELEMENT_NAME, 1 AS CHILD_COUNT, '' AS E_MAIL_ADDRESS, T.PROJECT_ID AS EXTERNAL_ID, '' AS TSK_ASSIGNMENT_ID, 0 AS REGULAR_COST, 0 AS SPECIAL_COST,
   PMO_TMP_IMXWBS_0002.CALENDAR_ID, PMO_TMP_IMXWBS_0002.CALENDAR_NAME, '' AS DAY_YEAR, '' AS WORKING, '' AS FROM1, '' AS TO1, '' AS FROM2, '' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5,
   TO_TIMESTAMP('1899-12-31 00:00:00') AS START_DT, TO_TIMESTAMP('1899-12-31 00:00:00') AS FINISH_DT
   FROM PMO_TMP_IMXWBS_0002 JOIN TMT_WBS T ON PMO_TMP_IMXWBS_0002.ELEMENT_ID = T.ELEMENT_ID
   UNION ALL SELECT
   0 AS LEVEL_ID, 13 AS TYPE_ID, '' AS RANK, '' AS PARENT_ID, R2.RESOURCE_ID AS ELEMENT_ID, R2.FULL_NAME AS ELEMENT_NAME, 1 AS CHILD_COUNT, R2.E_MAIL_ADDRESS, h_calendar_id AS EXTERNAL_ID,
   '' AS TSK_ASSIGNMENT_ID, B.REGULAR_COST, B.SPECIAL_COST, R1.CALENDAR_ID, R1.CALENDAR_NAME AS CALENDAR_NAME, '' AS DAY_YEAR, '' AS WORKING, '' AS FROM1, '' AS TO1, '' AS FROM2, '' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5,
   PA.START_DT, PA.FINISH_DT
   FROM PMO_TMP_IMXWBS_0003 R1 JOIN RESOURCES R2 ON R1.RESOURCE_ID = R2.RESOURCE_ID
   LEFT JOIN PMO_TMP_IMXWBS_00PA PA ON PA.RESOURCE_ID = R2.RESOURCE_ID
   LEFT OUTER JOIN RATES B ON R2.RATE_ID = B.RATE_ID
   UNION ALL SELECT
   0 AS LEVEL_ID, 41 AS TYPE_ID, '' AS RANK, '' AS PARENT_ID, P2.PROFILE_ID AS ELEMENT_ID, P2.PROFILE_NAME AS ELEMENT_NAME, 1 AS CHILD_COUNT, '' AS E_MAIL_ADDRESS, '' AS EXTERNAL_ID, '' AS TSK_ASSIGNMENT_ID,
   B.REGULAR_COST, B.SPECIAL_COST, P1.CALENDAR_ID, P1.CALENDAR_NAME, '' AS DAY_YEAR, '' AS WORKING, '' AS FROM1, '' AS TO1, '' AS FROM2, '' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5,
   PA.START_DT, PA.FINISH_DT
   FROM PMO_TMP_IMXWBS_0010 P1 LEFT OUTER JOIN PROFILES P2 ON P1.PROFILE_ID = P2.PROFILE_ID
   LEFT JOIN PMO_TMP_IMXWBS_00PA PA ON PA.RESOURCE_ID = P2.PROFILE_ID
   JOIN TMT_COMPETENCIES CMP1 ON P2.ROLE_ID = CMP1.ELEMENT_ID
   LEFT OUTER JOIN RATES B ON CMP1.RATE_ID = B.RATE_ID
   UNION ALL SELECT
   1 AS LEVEL_ID, 113 AS TYPE_ID, '' AS RANK, '' AS PARENT_ID, R.RESOURCE_ID AS ELEMENT_ID, 'Unavailableties' AS ELEMENT_NAME, 1 AS CHILD_COUNT, '' E_MAIL_ADDRESS, '' AS EXTERNAL_ID, TDOY.TSK_ASSIGNMENT_ID,
   0 AS REGULAR_COST, 0 AS SPECIAL_COST, R.CALENDAR_ID, R.CALENDAR_NAME AS CALENDAR_NAME, COALESCE(TO_CHAR(TDOY.DAY_YEAR),'') AS DAY_YEAR, case WHEN PMO_STPU.TOTAL_DAY(TDOY.BIT_SCHED) > 0 THEN '1' else '0' END AS WORKING, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 1, 5) AS FROM1,
   SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 7, 5) AS TO1, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 19, 5) AS TO2, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 31, 5) AS TO3,
   SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 43, 5) AS TO4, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 55, 5) AS TO5,
   TO_TIMESTAMP('1899-12-31 00:00:00') AS START_DT, TO_TIMESTAMP('1899-12-31 00:00:00') AS FINISH_DT
   FROM PMO_TMP_PROJECTS_01 TP, PMO_TMP_IMXWBS_0003 R
   JOIN TASK_ASSIGNMENTS FIX ON FIX.RESOURCE_ID = R.RESOURCE_ID AND FIX.BIT_FLAG = '2'
   JOIN TASK_DAY_OF_YEAR TDOY ON TDOY.PROJECT_ID = FIX.PROJECT_ID AND TDOY.TSK_ASSIGNMENT_ID = FIX.TSK_ASSIGNMENT_ID
   WHERE FIX.PROJECT_ID <> TP.ELEMENT_ID AND FIX.TSK_ASSIGNMENT_ID NOT IN (SELECT TA_ID FROM PMO_TMP_IMXWBS_0001)
   UNION ALL SELECT
   1 AS LEVEL_ID, 114 AS TYPE_ID, '' AS RANK, '' AS PARENT_ID, C.ELEMENT_ID, 'Calendar Exception' AS ELEMENT_NAME, 1 AS CHILD_COUNT, '' AS E_MAIL_ADDRESS, '' AS EXTERNAL_ID, '' AS TSK_ASSIGNMENT_ID,
   0 AS REGULAR_COST, 0 AS SPECIAL_COST, C.ELEMENT_ID AS CALENDAR_ID, C.CALENDAR_NAME, COALESCE(TO_CHAR(CEX.EXCEPTION_DATE), '') AS DAY_YEAR, CEX.DAY_STATUS AS WORKING, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 1, 5) AS FROM1,
   SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 7, 5) AS TO1, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 19, 5) AS TO2, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 25, 5) AS FROM3,
   SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 31, 5) AS TO3, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 43, 5) AS TO4, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 49, 5) AS FROM5,
   SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 55, 5) AS TO5, TO_TIMESTAMP('1899-12-31 00:00:00') AS START_DT, TO_TIMESTAMP('1899-12-31 00:00:00') AS FINISH_DT
   FROM CALENDAR_EXCEPTIONS CEX JOIN PMO_TMP_IMXWBS_0002 C
   ON (CEX.CALENDAR_RANK >= SUBSTR(C.CALENDAR_RANK, 1, 8) AND CEX.CALENDAR_RANK <= C.CALENDAR_RANK) AND
   (SUBSTR(C.CALENDAR_RANK, 1, CASE WHEN LENGTH(CEX.CALENDAR_RANK) > LENGTH(C.CALENDAR_RANK) THEN 1 ELSE LENGTH(CEX.CALENDAR_RANK) END) = CEX.CALENDAR_RANK)
   UNION ALL SELECT
   1 AS LEVEL_ID, 114 AS TYPE_ID, '' AS RANK, '' AS PARENT_ID, C.RESOURCE_ID AS ELEMENT_ID, 'Calendar Exception' AS ELEMENT_NAME, 1 AS CHILD_COUNT, '' AS E_MAIL_ADDRESS, '' AS EXTERNAL_ID, '' AS TSK_ASSIGNMENT_ID, 0 AS REGULAR_COST,
   0 AS SPECIAL_COST, C.RESOURCE_ID AS CALENDAR_ID, C.CALENDAR_NAME AS CALENDAR_NAME, COALESCE(TO_CHAR(CEX.EXCEPTION_DATE), '') AS DAY_YEAR, CEX.DAY_STATUS AS WORKING, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 1, 5) AS FROM1,
   SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 7, 5) AS TO1, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 19, 5) AS TO2, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 25, 5) AS FROM3,
   SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 31, 5) AS TO3, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 43, 5) AS TO4, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 49, 5) AS FROM5,
   SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 55, 5) AS TO5, TO_TIMESTAMP('1899-12-31 00:00:00') AS START_DT, TO_TIMESTAMP('1899-12-31 00:00:00') AS FINISH_DT
   FROM CALENDAR_EXCEPTIONS CEX, PMO_TMP_IMXWBS_0003 C WHERE CEX.CALENDAR_ID = C.RESOURCE_ID
   ORDER BY ELEMENT_ID, LEVEL_ID, TYPE_ID, DAY_YEAR;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_IMPEX_A_NEW', 214+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_IMPEX_A_NEW(215) : impexwbs.sql(1327) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN C_AAA;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_A_NEW', 222+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_A_NEW(223) : impexwbs.sql(1335) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_A_NEW', 234+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_A_NEW(235) : impexwbs.sql(1347) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_IMPEX_A_NEW;
ALTER FUNCTION SP_S_IMPEX_A_NEW COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION SP_S_IMPEX_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_S_IMPEX_ASSIGN
(
     p_id VARCHAR2
   , p_bigflag PLS_INTEGER
   , p_rec_status VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_calendar_id VARCHAR2(32);
   h_calendar_name VARCHAR2(254);
   h_rank VARCHAR2(200);
-- Declare Cursors
      -- Resultset Cursor Declaration
      C_BBB PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_IMPEX_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_IMPEX_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := SP_S_IMPEX_ASSIGN(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bigflag), 'NULL')
          || ', ' || '''' || p_rec_status || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_IMPEX_ASSIGN', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_IMPEX_ASSIGN(25) : impexwbs.sql(1388) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   BEGIN
      SELECT PROJECT_ID, RANK INTO h_calendar_id, h_rank FROM TMT_WBS WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_ASSIGN', 37+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_ASSIGN(38) : impexwbs.sql(1401) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT ELEMENT_NAME INTO h_calendar_name FROM TMT_CALENDARS WHERE ELEMENT_ID = h_calendar_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_ASSIGN', 50+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_ASSIGN(51) : impexwbs.sql(1414) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --if( h_calendar_name_Ind == -1)
   If h_calendar_name IS NULL Then
      h_calendar_name := '';
   End If;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0001';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0001
      SELECT TSK_ASSIGNMENT_ID AS TA_ID
      FROM TASK_ASSIGNMENTS T JOIN TMT_WBS W ON W.ELEMENT_ID = T.TASK_ID WHERE W.RANK LIKE h_rank || '%' AND SUBSTR(W.RANK, 1, LENGTH(h_rank)) = h_rank;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_ASSIGN', 78+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_ASSIGN(79) : impexwbs.sql(1442) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0002';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0002
      SELECT TW.ELEMENT_ID, CAL.ELEMENT_ID AS CALENDAR_ID, CAL.ELEMENT_NAME AS CALENDAR_NAME, CAL.RANK AS CALENDAR_RANK
      FROM TMT_WBS TW JOIN TMT_CALENDARS CAL ON TW.ELEMENT_ID = CAL.ELEMENT_ID WHERE TW.RANK LIKE h_rank || '%' AND SUBSTR(TW.RANK, 1,
      LENGTH(h_rank)) = h_rank AND TW.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_ASSIGN', 102+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_ASSIGN(103) : impexwbs.sql(1466) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0003';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0003
      SELECT DISTINCT A.RESOURCE_ID, CAL.ELEMENT_ID AS CALENDAR_ID, CAL.ELEMENT_NAME AS CALENDAR_NAME, CAL.RANK AS CALENDAR_RANK
      FROM PMO_TMP_IMXWBS_0001 JOIN TASK_ASSIGNMENTS A ON A.TSK_ASSIGNMENT_ID = PMO_TMP_IMXWBS_0001.TA_ID AND A.RESOURCE_TYPE = '0' JOIN TMT_CALENDARS CAL ON A.RESOURCE_ID = CAL.ELEMENT_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_ASSIGN', 125+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_ASSIGN(126) : impexwbs.sql(1489) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0004';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0004
      SELECT DISTINCT RESOURCE_ID AS PROFILE_ID
      FROM PMO_TMP_IMXWBS_0001 JOIN TASK_ASSIGNMENTS A ON A.TSK_ASSIGNMENT_ID = PMO_TMP_IMXWBS_0001.TA_ID AND A.RESOURCE_TYPE = '1';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_ASSIGN', 148+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_ASSIGN(149) : impexwbs.sql(1512) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0005';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0005
      SELECT ELEMENT_ID, CALENDAR_ID, CALENDAR_NAME, CALENDAR_RANK FROM PMO_TMP_IMXWBS_0002
      UNION ALL
      SELECT RESOURCE_ID AS ELEMENT_ID, CALENDAR_ID, CALENDAR_NAME, CALENDAR_RANK FROM PMO_TMP_IMXWBS_0003;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_ASSIGN', 172+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_ASSIGN(173) : impexwbs.sql(1536) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_BBB FOR SELECT
   0 AS LEVEL_ID, 13 AS TYPE_ID, R2.RESOURCE_ID, R2.FULL_NAME, R2.E_MAIL_ADDRESS, '' AS TSK_ASSIGNMENT_ID, B.REGULAR_COST, B.SPECIAL_COST, R1.CALENDAR_ID,
   R1.CALENDAR_NAME AS CALENDAR_NAME, '' AS DAY_YEAR, '' AS WORKING, '' AS FROM1, '' AS TO1, '' AS FROM2, '' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5
   FROM PMO_TMP_IMXWBS_0003 R1 JOIN RESOURCES R2 ON R1.RESOURCE_ID = R2.RESOURCE_ID LEFT OUTER JOIN RATES B ON R2.RATE_ID = B.RATE_ID
   UNION ALL SELECT
   0 AS LEVEL_ID, 41 AS TYPE_ID, P2.PROFILE_ID AS RESOURCE_ID, P2.PROFILE_NAME AS FULL_NAME, '' AS E_MAIL_ADDRESS, '' AS TSK_ASSIGNMENT_ID, B.REGULAR_COST,
   B.SPECIAL_COST, h_calendar_id AS CALENDAR_ID, h_calendar_name AS CALENDAR_NAME, '' AS DAY_YEAR, '' AS WORKING, '' AS FROM1, '' AS TO1, '' AS FROM2,
   '' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5
   FROM PMO_TMP_IMXWBS_0004 P1 LEFT OUTER JOIN PROFILES P2 ON P1.PROFILE_ID = P2.PROFILE_ID LEFT OUTER JOIN TMT_COMPETENCIES CMP1 ON P2.ROLE_ID = CMP1.ELEMENT_ID LEFT OUTER JOIN RATES B ON CMP1.RATE_ID = B.RATE_ID
   UNION ALL SELECT
   1 AS LEVEL_ID, 113 AS TYPE_ID, R.RESOURCE_ID, 'Unavailableties' AS FULL_NAME, '' E_MAIL_ADDRESS, TDOY.TSK_ASSIGNMENT_ID, 0 AS REGULAR_COST, 0 AS SPECIAL_COST,
   R.CALENDAR_ID, R.CALENDAR_NAME AS CALENDAR_NAME, COALESCE(TO_CHAR(TDOY.DAY_YEAR),'') AS DAY_YEAR, case WHEN PMO_STPU.TOTAL_DAY(TDOY.BIT_SCHED) > 0 THEN '1' else '0' END AS WORKING,
   SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 7, 5) AS TO1, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 13, 5) AS FROM2,
   SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 19, 5) AS TO2, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 31, 5) AS TO3,
   SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 43, 5) AS TO4, SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 49, 5) AS FROM5,
   SUBSTR(PMO_STPU.PRINT_TIMES(TDOY.BIT_SCHED), 55, 5) AS TO5
   FROM PMO_TMP_IMXWBS_0003 R LEFT OUTER JOIN TASK_DAY_OF_YEAR TDOY ON R.RESOURCE_ID = TDOY.RESOURCE_ID AND TDOY.BIT_FLAG = '2' WHERE TDOY.TSK_ASSIGNMENT_ID NOT IN (SELECT TA_ID FROM PMO_TMP_IMXWBS_0001)
   UNION ALL SELECT
   1 AS LEVEL_ID, 114 AS TYPE_ID, C.ELEMENT_ID AS RESOURCE_ID, 'Resource Calendar Exception' AS FULL_NAME, '' AS E_MAIL_ADDRESS, '' AS TSK_ASSIGNMENT_ID,
   0 AS REGULAR_COST, 0 AS SPECIAL_COST, C.CALENDAR_ID AS CALENDAR_ID, C.CALENDAR_NAME AS CALENDAR_NAME, COALESCE(TO_CHAR(CEX.EXCEPTION_DATE), '') AS DAY_YEAR,
   CEX.DAY_STATUS AS WORKING, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 7, 5) AS TO1,
   SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 19, 5) AS TO2, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 25, 5) AS FROM3,
   SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 31, 5) AS TO3, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 43, 5) AS TO4,
   SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 55, 5) AS TO5
   FROM CALENDAR_EXCEPTIONS CEX, PMO_TMP_IMXWBS_0005 C
   WHERE
   CEX.CALENDAR_RANK >= SUBSTR(C.CALENDAR_RANK, 1, 8) AND CEX.CALENDAR_RANK <= C.CALENDAR_RANK
   AND SUBSTR(C.CALENDAR_RANK, 1, CASE WHEN LENGTH(CEX.CALENDAR_RANK) > LENGTH(C.CALENDAR_RANK) THEN 1 ELSE LENGTH (CEX.CALENDAR_RANK) END) = CEX.CALENDAR_RANK
   ORDER BY RESOURCE_ID, LEVEL_ID, DAY_YEAR;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_IMPEX_ASSIGN', 207+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_IMPEX_ASSIGN(208) : impexwbs.sql(1571) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN C_BBB;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_ASSIGN', 215+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_ASSIGN(216) : impexwbs.sql(1579) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_ASSIGN', 227+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_ASSIGN(228) : impexwbs.sql(1591) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_IMPEX_ASSIGN;
ALTER FUNCTION SP_S_IMPEX_ASSIGN COMPILE DEBUG;

/*****************************************************************************************
Result Set Column Notes:
    ROW_TYPE:
        'C' = CALENDAR (ID, NAME) DOW = 0

        'D' = CALENDAR DAYS
            DOW = 1 = MONDAY,
            DOW = 2 = TUESDAY,
            DOW = 3 = WEDNESDAY,
            DOW = 4 = THURSDAY,
            DOW = 5 = FRIDAY,
            DOW = 6 = SATURDAY,
            DOW = 7 = SUNDAY

        'E' = CALENDAR EXCEPTIONS (EXCEPTION_DATE) DOW = 8

*****************************************************************************************/

Prompt == FUNCTION SP_S_IMPEX_CAL ==
CREATE OR REPLACE FUNCTION SP_S_IMPEX_CAL
(
     p_id VARCHAR2
   , p_bigflag PLS_INTEGER
   , p_rec_status VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_calendar_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_calendar_name VARCHAR2(254);
   h_rank VARCHAR2(200);
   h_type PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      C7752 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7753 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_IMPEX_CAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_IMPEX_CAL'), '-- ' || SYSTIMESTAMP, ':r := SP_S_IMPEX_CAL(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bigflag), 'NULL')
          || ', ' || '''' || p_rec_status || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_IMPEX_CAL', 29+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_IMPEX_CAL(30) : impexwbs.sql(1653) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --  SC_CHAR(h_ranklike, SC_H_FULL_RANK);
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_PROJECTS_01';
   PMO_DB2M.SC_STATE := 0;
   
   h_type := 0;
   h_calendar_id := p_id;
   h_resource_id := NULL;
   h_rank := NULL;
   
   PMO_DB2M.SC_STATE := FC_PORTF_PRJ(h_resource_id, h_calendar_id, h_rank, h_type);
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0006';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0006
      SELECT T.ELEMENT_ID, T.ELEMENT_NAME, C.PARENT_ID AS CALENDAR_ID
      FROM TMT_CALENDARS C
      JOIN TMT_WBS T ON C.ELEMENT_ID = T.ELEMENT_ID
      JOIN PMO_TMP_PROJECTS_01 TMP ON TMP.ELEMENT_ID = T.ELEMENT_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_CAL', 66+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_CAL(67) : impexwbs.sql(1690) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0007';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0007
      SELECT R.RESOURCE_ID AS ELEMENT_ID, R.FULL_NAME AS ELEMENT_NAME, C.PARENT_ID AS CALENDAR_ID
      FROM TMT_CALENDARS C
      JOIN PROJECT_ASSIGNMENTS PA ON C.ELEMENT_ID = PA.RESOURCE_ID
      JOIN RESOURCES R ON PA.RESOURCE_ID = R.RESOURCE_ID
      JOIN PMO_TMP_PROJECTS_01 TMP ON TMP.ELEMENT_ID = PA.PROJECT_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_CAL', 92+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_CAL(93) : impexwbs.sql(1716) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0008';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0008
      SELECT ELEMENT_ID,ELEMENT_NAME,CALENDAR_ID FROM PMO_TMP_IMXWBS_0006
      UNION
      SELECT ELEMENT_ID,ELEMENT_NAME,CALENDAR_ID FROM PMO_TMP_IMXWBS_0007;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_CAL', 116+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_CAL(117) : impexwbs.sql(1740) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_IMXWBS_0009';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_IMXWBS_0009
      SELECT PMO_TMP_IMXWBS_0008.ELEMENT_ID, C.RANK AS RANK FROM TMT_CALENDARS C JOIN PMO_TMP_IMXWBS_0008 ON C.ELEMENT_ID = PMO_TMP_IMXWBS_0008.CALENDAR_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_CAL', 138+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_CAL(139) : impexwbs.sql(1762) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_type, 0) = PMO_CNST.SC_FOLDER Then
      OPEN C7752 FOR SELECT 0 AS LEVEL_ID, 1 AS TYPE_ID, h_rank AS CALENDAR_RANK, h_calendar_id AS CALENDAR_ID, h_calendar_name AS CALENDAR_NAME,
      0 AS DOW, '' AS EXCEPTION_DATE, '0' AS WORKING, '' AS FROM1, '' AS TO1, '' AS FROM2, '' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5
      FROM DUAL
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, h_rank AS CALENDAR_RANK, h_calendar_id AS CALENDAR_ID, h_calendar_name AS CALENDAR_NAME,
      1 AS DOW, '' AS EXCEPTION_DATE, '0' AS WORKING, '' AS FROM1, '' AS TO1, '' AS FROM2, '' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5
      FROM DUAL
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, h_rank AS CALENDAR_RANK, h_calendar_id AS CALENDAR_ID, h_calendar_name AS CALENDAR_NAME,
      2 AS DOW, '' AS EXCEPTION_DATE, '1' AS WORKING, '08:00' AS FROM1, '12:00' AS TO1, '13:00' AS FROM2, '17:00' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5
      FROM DUAL
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, h_rank AS CALENDAR_RANK, h_calendar_id AS CALENDAR_ID, h_calendar_name AS CALENDAR_NAME,
      3 AS DOW, '' AS EXCEPTION_DATE, '1' AS WORKING, '08:00' AS FROM1, '12:00' AS TO1, '13:00' AS FROM2, '17:00' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5
      FROM DUAL
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, h_rank AS CALENDAR_RANK, h_calendar_id AS CALENDAR_ID, h_calendar_name AS CALENDAR_NAME,
      4 AS DOW, '' AS EXCEPTION_DATE, '1' AS WORKING, '08:00' AS FROM1, '12:00' AS TO1, '13:00' AS FROM2, '17:00' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5
      FROM DUAL
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, h_rank AS CALENDAR_RANK, h_calendar_id AS CALENDAR_ID, h_calendar_name AS CALENDAR_NAME,
      5 AS DOW, '' AS EXCEPTION_DATE, '1' AS WORKING, '08:00' AS FROM1, '12:00' AS TO1, '13:00' AS FROM2, '17:00' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5
      FROM DUAL
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, h_rank AS CALENDAR_RANK, h_calendar_id AS CALENDAR_ID, h_calendar_name AS CALENDAR_NAME,
      6 AS DOW, '' AS EXCEPTION_DATE, '1' AS WORKING, '08:00' AS FROM1, '12:00' AS TO1, '13:00' AS FROM2, '17:00' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5
      FROM DUAL
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, h_rank AS CALENDAR_RANK, h_calendar_id AS CALENDAR_ID, h_calendar_name AS CALENDAR_NAME,
      7 AS DOW, '' AS EXCEPTION_DATE, '0' AS WORKING, '' AS FROM1, '' AS TO1, '' AS FROM2, '' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5
      FROM DUAL
      UNION ALL
      SELECT 0 AS LEVEL_ID, 1 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME,
      0 AS DOW, '' AS EXCEPTION_DATE, '0' AS WORKING, '' AS FROM1, '' AS TO1, '' AS FROM2, '' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5
      FROM TMT_CALENDARS CAL
      JOIN PMO_TMP_IMXWBS_0008
      ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.CALENDAR_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 2 AS DOW,
      '' AS EXCEPTION_DATE, CAL.MONDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 3 AS DOW,
      '' AS EXCEPTION_DATE, CAL.TUESDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 4 AS DOW,
      '' AS EXCEPTION_DATE, CAL.WEDNESDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 5 AS DOW,
      '' AS EXCEPTION_DATE, CAL.THURSDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 6 AS DOW,
      '' AS EXCEPTION_DATE, CAL.FRIDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 7 AS DOW,
      '' AS EXCEPTION_DATE, CAL.SATURDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 1 AS DOW,
      '' AS EXCEPTION_DATE, CAL.SUNDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 3 AS TYPE_ID, ACR.RANK AS CALENDAR_RANK, ACR.ELEMENT_ID AS CALENDAR_ID, '' AS CALENDAR_NAME, 8 AS DOW,
      COALESCE(TO_CHAR(CEX.EXCEPTION_DATE), '') AS EXCEPTION_DATE, CEX.DAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 55, 5) AS TO5
      FROM CALENDAR_EXCEPTIONS CEX, PMO_TMP_IMXWBS_0009 ACR
      WHERE
      CEX.CALENDAR_RANK >= SUBSTR(ACR.RANK, 1, 8) AND CEX.CALENDAR_RANK <= ACR.RANK
      AND SUBSTR(ACR.RANK, 1, CASE WHEN LENGTH(CEX.CALENDAR_RANK) > LENGTH(ACR.RANK) THEN 1 ELSE LENGTH (CEX.CALENDAR_RANK) END) = CEX.CALENDAR_RANK
      ORDER BY CALENDAR_ID,DOW,EXCEPTION_DATE;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_IMPEX_CAL', 258+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_IMPEX_CAL(259) : impexwbs.sql(1882) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7752;
   Else
      OPEN C7753 FOR SELECT 0 AS LEVEL_ID, 1 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME,
      0 AS DOW, '' AS EXCEPTION_DATE, '0' AS WORKING, '' AS FROM1, '' AS TO1, '' AS FROM2, '' AS TO2, '' AS FROM3, '' AS TO3, '' AS FROM4, '' AS TO4, '' AS FROM5, '' AS TO5
      FROM TMT_CALENDARS CAL
      JOIN PMO_TMP_IMXWBS_0008
      ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.CALENDAR_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 2 AS DOW,
      '' AS EXCEPTION_DATE, CAL.MONDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(MONDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 3 AS DOW,
      '' AS EXCEPTION_DATE, CAL.TUESDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(TUESDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 4 AS DOW,
      '' AS EXCEPTION_DATE, CAL.WEDNESDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(WEDNESDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 5 AS DOW,
      '' AS EXCEPTION_DATE, CAL.THURSDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(THURSDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 6 AS DOW,
      '' AS EXCEPTION_DATE, CAL.FRIDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(FRIDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 7 AS DOW,
      '' AS EXCEPTION_DATE, CAL.SATURDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(SATURDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 2 AS TYPE_ID, CAL.RANK AS CALENDAR_RANK, PMO_TMP_IMXWBS_0008.ELEMENT_ID AS CALENDAR_ID, PMO_TMP_IMXWBS_0008.ELEMENT_NAME AS CALENDAR_NAME, 1 AS DOW,
      '' AS EXCEPTION_DATE, CAL.SUNDAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(SUNDAY_BIT), 55, 5) AS TO5
      FROM TMT_CALENDARS CAL JOIN PMO_TMP_IMXWBS_0008 ON CAL.ELEMENT_ID = PMO_TMP_IMXWBS_0008.ELEMENT_ID
      UNION ALL
      SELECT 1 AS LEVEL_ID, 3 AS TYPE_ID, ACR.RANK AS CALENDAR_RANK, ACR.ELEMENT_ID AS CALENDAR_ID, '' AS CALENDAR_NAME, 8 AS DOW,
      COALESCE(TO_CHAR(CEX.EXCEPTION_DATE), '') AS EXCEPTION_DATE, CEX.DAY_STATUS AS WORKING,
      SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 1, 5) AS FROM1, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 7, 5) AS TO1,
      SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 13, 5) AS FROM2, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 19, 5) AS TO2,
      SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 25, 5) AS FROM3, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 31, 5) AS TO3,
      SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 37, 5) AS FROM4, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 43, 5) AS TO4,
      SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 49, 5) AS FROM5, SUBSTR(PMO_STPU.PRINT_TIMES(CEX.EXCEPTION_BIT), 55, 5) AS TO5
      FROM CALENDAR_EXCEPTIONS CEX, PMO_TMP_IMXWBS_0009 ACR
      WHERE
      CEX.CALENDAR_RANK >= SUBSTR(ACR.RANK, 1, 8) AND CEX.CALENDAR_RANK <= ACR.RANK
      AND SUBSTR(ACR.RANK, 1, CASE WHEN LENGTH(CEX.CALENDAR_RANK) > LENGTH(ACR.RANK) THEN 1 ELSE LENGTH (CEX.CALENDAR_RANK) END) = CEX.CALENDAR_RANK
      ORDER BY CALENDAR_ID,DOW,EXCEPTION_DATE;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_IMPEX_CAL', 344+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_IMPEX_CAL(345) : impexwbs.sql(1968) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7753;
   End If;
   
   /**HALEH: THE ABOVE CODE IS THE TEMPORARY TABLE VERSION OF THE ABOVE "WITH STATEMENT".*/
   
   --return PROC_STATE;
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_CAL', 356+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_CAL(357) : impexwbs.sql(1980) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_IMPEX_CAL', 368+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_IMPEX_CAL(369) : impexwbs.sql(1992) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_IMPEX_CAL;
ALTER FUNCTION SP_S_IMPEX_CAL COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION SP_IMPEX_ROLLUPDT ==
CREATE OR REPLACE FUNCTION SP_IMPEX_ROLLUPDT
(
     p_id VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_IMPEX_ROLLUPDT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_IMPEX_ROLLUPDT'), '-- ' || SYSTIMESTAMP, ':r := SP_IMPEX_ROLLUPDT(' || 
       '''' || p_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_IMPEX_ROLLUPDT', 17+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_IMPEX_ROLLUPDT(18) : impexwbs.sql(2026) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_id := p_id;
   
   BEGIN
      SELECT PROJECT_ID INTO h_project_id FROM TMT_WBS WHERE ELEMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_IMPEX_ROLLUPDT', 32+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_IMPEX_ROLLUPDT(33) : impexwbs.sql(2041) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_temp_sc_state := FC_Rollup(h_project_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_IMPEX_ROLLUPDT', 40+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_IMPEX_ROLLUPDT(41) : impexwbs.sql(2049) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_IMPEX_ROLLUPDT', 47+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_IMPEX_ROLLUPDT(48) : impexwbs.sql(2056) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_IMPEX_ROLLUPDT', 59+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_IMPEX_ROLLUPDT(60) : impexwbs.sql(2068) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_IMPEX_ROLLUPDT;
ALTER FUNCTION SP_IMPEX_ROLLUPDT COMPILE DEBUG;

