/****************************************************************************************/

Prompt == FUNCTION FC_LOGON ==
CREATE OR REPLACE FUNCTION FC_LOGON
(
     h_logon_name IN OUT NOCOPY VARCHAR2
   , h_logon_password IN OUT NOCOPY VARCHAR2
   , sh_h_version PLS_INTEGER
   , sh_h_security_check PLS_INTEGER
   , h_resource_id IN OUT NOCOPY VARCHAR2
   , h_uniqueID IN OUT NOCOPY VARCHAR2
   , h_repository_type IN OUT NOCOPY PLS_INTEGER
   , h_last_login IN OUT NOCOPY TIMESTAMP
   , h_client_build_number IN OUT NOCOPY VARCHAR2
   , h_is_project_manager IN OUT NOCOPY PLS_INTEGER
   , h_is_del_manager IN OUT NOCOPY PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_version PLS_INTEGER := sh_h_version;
   h_security_check PLS_INTEGER := sh_h_security_check;
   h_active VARCHAR2(1);
   h_allowMultipleLogins VARCHAR2(1);
   h_password_expiries VARCHAR2(50);
   h_rec_status VARCHAR2(1);
   h_calendar_id VARCHAR2(32);
   h_session_id VARCHAR2(32);
   h_setting_value PLS_INTEGER := 0;
   h_max_password_failure_value PLS_INTEGER := 0;
   h_resource_pass_fail_value PLS_INTEGER := 0;
   h_user_dycrypted_password VARCHAR2(16);
   h_timeout PLS_INTEGER := 0;
   h_expiried PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_check_version PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_LOGON') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_LOGON'), '-- ' || SYSTIMESTAMP, ':r := FC_LOGON(' || 
       '''' || h_logon_name || ''''
          || ', ' || CASE WHEN h_logon_password IS NULL THEN '''''' ELSE '''*******''' END
          || ', ' || COALESCE(TO_CHAR(sh_h_version), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_security_check), 'NULL')
          || ', ' || '''' || h_resource_id || ''''
          || ', ' || '''' || h_uniqueID || ''''
          || ', ' || COALESCE(TO_CHAR(h_repository_type), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_last_login || '''',''''''), 'NULL')
          || ', ' || '''' || h_client_build_number || ''''
          || ', ' || COALESCE(TO_CHAR(h_is_project_manager), 'NULL')
          || ', ' || COALESCE(TO_CHAR(h_is_del_manager), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_LOGON', 48+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_LOGON(49) : sessions.sql(52) : ' || SYSTIMESTAMP);
   END IF; 

   --Declare host variables
   
   h_check_version := 0;
   
   h_temp_sc_state := 0;
   
   --Clean Session
   BEGIN
      SELECT SETTING_VALUE
      INTO h_setting_value
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'SESSION_TIMEOUT';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 69+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(70) : sessions.sql(73) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Get MAX_PASSWORD_FAILURE value FROM TMT_SETTINGS
   BEGIN
      SELECT SETTING_VALUE
      INTO h_max_password_failure_value
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'MAX_PASSWORD_FAILURE';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 86+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(87) : sessions.sql(90) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT SETTING_VALUE
      INTO h_repository_type
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'REPOSITORY_TYPE';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 102+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(103) : sessions.sql(106) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT ELEMENT_NAME
      INTO h_client_build_number
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'CLIENT_BUILD_NUMBER';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 118+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(119) : sessions.sql(122) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- resource records are unique on LOGON_NAME field
   -- fetch the resource with using just LOGON_NAME
   BEGIN
      SELECT RTRIM(RESOURCE_ID), PMO_ORASTD.PMO_DECRYPT(LOGON_PASSWORD, RESOURCE_ID), MAX_PASSWORD_FAILURE
      INTO h_resource_id, h_user_dycrypted_password, h_resource_pass_fail_value
      FROM RESOURCES
      WHERE LOGON_NAME = h_logon_name;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 136+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(137) : sessions.sql(140) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      h_temp_sc_state := PMO_CNST.USER_DOES_NOT_EXIST;
   ElsIf (Coalesce(h_security_check, 0) = 1) AND (Coalesce(h_resource_pass_fail_value, 0) = Coalesce(
                h_max_password_failure_value, 0)) Then
      h_temp_sc_state := PMO_CNST.INCORRECT_PASSWORD;
   Else
      BEGIN
         SELECT SETTING_VALUE
         INTO h_check_version
         FROM TMT_SETTINGS
         WHERE ELEMENT_ID = 'CHECK_APPLICATION_VERSION';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 158+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(159) : sessions.sql(162) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         h_check_version := 1;
      End If;
      
      -- The space in the element_id is required!
      If (PMO_ORASTD.strcmp(h_resource_id, 'PMO_SUPERBUILD') <> PMO_CNST.SC_MATCH) AND (Coalesce(
            h_check_version, 0) = 1) Then
         BEGIN
            SELECT SETTING_VALUE
            INTO h_version
            FROM TMT_SETTINGS
            WHERE ELEMENT_ID = 'APPLICATION_VERSION'
            AND SETTING_VALUE = h_version;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 182+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(183) : sessions.sql(186) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            h_temp_sc_state := PMO_CNST.INCOMPATIBLE_VERSION;
         End If;
      End If;
      
      If Coalesce(h_temp_sc_state, 0) = 0 Then
         -- Check for correct password
         If (Coalesce(h_security_check, 0) = 0) OR PMO_ORASTD.strcmp(h_logon_password
               , h_user_dycrypted_password) = PMO_CNST.SC_MATCH Then
            BEGIN
               SELECT
               CALENDAR_ID, PASSWORD_EXPIRES, CASE WHEN TO_DATE(SubStr(PASSWORD_EXPIRES, 1, 10)) < TO_DATE(SYSDATE) THEN 1 ELSE 0 END, ACTIVE, MULTIPLE_LOGINS, REC_STATUS
               INTO h_calendar_id, h_password_expiries, h_expiried, h_active, h_allowMultipleLogins, h_rec_status
               FROM RESOURCES
               WHERE RESOURCE_ID = h_resource_id
               AND TO_DATE(SubStr(COALESCE( EFFECTIVE_DATE, TO_DATE(SYSDATE)), 1, 10)) <= TO_DATE(SYSDATE)
               AND TO_DATE(SubStr(COALESCE( DISCONTINUE_DATE, TO_DATE(SYSDATE)), 1, 10)) >= TO_DATE(SYSDATE);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 210+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(211) : sessions.sql(214) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_temp_sc_state := PMO_CNST.RESOURCE_NOT_ACTIVE;
            Else
               If h_calendar_id IS NULL Then
                  h_temp_sc_state := PMO_CNST.RESOURCE_NOT_ACTIVE;
               Else
                  --was the resource deactivated
                  If (SubStr(h_active, 1, 1) = 'N') OR (SubStr(h_active, 1, 1) = '0') Then
                     h_temp_sc_state := PMO_CNST.RESOURCE_NOT_ACTIVE;
                  Else
                     If (SubStr(h_rec_status, 1, 1) = 'D') OR (SubStr(h_rec_status, 1, 1) = 'L') Then
                        h_temp_sc_state := PMO_CNST.RECORD_ALREADY_DELETED;
                     Else
                        If (Coalesce(h_security_check, 0) = 1) AND (Coalesce(h_expiried, 0) = 1) Then
                           h_temp_sc_state := PMO_CNST.PASSWORD_EXPIRED;
                        Else
                           h_uniqueID := SYS_GUID();
                           
                           --Reads last login date into h_last_login
                           BEGIN
                              SELECT LAST_LOGIN
                              INTO h_last_login
                              FROM RESOURCES
                              WHERE RESOURCE_ID = h_resource_id;
                              PMO_DB2M.SC_STATE := SQLCODE;
                           EXCEPTION
                              WHEN OTHERS THEN
                              BEGIN
                                 PMO_DB2M.SC_STATE := SQLCODE;
                                 IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 245+1) THEN 
                                    LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(246) : sessions.sql(249) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                                 END IF; 
                              END;
                           END;
                           
                           --Updates last login date to current date
                           BEGIN
                              UPDATE RESOURCES
                              SET LAST_LOGIN = SYSTIMESTAMP
                              WHERE RESOURCE_ID = h_resource_id;
                              IF SQL%FOUND = TRUE THEN
                                 PMO_DB2M.SC_STATE := 0;
                              ELSE
                                 PMO_DB2M.SC_STATE := 100;
                              END IF;
                           EXCEPTION
                              WHEN OTHERS THEN
                              BEGIN
                                 PMO_DB2M.SC_STATE := SQLCODE;
                                 IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 265+1) THEN 
                                    LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(266) : sessions.sql(269) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                                 END IF; 
                              END;
                           END;
                           
                           BEGIN
                              UPDATE RESOURCES
                              SET MAX_PASSWORD_FAILURE = 0
                              WHERE RESOURCE_ID = h_resource_id;
                              IF SQL%FOUND = TRUE THEN
                                 PMO_DB2M.SC_STATE := 0;
                              ELSE
                                 PMO_DB2M.SC_STATE := 100;
                              END IF;
                           EXCEPTION
                              WHEN OTHERS THEN
                              BEGIN
                                 PMO_DB2M.SC_STATE := SQLCODE;
                                 IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 284+1) THEN 
                                    LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(285) : sessions.sql(288) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                                 END IF; 
                              END;
                           END;
                        End If;
                     End If;
                  End If;
               End If;
            End If;
         Else
            BEGIN
               UPDATE RESOURCES
               SET MAX_PASSWORD_FAILURE = h_resource_pass_fail_value + 1
               WHERE RESOURCE_ID = h_resource_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 308+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(309) : sessions.sql(312) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            COMMIT;
            
            h_temp_sc_state := PMO_CNST.USER_DOES_NOT_EXIST;
         End If;
      End If;
   End If;
   
   If Coalesce(h_temp_sc_state, 0) <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_LOGON', 322+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_LOGON(323) : sessions.sql(326) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN Coalesce(h_temp_sc_state, 0);
   Else
      h_is_project_manager := 0;
      h_is_del_manager := 0;
      
      -- exclude personal project (156) and admin projects (11) for pools
      BEGIN
         SELECT CASE WHEN COUNT(PA.PROJECT_ID) > 0 then 1 else 0 end
         INTO h_is_project_manager
         FROM PROJECT_ASSIGNMENTS PA, TMT_WBS W
         WHERE PA.RESOURCE_ID = h_resource_id
         AND W.ELEMENT_ID = PA.PROJECT_ID
         AND W.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
         AND BITAND(PA.PARTICIPANT, 1219) > 0;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 344+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(345) : sessions.sql(348) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_is_project_manager, 0) = 1 Then
         h_is_del_manager := 1;
      Else
         -- exclude Admin Summary Task in personal project (156) and admin projects (11) for pools
         BEGIN
            SELECT case when COUNT(RA.ELEMENT_ID) > 0 then 1 else 0 end
            INTO h_is_del_manager
            FROM RESOURCE_ASSIGNMENTS RA, TMT_WBS W
            WHERE BITAND(RA.PARTICIPANT, PMO_CNST.SC_DLV_MANAGER) = PMO_CNST.SC_DLV_MANAGER
            AND RA.RESOURCE_ID = h_resource_id
            AND W.PROJECT_ID = RA.PROJECT_ID
            AND W.ELEMENT_ID = RA.ELEMENT_ID
            AND W.TYPE_ID IN (PMO_CNST.SC_DELIVERABLE,PMO_CNST.SC_DELIVERABLE_TYPE1,PMO_CNST.SC_DELIVERABLE_TYPE2,PMO_CNST.SC_DELIVERABLE_TYPE3,PMO_CNST.SC_WORK_PRODUCT,PMO_CNST.SC_SUMMARY_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST,PMO_CNST.SC_WBS_RISK,PMO_CNST.SC_WBS_ISSUE,PMO_CNST.SC_WBS_REQUIREMENT,PMO_CNST.SC_WBS_DEFECT,PMO_CNST.SC_WBS_SERVICE_REQUEST,PMO_CNST.SC_WBS_ACTION_ITEM)
            AND W.POOL_ID IS NuLL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LOGON', 369+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LOGON(370) : sessions.sql(373) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --trace_printf("\n h_is_project_manager:%d:%d", h_is_project_manager, h_is_project_manager_Ind);
      --trace_printf("\n h_is_del_manager:%d:%d", h_is_del_manager, h_is_del_manager_Ind);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_LOGON', 380+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_LOGON(381) : sessions.sql(384) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_LOGON;
ALTER FUNCTION FC_LOGON COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_LOGON ==
CREATE OR REPLACE FUNCTION SP_LOGON
(
     p_logon_name VARCHAR2
   , p_logon_password VARCHAR2
   , p_version PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_temp_sc_state PLS_INTEGER := 0;
   h_security_check PLS_INTEGER := 0;
   h_str_version VARCHAR2(32);
   h_logon_name VARCHAR2(100);
   h_logon_password VARCHAR2(16);
   h_version PLS_INTEGER := 0;
   h_uniqueID VARCHAR2(32);
   h_repository_type PLS_INTEGER := 0;
   h_last_login TIMESTAMP;
   h_client_build_number VARCHAR2(50);
   h_resource_id VARCHAR2(32);
   h_is_project_manager PLS_INTEGER := 0;
   h_is_del_manager PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      login_user PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_LOGON') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_LOGON'), '-- ' || SYSTIMESTAMP, ':r := SP_LOGON(' || 
       '''' || p_logon_name || ''''
          || ', ' || CASE WHEN p_logon_password IS NULL THEN '''''' ELSE '''*******''' END
          || ', ' || COALESCE(TO_CHAR(p_version), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_LOGON', 34+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_LOGON(35) : sessions.sql(429) : ' || SYSTIMESTAMP);
   END IF; 

   h_logon_name := p_logon_name;
   h_logon_password := p_logon_password;
   h_version := p_version;
   
   h_security_check := 1;
   
   h_uniqueID := NULL;
   h_repository_type := NULL;
   h_last_login := NULL;
   h_client_build_number := NULL;
   h_resource_id := NULL;
   
   --trace_printf("\1- h_version:%d", h_version);
   
   h_temp_sc_state := FC_LOGON(h_logon_name, h_logon_password, h_version, h_security_check
                            , h_resource_id, h_uniqueID, h_repository_type, h_last_login
                            , h_client_build_number, h_is_project_manager, h_is_del_manager);
   
   If Coalesce(h_temp_sc_state, 0) = 0 Then
      h_str_version := PMO_CNST.SC_VERSION;
      
      OPEN login_user FOR SELECT
      0 AS login_error,
      h_uniqueID AS SESSION_ID,
      R.RESOURCE_ID AS RESOURCE_ID,
      R.FULL_NAME AS FULL_NAME,
      COALESCE(R.E_MAIL_ADDRESS, '') AS E_MAIL_ADDRESS,
      COALESCE(R.LOGON_REMINDER, '') AS LOGIN_REMINDER,
      R.CLEARANCE_LEVEL_1,
      R.CLEARANCE_LEVEL_2,
      R.CLEARANCE_LEVEL_3,
      R.CLEARANCE_LEVEL_4,
      R.CLEARANCE_LEVEL_5,
      h_is_project_manager AS PROJECT_MANAGER,
      CASE WHEN EXISTS(SELECT ELEMENT_ID FROM RESOURCE_ASSIGNMENTS 
      WHERE ELEMENT_TYPE = 11 AND RESOURCE_TYPE = '0' 
      AND RESOURCE_ID = R.RESOURCE_ID
      AND TABLE_TYPE = PMO_CNST.SC_TABLE_POL) THEN 1 ELSE 0 END AS RESOURCE_MANAGER,
      MAXDAYHRS,
      h_repository_type AS REPOSITORY_TYPE,
      h_last_login AS LAST_LOGIN,
      NICK_NAME,
      PORTAL_PAGE,
      DASHBOARD,
      OVERWRITEFILE,
      USER_GROUP_ID,
      h_client_build_number AS CLIENT_BUILD_NUMBER,
      MY_PORTAL_URL_1,
      MY_PORTAL_URL_2,
      MY_PORTAL_URL_3,
      MY_PORTAL_URL_4,
      MY_PORTAL_URL_5,
      h_is_del_manager AS DELIVERABLE_MANAGER,
      h_str_version AS SERVER_VERSION
      FROM RESOURCES R
      WHERE R.RESOURCE_ID = h_resource_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_LOGON', 94+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_LOGON(95) : sessions.sql(489) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN login_user;
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_LOGON', 99+1) THEN 
         LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_LOGON(100) : sessions.sql(494) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_sc_state), 'NULL') || ' ID: ' || '''' || h_logon_name || '''');
      END IF; 
      PMODEFINES.PMO_RETURN_STATUS(Coalesce(h_temp_sc_state, 0), h_logon_name);
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_LOGON', 108+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_LOGON(109) : sessions.sql(503) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_LOGON', 120+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_LOGON(121) : sessions.sql(515) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_LOGON;
ALTER FUNCTION SP_LOGON COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_LOGON_SECURITY ==
CREATE OR REPLACE FUNCTION SP_LOGON_SECURITY
(
     p_logon_name VARCHAR2
   , p_logon_password VARCHAR2
   , p_version PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_temp_sc_state PLS_INTEGER := 0;
   h_security_check PLS_INTEGER := 0;
   h_logon_name VARCHAR2(100);
   h_logon_password VARCHAR2(16);
   h_version PLS_INTEGER := 0;
   h_uniqueID VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_repository_type PLS_INTEGER := 0;
   h_last_login TIMESTAMP;
   h_client_build_number VARCHAR2(50);
   h_is_project_manager PLS_INTEGER := 0;
   h_is_del_manager PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      C6667 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_LOGON_SECURITY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_LOGON_SECURITY'), '-- ' || SYSTIMESTAMP, ':r := SP_LOGON_SECURITY(' || 
       '''' || p_logon_name || ''''
          || ', ' || CASE WHEN p_logon_password IS NULL THEN '''''' ELSE '''*******''' END
          || ', ' || COALESCE(TO_CHAR(p_version), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_LOGON_SECURITY', 33+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_LOGON_SECURITY(34) : sessions.sql(565) : ' || SYSTIMESTAMP);
   END IF; 

   h_logon_name := p_logon_name;
   h_logon_password := p_logon_password;
   h_version := p_version;
   
   h_security_check := 0;
   
   h_uniqueID := NULL;
   h_repository_type := NULL;
   h_last_login := NULL;
   h_client_build_number := NULL;
   h_resource_id := NULL;
   
   h_temp_sc_state := FC_LOGON(h_logon_name, h_logon_password, h_version, h_security_check
                            , h_resource_id, h_uniqueID, h_repository_type, h_last_login
                            , h_client_build_number, h_is_project_manager, h_is_del_manager);
   
   If Coalesce(h_temp_sc_state, 0) = 0 Then
      OPEN C6667 FOR SELECT 0 AS login_error, h_uniqueID AS SESSION_ID, R.RESOURCE_ID AS RESOURCE_ID, R.FULL_NAME AS FULL_NAME,
      COALESCE(R.E_MAIL_ADDRESS, '') AS E_MAIL_ADDRESS, COALESCE(R.LOGON_REMINDER, '') AS LOGIN_REMINDER, R.CLEARANCE_LEVEL_1, R.CLEARANCE_LEVEL_2, R.CLEARANCE_LEVEL_3, R.CLEARANCE_LEVEL_4,R.CLEARANCE_LEVEL_5,
      h_is_project_manager AS PROJECT_MANAGER,
      CASE WHEN EXISTS(SELECT ELEMENT_ID FROM RESOURCE_ASSIGNMENTS WHERE ELEMENT_TYPE = 11 AND RESOURCE_TYPE = '0' AND RESOURCE_ID = R.RESOURCE_ID) THEN 1 ELSE 0 END AS RESOURCE_MANAGER, MAXDAYHRS,
      h_repository_type AS REPOSITORY_TYPE,h_last_login AS LAST_LOGIN,NICK_NAME,PORTAL_PAGE,DASHBOARD, OVERWRITEFILE, USER_GROUP_ID,
      h_client_build_number AS CLIENT_BUILD_NUMBER,
      MY_PORTAL_URL_1, MY_PORTAL_URL_2, MY_PORTAL_URL_3, MY_PORTAL_URL_4, MY_PORTAL_URL_5,
      h_is_del_manager AS DELIVERABLE_MANAGER
      FROM RESOURCES R
      WHERE R.RESOURCE_ID = h_resource_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_LOGON_SECURITY', 64+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_LOGON_SECURITY(65) : sessions.sql(596) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C6667;
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_LOGON_SECURITY', 69+1) THEN 
         LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_LOGON_SECURITY(70) : sessions.sql(601) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_sc_state), 'NULL') || ' ID: ' || '''' || h_logon_name || '''');
      END IF; 
      PMODEFINES.PMO_RETURN_STATUS(Coalesce(h_temp_sc_state, 0), h_logon_name);
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_LOGON_SECURITY', 78+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_LOGON_SECURITY(79) : sessions.sql(610) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_LOGON_SECURITY', 90+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_LOGON_SECURITY(91) : sessions.sql(622) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_LOGON_SECURITY;
ALTER FUNCTION SP_LOGON_SECURITY COMPILE DEBUG;

/**************************************************************************/

-- This stored procedure takes the ID of administrator and renew its login information to Password = 'pmoffice' and

-- MAX_PASSWORD_FAILURE = 0

Prompt == FUNCTION SP_RESERVED ==
CREATE OR REPLACE FUNCTION SP_RESERVED
(
     p_id VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_default_password VARCHAR2(16);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_RESERVED') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_RESERVED'), '-- ' || SYSTIMESTAMP, ':r := SP_RESERVED(' || 
       '''' || p_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_RESERVED', 15+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_RESERVED(16) : sessions.sql(658) : ' || SYSTIMESTAMP);
   END IF; 

   h_default_password := 'pmoffice';
   -- default password for admin
   
   BEGIN
      UPDATE RESOURCES SET
      LOGON_PASSWORD = PMO_ORASTD.PMO_ENCRYPT(h_default_password, RESOURCE_ID, ''),
      MAX_PASSWORD_FAILURE = 0,
      REC_DATETIME = SYSTIMESTAMP
      WHERE RESOURCE_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RESERVED', 37+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RESERVED(38) : sessions.sql(680) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_RESERVED', 43+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_RESERVED(44) : sessions.sql(686) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RESERVED', 51+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RESERVED(52) : sessions.sql(694) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RESERVED', 63+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RESERVED(64) : sessions.sql(706) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_RESERVED;
ALTER FUNCTION SP_RESERVED COMPILE DEBUG;

