CREATE OR REPLACE PACKAGE BODY PMODEFINES AS

PROCEDURE PMO_RETURN_STATUS(state IN PLS_INTEGER, host_id IN VARCHAR2) AS
BEGIN
   PMO_DB2M.CURRENT_ID := host_id;
   PMO_DB2M.CURRENT_STATE := state;
   RAISE PMO_DB2M.RETURN_ERROR_EXCEPTION;
END;

PROCEDURE PMO_RETURN_ERROR(state IN PLS_INTEGER, host_id IN VARCHAR2) AS
BEGIN
   PMO_DB2M.CURRENT_ID := host_id;
   PMO_DB2M.CURRENT_STATE := state;
   RAISE PMO_DB2M.RETURN_ERROR_EXCEPTION;
END;

PROCEDURE PMO_RETURN_INVALID(state IN PLS_INTEGER) AS
BEGIN
  PMO_DB2M.CURRENT_ID := PMO_CNST.INVALID;
  PMO_DB2M.CURRENT_STATE := state;
  RAISE PMO_DB2M.RETURN_ERROR_EXCEPTION;
END;

PROCEDURE PMO_VALIDATE_RECORD(tableName IN VARCHAR2, id IN CHAR, recStatus OUT NOCOPY VARCHAR2) AS
BEGIN
   EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || tableName || ' WHERE ELEMENT_ID = :i'
      INTO recStatus
      Using id;
EXCEPTION
   WHEN NO_DATA_FOUND THEN
      PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, id);
END;

PROCEDURE PMO_VALIDATE_DELETE(tableName VARCHAR2, id VARCHAR2, recStatus IN OUT NOCOPY CHAR) AS
BEGIN
   IF recStatus = 'X' THEN
      EXECUTE IMMEDIATE 'SELECT a.REC_STATUS FROM ' || tableName || ' a
      WHERE a.ELEMENT_ID = :i AND LENGTH(a.RANK) > 4 AND NOT a.REC_STATUS = '' L '' AND NOT EXISTS
      (SELECT b.REC_STATUS
               FROM TMT_ORGANIZATIONS b
              WHERE b.REC_STATUS = '' L '' AND instr(b.RANK, a.RANK) = 1)'
         INTO recStatus
         Using id;

   ELSE
      EXECUTE IMMEDIATE 'SELECT REC_STATUS FROM ' || tableName || '
           WHERE ELEMENT_ID = :i AND REC_STATUS NOT IN (''L'',''D'') '
         INTO recStatus
         Using id;

   END IF;

EXCEPTION
   WHEN NO_DATA_FOUND THEN
      PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND_OR_UNAVAIL, id);
   WHEN OTHERS THEN
      PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, id);

END;

PROCEDURE PMO_VALIDATE_DELETE2(tableName VARCHAR2, id VARCHAR2, parent_id VARCHAR2, recStatus IN OUT NOCOPY VARCHAR2) AS
BEGIN
   IF (recStatus = 'X') THEN
      EXECUTE IMMEDIATE 'SELECT a.REC_STATUS INTO :r FROM ' || tableName || ' a ' ||
                'WHERE a.ELEMENT_ID = :i AND a.PARENT_ID = :p AND LENGTH(a.RANK) > 4 ' ||
                ' AND NOT a.REC_STATUS = ''L'' AND NOT ' ||
                ' EXISTS(SELECT b.REC_STATUS FROM ' || tableName || ' b WHERE b.REC_STATUS = ''L'' AND instr(a.RANK, b.RANK) = 1) '
         INTO recStatus
         Using id, parent_id;
   ELSE
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || tableName
               || ' WHERE ELEMENT_ID = ''' || id || ''' AND PARENT_ID = '''
               || parent_id || ''' AND REC_STATUS NOT IN (''L'',''D'')'
         INTO recStatus;
   END IF;
EXCEPTION
   WHEN NO_DATA_FOUND THEN
      PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND_OR_UNAVAIL, id);
   WHEN OTHERS THEN
      PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, id);
END;

PROCEDURE PMO_VALIDATE_EXISTENCE(tableName IN VARCHAR2, id IN VARCHAR2, recStatus OUT NOCOPY VARCHAR2) AS
BEGIN
   EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || tableName || ' WHERE ELEMENT_ID = :i'
      INTO recStatus
      Using id;
EXCEPTION
   WHEN NO_DATA_FOUND THEN
      PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, id);
END;

PROCEDURE PMO_VALIDATE_RESOURCE(id IN VARCHAR2, recStatus OUT NOCOPY VARCHAR2) AS
BEGIN
   SELECT REC_STATUS INTO recStatus FROM RESOURCES WHERE RESOURCE_ID = id;

   IF SubStr(recStatus, 1, 1) IN ('D', 'L', 'A') THEN
      PMO_RETURN_INVALID(PMO_CNST.PARENT_MARKED_AS_DELETED);
   END IF;
EXCEPTION
   WHEN NO_DATA_FOUND THEN
      PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, id);
END;

PROCEDURE PMO_PMO_VERIFY_RANK(h_rank IN VARCHAR2, flag IN PLS_INTEGER) AS
BEGIN
   If (CASE WHEN h_rank IS NULL THEN -1 ELSE 0 END = -1) OR (CASE WHEN (Coalesce(
         flag, 0) = 0) THEN Coalesce(Length(h_rank), 0) ELSE Coalesce(Length(h_rank), 0)
         - 1 END < 4) OR (CASE WHEN (Coalesce(
         flag, 0) = 0) THEN Coalesce(Length(h_rank), 0) ELSE Coalesce(Length(h_rank), 0)
         - 1 END MOD 4 > 0) Then
           PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_rank);

   End If;
END;

PROCEDURE PMO_GET_ROOT_MAX_RANK(tableName IN VARCHAR2, hostVar OUT NOCOPY PLS_INTEGER) AS
BEGIN
   EXECUTE IMMEDIATE 'UPDATE ' || tableName || ' SET MAX_RANK = MAX_RANK + 1, CHILD_COUNT = CHILD_COUNT + 1
         WHERE ELEMENT_ID IS NULL
         RETURNING MAX_RANK INTO :r'
      Using OUT hostVar;
END;

PROCEDURE PMO_GET_MAX_RANK(tableName IN VARCHAR2, id IN CHAR, hostVar OUT NOCOPY PLS_INTEGER) AS
BEGIN
   EXECUTE IMMEDIATE 'UPDATE ' || tableName || ' SET MAX_RANK = MAX_RANK + 1, CHILD_COUNT = CHILD_COUNT + 1
         WHERE ELEMENT_ID = :i
         RETURNING MAX_RANK INTO :r'
      Using IN id, OUT hostVar;
END;

PROCEDURE PMO_GET_UNIQUE_MAX_RANK(tableName IN VARCHAR2, id IN VARCHAR2, ora_type IN PLS_INTEGER, hostVar OUT NOCOPY PLS_INTEGER) AS
BEGIN
   EXECUTE IMMEDIATE 'UPDATE ' || tableName || ' SET MAX_RANK = MAX_RANK + 1, CHILD_COUNT = CHILD_COUNT + 1
         WHERE ELEMENT_ID = :i AND TYPE_ID = :t
         RETURNING MAX_RANK INTO :r'
      Using IN id, IN ora_type, OUT hostVar;
END;

FUNCTION PMO_SELECT_READ(h_myStmt LONG) RETURN PMO_DB2M.REF_CURSOR AS
   out_cursor PMO_DB2M.REF_CURSOR;
BEGIN
   OPEN out_cursor FOR h_myStmt;

   RETURN out_cursor;
END;


END;
/


