CREATE OR REPLACE PACKAGE BODY PMO_FUNC AS

--------------------------------------------------------------------------------------------
-- Math Functions
--------------------------------------------------------------------------------------------

FUNCTION ShiftLeft(nOperand PLS_INTEGER, nShiftCount PLS_INTEGER) RETURN PLS_INTEGER AS
   nVal PLS_INTEGER;
BEGIN
   IF nShiftCount < 0 THEN
      nVal := Trunc(nOperand / Power(2, -nShiftCount));
      RETURN CASE WHEN nOperand < 0 AND nVal = 0 THEN -1 ELSE nVal END;
   ELSIF nShiftCount > 0 THEN
      IF nShiftCount >= 31 THEN
         RETURN 0;
      ELSE
         RETURN BitAnd(nOperand, NotOrMask(nShiftCount)) * Power(2, nShiftCount);
      END IF;
   ELSE
      RETURN nOperand;
   END IF;
END;

FUNCTION ShiftRight(nOperand PLS_INTEGER, nShiftCount PLS_INTEGER) RETURN PLS_INTEGER AS
   nVal PLS_INTEGER;
BEGIN
   IF nShiftCount < 0 THEN
      IF -nShiftCount >= 31 THEN
         RETURN 0;
      ELSE
         RETURN BitAnd(nOperand, NotOrMask(-nShiftCount)) * Power(2, -nShiftCount);
      END IF;
   ELSIF nShiftCount > 0 THEN
      nVal := Trunc(nOperand / Power(2, nShiftCount));
      RETURN CASE WHEN nOperand < 0 AND nVal = 0 THEN -1 ELSE nVal END;
   ELSE
      RETURN nOperand;
   END IF;
END;

-- These Assume a byte 0.255 operand and shift count of 0..7
-- no overflows
FUNCTION ShiftLeftByte(nOperand PLS_INTEGER, nShiftCount PLS_INTEGER) RETURN PLS_INTEGER AS
BEGIN
   RETURN nOperand * Power(2, nShiftCount);
END;

FUNCTION ShiftRightByte(nOperand PLS_INTEGER, nShiftCount PLS_INTEGER) RETURN PLS_INTEGER AS
BEGIN
   RETURN Trunc(nOperand / Power(2, nShiftCount));
END;

FUNCTION BIT_OR(nOperand PLS_INTEGER, nMask PLS_INTEGER) RETURN PLS_INTEGER AS
BEGIN
   RETURN (nOperand - BitAnd(nOperand, nMask)) + nMask;
END;

FUNCTION BIT_XOR(nOperand PLS_INTEGER, nMask PLS_INTEGER) RETURN PLS_INTEGER AS
BEGIN
   RETURN (nOperand - BitAnd(nOperand, nMask)) + (nMask - BitAnd(nOperand, nMask));
END;

--------------------------------------------------------------------------------------------
-- Assignment Functions
--------------------------------------------------------------------------------------------

FUNCTION AssignBool(anLValue IN OUT NOCOPY BOOLEAN, anRValue BOOLEAN) RETURN BOOLEAN AS
BEGIN
   anLValue := anRValue;
   RETURN anLValue;
END;

FUNCTION AssignLong(anLValue IN OUT NOCOPY PLS_INTEGER, anRValue PLS_INTEGER) RETURN PLS_INTEGER AS
BEGIN
   anLValue := anRValue;
   RETURN anLValue;
END;

FUNCTION AssignBigInt(anLValue IN OUT NOCOPY NUMBER, anRValue NUMBER) RETURN NUMBER AS
BEGIN
   anLValue := anRValue;
   RETURN anLValue;
END;

FUNCTION AssignDouble(anLValue IN OUT NOCOPY NUMBER, anRValue NUMBER) RETURN NUMBER AS
BEGIN
   anLValue := anRValue;
   RETURN anLValue;
END;

FUNCTION AssignString(anLValue IN OUT NOCOPY VARCHAR2, anRValue VARCHAR2) RETURN VARCHAR2 AS
BEGIN
   anLValue := anRValue;
   RETURN anLValue;
END;

FUNCTION AssignDate(anLValue IN OUT NOCOPY DATE, anRValue DATE) RETURN DATE AS
BEGIN
   anLValue := anRValue;
   RETURN anLValue;
END;

FUNCTION AssignTimestamp(anLValue IN OUT NOCOPY TIMESTAMP, anRValue TIMESTAMP) RETURN TIMESTAMP AS
BEGIN
   anLValue := anRValue;
   RETURN anLValue;
END;

FUNCTION AssignRaw(anLValue IN OUT NOCOPY RAW, anRValue RAW) RETURN RAW AS
BEGIN
   anLValue := anRValue;
   RETURN anLValue;
END;

--------------------------------------------------------------------------------------------
-- Assignment Functions with different return, Used for Post Increment/Decrement
--
-- But can be used anywhere where an assignment needs to be done with a value returned of same
-- type but different than the assigned value in an expression context
--
--------------------------------------------------------------------------------------------

FUNCTION AssignBoolRet(anLValue IN OUT NOCOPY BOOLEAN, anRValue BOOLEAN, aRetValue BOOLEAN) RETURN BOOLEAN AS
   aRetCopy BOOLEAN;
BEGIN
   aRetCopy := aRetValue;
   anLValue := anRValue;
   RETURN aRetCopy;
END;

FUNCTION AssignLongRet(anLValue IN OUT NOCOPY PLS_INTEGER, anRValue PLS_INTEGER, aRetValue PLS_INTEGER) RETURN PLS_INTEGER AS
   aRetCopy PLS_INTEGER;
BEGIN
   aRetCopy := aRetValue;
   anLValue := anRValue;
   RETURN aRetCopy;
END;

FUNCTION AssignBigIntRet(anLValue IN OUT NOCOPY NUMBER, anRValue NUMBER, aRetValue NUMBER) RETURN NUMBER AS
   aRetCopy NUMBER(19);
BEGIN
   aRetCopy := aRetValue;
   anLValue := anRValue;
   RETURN aRetCopy;
END;

FUNCTION AssignDoubleRet(anLValue IN OUT NOCOPY NUMBER, anRValue NUMBER, aRetValue NUMBER) RETURN NUMBER AS
   aRetCopy NUMBER;
BEGIN
   aRetCopy := aRetValue;
   anLValue := anRValue;
   RETURN aRetCopy;
END;

FUNCTION AssignStringRet(anLValue IN OUT VARCHAR2, anRValue VARCHAR2, aRetValue VARCHAR2) RETURN VARCHAR2 AS
BEGIN
   anLValue := anRValue;
   RETURN aRetValue;
END;

FUNCTION AssignDateRet(anLValue IN OUT NOCOPY DATE, anRValue DATE, aRetValue DATE) RETURN DATE AS
   aRetCopy DATE;
BEGIN
   aRetCopy := aRetValue;
   anLValue := anRValue;
   RETURN aRetCopy;
END;

FUNCTION AssignTimestampRet(anLValue IN OUT NOCOPY TIMESTAMP, anRValue TIMESTAMP, aRetValue TIMESTAMP) RETURN TIMESTAMP AS
   aRetCopy TIMESTAMP;
BEGIN
   aRetCopy := aRetValue;
   anLValue := anRValue;
   RETURN aRetCopy;
END;

FUNCTION AssignRawRet(anLValue IN OUT RAW, anRValue RAW, aRetValue RAW) RETURN RAW AS
BEGIN
   anLValue := anRValue;
   RETURN aRetValue;
END;

END;
/
