CREATE OR REPLACE PACKAGE BODY PMO_ORASTD AS

FUNCTION CLOCK RETURN PLS_INTEGER AS
BEGIN
   RETURN to_char(systimestamp,'SSSSS');
END;

FUNCTION DB2TIMESTAMP(h_date IN VARCHAR2, h_time IN VARCHAR2) RETURN TIMESTAMP AS
BEGIN
  RETURN TO_TIMESTAMP(CASE WHEN h_date IS NULL OR h_time IS NULL THEN NULL
           ELSE h_date || ' ' || CASE WHEN SUBSTR(h_time,1,2)='24' THEN '00' || SUBSTR(h_time,3) ELSE h_time END END);
END;

FUNCTION DB2TIMESTAMP(inDate IN VARCHAR2) RETURN TIMESTAMP AS
BEGIN
   RETURN TO_TIMESTAMP(SubStr(inDate,1,26));
END;

FUNCTION DB2DATE(inDate IN VARCHAR2) RETURN DATE AS
BEGIN
   RETURN TO_DATE(SubStr(inDate,1,10));
END;

FUNCTION DB2TIMESTAMPDIFF(in_unit IN PLS_INTEGER, str_Timestamp1 IN VARCHAR2, str_Timestamp2 IN VARCHAR2)
RETURN PLS_INTEGER
AS
    delta_rest PLS_INTEGER := 0;
    in_result PLS_INTEGER := 0;
    E_TIMESTAMPDIFF_INVALID_OPTION EXCEPTION;
BEGIN
/*    delta_years := TO_NUMBER(SUBSTR(str_Timestamp1,1,4)) - TO_NUMBER(SUBSTR(str_Timestamp2,1,4));
    delta_months := TO_NUMBER(SUBSTR(str_Timestamp1,6,2)) - TO_NUMBER(SUBSTR(str_Timestamp2,6,2));
    delta_days := TO_NUMBER(SUBSTR(str_Timestamp1,9,2)) - TO_NUMBER(SUBSTR(str_Timestamp2,9,2));
    delta_hours := TO_NUMBER(SUBSTR(str_Timestamp1,12,2)) - TO_NUMBER(SUBSTR(str_Timestamp2,12,2));
    delta_minutes := TO_NUMBER(SUBSTR(str_Timestamp1,15,2)) - TO_NUMBER(SUBSTR(str_Timestamp2,15,2));
    delta_seconds := TO_NUMBER(SUBSTR(str_Timestamp1,18,2)) - TO_NUMBER(SUBSTR(str_Timestamp2,18,2));
    delta_fractional_seconds := TO_NUMBER(SUBSTR(str_Timestamp1,21,6)) - TO_NUMBER(SUBSTR(str_Timestamp2,21,6));
*/
    CASE in_unit
        WHEN 4 THEN                 -- TIMESTAMP DIFFERENCE IN MINUTES
            IF SubStr(str_Timestamp1,1,10) = SubStr(str_Timestamp2,1,10) THEN
                in_result := (TO_NUMBER(SubStr(str_Timestamp1,12,2)) - TO_NUMBER(SubStr(str_Timestamp2,12,2))) * 60
                                    + TO_NUMBER(SubStr(str_Timestamp1,15,2)) - TO_NUMBER(SubStr(str_Timestamp2,15,2));
                IF in_result <> 0 AND SubStr(str_Timestamp1,18,26) <> SubStr(str_Timestamp2,18,26) THEN
                    IF SubStr(str_Timestamp1,18,26) > SubStr(str_Timestamp2,18,26) THEN
                        delta_rest := 1;
                    ELSE
                        delta_rest := - 1;
                    END IF;
                    IF in_result * delta_rest < 0 THEN
                        IF in_result < 0 THEN
                            in_result := in_result + 1;
                        ELSE
                            in_result := in_result - 1;
                        END IF;
                    END IF;
                END IF;
            ELSE
                RAISE E_TIMESTAMPDIFF_INVALID_OPTION;
            END IF;
        WHEN 64 THEN                -- TIMESTAMP DIFFERENCE IN MONTHES
            in_result := (TO_NUMBER(SubStr(str_Timestamp1,1,4)) - TO_NUMBER(SubStr(str_Timestamp2,1,4))) * 12
                                + TO_NUMBER(SubStr(str_Timestamp1,6,2)) - TO_NUMBER(SubStr(str_Timestamp2,6,2));
            IF in_result <> 0 AND SubStr(str_Timestamp1,9,26) <> SubStr(str_Timestamp2,9,26) THEN
                IF SubStr(str_Timestamp1,9,26) > SubStr(str_Timestamp2,9,26) THEN
                    delta_rest := 1;
                ELSE
                    delta_rest := - 1;
                END IF;
                IF in_result * delta_rest < 0 THEN
                    IF in_result < 0 THEN
                        in_result := in_result + 1;
                    ELSE
                        in_result := in_result - 1;
                    END IF;
                END IF;
            END IF;
        ELSE
            RAISE E_TIMESTAMPDIFF_INVALID_OPTION;
    END CASE;
    RETURN in_result;
EXCEPTION
    WHEN OTHERS THEN
        LFILE.FPUT( 'AllTrace', 'DB2TIMESTAMPDIFF with inputs ' || in_unit || ', ' || str_Timestamp1 || ', ' || str_Timestamp2 || ' is not supported.' );
END;

FUNCTION PMO_DECRYPT(encrypted_raw RAW, key_string VARCHAR2) RETURN VARCHAR2 AS
   temp VARCHAR2(2048);
   raw_key RAW(256) := UTL_RAW.CAST_TO_RAW(key_string);
   decrypted_raw RAW(2048);

BEGIN

   IF encrypted_raw IS NULL THEN
      RETURN NULL;
   ELSE
      dbms_obfuscation_toolkit.DESDecrypt(INPUT => encrypted_raw
                                         ,KEY => raw_key
                                         ,decrypted_data => decrypted_raw);
      temp := UTL_RAW.CAST_TO_VARCHAR2(decrypted_raw);
      temp := Ltrim(temp);
      RETURN temp;
   END IF;

END;

FUNCTION PMO_ENCRYPT(input_string VARCHAR2, key_string VARCHAR2, hint VARCHAR2 := '') RETURN RAW AS
   raw_input RAW(256);
   raw_key RAW(256) := UTL_RAW.CAST_TO_RAW(key_string);
   encrypted_raw RAW(2048);
   temp_input_str VARCHAR2(128);
BEGIN
   temp_input_str := input_string;

   IF Mod(Length(temp_input_str), 8) <> 0 THEN
      temp_input_str := Lpad(input_string, Length(input_string) + (8 - Mod(Length(input_string), 8)), ' ');
   END IF;

   raw_input := UTL_RAW.CAST_TO_RAW(temp_input_str);
   dbms_obfuscation_toolkit.DESEncrypt(INPUT => raw_input
                                      ,KEY => raw_key
                                      ,encrypted_data => encrypted_raw);
   RETURN encrypted_raw;
END;

FUNCTION strncmp(Mainstr IN VARCHAR2, Searchstr VARCHAR2, CompareLen PLS_INTEGER) RETURN PLS_INTEGER AS
BEGIN
   RETURN CASE WHEN Mainstr IS NULL AND searchstr IS NULL THEN 0
               WHEN Mainstr IS NULL AND searchstr IS NOT NULL THEN -1
               WHEN Mainstr IS NOT NULL AND searchstr IS NULL THEN 1
               WHEN SubStr(Mainstr, 1, compareLen) = SubStr(Searchstr, 1, compareLen) THEN 0
               WHEN SubStr(Mainstr, 1, compareLen) < SubStr(Searchstr, 1 ,compareLen) THEN -1 ELSE 1 END;
END;

FUNCTION strcmp(Mainstr IN VARCHAR2, Searchstr VARCHAR2) RETURN PLS_INTEGER AS
BEGIN
   RETURN CASE WHEN Mainstr IS NULL AND searchstr IS NULL THEN 0
               WHEN Mainstr IS NULL AND searchstr IS NOT NULL THEN -1
               WHEN Mainstr IS NOT NULL AND searchstr IS NULL THEN 1
               WHEN Mainstr = Searchstr THEN 0 WHEN Mainstr < Searchstr THEN -1 ELSE 1 END;
END;

-- TODO: needs rewrite
FUNCTION PMO_HEX(h_str VARCHAR2) RETURN VARCHAR2 AS
   nVal RAW(4000);
BEGIN
   IF h_str IS NULL THEN
      RETURN NULL;
   ELSE
      nVal := UTL_RAW.CAST_TO_RAW(h_str);
      RETURN nVal;
   END IF;
END;

FUNCTION TODECIMAL(str VARCHAR2, scale PLS_INTEGER := 19, pre PLS_INTEGER := 0) RETURN NUMBER AS
BEGIN
   RETURN Trunc(TO_NUMBER(str), pre);
END;

FUNCTION TODECIMAL(val NUMBER, scale PLS_INTEGER := 19, pre PLS_INTEGER := 0) RETURN NUMBER AS
BEGIN
   RETURN Trunc(val, pre);
END;

FUNCTION StringConcat(a VARCHAR2, b VARCHAR2) RETURN VARCHAR2 AS
BEGIN
   RETURN CASE WHEN a IS NULL OR b IS NULL THEN NULL ELSE a || b END;
END;

FUNCTION DAY(indate TIMESTAMP) RETURN PLS_INTEGER AS
BEGIN
   RETURN TO_NUMBER(TO_CHAR(inDate, 'DD'));
END;

FUNCTION DAYOFWEEK(in_date IN TIMESTAMP) RETURN PLS_INTEGER AS
BEGIN
   RETURN TO_CHAR(in_date, 'D');
END;

FUNCTION DAYOFWEEK_ISO(in_date IN TIMESTAMP) RETURN PLS_INTEGER AS
   NUM PLS_INTEGER;
BEGIN
   NUM := TO_NUMBER(TO_CHAR(in_date, 'D'))-1;
   RETURN CASE WHEN NUM <> 0 THEN NUM ELSE 7 END;
END;

FUNCTION DAYS(indate TIMESTAMP) RETURN PLS_INTEGER AS
BEGIN
   RETURN TO_NUMBER(TO_CHAR(inDate, 'j'));
END;

FUNCTION JULIAN_DAY(inDate VARCHAR2) RETURN PLS_INTEGER AS
BEGIN
   RETURN TO_NUMBER(TO_CHAR(TO_DATE(SubStr(inDate, 1, 10)), 'j'));
END;

FUNCTION MONTH(inDate TIMESTAMP) RETURN PLS_INTEGER AS
BEGIN
   RETURN TO_NUMBER(TO_CHAR(inDate, 'MM'));
END;

--- KLUDGE: ---
FUNCTION MONTH(inDiff PLS_INTEGER) RETURN PLS_INTEGER AS
BEGIN
   RETURN Trunc(inDiff/31);
END;

FUNCTION MONTH(inDate VARCHAR2) RETURN PLS_INTEGER AS
BEGIN
   RETURN TO_NUMBER(TO_CHAR(TO_DATE(inDate), 'MM'));
END;

FUNCTION MONTH(inDate DATE) RETURN PLS_INTEGER AS
BEGIN
   RETURN TO_NUMBER(TO_CHAR(inDate, 'MM'));
END;

FUNCTION LOCATE(in_str1 IN VARCHAR2, in_str2 IN VARCHAR2, pos PLS_INTEGER DEFAULT 1) RETURN VARCHAR2 AS
BEGIN
   RETURN InStr(in_str2, in_str1, pos);
END;

-- TODO: this may need attention
FUNCTION TIME(inTime VARCHAR2) RETURN VARCHAR2 AS
BEGIN
   RETURN CASE WHEN Length(intime) > 10 THEN SubStr(intime, 12, 8) ELSE inTime END;
END;

-- TODO: this may need attention
-- In DB2
--(CASE WHEN JULIAN_DAY(finish) > JULIAN_DAY(start) THEN COALESCE( (SELECT COALESCE(DURATION_S(BIT_SCHED, TIME(start)), 0) FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = project_id AND DAY_YEAR = DATE(start)), 0 ) + COALESCE( (SELECT COALESCE(SUM(AVAILABLE_MIN), 0) FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = project_id AND DAY_YEAR BETWEEN DATE(start) + 1 DAYS AND DATE(finish) - 1 DAYS), 0 ) + COALESCE( (SELECT COALESCE(DURATION_F(BIT_SCHED, TIME(finish)), 0) FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = project_id AND DAY_YEAR = DATE(finish)), 0 )ELSE COALESCE( (SELECT COALESCE(DURATION_SF(BIT_SCHED, TIME(start), TIME(finish)), 0) FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = project_id AND DAY_YEAR = DATE(start)), 0 )END)
FUNCTION CAL_DURATION(project_id CHAR, start_date VARCHAR2, finish VARCHAR2) RETURN PLS_INTEGER AS
   RESULT PLS_INTEGER;
BEGIN
   SELECT CASE
             WHEN JULIAN_DAY(finish) > JULIAN_DAY(start_date) THEN
              Coalesce((SELECT Coalesce(PMO_STPU.getNumberOfMinutesStartingAt(BIT_SCHED, TIME(start_date)), 0)
                         FROM CALENDAR_DAY_OF_YEAR
                        WHERE ELEMENT_ID = project_id AND DAY_YEAR = DB2DATE(start_date))
                      ,0) +
              Coalesce((SELECT Coalesce(Sum(AVAILABLE_MIN), 0)
                         FROM CALENDAR_DAY_OF_YEAR
                        WHERE ELEMENT_ID = project_id AND DAY_YEAR BETWEEN DB2DATE(start_date) + 1 AND
                              DB2DATE(finish) - 1)
                      ,0) + Coalesce((SELECT Coalesce(PMO_STPU.getNumberOfMinutesFinishingAt(BIT_SCHED, TIME(finish)), 0)
                                       FROM CALENDAR_DAY_OF_YEAR
                                      WHERE ELEMENT_ID = project_id AND DAY_YEAR = DB2DATE(finish))
                                    ,0)
             ELSE
              Coalesce((SELECT Coalesce(PMO_STPU.getNumberOfMinutesBetweenSAndF(BIT_SCHED
                                                  ,TIME(start_date)
                                                  ,TIME(finish))
                                      ,0)
                         FROM CALENDAR_DAY_OF_YEAR
                        WHERE ELEMENT_ID = project_id AND DAY_YEAR = DB2DATE(start_date))
                      ,0)
          END
     INTO RESULT
     FROM dual;

   RETURN RESULT;
END;

FUNCTION WEEK(inDate TIMESTAMP) RETURN PLS_INTEGER AS
BEGIN
   RETURN TO_NUMBER(TO_CHAR(inDate, 'WW'));
END;

FUNCTION WEEK_ISO(inDate TIMESTAMP) RETURN PLS_INTEGER AS
BEGIN
   RETURN TO_NUMBER(TO_CHAR(inDate, 'IW'));
END;

FUNCTION YEAR(inDate TIMESTAMP) RETURN PLS_INTEGER AS
BEGIN
   RETURN TO_NUMBER(TO_CHAR(inDate, 'YYYY'));
END;

FUNCTION LEFT(inStr VARCHAR2,Len PLS_INTEGER) RETURN VARCHAR2 AS
BEGIN
   RETURN SUBSTR(inStr,1,Len);
END;





END;
/


