CREATE OR REPLACE PACKAGE BODY PMO_STPU AS

FUNCTION strreplps(pResult IN OUT NOCOPY VARCHAR2, nResultSize IN PLS_INTEGER, pSrc IN VARCHAR2, pFind IN VARCHAR2, pReplPrefix IN VARCHAR2, pRepl IN VARCHAR2, pReplSuffix IN VARCHAR2) RETURN PLS_INTEGER AS
BEGIN
    pResult := SubStr(Replace(pSrc, pFind, pReplPrefix || pRepl || pReplSuffix), 1, nResultSize);
    RETURN Length(pResult);
END;

FUNCTION strrepl(pResult IN OUT NOCOPY VARCHAR2, nResultSize IN PLS_INTEGER, pSrc IN VARCHAR2, pFind IN VARCHAR2, pRepl IN VARCHAR2) RETURN PLS_INTEGER AS
BEGIN
    pResult := SubStr(Replace(pSrc, pFind, pRepl), 1, nResultSize);
    RETURN Length(pResult);
END;

FUNCTION getFirstBit(inRaw IN RAW) RETURN PLS_INTEGER AS
  dig PLS_INTEGER;
  i PLS_INTEGER;
  inRawChar VARCHAR2(72);
  bits PLS_INTEGER;
  nLen PLS_INTEGER;
BEGIN
  bits := 288;
  IF inRaw IS NOT NULL THEN
     inRawChar := inRaw;
     nLen := Length(inRawChar);
     FOR i IN 1 .. nLen LOOP
        dig := Ascii(SUBSTR(inRawChar, i, 1));

        IF dig <> 48 THEN
           bits := (i - 1) * 4 + 3 - LastBitsAsciiHex(dig + 1);
           EXIT;
        END IF;
     END LOOP;
  END IF;

  RETURN bits;
END;

FUNCTION getLastBit(inRaw IN RAW) RETURN PLS_INTEGER AS
  dig PLS_INTEGER;
  i PLS_INTEGER;
  inRawChar VARCHAR2(72);
  bits PLS_INTEGER;
  nLen PLS_INTEGER;
BEGIN
  bits := -1;
  IF inRaw IS NOT NULL THEN
     inRawChar := inRaw;
     nLen := Length(inRawChar);
     FOR i IN REVERSE 1 .. nLen LOOP
        dig := Ascii(SUBSTR(inRawChar, i, 1));

        IF dig <> 48 THEN
           bits := (i - 1) * 4 + 3 - FirstBitsAsciiHex(dig + 1);
           EXIT;
        END IF;
     END LOOP;
  END IF;

  RETURN bits + 1;
END;

FUNCTION getLastBitSkipFinish(inDay RAW, skip PLS_INTEGER) RETURN PLS_INTEGER AS
    lastBit PLS_INTEGER;
    skipCount PLS_INTEGER := 0;
    bits PLS_INTEGER;
    inRaw VARCHAR2(72) := inDay;
    i PLS_INTEGER;
BEGIN
    IF inDay IS NOT NULL THEN
        lastBit := getLastBit(inDay);

        IF skip = 0 OR lastBit = 0 THEN
           RETURN lastBit;
        END IF;

        FOR i IN REVERSE 1..Trunc((lastBit-1) / 4)+1 LOOP
            bits := BitsAsciiHex(Ascii(SubStr(inRaw, i, 1)) + 1);

            IF bits > 0 THEN
                skipCount := skipCount + CountBitsNibble(bits+1);

                IF skipCount > skip THEN
                    RETURN 4 * (i-1) + extraLastBits(bits*4 + skipCount - skip - 1 + 1) + 1;
                END IF;
            END IF;
        END LOOP;
    END IF;

    RETURN 0;
END;

FUNCTION getLastBitSkipStart(inDay RAW, skip PLS_INTEGER) RETURN PLS_INTEGER AS
    lastBit PLS_INTEGER;
    skipCount PLS_INTEGER := 0;
    bits PLS_INTEGER;
    inRaw VARCHAR2(72) := inDay;
    i PLS_INTEGER;
BEGIN
    IF inDay IS NOT NULL THEN
        lastBit := getLastBit(inDay);

        IF skip = 0 OR lastBit = 0 THEN
           RETURN lastBit;
        END IF;

        FOR i IN REVERSE 1..Trunc((lastBit-1) / 4)+1 LOOP
            bits := BitsAsciiHex(Ascii(SubStr(inRaw, i, 1)) + 1);

            IF bits > 0 THEN
                skipCount := skipCount + CountBitsNibble(bits+1);

                IF skipCount >= skip THEN
                    RETURN 4 * (i-1) + extraLastBits(bits*4 + skipCount - skip + 1);
                END IF;
            END IF;
        END LOOP;
    END IF;

    RETURN 0;
END;

FUNCTION getLastBitSkip(inDay RAW, skip PLS_INTEGER) RETURN PLS_INTEGER AS
BEGIN
    RETURN getLastBitSkipStart(inDay, skip);
END;

FUNCTION getFirstBitSkipStart(inDay RAW, skip IN PLS_INTEGER) RETURN PLS_INTEGER AS
    firstBit PLS_INTEGER;
    skipCount PLS_INTEGER := 0;
    bits PLS_INTEGER;
    inRaw VARCHAR2(72) := inDay;
BEGIN
    IF inDay IS NOT NULL THEN
        firstBit := getFirstBit(inDay);

        IF skip = 0 THEN
           RETURN firstBit;
        END IF;

        FOR i IN Trunc(firstBit / 4)+1..72 LOOP
            bits := BitsAsciiHex(Ascii(SubStr(inRaw, i, 1)) + 1);

            IF bits > 0 THEN
                skipCount := skipCount + CountBitsNibble(bits+1);

                IF skipCount > skip THEN
                    RETURN 4 * (i-1) + extraFirstBits(BitAnd(bits, 7)*4 + skipCount - skip - 1 + 1);
                END IF;
            END IF;
        END LOOP;
    END IF;

    RETURN 288;
END;

FUNCTION getFirstBitSkipFinish(inDay RAW, skip IN PLS_INTEGER) RETURN PLS_INTEGER AS
    firstBit PLS_INTEGER;
    skipCount PLS_INTEGER := 0;
    bits PLS_INTEGER;
    inRaw VARCHAR2(72) := inDay;
BEGIN
    IF inDay IS NOT NULL THEN
        firstBit := getFirstBit(inDay);

        IF skip = 0 THEN
           RETURN firstBit;
        END IF;

        FOR i IN Trunc(firstBit / 4)+1..72 LOOP
            bits := BitsAsciiHex(Ascii(SubStr(inRaw, i, 1)) + 1);

            IF bits > 0 THEN
                skipCount := skipCount + CountBitsNibble(bits+1);

                IF skipCount >= skip THEN
                    RETURN 4 * (i-1) + extraFirstBits(BitAnd(bits, 7)*4 + skipCount - skip + 1)+1;
                END IF;
            END IF;
        END LOOP;
    END IF;

    RETURN 288;
END;

FUNCTION getFirstBitSkip(inDay RAW, skip IN PLS_INTEGER) RETURN PLS_INTEGER AS
BEGIN
    RETURN getFirstBitSkipFinish(inDay, skip);
END;

FUNCTION doUpdate(update_stmt VARCHAR2) RETURN PLS_INTEGER AS
BEGIN
  BEGIN
     EXECUTE IMMEDIATE update_stmt;
     PMO_DB2M.SC_STATE := SQLCODE;
  EXCEPTION
     WHEN OTHERS THEN
        PMO_DB2M.SC_STATE := SQLCODE;
  END;
  RETURN PMO_DB2M.SC_STATE;
END;

FUNCTION IsDelimit(nextChar CHAR, delims VARCHAR2) RETURN PLS_INTEGER AS
BEGIN
  RETURN CASE WHEN instr(delims, nextChar) > 0 THEN 1 ELSE 0 END;
END;

FUNCTION REstrtok(str IN VARCHAR2, delims VARCHAR2, save IN OUT VARCHAR2) RETURN VARCHAR2 AS
  resultStr VARCHAR2(10000);
  nextChar CHAR(1);
  delIndx PLS_INTEGER := 1;
  StrIndx PLS_INTEGER;
  strStart VARCHAR2(1000);
BEGIN
  IF (str IS NULL) OR Length(str) = 0 THEN
     resultStr := save;
  ELSE
     resultStr := str;
  END IF;

  IF (resultStr IS NULL OR Length(resultStr) = 0) THEN
     RETURN NULL;
  END IF;
  --// Delimiters
  nextChar := SubStr(resultStr, delIndx, 1);
  WHILE (IsDelimit(nextChar, delims) = 1 AND nextChar IS NOT NULL) LOOP
     delIndx := delIndx + 1;
     nextChar := SubStr(resultStr, delIndx, 1);
  END LOOP;
  IF (nextChar IS NULL) THEN
     RETURN NULL;
  END IF;

  --// Characters

  StrIndx := delIndx;
  WHILE (IsDelimit(nextChar, delims) != 1 AND nextChar IS NOT NULL) LOOP
     StrIndx := StrIndx + 1;
     nextChar := SubStr(resultStr, StrIndx, 1);
  END LOOP;

  IF nextChar IS NULL THEN
     save := SubStr(resultStr, StrIndx);
  ELSE
     save := SubStr(resultStr, StrIndx + 1);
  END IF;
  strStart := SubStr(resultStr, delIndx, StrIndx - 1);
  RETURN strStart;
END;

PROCEDURE toBase64(num IN PLS_INTEGER, buf OUT VARCHAR2) AS
  DivNum PLS_INTEGER;
  ModNum PLS_INTEGER;
BEGIN

  IF (num > 16777215) THEN
     buf := 'qqqq';
     RETURN;
  END IF;

  DivNum := num;

  WHILE DivNum > 64 LOOP
     ModNum := Mod(DivNum, 64);

     IF ModNum > 46 THEN
        buf := Chr(ModNum + 50) || buf;
     ELSE
        buf := Chr(ModNum + 48) || buf;
     END IF;

     DivNum := Trunc(DivNum / 64);
  END LOOP;

  IF DivNum >= 47 THEN
     DivNum := DivNum + 2;
  END IF;

  buf := Chr(DivNum + 48) || buf;

  buf := Lpad(buf, 4, '0');
END;

PROCEDURE toBase64Long(num IN PLS_INTEGER, buf OUT VARCHAR2) AS
  indx PLS_INTEGER := 7;
  newNum PLS_INTEGER;
  BaseNum PLS_INTEGER := num;
BEGIN
  -- check if greater than the maximum
  buf := '';
  WHILE (BaseNum > 63) LOOP
     newNum := Trunc(BaseNum / 64);

     buf := intToBase64(BaseNum - newNum * 64 + 1) || buf;

     BaseNum := newNum;
     indx := indx - 1;
  END LOOP;
  buf := intToBase64(BaseNum + 1) || buf;
  IF (indx > 0) THEN
     buf := Lpad(buf, 8, '0');
  END IF;
END;

PROCEDURE incrBase64(in_buf IN OUT VARCHAR2, in_level IN PLS_INTEGER) AS
  h_FirstIndex PLS_INTEGER;
  h_Indx PLS_INTEGER;
  h_len PLS_INTEGER;
  h_val PLS_INTEGER;
  h_pref VARCHAR2(128);
  h_suff VARCHAR2(128);
BEGIN
  h_len := length(in_buf);
  h_FirstIndex := in_level * 4 + 1;
  FOR h_Indx IN REVERSE h_FirstIndex .. h_FirstIndex + 3 LOOP
     h_pref := '';
     h_suff := '';
     IF (substr(in_buf, h_Indx, 1) = 'q') THEN
        IF h_indx > 1 THEN
           h_pref := substr(in_buf, 1, h_Indx - 1);
        END IF;
        IF h_Indx + 1 <= h_len THEN
           h_suff := substr(in_buf, h_Indx + 1);
        END IF;
        in_buf := h_pref || '0' || h_suff;

     ELSE
        h_val := base64ToIntArr(ascii(substr(in_buf, h_Indx, 1)) - 48 + 1);

        IF h_indx > 1 THEN
           h_pref := substr(in_buf, 1, h_Indx - 1);
        END IF;
        IF h_Indx + 1 <= h_len THEN
           h_suff := substr(in_buf, h_Indx + 1);
        END IF;
        in_buf := h_pref || intToBase64(h_val + 1 + 1) || h_suff;

        EXIT;
     END IF;
  END LOOP;
END;

PROCEDURE Base64toInt(buf IN OUT VARCHAR2, result OUT VARCHAR2) AS

  LEVEL PLS_INTEGER;
  offset PLS_INTEGER;
  val PLS_INTEGER;
  i PLS_INTEGER;
  temp_index PLS_INTEGER;
  temp_value CHAR(9);

  temp_substr VARCHAR(128);
  temp_base64 PLS_INTEGER;
  temp_multi PLS_INTEGER;
  temp_ascii PLS_INTEGER;
BEGIN

  result := '';
  LEVEL := Coalesce(Length(buf), 0);
  i := 0;
  WHILE (i < Trunc(LEVEL / 4)) LOOP
     offset := i * 4;
     val := 0;

     temp_index := 3;
     WHILE (temp_index > -1) LOOP
        temp_substr := SubStr(buf, offset + temp_index + 1, 1);
        temp_ascii := Ascii(temp_substr);
        temp_base64 := base64ToIntArr(temp_ascii - 48+1);
        temp_multi := Base64MultiplicationTable(temp_index+1);

        val := val + temp_multi * (temp_base64);

        temp_index := temp_index - 1;
     END LOOP;
     temp_value := val;
     IF i != 0 THEN
        result := Rtrim(Ltrim(result)) || '.';
     END IF;
     result := Rtrim(Ltrim(result)) || Rtrim(Ltrim(temp_value));
     i := i + 1;

  END LOOP;

END;

FUNCTION GetMinutes(inTime IN VARCHAR2) RETURN PLS_INTEGER AS
BEGIN
   RETURN SubStr(inTime, 4, 2);
END;

FUNCTION GetHour(inTime IN VARCHAR2) RETURN PLS_INTEGER AS
BEGIN
   RETURN SubStr(inTime, 1, 2);
END;

PROCEDURE filldayStruct(time1 VARCHAR2, time2 VARCHAR2, inDay IN OUT NOCOPY RAW) AS
    inRaw VARCHAR(72) := inDay;
    retVal VARCHAR2(72);
    sTime VARCHAR2(8) := substr(time1,1,8);
    fTime VARCHAR2(8) := substr(time2,1,8);
    sBits PLS_INTEGER := GetHour(sTime) * 12 + Trunc(GetMinutes(sTime) / 5);
    sByte PLS_INTEGER := Trunc(sBits / 4);
    sBit PLS_INTEGER := BitAnd(sBits, 3);

    -- Change 00:00:00 to 24:00:00
    fBits PLS_INTEGER := CASE WHEN time2='00:00:00' THEN 24*12 ELSE GetHour(fTime) * 12 + Trunc(GetMinutes(fTime) / 5) END;
    fByte PLS_INTEGER := Trunc(fBits / 4);
    fBit PLS_INTEGER := BitAnd(fBits, 3);
BEGIN
    --START TIME breakdown
    IF sByte > 0 THEN
        retVal := SubStr(inRaw, 1, sByte);
    END IF;

    IF sByte = fByte THEN
        --Point OF START TIME encoding
        --Point OF Finish TIME encoding
        retVal := retVal || BitsHexToAscii(Bit_Or(BitsAsciiHex(Ascii(SubStr(inRaw, sByte+1, 1))+1), BitAnd(sTimeBits(sBit+1), fTimeBits(fBit+1)))+1);
    ELSE
        --Point OF START TIME encoding
        retVal := retVal || BitsHexToAscii(Bit_Or(BitsAsciiHex(Ascii(SubStr(inRaw, sByte+1, 1))+1), sTimeBits(sBit+1))+1);

        --BODY OF duration encoding
        IF fByte < 72 THEN
            IF sByte+1 < fByte THEN
                retVal := retVal || Lpad('F', fByte-sByte-1, 'F');
            END IF;
        ELSIF sByte+1 < 73 THEN
            retVal := retVal || Lpad('F', 73-sByte-1, 'F');
        END IF;

        --Point OF Finish TIME encoding
        IF fByte < 72 THEN
            retVal := retVal || BitsHexToAscii(Bit_Or(BitsAsciiHex(Ascii(SubStr(inRaw, fByte+1, 1))+1), fTimeBits(fBit+1))+1);
        END IF;
    END IF;

    IF fByte < 71 THEN
        retVal := retVal || SubStr(inRaw, fByte+2);
    END IF;

    inDay := retVal;
END;

PROCEDURE clearStartDayStruct(skip PLS_INTEGER, inDay IN OUT NOCOPY RAW) AS
   inRaw VARCHAR(72) := inDay;
   retVal VARCHAR2(72);

   --Finish TIME breakdown
   fBits PLS_INTEGER := getFirstBitSkip(inDAY, Trunc(skip / 5));
   fByte PLS_INTEGER := Trunc(fBits / 4);
   fBit PLS_INTEGER := BitAnd(fBits, 3);
BEGIN
    -- Point of finish time encoding
	IF fByte <72 THEN
    retVal := retVal || BitsHexToAscii(BitAnd(BitsAsciiHex(Ascii(SubStr(inRaw, fByte+1, 1))+1), sTimeBits(fBit+1))+1);
	END IF;

    IF fByte < 71 THEN
        retVal := retVal || SubStr(inRaw, fByte+2);
    END IF;

	IF retVal IS NULL THEN
		retVal := '0';
	END IF;

    inDay := LPad(retVal, 72, '0');
END;

PROCEDURE clearFinishDayStruct(skip PLS_INTEGER, inDay IN OUT NOCOPY RAW) AS
   inRaw VARCHAR(72) := inDay;
   retVal VARCHAR2(72);

   --START TIME breakdown
   sBits PLS_INTEGER := getFirstBitSkip(inDay, Trunc(skip / 5));
   sByte PLS_INTEGER := Trunc(sBits / 4);
   sBit PLS_INTEGER := BitAnd(sBits, 3);

BEGIN
    IF sByte=72 THEN
       return;
    END IF;
    IF sByte > 0 THEN
        retVal := SubStr(inRaw, 1, sByte);
    END IF;

    -- clear the start bits
    retVal := retVal || BitsHexToAscii(BitAnd(BitsAsciiHex(Ascii(SubStr(inRaw, sByte+1, 1))+1), fTimeBits(sBit+1))+1);

    inDay := RPad(retVal, 72, '0');
END;

PROCEDURE cleardayStruct(time1 VARCHAR2, time2 VARCHAR2, inDay IN OUT NOCOPY RAW) AS
    inRaw VARCHAR(72) := inDay;
    retVal VARCHAR2(72);

    sTime VARCHAR2(8) := SubStr(time1,1,8);
    fTime VARCHAR2(8) := SubStr(time2,1,8);

    --START TIME breakdown
    sBits PLS_INTEGER := GetHour(sTime) * 12 + Trunc(GetMinutes(sTime) / 5);
    sByte PLS_INTEGER := Trunc(sBits / 4);
    sBit PLS_INTEGER := BitAnd(sBits, 3);

    --FINISH TIME breakdown
    -- Change 00:00:00 to 24:00:00
    fBits PLS_INTEGER := CASE WHEN time2='00:00:00' THEN 24*12 ELSE GetHour(fTime) * 12 + Trunc(GetMinutes(fTime) / 5) END;
    fByte PLS_INTEGER := Trunc(fBits / 4);
    fBit PLS_INTEGER := BitAnd(fBits, 3);
BEGIN
    IF sByte > 0 THEN
        retVal := SubStr(inRaw, 1, sByte);
    END IF;

    IF sByte = fByte THEN
        --Point OF START TIME encoding
        --Point OF Finish TIME encoding
        retVal := retVal || BitsHexToAscii(BitAnd(BitsAsciiHex(Ascii(SubStr(inRaw, sByte+1, 1))+1), Bit_Or(fTimeBits(sBit+1), sTimeBits(fBit+1)))+1);
    ELSE
        --Point OF START TIME encoding
        retVal := retVal || BitsHexToAscii(BitAnd(BitsAsciiHex(Ascii(SubStr(inRaw, sByte+1, 1))+1), fTimeBits(sBit+1))+1);

        --BODY OF duration encoding
        IF fByte < 72 THEN
            IF sByte+1 < fByte THEN
                retVal := retVal || Lpad('0', fByte-sByte-1, '0');
            END IF;

            --Point OF Finish TIME encoding
            retVal := retVal || BitsHexToAscii(BitAnd(BitsAsciiHex(Ascii(SubStr(inRaw, fByte+1, 1))+1), sTimeBits(fBit+1))+1);
        ELSIF sByte+1 < 73 THEN
				retVal := retVal || Lpad('0', 72-sByte-1, '0');
        END IF;
    END IF;

    IF fByte < 71 THEN
        retVal := retVal || SubStr(inRaw, fByte+2);
    END IF;

    inDay := retVal;
END;

FUNCTION PRINT_TIMES(inDay RAW) RETURN VARCHAR2 AS
    shiftInProgress BOOLEAN := FALSE;
    hadShift BOOLEAN := FALSE;
    times VARCHAR2(1600) := '';  -- max len = 288/2 * 11 - 1 = 1583 characters
    inRaw VARCHAR2(72) := inDay;
    byte PLS_INTEGER;

    i PLS_INTEGER := 0;
    j PLS_INTEGER := 0;
    bitCount PLS_INTEGER := 0;
BEGIN
    IF inDay IS NOT NULL THEN
        FOR i IN 1..72 LOOP
            byte := BitsAsciiHex(Ascii(SubStr(inRaw, i, 1))+1);

            IF byte > 0 THEN
                FOR j IN 0..3 LOOP
                    bitCount := bitCount + 1;

                    IF BitAnd(byte, 8) <> 0 THEN
                        IF NOT shiftInProgress THEN
                            IF hadShift THEN
                                times := times || ',';
                            END IF;

                            times := times || Lpad(Trunc((bitCount-1)/12), 2, '0') || ':' || Lpad(Mod(bitCount-1, 12)*5, 2, '0') || ',';
                            shiftInProgress := TRUE;
                        END IF;
                    ELSE
                        IF shiftInProgress THEN
                            hadShift := TRUE;
                            times := times || Lpad(Trunc((bitCount-1)/12), 2, '0') || ':' || Lpad(Mod(bitCount-1, 12)*5, 2, '0');
                            shiftInProgress := FALSE;
                        END IF;
                    END IF;

                    byte := byte * 2;
                END LOOP;
            ELSE
                IF shiftInProgress THEN
                    times := times || Lpad(Trunc(bitCount/12), 2, '0') || ':' || Lpad(Mod(bitCount, 12)*5, 2, '0');
                    hadShift := TRUE;
                    shiftInProgress := FALSE;
                END IF;

                bitCount := bitCount + 4;
            END IF;
        END LOOP;

        IF shiftInProgress THEN
            -- October 23, 2003, Vladimir Schneider
            -- Unless the client chops up the times string and sends the times back to the server
            -- this should be left as 24;00 as it is in DB2
            times := times || '24:00';
        END IF;
    END IF;

    RETURN times;
END;

FUNCTION BIT_AND_DAY(day1 IN RAW, day2 IN RAW) RETURN RAW AS
BEGIN
  return UTL_RAW.BIT_AND(day1, day2);
END;

FUNCTION BIT_ANDNOT_DAY(day1 IN RAW, day2 IN RAW) RETURN RAW AS
BEGIN
  RETURN UTL_RAW.BIT_AND(day1, UTL_RAW.BIT_COMPLEMENT(day2));
END;

FUNCTION BIT_OR_DAY(day1 IN RAW, day2 IN RAW) RETURN RAW AS
BEGIN
  RETURN UTL_RAW.Bit_Or(day1, day2);
END;

FUNCTION BIT_EXOR_DAY(day1 IN RAW, day2 IN RAW) RETURN RAW AS
BEGIN
    RETURN UTL_RAW.Bit_Xor(day1, day2);
END;

FUNCTION getNumberOfBits(in_raw IN RAW) RETURN PLS_INTEGER AS
  i PLS_INTEGER;
  inRaw VARCHAR2(72);
  bits PLS_INTEGER;
  nLen PLS_INTEGER;
BEGIN
  inRaw := in_raw;
  bits := 0;
  nLen := Length(inRaw);
  FOR i IN 1 .. nLen LOOP
     bits := bits + CountBitsAsciiHex(Ascii(SubStr(inRaw, i, 1)) + 1);
  END LOOP;

  RETURN bits;
END;

FUNCTION BIT_OR(val PLS_INTEGER, mask PLS_INTEGER) RETURN PLS_INTEGER AS
BEGIN
   RETURN (val - BitAnd(val, mask)) + mask;
END;

FUNCTION BIT_ANDNOT(val PLS_INTEGER, mask PLS_INTEGER) RETURN PLS_INTEGER AS
BEGIN
   RETURN (BitAnd(val, 1-mask));
END;

FUNCTION TOTAL_DAY(day1 IN RAW) RETURN PLS_INTEGER AS
BEGIN
   RETURN getNumberOfBits(day1) * 5;
END;

FUNCTION TOTAL_DAY_2(day1 IN RAW, day2 IN RAW) RETURN PLS_INTEGER AS
BEGIN
   RETURN getNumberOfBits(UTL_RAW.BIT_AND(day1, day2)) * 5;
END;

FUNCTION FIRST_AVAIL_TIME(inDay IN RAW) RETURN VARCHAR2 AS
   wBits PLS_INTEGER;
   wHour PLS_INTEGER;
   wMin PLS_INTEGER;
BEGIN
   wBits := getFirstBit(inDay);
   wHour := Trunc(wBits / 12);
   wMin := Mod(wBits, 12) * 5;
   RETURN LPad(wHour, 2,'0') || ':' || LPad(wMin, 2, '0') || ':00';
END;

FUNCTION TOBASE64(num IN PLS_INTEGER, buf IN VARCHAR2) RETURN VARCHAR2 AS
   DivNum PLS_INTEGER;
   ModNum PLS_INTEGER;
   buf2 VARCHAR2(200);
BEGIN
   buf2 := buf;

   IF (num > 16777215) THEN
      buf2 := 'qqqq';
      RETURN buf2;
   END IF;

   DivNum := num;

   WHILE DivNum > 64 LOOP
      ModNum := Mod(DivNum, 64);

      IF ModNum > 46 THEN
         buf2 := Chr(ModNum + 50) || buf2;
      ELSE
         buf2 := Chr(ModNum + 48) || buf2;
      END IF;

      DivNum := Trunc(DivNum / 64);
   END LOOP;

   IF DivNum >= 47 THEN
      DivNum := DivNum + 2;
   END IF;
   buf2 := Chr(DivNum + 48) || buf2;

   SELECT Lpad(buf2, 4, '0') INTO buf2 FROM dual;
   RETURN buf2;
END;

FUNCTION LAST_AVAIL_TIME(inDay IN RAW) RETURN VARCHAR2 AS
   wBits PLS_INTEGER;
   wHour PLS_INTEGER;
   wMin PLS_INTEGER;
BEGIN
   wBits := getLastBit(inDay);
   wHour := Trunc(wBits / 12);
   wMin := Mod(wBits, 12) * 5;
   RETURN LPad(wHour, 2,'0') || ':' || LPad(wMin, 2, '0') || ':00';
END;

FUNCTION TIME_BEFORE_SCHED(sched PLS_INTEGER, in_day RAW) RETURN VARCHAR2 AS
   wBits PLS_INTEGER;
   wHour PLS_INTEGER;
   wMin PLS_INTEGER;
BEGIN
   wBits := getLastBitSkip(in_day, Trunc(sched / 5));
   wHour := Trunc(wBits / 12);
   wMin := Mod(wBits, 12) * 5;
   RETURN LPad(wHour, 2,'0') || ':' || LPad(wMin, 2, '0') || ':00';
END;

FUNCTION TIME_AFTER_SCHED(sched PLS_INTEGER, in_day RAW) RETURN VARCHAR2 AS
   wBits PLS_INTEGER;
   wHour PLS_INTEGER;
   wMin PLS_INTEGER;
BEGIN
   wBits := getFirstBitSkip(in_day, Trunc(sched / 5));
   wHour := Trunc(wBits / 12);
   wMin := Mod(wBits, 12) * 5;
   RETURN LPad(wHour, 2,'0') || ':' || LPad(wMin, 2, '0') || ':00';
END;

FUNCTION PACKDEPENDENCIES(id IN VARCHAR2) RETURN LONG AS
   h_dependency_type VARCHAR2(30);
   h_element_id VARCHAR2(50);
   h_element_name VARCHAR2(400);
   h_id VARCHAR2(40);
   h_project_id VARCHAR2(40);
   h_project_name VARCHAR2(400);
   h_resultStr LONG;
   h_str_type_id VARCHAR2(20);

   CURSOR C_DEPEND IS
      SELECT Rtrim(To_Char(DEP.DEP_TYPE)) AS DEP_TYPE
            ,DEP.PRED_ID AS ELEMENT_ID
            ,Rtrim(To_Char(DEP.PRED_OBJECT_TYPE)) AS TYPE_ID
            ,Rtrim(TW.ELEMENT_NAME) AS ELEMENT_NAME
            ,DEP.PRED_PROJECT_ID AS PROJECT_ID
            ,(SELECT Rtrim(ELEMENT_NAME) FROM TMT_WBS WHERE ELEMENT_ID = DEP.PRED_PROJECT_ID) AS PROJECT_NAME
        FROM DEPENDENCIES DEP, TMT_WBS TW
       WHERE TW.ELEMENT_ID = DEP.PRED_ID AND DEP.SUCC_ID = h_id AND DEP.REC_STATUS <> 'K';

BEGIN

   h_id := id;

   h_resultStr := '{RS}';

   OPEN C_DEPEND;

   LOOP

      FETCH C_DEPEND
         INTO h_dependency_type, h_element_id, h_str_type_id, h_element_name, h_project_id, h_project_name;

      IF C_DEPEND%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
         EXIT;
      END IF;

      h_resultStr := h_resultStr || '{DS}';
      h_resultStr := h_resultStr || 'DEP_TYPE=';
      h_resultStr := h_resultStr || h_dependency_type;
      h_resultStr := h_resultStr || ';ELEMENT_ID=';
      h_resultStr := h_resultStr || h_element_id;
      h_resultStr := h_resultStr || ';TYPE_ID=';
      h_resultStr := h_resultStr || h_str_type_id;
      h_resultStr := h_resultStr || ';ELEMENT_NAME=';
      h_resultStr := h_resultStr || h_element_name;
      h_resultStr := h_resultStr || ';PROJECT_ID=';
      h_resultStr := h_resultStr || h_project_id;
      h_resultStr := h_resultStr || ';PROJECT_NAME=';
      h_resultStr := h_resultStr || h_project_name;
      h_resultStr := h_resultStr || ';';
      h_resultStr := h_resultStr || '{DF}';
   END LOOP;

   BEGIN
      CLOSE C_DEPEND;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
         PMO_DB2M.SC_STATE := SQLCODE;
   END;

   h_resultStr := h_resultStr || '{RF}';

   RETURN h_resultStr;
END;

FUNCTION getNumberOfMinutesStartingAt(inday RAW, start_time VARCHAR2) RETURN PLS_INTEGER AS
  startTimeStr VARCHAR2(8) := SubStr(start_time, 1, 8);
  sBits PLS_INTEGER := GetHour(startTimeStr) * 12 + Trunc(GetMinutes(startTimeStr) / 5);
BEGIN
   RETURN getNumberOfMinutes(inDay, sBits, 36*8) * 5;
END;

FUNCTION getNumberOfMinutesFinishingAt(inDay RAW, start_time VARCHAR2) RETURN PLS_INTEGER AS
  finishTimeStr VARCHAR2(8) := SubStr(start_time, 1, 8);
  fBits PLS_INTEGER := GetHour(finishTimeStr) * 12 + Trunc(GetMinutes(finishTimeStr) / 5);
BEGIN
   RETURN getNumberOfMinutes(inDay, 0, fBits) * 5;
END;

FUNCTION getNumberOfMinutesBetweenSAndF(inDay RAW, start_time VARCHAR2, finish_time VARCHAR2)
  RETURN PLS_INTEGER AS
  startTimeStr VARCHAR2(8) := SubStr(start_time, 1, 8);
  sBits PLS_INTEGER := GetHour(startTimeStr) * 12 + Trunc(GetMinutes(startTimeStr) / 5);
  finishTimeStr VARCHAR2(8) := SubStr(finish_time, 1, 8);
  fBits PLS_INTEGER := GetHour(finishTimeStr) * 12 + Trunc((GetMinutes(finishTimeStr) / 5) + 0.9);
BEGIN
  RETURN getNumberOfMinutes(inDay, sBits, fBits) * 5;
END;

FUNCTION getNumberOfMinutes(inDay RAW, sBits PLS_INTEGER, fBits PLS_INTEGER)
RETURN PLS_INTEGER AS
   startIndex PLS_INTEGER := Trunc(sBits / 4);
   finishIndex PLS_INTEGER := Trunc(fBits / 4);
   startMask PLS_INTEGER := sTimeBits(BitAnd(sBits, 3)+1);
   finishMask PLS_INTEGER := fTimeBits(BitAnd(fBits, 3)+1);
   wBits PLS_INTEGER := 0;
   inRaw VARCHAR2(72) := inDay;
   i PLS_INTEGER;
BEGIN
    IF inDay IS NOT NULL THEN
       IF startIndex = finishIndex THEN
           wBits := CountBitsNibble(BitAnd(BitAnd(BitsAsciiHex(Ascii(SubStr(inRaw, startIndex+1, 1)) + 1), startMask), finishMask) + 1);
       ELSE
           wBits := CountBitsNibble(BitAnd(BitsAsciiHex(Ascii(SubStr(inRaw, startIndex+1, 1)) + 1), startMask) + 1);

           FOR i IN startIndex+1..finishIndex-1 LOOP
               wBits := wBits + CountBitsAsciiHex(Ascii(SubStr(inRaw, i+1, 1)) + 1);
           END LOOP;

           IF finishIndex < 72 THEN
               wBits := wBits + CountBitsNibble(BitAnd(BitsAsciiHex(Ascii(SubStr(inRaw, finishIndex+1, 1)) + 1), finishMask) + 1);
           END IF;
       END IF;
    END IF;

    RETURN wBits;
END;

FUNCTION GET_FINISH_TIME(inDay RAW, inEffort PLS_INTEGER) RETURN VARCHAR2 AS
    aBits PLS_INTEGER;
    wBits PLS_INTEGER;
    wHour PLS_INTEGER;
    wMin PLS_INTEGER;
    effort PLS_INTEGER := inEffort;
    inRaw VARCHAR2(72) := inDay;
    bits PLS_INTEGER;
    i PLS_INTEGER;
BEGIN
    IF inDay IS NOT NULL AND effort >= 5 THEN
        wBits := 0;
        aBits := 36*8;

        effort := Trunc(effort / 5);

        FOR i IN 1..72 LOOP
            bits := BitsAsciiHex(Ascii(SubStr(inRaw, i, 1)) + 1);

            IF bits > 0 THEN
                wBits := wBits + CountBitsNibble(bits+1);

                IF wBits >= effort THEN
                    aBits := 4 * (i-1);
                    aBits := aBits + extraFirstBits(BitAnd(bits, 7)*4 + wBits - effort + 1)+1;
                    EXIT;
                END IF;
            END IF;
        END LOOP;

        wHour := Trunc(aBits / 12);
        wMin := (aBits Mod 12) * 5;

        RETURN Lpad(wHour, 2, '0') || ':' || Lpad(wMin, 2, '0') || ':00';
    ELSE
        RETURN '';
    END IF;
END;


END;
/


