-- CONNECT TO PMOFFICE USER AllTrace USING AllTrace;

-------------------------------

-- DDL Statements for Triggers

-------------------------------
CREATE OR REPLACE TRIGGER depend_d AFTER DELETE ON dependencies REFERENCING OLD AS o FOR EACH ROW
    DECLARE indPublished VARCHAR2(1);
BEGIN
    SELECT published INTO indPublished FROM tmt_wbs WHERE element_id = :o.succ_project_id;
    IF indPublished = '1' THEN
        INSERT INTO alerts (element_id,project_id, ALERT_ID, parent_table, rec_user, value_list, flag) VALUES
        (:o.succ_id,:o.succ_project_id, 'SCDEPENDENCYDETAILSCHANGED', 'TMT_WBS', :o.rec_user,
        (SELECT element_name FROM tmt_wbs WHERE element_id = :o.pred_id) ||'#!'||
        (SELECT element_name FROM tmt_wbs WHERE element_id = :o.pred_project_id)|| '#!' ||
        (SELECT element_name FROM tmt_wbs WHERE element_id = :o.succ_id) ||'#!'||
        (SELECT element_name FROM tmt_wbs WHERE element_id = :o.succ_project_id)||'#!deleted', 'I');
    END IF;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER depend_d SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER depend_i AFTER INSERT ON dependencies REFERENCING NEW AS n FOR EACH ROW
    DECLARE indPublished VARCHAR2(1);
BEGIN
    SELECT published INTO indPublished FROM tmt_wbs WHERE element_id = :n.succ_project_id;
    IF indPublished = '1' THEN
        INSERT INTO alerts (element_id,project_id, ALERT_ID, parent_table, rec_user, value_list, flag) VALUES
        (:n.succ_id,:n.succ_project_id, CASE WHEN :n.pred_project_id = :n.succ_project_id THEN 'SCNEWDEPENDENCY' ELSE
        'SCNEWCROSSPROJECTDEPENDENCY' END, 'TMT_WBS', :n.rec_user, CASE WHEN :n.pred_project_id = :n.succ_project_id THEN
        (SELECT element_name FROM tmt_wbs WHERE element_id = :n.pred_id) ||'#!' ||
        (SELECT element_name FROM tmt_wbs WHERE element_id = :n.succ_id) ||'#!' ELSE
        (SELECT element_name FROM tmt_wbs WHERE element_id = :n.succ_project_id) ||'#!' END ||
        (SELECT element_name FROM tmt_wbs WHERE element_id = :n.pred_project_id), 'I');
    END IF;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER depend_i SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER depend_u AFTER UPDATE OF DEP_TYPE, LEAD_OR_LAG_TYPE, AMOUNT, LAG_DISPLAY_UNIT
ON dependencies  REFERENCING OLD AS o NEW AS n FOR EACH ROW
    DECLARE indPublished VARCHAR2(1);
BEGIN
    SELECT published INTO indPublished FROM tmt_wbs WHERE element_id = :n.succ_project_id;
    IF indPublished = '1' THEN
        INSERT INTO alerts (element_id,project_id, ALERT_ID, parent_table, rec_user, value_list, flag) VALUES
        (:n.succ_id,:n.succ_project_id, 'SCDEPENDENCYDETAILSCHANGED', 'TMT_WBS', :n.rec_user,
        (SELECT element_name FROM tmt_wbs WHERE element_id = :n.pred_id) ||'#!'||
        (SELECT element_name FROM tmt_wbs WHERE element_id = :n.pred_project_id)|| '#!' ||
        (SELECT element_name FROM tmt_wbs WHERE element_id = :n.succ_id)||'#!'||
        (SELECT element_name FROM tmt_wbs WHERE element_id = :n.succ_project_id) || '#!changed', 'I');
    END IF;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER depend_u SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER mailgroup_d AFTER DELETE ON tmt_contacts REFERENCING OLD
AS o FOR EACH ROW WHEN (o.parent_id IS NOT NULL and o.type_id = 13)
    DECLARE PRAGMA AUTONOMOUS_TRANSACTION;
    alertCount PLS_INTEGER;
BEGIN

    alertCount := PMO_Triggers.alertsRows.Count+1;
    PMO_Triggers.alertsRows(alertCount).element_id    := :o.element_id;
    PMO_Triggers.alertsRows(alertCount).RESOURCE_ID   := :o.element_id;
    PMO_Triggers.alertsRows(alertCount).ALERT_ID      := 'SCMAILGROUPMEMBERREMOVE';
    PMO_Triggers.alertsRows(alertCount).parent_table  := 'RESOURCES';
    PMO_Triggers.alertsRows(alertCount).rec_user      := :o.rec_user;
    SELECT element_name INTO PMO_Triggers.alertsRows(alertCount).value_list FROM tmt_contacts WHERE element_id = :o.parent_id;
    PMO_Triggers.alertsRows(alertCount).flag := 'I';
    PMO_Triggers.alertsRows(alertCount).REC_DATETIME := SYSTIMESTAMP;

EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER mailgroup_d SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER mailgroup_i AFTER INSERT ON tmt_contacts REFERENCING NEW AS n FOR EACH ROW
WHEN (n.parent_id IS NOT NULL AND n.element_id <> n.rec_user AND n.type_id = 13)
    DECLARE PRAGMA AUTONOMOUS_TRANSACTION;
    alertCount PLS_INTEGER;
BEGIN
--  INSERT INTO alerts (element_id, ALERT_ID, parent_table, rec_user, value_list, flag) VALUES
--  (n.element_id, 'SCMAILGROUPINVITATION', 'RESOURCES', n.rec_user,
    alertCount := PMO_Triggers.alertsRows.Count+1;
    PMO_Triggers.alertsRows(alertCount).element_id    := :n.element_id;
    PMO_Triggers.alertsRows(alertCount).RESOURCE_ID   := :n.element_id;
    PMO_Triggers.alertsRows(alertCount).ALERT_ID      := 'SCMAILGROUPMEMBERADD';
    PMO_Triggers.alertsRows(alertCount).parent_table  := 'RESOURCES';
    PMO_Triggers.alertsRows(alertCount).rec_user      := :n.rec_user;
    SELECT element_name INTO PMO_Triggers.alertsRows(alertCount).value_list FROM tmt_contacts WHERE element_id = :n.parent_id;
    PMO_Triggers.alertsRows(alertCount).flag := 'I';
    PMO_Triggers.alertsRows(alertCount).REC_DATETIME := SYSTIMESTAMP;

EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER mailgroup_i SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER mailgroup_stmt
AFTER DELETE OR INSERT
ON tmt_contacts
DECLARE
BEGIN

    IF PMO_Triggers.alertsRows.Count > 0 THEN
        FORALL elementID IN PMO_Triggers.alertsRows.FIRST..PMO_Triggers.alertsRows.LAST
          INSERT INTO alerts
              VALUES PMO_Triggers.alertsRows(elementID);

        PMO_Triggers.alertsRows.DELETE;
    END IF;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER mailgroup_stmt SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER resource_i AFTER INSERT ON history_resources REFERENCING
      NEW AS n FOR EACH ROW
BEGIN
    INSERT INTO alerts (element_id,RESOURCE_ID, ALERT_ID, parent_table, rec_user, value_list, flag)
    (SELECT p.manager_id,p.manager_id, 'SCRESOURCERECORDREAPPROVAL', 'RESOURCES', :n.rec_user, :n.full_name, 'I'
    FROM tmt_pools c, tmt_pools p WHERE c.element_id = :n.resource_id
    AND p.element_id = c.parent_id AND p.manager_id <> :n.rec_user);
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER resource_i SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER res_assign_d
AFTER DELETE ON resource_assignments
REFERENCING OLD AS o FOR EACH ROW
    DECLARE indPublished VARCHAR2(1);
			indExistence PLS_INTEGER;
BEGIN
    SELECT 1 INTO indExistence FROM TMT_WBS
    WHERE ELEMENT_ID =:O.ELEMENT_ID;
    IF SQLCODE=0 THEN
		SELECT published INTO indPublished FROM tmt_wbs WHERE element_id = :o.project_id;
		IF indPublished = '1' THEN
			INSERT INTO alerts (element_id,project_id, ALERT_ID, parent_table, rec_user, value_list, flag) VALUES
			(:o.element_id,:o.project_id, 'SCRESOURCEASSIGNMENTCHANGEREMOVE', 'TMT_WBS', :o.rec_user,
			(SELECT w.element_name FROM tmt_wbs w WHERE w.element_id = :o.element_id )
			||'#!'|| (SELECT element_name FROM tmt_wbs WHERE element_id =:o.project_id), 'I');
		END IF;
	END IF;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER res_assign_d SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER res_assign_i
AFTER INSERT ON resource_assignments
REFERENCING NEW AS n FOR EACH ROW
    DECLARE indPublished VARCHAR2(1);
			indExistence PLS_INTEGER;
BEGIN
    SELECT 1 INTO indExistence FROM TMT_WBS
    WHERE ELEMENT_ID =:n.ELEMENT_ID;
    IF SQLCODE=0 THEN
		SELECT published INTO indPublished FROM tmt_wbs WHERE element_id = :n.project_id;
		IF indPublished = '1' THEN
			INSERT INTO alerts (element_id,project_id, ALERT_ID, parent_table, rec_user, value_list, flag) VALUES
			(:n.element_id,:n.project_id, 'SCRESOURCEASSIGNMENTCHANGEADD', 'TMT_WBS', :n.rec_user,
			(SELECT w.element_name FROM tmt_wbs w WHERE w.element_id = :n.element_id )
			||'#!'|| (SELECT element_name FROM tmt_wbs WHERE element_id =:n.project_id), 'I');
		END IF;
	END IF;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER res_assign_i SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER tskassign_u_sched
AFTER UPDATE OF sched_start ON task_assignments
REFERENCING OLD AS o NEW AS n FOR EACH ROW
WHEN (o.sched_start IS NULL AND n.sched_start IS NOT NULL )
    DECLARE indPublished VARCHAR2(1);
BEGIN
    SELECT published INTO indPublished FROM tmt_wbs WHERE element_id = :n.project_id;
    IF indPublished = '1' THEN
        INSERT INTO alerts (element_id,project_id, ALERT_ID, parent_table,rec_user, value_list, flag,resource_id) VALUES
        (:n.task_id,:n.project_id, 'SCSCHEDULEASSIGNMENTADD', 'TMT_WBS',:n.rec_user,
        (select full_name from resources where resource_id = :n.resource_id) ||'#!'||
        (SELECT element_name FROM tmt_wbs WHERE element_id = :n.task_id) ||'#!'||
        (SELECT element_name FROM tmt_wbs WHERE element_id = :n.project_id) , 'I',:n.resource_id);
    END IF;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER tskassign_u_sched SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER tskassign_d_sched
AFTER DELETE ON task_assignments
REFERENCING OLD AS o FOR EACH ROW
WHEN (o.sched_start IS NOT NULL )
    DECLARE indPublished VARCHAR2(1);
			indExistence PLS_INTEGER;
BEGIN
    SELECT 1 INTO indExistence FROM TMT_WBS
    WHERE ELEMENT_ID =:o.TASK_ID;
    IF SQLCODE=0 THEN

		SELECT published INTO indPublished FROM tmt_wbs WHERE element_id = :o.project_id;
		IF indPublished = '1' THEN
			INSERT INTO alerts (element_id,project_id, ALERT_ID, parent_table,rec_user, value_list, flag,resource_id) VALUES
			(:o.task_id,:o.project_id,'SCSCHEDULEASSIGNMENTREMOVE', 'TMT_WBS', :o.rec_user,
	                (select full_name from resources where resource_id = :o.resource_id) ||'#!'||			
			(SELECT element_name FROM tmt_wbs WHERE element_id = :o.task_id) ||'#!'||
			(SELECT element_name FROM tmt_wbs WHERE element_id = :o.project_id), 'I',:o.resource_id);
		END IF;
	END IF;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER tskassign_d_sched SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

--CREATE OR REPLACE TRIGGER wbs_u_complete
--AFTER UPDATE OF actual_finish ON tmt_wbs
--REFERENCING OLD AS o NEW AS n FOR EACH ROW
--WHEN (o.actual_finish IS NULL AND n.actual_finish IS NOT NULL)
--    DECLARE
--        PRAGMA AUTONOMOUS_TRANSACTION;
--        indPublished VARCHAR2(1);
--        alertCount PLS_INTEGER;
--        elementName VARCHAR(4000);
--BEGIN
--    SELECT published INTO indPublished FROM tmt_wbs WHERE element_id = :n.project_id;
--    IF indPublished = '1' THEN
--        alertCount := PMO_Triggers.alertsRows.Count+1;
--        PMO_Triggers.alertsRows(alertCount).element_id    := :o.element_id;
--        PMO_Triggers.alertsRows(alertCount).ALERT_ID      := 'SCTASKCOMPLETION';
--        PMO_Triggers.alertsRows(alertCount).parent_table  := 'TMT_WBS';
--        PMO_Triggers.alertsRows(alertCount).rec_user      := :n.rec_user;
--
--        SELECT element_name INTO elementName FROM tmt_wbs WHERE element_id = :n.project_id;
--        PMO_Triggers.alertsRows(alertCount).value_list    := :n.element_name || '#!' || elementName;
--
--        PMO_Triggers.alertsRows(alertCount).flag          := 'I';
--        PMO_Triggers.alertsRows(alertCount).REC_DATETIME := SYSTIMESTAMP;
--    END IF;
--EXCEPTION
--    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
--        lfile.fput( 'AllTrace', 'TRIGGER wbs_u_complete SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
--END;
--/

CREATE OR REPLACE TRIGGER wbs_u_cost
AFTER UPDATE OF CURRENT_BUDGET ON tmt_wbs
REFERENCING OLD AS o NEW AS n FOR EACH ROW
WHEN (n.BASELINE1_BUDGET <> 0
        AND (n.CURRENT_BUDGET-n.BASELINE1_BUDGET) <> (o.CURRENT_BUDGET-o.BASELINE1_BUDGET))
DECLARE
    PRAGMA AUTONOMOUS_TRANSACTION;
    indPublished VARCHAR2(1);
    alertCount PLS_INTEGER;
    elementName VARCHAR(4000);
BEGIN
    SELECT published INTO indPublished FROM tmt_wbs WHERE element_id = :n.project_id;
    IF indPublished = '1' THEN
        alertCount := PMO_Triggers.alertsRows.Count+1;

        PMO_Triggers.alertsRows(alertCount).element_id   :=  :o.element_id;
        PMO_Triggers.alertsRows(alertCount).project_id   :=  :n.project_id;        
        PMO_Triggers.alertsRows(alertCount).ALERT_ID     :=  'SCCOSTVARIANCE';
        PMO_Triggers.alertsRows(alertCount).parent_table :=  'TMT_WBS';
        PMO_Triggers.alertsRows(alertCount).rec_user     :=  :n.rec_user;

        SELECT element_name INTO elementName FROM tmt_wbs WHERE element_id = :n.project_id;
        PMO_Triggers.alertsRows(alertCount).value_list   :=  :n.element_name
                                                                    || '#!' || elementName
                                                                    || '#!' || To_Char(:o.CURRENT_BUDGET)
                                                                    || '#!' || To_Char(:o.BASELINE1_BUDGET)
                                                                    || '#!' || To_Char(:n.CURRENT_BUDGET)
                                                                    || '#!' || To_Char(:n.BASELINE1_BUDGET);

        PMO_Triggers.alertsRows(alertCount).flag         :=  'I';
        PMO_Triggers.alertsRows(alertCount).VARIANCE     :=  Floor((:n.CURRENT_BUDGET-:n.BASELINE1_BUDGET)/:n.BASELINE1_BUDGET*100);
        PMO_Triggers.alertsRows(alertCount).REC_DATETIME := SYSTIMESTAMP;
    END IF;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER wbs_u_cost SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER wbs_u_published
AFTER UPDATE OF published ON tmt_wbs
REFERENCING OLD AS o  NEW AS n FOR EACH ROW
WHEN (o.published <> n.published AND n.published = '1')
BEGIN
    INSERT INTO alerts (element_id,project_id, ALERT_id, parent_table, rec_user, value_list, flag)
    VALUES (:o.element_id,:o.project_id, 'SCNEWPROJECTPUBLISHED', 'TMT_WBS', :n.rec_user, :n.element_name, 'I');
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER wbs_u_published SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER wbs_u_schedule
AFTER UPDATE OF BASELINE1_VARIANCE ON tmt_wbs
REFERENCING OLD AS o NEW AS n FOR EACH ROW
WHEN (n.BASELINE1_VARIANCE <> o.BASELINE1_VARIANCE AND n.BASELINE1_DURATION > 0)
DECLARE
    PRAGMA AUTONOMOUS_TRANSACTION;
    indPublished VARCHAR2(1);
    alertCount PLS_INTEGER;
    baseline1Variance NUMBER;
    baseline1VarianceDays PLS_INTEGER;
    baseline1VarianceHours PLS_INTEGER;
    baseline1VarianceMinutes PLS_INTEGER;
    baseline1VariancePercent PLS_INTEGER;
    elementName VARCHAR2(4000);
BEGIN
    SELECT published INTO indPublished FROM tmt_wbs WHERE element_id = :n.project_id;
    IF indPublished = '1' THEN
        alertCount := PMO_Triggers.alertsRows.Count+1;

        PMO_Triggers.alertsRows(alertCount).element_id     := :o.element_id;
        PMO_Triggers.alertsRows(alertCount).project_id     := :o.project_id;        
        PMO_Triggers.alertsRows(alertCount).ALERT_ID       := 'SCSCHEDULEVARIANCE';
        PMO_Triggers.alertsRows(alertCount).parent_table   := 'TMT_WBS';
        PMO_Triggers.alertsRows(alertCount).rec_user       := :n.rec_user;

        baseline1Variance := Floor(:n.BASELINE1_VARIANCE);
        baseline1VarianceDays := Floor(baseline1Variance/:n.MINUTE_PER_DAY);
        baseline1Variance := baseline1Variance - baseline1VarianceDays*:n.MINUTE_PER_DAY;

        baseline1VarianceHours := Floor(baseline1Variance/60);
        baseline1Variance := baseline1Variance - baseline1VarianceHours*60;

        baseline1VarianceMinutes := baseline1Variance;

        baseline1VariancePercent := Floor(:n.BASELINE1_VARIANCE/:n.BASELINE1_DURATION*100);

        SELECT element_name INTO elementName FROM tmt_wbs WHERE element_id = :n.project_id;
        PMO_Triggers.alertsRows(alertCount).value_list     := :n.element_name
                         || '#!' || elementName
                         || '#!' || Coalesce(To_Date(SubStr(:n.SCHED_FINISH,1,10)), To_Date(SubStr(:n.PLANNED_FINISH,1,10)))
                         || '#!' || To_Date(SubStr(:n.BASELINE1_FINISH,1,10))
                         || '#!' || baseline1VarianceDays || ' Day(s), '
                                 || baseline1VarianceHours || ' Hour(s), '
                                 || baseline1VarianceMinutes || ' Minute(s)'
                         || '#!' || baseline1VariancePercent;
        PMO_Triggers.alertsRows(alertCount).flag           := 'I';
        PMO_Triggers.alertsRows(alertCount).VARIANCE := baseline1VariancePercent;
        PMO_Triggers.alertsRows(alertCount).REC_DATETIME := SYSTIMESTAMP;
    END IF;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER wbs_u_schedule SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER wbs_u_template AFTER UPDATE OF published ON tmt_wbs REFERENCING
OLD AS o NEW AS n FOR EACH ROW
WHEN ((o.t_rank IS NULL AND n.t_rank IS NOT NULL) OR o.t_rank <> n.t_rank)
DECLARE
    PRAGMA AUTONOMOUS_TRANSACTION;
    alertCount PLS_INTEGER;
    elementName VARCHAR2(4000);
BEGIN
    alertCount := PMO_Triggers.alertsRows.Count+1;

    PMO_Triggers.alertsRows(alertCount).element_id    := :n.element_id;
    PMO_Triggers.alertsRows(alertCount).project_id    := :n.project_id;    
    PMO_Triggers.alertsRows(alertCount).ALERT_ID      := 'SCNEWTEMPLATE';
    PMO_Triggers.alertsRows(alertCount).parent_table  := 'TMT_WBS';
    PMO_Triggers.alertsRows(alertCount).rec_user      := :n.rec_user;

    SELECT element_name INTO elementName FROM tmt_wbs WHERE element_id = :n.project_id;
    PMO_Triggers.alertsRows(alertCount).value_list    := :n.element_name || '#!' || elementName;

    PMO_Triggers.alertsRows(alertCount).flag          := 'I';
    PMO_Triggers.alertsRows(alertCount).REC_DATETIME  := SYSTIMESTAMP;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER wbs_u_template SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER WBS_U_STMT
AFTER UPDATE OF actual_finish, CURRENT_BUDGET, BASELINE1_VARIANCE, published ON tmt_wbs
DECLARE
BEGIN
    IF PMO_Triggers.alertsRows.Count > 0 THEN
        FORALL elementID IN PMO_Triggers.alertsRows.FIRST..PMO_Triggers.alertsRows.LAST
          INSERT INTO alerts
              VALUES PMO_Triggers.alertsRows(elementID);

        PMO_Triggers.alertsRows.DELETE;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'trigger', 'TRIGGER WBS_U_STMT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

--CREATE OR REPLACE TRIGGER wbs_i_scope_qual
--AFTER INSERT ON wbs_rtf
--REFERENCING NEW AS n
--FOR EACH ROW
--WHEN (n.bit_flag IN (1, 2) )
--    DECLARE indPublished VARCHAR2(1);
--BEGIN
--    SELECT CASE WHEN c.type_id = 1 THEN c.published ELSE p.published END INTO indPublished
--    FROM tmt_wbs c, tmt_wbs p WHERE c.element_id = :n.element_id AND c.project_id = p.element_id;
--    IF indPublished = '1' THEN
--        INSERT INTO alerts (element_id,project_id, ALERT_ID, parent_table, rec_user, value_list, flag)
--        SELECT :n.element_id,w.project_id, CASE WHEN :n.bit_flag = 1 THEN 'SCSCOPEVARIANCE' ELSE 'SCQUALITYVARIANCE' END,
--        'TMT_WBS', w.rec_user, w.element_name||'#!'
--        ||(SELECT t1.element_name FROM tmt_wbs t1 WHERE t1.element_id = w.project_id), 'I'
--        FROM tmt_wbs w WHERE element_id = :n.element_id;
--    END IF;
--    PMO_DB2M.SC_STATE := SQLCODE;
--EXCEPTION
--    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
--        lfile.fput( 'AllTrace', 'TRIGGER wbs_i_scope_qual SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
--END;
--/

--CREATE OR REPLACE TRIGGER wbs_u_scope_qual
--AFTER UPDATE ON wbs_rtf
--REFERENCING  OLD AS o  NEW AS n
--FOR EACH ROW
--WHEN (n.bit_flag IN (1, 2) AND ((o.clob_data IS NULL AND n.clob_data IS NOT NULL)
--    OR To_Char(o.clob_data) <> To_Char(n.clob_data)))
--    DECLARE indPublished VARCHAR2(1);
--BEGIN
--    SELECT CASE WHEN c.type_id = 1 THEN c.published ELSE p.published END INTO indPublished
--    FROM tmt_wbs c, tmt_wbs p WHERE c.element_id = :n.element_id AND c.project_id = p.element_id;
--    IF indPublished = '1' THEN
--        INSERT INTO alerts (element_id,project_id, ALERT_ID, parent_table, rec_user, value_list, flag)
--        SELECT :o.element_id,w.project_id, CASE WHEN :n.bit_flag = 1 THEN 'SCSCOPEVARIANCE' ELSE 'SCQUALITYVARIANCE' END,
--        'TMT_WBS', w.rec_user, w.element_name||'#!'||
--        (SELECT t1.element_name FROM tmt_wbs t1 WHERE t1.element_id = w.project_id), 'I'
--        FROM tmt_wbs w WHERE element_id = :n.element_id;
--    END IF;
--    PMO_DB2M.SC_STATE := SQLCODE;
--EXCEPTION
--    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
--        lfile.fput( 'AllTrace', 'TRIGGER wbs_u_scope_qual SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
--END;
--/

CREATE OR REPLACE TRIGGER FS_TA_SCHED_U AFTER  UPDATE OF SCHED_START, SCHED_FINISH,
SCHED_WORK ON TASK_ASSIGNMENTS  REFERENCING  OLD AS o  NEW AS n  FOR EACH ROW
WHEN ( (n.SCHED_WORK>0) AND ((o.SCHED_START<>n.SCHED_START) OR (o.SCHED_FINISH<>n.SCHED_FINISH) OR (o.SCHED_WORK<>n.SCHED_WORK )))
BEGIN
    UPDATE TMT_WBS SET FS_STATE = CASE WHEN FS_STATE IN (0,4,5) THEN 1 WHEN  FS_STATE=6 THEN 7 WHEN FS_STATE=2 THEN 3 ELSE FS_STATE END
    WHERE TYPE_ID IN (3,59, 236, 237, 238, 239, 240, 241, 260) AND ELEMENT_ID=:n.TASK_ID AND FS_LABOR_CODE IS NOT NULL AND FS_LABOR_CODE <> 'PMOFFICE' AND FS_STATE NOT IN (3, 8);
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER FS_TA_SCHED_U SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER FS_TC_ASSIGN_I AFTER  INSERT  ON TMT_TIMECODES  REFERENCING NEW AS n  FOR EACH ROW
WHEN ( n.TYPE_ID IN (3,59, 236, 237, 238, 239, 240, 241, 260) )
BEGIN
    UPDATE TMT_WBS SET FS_STATE = CASE WHEN FS_STATE IN (0,4,5) THEN 1 WHEN  FS_STATE=6 THEN 7 WHEN FS_STATE=2 THEN 3 ELSE FS_STATE END
    WHERE ELEMENT_ID=:n.ELEMENT_ID AND FS_LABOR_CODE IS NOT NULL AND FS_LABOR_CODE <> 'PMOFFICE' AND FS_STATE NOT IN (3, 8);
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER FS_TC_ASSIGN_I SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER FS_TC_UNASSIGN_D AFTER  DELETE  ON TMT_TIMECODES  REFERENCING OLD AS o  FOR EACH ROW
WHEN ( o.TYPE_ID IN (3,59, 236, 237, 238, 239, 240, 241, 260) )
BEGIN
    UPDATE TMT_WBS SET FS_STATE = CASE WHEN FS_STATE IN (0,4,5) THEN 1 WHEN  FS_STATE=6 THEN 7 WHEN FS_STATE=2 THEN 3 ELSE FS_STATE END
    WHERE ELEMENT_ID=:o.ELEMENT_ID AND FS_LABOR_CODE IS NOT NULL AND FS_LABOR_CODE <> 'PMOFFICE' AND FS_STATE NOT IN (3, 8);
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER FS_TC_UNASSIGN_D SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER FS_WBS_DIRTY_U  BEFORE  UPDATE OF ELEMENT_NAME, FD ON TMT_WBS
REFERENCING  OLD AS o  NEW AS n  FOR EACH ROW
WHEN ( o.TYPE_ID IN (3,59, 236, 237, 238, 239, 240, 241, 260) AND o.FS_LABOR_CODE IS NOT NULL AND o.FS_LABOR_CODE <> 'PMOFFICE' AND o.FS_STATE NOT IN (3, 8) AND ((o.FD <> n.FD AND n.FD=39) OR (n.ELEMENT_NAME<>o.ELEMENT_NAME)) )
BEGIN
    :n.FS_STATE := CASE WHEN :o.FS_STATE IN (0,4,5) THEN 1 WHEN  :o.FS_STATE=6 THEN 7 WHEN :o.FS_STATE=2 THEN 3 ELSE :o.FS_STATE END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER FS_WBS_DIRTY_U SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TMT_COSTS_D AFTER DELETE ON TMT_COSTS REFERENCING OLD AS o FOR EACH ROW
BEGIN
    UPDATE PDA_LOOKUP SET REC_STATUS = 'D' WHERE ELEMENT_ID = :o.ELEMENT_ID;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TMT_COSTS_D SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER BUDGET_UPDATE
BEFORE UPDATE OF
    AUTO_FLAG,
    ESTIMATED_BUDGET,
    ESTIMATED_ACTUAL,
    ESTIMATED_ETC,
    ESTIMATED_PAID_TODATE,
    CONTINGENCY_BUDGET,
    CONTINGENCY_ETC,
    ETC,
    PAID_TODATE,
    PROB_PERCENT
ON BUDGETS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    o.AUTO_FLAG <> n.AUTO_FLAG OR
    o.ESTIMATED_BUDGET <> n.ESTIMATED_BUDGET OR
    o.ESTIMATED_ACTUAL <> n.ESTIMATED_ACTUAL OR
    o.ESTIMATED_ETC <> n.ESTIMATED_ETC OR
    o.ESTIMATED_PAID_TODATE <> n.ESTIMATED_PAID_TODATE OR
    o.CONTINGENCY_BUDGET <> n.CONTINGENCY_BUDGET OR
    o.CONTINGENCY_ETC <> n.CONTINGENCY_ETC OR
    o.ETC <> n.ETC OR
    o.PAID_TODATE <> n.PAID_TODATE OR
    o.PROB_PERCENT <> n.PROB_PERCENT
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 2097152) < 1048576) THEN :n.ROLLUP_BITS + 1048576 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER BUDGET_UPDATE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER BUDGET_BUDGET
BEFORE UPDATE OF
    BUDGET
ON BUDGETS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    o.BUDGET <> n.BUDGET
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 2097152) < 1048576) THEN :n.ROLLUP_BITS + 1048576 ELSE :n.ROLLUP_BITS END;
    :n.TIMEPHASED_BITS := CASE WHEN (Mod(:n.TIMEPHASED_BITS, 4) < 2) THEN :n.TIMEPHASED_BITS + 2 ELSE :n.TIMEPHASED_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER BUDGET_BUDGET SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER BUDGET_ACTUAL
BEFORE UPDATE OF
    ACTUAL
ON BUDGETS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    o.ACTUAL <> n.ACTUAL
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 2097152) < 1048576) THEN :n.ROLLUP_BITS + 1048576 ELSE :n.ROLLUP_BITS END;
    :n.TIMEPHASED_BITS := CASE WHEN (Mod(:n.TIMEPHASED_BITS, 8) < 4) THEN :n.TIMEPHASED_BITS + 4 ELSE :n.TIMEPHASED_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER BUDGET_ACTUAL SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER PBB_PROCESS_FLAG
BEFORE UPDATE OF
    PROCESS_FLAG
ON PROJECT_BUDGET_BREAKDOWN
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (o.PROCESS_FLAG = 'T' AND n.PROCESS_FLAG = 'D') OR
    (o.PROCESS_FLAG = 'D' AND n.PROCESS_FLAG = 'T')
)
BEGIN
    :n.PROCESS_FLAG := 'I';
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER PBB_PROCESS_FLAG SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER WBS_BUDGET_UPDATE
AFTER UPDATE OF
    AUTO_FLAG,
    ESTIMATED_BUDGET,
    ESTIMATED_ACTUAL,
    ESTIMATED_ETC,
    ESTIMATED_PAID_TODATE,
    CONTINGENCY_BUDGET,
    CONTINGENCY_ETC,
    BUDGET,
    ACTUAL,
    ETC,
    PAID_TODATE,
    PROB_PERCENT
ON BUDGETS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    o.AUTO_FLAG <> n.AUTO_FLAG OR
    o.ESTIMATED_BUDGET <> n.ESTIMATED_BUDGET OR
    o.ESTIMATED_ACTUAL <> n.ESTIMATED_ACTUAL OR
    o.ESTIMATED_ETC <> n.ESTIMATED_ETC OR
    o.ESTIMATED_PAID_TODATE <> n.ESTIMATED_PAID_TODATE OR
    o.CONTINGENCY_BUDGET <> n.CONTINGENCY_BUDGET OR
    o.CONTINGENCY_ETC <> n.CONTINGENCY_ETC OR
    o.BUDGET <> n.BUDGET OR
    o.ACTUAL <> n.ACTUAL OR
    o.ETC <> n.ETC OR
    o.PAID_TODATE <> n.PAID_TODATE OR
    o.PROB_PERCENT <> n.PROB_PERCENT
    )
)
BEGIN
    UPDATE TMT_WBS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END,
    TIMEPHASED_BITS = CASE WHEN (Mod(TIMEPHASED_BITS, 4) < 2) THEN TIMEPHASED_BITS + 2 ELSE TIMEPHASED_BITS END
    WHERE PROJECT_ID = :n.PROJECT_ID
    AND ELEMENT_ID = :n.ELEMENT_ID;
    UPDATE TMT_CRI
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END,
    TIMEPHASED_BITS = CASE WHEN (Mod(TIMEPHASED_BITS, 4) < 2) THEN TIMEPHASED_BITS + 2 ELSE TIMEPHASED_BITS END
    WHERE PROJECT_ID = :n.PROJECT_ID AND ELEMENT_ID = :n.ELEMENT_ID AND LEVEL_ID > 1;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER WBS_BUDGET_UPDATE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER WBS_ACTUAL_BIT
BEFORE UPDATE OF FD, MILESTONE_PERCENT, ACTUAL_START, ACTUAL_FINISH
ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    (o.FD <> n.FD) OR
    (o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT) OR
    Coalesce(o.ACTUAL_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.ACTUAL_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.ACTUAL_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.ACTUAL_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 2048) < 1024) THEN :n.ROLLUP_BITS + 1024 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER WBS_ACTUAL_BIT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_ACTUAL_BIT
BEFORE UPDATE OF ACTUAL_START, ACTUAL_FINISH, WORK_TODATE, MILESTONE_PERCENT
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.ACTUAL_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.ACTUAL_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.ACTUAL_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.ACTUAL_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    (o.WORK_TODATE <> n.WORK_TODATE ) OR
    (o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT )
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 2048) < 1024) THEN :n.ROLLUP_BITS + 1024 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_ACTUAL_BIT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_AD_WBS
AFTER UPDATE OF ACTUAL_START, ACTUAL_FINISH, WORK_TODATE, MILESTONE_PERCENT
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.ACTUAL_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.ACTUAL_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.ACTUAL_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.ACTUAL_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    (o.WORK_TODATE <> n.WORK_TODATE ) OR
    (o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT )
    )
)
BEGIN
    UPDATE TMT_WBS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = :n.TASK_ID;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_AD_WBS SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_AD_CRI
AFTER UPDATE OF ACTUAL_START, ACTUAL_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW
WHEN (
    (n.TABLE_TYPE = 3) AND (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.ACTUAL_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.ACTUAL_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.ACTUAL_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.ACTUAL_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    UPDATE TMT_CRI
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = :n.TASK_ID;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_AD_CRI SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER WBS_SCHED_BIT
BEFORE UPDATE OF SCHED_START, SCHED_FINISH, SCHED_DURATION
ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.SCHED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.SCHED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.SCHED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.SCHED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    (o.SCHED_DURATION <> n.SCHED_DURATION)
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 4096) < 2048) THEN :n.ROLLUP_BITS + 2048 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER WBS_SCHED_BIT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_SCHED_BIT
BEFORE UPDATE OF SCHED_START, SCHED_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.SCHED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.SCHED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.SCHED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.SCHED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 4096) < 2048) THEN :n.ROLLUP_BITS + 2048 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_SCHED_BIT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_SD_WBS
AFTER UPDATE OF SCHED_START, SCHED_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW
WHEN (
    (n.TABLE_TYPE = 1) AND (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.SCHED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.SCHED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.SCHED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.SCHED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    UPDATE TMT_WBS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = :n.TASK_ID;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_SD_WBS SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_SD_CRI
AFTER UPDATE OF SCHED_START, SCHED_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW
WHEN (
    (n.TABLE_TYPE = 3) AND (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.SCHED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.SCHED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.SCHED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.SCHED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    UPDATE TMT_CRI
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = :n.TASK_ID;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_SD_CRI SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER WBS_PLANNED_BIT
BEFORE UPDATE OF PLANNED_START, PLANNED_FINISH
ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.PLANNED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.PLANNED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.PLANNED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.PLANNED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 8192) < 4096) THEN :n.ROLLUP_BITS + 4096 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER WBS_PLANNED_BIT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_PLANNED_BIT
BEFORE UPDATE OF PLANNED_START, PLANNED_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.PLANNED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.PLANNED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.PLANNED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.PLANNED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 8192) < 4096) THEN :n.ROLLUP_BITS + 4096 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_PLANNED_BIT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_ND_WBS
AFTER UPDATE OF PLANNED_START, PLANNED_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW
WHEN (
    (n.TABLE_TYPE = 1) AND (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.PLANNED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.PLANNED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.PLANNED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.PLANNED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    UPDATE TMT_WBS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = :n.TASK_ID;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_ND_WBS SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_ND_CRI
AFTER UPDATE OF PLANNED_START, PLANNED_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW
WHEN (
    (n.TABLE_TYPE = 3) AND (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.PLANNED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.PLANNED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.PLANNED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.PLANNED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    UPDATE TMT_CRI
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = :n.TASK_ID;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_ND_CRI SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER WBS_FORECAST_BIT
BEFORE UPDATE OF FORECAST_START, FORECAST_FINISH, FORECAST_DURATION
ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.FORECAST_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.FORECAST_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.FORECAST_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.FORECAST_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    (o.FORECAST_DURATION <> n.FORECAST_DURATION)
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 16384) < 8192) THEN :n.ROLLUP_BITS + 8192 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER WBS_FORECAST_BIT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_FORECAST_BIT
BEFORE UPDATE OF FORECAST_START, FORECAST_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.FORECAST_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.FORECAST_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.FORECAST_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.FORECAST_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 16384) < 8192) THEN :n.ROLLUP_BITS + 8192 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_FORECAST_BIT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_FD_WBS
AFTER UPDATE OF FORECAST_START, FORECAST_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW
WHEN (
    (n.TABLE_TYPE = 1) AND (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.FORECAST_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.FORECAST_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.FORECAST_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.FORECAST_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    UPDATE TMT_WBS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = :n.TASK_ID;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_FD_WBS SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_FD_CRI
AFTER UPDATE OF FORECAST_START, FORECAST_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW
WHEN (
    (n.TABLE_TYPE = 3) AND (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.FORECAST_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.FORECAST_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.FORECAST_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.FORECAST_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    UPDATE TMT_CRI
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = :n.TASK_ID;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_FD_CRI SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_EFFORTS_BEFORE
BEFORE UPDATE OF
    PLANNED_WORK,
    SCHED_WORK,
    ETC,
    WORK_TODATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    o.PLANNED_WORK <> n.PLANNED_WORK OR
    o.SCHED_WORK <> n.SCHED_WORK OR
    o.ETC <> n.ETC OR
    o.WORK_TODATE <> n.WORK_TODATE
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 8) < 4) THEN :n.ROLLUP_BITS + 4 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_EFFORTS_BEFORE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_EFFORTS_WBS
AFTER UPDATE OF
    PLANNED_WORK,
    SCHED_WORK,
    ETC,
    WORK_TODATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.TABLE_TYPE = 1) AND (n.ROLLUP_BITS <> -1) AND
    (
    o.PLANNED_WORK <> n.PLANNED_WORK OR
    o.SCHED_WORK <> n.SCHED_WORK OR
    o.ETC <> n.ETC OR
    o.WORK_TODATE <> n.WORK_TODATE
    )
)
BEGIN
    UPDATE TMT_WBS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2) < 1) AND (Mod(ROLLUP_BITS, 8) < 4) THEN ROLLUP_BITS + 5
        WHEN (Mod(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1
        WHEN (Mod(ROLLUP_BITS, 8) < 4) THEN ROLLUP_BITS + 4 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = :n.TASK_ID; UPDATE BUDGETS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
    WHERE PROJECT_ID = :n.PROJECT_ID AND ELEMENT_ID = :n.TASK_ID
    AND Rtrim(CHARGECODE_ID) IN ('EXPENSE_LABOR', 'CAPITAL_LABOR', 'BILLABLE_BENEFIT_LABOR', 'EXPENSE_HOURS', 'CAPITAL_HOURS', 'BENEFIT_HOURS');
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_EFFORTS_WBS SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_EFFORTS_CRI
AFTER UPDATE OF
    PLANNED_WORK,
    SCHED_WORK,
    ETC,
    WORK_TODATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.TABLE_TYPE = 3) AND (n.ROLLUP_BITS <> -1) AND
    (
    o.PLANNED_WORK <> n.PLANNED_WORK OR
    o.SCHED_WORK <> n.SCHED_WORK OR
    o.ETC <> n.ETC OR
    o.WORK_TODATE <> n.WORK_TODATE
    )
)
BEGIN
    UPDATE TMT_CRI
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = :n.TASK_ID; UPDATE BUDGETS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
    WHERE PROJECT_ID = :n.PROJECT_ID AND ELEMENT_ID = :n.TASK_ID
    AND Rtrim(CHARGECODE_ID) IN ('EXPENSE_LABOR', 'CAPITAL_LABOR', 'BILLABLE_BENEFIT_LABOR', 'EXPENSE_HOURS', 'CAPITAL_HOURS', 'BENEFIT_HOURS');
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_EFFORTS_CRI SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_RATES_BEFORE
BEFORE UPDATE OF
    BILLABLE,
    CAPITAL_EXPENSE,
    REGULAR_COST,
    REGULAR_RATE,
    SPECIAL_COST,
    SPECIAL_RATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.ROLLUP_BITS <> -1) AND
    (
    o.BILLABLE <> n.BILLABLE OR
    o.CAPITAL_EXPENSE <> n.CAPITAL_EXPENSE OR
    o.REGULAR_COST <> n.REGULAR_COST OR
    o.REGULAR_RATE <> n.REGULAR_RATE OR
    o.SPECIAL_COST <> n.SPECIAL_COST OR
    o.SPECIAL_RATE <> n.SPECIAL_RATE
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 8) < 4) THEN :n.ROLLUP_BITS + 4 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_RATES_BEFORE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_RATES_WBS
AFTER UPDATE OF
    BILLABLE,
    CAPITAL_EXPENSE,
    REGULAR_COST,
    REGULAR_RATE,
    SPECIAL_COST,
    SPECIAL_RATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.TABLE_TYPE = 1) AND (n.ROLLUP_BITS <> -1) AND
    (
    o.BILLABLE <> n.BILLABLE OR
    o.CAPITAL_EXPENSE <> n.CAPITAL_EXPENSE OR
    o.REGULAR_COST <> n.REGULAR_COST OR
    o.REGULAR_RATE <> n.REGULAR_RATE OR
    o.SPECIAL_COST <> n.SPECIAL_COST OR
    o.SPECIAL_RATE <> n.SPECIAL_RATE
    )
)
BEGIN
    UPDATE TMT_WBS
    SET ROLLUP_BITS = ROLLUP_BITS + CASE WHEN (Mod(ROLLUP_BITS, 2097152) < 1048576) THEN 1048576 ELSE 0 END
    + CASE WHEN (Mod(ROLLUP_BITS, 8) < 4) THEN 4 ELSE 0 END
    + CASE WHEN (Mod(ROLLUP_BITS, 2) < 1) THEN 1 ELSE 0 END
    WHERE PROJECT_ID = :n.PROJECT_ID
    AND ELEMENT_ID = :n.TASK_ID;
    UPDATE BUDGETS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
    WHERE PROJECT_ID = :n.PROJECT_ID AND ELEMENT_ID = :n.TASK_ID;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_RATES_WBS SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_RATES_CRI
AFTER UPDATE OF
    BILLABLE,
    CAPITAL_EXPENSE,
    REGULAR_COST,
    REGULAR_RATE,
    SPECIAL_COST,
    SPECIAL_RATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.TABLE_TYPE = 3) AND (n.ROLLUP_BITS <> -1) AND
    (
    o.BILLABLE <> n.BILLABLE OR
    o.CAPITAL_EXPENSE <> n.CAPITAL_EXPENSE OR
    o.REGULAR_COST <> n.REGULAR_COST OR
    o.REGULAR_RATE <> n.REGULAR_RATE OR
    o.SPECIAL_COST <> n.SPECIAL_COST OR
    o.SPECIAL_RATE <> n.SPECIAL_RATE
    )
)
BEGIN
    UPDATE TMT_CRI
    SET ROLLUP_BITS = ROLLUP_BITS + CASE WHEN (Mod(ROLLUP_BITS, 2097152) < 1048576) THEN 1048576 ELSE 0 END
    + CASE WHEN (Mod(ROLLUP_BITS, 8) < 4) THEN 4 ELSE 0 END
    + CASE WHEN (Mod(ROLLUP_BITS, 2) < 1) THEN 1 ELSE 0 END
    WHERE PROJECT_ID = :n.PROJECT_ID
    AND ELEMENT_ID = :n.TASK_ID;
    UPDATE BUDGETS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
    WHERE PROJECT_ID = :n.PROJECT_ID AND ELEMENT_ID = :n.TASK_ID;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_RATES_CRI SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_LABOR
AFTER INSERT
ON TASK_ASSIGNMENTS
REFERENCING NEW AS n
FOR EACH ROW
BEGIN
    UPDATE BUDGETS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2097152) < 1048576) THEN
    ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
    WHERE PROJECT_ID = :n.PROJECT_ID AND ELEMENT_ID = :n.TASK_ID
    AND Rtrim(CHARGECODE_ID) IN ('EXPENSE_LABOR', 'CAPITAL_LABOR',
    'BILLABLE_BENEFIT_LABOR', 'EXPENSE_HOURS', 'CAPITAL_HOURS','BENEFIT_HOURS');
    UPDATE TMT_WBS
--      SET ROLLUP_BITS = ROLLUP_BITS + CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN 1048576 ELSE 0 END
--      + CASE WHEN (MOD(ROLLUP_BITS, 8192) < 4096) THEN 4096 ELSE 0 END
--      + CASE WHEN (MOD(ROLLUP_BITS, 8) < 4) THEN 4 ELSE 0 END
--      + CASE WHEN (MOD(ROLLUP_BITS, 4) < 2) THEN 2 ELSE 0 END
--      + CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN 1 ELSE 0 END,
        --SET ROLLUP_BITS = ROLLUP_BITS - BITAND(ROLLUP_BITS, 1048576+4096+4+2+1) + 1048576+4096+4+2+1,
        SET ROLLUP_BITS = ROLLUP_BITS - BitAnd(ROLLUP_BITS, 1052679) + 1052679
        WHERE ELEMENT_ID = :n.TASK_ID;

    UPDATE TMT_CRI
--  SET ROLLUP_BITS = ROLLUP_BITS + CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN 1048576 ELSE 0 END
--  + CASE WHEN (MOD(ROLLUP_BITS, 8192) < 4096) THEN 4096 ELSE 0 END
--  + CASE WHEN (MOD(ROLLUP_BITS, 8) < 4) THEN 4 ELSE 0 END
--  + CASE WHEN (MOD(ROLLUP_BITS, 4) < 2) THEN 2 ELSE 0 END
--  + CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN 1 ELSE 0 END
        SET ROLLUP_BITS = ROLLUP_BITS - BitAnd(ROLLUP_BITS, 1052679) + 1052679
    WHERE ELEMENT_ID = :n.TASK_ID;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_LABOR SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER COST_DELETE
AFTER DELETE
ON TMT_COSTS
REFERENCING OLD AS o
FOR EACH ROW
BEGIN
    UPDATE BUDGETS
    SET ROLLUP_BITS = ROLLUP_BITS + CASE WHEN (Mod(ROLLUP_BITS, 4194304) < 2097152) THEN 2097152 ELSE 0 END
    + CASE WHEN (Mod(ROLLUP_BITS, 8388608) < 4194304) THEN 4194304 ELSE 0 END
    WHERE PROJECT_ID = :o.PROJECT_ID
    AND BUDGET_ID = :o.BUDGET_ID;
    UPDATE TMT_WBS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 4194304) < 2097152) THEN ROLLUP_BITS + 2097152 ELSE ROLLUP_BITS END 
    WHERE ELEMENT_ID = (SELECT ELEMENT_ID FROM BUDGETS
    WHERE BUDGET_ID = :o.BUDGET_ID AND PROJECT_ID = :o.PROJECT_ID);
    UPDATE TMT_CRI
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 4194304) < 2097152) THEN ROLLUP_BITS + 2097152 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = (SELECT ELEMENT_ID FROM BUDGETS
    WHERE BUDGET_ID = :o.BUDGET_ID AND PROJECT_ID = :o.PROJECT_ID) AND LEVEL_ID > 1;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER COST_DELETE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER COST_UPDATE
AFTER UPDATE OF
    BILLABLE, ACTUAL_AMOUNT, TAX1_AMOUNT, TAX2_AMOUNT, EXCHANGE_RATE
ON TMT_COSTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    o.BILLABLE <> n.BILLABLE OR
    o.ACTUAL_AMOUNT <> n.ACTUAL_AMOUNT OR
    o.TAX1_AMOUNT <> n.TAX1_AMOUNT OR
    o.TAX2_AMOUNT <> n.TAX2_AMOUNT OR
    o.EXCHANGE_RATE <> n.EXCHANGE_RATE
)
BEGIN
    UPDATE BUDGETS
    SET ROLLUP_BITS = ROLLUP_BITS + CASE WHEN (Mod(ROLLUP_BITS, 2097152) < 1048576) THEN 1048576 ELSE 0 END
    + CASE WHEN (Mod(ROLLUP_BITS, 8388608) < 4194304) THEN 4194304 ELSE 0 END
    WHERE BUDGET_ID = :n.BUDGET_ID
    AND PROJECT_ID = :n.PROJECT_ID; UPDATE TMT_WBS
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = (SELECT ELEMENT_ID FROM BUDGETS
    WHERE BUDGET_ID = :n.BUDGET_ID AND PROJECT_ID = :n.PROJECT_ID); UPDATE TMT_CRI
    SET ROLLUP_BITS = CASE WHEN (Mod(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
    WHERE ELEMENT_ID = (SELECT ELEMENT_ID FROM BUDGETS WHERE BUDGET_ID = :n.BUDGET_ID) AND LEVEL_ID > 1;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER COST_UPDATE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER WBS_ASSIGNED_INS
AFTER INSERT ON TMT_CRI
REFERENCING NEW AS n
FOR EACH ROW
WHEN ( (n.WBS_ID IS NOT NULL) AND (n.WBS_ID <> n.ELEMENT_ID) )

DECLARE PRAGMA AUTONOMOUS_TRANSACTION;
elementID VARCHAR2(32);

--BEGIN ATOMIC

CURSOR parentIDList IS
    SELECT element_id
    FROM tmt_cri
     --UPDATE TMT_CRI SET WBS_ASSIGNED = WBS_ASSIGNED + 1
    WHERE type_id IN (4,5,6,35,213,215,226,227,244,257,64) AND rank > SubStr(:n.rank,1,12) AND rank < :n.rank AND SubStr(:n.rank, 1, CASE WHEN Length(rank) > length(:n.rank) THEN 1 ELSE length(rank) END) = rank;
BEGIN
    OPEN parentIDList;
    LOOP
        FETCH parentIDList INTO elementID;
        EXIT WHEN parentIDList%notfound;

        PMO_Triggers.element_id_Count(elementID) := Coalesce(PMO_Triggers.element_id_Count(elementID), 0) + 1;
    END LOOP;
    CLOSE parentIDList;

    EXCEPTION
        WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'trigger', 'TRIGGER WBS_ASSIGNED_INS SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END WBS_ASSIGNED_INS;
/

CREATE OR REPLACE TRIGGER WBS_ASSIGNED_DEL
AFTER DELETE
ON TMT_CRI
REFERENCING OLD AS O
FOR EACH ROW
WHEN ( (o.WBS_ID IS NOT NULL) AND (o.WBS_ID <> o.ELEMENT_ID) )

DECLARE PRAGMA AUTONOMOUS_TRANSACTION;
elementID VARCHAR2(32);

--BEGIN ATOMIC

CURSOR parentIDList IS
    SELECT element_id
    FROM tmt_cri
     --UPDATE TMT_CRI SET WBS_ASSIGNED = WBS_ASSIGNED - 1
    WHERE type_id IN (4,5,6,35,213,215,226,227,244,257,64) AND rank > SubStr(:o.rank,1,12) AND rank < :o.rank AND SubStr(:o.rank, 1, CASE WHEN Length(rank) > length(:o.rank) THEN 1 ELSE length(rank) END) = rank;
BEGIN
    OPEN parentIDList;
    LOOP
        FETCH parentIDList INTO elementID;
        EXIT WHEN parentIDList%notfound;

        PMO_Triggers.element_id_Count(elementID) := Coalesce(PMO_Triggers.element_id_Count(elementID), 0) - 1;
    END LOOP;
    CLOSE parentIDList;

    EXCEPTION
        WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'trigger', 'TRIGGER WBS_ASSIGNED_DEL SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END WBS_ASSIGNED_DEL;
/

CREATE OR REPLACE TRIGGER WBS_ASSIGNED_MOVE
AFTER  UPDATE OF PARENT_ID ON TMT_CRI
REFERENCING  OLD AS o NEW AS n
FOR EACH ROW
WHEN ( (o.PARENT_ID <> n.PARENT_ID) AND (n.WBS_ASSIGNED > 0) )
DECLARE PRAGMA AUTONOMOUS_TRANSACTION;
elementID VARCHAR2(32);

--BEGIN ATOMIC

CURSOR parentIDListNew IS
    SELECT element_id
    FROM tmt_cri
--   UPDATE TMT_CRI SET WBS_ASSIGNED = WBS_ASSIGNED + n.WBS_ASSIGNED
    WHERE TYPE_ID IN (4,5,6,35,213,215,226,227,244,257,64)
            AND (LEVEL_ID < :n.LEVEL_ID)
            AND RANK >SubStr(:n.RANK,1,12)
            AND RANK < :n.RANK
            AND SubStr(:n.RANK, 1, CASE WHEN Length(RANK) > Length(:n.RANK) THEN 1 ELSE Length(RANK) END) = RANK;

CURSOR parentIDListOld IS
    SELECT element_id
    FROM tmt_cri
--  UPDATE TMT_CRI SET WBS_ASSIGNED = WBS_ASSIGNED - :o.WBS_ASSIGNED
    WHERE TYPE_ID IN (4,5,6,35,213,215,226,227,244,257,64)
            AND RANK >SubStr(:o.RANK,1,12)
            AND RANK < :o.RANK
            AND SubStr(:o.RANK, 1, CASE WHEN Length(RANK) > Length(:o.RANK) THEN 1 ELSE Length(RANK) END) = RANK;

BEGIN
    OPEN parentIDListNew;
    LOOP
        FETCH parentIDListNew INTO elementID;
        EXIT WHEN parentIDListNew%notfound;

        PMO_Triggers.element_id_Count(elementID) := Coalesce(PMO_Triggers.element_id_Count(elementID), 0) + :n.WBS_ASSIGNED;
    END LOOP;
    CLOSE parentIDListNew;

    OPEN parentIDListOld;
    LOOP
        FETCH parentIDListOld INTO elementID;
        EXIT WHEN parentIDListOld%notfound;

        PMO_Triggers.element_id_Count(elementID) := Coalesce(PMO_Triggers.element_id_Count(elementID), 0) - :o.WBS_ASSIGNED;
    END LOOP;
    CLOSE parentIDListOld;

    EXCEPTION
        WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'trigger', 'TRIGGER WBS_ASSIGNED_MOVE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
        ROLLBACK;
END;
/

CREATE OR REPLACE TRIGGER WBS_ASSIGNED_STMT
AFTER DELETE OR INSERT OR UPDATE OF PARENT_ID
ON TMT_CRI
DECLARE
      elementID VARCHAR(32);
BEGIN

    IF PMO_Triggers.element_id_Count.Count > 0 THEN
        elementID := PMO_Triggers.element_id_Count.FIRST;

        WHILE elementID IS NOT NULL LOOP
            UPDATE tmt_cri
                    SET wbs_assigned = wbs_assigned + PMO_Triggers.element_id_Count(elementID)
                WHERE element_id = elementID;

            elementID := PMO_Triggers.element_id_Count.Next(elementID);
        END LOOP;

        PMO_Triggers.element_id_Count.DELETE;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'trigger', 'TRIGGER WBS_ASSIGNED_STMT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER CRI_ACTUAL_BIT
BEFORE UPDATE OF FD, MILESTONE_PERCENT, ACTUAL_START, ACTUAL_FINISH
ON TMT_CRI
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.WBS_ID IS NULL) AND
    (n.ROLLUP_BITS <> -1) AND
    (
    (o.FD <> n.FD) OR
    (o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT) OR
    Coalesce(o.ACTUAL_START,To_Timestamp('1900-01-01 00:00:00.000000')) <> Coalesce(n.ACTUAL_START,To_Timestamp('1900-01-01 00:00:00.000000')) OR
    Coalesce(o.ACTUAL_FINISH,To_Timestamp('2100-01-01 00:00:00.000000')) <> Coalesce(n.ACTUAL_FINISH,To_Timestamp('2100-01-01 00:00:00.000000'))
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 2048) < 1024) THEN :n.ROLLUP_BITS + 1024 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER CRI_ACTUAL_BIT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER CRI_SCHED_BIT
BEFORE UPDATE OF SCHED_START, SCHED_FINISH
ON TMT_CRI
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.WBS_ID IS NULL) AND
    (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.SCHED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.SCHED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.SCHED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.SCHED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 4096) < 2048) THEN :n.ROLLUP_BITS + 2048 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER CRI_SCHED_BIT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER CRI_FORECAST_BIT
BEFORE UPDATE OF FORECAST_START, FORECAST_FINISH
ON TMT_CRI
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.WBS_ID IS NULL) AND
    (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.FORECAST_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.FORECAST_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.FORECAST_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.FORECAST_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 16384) < 8192) THEN :n.ROLLUP_BITS + 8192 ELSE :n.ROLLUP_BITS END ;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER CRI_FORECAST_BIT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER CRI_PLANNED_BIT
BEFORE UPDATE OF PLANNED_START, PLANNED_FINISH
ON TMT_CRI
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    (n.WBS_ID IS NULL) AND
    (n.ROLLUP_BITS <> -1) AND
    (
    Coalesce(o.PLANNED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.PLANNED_START,To_Timestamp('1900-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) OR
    Coalesce(o.PLANNED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF')) <> Coalesce(n.PLANNED_FINISH,To_Timestamp('2100-01-01 00:00:00.000000','YYYY-MM-DD HH24:MI:SS.FF'))
    )
)
BEGIN
    :n.ROLLUP_BITS := CASE WHEN (Mod(:n.ROLLUP_BITS, 8192) < 4096) THEN :n.ROLLUP_BITS + 4096 ELSE :n.ROLLUP_BITS END;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER CRI_PLANNED_BIT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER CRI_ASSIGN_NAMES
AFTER UPDATE OF ASSIGN_NAMES ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN ( (n.TRANSFER_FLAG = 'C') OR (n.TRANSFER_FLAG = 'W') )
BEGIN
    UPDATE TMT_CRI SET ASSIGN_NAMES = :n.ASSIGN_NAMES WHERE PROJECT_ID = :n.PROJECT_ID AND WBS_ID = :n.ELEMENT_ID AND LEVEL_ID > 1;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER CRI_ASSIGN_NAMES SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER WBS_STAGE_INSERT
AFTER INSERT ON TMT_WBS
REFERENCING NEW AS n
FOR EACH ROW
    DECLARE indExistence PLS_INTEGER;
BEGIN
    SELECT STAGE_ID INTO indExistence FROM PROJECT_STATE_DEF WHERE PROJECT_ID = :n.PROJECT_ID
    AND ELEMENT_TYPE = CASE WHEN :n.TYPE_ID IN (154, 1) THEN 1
     WHEN :n.TYPE_ID IN (2,36,37,39,58,60,225,230,231,259) THEN 2
     WHEN :n.TYPE_ID IN (3,59,236,237,238,239,240,241,260) THEN 3 END
    AND STAGE_ID = :n.STAGE_ID AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        INSERT INTO PROJECT_STATE_TRG VALUES(:n.PROJECT_ID, 1, :n.TYPE_ID, :n.ELEMENT_ID, :n.STAGE_ID, '0', :n.REC_USER, NULL) ;
        PMO_DB2M.SC_STATE := SQLCODE;
    END IF;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER WBS_STAGE_INSERT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER WBS_STAGE_AFTER
AFTER UPDATE OF STAGE_ID ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN ( o.STAGE_ID <> n.STAGE_ID )
    DECLARE indExistence PLS_INTEGER;
BEGIN
    SELECT STAGE_ID INTO indExistence FROM PROJECT_STATE_DEF WHERE PROJECT_ID = :n.PROJECT_ID
    AND ELEMENT_TYPE = CASE WHEN :n.TYPE_ID IN (154, 1) THEN 1
        WHEN :n.TYPE_ID IN (2,36,37,39,58,60,225,230,231,259) THEN 2
        WHEN :n.TYPE_ID IN (3,59,236,237,238,239,240,241,260) THEN 3 END
    AND STAGE_ID = :n.STAGE_ID AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        INSERT INTO PROJECT_STATE_TRG VALUES(:n.PROJECT_ID, 1, :n.TYPE_ID, :n.ELEMENT_ID, :n.STAGE_ID, '0', :n.REC_USER, :n.REVISION_HISTORY) ;
        PMO_DB2M.SC_STATE := SQLCODE;
    END IF;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER WBS_STAGE_AFTER SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER CRI_STAGE_INSERT
AFTER INSERT ON TMT_CRI
REFERENCING NEW AS n
FOR EACH ROW
    DECLARE indExistence PLS_INTEGER;
BEGIN
    SELECT STAGE_ID INTO indExistence FROM PROJECT_STATE_DEF WHERE PROJECT_ID = :n.PROJECT_ID
    AND ELEMENT_TYPE = CASE WHEN :n.TYPE_ID IN (4, 65) THEN 4
         WHEN :n.TYPE_ID IN (257, 258) THEN 257
         WHEN :n.TYPE_ID IN (5,66) THEN 5
         WHEN :n.TYPE_ID IN (6,67) THEN 6
         WHEN :n.TYPE_ID IN (213, 235) THEN 213
         WHEN :n.TYPE_ID IN (226, 233) THEN 226
         WHEN :n.TYPE_ID IN (227, 234) THEN 227 END
    AND STAGE_ID = :n.STAGE_ID AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        INSERT INTO PROJECT_STATE_TRG VALUES(:n.PROJECT_ID, 3, :n.TYPE_ID, :n.ELEMENT_ID, :n.STAGE_ID, '0', :n.REC_USER, NULL) ;
        PMO_DB2M.SC_STATE := SQLCODE;
    END IF;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER CRI_STAGE_INSERT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER CRI_STAGE_AFTER
AFTER UPDATE OF STAGE_ID ON TMT_CRI
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN ( o.STAGE_ID <> n.STAGE_ID )
    DECLARE indExistence PLS_INTEGER;
BEGIN
    SELECT STAGE_ID INTO indExistence FROM PROJECT_STATE_DEF WHERE PROJECT_ID = :n.PROJECT_ID
    AND ELEMENT_TYPE = CASE WHEN :n.TYPE_ID IN (4, 65) THEN 4
         WHEN :n.TYPE_ID IN (257, 258) THEN 257
         WHEN :n.TYPE_ID IN (5,66) THEN 5
         WHEN :n.TYPE_ID IN (6,67) THEN 6
         WHEN :n.TYPE_ID IN (213, 235) THEN 213
         WHEN :n.TYPE_ID IN (226, 233) THEN 226
         WHEN :n.TYPE_ID IN (227, 234) THEN 227 END
    AND STAGE_ID = :n.STAGE_ID AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        INSERT INTO PROJECT_STATE_TRG VALUES(:n.PROJECT_ID, 3, :n.TYPE_ID, :n.ELEMENT_ID, :n.STAGE_ID, '0', :n.REC_USER, :n.REVISION_HISTORY) ;
        PMO_DB2M.SC_STATE := SQLCODE;
    END IF;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER CRI_STAGE_AFTER SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER TA_DB_LABOR
AFTER DELETE ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o
FOR EACH ROW
WHEN ( o.PROJECT_ID <> o.TASK_ID )
BEGIN
    --SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 8) < 4) THEN ROLLUP_BITS + 4 ELSE ROLLUP_BITS END
    UPDATE BUDGETS
            SET ROLLUP_BITS = ROLLUP_BITS + 4
        WHERE PROJECT_ID = :o.PROJECT_ID AND ELEMENT_ID = :o.TASK_ID
            AND BitAnd(ROLLUP_BITS, 4) = 0;

    PMO_DB2M.SC_STATE := SQLCODE;
    
    UPDATE TMT_WBS 
    	    SET EV_FLAG = '1'
     	WHERE PROJECT_ID = :o.PROJECT_ID AND ELEMENT_ID = :o.TASK_ID; 
     	
    PMO_DB2M.SC_STATE := SQLCODE;
    
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_DB_LABOR SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

--CREATE OR REPLACE TRIGGER TA_U_LEVEL_CHANGE
--AFTER UPDATE OF RESOURCE_ID, RESOURCE_TYPE, BIT_FLAG, ASSIGN_STRING, ACCEPTED_DATE
--ON TASK_ASSIGNMENTS
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW
--WHEN (
--    ((o.RESOURCE_ID <> n.RESOURCE_ID) OR (o.RESOURCE_TYPE <> n.RESOURCE_TYPE) OR (o.BIT_FLAG <> n.BIT_FLAG) OR
--    (o.ASSIGN_STRING <> n.ASSIGN_STRING) OR (o.ACCEPTED_DATE <> n.ACCEPTED_DATE))
--)
--    DECLARE indExistence PLS_INTEGER;
--BEGIN
--    SELECT 1 INTO indExistence FROM TMT_WBS
--    WHERE ELEMENT_ID = :n.PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_COPY_DATETIME IS NULL AND ROWNUM = 1;

--    IF SQLCODE=0 THEN
--        --SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END,
--        UPDATE TMT_WBS
--            SET ROLLUP_BITS = ROLLUP_BITS + 131072,
--            LEVEL_LAST_CHANGE_DT = CURRENT_TIMESTAMP
--        WHERE ELEMENT_ID = :n.TASK_ID
--                AND BitAnd(ROLLUP_BITS, 131072) = 0;

--        PMO_DB2M.SC_STATE := SQLCODE;
--    END IF;
--EXCEPTION
--    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
--        lfile.fput( 'AllTrace', 'TRIGGER TA_U_LEVEL_CHANGE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
--END;
--/

CREATE OR REPLACE TRIGGER TA_U_LEVEL_CHANGE
AFTER UPDATE OF RESOURCE_ID, RESOURCE_TYPE, BIT_FLAG, ASSIGN_STRING, ACCEPTED_DATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN (
    ((o.RESOURCE_ID <> n.RESOURCE_ID) OR (o.RESOURCE_TYPE <> n.RESOURCE_TYPE) OR (o.BIT_FLAG <> n.BIT_FLAG) OR
    (o.ASSIGN_STRING <> n.ASSIGN_STRING) OR (o.ACCEPTED_DATE <> n.ACCEPTED_DATE))
)
BEGIN
	UPDATE TMT_WBS
	SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) AND (LEVEL_STATE NOT IN (0, 4)) 
     	AND ((SELECT LEVEL_LOCK FROM PROJECT_DETAILS WHERE PROJECT_ID = :n.PROJECT_ID) = 0) 
     	THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END, 
	LEVEL_STATE = CASE WHEN ((SELECT LEVEL_LOCK FROM PROJECT_DETAILS WHERE PROJECT_ID = :n.PROJECT_ID) > 0)
     	THEN LEVEL_STATE ELSE 0 END  
	WHERE ELEMENT_ID = :n.TASK_ID;

    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER TA_U_LEVEL_CHANGE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

--CREATE OR REPLACE TRIGGER WBS_LEVEL_CHANGE
--BEFORE UPDATE OF RANK, TYPE_ID, PRIORITY, CONSTRAINT_DT, CONSTRAINT_TYPE
--ON TMT_WBS
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW
--WHEN (
--    ((o.RANK <> n.RANK) OR (o.TYPE_ID <> n.TYPE_ID) OR (o.PRIORITY <> n.PRIORITY) OR
--    (o.CONSTRAINT_DT <> n.CONSTRAINT_DT) OR (o.CONSTRAINT_TYPE <> n.CONSTRAINT_TYPE))
--)
--DECLARE
--    PRAGMA AUTONOMOUS_TRANSACTION;
--    indExistence PLS_INTEGER;
--BEGIN
--    SELECT 1 INTO indExistence FROM TMT_WBS
--        WHERE ELEMENT_ID = :n.PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_COPY_DATETIME IS NULL AND ROWNUM = 1;

--    IF SQLCODE=0 THEN
--        :n.ROLLUP_BITS := :n.ROLLUP_BITS - BitAnd(:n.ROLLUP_BITS, 131072) + 131072;
--        :n.LEVEL_LAST_CHANGE_DT := CURRENT_TIMESTAMP;
--    END IF;

--EXCEPTION
--    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
--        lfile.fput( 'AllTrace', 'TRIGGER WBS_LEVEL_CHANGE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
--END;
--/

--CREATE OR REPLACE TRIGGER TA_D_LEVEL_CHANGE
--AFTER DELETE ON TASK_ASSIGNMENTS
--REFERENCING OLD AS o FOR EACH ROW
--    DECLARE indExistence PLS_INTEGER;
--BEGIN
--    SELECT 1 INTO indExistence FROM TMT_WBS
--    WHERE ELEMENT_ID = :o.PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_COPY_DATETIME IS NULL AND ROWNUM = 1;

--    IF SQLCODE=0 THEN
--        --SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END,
--        UPDATE TMT_WBS
--                SET ROLLUP_BITS = ROLLUP_BITS + 131072,
--                LEVEL_LAST_CHANGE_DT = CURRENT_TIMESTAMP
--            WHERE ELEMENT_ID = :o.TASK_ID
--                AND BitAnd(ROLLUP_BITS, 131072) = 0;

--        PMO_DB2M.SC_STATE := SQLCODE;
--    END IF;
--EXCEPTION
--    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
--        lfile.fput( 'AllTrace', 'TRIGGER TA_D_LEVEL_CHANGE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
--END;
--/

CREATE OR REPLACE TRIGGER DEP_D_DEPAND_FLAG
AFTER DELETE ON DEPENDENCIES
REFERENCING OLD AS o FOR EACH ROW
BEGIN
    --SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 524288) < 262144) THEN ROLLUP_BITS + 262144 ELSE ROLLUP_BITS END,
    UPDATE TMT_WBS
            SET ROLLUP_BITS = ROLLUP_BITS + 262144
        WHERE ELEMENT_ID = :o.SUCC_ID
            AND BitAnd(ROLLUP_BITS, 262144) = 0;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER DEP_D_DEPAND_FLAG SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER DEP_I_DEPAND_FLAG
AFTER INSERT ON DEPENDENCIES
REFERENCING NEW AS n FOR EACH ROW
BEGIN
    --SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 1048576) < 524288) THEN ROLLUP_BITS + 524288 ELSE ROLLUP_BITS END,
    UPDATE TMT_WBS
            SET ROLLUP_BITS = ROLLUP_BITS + 524288
        WHERE ELEMENT_ID = :n.PRED_ID
            AND BitAnd(ROLLUP_BITS, 524288) = 0;

    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER DEP_I_DEPAND_FLAG SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER DEP_D_LEVEL_CHANGE
AFTER DELETE ON DEPENDENCIES
REFERENCING OLD AS o FOR EACH ROW
    DECLARE indExistence PLS_INTEGER;
BEGIN
    SELECT 1 INTO indExistence FROM TMT_WBS
    WHERE ELEMENT_ID = :o.PRED_PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0 AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        --SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END,
        UPDATE TMT_WBS
                SET ROLLUP_BITS = ROLLUP_BITS + 131072
            WHERE ELEMENT_ID = :o.PRED_ID
                AND BitAnd(ROLLUP_BITS, 131072) = 0;

        PMO_DB2M.SC_STATE := SQLCODE;
    END IF;

    SELECT 1 INTO indExistence FROM TMT_WBS
    WHERE ELEMENT_ID = :o.SUCC_PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0 AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        --SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END,
        UPDATE TMT_WBS
                SET ROLLUP_BITS = ROLLUP_BITS + 131072
            WHERE ELEMENT_ID = :o.SUCC_ID
                AND BitAnd(ROLLUP_BITS, 131072) = 0;

        PMO_DB2M.SC_STATE := SQLCODE;
    END IF;

EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER DEP_D_LEVEL_CHANGE SQLCODE:' || SQLCODE || ' SQLERRM:' || SUBSTR(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER DEP_I_LEVEL_CHANGE
AFTER INSERT ON DEPENDENCIES
REFERENCING NEW AS n FOR EACH ROW
    DECLARE indExistence PLS_INTEGER;
BEGIN
    SELECT 1 INTO indExistence FROM TMT_WBS
    WHERE ELEMENT_ID = :n.PRED_PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0 AND ROWNUM = 1;

    IF SQLCODE=0 THEN
        --SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END,
        UPDATE TMT_WBS
                SET ROLLUP_BITS = ROLLUP_BITS + 131072
            WHERE ELEMENT_ID = :n.PRED_ID
                AND BitAnd(ROLLUP_BITS, 131072) = 0;

        PMO_DB2M.SC_STATE := SQLCODE;
    END IF;

EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER DEP_I_LEVEL_CHANGE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER DEP_I_LEVEL_SUCC
AFTER INSERT ON DEPENDENCIES
REFERENCING NEW AS n FOR EACH ROW
    DECLARE indExistence PLS_INTEGER;
BEGIN
    SELECT 1 INTO indExistence FROM TMT_WBS
    WHERE ELEMENT_ID = :n.SUCC_PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0 AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        --SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END,
        UPDATE TMT_WBS
                SET ROLLUP_BITS = ROLLUP_BITS + 131072
            WHERE ELEMENT_ID = :n.SUCC_ID
                AND BitAnd(ROLLUP_BITS, 131072) = 0;
    END IF;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER DEP_I_LEVEL_SUCC SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER DEP_U_LEVEL_CHANGE
AFTER UPDATE ON DEPENDENCIES
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
    DECLARE indExistence PLS_INTEGER;
BEGIN
    SELECT 1 INTO indExistence FROM TMT_WBS
    WHERE ELEMENT_ID = :n.PRED_PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0 AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        UPDATE TMT_WBS
                SET ROLLUP_BITS = ROLLUP_BITS + 131072
            WHERE ELEMENT_ID = :n.PRED_ID
                AND BitAnd(ROLLUP_BITS, 131072) = 0;
        PMO_DB2M.SC_STATE := SQLCODE;
    END IF;

    SELECT 1 INTO indExistence FROM TMT_WBS
    WHERE ELEMENT_ID = :n.SUCC_PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0 AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        --SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END,
        UPDATE TMT_WBS
                SET ROLLUP_BITS = ROLLUP_BITS + 131072
            WHERE ELEMENT_ID = :n.SUCC_ID
                AND BitAnd(ROLLUP_BITS, 131072) = 0;

        PMO_DB2M.SC_STATE := SQLCODE;
    END IF;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER DEP_U_LEVEL_CHANGE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

--CREATE OR REPLACE TRIGGER WBS_U_LEVEL_CHANGE
--BEFORE UPDATE OF SD, FD, START_DT, FINISH_DT, DURATION
--ON TMT_WBS
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW
--WHEN (  (n.TYPE_ID NOT IN (3,59) ) AND
--    ( (o.SD <> n.SD) OR (o.FD <> n.FD) OR (o.START_DT <> n.START_DT) OR (o.FINISH_DT <> n.FINISH_DT) OR (o.DURATION <> n.DURATION) )
--)
--DECLARE
--    PRAGMA AUTONOMOUS_TRANSACTION;
--    indExistence PLS_INTEGER;
--BEGIN
--    SELECT 1 INTO indExistence FROM TMT_WBS
--    WHERE ELEMENT_ID = :n.PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_COPY_DATETIME IS NULL AND ROWNUM = 1;

--    IF SQLCODE=0 THEN
--        :n.ROLLUP_BITS := :n.ROLLUP_BITS - BitAnd(:n.ROLLUP_BITS, 131072) + 131072;
--        :n.LEVEL_LAST_CHANGE_DT := CURRENT_TIMESTAMP;
--        PMO_DB2M.SC_STATE := SQLCODE;
--    END IF;
--EXCEPTION
--    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
--        lfile.fput( 'AllTrace', 'TRIGGER WBS_U_LEVEL_CHANGE SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
--END;
--/

CREATE OR REPLACE TRIGGER DOC_STAGE_AFTER
AFTER UPDATE OF STAGE_ID ON TMT_DOCUMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN ( o.STAGE_ID <> n.STAGE_ID AND n.LINK_DOC <> 'Y')
    DECLARE indExistence PLS_INTEGER;
BEGIN
    SELECT 1 INTO indExistence FROM PROJECT_STATE_DEF
    WHERE PROJECT_ID = :n.PROJECT_ID
    AND ELEMENT_TYPE = CASE WHEN :n.TYPE_ID IN (14,61,62,63,140) THEN 14 WHEN :n.TYPE_ID = 242 THEN 242 END
    AND STAGE_ID = :n.STAGE_ID AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        INSERT INTO PROJECT_STATE_TRG VALUES(:n.PROJECT_ID, 4, :n.TYPE_ID, :n.ELEMENT_ID, :n.STAGE_ID, '0', :n.REC_USER, :n.REVISION_HISTORY) ;
    END IF;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER DOC_STAGE_AFTER SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER WBS_TO100_FLAG
BEFORE UPDATE OF MILESTONE_PERCENT
ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN ( (n.ROLLUP_BITS <> -1) AND (o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT) AND (n.MILESTONE_PERCENT = 10000))
    DECLARE indExistence PLS_INTEGER;
BEGIN
    IF BitAnd(:n.ROLLUP_BITS,524288) = 0 THEN
        SELECT 1 INTO indExistence FROM DEPENDENCIES
            WHERE PRED_ID = :n.ELEMENT_ID AND ROWNUM = 1;

        IF SQLCODE=0 THEN
            --:n.ROLLUP_BITS := CASE WHEN (MOD(:n.ROLLUP_BITS, 1048576) < 524288) THEN :n.ROLLUP_BITS + 524288 ELSE :n.ROLLUP_BITS END;
            :n.ROLLUP_BITS := :n.ROLLUP_BITS + 524288;

            PMO_DB2M.SC_STATE := SQLCODE;
        END IF;
    ELSE
        -- NOTE: this differs from DB2 code which will only reset this value if existence is
        -- checked
        PMO_DB2M.SC_STATE := SQLCODE;
    END IF;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER WBS_TO100_FLAG SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER WBS_FROM100_FLAG
BEFORE UPDATE OF MILESTONE_PERCENT
ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
WHEN ( (n.ROLLUP_BITS <> -1) AND (o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT) AND (o.MILESTONE_PERCENT = 10000))
    DECLARE indExistence PLS_INTEGER;
BEGIN
    IF BitAnd(:n.ROLLUP_BITS,524288) = 0 THEN
        SELECT 1 INTO indExistence FROM DEPENDENCIES
            WHERE PRED_ID = :n.ELEMENT_ID AND ROWNUM = 1;

        IF SQLCODE=0 THEN
            --:n.ROLLUP_BITS := CASE WHEN (MOD(:n.ROLLUP_BITS, 1048576) < 524288) THEN :n.ROLLUP_BITS + 524288 ELSE :n.ROLLUP_BITS END;
            :n.ROLLUP_BITS := :n.ROLLUP_BITS + 524288;
            PMO_DB2M.SC_STATE := SQLCODE;
        END IF;
    ELSE
        -- NOTE: this differs from DB2 code which will only reset this value if existence is
        -- checked
        PMO_DB2M.SC_STATE := SQLCODE;
    END IF;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER WBS_FROM100_FLAG SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER SC_STAGE_INSERT
AFTER INSERT ON TMT_ASSIGNED_SCORECARDS
REFERENCING NEW AS n
FOR EACH ROW
    DECLARE indExistence PLS_INTEGER;
BEGIN
    SELECT 1 INTO indExistence FROM PROJECT_STATE_DEF
    WHERE PROJECT_ID = :n.PROJECT_ID AND ELEMENT_TYPE = :n.TYPE_ID AND STAGE_ID = :n.STAGE_ID AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        INSERT INTO PROJECT_STATE_TRG VALUES(:n.PROJECT_ID, 10, :n.TYPE_ID, :n.ELEMENT_ID, :n.STAGE_ID, '0', :n.REC_USER, NULL) ;
    END IF;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER SC_STAGE_INSERT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER SC_STAGE_AFTER
AFTER UPDATE OF STAGE_ID ON TMT_ASSIGNED_SCORECARDS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW
    DECLARE indExistence PLS_INTEGER;
BEGIN
    SELECT 1 INTO indExistence FROM PROJECT_STATE_DEF
    WHERE PROJECT_ID = :n.PROJECT_ID AND ELEMENT_TYPE = :n.TYPE_ID AND STAGE_ID = :n.STAGE_ID AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        INSERT INTO PROJECT_STATE_TRG VALUES(:n.PROJECT_ID, 10, :n.TYPE_ID, :n.ELEMENT_ID, :n.STAGE_ID, '0', :n.REC_USER, :n.REVISION_HISTORY) ;
    END IF;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER SC_STAGE_AFTER SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

CREATE OR REPLACE TRIGGER DOC_STAGE_INSERT
AFTER INSERT ON TMT_DOCUMENTS
REFERENCING NEW AS n
FOR EACH ROW
    DECLARE indExistence PLS_INTEGER;
BEGIN
    SELECT 1 INTO indExistence FROM PROJECT_STATE_DEF
    WHERE PROJECT_ID = :n.PROJECT_ID
    AND ELEMENT_TYPE = CASE WHEN :n.TYPE_ID IN (14,61,62,63,140) THEN 14 WHEN :n.TYPE_ID = 242 THEN 242 END
    AND STAGE_ID = :n.STAGE_ID AND :n.LINK_DOC <> 'Y' AND ROWNUM = 1;
    IF SQLCODE=0 THEN
        INSERT INTO PROJECT_STATE_TRG VALUES(:n.PROJECT_ID, 4, :n.TYPE_ID, :n.ELEMENT_ID, :n.STAGE_ID, '0', :n.REC_USER, NULL) ;
    END IF;
    PMO_DB2M.SC_STATE := SQLCODE;
EXCEPTION
    WHEN OTHERS THEN PMO_DB2M.SC_STATE := SQLCODE;
        lfile.fput( 'AllTrace', 'TRIGGER DOC_STAGE_INSERT SQLCODE:' || SQLCODE || ' SQLERRM:' || SubStr(SQLERRM, 1, 100));
END;
/

