-- already in stdafx.h : #include <math.h>

/*************************************************************************************************/

-- the following function returns the type_id that used as a group type for different sets of elements

-- according to the following list

--

-- Group Type                  set of type_id

----------------------------------------------------------------------------------------------------------------

-- SC_PROJECT              --> SC_PROJECT, SC_PROPOSAL, SC_PROJ_TYPE1, SC_PROJ_TYPE2, SC_PROJ_TYPE3

-- SC_DELIVERABLE          --> SC_DELIVREABLE, SC_WORK_PRODUCT, SC_SUMMARY_TASK, SC_WBS_CHANGE_REQUEST, SC_WBS_ACTION_ITEM, SC_WBS_RISK, SC_WBS_ISSUE, SC_WBS_REQUIREMENT, SC_WBS_DEFECT, SC_WBS_SERVICE_REQUEST

-- SC_TASK                 --> SC_TASK, SC_MILESTONE, SC_WBS_CHANGE_REQUEST_TASK, SC_WBS_ACTION_ITEM_TASK, SC_WBS_RISK_TASK, SC_WBS_ISSUE_TASK, SC_WBS_REQUIREMENT_TASK, SC_WBS_DEFECT_TASK, SC_WBS_SERVICE_REQUEST_TASK

-- SC_CRI_CHANGE_REQUEST   --> SC_CRI_CHANGE_REQUEST, SC_CRI_CHANGE_REQUEST_TASK

-- SC_CRI_ACTION_ITEM      --> SC_CRI_ACTION_ITEM, SC_CRI_ACTION_ITEM_TASK

-- SC_CRI_RISK             --> SC_CRI_RISK, SC_CRI_RISK_TASK

-- SC_CRI_ISSUE            --> SC_CRI_ISSUE, SC_CRI_ISSUE_TASK

-- SC_CRI_REQUIREMENT      --> SC_CRI_REQUIREMENT, SC_CRI_REQUIREMENT_TASK

-- SC_CRI_DEFECT           --> SC_CRI_DEFECT, SC_CRI_DEFECT_TASK

-- SC_CRI_SERVICE_REQUEST  --> SC_CRI_SERVICE_REQUEST_TASK

-- SC_DOCUMENT             --> SC_WORK_INPUT, SC_WP_LINK, SC_SOFTWARE, SC_LINK, SC_

Prompt == FUNCTION FC_GET_STATE_GROUP_TYPE ==
CREATE OR REPLACE FUNCTION FC_GET_STATE_GROUP_TYPE
(
     sh_h_type_id PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := Coalesce(sh_h_type_id, 0);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_STATE_GROUP_TYPE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_STATE_GROUP_TYPE'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_STATE_GROUP_TYPE(' || 
       COALESCE(TO_CHAR(sh_h_type_id), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 13+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_STATE_GROUP_TYPE(14) : alerts.sql(51) : ' || SYSTIMESTAMP);
   END IF; 

   -- project group type
   If isProjectType(h_type_id) <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 19+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(20) : alerts.sql(57) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_PROJECT;
   End If;
   
   -- deliverable group type
   If isWBSDeliv(h_type_id) <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 27+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(28) : alerts.sql(65) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_DELIVERABLE;
   End If;
   
   -- task group type
   If isWBSTask(h_type_id) <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 35+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(36) : alerts.sql(73) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_TASK;
   End If;
   
   -- change request group type
   If (h_type_id = PMO_CNST.SC_CRI_CHANGE_REQUEST) 
         OR (h_type_id = PMO_CNST.SC_CRI_CHANGE_REQUEST_TASK) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 44+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(45) : alerts.sql(82) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_CRI_CHANGE_REQUEST;
   End If;
   
   -- action item group type
   If (h_type_id = PMO_CNST.SC_CRI_ACTION_ITEM) OR (h_type_id = PMO_CNST.SC_CRI_ACTION_ITEM_TASK) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 52+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(53) : alerts.sql(90) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_CRI_ACTION_ITEM;
   End If;
   
   -- risk group type
   If (h_type_id = PMO_CNST.SC_CRI_RISK) OR (h_type_id = PMO_CNST.SC_CRI_RISK_TASK) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 60+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(61) : alerts.sql(98) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_CRI_RISK;
   End If;
   
   -- issue group type
   If (h_type_id = PMO_CNST.SC_CRI_ISSUE) OR (h_type_id = PMO_CNST.SC_CRI_ISSUE_TASK) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 68+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(69) : alerts.sql(106) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_CRI_ISSUE;
   End If;
   
   -- requirement group type
   If (h_type_id = PMO_CNST.SC_CRI_REQUIREMENT) OR (h_type_id = PMO_CNST.SC_CRI_REQUIREMENT_TASK) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 76+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(77) : alerts.sql(114) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_CRI_REQUIREMENT;
   End If;
   
   -- defect group type
   If (h_type_id = PMO_CNST.SC_CRI_DEFECT) OR (h_type_id = PMO_CNST.SC_CRI_DEFECT_TASK) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 84+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(85) : alerts.sql(122) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_CRI_DEFECT;
   End If;
   
   -- service request group type
   If (h_type_id = PMO_CNST.SC_CRI_SERVICE_REQUEST) 
         OR (h_type_id = PMO_CNST.SC_CRI_SERVICE_REQUEST_TASK) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 93+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(94) : alerts.sql(131) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_CRI_SERVICE_REQUEST;
   End If;
   
   -- service request group type
   If (h_type_id = PMO_CNST.SC_CRI_SERVICE_REQUEST) 
         OR (h_type_id = PMO_CNST.SC_CRI_SERVICE_REQUEST_TASK) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 102+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(103) : alerts.sql(140) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_CRI_SERVICE_REQUEST;
   End If;
   
   -- document group type
   If (h_type_id = PMO_CNST.SC_DOCUMENT) OR (h_type_id = PMO_CNST.SC_WORK_INPUT) 
         OR (h_type_id = PMO_CNST.SC_WP_LINK) OR (h_type_id = PMO_CNST.SC_SOFTWARE) 
         OR (h_type_id = PMO_CNST.SC_LINK) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 112+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(113) : alerts.sql(150) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_DOCUMENT;
   End If;
   
   If (h_type_id = PMO_CNST.SC_SCORECARD) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 119+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(120) : alerts.sql(157) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_SCORECARD;
   End If;
   
   If (h_type_id = PMO_CNST.SC_NOTE_MINUTES) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 126+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(127) : alerts.sql(164) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_NOTE_MINUTES;
   End If;
   
   If h_type_id = PMO_CNST.SC_ALERT_GROUP Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 133+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(134) : alerts.sql(171) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SC_ALERT_GROUP;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_STATE_GROUP_TYPE', 139+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_STATE_GROUP_TYPE(140) : alerts.sql(177) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN 0;
   -- default value
END;
/
SHOW ERRORS FUNCTION FC_GET_STATE_GROUP_TYPE;
ALTER FUNCTION FC_GET_STATE_GROUP_TYPE COMPILE DEBUG;

/***********************************************************************************************************/

Prompt == FUNCTION FC_PRJ_STATE_WKF ==
CREATE OR REPLACE FUNCTION FC_PRJ_STATE_WKF RETURN PLS_INTEGER
AS
   h_tmp PLS_INTEGER := 0;
   h_wkf_def_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_rm_id VARCHAR2(32);
   h_pm_id VARCHAR2(32);
   h_co_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_fname VARCHAR2(100);
   h_assign_flag VARCHAR2(1);
   h_null VARCHAR2(1);
   h_revision VARCHAR2(16);
   h_stage_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_participants VARCHAR2(4000);
   h_security_ids VARCHAR2(4000);
   h_group_ids VARCHAR2(4000);
   h_subject VARCHAR2(255);
   h_temp_id VARCHAR2(32);
   h_temp_id2 VARCHAR2(32);
   h_week_of DATE;
   h_temp_sc_state PLS_INTEGER := 0;
   h_message VARCHAR2(16384);
   stmt VARCHAR2(4000);
   h_state_group_type_id PLS_INTEGER := 0;
   h_rev_history VARCHAR2(2048);
   h_description VARCHAR2(4000);
   result PLS_INTEGER;
-- Declare Cursors
      CURSOR C_NOTIF IS
         SELECT PROJECT_ID, TABLE_TYPE, ELEMENT_TYPE, ELEMENT_ID, STAGE_ID, MESSAGE,
         CASE WHEN REC_USER IS NOT NuLL THEN REC_USER ELSE 'PMO_SUPERVISOR' END
         FROM PROJECT_STATE_TRG
         WHERE NOTIFIED_FLAG = '1'; -- FOR UPDATE OF NOTIFIED_FLAG

      -- Dynamic Cursor Declaration
      C_ASSIGN PMO_DB2M.REF_CURSOR;

      CURSOR C_H_WBS IS
         SELECT * FROM (SELECT RTRIM(TO_CHAR(MAJOR_REVISION)) || '.' || RTRIM(TO_CHAR(MINOR_REVISION))
         FROM HISTORY_WBS
         WHERE ELEMENT_ID = h_id AND TYPE_ID = h_type_id AND STAGE_ID = h_stage_id
         ORDER BY REC_DATETIME DESC ) WHERE ROWNUM<=1;

      CURSOR C_H_CRI IS
         SELECT * FROM (SELECT RTRIM(TO_CHAR(MAJOR_REVISION)) || '.' || RTRIM(TO_CHAR(MINOR_REVISION))
         FROM HISTORY_CRI
         WHERE ELEMENT_ID = h_id AND TYPE_ID = h_type_id AND STAGE_ID = h_stage_id
         ORDER BY REC_DATETIME DESC ) WHERE ROWNUM<=1;

      CURSOR C_H_DOC IS
         SELECT * FROM (SELECT RTRIM(TO_CHAR(MAJOR_REVISION)) || '.' || RTRIM(TO_CHAR(MINOR_REVISION))
         FROM HISTORY_DOCUMENTS
         WHERE ELEMENT_ID = h_id AND TYPE_ID = h_type_id AND STAGE_ID = h_stage_id
         ORDER BY REC_DATETIME DESC ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_PRJ_STATE_WKF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_PRJ_STATE_WKF'), '-- ' || SYSTIMESTAMP, ':r := FC_PRJ_STATE_WKF();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_PRJ_STATE_WKF', 69+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_PRJ_STATE_WKF(70) : alerts.sql(258) : ' || SYSTIMESTAMP);
   END IF; 

   h_null := NULL;
   h_rev_history := NULL;
   h_rec_user := 'PMO_SUPERVISOR';
   h_revision := '0.1';
   
   BEGIN
      SELECT ELEMENT_ID INTO h_wkf_def_id
      FROM TMT_WORKFLOW_CYCLES
      WHERE PARENT_ID = 'NOTIFICATION';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 87+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(88) : alerts.sql(276) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE PROJECT_STATE_TRG SET NOTIFIED_FLAG = '1'
      WHERE NOTIFIED_FLAG = '0';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 105+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(106) : alerts.sql(294) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_NOTIF;
   
   LOOP
      FETCH C_NOTIF INTO h_project_id, h_table_type, h_type_id, h_id,
      h_stage_id, h_rev_history, h_rec_user;
      IF C_NOTIF%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      h_assign_flag := NULL;
      h_count := 0;
      h_uniqueID := '';
      --trace_printf("0_ h_table_type: %d, h_stage_id: %d, h_rec_user: %d:%s, h_id: %d:%s\nh_rev_history: %s\n",h_table_type,h_stage_id,h_rec_user_Ind,h_rec_user,h_id_Ind,h_id,h_rev_history); //trace_flush();
      
      If (Coalesce(h_stage_id, 0) = PMO_CNST.SC_APV_STATE_RM_APPROVED) OR (Coalesce(
             h_stage_id, 0) = PMO_CNST.SC_APV_STATE_PM_APPROVED) OR (Coalesce(
             h_stage_id, 0) = PMO_CNST.SC_APV_STATE_RM_REJECTED) OR (Coalesce(
             h_stage_id, 0) = PMO_CNST.SC_APV_STATE_PM_REJECTED) OR (Coalesce(
             h_stage_id, 0) = PMO_CNST.SC_APV_STATE_SUBMITTED) Then
         result := InStr(h_rev_history, PMO_CNST.SC_STAR_CHAR) - 1;
         
         --trace_printf("1_ result: %d,h_rev_history: %s\n",result,h_rev_history); //trace_flush();
         If result > 0 Then
            h_revision := SubStr(h_rev_history, 1, result);
            h_rev_history := SubStr(h_rev_history, result + 1 + 1, Coalesce(Length(h_rev_history), 0));
         End If;
         --trace_printf("3_ h_revision: %s, h_rev_history: %s\n", h_revision,h_rev_history); //trace_flush();
         
         result := InStr(h_rev_history, PMO_CNST.SC_STAR_CHAR) - 1;
         
         --trace_printf("2_ result: %d,h_rev_history: %s\n",result,h_rev_history); //trace_flush();
         If result > 0 Then
            h_week_of := TO_DATE(SubStr(SubStr(h_rev_history, 1, result),1, 10));
            h_message := SubStr(h_rev_history, result + 1 + 1, Coalesce(Length(h_rev_history), 0));
         End If;
         --trace_printf("4_ h_week_of: %s, h_message: %s\n", h_week_of,h_message); //trace_flush();
         
         h_subject := 'Week of ';
         h_subject := h_subject || h_week_of;
         
         If Coalesce(h_stage_id, 0) = PMO_CNST.SC_APV_STATE_SUBMITTED Then
            If Coalesce(h_table_type, 0) = 3 Then
               BEGIN
                  SELECT RA.RESOURCE_ID INTO h_rm_id
                  FROM TMT_POOLS TP, RESOURCE_ASSIGNMENTS RA
                  WHERE TP.ELEMENT_ID = h_id
                  AND TP.PARENT_ID = RA.ELEMENT_ID
                  AND RA.ELEMENT_TYPE = 11
                  AND BITAND(RA.PARTICIPANT,PMO_CNST.SC_RESOURCE_MANAGER) > 0;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 171+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(172) : alerts.sql(360) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  h_participants := h_rm_id;
               End If;
               
               BEGIN
                  SELECT RA.RESOURCE_ID INTO h_pm_id
                  FROM RESOURCE_ASSIGNMENTS RA
                  WHERE RA.ELEMENT_ID = h_project_id
                  AND RA.ELEMENT_TYPE IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
                  AND BITAND(RA.PARTICIPANT,PMO_CNST.SC_MANAGER) > 0;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 192+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(193) : alerts.sql(381) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  h_participants := h_participants || ',';
                  h_participants := h_participants || h_pm_id;
               End If;
               
               BEGIN
                  SELECT RA.RESOURCE_ID INTO h_co_id
                  FROM RESOURCE_ASSIGNMENTS RA
                  WHERE RA.ELEMENT_ID = h_project_id
                  AND RA.ELEMENT_TYPE IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
                  AND BITAND(RA.PARTICIPANT,PMO_CNST.SC_CO_MANAGER) > 0;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 214+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(215) : alerts.sql(403) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  h_participants := h_participants || ',';
                  h_participants := h_participants || h_co_id;
               End If;
            ElsIf Coalesce(h_table_type, 0) = 1 Then
               BEGIN
                  SELECT RA.RESOURCE_ID INTO h_pm_id
                  FROM RESOURCE_ASSIGNMENTS RA
                  WHERE RA.ELEMENT_ID = h_project_id
                  AND RA.ELEMENT_TYPE IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
                  AND BITAND(RA.PARTICIPANT,PMO_CNST.SC_MANAGER) > 0;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 236+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(237) : alerts.sql(425) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  h_participants := h_pm_id;
               End If;
               
               BEGIN
                  SELECT RA.RESOURCE_ID INTO h_co_id
                  FROM RESOURCE_ASSIGNMENTS RA
                  WHERE RA.ELEMENT_ID = h_project_id
                  AND RA.ELEMENT_TYPE IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
                  AND BITAND(RA.PARTICIPANT,PMO_CNST.SC_CO_MANAGER) > 0;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 257+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(258) : alerts.sql(446) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  h_participants := h_participants || ',';
                  h_participants := h_participants || h_co_id;
               End If;
            Else
               BEGIN
                  SELECT RA.RESOURCE_ID INTO h_rm_id
                  FROM TMT_POOLS TP, RESOURCE_ASSIGNMENTS RA
                  WHERE TP.ELEMENT_ID = h_id
                  AND TP.PARENT_ID = RA.ELEMENT_ID
                  AND RA.ELEMENT_TYPE = 11
                  AND BITAND(RA.PARTICIPANT, PMO_CNST.SC_RESOURCE_MANAGER) > 0;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 280+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(281) : alerts.sql(469) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  h_participants := h_rm_id;
               End If;
            End If;
            
            h_subject := h_subject || ' has been submitted by ';
            
            BEGIN
               SELECT FULL_NAME INTO h_fname FROM RESOURCES WHERE RESOURCE_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 300+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(301) : alerts.sql(489) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_subject := h_subject || h_fname;
            End If;
         Else
            h_participants := h_id;
            
            If (Coalesce(h_table_type, 0) > 1) AND (Coalesce(
                  h_stage_id, 0) = PMO_CNST.SC_APV_STATE_PM_APPROVED) Then
               BEGIN
                  SELECT RA.RESOURCE_ID INTO h_rm_id
                  FROM TMT_POOLS TP, RESOURCE_ASSIGNMENTS RA
                  WHERE TP.ELEMENT_ID = h_id
                  AND TP.PARENT_ID = RA.ELEMENT_ID
                  AND RA.ELEMENT_TYPE = 11
                  AND BITAND(RA.PARTICIPANT,PMO_CNST.SC_RESOURCE_MANAGER) > 0;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 326+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(327) : alerts.sql(515) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  If Coalesce(h_table_type, 0) = 3 Then
                     h_participants := h_participants || ',';
                     h_participants := h_participants || h_rm_id;
                  Else
                     h_participants := h_rm_id;
                  End If;
               End If;
            End If;
            
            If (Coalesce(h_stage_id, 0) = PMO_CNST.SC_APV_STATE_RM_APPROVED) OR (Coalesce(
                  h_stage_id, 0) = PMO_CNST.SC_APV_STATE_PM_APPROVED) Then
               h_subject := h_subject || ' has been approved by ';
            Else
               h_subject := h_subject || ' has been rejected by ';
            End If;
            
            BEGIN
               SELECT FULL_NAME
               INTO h_fname
               FROM RESOURCES
               WHERE RESOURCE_ID = h_rec_user;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 359+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(360) : alerts.sql(548) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_subject := h_subject || h_fname;
            End If;
         End If;
         
         h_table_type := 1;
         
         h_temp_sc_state := FC_A_NOTIFICATION(h_table_type, h_project_id, h_wkf_def_id, h_type_id, h_rec_user
                                     , h_uniqueID);
         
         --trace_printf("h_temp_sc_state: %d, h_uniqueID: %s, h_subject: %s\n", h_temp_sc_state,h_uniqueID,h_subject);
         If Coalesce(h_temp_sc_state, 0) = 0 Then
            h_temp_id := NULL;
            h_temp_id2 := NULL;
            
            h_temp_sc_state := FC_U_WORKFLOW(h_uniqueID, h_revision, h_count, h_count, h_null, h_count
                                 , h_subject, h_message, h_participants, h_group_ids, h_assign_flag
                                 , h_count, h_temp_id, h_temp_id2, h_rec_user);
            --trace_printf("h_temp_sc_state: %d, h_revision: %s, h_participants: %s\nh_message: %s\n", h_temp_sc_state,h_revision,h_participants,h_message);
         End If;
      Else
         h_subject := '';
         h_participants := '';
         
         --PROJECT_STATE_TRG table contains physical type_id for each element but PROJECT_STATE_DEF contains
         --the group type_id
         h_state_group_type_id := FC_GET_STATE_GROUP_TYPE(Coalesce(h_type_id, 0));
         
         BEGIN
            SELECT ASSIGN_FLAG, GROUP_IDS, PARTICIPANT_IDS, REPLACE(SECURITY_IDS,',',''','''),
            SUBJECT, MESSAGE
            INTO h_assign_flag, h_group_ids, h_participants, h_security_ids,
            h_subject, h_message
            FROM PROJECT_STATE_DEF
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_TYPE = h_state_group_type_id
            AND STAGE_ID = h_stage_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 407+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(408) : alerts.sql(596) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (SubStr(h_assign_flag, 1, 1) = '1') OR (h_security_ids IS NOT NULL) Then
            If h_participants IS NULL Then
               h_count := 0;
               h_participants := '';
            Else
               h_count := 1;
            End If;
            
            If (SubStr(h_assign_flag, 1, 1) = '1') AND (h_security_ids IS NOT NULL) Then
               stmt := 'SELECT RESOURCE_ID FROM RESOURCE_ASSIGNMENTS'
                      || ' WHERE PROJECT_ID = ''' || h_project_id || ''' AND ELEMENT_ID = ''' || h_id 
                           || ''' AND RESOURCE_TYPE = ''0'''
                      || ' UNION ALL'
                      || ' SELECT RA.RESOURCE_ID FROM RESOURCE_ASSIGNMENTS RA, TMT_GLOBAL_SECURITY GS'
                      || ' WHERE RA.PROJECT_ID = ''' || h_project_id || ''' AND RA.ELEMENT_ID IN (''' || h_project_id 
                           || ''',''' || h_id || ''') AND RA.RESOURCE_TYPE = ''0'''
                      || ' AND GS.ELEMENT_ID = RA.SECURITY_ID'
                      || ' AND (GS.ELEMENT_ID IN (''' || h_security_ids || ''') OR GS.PARENT_ID IN (''' 
                           || h_security_ids || '''))';
            ElsIf SubStr(h_assign_flag, 1, 1) = '1' Then
               stmt := 'SELECT RESOURCE_ID FROM RESOURCE_ASSIGNMENTS'
                      || ' WHERE PROJECT_ID = ''' || h_project_id || ''' AND ELEMENT_ID = ''' || h_id 
                           || ''' AND RESOURCE_TYPE = ''0''';
            Else
               stmt := 'SELECT RA.RESOURCE_ID FROM RESOURCE_ASSIGNMENTS RA, TMT_GLOBAL_SECURITY GS'
                      || ' WHERE RA.PROJECT_ID = ''' || h_project_id || ''' AND RA.ELEMENT_ID IN (''' || h_project_id 
                           || ''',''' || h_id || ''') AND RA.RESOURCE_TYPE = ''0'''
                      || ' AND GS.ELEMENT_ID = RA.SECURITY_ID'
                      || ' AND (GS.ELEMENT_ID IN (''' || h_security_ids || ''') OR GS.PARENT_ID IN (''' 
                           || h_security_ids || '''))';
            End If;
            
            OPEN C_ASSIGN FOR stmt;
            
            LOOP
               FETCH C_ASSIGN INTO h_res_id;
               IF C_ASSIGN%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
               
               If Coalesce(h_count, 0) > 0 Then
                  h_participants := h_participants || ',';
               End If;
               h_participants := h_participants || h_res_id;
               h_count := Coalesce(h_count, 0) + 1;
            END LOOP;
            
            BEGIN
               CLOSE C_ASSIGN ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 473+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(474) : alerts.sql(662) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         h_temp_sc_state := FC_A_NOTIFICATION(h_table_type, h_id, h_wkf_def_id, h_type_id, h_rec_user
                                     , h_uniqueID);
         
         If Coalesce(h_temp_sc_state, 0) = 0 Then
            h_temp_id := NULL;
            h_temp_id2 := NULL;
            
            If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS Then
               OPEN C_H_WBS;
               
               FETCH C_H_WBS INTO h_revision;
               IF C_H_WBS%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               BEGIN
                  CLOSE C_H_WBS ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 504+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(505) : alerts.sql(693) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_CRI Then
               OPEN C_H_CRI;
               
               FETCH C_H_CRI INTO h_revision;
               IF C_H_CRI%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               BEGIN
                  CLOSE C_H_CRI ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 526+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(527) : alerts.sql(715) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_DOC Then
               OPEN C_H_DOC;
               
               FETCH C_H_DOC INTO h_revision;
               IF C_H_DOC%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               BEGIN
                  CLOSE C_H_DOC ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 548+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(549) : alerts.sql(737) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If h_message IS NOT NULL Then
               If h_rev_history IS NOT NULL Then
                  h_description := h_message || Chr(10) || h_rev_history;
               Else
                  h_description := h_message;
               End If;
            Else
               If h_rev_history IS NOT NULL Then
                  h_description := h_rev_history;
               Else
                  h_description := '';
               End If;
            End If;
            h_message := h_description;
            
            h_temp_sc_state := FC_U_WORKFLOW(h_uniqueID, h_revision, h_count, h_count, h_null, h_count
                                 , h_subject, h_message, h_participants, h_group_ids, h_assign_flag
                                 , h_count, h_temp_id, h_temp_id2, h_rec_user);
         End If;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_NOTIF ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 584+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(585) : alerts.sql(773) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      DELETE FROM PROJECT_STATE_TRG WHERE NOTIFIED_FLAG = '1';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_PRJ_STATE_WKF', 601+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_PRJ_STATE_WKF(602) : alerts.sql(790) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_PRJ_STATE_WKF', 607+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_PRJ_STATE_WKF(608) : alerts.sql(796) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_PRJ_STATE_WKF;
ALTER FUNCTION FC_PRJ_STATE_WKF COMPILE DEBUG;

/************************************************************************/

Prompt == FUNCTION FC_ISSUE_NOTIFICATION ==
CREATE OR REPLACE FUNCTION FC_ISSUE_NOTIFICATION
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_id IN OUT NOCOPY VARCHAR2
   , h_alert_id IN OUT NOCOPY VARCHAR2
   , sh_h_table_type PLS_INTEGER
   , sh_h_type_id PLS_INTEGER
   , sh_h_variance PLS_INTEGER
   , h_assign_flag IN OUT NOCOPY VARCHAR2
   , sh_h_stage_id PLS_INTEGER
   , h_subject IN OUT NOCOPY VARCHAR2
   , h_message IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_table_type PLS_INTEGER := sh_h_table_type;
   h_type_id PLS_INTEGER := sh_h_type_id;
   h_variance PLS_INTEGER := sh_h_variance;
   h_stage_id PLS_INTEGER := sh_h_stage_id;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_element_id VARCHAR2(32);
   h_wkf_def_id VARCHAR2(32);
   h_temp_id VARCHAR2(32);
   h_temp_id2 VARCHAR2(32);
   h_count PLS_INTEGER := 0;
   stmt VARCHAR2(4000);
   h_res_id VARCHAR2(32);
   h_revision VARCHAR2(16);
   h_rev_history VARCHAR2(2048);
   h_description VARCHAR2(4000);
   h_null VARCHAR2(1);
   h_rec_user VARCHAR2(32);
   h_temp_sc_state PLS_INTEGER;
   h_temp_length PLS_INTEGER;
   h_participant_ids VARCHAR2(4000);
   h_full_participant_ids VARCHAR2(4000);
   h_flag_no_alert PLS_INTEGER := 0;
   h_group_ids VARCHAR2(2048);
-- Declare Cursors
      CURSOR C_ASSIGN2 IS
         SELECT RESOURCE_ID
         FROM PMO_ALERT_CONTACT_LIST
         WHERE PROJECT_ID = h_project_id
         AND ELEMENT_ID = h_id
         AND ALERT_ID = h_alert_id;

      CURSOR C_WF_SENDER IS
         SELECT COALESCE(REC_USER, 'PMO_SUPERVISOR')
         FROM ALERTS
         WHERE PROJECT_ID = h_project_id
         AND ELEMENT_ID = h_id
         AND ALERT_ID = h_alert_id
         AND (ALERT_ID NOT IN ('SCCOSTVARIANCE', 'SCSCHEDULEVARIANCE') OR VARIANCE >= h_variance);

      CURSOR C_H_WBS2 IS
         SELECT * FROM (SELECT RTRIM(TO_CHAR(MAJOR_REVISION)) || '.' || RTRIM(TO_CHAR(MINOR_REVISION))
         FROM HISTORY_WBS
         WHERE ELEMENT_ID = h_id AND TYPE_ID = h_type_id AND STAGE_ID = h_stage_id
         ORDER BY REC_DATETIME DESC ) WHERE ROWNUM<=1;

      CURSOR C_H_CRI2 IS
         SELECT * FROM (SELECT RTRIM(TO_CHAR(MAJOR_REVISION)) || '.' || RTRIM(TO_CHAR(MINOR_REVISION))
         FROM HISTORY_CRI
         WHERE ELEMENT_ID = h_id AND TYPE_ID = h_type_id AND STAGE_ID = h_stage_id
         ORDER BY REC_DATETIME DESC ) WHERE ROWNUM<=1;

      CURSOR C_H_DOC2 IS
         SELECT * FROM (SELECT RTRIM(TO_CHAR(MAJOR_REVISION)) || '.' || RTRIM(TO_CHAR(MINOR_REVISION))
         FROM HISTORY_DOCUMENTS
         WHERE ELEMENT_ID = h_id AND TYPE_ID = h_type_id AND STAGE_ID = h_stage_id
         ORDER BY REC_DATETIME DESC ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ISSUE_NOTIFICATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ISSUE_NOTIFICATION'), '-- ' || SYSTIMESTAMP, ':r := FC_ISSUE_NOTIFICATION(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_id || ''''
          || ', ' || '''' || h_alert_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_table_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type_id), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_variance), 'NULL')
          || ', ' || '''' || h_assign_flag || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_stage_id), 'NULL')
          || ', ' || '''' || h_subject || ''''
          || ', ' || '''' || h_message || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ISSUE_NOTIFICATION', 89+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ISSUE_NOTIFICATION(90) : alerts.sql(896) : ' || SYSTIMESTAMP);
   END IF; 

   h_null := NULL;
   h_rev_history := NULL;
   h_revision := '0.1';
   h_group_ids := NULL;
   
   BEGIN
      SELECT ELEMENT_ID INTO h_wkf_def_id
      FROM TMT_WORKFLOW_CYCLES
      WHERE PARENT_ID = 'NOTIFICATION';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_NOTIFICATION', 107+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_NOTIFICATION(108) : alerts.sql(914) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_ASSIGN2;
   
   h_count := 0;
   h_full_participant_ids := '';
   h_flag_no_alert := -2;
   
   LOOP
      FETCH C_ASSIGN2 INTO h_res_id;
      IF C_ASSIGN2%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      If Coalesce(h_count, 0) > 0 Then
         h_full_participant_ids := h_full_participant_ids || ',';
      End If;
      h_full_participant_ids := h_full_participant_ids || h_res_id;
      h_count := Coalesce(h_count, 0) + 1;
   END LOOP;
   
   BEGIN
      CLOSE C_ASSIGN2 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_NOTIFICATION', 145+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_NOTIFICATION(146) : alerts.sql(952) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("6 PROJECT_ID: %s,ALERT_ID: %s,ELEMENT_ID: %s,h_participant_ids: %s\n", h_project_id,h_alert_id,h_id,h_full_participant_ids);
   
   OPEN C_WF_SENDER;
   
   LOOP
      FETCH C_WF_SENDER INTO h_rec_user;
      IF C_WF_SENDER%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      h_element_id := '';
      
      h_temp_sc_state := FC_A_NOTIFICATION(h_table_type, h_id, h_wkf_def_id, h_type_id, h_rec_user
                                     , h_element_id);
      
      If h_temp_sc_state = 0 Then
         h_temp_id := NULL;
         h_temp_id2 := NULL;
         h_participant_ids := h_full_participant_ids;
         
         If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS Then
            OPEN C_H_WBS2;
            
            FETCH C_H_WBS2 INTO h_revision;
            IF C_H_WBS2%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            BEGIN
               CLOSE C_H_WBS2 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_NOTIFICATION', 193+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_NOTIFICATION(194) : alerts.sql(1000) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_CRI Then
            OPEN C_H_CRI2;
            
            FETCH C_H_CRI2 INTO h_revision;
            IF C_H_CRI2%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            BEGIN
               CLOSE C_H_CRI2 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_NOTIFICATION', 215+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_NOTIFICATION(216) : alerts.sql(1022) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_DOC Then
            OPEN C_H_DOC2;
            
            FETCH C_H_DOC2 INTO h_revision;
            IF C_H_DOC2%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            BEGIN
               CLOSE C_H_DOC2 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_NOTIFICATION', 237+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_NOTIFICATION(238) : alerts.sql(1044) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If h_message IS NOT NULL Then
            If h_rev_history IS NOT NULL Then
               h_description := h_message || Chr(10) || h_rev_history;
            Else
               h_description := h_message;
            End If;
         Else
            If h_rev_history IS NOT NULL Then
               h_description := h_rev_history;
            Else
               h_description := '';
            End If;
         End If;
         h_message := h_description;
         
         --remove sender from receiver list
         BEGIN
            SELECT REPLACE(PMO_ORASTD.StringConcat(h_participant_ids, ','),PMO_ORASTD.StringConcat(h_rec_user, ','),'') INTO h_participant_ids FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_NOTIFICATION', 267+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_NOTIFICATION(268) : alerts.sql(1074) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_temp_length := Coalesce(Length(h_participant_ids), 0);
         
         If (h_participant_ids IS NOT NULL) AND (h_temp_length > 1) Then
            h_participant_ids := SubStr(h_participant_ids, 1, h_temp_length - 1);
            --trace_printf("7 Sender_ID: %s,h_participant_ids: %s\n", h_rec_user,h_participant_ids);
            
            h_temp_sc_state := FC_U_WORKFLOW(h_element_id, h_revision, h_count, h_count, h_null, h_count
                                 , h_subject, h_message, h_participant_ids, h_group_ids
                                 , h_assign_flag, h_flag_no_alert, h_temp_id, h_temp_id2
                                 , h_rec_user);
         End If;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_WF_SENDER ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_NOTIFICATION', 294+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_NOTIFICATION(295) : alerts.sql(1101) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ISSUE_NOTIFICATION', 300+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ISSUE_NOTIFICATION(301) : alerts.sql(1107) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ISSUE_NOTIFICATION;
ALTER FUNCTION FC_ISSUE_NOTIFICATION COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION FC_BUILD_ALERT_CONTACT_LIST ==
CREATE OR REPLACE FUNCTION FC_BUILD_ALERT_CONTACT_LIST RETURN PLS_INTEGER
AS
   h_alert_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_email_address VARCHAR2(100);
   h_logon_name VARCHAR2(100);
   h_group_id VARCHAR2(32);
   h_participant_ids VARCHAR2(4000);
   h_security_ids VARCHAR2(2048);
   h_group_ids VARCHAR2(4000);
   h_assign_flag VARCHAR2(1);
   h_alert_flag VARCHAR2(1);
   h_length PLS_INTEGER := 0;
   h_loc PLS_INTEGER := 0;
   h_variance PLS_INTEGER := 0;
   h_resource_cnt PLS_INTEGER := 0;
   h_rank VARCHAR2(200);
   h_language_iso_code VARCHAR2(50);
   h_element_id VARCHAR2(32);
   h_tmp PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_stage_id PLS_INTEGER := 0;
   h_subject VARCHAR2(255);
   h_message VARCHAR2(16384);
   h_to_resource_id VARCHAR2(32);
   myStmt VARCHAR2(4000);
   null_return LONG;
-- Declare Cursors
      CURSOR target_resources IS
         SELECT DISTINCT A.PROJECT_ID, A.ELEMENT_ID, A.ALERT_ID, PS.PARTICIPANT_IDS, PS.SECURITY_IDS,
         PS.GROUP_IDS, PS.ASSIGN_FLAG, PS.VARIANCE, PS.STAGE_ID, COALESCE(PS.SUBJECT, AM.ALERT_NAME), PS.MESSAGE, A.FLAG, A.RESOURCE_ID
         FROM ALERTS A, ALERT_MESSAGES AM, PROJECT_STATE_DEF PS
         WHERE AM.ALERT_ID = A.ALERT_ID AND AM.LANGUAGE_CODE = 'EN'
         AND AM.ALERT_BIT = 1
         AND A.FLAG IN ('U','D')
         AND PS.STAGE_ID = AM.TYPE_ID AND PS.PROJECT_ID = A.PROJECT_ID
         AND (A.ALERT_ID NOT IN ('SCCOSTVARIANCE', 'SCSCHEDULEVARIANCE') OR A.VARIANCE >= PS.VARIANCE);

      CURSOR group_resources IS
         SELECT DISTINCT R.RESOURCE_ID, R.E_MAIL_ADDRESS, R.LOGON_NAME, R.LANGUAGE_ISO_CODE
         FROM TMT_CONTACTS T JOIN RESOURCES R ON T.ELEMENT_ID = R.RESOURCE_ID
         WHERE T.TYPE_ID = PMO_CNST.SC_RESOURCE AND RANK LIKE h_rank || '%';

      CURSOR role_resources IS
         SELECT DISTINCT R.RESOURCE_ID, R.E_MAIL_ADDRESS, R.LOGON_NAME, R.LANGUAGE_ISO_CODE
         FROM RESOURCE_ASSIGNMENTS RA, TMT_WBS W, TMT_WBS WP, RESOURCES R
         WHERE RA.PROJECT_ID = h_project_id
         AND WP.ELEMENT_ID = h_element_id
         AND RA.ELEMENT_ID = W.ELEMENT_ID AND RA.RESOURCE_ID = R.RESOURCE_ID
         AND ((
         h_alert_id IN ('SCNEWPROJECTPUBLISHED', 'SCNEWTEMPLATE') AND
         SUBSTR(W.RANK, 1, LENGTH(WP.RANK)) = WP.RANK
         )
         OR
         (
         W.RANK >= SUBSTR(WP.RANK, 1, 8) AND W.RANK <= WP.RANK AND
         SUBSTR(WP.RANK, 1, CASE WHEN LENGTH(W.RANK)>LENGTH(WP.RANK) THEN 1 ELSE LENGTH(W.RANK) END) = W.RANK
         )
         )
         AND PMO_ORASTD.LOCATE(PMO_ORASTD.StringConcat(RTRIM(RA.SECURITY_ID), ','), PMO_ORASTD.StringConcat(h_security_ids, ',')) > 0;

      CURSOR assigned_resources IS
         SELECT DISTINCT R.RESOURCE_ID, R.E_MAIL_ADDRESS, R.LOGON_NAME, R.LANGUAGE_ISO_CODE
         FROM RESOURCE_ASSIGNMENTS RA, RESOURCES R
         WHERE RA.PROJECT_ID = h_project_id
         AND (RA.ELEMENT_ID = h_element_id OR h_alert_id = 'SCNEWPROJECTPUBLISHED')
         AND RA.RESOURCE_ID = R.RESOURCE_ID
         AND RESOURCE_TYPE = '0';

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_BUILD_ALERT_CONTACT_LIST') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_BUILD_ALERT_CONTACT_LIST'), '-- ' || SYSTIMESTAMP, ':r := FC_BUILD_ALERT_CONTACT_LIST();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_BUILD_ALERT_CONTACT_LIST', 77+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_BUILD_ALERT_CONTACT_LIST(78) : alerts.sql(1195) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_ALERT_CONTACT_LIST';
   PMO_DB2M.SC_STATE := 0;
   
   OPEN target_resources;
   
   LOOP
      FETCH target_resources INTO
      h_project_id, h_element_id, h_alert_id, h_participant_ids, h_security_ids,
      h_group_ids, h_assign_flag, h_variance, h_stage_id, h_subject, h_message,
      h_alert_flag, h_to_resource_id;
      IF target_resources%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      -- get resources in h_participant_ids (alerts 'SCSCHEDULEASSIGNMENTADD' and 'SCSCHEDULEASSIGNMENTREMOVE' are special)
      If h_participant_ids IS NOT NULL AND (PMO_ORASTD.strncmp(h_alert_id, 'SCSCHEDULEASSIGNMENT'
            , 20) <> 0) Then
         --trace_printf("h_participant_ids: %s\n", h_participant_ids);
         h_length := Coalesce(Length(h_participant_ids), 0);
         h_loc := 1;
         
         WHILE Coalesce(h_loc, 0) < Coalesce(h_length, 0) LOOP
            BEGIN
               SELECT SUBSTR(h_participant_ids, h_loc, 32)
               INTO h_resource_id
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 119+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(120) : alerts.sql(1237) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_loc := Coalesce(h_loc, 0) + 33;
            --trace_printf("1 PROJECT_ID: %s,ALERT_ID: %s,ELEMENT_ID: %s,RESOURCE_ID: %s,VARIANCE: %d\n", h_project_id,h_alert_id,h_element_id,h_resource_id,h_variance);
            
            If PMO_ORASTD.strncmp(h_alert_id, 'SCPERSONALCALENDARCHANGED', 25) = 0 Then
               h_element_id := h_project_id;
            End If;
            
            h_resource_cnt := 0;
            BEGIN
               SELECT COUNT(*) INTO h_resource_cnt
               FROM PMO_ALERT_CONTACT_LIST
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_element_id
               AND ALERT_ID = h_alert_id
               AND RESOURCE_ID = h_resource_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 145+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(146) : alerts.sql(1263) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_resource_cnt, 0) = 0 Then
               BEGIN
                  INSERT INTO PMO_ALERT_CONTACT_LIST(PROJECT_ID, ALERT_ID, ELEMENT_ID, RESOURCE_ID, EMAIL_ADDRESS, LOGON_NAME, LANGUAGE_ISO_CODE, SUBJECT, VARIANCE)
                  SELECT h_project_id, h_alert_id, h_element_id, RESOURCE_ID, E_MAIL_ADDRESS, LOGON_NAME, LANGUAGE_ISO_CODE, h_subject, h_variance
                  FROM RESOURCES
                  WHERE RESOURCE_ID = h_resource_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 166+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(167) : alerts.sql(1284) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         END LOOP;
      End If;
      
      --trace_printf("\n h_group_ids: %s", h_group_ids);
      
      -- get resources under a group (alerts 'SCSCHEDULEASSIGNMENTADD' and 'SCSCHEDULEASSIGNMENTREMOVE' are special)
      If h_group_ids IS NOT NULL AND (PMO_ORASTD.strncmp(h_alert_id, 'SCSCHEDULEASSIGNMENT'
            , 20) <> 0) Then
         h_length := Coalesce(Length(h_group_ids), 0);
         h_loc := 1;
         WHILE Coalesce(h_loc, 0) < Coalesce(h_length, 0) LOOP
            BEGIN
               SELECT SUBSTR(h_group_ids, h_loc, 32)
               INTO h_group_id
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 192+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(193) : alerts.sql(1310) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_loc := Coalesce(h_loc, 0) + 33;
            
            BEGIN
               SELECT RANK
               INTO h_rank
               FROM TMT_CONTACTS
               WHERE ELEMENT_ID = h_group_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 210+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(211) : alerts.sql(1328) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            OPEN group_resources;
            
            LOOP
               FETCH group_resources INTO h_resource_id, h_email_address, h_logon_name, h_language_iso_code;
               IF group_resources%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If PMO_DB2M.SC_STATE <> 0 Then
                  EXIT;
               End If;
               
               If PMO_ORASTD.strncmp(h_alert_id, 'SCPERSONALCALENDARCHANGED', 25) = 0 Then
                  h_element_id := h_project_id;
               End If;
               
               h_resource_cnt := 0;
               BEGIN
                  SELECT COUNT(*) INTO h_resource_cnt
                  FROM PMO_ALERT_CONTACT_LIST
                  WHERE PROJECT_ID = h_project_id
                  AND ELEMENT_ID = h_element_id
                  AND ALERT_ID = h_alert_id
                  AND RESOURCE_ID = h_resource_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 247+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(248) : alerts.sql(1365) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("2 PROJECT_ID: %s,ALERT_ID: %s,ELEMENT_ID: %s,RESOURCE_ID: %s,VARIANCE: %d\n", h_project_id,h_alert_id,h_element_id,h_resource_id,h_variance);
               
               If Coalesce(h_resource_cnt, 0) = 0 Then
                  BEGIN
                     INSERT INTO PMO_ALERT_CONTACT_LIST(PROJECT_ID, ELEMENT_ID, ALERT_ID, RESOURCE_ID, EMAIL_ADDRESS, LOGON_NAME, LANGUAGE_ISO_CODE, SUBJECT, VARIANCE)
                     VALUES( h_project_id, h_element_id, h_alert_id, h_resource_id, h_email_address, h_logon_name, h_language_iso_code, h_subject, h_variance);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 267+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(268) : alerts.sql(1385) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            END LOOP;
            
            BEGIN
               CLOSE group_resources ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 282+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(283) : alerts.sql(1400) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         END LOOP;
      End If;
      
      --trace_printf("\n h_security_ids: %s", h_security_ids);
      -- get resources under roles (alerts 'SCSCHEDULEASSIGNMENTADD' and 'SCSCHEDULEASSIGNMENTREMOVE' are special)
      If h_security_ids IS NOT NULL AND (PMO_ORASTD.strncmp(h_alert_id, 'SCSCHEDULEASSIGNMENT'
            , 20) <> 0) Then
         OPEN role_resources;
         
         LOOP
            FETCH role_resources  INTO h_resource_id, h_email_address, h_logon_name, h_language_iso_code;
            IF role_resources%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE <> 0 Then
               EXIT;
            End If;
            
            If PMO_ORASTD.strncmp(h_alert_id, 'SCPERSONALCALENDARCHANGED', 25) = 0 Then
               h_element_id := h_project_id;
            End If;
            
            h_resource_cnt := 0;
            BEGIN
               SELECT COUNT(*) INTO h_resource_cnt
               FROM PMO_ALERT_CONTACT_LIST
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_element_id
               AND ALERT_ID = h_alert_id
               AND RESOURCE_ID = h_resource_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 325+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(326) : alerts.sql(1443) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --trace_printf("3 PROJECT_ID: %s,ALERT_ID: %s,ELEMENT_ID: %s,RESOURCE_ID: %s,VARIANCE: %d\n", h_project_id,h_alert_id,h_element_id,h_resource_id,h_variance);
            If Coalesce(h_resource_cnt, 0) = 0 Then
               BEGIN
                  INSERT INTO PMO_ALERT_CONTACT_LIST(PROJECT_ID, ELEMENT_ID, ALERT_ID, RESOURCE_ID, EMAIL_ADDRESS, LOGON_NAME, LANGUAGE_ISO_CODE, SUBJECT, VARIANCE)
                  VALUES( h_project_id, h_element_id, h_alert_id, h_resource_id, h_email_address, h_logon_name, h_language_iso_code, h_subject, h_variance);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 345+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(346) : alerts.sql(1463) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         END LOOP;
         
         BEGIN
            CLOSE role_resources  ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 360+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(361) : alerts.sql(1478) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      -- get resources assigned to the WBS element (alerts 'SCSCHEDULEASSIGNMENTADD' and 'SCSCHEDULEASSIGNMENTREMOVE' are special)
      If SubStr(h_assign_flag, 1, 1) = '1' Then
         If PMO_ORASTD.strncmp(h_alert_id, 'SCSCHEDULEASSIGNMENT', 20) <> 0 Then
            OPEN assigned_resources;
            
            LOOP
               FETCH assigned_resources  INTO h_resource_id, h_email_address, h_logon_name, h_language_iso_code;
               IF assigned_resources%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If PMO_DB2M.SC_STATE <> 0 Then
                  EXIT;
               End If;
               
               If PMO_ORASTD.strncmp(h_alert_id, 'SCPERSONALCALENDARCHANGED', 25) = 0 Then
                  h_element_id := h_project_id;
               End If;
               
               h_resource_cnt := 0;
               BEGIN
                  SELECT COUNT(*) INTO h_resource_cnt
                  FROM PMO_ALERT_CONTACT_LIST
                  WHERE PROJECT_ID = h_project_id
                  AND ELEMENT_ID = h_element_id
                  AND ALERT_ID = h_alert_id
                  AND RESOURCE_ID = h_resource_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 401+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(402) : alerts.sql(1519) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --trace_printf("4.1 PROJECT_ID: %s,ALERT_ID: %s,ELEMENT_ID: %s,RESOURCE_ID: %s,VARIANCE: %d\n", h_project_id,h_alert_id,h_element_id,h_resource_id,h_variance);
               If Coalesce(h_resource_cnt, 0) = 0 Then
                  BEGIN
                     INSERT INTO PMO_ALERT_CONTACT_LIST(PROJECT_ID, ELEMENT_ID, ALERT_ID, RESOURCE_ID, EMAIL_ADDRESS, LOGON_NAME, LANGUAGE_ISO_CODE, SUBJECT, VARIANCE)
                     VALUES( h_project_id, h_element_id, h_alert_id, h_resource_id, h_email_address, h_logon_name, h_language_iso_code, h_subject, h_variance);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 421+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(422) : alerts.sql(1539) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            END LOOP;
            
            BEGIN
               CLOSE assigned_resources  ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 436+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(437) : alerts.sql(1554) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            --trace_printf("4.2 PROJECT_ID: %s,ALERT_ID: %s,ELEMENT_ID: %s,RESOURCE_ID: %s,VARIANCE: %d\n", h_project_id,h_alert_id,h_element_id,h_to_resource_id,h_variance);
            BEGIN
               INSERT INTO PMO_ALERT_CONTACT_LIST(PROJECT_ID, ALERT_ID, ELEMENT_ID, RESOURCE_ID, EMAIL_ADDRESS, LOGON_NAME, LANGUAGE_ISO_CODE, SUBJECT, VARIANCE)
               SELECT h_project_id, h_alert_id, h_element_id, RESOURCE_ID, E_MAIL_ADDRESS, LOGON_NAME, LANGUAGE_ISO_CODE, h_subject, h_variance
               FROM RESOURCES
               WHERE RESOURCE_ID = h_to_resource_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 457+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(458) : alerts.sql(1575) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      If SubStr(h_alert_flag, 1, 1) = 'U' AND (PMO_ORASTD.strncmp(h_alert_id, 'SCPERSONALCALENDARCHANGED'
            , 25) <> 0) Then
         h_table_type := 1;
         h_type_id := PMO_CNST.SC_ALERT_GROUP;
         h_tmp := 1;
         --trace_printf("5 PROJECT_ID: %s,ALERT_ID: %s,ELEMENT_ID: %s\n", h_project_id,h_alert_id,h_element_id);
         
         null_return := FC_ISSUE_NOTIFICATION(h_project_id, h_element_id, h_alert_id, h_table_type, h_type_id
                                     , h_variance, h_assign_flag, h_stage_id, h_subject, h_message
                                     , h_tmp);
      End If;
   END LOOP;
   
   BEGIN
      CLOSE target_resources;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_CONTACT_LIST', 485+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_CONTACT_LIST(486) : alerts.sql(1603) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_BUILD_ALERT_CONTACT_LIST', 491+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_BUILD_ALERT_CONTACT_LIST(492) : alerts.sql(1609) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_BUILD_ALERT_CONTACT_LIST;
ALTER FUNCTION FC_BUILD_ALERT_CONTACT_LIST COMPILE DEBUG;

/*****************************************************************************************/

-- issue notification for alert 'SCRESOURCERECORDREAPPROVAL' (parent_table 'RESOURCES')

/*****************************************************************************************/

Prompt == FUNCTION FC_ISSUE_RESOURCE_NOTIFICATION ==
CREATE OR REPLACE FUNCTION FC_ISSUE_RESOURCE_NOTIFICATION RETURN PLS_INTEGER
AS
   h_alert_id VARCHAR2(32);
   h_element_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_assign_flag VARCHAR2(1);
   h_count PLS_INTEGER := 0;
   h_tmp PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_subject VARCHAR2(255);
   h_message VARCHAR2(16384);
   h_wkf_def_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_temp_sc_state PLS_INTEGER;
   h_flag_no_alert PLS_INTEGER := 0;
   h_revision VARCHAR2(16);
   h_temp_id VARCHAR2(32);
-- Declare Cursors
      CURSOR notify_list IS
         SELECT DISTINCT A.ELEMENT_ID, A.REC_USER, A.ALERT_ID, AM.ALERT_NAME, AM.ALERT_NAME
         FROM ALERTS A, ALERT_MESSAGES AM
         WHERE AM.ALERT_ID = A.ALERT_ID AND AM.LANGUAGE_CODE = 'EN'
         AND AM.ALERT_BIT = 1
         AND A.FLAG = 'U'
         AND A.ALERT_ID= 'SCRESOURCERECORDREAPPROVAL';

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ISSUE_RESOURCE_NOTIFICATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ISSUE_RESOURCE_NOTIFICATION'), '-- ' || SYSTIMESTAMP, ':r := FC_ISSUE_RESOURCE_NOTIFICATION();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ISSUE_RESOURCE_NOTIFICATION', 33+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ISSUE_RESOURCE_NOTIFICATION(34) : alerts.sql(1657) : ' || SYSTIMESTAMP);
   END IF; 

   h_table_type := PMO_CNST.SC_TABLE_RES;
   h_type_id := PMO_CNST.SC_ALERT_GROUP;
   h_count := 1;
   h_assign_flag := NULL;
   h_tmp := 2;
   h_flag_no_alert := -2;
   h_revision := '0.1';
   h_temp_id := NULL;
   
   BEGIN
      SELECT ELEMENT_ID INTO h_wkf_def_id
      FROM TMT_WORKFLOW_CYCLES
      WHERE PARENT_ID = 'NOTIFICATION';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_RESOURCE_NOTIFICATION', 55+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_RESOURCE_NOTIFICATION(56) : alerts.sql(1679) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN notify_list;
   
   LOOP
      FETCH notify_list INTO
      h_resource_id, h_rec_user, h_alert_id, h_subject, h_message;
      IF notify_list%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      h_temp_sc_state := FC_A_NOTIFICATION(h_table_type, h_rec_user, h_wkf_def_id, h_type_id, h_rec_user
                                     , h_element_id);
      
      If h_temp_sc_state = 0 Then
         h_temp_id := NULL;
         
         h_temp_sc_state := FC_U_WORKFLOW(h_element_id, h_revision, h_count, h_count, h_temp_id, h_count
                                 , h_subject, h_message, h_resource_id, h_temp_id, h_assign_flag
                                 , h_flag_no_alert, h_temp_id, h_temp_id, h_rec_user);
      End If;
   END LOOP;
   
   BEGIN
      CLOSE notify_list;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_RESOURCE_NOTIFICATION', 95+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_RESOURCE_NOTIFICATION(96) : alerts.sql(1719) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ISSUE_RESOURCE_NOTIFICATION', 101+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ISSUE_RESOURCE_NOTIFICATION(102) : alerts.sql(1725) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ISSUE_RESOURCE_NOTIFICATION;
ALTER FUNCTION FC_ISSUE_RESOURCE_NOTIFICATION COMPILE DEBUG;

/*****************************************************************************************/

-- load project calendar to a temporary table if it wasn't loaded

/*****************************************************************************************/

Prompt == FUNCTION FC_Alert_LoadCalendar ==
CREATE OR REPLACE FUNCTION FC_Alert_LoadCalendar
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , sh_h_minute_per_day PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_minute_per_day PLS_INTEGER := sh_h_minute_per_day;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_Alert_LoadCalendar') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_Alert_LoadCalendar'), '-- ' || SYSTIMESTAMP, ':r := FC_Alert_LoadCalendar(' || 
       '''' || h_project_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_minute_per_day), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_Alert_LoadCalendar', 15+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_Alert_LoadCalendar(16) : alerts.sql(1755) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_RES_MIN_MAX';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_MIN_MAX';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CDY';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_CDY
      SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
      FROM CALENDAR_DAY_OF_YEAR
      WHERE ELEMENT_ID = h_project_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_Alert_LoadCalendar', 45+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_Alert_LoadCalendar(46) : alerts.sql(1785) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO PMO_TMP_MIN_MAX
      SELECT MIN(DAY_YEAR), MAX(DAY_YEAR), h_minute_per_day, '1899-12-30 8:00:00','1899-12-30 8:00:00', 0
      FROM PMO_TMP_CDY;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_Alert_LoadCalendar', 64+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_Alert_LoadCalendar(65) : alerts.sql(1804) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_Alert_LoadCalendar', 70+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_Alert_LoadCalendar(71) : alerts.sql(1810) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_Alert_LoadCalendar;
ALTER FUNCTION FC_Alert_LoadCalendar COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION FC_SEND_PRED_CHANGED_ALERT ==
CREATE OR REPLACE FUNCTION FC_SEND_PRED_CHANGED_ALERT RETURN PLS_INTEGER
AS
   h_depend_id VARCHAR2(32);
   h_alert_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_temp_var PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_project_id VARCHAR2(32);
   h_stage_id PLS_INTEGER := 0;
   h_alert_bit PLS_INTEGER := 0;
   h_succ_project_id VARCHAR2(32);
   h_pred_start_dt TIMESTAMP;
   h_pred_finish_dt TIMESTAMP;
   h_succ_start_dt TIMESTAMP;
   h_succ_finish_dt TIMESTAMP;
   h_new_pred_start_dt TIMESTAMP;
   h_new_pred_finish_dt TIMESTAMP;
   h_new_succ_start_dt TIMESTAMP;
   h_new_succ_finish_dt TIMESTAMP;
   h_dep_type PLS_INTEGER := 0;
   h_lead_or_lag_type PLS_INTEGER := 0;
   h_amount PLS_INTEGER := 0;
   h_lag_display_unit PLS_INTEGER := 0;
   h_minute_per_day PLS_INTEGER := 0;
   h_start_dt TIMESTAMP;
   h_finish_dt TIMESTAMP;
   h_isLoaded PLS_INTEGER;
   h_send_pred_date_change_alert PLS_INTEGER;
   h_alert_pred_date_change PLS_INTEGER := 0;
   h_tmp PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR pred_dates_changed IS
         SELECT DEPEND_ID, SUCC_PROJECT_ID, DEP_TYPE, LEAD_OR_LAG_TYPE, AMOUNT, LAG_DISPLAY_UNIT, PRED_START_DT, PRED_FINISH_DT, SUCC_START_DT, SUCC_FINISH_DT,
         NEW_PRED_START_DT, NEW_PRED_FINISH_DT, NEW_SUCC_START_DT, NEW_SUCC_FINISH_DT, MINUTE_PER_DAY
         FROM PMO_PRED_DATES_CHANGED_LIST
         ORDER BY SUCC_PROJECT_ID;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_SEND_PRED_CHANGED_ALERT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_SEND_PRED_CHANGED_ALERT'), '-- ' || SYSTIMESTAMP, ':r := FC_SEND_PRED_CHANGED_ALERT();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEND_PRED_CHANGED_ALERT', 47+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_SEND_PRED_CHANGED_ALERT(48) : alerts.sql(1868) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_PRED_DATES_CHANGED_LIST';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_NEW_DATES_LIST';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      UPDATE DEPENDENCIES SET
      PRED_UPDATED = 2
      WHERE PRED_UPDATED = 1;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PRED_CHANGED_ALERT', 72+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PRED_CHANGED_ALERT(73) : alerts.sql(1893) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO PMO_PRED_DATES_CHANGED_LIST (DEPEND_ID, PRED_ID, PRED_PROJECT_ID, SUCC_ID, SUCC_PROJECT_ID,
      DEP_TYPE, LEAD_OR_LAG_TYPE, AMOUNT, LAG_DISPLAY_UNIT, PRED_START_DT, PRED_FINISH_DT, SUCC_START_DT, SUCC_FINISH_DT)
      SELECT DEPEND_ID, PRED_ID, PRED_PROJECT_ID, SUCC_ID, SUCC_PROJECT_ID, DEP_TYPE, LEAD_OR_LAG_TYPE, AMOUNT,
      LAG_DISPLAY_UNIT, PRED_START_DT, PRED_FINISH_DT, SUCC_START_DT, SUCC_FINISH_DT
      FROM DEPENDENCIES
      WHERE PRED_UPDATED = 2;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PRED_CHANGED_ALERT', 94+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PRED_CHANGED_ALERT(95) : alerts.sql(1915) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEND_PRED_CHANGED_ALERT', 101+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_SEND_PRED_CHANGED_ALERT(102) : alerts.sql(1922) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_DB2M.SC_STATE;
   End If;
   
   BEGIN
      INSERT INTO PMO_NEW_DATES_LIST (PROJECT_ID, ELEMENT_ID)
      SELECT PRED_PROJECT_ID, PRED_ID
      FROM PMO_PRED_DATES_CHANGED_LIST
      UNION
      SELECT SUCC_PROJECT_ID, SUCC_ID
      FROM PMO_PRED_DATES_CHANGED_LIST;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PRED_CHANGED_ALERT', 123+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PRED_CHANGED_ALERT(124) : alerts.sql(1944) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE PMO_NEW_DATES_LIST N SET
      (START_DT, FINISH_DT, ELEMENT_NAME, MINUTE_PER_DAY) =
      (SELECT START_DT, FINISH_DT, ELEMENT_NAME, MINUTE_PER_DAY FROM TMT_WBS W
      WHERE W.ELEMENT_ID = N.ELEMENT_ID AND W.PROJECT_ID = N.PROJECT_ID);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PRED_CHANGED_ALERT', 143+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PRED_CHANGED_ALERT(144) : alerts.sql(1964) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE PMO_PRED_DATES_CHANGED_LIST D SET
      (NEW_PRED_START_DT, NEW_PRED_FINISH_DT, PRED_ELEMENT_NAME) =
      (SELECT START_DT, FINISH_DT, ELEMENT_NAME FROM PMO_NEW_DATES_LIST N WHERE N.ELEMENT_ID = D.PRED_ID  AND N.PROJECT_ID = D.PRED_PROJECT_ID),
      (NEW_SUCC_START_DT, NEW_SUCC_FINISH_DT, SUCC_ELEMENT_NAME, MINUTE_PER_DAY) =
      (SELECT START_DT, FINISH_DT, ELEMENT_NAME, MINUTE_PER_DAY FROM PMO_NEW_DATES_LIST W WHERE W.ELEMENT_ID = D.SUCC_ID  AND W.PROJECT_ID = D.SUCC_PROJECT_ID);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PRED_CHANGED_ALERT', 164+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PRED_CHANGED_ALERT(165) : alerts.sql(1985) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_alert_id := 'SCPREDECESSORDATESCHANGED';
   h_temp_var := 3;
   h_rec_user := 'PMO_SUPERVISOR';
   
   OPEN pred_dates_changed;
   
   h_project_id := 'INITIAL_PROJECT_ID90123456789012';
   h_alert_pred_date_change := 0;
   
   LOOP
      FETCH pred_dates_changed INTO h_depend_id, h_succ_project_id, h_dep_type, h_lead_or_lag_type, h_amount,
      h_lag_display_unit, h_pred_start_dt, h_pred_finish_dt, h_succ_start_dt, h_succ_finish_dt,
      h_new_pred_start_dt, h_new_pred_finish_dt, h_new_succ_start_dt, h_new_succ_finish_dt,
      h_minute_per_day;
      IF pred_dates_changed%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("Old PRED_FINISH_DT: %s, SUCC_START_DT: %s\n", h_pred_finish_dt,h_succ_start_dt);
      --trace_printf("New PRED_FINISH_DT: %s, SUCC_START_DT: %s\n", h_new_pred_finish_dt,h_new_succ_start_dt);
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      If PMO_ORASTD.strcmp(h_project_id, h_succ_project_id) <> 0 Then
         h_project_id := h_succ_project_id;
         h_isLoaded := 0;
         
         BEGIN
            SELECT COUNT(*) INTO h_alert_pred_date_change
            FROM PROJECT_STATE_DEF
            WHERE PROJECT_ID = h_project_id AND STAGE_ID = 1335;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PRED_CHANGED_ALERT', 208+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PRED_CHANGED_ALERT(209) : alerts.sql(2029) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_alert_pred_date_change, 0) = 0 Then
         GOTO Continue0000;
      Else
         h_send_pred_date_change_alert := 1;
      End If;
      
      CASE h_dep_type
         
         WHEN 0 THEN
            --Finish-Start
            If (Coalesce(h_amount, 0) = 0) AND (PMO_ORASTD.strcmp(h_pred_finish_dt, h_succ_start_dt) > 0) Then
               h_send_pred_date_change_alert := 0;
               GOTO CaseEnd0000;
            End If;
            
            -- load project calendar to a temporary table if it wasn't loaded
            If h_isLoaded = 0 Then
               h_tmp := FC_Alert_LoadCalendar(h_project_id, h_minute_per_day);
               h_isLoaded := 1;
            End If;
            
            h_tmp := 0;
            
            If PMO_ORASTD.strcmp(h_pred_finish_dt, h_succ_start_dt) <= 0 Then
               h_duration := GET_FD_DURATION(h_project_id, h_pred_finish_dt, h_succ_start_dt, h_tmp);
            Else
               h_duration := -GET_FD_DURATION(h_project_id, h_succ_start_dt, h_pred_finish_dt, h_tmp);
            End If;
            
            If Coalesce(h_duration, 0) <> Coalesce(h_amount, 0) * CASE WHEN (Coalesce(
                                       h_lead_or_lag_type, 0) = 0) THEN 1 ELSE -1 END 
                                       * CASE WHEN (Coalesce(
                                       h_lag_display_unit, 0) = 0) THEN Coalesce(
                                       h_minute_per_day, 0) ELSE 60 END Then
               h_send_pred_date_change_alert := 0;
            Else
               h_tmp := 0;
               
               If PMO_ORASTD.strcmp(h_new_pred_finish_dt, h_new_succ_start_dt) <= 0 Then
                  h_duration := GET_FD_DURATION(h_project_id, h_new_pred_finish_dt, h_new_succ_start_dt, h_tmp);
               Else
                  h_duration := -GET_FD_DURATION(h_project_id, h_new_succ_start_dt, h_new_pred_finish_dt, h_tmp);
               End If;
               
               If Coalesce(h_duration, 0) = Coalesce(h_amount, 0) * CASE WHEN (Coalesce(
                                      h_lead_or_lag_type, 0) = 0) THEN 1 ELSE -1 END 
                                      * CASE WHEN (Coalesce(
                                      h_lag_display_unit, 0) = 0) THEN Coalesce(
                                      h_minute_per_day, 0) ELSE 60 END Then
                  h_send_pred_date_change_alert := 0;
               End If;
            End If;
            
            <<CaseEnd0000>>
            NULL;
         
         WHEN 1 THEN
            --Start-Start
            If (Coalesce(h_amount, 0) = 0) AND (PMO_ORASTD.strcmp(h_pred_start_dt, h_succ_start_dt) > 0) Then
               h_send_pred_date_change_alert := 0;
               GOTO CaseEnd0001;
            End If;
            
            -- load project calendar to a temporary table if it wasn't loaded
            If h_isLoaded = 0 Then
               h_tmp := FC_Alert_LoadCalendar(h_project_id, h_minute_per_day);
               h_isLoaded := 1;
            End If;
            
            h_tmp := 0;
            
            If PMO_ORASTD.strcmp(h_pred_start_dt, h_succ_start_dt) <= 0 Then
               h_duration := GET_FD_DURATION(h_project_id, h_pred_start_dt, h_succ_start_dt, h_tmp);
            Else
               h_duration := -GET_FD_DURATION(h_project_id, h_succ_start_dt, h_pred_start_dt, h_tmp);
            End If;
            
            If Coalesce(h_duration, 0) <> Coalesce(h_amount, 0) * CASE WHEN (Coalesce(
                                       h_lead_or_lag_type, 0) = 0) THEN 1 ELSE -1 END 
                                       * CASE WHEN (Coalesce(
                                       h_lag_display_unit, 0) = 0) THEN Coalesce(
                                       h_minute_per_day, 0) ELSE 60 END Then
               h_send_pred_date_change_alert := 0;
            Else
               h_tmp := 0;
               
               If PMO_ORASTD.strcmp(h_new_pred_start_dt, h_new_succ_start_dt) <= 0 Then
                  h_duration := GET_FD_DURATION(h_project_id, h_new_pred_start_dt, h_new_succ_start_dt, h_tmp);
               Else
                  h_duration := -GET_FD_DURATION(h_project_id, h_new_succ_start_dt, h_new_pred_start_dt, h_tmp);
               End If;
               
               If Coalesce(h_duration, 0) = Coalesce(h_amount, 0) * CASE WHEN (Coalesce(
                                      h_lead_or_lag_type, 0) = 0) THEN 1 ELSE -1 END 
                                      * CASE WHEN (Coalesce(
                                      h_lag_display_unit, 0) = 0) THEN Coalesce(
                                      h_minute_per_day, 0) ELSE 60 END Then
                  h_send_pred_date_change_alert := 0;
               End If;
            End If;
            
            <<CaseEnd0001>>
            NULL;
         
         WHEN 2 THEN
            --Finish-Finish
            If (Coalesce(h_amount, 0) = 0) AND (PMO_ORASTD.strcmp(h_pred_finish_dt, h_succ_finish_dt) > 0) Then
               h_send_pred_date_change_alert := 0;
               GOTO CaseEnd0002;
            End If;
            
            -- load project calendar to a temporary table if it wasn't loaded
            If h_isLoaded = 0 Then
               h_tmp := FC_Alert_LoadCalendar(h_project_id, h_minute_per_day);
               h_isLoaded := 1;
            End If;
            
            h_tmp := 0;
            
            If PMO_ORASTD.strcmp(h_pred_finish_dt, h_succ_finish_dt) <= 0 Then
               h_duration := GET_FD_DURATION(h_project_id, h_pred_finish_dt, h_succ_finish_dt, h_tmp);
            Else
               h_duration := -GET_FD_DURATION(h_project_id, h_succ_finish_dt, h_pred_finish_dt, h_tmp);
            End If;
            
            If Coalesce(h_duration, 0) <> Coalesce(h_amount, 0) * CASE WHEN (Coalesce(
                                       h_lead_or_lag_type, 0) = 0) THEN 1 ELSE -1 END 
                                       * CASE WHEN (Coalesce(
                                       h_lag_display_unit, 0) = 0) THEN Coalesce(
                                       h_minute_per_day, 0) ELSE 60 END Then
               h_send_pred_date_change_alert := 0;
            Else
               h_tmp := 0;
               
               If PMO_ORASTD.strcmp(h_new_pred_finish_dt, h_new_succ_finish_dt) <= 0 Then
                  h_duration := GET_FD_DURATION(h_project_id, h_new_pred_finish_dt, h_new_succ_finish_dt, h_tmp);
               Else
                  h_duration := -GET_FD_DURATION(h_project_id, h_new_succ_finish_dt, h_new_pred_finish_dt, h_tmp);
               End If;
               
               If Coalesce(h_duration, 0) = Coalesce(h_amount, 0) * CASE WHEN (Coalesce(
                                      h_lead_or_lag_type, 0) = 0) THEN 1 ELSE -1 END 
                                      * CASE WHEN (Coalesce(
                                      h_lag_display_unit, 0) = 0) THEN Coalesce(
                                      h_minute_per_day, 0) ELSE 60 END Then
                  h_send_pred_date_change_alert := 0;
               End If;
            End If;
            
            <<CaseEnd0002>>
            NULL;
         
         WHEN 3 THEN
            --Start-Finish
            If (Coalesce(h_amount, 0) = 0) AND (PMO_ORASTD.strcmp(h_pred_start_dt, h_succ_finish_dt) > 0) Then
               h_send_pred_date_change_alert := 0;
               GOTO CaseEnd0003;
            End If;
            
            -- load project calendar to a temporary table if it wasn't loaded
            If h_isLoaded = 0 Then
               h_tmp := FC_Alert_LoadCalendar(h_project_id, h_minute_per_day);
               h_isLoaded := 1;
            End If;
            
            h_tmp := 0;
            
            If PMO_ORASTD.strcmp(h_pred_start_dt, h_succ_finish_dt) <= 0 Then
               h_duration := GET_FD_DURATION(h_project_id, h_pred_start_dt, h_succ_finish_dt, h_tmp);
            Else
               h_duration := -GET_FD_DURATION(h_project_id, h_succ_finish_dt, h_pred_start_dt, h_tmp);
            End If;
            
            If Coalesce(h_duration, 0) <> Coalesce(h_amount, 0) * CASE WHEN (Coalesce(
                                       h_lead_or_lag_type, 0) = 0) THEN 1 ELSE -1 END 
                                       * CASE WHEN (Coalesce(
                                       h_lag_display_unit, 0) = 0) THEN Coalesce(
                                       h_minute_per_day, 0) ELSE 60 END Then
               h_send_pred_date_change_alert := 0;
            Else
               h_tmp := 0;
               
               If PMO_ORASTD.strcmp(h_new_pred_start_dt, h_new_succ_finish_dt) <= 0 Then
                  h_duration := GET_FD_DURATION(h_project_id, h_new_pred_start_dt, h_new_succ_finish_dt, h_tmp);
               Else
                  h_duration := -GET_FD_DURATION(h_project_id, h_new_succ_finish_dt, h_new_pred_start_dt, h_tmp);
               End If;
               
               If Coalesce(h_duration, 0) = Coalesce(h_amount, 0) * CASE WHEN (Coalesce(
                                      h_lead_or_lag_type, 0) = 0) THEN 1 ELSE -1 END 
                                      * CASE WHEN (Coalesce(
                                      h_lag_display_unit, 0) = 0) THEN Coalesce(
                                      h_minute_per_day, 0) ELSE 60 END Then
                  h_send_pred_date_change_alert := 0;
               End If;
            End If;
            <<CaseEnd0003>>
            NULL;
         ELSE
            NULL;
      END CASE;
      
      If h_send_pred_date_change_alert = 1 Then
         null_return := FC_ISSUE_ALERT(h_alert_id, h_depend_id, h_rec_user, h_temp_var, 0);
      End If;
      <<Continue0000>>
      NULL;
   END LOOP;
   
   BEGIN
      CLOSE pred_dates_changed;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PRED_CHANGED_ALERT', 431+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PRED_CHANGED_ALERT(432) : alerts.sql(2252) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE DEPENDENCIES D SET
      (D.PRED_START_DT, D.PRED_FINISH_DT, D.SUCC_START_DT, D.SUCC_FINISH_DT, D.PRED_UPDATED) =
      (SELECT T.NEW_PRED_START_DT, T.NEW_PRED_FINISH_DT, T.NEW_SUCC_START_DT, T.NEW_SUCC_FINISH_DT, 0
      FROM PMO_PRED_DATES_CHANGED_LIST T
      WHERE T.DEPEND_ID = D.DEPEND_ID)
      WHERE D.PRED_UPDATED = 2;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PRED_CHANGED_ALERT', 453+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PRED_CHANGED_ALERT(454) : alerts.sql(2274) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEND_PRED_CHANGED_ALERT', 459+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_SEND_PRED_CHANGED_ALERT(460) : alerts.sql(2280) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_SEND_PRED_CHANGED_ALERT;
ALTER FUNCTION FC_SEND_PRED_CHANGED_ALERT COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION FC_SEND_PERIODIC_ALERTS ==
CREATE OR REPLACE FUNCTION FC_SEND_PERIODIC_ALERTS
(
     sh_h_bitflag PLS_INTEGER
   , h_date IN OUT NOCOPY DATE
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_task_daily_alert PLS_INTEGER := 0;
   h_workflow_daily_alert PLS_INTEGER := 0;
   h_timesheet_weekly_alert PLS_INTEGER := 0;
   h_alert_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_temp_var PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_actual_start DATE;
   h_actual_finish DATE;
   h_unresolved_start DATE;
   h_unresolved_finish DATE;
   h_current_date DATE;
   h_date_week_of DATE;
   h_remained_days PLS_INTEGER := 0;
   h_day_week_of PLS_INTEGER := 0;
   h_day_today PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_project_id VARCHAR2(32);
   h_stage_id PLS_INTEGER := 0;
   h_alert_bit PLS_INTEGER := 0;
   h_current_date_time TIMESTAMP;
   null_return LONG;
-- Declare Cursors
      CURSOR task_list1 IS
         SELECT T.PROJECT_ID, T.ELEMENT_ID
         FROM TMT_WBS T, PROJECT_STATE_DEF P
         WHERE T.PROJECT_ID = P.PROJECT_ID AND P.STAGE_ID = h_stage_id
         AND T.TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
         AND T.ACTUAL_START IS Null
         AND TO_DATE(SubStr(T.UNRESOLVED_START, 1, 10)) < TO_DATE(SubStr(h_current_date, 1, 10));

      CURSOR task_list2 IS
         SELECT T.PROJECT_ID, T.ELEMENT_ID
         FROM TMT_WBS T, PROJECT_STATE_DEF P
         WHERE T.PROJECT_ID = P.PROJECT_ID AND P.STAGE_ID = h_stage_id
         AND T.TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
         AND T.ACTUAL_FINISH IS Null
         AND TO_DATE(SubStr(T.UNRESOLVED_FINISH, 1, 10)) < TO_DATE(SubStr(h_current_date, 1, 10));

      CURSOR task_list3 IS
         SELECT T.PROJECT_ID, T.ELEMENT_ID, TO_DATE(SubStr(T.UNRESOLVED_START, 1, 10)) - TO_DATE(SubStr(h_current_date, 1, 10))
         FROM TMT_WBS T, PROJECT_STATE_DEF P
         WHERE T.PROJECT_ID = P.PROJECT_ID AND P.STAGE_ID = h_stage_id
         AND T.TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
         AND T.ACTUAL_START IS Null
         AND TO_DATE(SubStr(T.UNRESOLVED_START, 1, 10)) - P.VARIANCE     < TO_DATE(SubStr(h_current_date, 1, 10))
         AND TO_DATE(SubStr(T.UNRESOLVED_START, 1, 10)) > TO_DATE(SubStr(h_current_date, 1, 10));

      CURSOR workflowitem_list IS
         SELECT WR.ELEMENT_ID, WR.RESOURCE_ID
         FROM WORKFLOW_REPLIES WR, TMT_WORKFLOWS WF
         WHERE WF.ELEMENT_ID = WR.ELEMENT_ID
         AND TO_DATE(SubStr(WF.EXPIRES_DATE, 1, 10)) < COALESCE(TO_DATE(SubStr(h_date, 1, 10)), TO_DATE(SYSDATE))
         AND WF.END_CYCLE = '0'
         AND WF.TYPE_ID NOT IN (85, 297, 300)
         AND WR.REPLY_TYPE <> PMO_CNST.SC_HIDEN_REPLY
         AND WR.REPLY_STATUS = 0;

      CURSOR timesheet_list IS
         SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID
         FROM TASK_ASSIGNMENTS TA
         WHERE TO_DATE(SubStr(SCHED_START, 1, 10)) <= TO_DATE(SubStr(h_date_week_of, 1, 10)) + 6    
         AND TO_DATE(SubStr(SCHED_FINISH, 1, 10)) >= TO_DATE(SubStr(h_date_week_of, 1, 10))
         AND NOT EXISTS (
         SELECT 1
         FROM TIMESHEET_APPROVAL_STATUS
         WHERE ELEMENT_ID = TA.PROJECT_ID
         AND RESOURCE_ID = TA.RESOURCE_ID
         AND WEEK_OF = TO_DATE(SubStr(h_date_week_of, 1, 10)));

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_SEND_PERIODIC_ALERTS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_SEND_PERIODIC_ALERTS'), '-- ' || SYSTIMESTAMP, ':r := FC_SEND_PERIODIC_ALERTS(' || 
       COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_date || '''',''''''), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEND_PERIODIC_ALERTS', 86+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_SEND_PERIODIC_ALERTS(87) : alerts.sql(2377) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT COALESCE(h_date,TO_DATE(SYSDATE)), SYSTIMESTAMP
      INTO h_current_date, h_current_date_time
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 99+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(100) : alerts.sql(2390) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --check if to send the dependency predecessor dates changed alert
   BEGIN
      SELECT COUNT(1) INTO h_temp_var
      FROM TMT_SETTINGS
      WHERE (PMO_ORASTD.DAYS(PMO_ORASTD.DB2TIMESTAMP(DESCRIPTION)) < PMO_ORASTD.DAYS(PMO_ORASTD.DB2TIMESTAMP(h_current_date_time))
      OR PMO_ORASTD.DB2TIMESTAMPDIFF(4, PMO_ORASTD.DB2TIMESTAMP(h_current_date_time), PMO_ORASTD.DB2TIMESTAMP(DESCRIPTION)) > SETTING_VALUE)
      AND ELEMENT_ID = 'DEPENDENCY_ALERT_DT_INTERVAL';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 117+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(118) : alerts.sql(2408) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_temp_var, 0) = 1 Then
      BEGIN
         UPDATE TMT_SETTINGS
         SET DESCRIPTION = h_current_date_time
         WHERE ELEMENT_ID = 'DEPENDENCY_ALERT_DT_INTERVAL';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 137+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(138) : alerts.sql(2428) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_temp_var := FC_SEND_PRED_CHANGED_ALERT();
   End If;
   
   --check if to send other periodic alerts
   If (BITAND(PMO_CNST.SC_TASK_DAILY_ALERT, Coalesce(
         h_bitflag, 0))) = PMO_CNST.SC_TASK_DAILY_ALERT Then
      h_task_daily_alert := 1;
   End If;
   
   If (BITAND(PMO_CNST.SC_WORKFLOW_DAILY_ALERT, Coalesce(
         h_bitflag, 0))) = PMO_CNST.SC_WORKFLOW_DAILY_ALERT Then
      h_workflow_daily_alert := 1;
   End If;
   
   If h_date IS NULL AND (Coalesce(h_task_daily_alert, 0) = 1 OR Coalesce(
         h_workflow_daily_alert, 0) = 1) Then
      BEGIN
         SELECT COUNT(1) INTO h_temp_var
         FROM TMT_SETTINGS
         WHERE DESCRIPTION < h_current_date
         AND ELEMENT_ID = 'NEXT_DAILY_ALERT_DATE';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 169+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(170) : alerts.sql(2460) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_task_daily_alert := Coalesce(h_task_daily_alert, 0) * Coalesce(h_temp_var, 0);
      h_workflow_daily_alert := Coalesce(h_workflow_daily_alert, 0) * Coalesce(h_temp_var, 0);
   End If;
   
   If (BITAND(PMO_CNST.SC_TIMESHEET_WEEKLY_ALERT, Coalesce(
         h_bitflag, 0))) = PMO_CNST.SC_TIMESHEET_WEEKLY_ALERT Then
      h_timesheet_weekly_alert := 1;
   End If;
   
   If h_date IS NULL AND (Coalesce(h_timesheet_weekly_alert, 0) = 1) Then
      BEGIN
         SELECT COUNT(1) INTO h_temp_var
         FROM TMT_SETTINGS
         WHERE TO_DATE(SubStr(DESCRIPTION, 1, 10)) + 6     < TO_DATE(SubStr(h_current_date, 1, 10))
         AND ELEMENT_ID = 'NEXT_WEEKLY_ALERT_DATE';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 195+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(196) : alerts.sql(2486) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_timesheet_weekly_alert := Coalesce(h_timesheet_weekly_alert, 0) * Coalesce(h_temp_var, 0);
   End If;
   
   h_rec_user := 'PMO_SUPERVISOR';
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_ALERTS_PARAM_VALUE';
   PMO_DB2M.SC_STATE := 0;
   
   If Coalesce(h_task_daily_alert, 0) = 1 Then
      h_table_type := 1;
      h_type_id := PMO_CNST.SC_ALERT_GROUP;
      
      --for Alerts SCTASKLATETOSTART
      h_alert_id := 'SCTASKLATETOSTART';
      
      BEGIN
         SELECT TYPE_ID, ALERT_BIT
         INTO h_stage_id, h_alert_bit
         FROM ALERT_MESSAGES
         WHERE ALERT_ID = h_alert_id
         AND LANGUAGE_CODE = 'EN';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 228+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(229) : alerts.sql(2519) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (h_alert_bit IS NOT NULL) AND (Coalesce(h_alert_bit, 0) = 1) Then
         OPEN task_list1;
         
         LOOP
            FETCH task_list1 INTO h_project_id, h_id;
            IF task_list1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            h_temp_var := 3;
            null_return := FC_ISSUE_ALERT(h_alert_id, h_id, h_rec_user, h_temp_var, 0);
         END LOOP;
         
         BEGIN
            CLOSE task_list1;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 260+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(261) : alerts.sql(2551) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --for Alerts SCTASKLATETOFINISH
      h_alert_id := 'SCTASKLATETOFINISH';
      
      BEGIN
         SELECT TYPE_ID, ALERT_BIT
         INTO h_stage_id, h_alert_bit
         FROM ALERT_MESSAGES
         WHERE ALERT_ID = h_alert_id
         AND LANGUAGE_CODE = 'EN';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 281+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(282) : alerts.sql(2572) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (h_alert_bit IS NOT NULL) AND (Coalesce(h_alert_bit, 0) = 1) Then
         OPEN task_list2;
         
         LOOP
            FETCH task_list2 INTO h_project_id, h_id;
            IF task_list2%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            h_temp_var := 3;
            null_return := FC_ISSUE_ALERT(h_alert_id, h_id, h_rec_user, h_temp_var, 0);
         END LOOP;
         
         BEGIN
            CLOSE task_list2;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 313+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(314) : alerts.sql(2604) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --for Alerts SCTASKTOBESTARTED
      h_alert_id := 'SCTASKTOBESTARTED';
      
      BEGIN
         SELECT TYPE_ID, ALERT_BIT
         INTO h_stage_id, h_alert_bit
         FROM ALERT_MESSAGES
         WHERE ALERT_ID = h_alert_id
         AND LANGUAGE_CODE = 'EN';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 334+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(335) : alerts.sql(2625) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (h_alert_bit IS NOT NULL) AND (Coalesce(h_alert_bit, 0) = 1) Then
         OPEN task_list3;
         
         LOOP
            FETCH task_list3 INTO h_project_id, h_id, h_remained_days;
            IF task_list3%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            h_temp_var := 3;
            null_return := FC_ISSUE_ALERT(h_alert_id, h_id, h_rec_user, h_temp_var, 0);
         END LOOP;
         
         BEGIN
            CLOSE task_list3;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 366+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(367) : alerts.sql(2657) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If Coalesce(h_workflow_daily_alert, 0) = 1 Then
      BEGIN
         SELECT ALERT_BIT
         INTO h_alert_bit
         FROM ALERT_MESSAGES
         WHERE ALERT_ID = 'SCNOTRESPONDEDTOWORKFLOWITEM'
         AND LANGUAGE_CODE = 'EN';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 386+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(387) : alerts.sql(2677) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (h_alert_bit IS NOT NULL) AND (Coalesce(h_alert_bit, 0) = 1) Then
         --for Alert SCNOTRESPONDEDTOWORKFLOWITEM
         
         OPEN workflowitem_list;
         
         LOOP
            FETCH workflowitem_list
            INTO h_id, h_rec_user;
            IF workflowitem_list
            %FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            h_alert_id := 'SCNOTRESPONDEDTOWORKFLOWITEM';
            h_temp_var := 3;
            null_return := FC_ISSUE_ALERT(h_alert_id, h_id, h_rec_user, h_temp_var, 0);
         END LOOP;
         
         BEGIN
            CLOSE workflowitem_list;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 423+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(424) : alerts.sql(2714) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If h_date IS NULL AND (Coalesce(h_task_daily_alert, 0) = 1 OR Coalesce(
         h_workflow_daily_alert, 0) = 1) Then
      BEGIN
         UPDATE TMT_SETTINGS
         SET DESCRIPTION = TO_CHAR(TO_DATE(SYSDATE) + 1    )
         WHERE ELEMENT_ID = 'NEXT_DAILY_ALERT_DATE';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 446+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(447) : alerts.sql(2737) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If Coalesce(h_timesheet_weekly_alert, 0) = 1 Then
      BEGIN
         SELECT ALERT_BIT
         INTO h_alert_bit
         FROM ALERT_MESSAGES
         WHERE ALERT_ID = 'SCTIMESHEETLATEREMINDER'
         AND LANGUAGE_CODE = 'EN';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 465+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(466) : alerts.sql(2756) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (h_alert_bit IS NOT NULL) AND (Coalesce(h_alert_bit, 0) = 1) Then
         --for Alert SCTIMESHEETLATEREMINDER
         BEGIN
            SELECT SETTING_VALUE, PMO_ORASTD.DAYOFWEEK_ISO(TO_DATE(SYSDATE))
            INTO h_day_week_of, h_day_today
            FROM TMT_SETTINGS
            WHERE ELEMENT_ID = 'TIMESHEET_START_WEEKDAY';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 483+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(484) : alerts.sql(2774) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If h_date IS NULL Then
            BEGIN
               SELECT CASE WHEN h_day_today >= h_day_week_of THEN TO_CHAR(TO_DATE(SYSDATE) - (7 + h_day_today - h_day_week_of)    )
               ELSE TO_CHAR(TO_DATE(SYSDATE) - (14 + h_day_today - h_day_week_of)    ) END CASE
               INTO h_date_week_of
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 500+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(501) : alerts.sql(2791) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT COUNT(1) INTO h_temp_var
               FROM TMT_SETTINGS
               WHERE ELEMENT_ID = 'TIMESHEET_START_WEEKDAY'
               AND PMO_ORASTD.DAYOFWEEK_ISO(TO_DATE(SubStr(h_date, 1, 10))) = SETTING_VALUE;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 516+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(517) : alerts.sql(2807) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_temp_var, 0) = 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEND_PERIODIC_ALERTS', 523+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_SEND_PERIODIC_ALERTS(524) : alerts.sql(2814) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.INVALID_PARAMS;
            Else
               h_date_week_of := h_date;
            End If;
         End If;
         
         BEGIN
            INSERT INTO PMO_ALERTS_PARAM_VALUE (ALERT_ID, ELEMENT_ID, VALUE)
            VALUES ('SCTIMESHEETLATEREMINDER', 'WEEK_OF', h_date_week_of);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 544+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(545) : alerts.sql(2835) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         OPEN timesheet_list;
         
         LOOP
            FETCH timesheet_list
            INTO h_id, h_rec_user;
            IF timesheet_list
            %FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            h_alert_id := 'SCTIMESHEETLATEREMINDER';
            h_temp_var := 3;
            null_return := FC_ISSUE_ALERT(h_alert_id, h_id, h_rec_user, h_temp_var, 0);
         END LOOP;
         
         BEGIN
            CLOSE timesheet_list;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 578+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(579) : alerts.sql(2869) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM PMO_ALERTS_PARAM_VALUE
            WHERE ALERT_ID = h_alert_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 596+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(597) : alerts.sql(2887) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If h_date IS NULL Then
            BEGIN
               UPDATE TMT_SETTINGS
               SET DESCRIPTION = TO_CHAR(TO_DATE(SubStr(h_date_week_of, 1, 10)) + 7    )
               WHERE ELEMENT_ID = 'NEXT_WEEKLY_ALERT_DATE';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEND_PERIODIC_ALERTS', 616+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEND_PERIODIC_ALERTS(617) : alerts.sql(2907) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEND_PERIODIC_ALERTS', 625+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_SEND_PERIODIC_ALERTS(626) : alerts.sql(2916) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_SEND_PERIODIC_ALERTS;
ALTER FUNCTION FC_SEND_PERIODIC_ALERTS COMPILE DEBUG;

/***************************************************************************************************/

Prompt == FUNCTION SP_S_ALERT ==
CREATE OR REPLACE FUNCTION SP_S_ALERT
(
     p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_temp_sc_state PLS_INTEGER := 0;
   h_alert_bitflag PLS_INTEGER := 0;
   h_date DATE;
-- Declare Cursors
      -- Resultset Cursor Declaration
      SELECT_READ1 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_ALERT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_ALERT'), '-- ' || SYSTIMESTAMP, ':r := SP_S_ALERT(' || 
       COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ALERT', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_ALERT(21) : alerts.sql(2947) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      UPDATE ALERTS SET FLAG = 'U' WHERE FLAG = 'I';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ALERT', 35+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ALERT(36) : alerts.sql(2962) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_temp_sc_state := FC_PRJ_STATE_WKF();
   COMMIT;
   
   h_alert_bitflag := 7;
   h_date := NULL;
   h_temp_sc_state := FC_SEND_PERIODIC_ALERTS(h_alert_bitflag, h_date);
   COMMIT;
   --trace_printf("FC_SEND_PERIODIC_ALERTS result code : %d \n", h_temp_sc_state);
   
   h_temp_sc_state := FC_BUILD_ALERT_CONTACT_LIST();
   COMMIT;
   
   h_temp_sc_state := FC_ISSUE_RESOURCE_NOTIFICATION();
   COMMIT;
   
   BEGIN
      UPDATE ALERTS SET FLAG = 'D' WHERE FLAG = 'U';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ALERT', 67+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ALERT(68) : alerts.sql(2994) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   COMMIT;
   
   OPEN SELECT_READ1 FOR SELECT R.RESOURCE_ID AS ELEMENT_ID, R.EMAIL_ADDRESS AS TO_ADDRESS, AM.ALERT_MESSAGE, A.VALUE_LIST, R.LOGON_NAME, TO_DATE(SubStr(A.REC_DATETIME, 1, 10)) AS ALERT_DATE,
   CASE WHEN A.ALERT_ID = 'SCPERSONALCALENDARCHANGED' THEN 'RESOURCES' ELSE A.PARENT_TABLE END AS PARENT_TABLE,
   CASE WHEN A.ALERT_ID = 'SCPERSONALCALENDARCHANGED' THEN R.RESOURCE_ID ELSE A.ELEMENT_ID END AS PMO_ELEMENT_ID,
   COALESCE(RS.E_MAIL_ADDRESS, ' ') AS FROM_ADDRESS, R.SUBJECT
   FROM ALERTS A, ALERT_MESSAGES AM, PMO_ALERT_CONTACT_LIST R, RESOURCES RS
   WHERE A.ALERT_ID = R.ALERT_ID
   AND A.PROJECT_ID = R.PROJECT_ID
   AND (A.ELEMENT_ID = R.ELEMENT_ID OR A.ALERT_ID = 'SCPERSONALCALENDARCHANGED')
   AND AM.ALERT_ID = A.ALERT_ID
   AND (A.ALERT_ID NOT IN ('SCCOSTVARIANCE', 'SCSCHEDULEVARIANCE') OR A.VARIANCE >= R.VARIANCE)
   AND A.FLAG = 'D'
   AND AM.ALERT_BIT = 1
   AND R.EMAIL_ADDRESS IS NOT Null
   AND COALESCE(A.REC_USER, 'PMO_SUPERVISOR') = RS.RESOURCE_ID
   AND COALESCE(A.REC_USER, 'PMO_SUPERVISOR') <> R.RESOURCE_ID
   AND AM.LANGUAGE_CODE = COALESCE((SELECT TD.ELEMENT_NAME FROM ALERT_MESSAGES B, RESOURCE_READINESS RR, TMT_DATAFIELDS TD
   WHERE B.ALERT_ID = AM.ALERT_ID AND RR.RESOURCE_ID = R.RESOURCE_ID AND RR.DEFAULT_LANGUAGE = 'Y'
   AND RR.LANGUAGE_ID = TD.ELEMENT_ID AND B.LANGUAGE_CODE = TD.ELEMENT_NAME) ,'EN')
   UNION ALL
   SELECT R.RESOURCE_ID AS ELEMENT_ID, R.E_MAIL_ADDRESS AS TO_ADDRESS, AM.ALERT_MESSAGE, A.VALUE_LIST, R.LOGON_NAME, TO_DATE(SubStr(A.REC_DATETIME, 1, 10)) AS ALERT_DATE, A.PARENT_TABLE,
   A.ELEMENT_ID AS PMO_ELEMENT_ID, COALESCE(RS.E_MAIL_ADDRESS, ' ') AS FROM_ADDRESS, AM.ALERT_NAME AS SUBJECT
   FROM ALERTS A, RESOURCES R, ALERT_MESSAGES AM, RESOURCES RS
   WHERE A.RESOURCE_ID = R.RESOURCE_ID
   AND A.PARENT_TABLE IN ('RESOURCES', 'TMT_WORKFLOWS')
   AND A.FLAG = 'D'
   AND AM.ALERT_BIT = 1
   AND AM.ALERT_ID = A.ALERT_ID
   AND AM.LANGUAGE_CODE = COALESCE((SELECT TD.ELEMENT_NAME FROM ALERT_MESSAGES B, RESOURCE_READINESS RR, TMT_DATAFIELDS TD
   WHERE B.ALERT_ID = AM.ALERT_ID AND RR.RESOURCE_ID = R.RESOURCE_ID AND RR.DEFAULT_LANGUAGE = 'Y'
   AND RR.LANGUAGE_ID = TD.ELEMENT_ID AND B.LANGUAGE_CODE = TD.ELEMENT_NAME) ,'EN')
   AND R.E_MAIL_ADDRESS IS NOT Null
   AND COALESCE(A.REC_USER, 'PMO_SUPERVISOR') = RS.RESOURCE_ID
   AND COALESCE(A.REC_USER, 'PMO_SUPERVISOR') <> A.ELEMENT_ID
   ORDER BY TO_ADDRESS, LOGON_NAME, FROM_ADDRESS, SUBJECT, ALERT_DATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ALERT', 109+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_ALERT(110) : alerts.sql(3036) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN SELECT_READ1;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ALERT', 117+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ALERT(118) : alerts.sql(3044) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ALERT', 129+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ALERT(130) : alerts.sql(3056) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_ALERT;
ALTER FUNCTION SP_S_ALERT COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION SP_CHECK_ALERT ==
CREATE OR REPLACE FUNCTION SP_CHECK_ALERT
(
     p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
      -- Resultset Cursor Declaration
      SELECT_READ2 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_CHECK_ALERT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_CHECK_ALERT'), '-- ' || SYSTIMESTAMP, ':r := SP_CHECK_ALERT(' || 
       COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CHECK_ALERT', 17+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_CHECK_ALERT(18) : alerts.sql(3090) : ' || SYSTIMESTAMP);
   END IF; 

   OPEN SELECT_READ2 FOR SELECT 100 AS RETURN_VALUE FROM DUAL;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CHECK_ALERT', 22+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_CHECK_ALERT(23) : alerts.sql(3095) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN SELECT_READ2;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CHECK_ALERT', 30+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CHECK_ALERT(31) : alerts.sql(3103) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CHECK_ALERT', 42+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CHECK_ALERT(43) : alerts.sql(3115) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_CHECK_ALERT;
ALTER FUNCTION SP_CHECK_ALERT COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION SP_D_ALERT ==
CREATE OR REPLACE FUNCTION SP_D_ALERT
(
     p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_uniqueID VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_ALERT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_ALERT'), '-- ' || SYSTIMESTAMP, ':r := SP_D_ALERT(' || 
       COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_ALERT', 15+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_ALERT(16) : alerts.sql(3147) : ' || SYSTIMESTAMP);
   END IF; 

   h_uniqueID := '0';
   
   BEGIN
      DELETE FROM ALERTS WHERE FLAG = 'D';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ALERT', 32+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ALERT(33) : alerts.sql(3164) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_ALERT', 38+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_ALERT(39) : alerts.sql(3170) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ALERT', 45+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ALERT(46) : alerts.sql(3177) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ALERT', 57+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ALERT(58) : alerts.sql(3189) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_ALERT;
ALTER FUNCTION SP_D_ALERT COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION SP_A_PROJ_ALERT ==
CREATE OR REPLACE FUNCTION SP_A_PROJ_ALERT
(
     p_project_id VARCHAR2
   , p_alert_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_uniqueID VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_PROJ_ALERT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_PROJ_ALERT'), '-- ' || SYSTIMESTAMP, ':r := SP_A_PROJ_ALERT(' || 
       '''' || p_project_id || ''''
          || ', ' || '''' || p_alert_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_PROJ_ALERT', 19+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_PROJ_ALERT(20) : alerts.sql(3225) : ' || SYSTIMESTAMP);
   END IF; 

   If p_project_id IS NULL OR p_alert_id IS NULL Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_PROJ_ALERT', 24+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_PROJ_ALERT(25) : alerts.sql(3230) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ELEMENT_ID_REQUIRED);
   End If;
   
   h_uniqueID := SYS_GUID();
   
   BEGIN
      INSERT INTO PROJECT_ALERTS (PROJECT_ALERT_ID, PROJECT_ID, ALERT_ID, VARIANCE, REC_USER)
      VALUES (h_uniqueID, p_project_id, p_alert_id, -1, p_rec_user);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_PROJ_ALERT', 44+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_PROJ_ALERT(45) : alerts.sql(3250) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_PROJ_ALERT', 50+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_PROJ_ALERT(51) : alerts.sql(3256) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_PROJ_ALERT', 57+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_PROJ_ALERT(58) : alerts.sql(3263) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_PROJ_ALERT', 69+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_PROJ_ALERT(70) : alerts.sql(3275) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_PROJ_ALERT;
ALTER FUNCTION SP_A_PROJ_ALERT COMPILE DEBUG;

/************************************************************************************************

  Parameter:
    alert_id:   the id of the project
    project_id: the id of the message
    variance:   value of the variance.
                0: disable the alert in the current project (insert or update)
                1: delete record from project_alerts table
                >1: insert (insert or update)
    res_user:   the id of the user who updates the current element
*************************************************************************************************/

Prompt == FUNCTION SP_U_PROJ_ALERT ==
CREATE OR REPLACE FUNCTION SP_U_PROJ_ALERT
(
     p_alert_id VARCHAR2
   , p_project_id VARCHAR2
   , p_variance PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_uniqueID VARCHAR2(32);
   h_count PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_PROJ_ALERT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_PROJ_ALERT'), '-- ' || SYSTIMESTAMP, ':r := SP_U_PROJ_ALERT(' || 
       '''' || p_alert_id || ''''
          || ', ' || '''' || p_project_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_variance), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PROJ_ALERT', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_PROJ_ALERT(23) : alerts.sql(3324) : ' || SYSTIMESTAMP);
   END IF; 

   If p_project_id IS NULL OR p_alert_id IS NULL Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_PROJ_ALERT', 27+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_PROJ_ALERT(28) : alerts.sql(3329) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ELEMENT_ID_REQUIRED);
   End If;
   
   BEGIN
      SELECT COUNT(*) INTO h_count
      FROM PROJECT_ALERTS
      WHERE PROJECT_ID = p_project_id AND ALERT_ID = p_alert_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PROJ_ALERT', 42+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PROJ_ALERT(43) : alerts.sql(3344) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) > 0 Then
      If Coalesce(p_variance, 0) = 1 Then
         BEGIN
            DELETE FROM PROJECT_ALERTS
            WHERE PROJECT_ID = p_project_id and ALERT_ID = p_alert_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PROJ_ALERT', 62+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PROJ_ALERT(63) : alerts.sql(3364) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE PROJECT_ALERTS
            SET VARIANCE = p_variance,
            REC_USER = p_rec_user
            WHERE PROJECT_ID = p_project_id and ALERT_ID = p_alert_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PROJ_ALERT', 82+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PROJ_ALERT(83) : alerts.sql(3384) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      If Coalesce(p_variance, 0) <> 1 Then
         h_uniqueID := SYS_GUID();
         
         BEGIN
            INSERT INTO PROJECT_ALERTS (PROJECT_ALERT_ID, PROJECT_ID, ALERT_ID, VARIANCE, REC_USER)
            VALUES (h_uniqueID, p_project_id, p_alert_id, p_variance, p_rec_user);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PROJ_ALERT', 104+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PROJ_ALERT(105) : alerts.sql(3406) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_PROJ_ALERT', 112+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_PROJ_ALERT(113) : alerts.sql(3414) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_project_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_project_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PROJ_ALERT', 119+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PROJ_ALERT(120) : alerts.sql(3421) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PROJ_ALERT', 131+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PROJ_ALERT(132) : alerts.sql(3433) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_PROJ_ALERT;
ALTER FUNCTION SP_U_PROJ_ALERT COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION SP_S_PROJ_ALERT ==
CREATE OR REPLACE FUNCTION SP_S_PROJ_ALERT
(
     p_project_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_lang_code VARCHAR2(32);
-- Declare Cursors
      -- Resultset Cursor Declaration
      SELECT_READ3 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_PROJ_ALERT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_PROJ_ALERT'), '-- ' || SYSTIMESTAMP, ':r := SP_S_PROJ_ALERT(' || 
       '''' || p_project_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROJ_ALERT', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_PROJ_ALERT(23) : alerts.sql(3472) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT LANGUAGE_ISO_CODE INTO h_lang_code
      FROM RESOURCES
      WHERE RESOURCE_ID = p_rec_user;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROJ_ALERT', 35+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROJ_ALERT(36) : alerts.sql(3485) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(Length(h_lang_code), 0) < 1 Then
      h_lang_code := 'EN';
   End If;
   
   If p_project_id IS NULL Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_PROJ_ALERT', 46+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_PROJ_ALERT(47) : alerts.sql(3496) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.ELEMENT_ID_REQUIRED);
   End If;
   
   OPEN SELECT_READ3 FOR SELECT A.ALERT_ID AS ELEMENT_ID, p_project_id AS ORIGINAL_ID,
   A.ALERT_NAME AS ELEMENT_NAME, 400 AS TYPE_ID,
   CASE WHEN EXISTS(SELECT P1.PROJECT_ID FROM PROJECT_ALERTS P1
   WHERE P1.ALERT_ID = A.ALERT_ID AND P1.PROJECT_ID = p_project_id)
   THEN (SELECT P2.VARIANCE FROM PROJECT_ALERTS P2
   WHERE P2.ALERT_ID = A.ALERT_ID AND P2.PROJECT_ID = p_project_id)
   ELSE 1 END AS VARIANCE
   FROM ALERT_MESSAGES A
   WHERE A.ALERT_ID NOT IN ('SCRESOURCERECORDREAPPROVAL', 'SCMAILGROUPMEMBERADD', 'SCMAILGROUPMEMBERREMOVE',
   'SCSCHEDULEASSIGNMENTADD', 'SCSCHEDULEASSIGNMENTREMOVE', 'SCWORKFLOWVOTESCOMPLETE', 'SCFSWEBSERVERPATH',
   'SCNEWWORKFLOW', 'SCCOMPLETEDWORKFLOW', 'SCREVOTEDWORKFLOW', 'SCCLOSEDWORKFLOW')
   AND A.LANGUAGE_CODE = CASE WHEN EXISTS(SELECT B.ALERT_ID FROM ALERT_MESSAGES B
   WHERE B.ALERT_ID = A.ALERT_ID AND B.LANGUAGE_CODE = h_lang_code)
   THEN h_lang_code ELSE 'EN' END
   ORDER BY A.ALERT_ID;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROJ_ALERT', 67+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROJ_ALERT(68) : alerts.sql(3517) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN SELECT_READ3;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROJ_ALERT', 75+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROJ_ALERT(76) : alerts.sql(3525) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROJ_ALERT', 87+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROJ_ALERT(88) : alerts.sql(3537) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_PROJ_ALERT;
ALTER FUNCTION SP_S_PROJ_ALERT COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION SP_AVAILABLE_ALERT ==
CREATE OR REPLACE FUNCTION SP_AVAILABLE_ALERT
(
     p_project_id VARCHAR2
   , p_lang_code VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_lang_code VARCHAR2(32);
-- Declare Cursors
      -- Resultset Cursor Declaration
      SELECT_READ4 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SELECT_READ5 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_AVAILABLE_ALERT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_AVAILABLE_ALERT'), '-- ' || SYSTIMESTAMP, ':r := SP_AVAILABLE_ALERT(' || 
       '''' || p_project_id || ''''
          || ', ' || '''' || p_lang_code || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_AVAILABLE_ALERT', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_AVAILABLE_ALERT(26) : alerts.sql(3579) : ' || SYSTIMESTAMP);
   END IF; 

   If p_lang_code IS NULL Then
      h_lang_code := 'EN';
   Else
      h_lang_code := p_lang_code;
   End If;
   
   If p_project_id IS NOT NULL Then
      OPEN SELECT_READ4 FOR SELECT A.ALERT_ID, A.ALERT_NAME, A.ALERT_MESSAGE, A.ALERT_BIT
      FROM ALERT_MESSAGES A
      WHERE A.LANGUAGE_CODE = CASE WHEN EXISTS(SELECT B.ALERT_ID FROM ALERT_MESSAGES B
      WHERE B.ALERT_ID = A.ALERT_ID AND B.LANGUAGE_CODE = h_lang_code)
      THEN h_lang_code ELSE 'EN' END
      AND NOT EXISTS(SELECT * FROM PROJECT_ALERTS P
      WHERE P.ALERT_ID = A.ALERT_ID AND P.PROJECT_ID = p_project_id)
      ORDER BY ALERT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_AVAILABLE_ALERT', 44+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_AVAILABLE_ALERT(45) : alerts.sql(3598) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_READ4;
   Else
      OPEN SELECT_READ5 FOR SELECT A.ALERT_ID, A.ALERT_NAME, A.ALERT_MESSAGE, A.ALERT_BIT
      FROM ALERT_MESSAGES A
      WHERE A.LANGUAGE_CODE = CASE WHEN EXISTS(SELECT B.ALERT_ID FROM ALERT_MESSAGES B
      WHERE B.ALERT_ID = A.ALERT_ID AND B.LANGUAGE_CODE = h_lang_code)
      THEN h_lang_code ELSE 'EN' END
      ORDER BY ALERT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_AVAILABLE_ALERT', 55+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_AVAILABLE_ALERT(56) : alerts.sql(3609) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_READ5;
   End If;
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_AVAILABLE_ALERT', 63+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_AVAILABLE_ALERT(64) : alerts.sql(3617) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_AVAILABLE_ALERT', 75+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_AVAILABLE_ALERT(76) : alerts.sql(3629) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_AVAILABLE_ALERT;
ALTER FUNCTION SP_AVAILABLE_ALERT COMPILE DEBUG;

/******************************************************************************/

-- select alert configuration

Prompt == FUNCTION SP_S_ALERT_CONFIG ==
CREATE OR REPLACE FUNCTION SP_S_ALERT_CONFIG
(
     p_alert_id VARCHAR2
   , p_lang_code VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_lang_code VARCHAR2(32);
   h_alert_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      SELECT_ALERT_NAME PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SELECT_ALERT_MSG PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SELECT_ALERT_COL PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_ALERT_CONFIG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_ALERT_CONFIG'), '-- ' || SYSTIMESTAMP, ':r := SP_S_ALERT_CONFIG(' || 
       '''' || p_alert_id || ''''
          || ', ' || '''' || p_lang_code || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ALERT_CONFIG', 33+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_ALERT_CONFIG(34) : alerts.sql(3681) : ' || SYSTIMESTAMP);
   END IF; 

   h_alert_id := p_alert_id;
   h_lang_code := p_lang_code;
   h_rec_user := p_rec_user;
   h_bitflag := p_bitflag;
   
   If (h_alert_id IS NULL) AND (h_lang_code IS NULL) Then
      -- return all alert name
      
      OPEN SELECT_ALERT_NAME FOR SELECT DISTINCT ALERT_ID, ALERT_NAME, ALERT_BIT, '' AS CLIENT_ALERT_NAME 
      FROM ALERT_MESSAGES
      WHERE LANGUAGE_CODE = 'EN'
      ORDER BY ALERT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ALERT_CONFIG', 49+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_ALERT_CONFIG(50) : alerts.sql(3697) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_ALERT_NAME;
   ElsIf (h_alert_id IS NOT NULL) AND (h_lang_code IS NULL) Then
      -- return alert messages for all language
      
      OPEN SELECT_ALERT_MSG FOR SELECT ALERT_ID, LANGUAGE_CODE, TD.DESCRIPTION AS LANGUAGE_NAME, CLIENT_MESSAGE_PATTERN AS ALERT_MESSAGE, '' AS CLIENT_ALERT_NAME 
      FROM ALERT_MESSAGES, TMT_DATAFIELDS TD
      WHERE ALERT_ID = h_alert_id AND LANGUAGE_CODE = TD.ELEMENT_NAME AND TD.TYPE_ID = 33
      ORDER BY LANGUAGE_CODE;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ALERT_CONFIG', 60+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_ALERT_CONFIG(61) : alerts.sql(3708) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_ALERT_MSG;
   Else
      -- return alert messages columns
      
      OPEN SELECT_ALERT_COL FOR SELECT ALERT_ID, ELEMENT_NAME, COLUMN_TYPE, FORMAT
      FROM ALERT_MSG_PREDEF_COLUMN
      WHERE ALERT_ID = h_alert_id
      ORDER BY ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ALERT_CONFIG', 71+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_ALERT_CONFIG(72) : alerts.sql(3719) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_ALERT_COL;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ALERT_CONFIG', 80+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ALERT_CONFIG(81) : alerts.sql(3728) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ALERT_CONFIG', 92+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ALERT_CONFIG(93) : alerts.sql(3740) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_ALERT_CONFIG;
ALTER FUNCTION SP_S_ALERT_CONFIG COMPILE DEBUG;

/******************************************************************************/

/*
    this function uses new client message pattern and :
    1- checks that if the new message is different from current message pattren or not(the number of selected columns
    and selected column set are used for comparision)
    2- rebuild a new message pattern
*/

Prompt == FUNCTION FC_BUILD_ALERT_MESSAGE ==
CREATE OR REPLACE FUNCTION FC_BUILD_ALERT_MESSAGE
(
     sh_h_rebuild_pattern PLS_INTEGER
   , h_alert_id IN OUT NOCOPY VARCHAR2
   , h_client_message_pattern IN OUT NOCOPY VARCHAR2
   , h_alert_message IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_rebuild_pattern PLS_INTEGER := sh_h_rebuild_pattern;
   h_temp_sc_state PLS_INTEGER NOT NULL := 0;
   h_buffer VARCHAR2(1024);
   h_start_of_exp PLS_INTEGER := 0;
   h_end_of_exp PLS_INTEGER := 0;
   h_expression VARCHAR2(255);
   h_column_location PLS_INTEGER := 0;
   h_default_column_location PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   eom PLS_INTEGER NOT NULL := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_BUILD_ALERT_MESSAGE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_BUILD_ALERT_MESSAGE'), '-- ' || SYSTIMESTAMP, ':r := FC_BUILD_ALERT_MESSAGE(' || 
       COALESCE(TO_CHAR(sh_h_rebuild_pattern), 'NULL')
          || ', ' || '''' || h_alert_id || ''''
          || ', ' || '''' || h_client_message_pattern || ''''
          || ', ' || '''' || h_alert_message || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_BUILD_ALERT_MESSAGE', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_BUILD_ALERT_MESSAGE(29) : alerts.sql(3792) : ' || SYSTIMESTAMP);
   END IF; 

   h_column_location := 0;
   
   If Coalesce(h_rebuild_pattern, 0) = 1 Then
      If NOT ((PMO_ORASTD.strncmp(h_alert_id, 'SCCLOSEDWORKFLOW', 16) = 0) OR (PMO_ORASTD.strncmp(
            h_alert_id, 'SCCOMPLETEDWORKFLOW', 19) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
            , 'SCCOSTVARIANCE', 14) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
            , 'SCDEPENDENCYDETAILSCHANGED', 26) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
            , 'SCFSPROJECTCREATED', 18) = 0) OR (PMO_ORASTD.strncmp(h_alert_id, 'SCFSTASKREJECTED'
            , 16) = 0) OR (PMO_ORASTD.strncmp(h_alert_id, 'SCFSWEBSERVERPATH', 17) = 0) 
            OR (PMO_ORASTD.strncmp(h_alert_id, 'SCMAILGROUPMEMBER', 17) = 0) OR (PMO_ORASTD.strncmp(
            h_alert_id, 'SCNEWCROSSPROJECTDEPENDENCY', 27) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
            , 'SCNEWDEPENDENCY', 15) = 0) OR (PMO_ORASTD.strncmp(h_alert_id, 'SCNEWPROJECTPUBLISHED'
            , 21) = 0) OR (PMO_ORASTD.strncmp(h_alert_id, 'SCNEWTEMPLATE', 13) = 0) 
            OR (PMO_ORASTD.strncmp(h_alert_id, 'SCNEWWORKFLOW', 13) = 0) OR (PMO_ORASTD.strncmp(
            h_alert_id, 'SCRESOURCEASSIGNMENTCHANGE', 26) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
            , 'SCRESOURCERECORDREAPPROVAL', 26) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
            , 'SCREVOTEDWORKFLOW', 17) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
            , 'SCSCHEDULEASSIGNMENT', 20) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
            , 'SCSCHEDULEVARIANCE', 18) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
            , 'SCWORKFLOWVOTESCOMPLETE', 23) = 0)) Then
         BEGIN
            UPDATE ALERT_MSG_PREDEF_COLUMN
            SET COLUMN_LOCATION = 0
            WHERE ALERT_ID = h_alert_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_MESSAGE', 65+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_MESSAGE(66) : alerts.sql(3829) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   h_alert_message := '';
   h_buffer := h_client_message_pattern;
   
   WHILE eom = 0 LOOP
      BEGIN
         SELECT PMO_ORASTD.LOCATE('[', h_buffer), PMO_ORASTD.LOCATE(']', h_buffer)
         INTO h_start_of_exp, h_end_of_exp
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_MESSAGE', 86+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_MESSAGE(87) : alerts.sql(3850) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (Coalesce(h_start_of_exp, 0) = 0) AND (Coalesce(h_end_of_exp, 0) = 0) Then
         -- end of message (no more expression)
         eom := 1;
         
         BEGIN
            SELECT  h_alert_message || h_buffer
            INTO h_alert_message
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_MESSAGE', 105+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_MESSAGE(106) : alerts.sql(3869) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf ((Coalesce(h_start_of_exp, 0) = 0) AND (Coalesce(h_end_of_exp, 0) <> 0)) OR ((Coalesce(
                 h_start_of_exp, 0) <> 0) AND (Coalesce(h_end_of_exp, 0) = 0)) Then
         -- incorrect expression definition
         eom := 1;
         h_temp_sc_state := PMO_CNST.INVALID_PARAMS;
      Else
         -- an expression has been found
         BEGIN
            SELECT  SUBSTR(h_buffer, h_start_of_exp + 1, h_end_of_exp - h_start_of_exp - 1)
            INTO h_expression
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_MESSAGE', 126+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_MESSAGE(127) : alerts.sql(3890) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT COLUMN_LOCATION
            INTO h_default_column_location
            FROM ALERT_MSG_PREDEF_COLUMN
            WHERE ALERT_ID  = h_alert_id
            AND ELEMENT_NAME = h_expression;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_MESSAGE', 143+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_MESSAGE(144) : alerts.sql(3907) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA OR ((Coalesce(h_default_column_location, 0) = 0) 
               AND (Coalesce(h_rebuild_pattern, 0) <> 1)) Then
            h_temp_sc_state := PMO_CNST.INVALID_PARAMS;
            eom := 1;
            h_alert_message := NULL;
         Else
            If Coalesce(h_rebuild_pattern, 0) = 1 Then
               If Coalesce(h_default_column_location, 0) <> 0 Then
                  h_column_location := Coalesce(h_default_column_location, 0);
               Else
                  h_column_location := Coalesce(h_column_location, 0) + 1;
                  
                  BEGIN
                     UPDATE ALERT_MSG_PREDEF_COLUMN
                     SET COLUMN_LOCATION = h_column_location
                     WHERE ALERT_ID  = h_alert_id
                     AND ELEMENT_NAME = h_expression;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_MESSAGE', 175+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_MESSAGE(176) : alerts.sql(3939) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               h_column_location := Coalesce(h_default_column_location, 0);
            End If;
            
            BEGIN
               SELECT  h_alert_message || SUBSTR(h_buffer, 1, h_start_of_exp-1) || '%' || RTRIM(TO_CHAR(h_column_location)) || '%',
               SUBSTR(h_buffer, h_end_of_exp+1, LENGTH(h_buffer)-h_end_of_exp)
               INTO h_alert_message, h_buffer
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_BUILD_ALERT_MESSAGE', 195+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_BUILD_ALERT_MESSAGE(196) : alerts.sql(3959) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            h_end_of_exp := 0;
            h_start_of_exp := 0;
         End If;
      End If;
   END LOOP;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_BUILD_ALERT_MESSAGE', 206+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_BUILD_ALERT_MESSAGE(207) : alerts.sql(3970) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN h_temp_sc_state;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_BUILD_ALERT_MESSAGE', 210+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_BUILD_ALERT_MESSAGE(211) : alerts.sql(3974) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_BUILD_ALERT_MESSAGE;
ALTER FUNCTION FC_BUILD_ALERT_MESSAGE COMPILE DEBUG;

/******************************************************************************/

-- update alert configuartion

Prompt == FUNCTION SP_U_ALERT_CONFIG ==
CREATE OR REPLACE FUNCTION SP_U_ALERT_CONFIG
(
     p_alert_id VARCHAR2
   , p_lang_code VARCHAR2
   , p_client_message_pattern VARCHAR2
   , p_client_alert_name VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_client_message_pattern VARCHAR2(1024);
   h_alert_message VARCHAR2(1024);
   h_lang_code VARCHAR2(32);
   h_alert_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_rebuild_pattern PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_ALERT_CONFIG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_ALERT_CONFIG'), '-- ' || SYSTIMESTAMP, ':r := SP_U_ALERT_CONFIG(' || 
       '''' || p_alert_id || ''''
          || ', ' || '''' || p_lang_code || ''''
          || ', ' || '''' || p_client_message_pattern || ''''
          || ', ' || '''' || p_client_alert_name || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_ALERT_CONFIG', 32+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_ALERT_CONFIG(33) : alerts.sql(4019) : ' || SYSTIMESTAMP);
   END IF; 

   h_alert_id := p_alert_id;
   h_lang_code := p_lang_code;
   h_client_message_pattern := p_client_message_pattern;
   h_bitflag := p_bitflag;
   h_rec_user := p_rec_user;
   
   -- rules:
   --1- default language is 'EN'
   --2- if user update the default language pattern then copy the new pattern to all other defined languages for the selected message
   --3- if the number of selected columns or selected columns for a message updated then rebuild the default message pattern (EN) and
   --      also copy this new pattern to other languages.
   
   BEGIN
      UPDATE ALERT_MESSAGES
      SET ALERT_BIT = h_bitflag
      WHERE ALERT_ID = h_alert_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ALERT_CONFIG', 61+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ALERT_CONFIG(62) : alerts.sql(4048) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- request to change the default language
   If PMO_ORASTD.strncmp(h_lang_code, 'EN', 2) = 0 Then
      h_rebuild_pattern := 1;
      h_temp_sc_state := FC_BUILD_ALERT_MESSAGE(h_rebuild_pattern, h_alert_id, h_client_message_pattern
                                          , h_alert_message);
      
      If h_temp_sc_state <> 0 Then
         -- copy new message patten to the other predefined languages
         BEGIN
            UPDATE ALERT_MESSAGES
            SET ALERT_MESSAGE = h_alert_message,
            CLIENT_MESSAGE_PATTERN = h_client_message_pattern
            WHERE ALERT_ID = h_alert_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ALERT_CONFIG', 89+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ALERT_CONFIG(90) : alerts.sql(4076) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         -- just update selected language
         -- update string
         BEGIN
            UPDATE ALERT_MESSAGES
            SET ALERT_MESSAGE = h_alert_message,
            CLIENT_MESSAGE_PATTERN = h_client_message_pattern
            WHERE ALERT_ID = h_alert_id
            AND LANGUAGE_CODE = h_lang_code;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ALERT_CONFIG', 112+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ALERT_CONFIG(113) : alerts.sql(4099) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      -- request to update other languages
      h_rebuild_pattern := 0;
      h_temp_sc_state := FC_BUILD_ALERT_MESSAGE(h_rebuild_pattern, h_alert_id, h_client_message_pattern
                                          , h_alert_message);
      
      If h_temp_sc_state <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_ALERT_CONFIG', 125+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_ALERT_CONFIG(126) : alerts.sql(4112) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_sc_state), 'NULL') || ' ID: ' || '''' || h_client_message_pattern || '''');
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR(h_temp_sc_state, h_client_message_pattern);
      Else
         -- update string
         BEGIN
            UPDATE ALERT_MESSAGES
            SET ALERT_MESSAGE = h_alert_message,
            CLIENT_MESSAGE_PATTERN = h_client_message_pattern
            WHERE ALERT_ID = h_alert_id
            AND LANGUAGE_CODE = h_lang_code;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ALERT_CONFIG', 146+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ALERT_CONFIG(147) : alerts.sql(4133) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_ALERT_CONFIG', 154+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_ALERT_CONFIG(155) : alerts.sql(4141) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_alert_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_alert_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ALERT_CONFIG', 161+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ALERT_CONFIG(162) : alerts.sql(4148) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ALERT_CONFIG', 173+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ALERT_CONFIG(174) : alerts.sql(4160) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_ALERT_CONFIG;
ALTER FUNCTION SP_U_ALERT_CONFIG COMPILE DEBUG;

/************************************************************************/

Prompt == FUNCTION FC_APPLY_FORMAT ==
CREATE OR REPLACE FUNCTION FC_APPLY_FORMAT
(
     h_value IN OUT NOCOPY VARCHAR2
   , h_format_str IN OUT NOCOPY VARCHAR2
   , h_column_type IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_APPLY_FORMAT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_APPLY_FORMAT'), '-- ' || SYSTIMESTAMP, ':r := FC_APPLY_FORMAT(' || 
       '''' || h_value || ''''
          || ', ' || '''' || h_format_str || ''''
          || ', ' || '''' || h_column_type || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_APPLY_FORMAT', 16+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_APPLY_FORMAT(17) : alerts.sql(4193) : ' || SYSTIMESTAMP);
   END IF; 

   If PMO_ORASTD.strncmp(h_format_str, PMO_CNST.SC_INT_FORMAT, 2) = 0 Then
      h_value := TO_CHAR(TO_NUMBER(h_value));
   ElsIf PMO_ORASTD.strncmp(h_format_str, PMO_CNST.SC_TIME_FORMAT, 2) = 0 Then
      /*      strncpy(h_value, h_value+11, 8);
              *(h_value + 8) = '\0';
              *(h_value + 2) = ':';
              *(h_value + 5) = ':';*/
      BEGIN
         SELECT TO_CHAR(PMO_ORASTD.TIME(h_value))
         INTO h_value
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_APPLY_FORMAT', 36+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_APPLY_FORMAT(37) : alerts.sql(4213) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf PMO_ORASTD.strncmp(h_format_str, PMO_CNST.SC_DATE_FORMAT, 2) = 0 Then
      h_value := SubStr(h_value, 1, 10);
   ElsIf PMO_ORASTD.strncmp(h_format_str, PMO_CNST.SC_DOLLAR_FORMAT, 2) = 0 Then
      h_value := '$' || TO_CHAR(TO_NUMBER(h_value), 'FM999999999999999999.99');
   ElsIf PMO_ORASTD.strncmp(h_format_str, PMO_CNST.SC_DECIMAL_FORMAT, 2) = 0 Then
      h_value := TO_CHAR(TO_NUMBER(h_value), 'FM999999999999999999.99');
   ElsIf PMO_ORASTD.strncmp(h_format_str, PMO_CNST.SC_HOUR_FORMAT, 2) = 0 Then
      h_value := TO_CHAR(TO_NUMBER(h_value) / 60.0, 'FM999999999999999999.99') || ' hours';
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_APPLY_FORMAT', 51+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_APPLY_FORMAT(52) : alerts.sql(4228) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_APPLY_FORMAT;
ALTER FUNCTION FC_APPLY_FORMAT COMPILE DEBUG;

/************************************************************************/

Prompt == FUNCTION FC_CALCULATE_DELTA_ETC ==
CREATE OR REPLACE FUNCTION FC_CALCULATE_DELTA_ETC
(
     h_alert_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_old_sched_work PLS_INTEGER := 0;
   h_old_planned_work PLS_INTEGER := 0;
   h_old_work_todate PLS_INTEGER := 0;
   h_old_etc PLS_INTEGER := 0;
   h_new_sched_work PLS_INTEGER := 0;
   h_new_planned_work PLS_INTEGER := 0;
   h_new_work_todate PLS_INTEGER := 0;
   h_new_etc PLS_INTEGER := 0;
   h_variance PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CALCULATE_DELTA_ETC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CALCULATE_DELTA_ETC'), '-- ' || SYSTIMESTAMP, ':r := FC_CALCULATE_DELTA_ETC(' || 
       '''' || h_alert_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALCULATE_DELTA_ETC', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CALCULATE_DELTA_ETC(22) : alerts.sql(4260) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT Trunc(OLD_VALUE), Trunc(NEW_VALUE)
      INTO h_old_etc, h_new_etc
      FROM PMO_ALERTS_VALUE_LIST
      WHERE ALERT_ID = h_alert_id
      AND ELEMENT_ID = 'OLD_ETC';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CALCULATE_DELTA_ETC', 36+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CALCULATE_DELTA_ETC(37) : alerts.sql(4275) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT Trunc(OLD_VALUE), Trunc(NEW_VALUE)
      INTO h_old_planned_work, h_new_planned_work
      FROM PMO_ALERTS_VALUE_LIST
      WHERE ALERT_ID = h_alert_id
      AND ELEMENT_ID = 'OLD_PLANNED_WORK';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CALCULATE_DELTA_ETC', 53+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CALCULATE_DELTA_ETC(54) : alerts.sql(4292) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT Trunc(OLD_VALUE), Trunc(NEW_VALUE)
      INTO h_old_sched_work, h_new_sched_work
      FROM PMO_ALERTS_VALUE_LIST
      WHERE ALERT_ID = h_alert_id
      AND ELEMENT_ID = 'OLD_SCHED_WORK';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CALCULATE_DELTA_ETC', 70+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CALCULATE_DELTA_ETC(71) : alerts.sql(4309) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT Trunc(OLD_VALUE), Trunc(NEW_VALUE)
      INTO h_old_work_todate, h_new_work_todate
      FROM PMO_ALERTS_VALUE_LIST
      WHERE ALERT_ID = h_alert_id
      AND ELEMENT_ID = 'OLD_WORK_TODATE';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CALCULATE_DELTA_ETC', 87+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CALCULATE_DELTA_ETC(88) : alerts.sql(4326) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_old_work_todate, 0) = Coalesce(h_new_work_todate, 0) AND Coalesce(
                                             h_old_etc, 0) <> Coalesce(h_new_etc, 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALCULATE_DELTA_ETC', 95+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_CALCULATE_DELTA_ETC(96) : alerts.sql(4334) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 1;
   End If;
   
   h_variance := (Coalesce(h_new_sched_work, 0) + Coalesce(h_new_planned_work, 0)) - Coalesce(
                        h_new_work_todate, 0) - Coalesce(h_new_etc, 0);
   
   If PMO_ORASTD.strncmp(h_alert_id, 'SCETCFORCASTCHANGED', 19) = 0 Then
      If Coalesce(h_variance, 0) <= 0 Then
         h_variance := Coalesce(h_variance, 0) - (Coalesce(h_old_sched_work, 0) + Coalesce(
                       h_old_planned_work, 0)) - Coalesce(h_old_work_todate, 0) - Coalesce(
                       h_old_etc, 0);
         h_variance := CASE WHEN (Coalesce(h_variance, 0) = 0) THEN 0 ELSE 1 END;
      Else
         h_variance := 0;
         NULL;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALCULATE_DELTA_ETC', 116+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CALCULATE_DELTA_ETC(117) : alerts.sql(4355) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN Coalesce(h_variance, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALCULATE_DELTA_ETC', 121+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CALCULATE_DELTA_ETC(122) : alerts.sql(4360) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CALCULATE_DELTA_ETC;
ALTER FUNCTION FC_CALCULATE_DELTA_ETC COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_GetRTFItemName ==
CREATE OR REPLACE FUNCTION FC_GetRTFItemName
(
     sh_h_element_type PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , h_rtf_item_name IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_element_type PLS_INTEGER := sh_h_element_type;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_parent_id PLS_INTEGER := 0;
   h_position PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GetRTFItemName') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GetRTFItemName'), '-- ' || SYSTIMESTAMP, ':r := FC_GetRTFItemName(' || 
       COALESCE(TO_CHAR(sh_h_element_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_rtf_item_name || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GetRTFItemName', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GetRTFItemName(21) : alerts.sql(4391) : ' || SYSTIMESTAMP);
   END IF; 

   --Declare local variables
   
   If (Coalesce(h_element_type, 0) = PMO_CNST.SC_PROJECT) OR (Coalesce(
             h_element_type, 0) = PMO_CNST.SC_PROPOSAL) OR (Coalesce(
             h_element_type, 0) = PMO_CNST.SC_PROJ_TYPE1) OR (Coalesce(
             h_element_type, 0) = PMO_CNST.SC_PROJ_TYPE2) OR (Coalesce(
             h_element_type, 0) = PMO_CNST.SC_PROJ_TYPE3) Then
      h_parent_id := 5562;
   ElsIf (Coalesce(h_element_type, 0) = PMO_CNST.SC_DELIVERABLE) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_DELIVERABLE) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_DELIVERABLE_TYPE1) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_DELIVERABLE_TYPE2) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_DELIVERABLE_TYPE3) Then
      h_parent_id := 5563;
   ElsIf (Coalesce(h_element_type, 0) = PMO_CNST.SC_WORK_PRODUCT) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_WORK_PRODUCT) Then
      h_parent_id := 5564;
   ElsIf (Coalesce(h_element_type, 0) = PMO_CNST.SC_SUMMARY_TASK) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_SUMMARY_TASK) Then
      h_parent_id := 5565;
   ElsIf (Coalesce(h_element_type, 0) = PMO_CNST.SC_MILESTONE) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_MILESTONE) Then
      h_parent_id := 5566;
   ElsIf (Coalesce(h_element_type, 0) = PMO_CNST.SC_TASK) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_TASK) Then
      h_parent_id := 5567;
   ElsIf (Coalesce(h_element_type, 0) = PMO_CNST.SC_CRI_REQUIREMENT) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_REQUIREMENT_TASK) Then
      h_parent_id := 5568;
   ElsIf (Coalesce(h_element_type, 0) = PMO_CNST.SC_CRI_CHANGE_REQUEST) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_CHANGE_REQUEST_TASK) Then
      h_parent_id := 5569;
   ElsIf (Coalesce(h_element_type, 0) = PMO_CNST.SC_CRI_SERVICE_REQUEST) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_SERVICE_REQUEST_TASK) Then
      h_parent_id := 5570;
   ElsIf (Coalesce(h_element_type, 0) = PMO_CNST.SC_CRI_DEFECT) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_DEFECT_TASK) Then
      h_parent_id := 5571;
   ElsIf (Coalesce(h_element_type, 0) = PMO_CNST.SC_CRI_ISSUE) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_ISSUE_TASK) Then
      h_parent_id := 5572;
   ElsIf (Coalesce(h_element_type, 0) = PMO_CNST.SC_CRI_RISK) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_RISK_TASK) Then
      h_parent_id := 5573;
   ElsIf Coalesce(h_element_type, 0) = PMO_CNST.SC_ASSETS Then
      h_parent_id := 6039;
   ElsIf Coalesce(h_element_type, 0) = PMO_CNST.SC_CLIENTS Then
      h_parent_id := 6040;
   ElsIf (Coalesce(h_element_type, 0) = PMO_CNST.SC_WBS_ACTION_ITEM) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_ACTION_ITEM) OR (Coalesce(
                h_element_type, 0) = PMO_CNST.SC_CRI_ACTION_ITEM_TASK) Then
      h_parent_id := 6083;
   ElsIf Coalesce(h_element_type, 0) = PMO_CNST.SC_NOTE_MINUTES Then
      h_parent_id := 6232;
   End If;
   
   h_position := 1;
   
   WHILE Coalesce(h_bitflag, 0) > 1 LOOP
      h_bitflag := PMO_FUNC.ShiftRight(Coalesce(h_bitflag, 0), 1);
      h_position := Coalesce(h_position, 0) + 1;
   END LOOP;
   
   BEGIN
      SELECT DESCRIPTION INTO h_rtf_item_name
      FROM TMT_DATAFIELDS
      WHERE PARENT_ID = h_parent_id
      AND ELEMENT_NAME = TO_CHAR(h_position);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GetRTFItemName', 97+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GetRTFItemName(98) : alerts.sql(4468) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GetRTFItemName', 103+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GetRTFItemName(104) : alerts.sql(4474) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GetRTFItemName', 107+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GetRTFItemName(108) : alerts.sql(4478) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GetRTFItemName;
ALTER FUNCTION FC_GetRTFItemName COMPILE DEBUG;

/**********************************************************************************************/

Prompt == FUNCTION FC_ISSUE_RTF_ALERT ==
CREATE OR REPLACE FUNCTION FC_ISSUE_RTF_ALERT
(
     h_alert_id IN OUT NOCOPY VARCHAR2
   , h_id IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_calltype PLS_INTEGER
   , sh_h_table_type PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_calltype PLS_INTEGER := sh_h_calltype;
   h_table_type PLS_INTEGER := Coalesce(sh_h_table_type, 0);
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_element_type PLS_INTEGER := 0;
   h_rtf_item_name VARCHAR2(50);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ISSUE_RTF_ALERT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ISSUE_RTF_ALERT'), '-- ' || SYSTIMESTAMP, ':r := FC_ISSUE_RTF_ALERT(' || 
       '''' || h_alert_id || ''''
          || ', ' || '''' || h_id || ''''
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_calltype), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_table_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ISSUE_RTF_ALERT', 27+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ISSUE_RTF_ALERT(28) : alerts.sql(4516) : ' || SYSTIMESTAMP);
   END IF; 

   CASE h_table_type
      WHEN PMO_CNST.SC_TABLE_WBS THEN
         BEGIN
            SELECT TYPE_ID INTO h_element_type
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_RTF_ALERT', 42+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_RTF_ALERT(43) : alerts.sql(4531) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_CRI THEN
         BEGIN
            SELECT TYPE_ID INTO h_element_type
            FROM TMT_CRI
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_RTF_ALERT', 60+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_RTF_ALERT(61) : alerts.sql(4549) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_DOC THEN
         BEGIN
            SELECT TYPE_ID INTO h_element_type
            FROM TMT_DOCUMENTS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_RTF_ALERT', 78+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_RTF_ALERT(79) : alerts.sql(4567) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      ELSE
         NULL;
   END CASE;
   
   PMO_DB2M.SC_STATE := FC_GetRTFItemName(h_element_type, h_bitflag, h_rtf_item_name);
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_ALERTS_PARAM_VALUE';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_ALERTS_PARAM_VALUE (ALERT_ID, ELEMENT_ID, VALUE)
      VALUES ('SCRTFITEMCHANGED', 'RTF_ITEM_NAME', h_rtf_item_name);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_RTF_ALERT', 106+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_RTF_ALERT(107) : alerts.sql(4595) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_DB2M.SC_STATE := FC_ISSUE_ALERT(h_alert_id, h_id, h_rec_user, h_calltype, h_table_type);
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ISSUE_RTF_ALERT', 115+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ISSUE_RTF_ALERT(116) : alerts.sql(4604) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ISSUE_RTF_ALERT;
ALTER FUNCTION FC_ISSUE_RTF_ALERT COMPILE DEBUG;

/************************************************************************/

-- if h_calltype = 0 it means first call (dont create temporary table)

-- if h_calltype = 1 it means first call (create temporary table)

-- if h_calltype = 2 it means second call

-- if h_calltype = 3 it means first and only call (create temporary table)

Prompt == FUNCTION FC_ISSUE_ALERT ==
CREATE OR REPLACE FUNCTION FC_ISSUE_ALERT
(
     h_alert_id IN OUT NOCOPY VARCHAR2
   , h_id IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_calltype PLS_INTEGER
   , sh_h_table_type PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_calltype PLS_INTEGER := sh_h_calltype;
   h_table_type PLS_INTEGER := Coalesce(sh_h_table_type, 0);
   h_msg_rank VARCHAR2(200);
   h_column_name VARCHAR2(50);
   h_element_id VARCHAR2(32);
   h_table_name VARCHAR2(50);
   h_column_location PLS_INTEGER := 0;
   h_primary_key VARCHAR2(32);
   h_main_table_name VARCHAR2(50);
   h_parent_table VARCHAR2(50);
   h_format_str VARCHAR2(50);
   h_column_type VARCHAR2(50);
   h_uniqueID VARCHAR2(32);
   h_value VARCHAR2(1024);
   h_type_id PLS_INTEGER := 0;
   h_task_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   myStmt VARCHAR2(4000);
   h_value_list VARCHAR2(4000);
   tempStr VARCHAR2(49);
   h_variance PLS_INTEGER := 0;
   h_issue_alert PLS_INTEGER := 0;
   h_wkf_def_id VARCHAR2(32);
   h_temp_id VARCHAR2(32);
   h_stage_id PLS_INTEGER := 0;
   h_pred_id VARCHAR2(32);
   h_pred_project_id VARCHAR2(32);
   h_succ_id VARCHAR2(32);
   h_succ_project_id VARCHAR2(32);
   h_pred_start_dt TIMESTAMP;
   h_pred_finish_dt TIMESTAMP;
   h_succ_start_dt TIMESTAMP;
   h_succ_finish_dt TIMESTAMP;
   h_new_pred_start_dt TIMESTAMP;
   h_new_pred_finish_dt TIMESTAMP;
   h_new_succ_start_dt TIMESTAMP;
   h_new_succ_finish_dt TIMESTAMP;
   h_pred_element_name VARCHAR2(255);
   h_succ_element_name VARCHAR2(255);
   null_return LONG;
-- Declare Cursors
      CURSOR old_column_recset IS
         SELECT ELEMENT_ID, TABLE_NAME, COLUMN_NAME, COLUMN_LOCATION, FORMAT, COLUMN_TYPE
         FROM ALERT_MSG_PREDEF_COLUMN
         WHERE ALERT_ID = h_alert_id
         AND TYPE_ID = PMO_CNST.SC_OLD_VALUE_COLUMN;

      CURSOR trig_col_recset IS
         SELECT ELEMENT_ID, TABLE_NAME, COLUMN_NAME
         FROM ALERT_MSG_PREDEF_COLUMN
         WHERE ALERT_ID = h_alert_id
         AND TYPE_ID = PMO_CNST.SC_OLD_VALUE_COLUMN;

      CURSOR realColumnRecset IS
         SELECT ELEMENT_ID, TABLE_NAME, COLUMN_NAME, COLUMN_LOCATION, FORMAT, COLUMN_TYPE
         FROM ALERT_MSG_PREDEF_COLUMN P
         WHERE ALERT_ID = h_alert_id
         AND ELEMENT_ID NOT IN (SELECT ELEMENT_ID FROM PMO_ALERTS_VALUE_LIST WHERE ALERT_ID = h_alert_id)
         AND COLUMN_LOCATION <> 0;

      -- Dynamic Cursor Declaration
      value_recset1 PMO_DB2M.REF_CURSOR;

      -- Dynamic Cursor Declaration
      value_recset2 PMO_DB2M.REF_CURSOR;

      -- Dynamic Cursor Declaration
      value_recset3 PMO_DB2M.REF_CURSOR;

      -- Dynamic Cursor Declaration
      value_recset4 PMO_DB2M.REF_CURSOR;

      -- Dynamic Cursor Declaration
      value_recset5 PMO_DB2M.REF_CURSOR;

      CURSOR valueListRecset IS
         SELECT RTRIM(OLD_VALUE), TYPE_ID, FORMAT, COLUMN_TYPE
         FROM PMO_ALERTS_VALUE_LIST
         WHERE LOC <>0
         AND ALERT_ID = h_alert_id
         ORDER BY LOC;

      CURSOR res_prj_set IS
         SELECT PROJECT_ID, TASK_ID
         FROM TASK_ASSIGNMENTS
         WHERE RESOURCE_ID = h_rec_user;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ISSUE_ALERT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ISSUE_ALERT'), '-- ' || SYSTIMESTAMP, ':r := FC_ISSUE_ALERT(' || 
       '''' || h_alert_id || ''''
          || ', ' || '''' || h_id || ''''
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_calltype), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_table_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ISSUE_ALERT', 107+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ISSUE_ALERT(108) : alerts.sql(4730) : ' || SYSTIMESTAMP);
   END IF; 

   h_type_id := 0;
   
   If (PMO_ORASTD.strncmp(h_alert_id, 'SCETCFORCASTCHANGED', 19) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
         , 'SCETCFORCASTREJECTED', 20) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
         , 'SCETCFORCASTACCEPTED', 20) = 0) OR (PMO_ORASTD.strncmp(h_alert_id, 'SCREDUCEDETC'
         , 12) = 0) OR (PMO_ORASTD.strncmp(h_alert_id, 'SCTIMESHEETLATEREMINDER', 23) = 0) Then
      h_primary_key := 'TSK_ASSIGNMENT_ID';
      h_main_table_name := 'TASK_ASSIGNMENTS';
      h_parent_table := 'TMT_WBS';
   ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCPERSONALCALENDARCHANGED', 25) = 0 Then
      h_primary_key := 'EXCEPTION_ID';
      h_main_table_name := 'CALENDAR_EXCEPTIONS';
      h_parent_table := 'TMT_WBS';
   ElsIf (PMO_ORASTD.strncmp(h_alert_id, 'SCTASKCOMPLETED', 15) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
         , 'SCTASKREADYTOSTART', 18) = 0) OR (PMO_ORASTD.strncmp(h_alert_id, 'SCTASKTOBESTARTED'
         , 17) = 0) OR (PMO_ORASTD.strncmp(h_alert_id, 'SCTASKLATETOSTART', 17) = 0) 
         OR (PMO_ORASTD.strncmp(h_alert_id, 'SCTASKLATETOFINISH', 18) = 0) OR (PMO_ORASTD.strncmp(
         h_alert_id, 'SCRTFITEMCHANGED', 16) = 0) Then
      h_primary_key := 'ELEMENT_ID';
      h_main_table_name := 'TMT_WBS';
      h_parent_table := 'TMT_WBS';
   ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCNOTRESPONDEDTOWORKFLOWITEM', 28) = 0 Then
      h_primary_key := 'ELEMENT_ID';
      h_main_table_name := 'TMT_WORKFLOWS';
      h_parent_table := 'TMT_WORKFLOWS';
   ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCPREDECESSORDATESCHANGED', 25) = 0 Then
      h_primary_key := 'DEPEND_ID';
      h_main_table_name := 'DEPENDENCIES';
      h_parent_table := 'TMT_WBS';
      
      BEGIN
         SELECT PRED_ID, PRED_PROJECT_ID, SUCC_ID, SUCC_PROJECT_ID, PRED_START_DT, PRED_FINISH_DT,
         NEW_PRED_START_DT, NEW_PRED_FINISH_DT, PRED_ELEMENT_NAME, SUCC_ELEMENT_NAME
         INTO h_pred_id, h_pred_project_id, h_succ_id, h_succ_project_id, h_pred_start_dt,
         h_pred_finish_dt, h_new_pred_start_dt, h_new_pred_finish_dt, h_pred_element_name, h_succ_element_name
         FROM PMO_PRED_DATES_CHANGED_LIST
         WHERE DEPEND_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 153+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(154) : alerts.sql(4776) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If Coalesce(h_calltype, 0) <> 2 Then
      If Coalesce(h_calltype, 0) <> 0 Then
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM PMO_ALERTS_VALUE_LIST';
         PMO_DB2M.SC_STATE := 0;
      End If;
      
      -- select all trigger/old columns if existing
      
      OPEN old_column_recset;
      
      LOOP
         FETCH old_column_recset INTO h_element_id, h_table_name, h_column_name,
         h_column_location, h_format_str, h_column_type;
         IF old_column_recset%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         myStmt := 
               'INSERT INTO PMO_ALERTS_VALUE_LIST (ALERT_ID, ELEMENT_ID, OLD_VALUE, LOC, TYPE_ID, FORMAT, COLUMN_TYPE) ';
         myStmt := myStmt || ' SELECT ''';
         myStmt := myStmt || h_alert_id;
         myStmt := myStmt || ''', ''';
         myStmt := myStmt || h_element_id;
         
         If PMO_ORASTD.strncmp(h_column_type, PMO_CNST.SC_CHAR_COLUMN_TYPE
               , Length(PMO_CNST.SC_CHAR_COLUMN_TYPE)) = 0 Then
            myStmt := myStmt || ''', ';
            myStmt := myStmt || h_column_name;
            myStmt := myStmt || ', ';
         Else
            myStmt := myStmt || ''', TO_CHAR(';
            myStmt := myStmt || h_column_name;
            myStmt := myStmt || '), ';
         End If;
         
         tempStr := TO_CHAR(Coalesce(h_column_location, 0));
         myStmt := myStmt || tempStr;
         myStmt := myStmt || ', ';
         tempStr := TO_CHAR(PMO_CNST.SC_OLD_VALUE_COLUMN);
         myStmt := myStmt || tempStr;
         myStmt := myStmt || ', ''';
         myStmt := myStmt || h_format_str;
         myStmt := myStmt || ''', ''';
         myStmt := myStmt || h_column_type;
         myStmt := myStmt || '''';
         myStmt := myStmt || ' FROM ';
         myStmt := myStmt || h_main_table_name;
         myStmt := myStmt || ' WHERE ';
         myStmt := myStmt || h_primary_key;
         myStmt := myStmt || ' = ''';
         myStmt := myStmt || h_id;
         myStmt := myStmt || '''';
         
         BEGIN
            EXECUTE IMMEDIATE myStmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 231+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(232) : alerts.sql(4854) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || myStmt);
               END IF; 
            END;
         END;
      END LOOP;
      
      BEGIN
         CLOSE old_column_recset;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 246+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(247) : alerts.sql(4869) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If Coalesce(h_calltype, 0) >= 2 Then
      -- fetch new values for old columns
      
      OPEN trig_col_recset;
      
      LOOP
         FETCH trig_col_recset INTO h_element_id, h_table_name, h_column_name;
         IF trig_col_recset%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         myStmt := ' UPDATE PMO_ALERTS_VALUE_LIST ';
         myStmt := myStmt || ' SET NEW_VALUE = ( ';
         
         If PMO_ORASTD.strncmp(h_column_type, PMO_CNST.SC_CHAR_COLUMN_TYPE
               , Length(PMO_CNST.SC_CHAR_COLUMN_TYPE)) = 0 Then
            myStmt := myStmt || ' SELECT ';
            myStmt := myStmt || h_column_name;
         Else
            myStmt := myStmt || ' SELECT TO_CHAR(';
            myStmt := myStmt || h_column_name;
            myStmt := myStmt || ') ';
         End If;
         
         myStmt := myStmt || ' FROM ';
         myStmt := myStmt || h_main_table_name;
         myStmt := myStmt || ' WHERE ';
         myStmt := myStmt || h_primary_key;
         myStmt := myStmt || ' = ''';
         myStmt := myStmt || h_id;
         myStmt := myStmt || ''') ';
         myStmt := myStmt || ' WHERE ALERT_ID = ''';
         myStmt := myStmt || h_alert_id;
         myStmt := myStmt || ' '' AND ELEMENT_ID = ''';
         myStmt := myStmt || h_element_id;
         myStmt := myStmt || '''';
         
         BEGIN
            EXECUTE IMMEDIATE myStmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 307+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(308) : alerts.sql(4930) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || myStmt);
               END IF; 
            END;
         END;
      END LOOP;
      
      BEGIN
         CLOSE trig_col_recset;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 322+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(323) : alerts.sql(4945) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- decide to issue alert or not
      h_issue_alert := 1;
      h_variance := 0;
      
      If PMO_ORASTD.strncmp(h_alert_id, 'SCETCFORCASTCHANGED', 19) = 0 Then
         -- alert issues when:
         -- forecast_start or forecast_finish changed or
         -- new_etc_delta <> old_etc_delta
         BEGIN
            SELECT COUNT(*)
            INTO h_issue_alert
            FROM PMO_ALERTS_VALUE_LIST
            WHERE (ELEMENT_ID = 'OLD_FORECAST_START' AND COALESCE(OLD_VALUE, '') <>  COALESCE(NEW_VALUE, '')) OR
            (ELEMENT_ID = 'OLD_FORECAST_FINISH' AND COALESCE(OLD_VALUE, '') <> COALESCE(NEW_VALUE, ''));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 347+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(348) : alerts.sql(4970) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_issue_alert, 0) = 0 Then
            h_issue_alert := FC_CALCULATE_DELTA_ETC(h_alert_id);
         End If;
      ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCREDUCEDETC', 12) = 0 Then
         h_issue_alert := FC_CALCULATE_DELTA_ETC(h_alert_id);
      End If;
      
      If Coalesce(h_issue_alert, 0) > 0 Then
         -- add required fields except old value fields to temporary table.
         
         OPEN realColumnRecset;
         
         LOOP
            h_value := NULL;
            
            FETCH realColumnRecset INTO h_element_id, h_table_name, h_column_name,
            h_column_location, h_format_str, h_column_type;
            IF realColumnRecset%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            If (PMO_ORASTD.strncmp(h_alert_id, 'SCETCFORCASTCHANGED', 19) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
                  , 'SCETCFORCASTREJECTED', 20) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
                  , 'SCETCFORCASTACCEPTED', 20) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
                  , 'SCREDUCEDETC', 12) = 0) Then
               If PMO_ORASTD.strcmp(h_element_id, 'REC_DATETIME') = 0 Then
                  BEGIN
                     SELECT TO_CHAR(TO_DATE(SYSDATE))
                     INTO h_value
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 394+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(395) : alerts.sql(5017) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf h_table_type <> PMO_CNST.SC_TABLE_WBS Then
                  If PMO_ORASTD.strcmp(h_element_id, 'PROJECT_NAME') = 0 Then
                     BEGIN
                        SELECT (SELECT ELEMENT_NAME FROM TMT_WBS WHERE ELEMENT_ID = TA.PROJECT_ID)
                        INTO h_value
                        FROM TASK_ASSIGNMENTS TA
                        WHERE TA.TSK_ASSIGNMENT_ID = h_id;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 411+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(412) : alerts.sql(5034) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  ElsIf PMO_ORASTD.strcmp(h_element_id, 'PARENT_NAME') = 0 Then
                     BEGIN
                        SELECT (SELECT ELEMENT_NAME FROM TMT_WBS P WHERE ELEMENT_ID = C.PARENT_ID)
                        INTO h_value
                        FROM TASK_ASSIGNMENTS TA, TMT_WBS C
                        WHERE TA.TSK_ASSIGNMENT_ID = h_id
                        AND TA.TASK_ID = C.ELEMENT_ID
                        AND TA.PROJECT_ID = C.PROJECT_ID;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 429+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(430) : alerts.sql(5052) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  ElsIf PMO_ORASTD.strcmp(h_element_id, 'LOGON_NAME') = 0 Then
                     BEGIN
                        SELECT FULL_NAME
                        INTO h_value
                        FROM RESOURCES
                        WHERE RESOURCE_ID = h_rec_user;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 445+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(446) : alerts.sql(5068) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     --trace_printf("\n h_column_type:%s", h_column_type);
                     myStmt := ' SELECT ';
                     
                     If PMO_ORASTD.strncmp(h_column_type, PMO_CNST.SC_CHAR_COLUMN_TYPE
                           , Length(PMO_CNST.SC_CHAR_COLUMN_TYPE)) = 0 Then
                        myStmt := myStmt || ' ';
                        myStmt := myStmt || h_table_name;
                        myStmt := myStmt || '.';
                        myStmt := myStmt || h_column_name;
                        myStmt := myStmt || ' FROM TMT_WBS, TASK_ASSIGNMENTS, RESOURCES';
                     Else
                        myStmt := myStmt || ' TO_CHAR(';
                        myStmt := myStmt || h_table_name;
                        myStmt := myStmt || '.';
                        myStmt := myStmt || h_column_name;
                        myStmt := myStmt || ') FROM TMT_WBS, TASK_ASSIGNMENTS, RESOURCES';
                     End If;
                     
                     myStmt := myStmt || ' WHERE TASK_ASSIGNMENTS.TSK_ASSIGNMENT_ID = ''';
                     myStmt := myStmt || h_id;
                     myStmt := myStmt || '''  AND TMT_WBS.ELEMENT_ID = TASK_ASSIGNMENTS.TASK_ID ';
                     myStmt := myStmt || ' AND RESOURCES.RESOURCE_ID = TASK_ASSIGNMENTS.RESOURCE_ID ';
                     
                     OPEN value_recset1 FOR myStmt;
                     
                     FETCH value_recset1 INTO h_value;
                     IF value_recset1%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                     
                     BEGIN
                        CLOSE value_recset1;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 490+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(491) : alerts.sql(5113) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  If PMO_ORASTD.strcmp(h_element_id, 'PROJECT_NAME') = 0 Then
                     BEGIN
                        SELECT (SELECT ELEMENT_NAME FROM TMT_WBS WHERE ELEMENT_ID = TW.PROJECT_ID)
                        INTO h_value
                        FROM TMT_WBS TW
                        WHERE TW.ELEMENT_ID = h_id;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 508+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(509) : alerts.sql(5131) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  ElsIf PMO_ORASTD.strcmp(h_element_id, 'PARENT_NAME') = 0 Then
                     BEGIN
                        SELECT (SELECT ELEMENT_NAME FROM TMT_WBS WHERE ELEMENT_ID = TW.PARENT_ID)
                        INTO h_value
                        FROM TMT_WBS TW
                        WHERE TW.ELEMENT_ID = h_id;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 524+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(525) : alerts.sql(5147) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  ElsIf PMO_ORASTD.strcmp(h_element_id, 'LOGON_NAME') = 0 Then
                     BEGIN
                        SELECT FULL_NAME
                        INTO h_value
                        FROM RESOURCES
                        WHERE RESOURCE_ID = h_rec_user;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 540+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(541) : alerts.sql(5163) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     myStmt := ' SELECT ';
                     
                     If PMO_ORASTD.strncmp(h_column_type, PMO_CNST.SC_CHAR_COLUMN_TYPE
                           , Length(PMO_CNST.SC_CHAR_COLUMN_TYPE)) = 0 Then
                        myStmt := myStmt || ' ';
                        myStmt := myStmt || h_table_name;
                        myStmt := myStmt || '.';
                        myStmt := myStmt || h_column_name;
                        myStmt := myStmt || ' FROM TMT_WBS, TASK_ASSIGNMENTS, RESOURCES';
                     Else
                        myStmt := myStmt || ' TO_CHAR(';
                        myStmt := myStmt || h_table_name;
                        myStmt := myStmt || '.';
                        myStmt := myStmt || h_column_name;
                        myStmt := myStmt || ') FROM TMT_WBS, TASK_ASSIGNMENTS, RESOURCES';
                     End If;
                     
                     myStmt := myStmt || ' WHERE TMT_WBS.ELEMENT_ID = ''';
                     myStmt := myStmt || h_id;
                     myStmt := myStmt || '''  AND TMT_WBS.ELEMENT_ID = TASK_ASSIGNMENTS.TASK_ID ';
                     myStmt := myStmt || ' AND RESOURCES.RESOURCE_ID = TASK_ASSIGNMENTS.RESOURCE_ID ';
                     
                     OPEN value_recset2 FOR myStmt;
                     
                     FETCH value_recset2 INTO h_value;
                     IF value_recset2%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                     
                     BEGIN
                        CLOSE value_recset2;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 584+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(585) : alerts.sql(5207) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCPERSONALCALENDARCHANGED', 25) = 0 Then
               If PMO_ORASTD.strcmp(h_element_id, 'LOGON_NAME') = 0 Then
                  BEGIN
                     SELECT FULL_NAME
                     INTO h_value
                     FROM RESOURCES
                     WHERE RESOURCE_ID = h_rec_user;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 603+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(604) : alerts.sql(5226) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'REC_DATETIME') = 0 Then
                  BEGIN
                     SELECT TO_CHAR(TO_DATE(SYSDATE))
                     INTO h_value
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 618+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(619) : alerts.sql(5241) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            ElsIf (PMO_ORASTD.strncmp(h_alert_id, 'SCTASKCOMPLETED', 15) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
                  , 'SCTASKREADYTOSTART', 18) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
                  , 'SCTASKTOBESTARTED', 17) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
                  , 'SCTASKLATETOSTART', 17) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
                  , 'SCTASKLATETOFINISH', 18) = 0) Then
               If PMO_ORASTD.strcmp(h_element_id, 'PROJECT_NAME') = 0 Then
                  BEGIN
                     SELECT P.ELEMENT_NAME
                     INTO h_value
                     FROM TMT_WBS P, TMT_WBS T
                     WHERE T.ELEMENT_ID = h_id
                     AND P.ELEMENT_ID = T.PROJECT_ID;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 641+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(642) : alerts.sql(5264) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'PARENT_NAME') = 0 Then
                  BEGIN
                     SELECT P.ELEMENT_NAME
                     INTO h_value
                     FROM TMT_WBS P, TMT_WBS C
                     WHERE C.ELEMENT_ID = h_id
                     AND P.ELEMENT_ID = C.PARENT_ID;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 658+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(659) : alerts.sql(5281) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'PLANED_FINISH_DATE') = 0 Then
                  BEGIN
                     SELECT TO_DATE(SubStr(COALESCE(A.UNRESOLVED_FINISH, T.UNRESOLVED_FINISH), 1, 10))
                     INTO h_value
                     FROM TMT_WBS T
                     JOIN TASK_ASSIGNMENTS A ON A.PROJECT_ID = T.PROJECT_ID
                     AND A.TASK_ID = T.ELEMENT_ID
                     WHERE T.ACTUAL_START IS NuLL
                     AND T.ELEMENT_ID = h_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 677+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(678) : alerts.sql(5300) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'PLANED_START_DATE') = 0 Then
                  BEGIN
                     SELECT TO_DATE(SubStr(COALESCE(A.UNRESOLVED_START, T.UNRESOLVED_START), 1, 10))
                     INTO h_value
                     FROM TMT_WBS T
                     JOIN TASK_ASSIGNMENTS A ON A.PROJECT_ID = T.PROJECT_ID
                     AND A.TASK_ID = T.ELEMENT_ID
                     WHERE T.ACTUAL_START IS NuLL
                     AND T.ELEMENT_ID = h_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 696+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(697) : alerts.sql(5319) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf PMO_ORASTD.strncmp(h_element_id, 'REMAINING_DAYS', 14) = 0 Then
                  BEGIN
                     SELECT VALUE
                     INTO h_value
                     FROM PMO_ALERTS_PARAM_VALUE
                     WHERE ALERT_ID = h_alert_id
                     AND ELEMENT_ID = 'REMAINING_DAYS';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 713+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(714) : alerts.sql(5336) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  myStmt := ' SELECT ';
                  
                  If PMO_ORASTD.strncmp(h_column_type, PMO_CNST.SC_CHAR_COLUMN_TYPE
                        , Length(PMO_CNST.SC_CHAR_COLUMN_TYPE)) = 0 Then
                     myStmt := myStmt || ' ';
                     myStmt := myStmt || h_table_name;
                     myStmt := myStmt || '.';
                     myStmt := myStmt || h_column_name;
                     myStmt := myStmt || ' FROM TMT_WBS';
                  Else
                     myStmt := myStmt || ' TO_CHAR(';
                     myStmt := myStmt || h_table_name;
                     myStmt := myStmt || '.';
                     myStmt := myStmt || h_column_name;
                     myStmt := myStmt || ') FROM TMT_WBS';
                  End If;
                  
                  myStmt := myStmt || ' WHERE ELEMENT_ID = ''';
                  myStmt := myStmt || h_id;
                  myStmt := myStmt || ''' ';
                  
                  OPEN value_recset3 FOR myStmt;
                  
                  FETCH value_recset3 INTO h_value;
                  IF value_recset3%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  
                  BEGIN
                     CLOSE value_recset3;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 756+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(757) : alerts.sql(5379) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCNOTRESPONDEDTOWORKFLOWITEM', 28) = 0 Then
               If PMO_ORASTD.strcmp(h_element_id, 'RESOURCE_NAME') = 0 Then
                  BEGIN
                     SELECT FULL_NAME
                     INTO h_value
                     FROM RESOURCES
                     WHERE RESOURCE_ID = h_rec_user;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 774+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(775) : alerts.sql(5397) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               If PMO_ORASTD.strcmp(h_element_id, 'WORKFLOWITEMNAME') = 0 Then
                  BEGIN
                     SELECT ELEMENT_NAME
                     INTO h_value
                     FROM TMT_WORKFLOWS
                     WHERE ELEMENT_ID = h_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 791+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(792) : alerts.sql(5414) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  myStmt := ' SELECT ';
                  myStmt := myStmt || ' TO_CHAR(';
                  myStmt := myStmt || h_table_name;
                  myStmt := myStmt || '.';
                  myStmt := myStmt || h_column_name;
                  myStmt := myStmt || ') FROM TMT_WORKFLOWS ';
                  myStmt := myStmt || ' WHERE ELEMENT_ID = ''';
                  myStmt := myStmt || h_id;
                  myStmt := myStmt || ''' ';
                  
                  OPEN value_recset4 FOR myStmt;
                  
                  FETCH value_recset4 INTO h_value;
                  IF value_recset4%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  
                  BEGIN
                     CLOSE value_recset4;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 823+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(824) : alerts.sql(5446) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCTIMESHEETLATEREMINDER', 23) = 0 Then
               If PMO_ORASTD.strcmp(h_element_id, 'RESOURCE_NAME') = 0 Then
                  BEGIN
                     SELECT FULL_NAME
                     INTO h_value
                     FROM RESOURCES
                     WHERE RESOURCE_ID = h_rec_user;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 841+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(842) : alerts.sql(5464) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'DUE_DATE') = 0 Then
                  BEGIN
                     SELECT TO_CHAR(TO_DATE(SubStr(VALUE, 1, 10)) - 1    )
                     INTO h_value
                     FROM PMO_ALERTS_PARAM_VALUE
                     WHERE ALERT_ID = h_alert_id
                     AND ELEMENT_ID = h_element_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 858+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(859) : alerts.sql(5481) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'WEEK_OF') = 0 Then
                  BEGIN
                     SELECT VALUE
                     INTO h_value
                     FROM PMO_ALERTS_PARAM_VALUE
                     WHERE ALERT_ID = h_alert_id
                     AND ELEMENT_ID = h_element_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 875+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(876) : alerts.sql(5498) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  myStmt := ' SELECT ';
                  myStmt := myStmt || ' TO_CHAR(';
                  myStmt := myStmt || h_table_name;
                  myStmt := myStmt || '.';
                  myStmt := myStmt || h_column_name;
                  myStmt := myStmt || ') FROM TASK_ASSIGNMENTS, ';
                  myStmt := myStmt || ' WHERE TSK_ASSIGNMENT_ID = ''';
                  myStmt := myStmt || h_id;
                  myStmt := myStmt || ''' ';
                  
                  OPEN value_recset5 FOR myStmt;
                  
                  FETCH value_recset5 INTO h_value;
                  IF value_recset5%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  
                  BEGIN
                     CLOSE value_recset5;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 907+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(908) : alerts.sql(5530) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCRTFITEMCHANGED', 16) = 0 Then
               If PMO_ORASTD.strcmp(h_element_id, 'MODIFIED_BY') = 0 Then
                  BEGIN
                     SELECT FULL_NAME
                     INTO h_value
                     FROM RESOURCES
                     WHERE RESOURCE_ID = h_rec_user;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 925+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(926) : alerts.sql(5548) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'RTF_ITEM_NAME') = 0 Then
                  BEGIN
                     SELECT VALUE
                     INTO h_value
                     FROM PMO_ALERTS_PARAM_VALUE
                     WHERE ALERT_ID = h_alert_id
                     AND ELEMENT_ID = h_element_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 942+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(943) : alerts.sql(5565) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'ELEMENT_NAME') = 0 Then
                  CASE h_table_type
                     WHEN PMO_CNST.SC_TABLE_WBS THEN
                        BEGIN
                           SELECT ELEMENT_NAME
                           INTO h_value
                           FROM TMT_WBS
                           WHERE ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 960+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(961) : alerts.sql(5583) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     
                     WHEN PMO_CNST.SC_TABLE_CRI THEN
                        BEGIN
                           SELECT ELEMENT_NAME
                           INTO h_value
                           FROM TMT_CRI
                           WHERE ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 977+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(978) : alerts.sql(5600) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     
                     WHEN PMO_CNST.SC_TABLE_DOC THEN
                        BEGIN
                           SELECT ELEMENT_NAME
                           INTO h_value
                           FROM TMT_DOCUMENTS
                           WHERE ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 994+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(995) : alerts.sql(5617) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     ELSE
                        NULL;
                  END CASE;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'PROJECT_NAME') = 0 Then
                  CASE h_table_type
                     WHEN PMO_CNST.SC_TABLE_WBS THEN
                        BEGIN
                           SELECT P.ELEMENT_NAME
                           INTO h_value
                           FROM TMT_WBS C, TMT_WBS P
                           WHERE P.ELEMENT_ID = C.PROJECT_ID
                           AND C.ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1016+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1017) : alerts.sql(5639) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     
                     WHEN PMO_CNST.SC_TABLE_CRI THEN
                        BEGIN
                           SELECT P.ELEMENT_NAME
                           INTO h_value
                           FROM TMT_CRI C, TMT_CRI P
                           WHERE P.ELEMENT_ID = C.PROJECT_ID
                           AND C.ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1034+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1035) : alerts.sql(5657) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     
                     WHEN PMO_CNST.SC_TABLE_DOC THEN
                        BEGIN
                           SELECT P.ELEMENT_NAME
                           INTO h_value
                           FROM TMT_DOCUMENTS C, TMT_DOCUMENTS P
                           WHERE P.ELEMENT_ID = C.PROJECT_ID
                           AND C.ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1052+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1053) : alerts.sql(5675) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     ELSE
                        NULL;
                  END CASE;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'PARENT_NAME') = 0 Then
                  CASE h_table_type
                     WHEN PMO_CNST.SC_TABLE_WBS THEN
                        BEGIN
                           SELECT P.ELEMENT_NAME
                           INTO h_value
                           FROM TMT_WBS C, TMT_WBS P
                           WHERE P.ELEMENT_ID = C.PARENT_ID
                           AND C.ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1074+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1075) : alerts.sql(5697) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     
                     WHEN PMO_CNST.SC_TABLE_CRI THEN
                        BEGIN
                           SELECT P.ELEMENT_NAME
                           INTO h_value
                           FROM TMT_CRI C, TMT_CRI P
                           WHERE P.ELEMENT_ID = C.PARENT_ID
                           AND C.ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1092+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1093) : alerts.sql(5715) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     
                     WHEN PMO_CNST.SC_TABLE_DOC THEN
                        BEGIN
                           SELECT P.ELEMENT_NAME
                           INTO h_value
                           FROM TMT_DOCUMENTS C, TMT_DOCUMENTS P
                           WHERE P.ELEMENT_ID = C.PARENT_ID
                           AND C.ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1110+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1111) : alerts.sql(5733) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     ELSE
                        NULL;
                  END CASE;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'MAJOR_REVISION') = 0 Then
                  CASE h_table_type
                     WHEN PMO_CNST.SC_TABLE_WBS THEN
                        BEGIN
                           SELECT TO_CHAR(MAJOR_REVISION)
                           INTO h_value
                           FROM TMT_WBS
                           WHERE ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1131+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1132) : alerts.sql(5754) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     
                     WHEN PMO_CNST.SC_TABLE_CRI THEN
                        BEGIN
                           SELECT TO_CHAR(MAJOR_REVISION)
                           INTO h_value
                           FROM TMT_CRI
                           WHERE ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1148+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1149) : alerts.sql(5771) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     
                     WHEN PMO_CNST.SC_TABLE_DOC THEN
                        BEGIN
                           SELECT TO_CHAR(MAJOR_REVISION)
                           INTO h_value
                           FROM TMT_DOCUMENTS
                           WHERE ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1165+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1166) : alerts.sql(5788) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     ELSE
                        NULL;
                  END CASE;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'MINOR_REVISION') = 0 Then
                  CASE h_table_type
                     WHEN PMO_CNST.SC_TABLE_WBS THEN
                        BEGIN
                           SELECT TO_CHAR(MINOR_REVISION)
                           INTO h_value
                           FROM TMT_WBS
                           WHERE ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1186+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1187) : alerts.sql(5809) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     
                     WHEN PMO_CNST.SC_TABLE_CRI THEN
                        BEGIN
                           SELECT TO_CHAR(MINOR_REVISION)
                           INTO h_value
                           FROM TMT_CRI
                           WHERE ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1203+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1204) : alerts.sql(5826) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     
                     WHEN PMO_CNST.SC_TABLE_DOC THEN
                        BEGIN
                           SELECT TO_CHAR(MINOR_REVISION)
                           INTO h_value
                           FROM TMT_DOCUMENTS
                           WHERE ELEMENT_ID = h_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1220+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1221) : alerts.sql(5843) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     ELSE
                        NULL;
                  END CASE;
               End If;
            ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCPREDECESSORDATESCHANGED', 25) = 0 Then
               If PMO_ORASTD.strcmp(h_element_id, 'SUCCESSOR_PROJECT_NAME') = 0 Then
                  BEGIN
                     SELECT ELEMENT_NAME
                     INTO h_value
                     FROM TMT_WBS
                     WHERE ELEMENT_ID = h_succ_project_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1241+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1242) : alerts.sql(5864) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'PREDECESSOR_PROJECT_NAME') = 0 Then
                  BEGIN
                     SELECT ELEMENT_NAME
                     INTO h_value
                     FROM TMT_WBS
                     WHERE ELEMENT_ID = h_pred_project_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1257+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1258) : alerts.sql(5880) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'SUCCESSOR_ELEMENT_NAME') = 0 Then
                  h_value := h_succ_element_name;
               ElsIf PMO_ORASTD.strcmp(h_element_id, 'PREDECESSOR_ELEMENT_NAME') = 0 Then
                  h_value := h_pred_element_name;
               End If;
            End If;
            
            BEGIN
               INSERT INTO PMO_ALERTS_VALUE_LIST (ALERT_ID, ELEMENT_ID, LOC, TYPE_ID, OLD_VALUE, FORMAT, COLUMN_TYPE)
               VALUES( h_alert_id, h_element_id, h_column_location, h_type_id, h_value, h_format_str, h_column_type );
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1281+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1282) : alerts.sql(5904) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         END LOOP;
         
         BEGIN
            CLOSE realColumnRecset;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1295+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1296) : alerts.sql(5918) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         -- build value list with using session.PMO_ALERTS_VALUE_LIST
         
         OPEN valueListRecset;
         
         h_value_list := '';
         LOOP
            FETCH valueListRecset INTO h_value, h_type_id, h_format_str, h_column_type;
            IF valueListRecset%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            null_return := FC_APPLY_FORMAT(h_value, h_format_str, h_column_type);
            
            h_value_list := h_value_list || h_value;
            h_value_list := h_value_list || '#!';
         END LOOP;
         
         BEGIN
            CLOSE valueListRecset;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1331+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1332) : alerts.sql(5954) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT TYPE_ID
            INTO h_stage_id
            FROM ALERT_MESSAGES
            WHERE ALERT_ID = h_alert_id
            AND LANGUAGE_CODE = 'EN';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1348+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1349) : alerts.sql(5971) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_ORASTD.strncmp(h_alert_id, 'SCETCFORCASTCHANGED', 19) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
               , 'SCETCFORCASTREJECTED', 20) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
               , 'SCETCFORCASTACCEPTED', 20) = 0) OR (PMO_ORASTD.strncmp(h_alert_id, 'SCREDUCEDETC'
               , 12) = 0) OR (PMO_ORASTD.strncmp(h_alert_id, 'SCTIMESHEETLATEREMINDER'
               , 23) = 0) Then
            If h_table_type <> PMO_CNST.SC_TABLE_WBS Then
               BEGIN
                  SELECT TASK_ID, PROJECT_ID
                  INTO h_task_id, h_project_id
                  FROM TASK_ASSIGNMENTS
                  WHERE TSK_ASSIGNMENT_ID = h_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1370+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1371) : alerts.sql(5993) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  INSERT INTO ALERTS(ELEMENT_ID, PROJECT_ID, ALERT_ID, PARENT_TABLE, VALUE_LIST, VARIANCE, FLAG, REC_USER)
                  VALUES(h_task_id, h_project_id, h_alert_id, h_parent_table, h_value_list, h_variance, 'I', h_rec_user );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1388+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1389) : alerts.sql(6011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT PROJECT_ID
                  INTO h_project_id
                  FROM TMT_WBS
                  WHERE ELEMENT_ID = h_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1404+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1405) : alerts.sql(6027) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  INSERT INTO ALERTS(ELEMENT_ID, PROJECT_ID, ALERT_ID, PARENT_TABLE, VALUE_LIST, VARIANCE, FLAG, REC_USER)
                  VALUES(h_id, h_project_id, h_alert_id, h_parent_table, h_value_list, h_variance, 'I', h_rec_user );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1422+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1423) : alerts.sql(6045) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCPERSONALCALENDARCHANGED', 25) = 0 Then
            OPEN res_prj_set;
            
            LOOP
               FETCH res_prj_set INTO h_project_id, h_task_id;
               IF res_prj_set%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
               
               BEGIN
                  INSERT INTO ALERTS(ELEMENT_ID, PROJECT_ID, ALERT_ID, PARENT_TABLE, VALUE_LIST, VARIANCE, FLAG, REC_USER)
                  VALUES( h_task_id, h_project_id, h_alert_id, h_parent_table, h_value_list, h_variance, 'I', h_rec_user);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1455+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1456) : alerts.sql(6078) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            END LOOP;
            
            BEGIN
               CLOSE res_prj_set;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1469+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1470) : alerts.sql(6092) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf (PMO_ORASTD.strncmp(h_alert_id, 'SCTASKCOMPLETED', 15) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
               , 'SCTASKREADYTOSTART', 18) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
               , 'SCTASKTOBESTARTED', 17) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
               , 'SCTASKLATETOSTART', 17) = 0) OR (PMO_ORASTD.strncmp(h_alert_id
               , 'SCTASKLATETOFINISH', 18) = 0) Then
            BEGIN
               SELECT PROJECT_ID
               INTO h_project_id
               FROM TMT_WBS
               WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1489+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1490) : alerts.sql(6112) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               INSERT INTO ALERTS(ELEMENT_ID, PROJECT_ID, ALERT_ID, PARENT_TABLE, VALUE_LIST, VARIANCE, FLAG, REC_USER)
               VALUES(h_id, h_project_id, h_alert_id, h_parent_table, h_value_list, h_variance, 'I', h_rec_user );
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1507+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1508) : alerts.sql(6130) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCNOTRESPONDEDTOWORKFLOWITEM', 28) = 0 Then
            BEGIN
               INSERT INTO ALERTS(ELEMENT_ID, RESOURCE_ID, ALERT_ID, PARENT_TABLE, VALUE_LIST, FLAG, REC_USER)
               VALUES(h_id, h_rec_user, h_alert_id, h_parent_table, h_value_list, 'I', h_rec_user );
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1525+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1526) : alerts.sql(6148) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCRTFITEMCHANGED', 16) = 0 Then
            CASE h_table_type
               WHEN PMO_CNST.SC_TABLE_WBS THEN
                  BEGIN
                     SELECT PROJECT_ID
                     INTO h_project_id
                     FROM TMT_WBS
                     WHERE ELEMENT_ID = h_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1543+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1544) : alerts.sql(6166) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               
               WHEN PMO_CNST.SC_TABLE_CRI THEN
                  BEGIN
                     SELECT PROJECT_ID
                     INTO h_project_id
                     FROM TMT_CRI
                     WHERE ELEMENT_ID = h_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1560+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1561) : alerts.sql(6183) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               
               WHEN PMO_CNST.SC_TABLE_DOC THEN
                  BEGIN
                     SELECT PROJECT_ID
                     INTO h_project_id
                     FROM TMT_DOCUMENTS
                     WHERE ELEMENT_ID = h_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1577+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1578) : alerts.sql(6200) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ELSE
                  NULL;
            END CASE;
            
            BEGIN
               INSERT INTO ALERTS(ELEMENT_ID, PROJECT_ID, ALERT_ID, PARENT_TABLE, VALUE_LIST, VARIANCE, FLAG, REC_USER)
               VALUES(h_id, h_project_id, h_alert_id, h_parent_table, h_value_list, h_variance, 'I', h_rec_user );
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1598+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1599) : alerts.sql(6221) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf PMO_ORASTD.strncmp(h_alert_id, 'SCPREDECESSORDATESCHANGED', 25) = 0 Then
            BEGIN
               INSERT INTO ALERTS(ELEMENT_ID, PROJECT_ID, ALERT_ID, PARENT_TABLE, VALUE_LIST, FLAG, REC_USER)
               VALUES(h_succ_id, h_succ_project_id, h_alert_id, h_parent_table, h_value_list, 'I', h_rec_user );
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1616+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1617) : alerts.sql(6239) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         BEGIN
            DELETE FROM PMO_ALERTS_VALUE_LIST
            WHERE ALERT_ID = h_alert_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ISSUE_ALERT', 1635+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ISSUE_ALERT(1636) : alerts.sql(6258) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ISSUE_ALERT', 1644+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ISSUE_ALERT(1645) : alerts.sql(6267) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ISSUE_ALERT;
ALTER FUNCTION FC_ISSUE_ALERT COMPILE DEBUG;

/***************************************************************************************************/

Prompt == FUNCTION SP_SEND_PERIODIC_ALERTS ==
CREATE OR REPLACE FUNCTION SP_SEND_PERIODIC_ALERTS
(
     p_bitflag PLS_INTEGER
   , p_date VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_date DATE;
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_SEND_PERIODIC_ALERTS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_SEND_PERIODIC_ALERTS'), '-- ' || SYSTIMESTAMP, ':r := SP_SEND_PERIODIC_ALERTS(' || 
       COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_date || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEND_PERIODIC_ALERTS', 18+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_SEND_PERIODIC_ALERTS(19) : alerts.sql(6296) : ' || SYSTIMESTAMP);
   END IF; 

   h_bitflag := p_bitflag;
   h_date := p_date;
   
   PMO_DB2M.SC_STATE := FC_SEND_PERIODIC_ALERTS(h_bitflag, h_date);
   
   If h_date IS NULL Then
      h_date := '2000-01-31';
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_SEND_PERIODIC_ALERTS', 31+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_SEND_PERIODIC_ALERTS(32) : alerts.sql(6309) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || COALESCE(NULLIF('''' || h_date || '''',''''''), 'NULL'));
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_date);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEND_PERIODIC_ALERTS', 39+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEND_PERIODIC_ALERTS(40) : alerts.sql(6317) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEND_PERIODIC_ALERTS', 51+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEND_PERIODIC_ALERTS(52) : alerts.sql(6329) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_SEND_PERIODIC_ALERTS;
ALTER FUNCTION SP_SEND_PERIODIC_ALERTS COMPILE DEBUG;

