/**
 * Copyright 2000 Systemcorp - All rights reserved. Made in Canada.
 *

 * Description:
 *
 * $Author: Tudor Toma $
 * $Revision: 42 $
 * $Date: 7/31/00 6:18p $
 *
 * $Log: /PMOffice_4_0_server/PMOv3/C-StoredProcs/cplan.cpp $
 *
 **/

-- already in pmo_sqlprep.h : #include "pmodefines.h"

-- already in stdafx.h : #include <time.h>

/*****************************************************************************************/

-- Get Finish Date OR Duration

/*****************************************************************************************/

Prompt == FUNCTION FC_ASSN_DURATION ==
CREATE OR REPLACE FUNCTION FC_ASSN_DURATION
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_duration PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_duration PLS_INTEGER := sh_h_duration;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_dayofyear DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_calendar_id VARCHAR2(32);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_min_per_day PLS_INTEGER := 0;
   h_minute PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_available PLS_INTEGER := 0;
   h_julian_start PLS_INTEGER := 0;
   h_julian_finish PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_SF PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_id VARCHAR2(32);
   tempDuration PLS_INTEGER;
-- Declare Cursors
      CURSOR CRES1 IS
         SELECT DAY_YEAR, AVAILABLE_MIN, BIT_SCHED FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_dayofyear, 1, 10)) ORDER BY DAY_YEAR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ASSN_DURATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ASSN_DURATION'), '-- ' || SYSTIMESTAMP, ':r := FC_ASSN_DURATION(' || 
       '''' || h_project_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 45+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ASSN_DURATION(46) : cplan_fd.sql(71) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);//trace_flush();
   If Coalesce(Length(h_start_DT), 0) = 0 Then
      --trace_printf("*** Warning: Start date is null\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 52+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(53) : cplan_fd.sql(78) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(h_duration, 0) < 0 Then
      --trace_printf("*** Warning: Duration is negative: %d\n",h_duration);
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 60+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(61) : cplan_fd.sql(86) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(Length(h_finish_DT), 0) = 0 Then
      --trace_printf("*** Warning: Finish date is null\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 68+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(69) : cplan_fd.sql(94) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If (Coalesce(Length(h_start_DT), 0) > 9) AND (Coalesce(Length(h_finish_DT), 0) > 9) 
         AND (PMO_ORASTD.strcmp(h_finish_DT, h_start_DT) < 0) Then
      --trace_printf("*** Warning: Finish date is earlier than Start date\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 77+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(78) : cplan_fd.sql(103) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(Length(h_finish_DT), 0) > 10 Then
      h_duration := 0;
      BEGIN
         SELECT PMO_ORASTD.TIME(h_start_DT), PMO_ORASTD.TIME(h_finish_DT) INTO h_start_time,h_finish_time FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 92+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(93) : cplan_fd.sql(118) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(Length(h_start_DT), 0) > 10 Then
      BEGIN
         SELECT PMO_ORASTD.TIME(h_start_DT) INTO h_start_time FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 105+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(106) : cplan_fd.sql(131) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_finish_DT := '';
      h_finish_time := '';
   Else
      h_start_time := '08.00.00';
      
      h_finish_DT := '';
      h_finish_time := '';
   End If;
   
   --trace_printf("h_start_time = %s,h_finish_time = %s\n", h_start_time,h_finish_time);//trace_flush();
   
   h_start_D := '';
   h_finish_D := '';
   h_start_date := '';
   h_finish_date := '';
   h_SF := 0;
   
   BEGIN
      SELECT COALESCE(MINUTE_PER_DAY,480), MIN_DATE - 1     , MAX_DATE + 1     , START_DATE, FINISH_DATE, DURATION
      INTO h_min_per_day, h_start_D, h_finish_D, h_start_date, h_finish_date, h_minute
      FROM PMO_RES_MIN_MAX WHERE RESOURCE_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 137+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(138) : cplan_fd.sql(163) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE = %d, h_start_D = %s, h_finish_D = %s\n",SC_STATE,h_start_D,h_finish_D);
   --trace_printf("h_start_date = %s, h_finish_date = %s, h_minute = %d\n",h_start_date,h_finish_date,h_minute);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      BEGIN
         SELECT COALESCE(MINUTES_PER_DAY,480) INTO h_min_per_day
         FROM TMT_CALENDARS
         WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 155+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(156) : cplan_fd.sql(181) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT MIN(DAY_YEAR), MAX(DAY_YEAR) INTO h_start_D, h_finish_D
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 169+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(170) : cplan_fd.sql(195) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_RES_MIN_MAX
         VALUES (h_project_id, h_start_D, h_finish_D, h_min_per_day, '1899-12-30 8:00:00','1899-12-30 8:00:00', 0);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 187+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(188) : cplan_fd.sql(213) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT TO_DATE(SubStr(h_start_D, 1, 10)) - 1     , TO_DATE(SubStr(h_start_D, 1, 10)) + 1      INTO h_start_D, h_finish_D FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 200+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(201) : cplan_fd.sql(226) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_calendar_id := h_project_id;
   If PMO_ORASTD.strncmp(h_start_DT, h_start_D, 10) <= 0 Then
      --strncpy(h_dayofyear, h_start_DT, 10);
      --h_dayofyear[10] = '\0';
      
      h_dayofyear := SubStr(h_start_DT, 1, 10);
      h_temp_var := PMO_CNST.SC_BIT_30;
      
      h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_dayofyear, h_start_D, h_duration);
      
      h_start_D := h_dayofyear;
      --strcpy(h_start_D, h_dayofyear);
      
      BEGIN
         UPDATE PMO_RES_MIN_MAX SET MIN_DATE = h_start_D WHERE RESOURCE_ID = h_project_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 231+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(232) : cplan_fd.sql(257) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("1__MIN_DATE *____*: SC_STATE: %d, h_start_D: %s\n", SC_STATE,h_start_D);
   ElsIf PMO_ORASTD.strncmp(h_start_DT, h_finish_D, 10) >= 0 Then
      --trace_printf("h_finish_D = %s,h_dayofyear = %s, h_duration = %d\n",h_finish_D,h_dayofyear,h_duration);
      If Coalesce(h_duration, 0) = 0 Then
         --strncpy(h_dayofyear, h_finish_DT, 10);
         --h_dayofyear[10] = '\0';
         h_dayofyear := SubStr(h_finish_DT, 1, 10);
         h_temp_var := 0;
         
         h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_finish_D, h_dayofyear, h_temp_var);
         
         h_finish_D := h_dayofyear;
         --strcpy(h_finish_D, h_dayofyear);
         
         BEGIN
            UPDATE PMO_RES_MIN_MAX SET MAX_DATE = h_finish_D WHERE RESOURCE_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 261+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(262) : cplan_fd.sql(287) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("2__MAX_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
      Else
         --strcpy(h_dayofyear, "");
         h_dayofyear := '';
         h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_finish_D, h_dayofyear, h_duration);
         
         BEGIN
            SELECT MAX(DAY_YEAR) INTO h_finish_D FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 279+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(280) : cplan_fd.sql(305) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE PMO_RES_MIN_MAX SET MAX_DATE = h_finish_D WHERE RESOURCE_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 296+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(297) : cplan_fd.sql(322) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("3__MAX_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
      End If;
   End If;
   --trace_printf("h_start_D: %s, h_finish_D = %s, h_duration = %d\n",h_start_D,h_finish_D,h_duration);
   --trace_printf("h_start_DT = %s, h_finish_DT = %s\n",h_start_DT,h_finish_DT);
   
   If Coalesce(h_duration, 0) = 0 Then
      If (PMO_ORASTD.strcmp(h_start_date, h_start_DT) = 0) AND (PMO_ORASTD.strcmp(h_finish_date
            , h_finish_DT) = 0) Then
         h_duration := Coalesce(h_minute, 0);
         --trace_printf("RETURN_1: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 312+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(313) : cplan_fd.sql(338) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_duration, 0);
      ElsIf PMO_ORASTD.strcmp(h_start_DT, h_finish_DT) = 0 Then
         h_duration := 0;
         --trace_printf("RETURN_2: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 319+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(320) : cplan_fd.sql(345) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_duration, 0);
      End If;
      
      BEGIN
         SELECT DAY_YEAR INTO h_dayofyear FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = TO_DATE(SubStr(h_finish_DT, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 333+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(334) : cplan_fd.sql(359) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("2__* SC_STATE: %d, h_SF: %d\n", SC_STATE,h_SF);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA AND (PMO_ORASTD.strncmp(h_finish_DT, h_start_D
            , 10) >= 0) AND (PMO_ORASTD.strncmp(h_finish_DT, h_finish_D, 10) <= 0) Then
         -- Finish date is non-working day
         If PMO_ORASTD.strncmp(h_start_DT, h_finish_DT, 10) = 0 Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMPDIFF(4, PMO_ORASTD.DB2TIMESTAMP(h_finish_DT), PMO_ORASTD.DB2TIMESTAMP(h_start_DT))
               INTO h_duration
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 353+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(354) : cplan_fd.sql(379) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RETURN_3.1: h_min_per_day = %d, h_duration = %d\n",h_min_per_day,h_duration);
            
            If Coalesce(h_duration, 0) > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_min_per_day, 0);
            End If;
            --trace_printf("RETURN_3.2: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 364+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(365) : cplan_fd.sql(390) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_duration, 0);
         Else
            h_finish_time := '17.00.00';
            h_minute := Coalesce(h_min_per_day, 0);
         End If;
         --trace_printf("Finish date is non-working day: h_finish_time: %s, h_minute = %d\n",h_finish_time,h_minute);
      Else
         h_minute := 0;
      End If;
   Else
      If (PMO_ORASTD.strcmp(h_start_date, h_start_DT) = 0) AND (Coalesce(h_duration, 0) = Coalesce(
            h_minute, 0)) Then
         h_finish_DT := h_finish_date;
         --trace_printf("RETURN_4: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 381+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(382) : cplan_fd.sql(407) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_duration, 0);
      End If;
      
      h_minute := 0;
   End If;
   
   BEGIN
      SELECT DAY_YEAR INTO h_dayofyear FROM CALENDAR_DAY_OF_YEAR
      WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = TO_DATE(SubStr(h_start_DT, 1, 10));
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 398+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(399) : cplan_fd.sql(424) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("1__* SC_STATE: %d, h_dayofyear: %s\n", SC_STATE,h_dayofyear);
   
   --trace_printf("n.1__* h_start_DT: %s, h_start_D: %s, h_finish_D: %s\n", h_start_DT,h_start_D,h_finish_D);
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA AND (PMO_ORASTD.strncmp(h_start_DT, h_start_D
         , 10) >= 0) AND (PMO_ORASTD.strncmp(h_start_DT, h_finish_D, 10) <= 0) Then
      h_start_time := '08.00.00';
      If Coalesce(h_duration, 0) > 0 Then
         If Coalesce(h_duration, 0) = Coalesce(h_min_per_day, 0) Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD.TIME('17.00.00'))
               INTO h_finish_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 420+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(421) : cplan_fd.sql(446) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RETURN_5: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 426+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(427) : cplan_fd.sql(452) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_duration, 0);
         ElsIf Coalesce(h_duration, 0) <= Coalesce(h_min_per_day, 0) Then
            BEGIN
               SELECT TO_TIMESTAMP(PMO_ORASTD.DB2TIMESTAMP(h_start_DT) + (h_duration / 1440.0))
               INTO h_finish_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 440+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(441) : cplan_fd.sql(466) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RETURN_6: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 446+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(447) : cplan_fd.sql(472) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_duration, 0);
         End If;
      End If;
      
      h_minute := Coalesce(h_minute, 0) + Coalesce(h_min_per_day, 0);
      --trace_printf("Start date is non-working day: h_start_time: %s, h_minute = %d\n",h_start_time,h_minute);
   End If;
   
   --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_minute = %d\n",h_start_DT,h_finish_DT,h_minute);
   --trace_printf("h_start_D = %s, h_finish_D = %s, h_calendar_id = %s\n",h_start_D,h_finish_D, h_calendar_id);
   If (Coalesce(Length(h_start_D), 0) = 0) AND (Coalesce(Length(h_finish_D), 0) = 0) Then
      --trace_printf("*** Warning: h_start_D and h_finish_D are empty\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 461+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(462) : cplan_fd.sql(487) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   --trace_printf("1_** h_minute = %d, h_total = %d, h_duration = %d\n",h_minute,h_total,h_duration);
   If Coalesce(h_duration, 0) = 0 Then
      --trace_printf("h_finish_DT = %s,h_finish_D = %s\n", h_finish_DT,h_finish_D);
      If PMO_ORASTD.strncmp(h_finish_DT, h_finish_D, 10) > 0 Then
         --strncpy(h_dayofyear, h_finish_DT, 10);
         --h_dayofyear[10] = '\0';
         h_dayofyear := SubStr(h_finish_DT, 1, 10);
         h_temp_var := 0;
         h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_finish_D, h_dayofyear, h_temp_var);
         
         h_finish_D := h_dayofyear;
         BEGIN
            UPDATE PMO_RES_MIN_MAX SET MAX_DATE = h_finish_D WHERE RESOURCE_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 489+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(490) : cplan_fd.sql(515) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("4__MAX_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
      ElsIf PMO_ORASTD.strncmp(h_finish_DT, h_start_D, 10) < 0 Then
         --strncpy(h_dayofyear, h_start_DT, 10);
         --h_dayofyear[10] = '\0';
         h_dayofyear := SubStr(h_start_DT, 1, 10);
         h_temp_var := PMO_CNST.SC_BIT_30;
         h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_dayofyear, h_start_D, h_temp_var);
         
         h_start_D := h_dayofyear;
         BEGIN
            UPDATE PMO_RES_MIN_MAX SET MIN_DATE = h_start_D WHERE RESOURCE_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 514+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(515) : cplan_fd.sql(540) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("5__MIN_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
      End If;
      
      --trace_printf("h_start_D = %s, h_finish_D = %s\n",h_start_D,h_finish_D);
      BEGIN
         SELECT PMO_ORASTD.TIME(h_start_DT), PMO_ORASTD.TIME(h_finish_DT)
         INTO h_start_time, h_finish_time
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 532+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(533) : cplan_fd.sql(558) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT PMO_ORASTD.JULIAN_DAY(h_start_DT), PMO_ORASTD.JULIAN_DAY(h_finish_DT)
         INTO h_julian_start, h_julian_finish
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 547+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(548) : cplan_fd.sql(573) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("h_start_time = %s, h_finish_time = %s, h_julian_start = %d, h_julian_finish = %d\n",h_start_time,h_finish_time,h_julian_start,h_julian_finish);
      If Coalesce(h_julian_start, 0) > Coalesce(h_julian_finish, 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 555+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(556) : cplan_fd.sql(581) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
      
      If Coalesce(h_julian_start, 0) = Coalesce(h_julian_finish, 0) Then
         BEGIN
            SELECT BIT_SCHED INTO h_bitID FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = TO_DATE(SubStr(h_start_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 570+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(571) : cplan_fd.sql(596) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_duration: %d\n", SC_STATE,h_duration);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            --              h_duration = getNumberOfMinutesBetweenStartingAndFinishing( (unsigned char *) h_bitID.h_value, h_start_time, h_finish_time );
            tempDuration := PMO_STPU.getNumberOfMinutesBetweenSAndF(h_bitID, h_start_time, h_finish_time);
            If tempDuration > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_min_per_day, 0);
            Else
               h_duration := tempDuration;
            End If;
         Else
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 586+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(587) : cplan_fd.sql(612) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN 0;
         End If;
      Else
         BEGIN
            SELECT BIT_SCHED INTO h_bitID FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = TO_DATE(SubStr(h_start_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 600+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(601) : cplan_fd.sql(626) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            --              h_duration = getNumberOfMinutesStartingAt( (unsigned char *) h_bitID.h_value, h_start_time );
            tempDuration := PMO_STPU.getNumberOfMinutesStartingAt(h_bitID, h_start_time);
            If tempDuration > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_min_per_day, 0);
            Else
               h_duration := tempDuration;
            End If;
            --trace_printf("h_start_time = %s, h_start_DT = %s, h_duration = %d\n",h_start_time,h_start_DT,h_duration);
         End If;
         
         BEGIN
            SELECT BIT_SCHED INTO h_bitID FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 625+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(626) : cplan_fd.sql(651) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            --              h_duration += getNumberOfMinutesFinishingAt( (unsigned char *) h_bitID.h_value, h_finish_time );
            tempDuration := PMO_STPU.getNumberOfMinutesFinishingAt(h_bitID, h_finish_time);
            If tempDuration > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_duration, 0) + Coalesce(h_min_per_day, 0);
            Else
               h_duration := Coalesce(h_duration, 0) + tempDuration;
            End If;
            
            --trace_printf("h_finish_time = %s, h_finish_DT = %s, h_duration = %d\n",h_finish_time,h_finish_DT,h_duration);
         End If;
         
         BEGIN
            SELECT h_duration + COALESCE(SUM(AVAILABLE_MIN),0) INTO h_duration
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_project_id AND DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 652+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(653) : cplan_fd.sql(678) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_duration = %d, %d\n",h_start_DT,h_finish_DT,h_duration_Ind,h_duration);
         h_duration := Coalesce(h_duration, 0);
         --h_duration_Ind = 0;
      End If;
      
      h_duration := Coalesce(h_duration, 0) + Coalesce(h_minute, 0);
   Else
      h_dayofyear := SubStr(h_start_DT, 1, 10);
      --trace_printf("2_** h_minute = %d, h_total = %d, h_duration = %d\n",h_minute,h_total,h_duration);
      h_total := Coalesce(h_minute, 0);
      
      BEGIN
         SELECT  PMO_ORASTD.TIME(h_start_DT) INTO h_start_time FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 675+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(676) : cplan_fd.sql(701) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("h_dayofyear = %s, h_start_time = %s, h_total = %d, h_duration = %d\n",h_dayofyear,h_start_time,h_total, h_duration);
      
      WHILE Coalesce(h_total, 0) < Coalesce(h_duration, 0) LOOP
         h_available := 0;
         h_minutes := Coalesce(h_duration, 0) - Coalesce(h_total, 0);
         
         OPEN CRES1;
         
         WHILE Coalesce(h_total, 0) < Coalesce(h_duration, 0) LOOP
            h_minutes := Coalesce(h_duration, 0) - Coalesce(h_total, 0);
            
            FETCH CRES1 INTO h_dayofyear, h_available, h_bitID;
            IF CRES1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("h_dayofyear = %s, h_available = %d\n",h_dayofyear,h_available);
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            If Coalesce(h_total, 0) = 0 Then
               h_total := PMO_STPU.getNumberOfMinutesStartingAt(h_bitID, h_start_time);
               
               --trace_printf("*** h_minutes: %d, h_duration: %d, h_total = %d\n", h_minutes,h_duration,h_total);
               If Coalesce(h_duration, 0) <= Coalesce(h_total, 0) Then
                  h_minutes := Coalesce(h_duration, 0) + Coalesce(h_available, 0) - Coalesce(h_total, 0);
                  h_total := 0;
                  EXIT;
               End If;
               --trace_printf("*** h_total = %d\n", h_total);
            Else
               h_total := Coalesce(h_total, 0) + Coalesce(h_available, 0);
            End If;
         END LOOP;
         
         BEGIN
            CLOSE CRES1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 725+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(726) : cplan_fd.sql(751) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --trace_printf("h_dayofyear = %s, h_duration = %d, h_total = %d\n",h_dayofyear,h_duration,h_total);
         If Coalesce(h_total, 0) = 0 Then
            EXIT;
         End If;
         
         If Coalesce(h_total, 0) < Coalesce(h_duration, 0) Then
            BEGIN
               SELECT  TO_DATE(SubStr(h_dayofyear, 1, 10)) + 1       INTO  h_dayofyear FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 744+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(745) : cplan_fd.sql(770) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_finish_D := '';
            h_available := Trunc((Coalesce(h_duration, 0) - Coalesce(h_total, 0)) / 480);
            If Coalesce(h_available, 0) < 135 Then
               h_temp_var := 135;
               h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_dayofyear, h_finish_D, h_temp_var);
            Else
               h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_dayofyear, h_finish_D, h_available);
            End If;
            
            BEGIN
               SELECT MAX(DAY_YEAR) INTO h_finish_D FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 766+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(767) : cplan_fd.sql(792) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE PMO_RES_MIN_MAX SET MAX_DATE = h_finish_D WHERE RESOURCE_ID = h_project_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 783+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(784) : cplan_fd.sql(809) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("6__MIN_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
         End If;
      END LOOP;
      
      --trace_printf("END__ h_dayofyear = %s, h_duration = %d, h_total = %d, h_minutes: %d\n",h_dayofyear,h_duration,h_total,h_minutes);
      If Coalesce(h_minutes, 0) > 0 Then
         h_finish_time := PMO_STPU.GET_FINISH_TIME(h_bitID, Coalesce(h_minutes, 0));
         --trace_printf("h_minutes = %d, h_finish_time = %s\n",h_minutes,h_finish_time);
         
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_finish_time) ) INTO h_finish_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 805+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(806) : cplan_fd.sql(831) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   --trace_printf("h_minute = %d, h_start_DT = %s, h_finish_DT = %s, h_duration = %d, %d\n",h_minute,h_start_DT,h_finish_DT,h_duration_Ind,h_duration);
   
   BEGIN
      UPDATE PMO_RES_MIN_MAX SET  START_DATE  = h_start_DT,
      FINISH_DATE = h_finish_DT,
      DURATION    = h_duration
      WHERE RESOURCE_ID = h_project_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 829+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(830) : cplan_fd.sql(855) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --timeLog (file,"RETURN FC_ASSN_DURATION:");
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 836+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(837) : cplan_fd.sql(862) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN Coalesce(h_duration, 0);
   
END;
/
SHOW ERRORS FUNCTION FC_ASSN_DURATION;
ALTER FUNCTION FC_ASSN_DURATION COMPILE DEBUG;

/*****************************************************************************************/

-- Get Finish Date OR Duration

/*****************************************************************************************/

Prompt == FUNCTION GET_FD_DURATION ==
CREATE OR REPLACE FUNCTION GET_FD_DURATION
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY VARCHAR2
   , h_finish_DT IN OUT NOCOPY VARCHAR2
   , sh_h_duration PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_duration PLS_INTEGER := sh_h_duration;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_dayofyear DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_calendar_id VARCHAR2(32);
   h_start_date VARCHAR2(26);
   h_finish_date VARCHAR2(26);
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_min_per_day PLS_INTEGER := 0;
   h_minute PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_available PLS_INTEGER := 0;
   h_julian_start PLS_INTEGER := 0;
   h_julian_finish PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_SF PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_id VARCHAR2(32);
   tempDuration PLS_INTEGER;
-- Declare Cursors
      CURSOR CRUP1 IS
         SELECT DAY_YEAR, AVAILABLE_MIN, BIT_SCHED FROM PMO_TMP_CDY
         WHERE DAY_YEAR >= TO_DATE(SubStr(h_dayofyear, 1, 10)) ORDER BY DAY_YEAR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('GET_FD_DURATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('GET_FD_DURATION'), '-- ' || SYSTIMESTAMP, ':r := GET_FD_DURATION(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_start_DT || ''''
          || ', ' || '''' || h_finish_DT || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 45+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: GET_FD_DURATION(46) : cplan_fd.sql(923) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);//trace_flush();
   If Coalesce(Length(h_start_DT), 0) = 0 Then
      --trace_printf("*** Warning: Start date is null\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 52+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(53) : cplan_fd.sql(930) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(h_duration, 0) < 0 Then
      --trace_printf("*** Warning: Duration is negative: %d\n",h_duration);
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 60+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(61) : cplan_fd.sql(938) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If (Coalesce(Length(h_finish_DT), 0) = 0) AND (Coalesce(h_duration, 0) = 0) Then
      --trace_printf("*** Warning: Finish date is null\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 68+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(69) : cplan_fd.sql(946) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If (Coalesce(Length(h_start_DT), 0) > 9) AND (Coalesce(Length(h_finish_DT), 0) > 9) 
         AND (PMO_ORASTD.strcmp(h_finish_DT, h_start_DT) < 0) Then
      --trace_printf("*** Warning: Finish date is earlier than Start date\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 77+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(78) : cplan_fd.sql(955) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(Length(h_finish_DT), 0) > 10 Then
      h_duration := 0;
      BEGIN
         SELECT PMO_ORASTD.TIME(h_start_DT), PMO_ORASTD.TIME(h_finish_DT) INTO h_start_time,h_finish_time FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 92+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(93) : cplan_fd.sql(970) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(Length(h_start_DT), 0) > 10 Then
      BEGIN
         SELECT PMO_ORASTD.TIME(h_start_DT) INTO h_start_time FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 105+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(106) : cplan_fd.sql(983) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_finish_DT := '';
      h_finish_time := '';
   Else
      h_start_time := '08.00.00';
      
      h_finish_DT := '';
      h_finish_time := '';
   End If;
   
   --trace_printf("h_start_time = %s,h_finish_time = %s\n", h_start_time,h_finish_time);//trace_flush();
   
   h_start_D := '';
   h_finish_D := '';
   h_start_date := '';
   h_finish_date := '';
   h_SF := 0;
   
   BEGIN
      SELECT COALESCE(MINUTE_PER_DAY,480), MIN_DATE - 1     , MAX_DATE + 1     , START_DATE, FINISH_DATE, DURATION
      INTO h_min_per_day, h_start_D, h_finish_D, h_start_date, h_finish_date, h_minute
      FROM PMO_TMP_MIN_MAX;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 137+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(138) : cplan_fd.sql(1015) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE = %d, h_start_D = %s, h_finish_D = %s\n",SC_STATE,h_start_D,h_finish_D);
   --trace_printf("h_start_date = %s, h_finish_date = %s, h_minute = %d\n",h_start_date,h_finish_date,h_minute);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      --trace_printf("*** Warning: No data in temporary table, h_project_id: %d;%s\n",h_project_id_Ind,h_project_id);
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 147+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(148) : cplan_fd.sql(1025) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   h_calendar_id := h_project_id;
   If PMO_ORASTD.strncmp(h_start_DT, h_start_D, 10) <= 0 Then
      --strncpy(h_dayofyear, h_start_DT, 10);
      --h_dayofyear[10] = '\0';
      
      h_dayofyear := SubStr(h_start_DT, 1, 10);
      h_temp_var := PMO_CNST.SC_BIT_30;
      
      h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_dayofyear, h_start_D
                                         , h_temp_var);
      
      h_start_D := h_dayofyear;
      --strcpy(h_start_D, h_dayofyear);
      
      BEGIN
         UPDATE PMO_TMP_MIN_MAX SET MIN_DATE = h_start_D;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 178+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(179) : cplan_fd.sql(1056) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("1__MIN_DATE *____*: SC_STATE: %d, h_start_D: %s\n", SC_STATE,h_start_D);
   ElsIf PMO_ORASTD.strncmp(h_start_DT, h_finish_D, 10) >= 0 Then
      --trace_printf("h_finish_D = %s,h_dayofyear = %s, h_duration = %d\n",h_finish_D,h_dayofyear,h_duration);
      If Coalesce(h_duration, 0) = 0 Then
         --strncpy(h_dayofyear, h_finish_DT, 10);
         --h_dayofyear[10] = '\0';
         h_dayofyear := SubStr(h_finish_DT, 1, 10);
         h_temp_var := 0;
         
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_finish_D, h_dayofyear
                                         , h_temp_var);
         
         h_finish_D := h_dayofyear;
         --strcpy(h_finish_D, h_dayofyear);
         
         BEGIN
            UPDATE PMO_TMP_MIN_MAX SET MAX_DATE = h_finish_D;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 209+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(210) : cplan_fd.sql(1087) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("2__MAX_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
      Else
         --strcpy(h_dayofyear, "");
         h_dayofyear := '';
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_finish_D, h_dayofyear
                                         , h_duration);
         
         BEGIN
            SELECT MAX(DAY_YEAR) INTO h_finish_D FROM PMO_TMP_CDY;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 228+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(229) : cplan_fd.sql(1106) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE PMO_TMP_MIN_MAX SET MAX_DATE = h_finish_D;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 245+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(246) : cplan_fd.sql(1123) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("3__MAX_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
      End If;
   End If;
   --trace_printf("h_start_D: %s, h_finish_D = %s, h_duration = %d\n",h_start_D,h_finish_D,h_duration);
   --trace_printf("h_start_DT = %s, h_finish_DT = %s\n",h_start_DT,h_finish_DT);
   
   If Coalesce(h_duration, 0) = 0 Then
      If (PMO_ORASTD.strcmp(h_start_date, h_start_DT) = 0) AND (PMO_ORASTD.strcmp(h_finish_date
            , h_finish_DT) = 0) Then
         h_duration := Coalesce(h_minute, 0);
         --trace_printf("RETURN_1: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 261+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(262) : cplan_fd.sql(1139) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_duration, 0);
      ElsIf PMO_ORASTD.strcmp(h_start_DT, h_finish_DT) = 0 Then
         h_duration := 0;
         --trace_printf("RETURN_2: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 268+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(269) : cplan_fd.sql(1146) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_duration, 0);
      End If;
      
      BEGIN
         SELECT DAY_YEAR INTO h_dayofyear FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_finish_DT, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 281+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(282) : cplan_fd.sql(1159) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("2__* SC_STATE: %d, h_SF: %d\n", SC_STATE,h_SF);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA AND (PMO_ORASTD.strncmp(h_finish_DT, h_start_D
            , 10) >= 0) AND (PMO_ORASTD.strncmp(h_finish_DT, h_finish_D, 10) <= 0) Then
         -- Finish date is non-working day
         If PMO_ORASTD.strncmp(h_start_DT, h_finish_DT, 10) = 0 Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMPDIFF(4, PMO_ORASTD.DB2TIMESTAMP(h_finish_DT), PMO_ORASTD.DB2TIMESTAMP(h_start_DT))
               INTO h_duration
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 301+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(302) : cplan_fd.sql(1179) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RETURN_3.1: h_min_per_day = %d, h_duration = %d\n",h_min_per_day,h_duration);
            
            If Coalesce(h_duration, 0) > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_min_per_day, 0);
            End If;
            --trace_printf("RETURN_3.2: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 312+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(313) : cplan_fd.sql(1190) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_duration, 0);
         Else
            h_finish_time := '17.00.00';
            h_minute := Coalesce(h_min_per_day, 0);
         End If;
         --trace_printf("Finish date is non-working day: h_finish_time: %s, h_minute = %d\n",h_finish_time,h_minute);
      Else
         h_minute := 0;
      End If;
   Else
      If (PMO_ORASTD.strcmp(h_start_date, h_start_DT) = 0) AND (Coalesce(h_duration, 0) = Coalesce(
            h_minute, 0)) Then
         h_finish_DT := h_finish_date;
         --trace_printf("RETURN_4: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 329+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(330) : cplan_fd.sql(1207) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_duration, 0);
      End If;
      
      h_minute := 0;
   End If;
   
   BEGIN
      SELECT DAY_YEAR INTO h_dayofyear FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_start_DT, 1, 10));
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 345+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(346) : cplan_fd.sql(1223) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("1__* SC_STATE: %d, h_dayofyear: %s\n", SC_STATE,h_dayofyear);
   
   --trace_printf("n.1__* h_start_DT: %s, h_start_D: %s, h_finish_D: %s\n", h_start_DT,h_start_D,h_finish_D);
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA AND (PMO_ORASTD.strncmp(h_start_DT, h_start_D
         , 10) >= 0) AND (PMO_ORASTD.strncmp(h_start_DT, h_finish_D, 10) <= 0) Then
      h_start_time := '08.00.00';
      If Coalesce(h_duration, 0) > 0 Then
         If Coalesce(h_duration, 0) = Coalesce(h_min_per_day, 0) Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD.TIME('17.00.00'))
               INTO h_finish_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 367+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(368) : cplan_fd.sql(1245) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RETURN_5: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 373+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(374) : cplan_fd.sql(1251) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_duration, 0);
         ElsIf Coalesce(h_duration, 0) <= Coalesce(h_min_per_day, 0) Then
            BEGIN
               SELECT TO_TIMESTAMP(PMO_ORASTD.DB2TIMESTAMP(h_start_DT) + (h_duration / 1440.0))
               INTO h_finish_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 387+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(388) : cplan_fd.sql(1265) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RETURN_6: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 393+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(394) : cplan_fd.sql(1271) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_duration, 0);
         End If;
      End If;
      
      h_minute := Coalesce(h_minute, 0) + Coalesce(h_min_per_day, 0);
      --trace_printf("Start date is non-working day: h_start_time: %s, h_minute = %d\n",h_start_time,h_minute);
   End If;
   
   --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_minute = %d\n",h_start_DT,h_finish_DT,h_minute);
   --trace_printf("h_start_D = %s, h_finish_D = %s, h_calendar_id = %s\n",h_start_D,h_finish_D, h_calendar_id);
   If (Coalesce(Length(h_start_D), 0) = 0) AND (Coalesce(Length(h_finish_D), 0) = 0) Then
      --trace_printf("*** Warning: h_start_D and h_finish_D are empty\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 408+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(409) : cplan_fd.sql(1286) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   --trace_printf("1_** h_minute = %d, h_total = %d, h_duration = %d\n",h_minute,h_total,h_duration);
   If Coalesce(h_duration, 0) = 0 Then
      --trace_printf("h_finish_DT = %s,h_finish_D = %s\n", h_finish_DT,h_finish_D);
      If PMO_ORASTD.strncmp(h_finish_DT, h_finish_D, 10) > 0 Then
         --strncpy(h_dayofyear, h_finish_DT, 10);
         --h_dayofyear[10] = '\0';
         h_dayofyear := SubStr(h_finish_DT, 1, 10);
         h_temp_var := 0;
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_finish_D, h_dayofyear
                                         , h_temp_var);
         
         h_finish_D := h_dayofyear;
         BEGIN
            UPDATE PMO_TMP_MIN_MAX SET MAX_DATE = h_finish_D;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 437+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(438) : cplan_fd.sql(1315) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("4__MAX_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
      ElsIf PMO_ORASTD.strncmp(h_finish_DT, h_start_D, 10) < 0 Then
         --strncpy(h_dayofyear, h_start_DT, 10);
         --h_dayofyear[10] = '\0';
         h_dayofyear := SubStr(h_start_DT, 1, 10);
         h_temp_var := PMO_CNST.SC_BIT_30;
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_dayofyear, h_start_D
                                         , h_temp_var);
         
         h_start_D := h_dayofyear;
         BEGIN
            UPDATE PMO_TMP_MIN_MAX SET MIN_DATE = h_start_D;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 463+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(464) : cplan_fd.sql(1341) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("5__MIN_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
      End If;
      
      --trace_printf("h_start_D = %s, h_finish_D = %s\n",h_start_D,h_finish_D);
      BEGIN
         SELECT PMO_ORASTD.TIME(h_start_DT), PMO_ORASTD.TIME(h_finish_DT)
         INTO h_start_time, h_finish_time
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 481+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(482) : cplan_fd.sql(1359) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT PMO_ORASTD.JULIAN_DAY(h_start_DT), PMO_ORASTD.JULIAN_DAY(h_finish_DT)
         INTO h_julian_start, h_julian_finish
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 496+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(497) : cplan_fd.sql(1374) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("h_start_time = %s, h_finish_time = %s, h_julian_start = %d, h_julian_finish = %d\n",h_start_time,h_finish_time,h_julian_start,h_julian_finish);
      If Coalesce(h_julian_start, 0) > Coalesce(h_julian_finish, 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 504+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(505) : cplan_fd.sql(1382) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
      
      If Coalesce(h_julian_start, 0) = Coalesce(h_julian_finish, 0) Then
         BEGIN
            SELECT BIT_SCHED INTO h_bitID FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_start_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 518+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(519) : cplan_fd.sql(1396) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_duration: %d\n", SC_STATE,h_duration);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            --              h_duration = getNumberOfMinutesBetweenStartingAndFinishing( (unsigned char *) h_bitID.h_value, h_start_time, h_finish_time );
            tempDuration := PMO_STPU.getNumberOfMinutesBetweenSAndF(h_bitID, h_start_time, h_finish_time);
            If tempDuration > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_min_per_day, 0);
            Else
               h_duration := tempDuration;
            End If;
         Else
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 534+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(535) : cplan_fd.sql(1412) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN 0;
         End If;
      Else
         BEGIN
            SELECT BIT_SCHED INTO h_bitID FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_start_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 547+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(548) : cplan_fd.sql(1425) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            --              h_duration = getNumberOfMinutesStartingAt( (unsigned char *) h_bitID.h_value, h_start_time );
            tempDuration := PMO_STPU.getNumberOfMinutesStartingAt(h_bitID, h_start_time);
            If tempDuration > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_min_per_day, 0);
            Else
               h_duration := tempDuration;
            End If;
            --trace_printf("h_start_time = %s, h_start_DT = %s, h_duration = %d\n",h_start_time,h_start_DT,h_duration);
         End If;
         
         BEGIN
            SELECT BIT_SCHED INTO h_bitID FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 571+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(572) : cplan_fd.sql(1449) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            --              h_duration += getNumberOfMinutesFinishingAt( (unsigned char *) h_bitID.h_value, h_finish_time );
            tempDuration := PMO_STPU.getNumberOfMinutesFinishingAt(h_bitID, h_finish_time);
            If tempDuration > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_duration, 0) + Coalesce(h_min_per_day, 0);
            Else
               h_duration := Coalesce(h_duration, 0) + tempDuration;
            End If;
            --trace_printf("h_finish_time = %s, h_finish_DT = %s, h_duration = %d\n",h_finish_time,h_finish_DT,h_duration);
         End If;
         
         BEGIN
            SELECT h_duration + COALESCE(SUM(AVAILABLE_MIN),0) INTO h_duration
            FROM PMO_TMP_CDY WHERE DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 596+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(597) : cplan_fd.sql(1474) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_duration = %d, %d\n",h_start_DT,h_finish_DT,h_duration_Ind,h_duration);
         h_duration := Coalesce(h_duration, 0);
         --h_duration_Ind = 0;
      End If;
      --trace_printf("A_1.1 h_minute = %d, h_duration = %d\n",h_minute,h_duration);
      
      h_duration := Coalesce(h_duration, 0) + Coalesce(h_minute, 0);
   Else
      h_dayofyear := SubStr(h_start_DT, 1, 10);
      --trace_printf("2_** h_minute = %d, h_total = %d, h_duration = %d\n",h_minute,h_total,h_duration);
      h_total := Coalesce(h_minute, 0);
      
      BEGIN
         SELECT  PMO_ORASTD.TIME(h_start_DT) INTO h_start_time FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 620+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(621) : cplan_fd.sql(1498) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("h_dayofyear = %s, h_start_time = %s, h_total = %d, h_duration = %d\n",h_dayofyear,h_start_time,h_total, h_duration);
      
      WHILE Coalesce(h_total, 0) < Coalesce(h_duration, 0) LOOP
         h_available := 0;
         h_minutes := Coalesce(h_duration, 0) - Coalesce(h_total, 0);
         
         OPEN CRUP1;
         
         WHILE Coalesce(h_total, 0) < Coalesce(h_duration, 0) LOOP
            h_minutes := Coalesce(h_duration, 0) - Coalesce(h_total, 0);
            
            FETCH CRUP1 INTO h_dayofyear, h_available, h_bitID;
            IF CRUP1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("h_dayofyear = %s, h_available = %d\n",h_dayofyear,h_available);
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            If Coalesce(h_total, 0) = 0 Then
               h_total := PMO_STPU.getNumberOfMinutesStartingAt(h_bitID, h_start_time);
               
               --trace_printf("*** h_minutes: %d, h_duration: %d, h_total = %d\n", h_minutes,h_duration,h_total);
               If Coalesce(h_duration, 0) <= Coalesce(h_total, 0) Then
                  h_minutes := Coalesce(h_duration, 0) + Coalesce(h_available, 0) - Coalesce(h_total, 0);
                  h_total := 0;
                  EXIT;
               End If;
               --trace_printf("*** h_total = %d\n", h_total);
            Else
               h_total := Coalesce(h_total, 0) + Coalesce(h_available, 0);
            End If;
         END LOOP;
         
         BEGIN
            CLOSE CRUP1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 670+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(671) : cplan_fd.sql(1548) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --trace_printf("h_dayofyear = %s, h_duration = %d, h_total = %d\n",h_dayofyear,h_duration,h_total);
         If Coalesce(h_total, 0) = 0 Then
            EXIT;
         End If;
         
         If Coalesce(h_total, 0) < Coalesce(h_duration, 0) Then
            BEGIN
               SELECT  TO_DATE(SubStr(h_dayofyear, 1, 10)) + 1       INTO  h_dayofyear FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 689+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(690) : cplan_fd.sql(1567) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_finish_D := '';
            h_available := Trunc((Coalesce(h_duration, 0) - Coalesce(h_total, 0)) / 480);
            If Coalesce(h_available, 0) < 135 Then
               h_temp_var := 135;
               h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_dayofyear, h_finish_D
                                         , h_temp_var);
            Else
               h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_dayofyear, h_finish_D
                                         , h_available);
            End If;
            
            BEGIN
               SELECT MAX(DAY_YEAR) INTO h_finish_D FROM PMO_TMP_CDY;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 713+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(714) : cplan_fd.sql(1591) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE PMO_TMP_MIN_MAX SET MAX_DATE = h_finish_D;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 730+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(731) : cplan_fd.sql(1608) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("6__MIN_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
         End If;
      END LOOP;
      
      --trace_printf("END__ h_dayofyear = %s, h_duration = %d, h_total = %d, h_minutes: %d\n",h_dayofyear,h_duration,h_total,h_minutes);
      If Coalesce(h_minutes, 0) > 0 Then
         h_finish_time := PMO_STPU.GET_FINISH_TIME(h_bitID, Coalesce(h_minutes, 0));
         --trace_printf("h_minutes = %d, h_finish_time = %s\n",h_minutes,h_finish_time);
         
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_finish_time) ) INTO h_finish_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 752+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(753) : cplan_fd.sql(1630) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   --trace_printf("h_minute = %d, h_start_DT = %s, h_finish_DT = %s, h_duration = %d, %d\n",h_minute,h_start_DT,h_finish_DT,h_duration_Ind,h_duration);
   
   BEGIN
      UPDATE PMO_TMP_MIN_MAX SET  START_DATE  = h_start_DT,
      FINISH_DATE = h_finish_DT,
      DURATION    = h_duration;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 775+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(776) : cplan_fd.sql(1653) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --timeLog (file,"RETURN GET_FD_DURATION:");
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 782+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(783) : cplan_fd.sql(1660) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN Coalesce(h_duration, 0);
   
END;
/
SHOW ERRORS FUNCTION GET_FD_DURATION;
ALTER FUNCTION GET_FD_DURATION COMPILE DEBUG;

-- ********************************************************************************

-- h_bitflag = 1,   h_task_start_DT has been changed

--           = 2,   h_task_finish_DT (h_ta_finish_DT) has been changed

--           = 3,   h_task_start_DT  h_task_finish_DT have been changed

--           = 4,   h_ta_start_DT has been changed (only at the assignment level)

-- ================================================================================

-- h_bitflag1 > 0,  new date > old date

--            < 0,  new date < old date

-- ********************************************************************************

Prompt == FUNCTION FC_FD_CTD_TDY ==
CREATE OR REPLACE FUNCTION FC_FD_CTD_TDY
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_res_type IN OUT NOCOPY VARCHAR2
   , h_SD IN OUT NOCOPY DATE
   , h_FD IN OUT NOCOPY DATE
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_work_type PLS_INTEGER
   , sh_h_per_day PLS_INTEGER
   , sh_h_percent_units PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , sh_h_bitflag1 PLS_INTEGER
   , sh_h_leveling PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_work_type PLS_INTEGER := sh_h_work_type;
   h_per_day PLS_INTEGER := sh_h_per_day;
   h_percent_units PLS_INTEGER := sh_h_percent_units;
   h_bitflag PLS_INTEGER := Coalesce(sh_h_bitflag, 0);
   h_bitflag1 PLS_INTEGER := Coalesce(sh_h_bitflag1, 0);
   h_leveling PLS_INTEGER := Coalesce(sh_h_leveling, 0);
   h_parent_id VARCHAR2(32);
   h_dayofyear DATE;
   h_bit_flag VARCHAR2(1);
   h_null_str VARCHAR2(1);
   h_start_D DATE;
   h_finish_D DATE;
   h_minutes PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_cnt PLS_INTEGER := 0;
   h_ins_cnt PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_res_cost NUMBER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_assign_string VARCHAR2(8192);
   stmt VARCHAR2(7999);
   i PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR C_D1 IS
         SELECT SCHED_MIN, BIT_FLAG, PARENT_ID, BIT_SCHED
         FROM TASK_DAY_OF_YEAR
         WHERE  ROWNUM <= 1 AND  PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR >= TO_DATE(SubStr(h_SD, 1, 10));

      CURSOR C_FDD1 IS
         SELECT DAY_YEAR FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR <= TO_DATE(SubStr(h_FD, 1, 10))
         ORDER BY DAY_YEAR DESC;

      -- Dynamic Cursor Declaration
      C_FDI1 PMO_DB2M.REF_CURSOR;

      -- Dynamic Cursor Declaration
      C_FDI2 PMO_DB2M.REF_CURSOR;

      CURSOR C_FSTR IS
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR) AS JDY, SCHED_MIN FROM LEVEL_TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         ORDER BY DAY_YEAR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FD_CTD_TDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FD_CTD_TDY'), '-- ' || SYSTIMESTAMP, ':r := FC_FD_CTD_TDY(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_res_type || ''''
          || ', ' || COALESCE(NULLIF('''' || h_SD || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_FD || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_percent_units), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag1), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_leveling), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_CTD_TDY', 88+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FD_CTD_TDY(89) : cplan_fd.sql(1776) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_leveling: %d, h_bitflag: %d, h_bitflag1: %d, h_start_DT: %d %s, h_finish_DT: %d %s\n", h_leveling,h_bitflag,h_bitflag1,h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);
   --trace_printf("h_tsk_assignment_id: %s, h_SD: %d %s, h_FD: %d %s\n", h_tsk_assignment_id,h_SD_Ind,h_SD,h_FD_Ind,h_FD);
   h_temp_var := 0;
   h_ins_cnt := 0;
   h_count := 0;
   h_cnt := 0;
   
   -- get first day
   /*  BEGIN_SQL
           SELECT SCHED_MIN, BIT_FLAG, PARENT_ID, BIT_SCHED
           INTO HOST(h_minutes), HOST(h_bit_flag), HOST(h_parent_id), HOST(h_bitID)
           FROM TASK_DAY_OF_YEAR
           WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) AND DAY_YEAR = DATE(HOST(h_SD));
       END_SQL;
   
       if (SC_NODATA)
       {
           return -1;
       }
   */
   
   OPEN C_D1;
   
   FETCH C_D1 INTO h_minutes, h_bit_flag, h_parent_id, h_bitID;
   IF C_D1%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         CLOSE C_D1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 130+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(131) : cplan_fd.sql(1818) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_CTD_TDY', 135+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_FD_CTD_TDY(136) : cplan_fd.sql(1823) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN -1;
   Else
      BEGIN
         CLOSE C_D1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 147+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(148) : cplan_fd.sql(1835) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --trace_printf("SC_STATE: %d, h_minutes: %d,h_bit_flag: %s,h_parent_id: %s\n",SC_STATE,h_minutes,h_bit_flag,h_parent_id);//trace_flush();
   
   If h_bitflag = 2 Then
      If h_bitflag1 < 0 Then
         If h_leveling = 0 Then
            -- delete TDY between new task date and old ta date
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND ACTUAL_FLAG = 0 AND DAY_YEAR > TO_DATE(SubStr(h_finish_DT, 1, 10)) AND DAY_YEAR <= TO_DATE(SubStr(h_FD, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 173+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(174) : cplan_fd.sql(1861) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("DELETE (h_bitflag1 < 0): SC_STATE: %d, h_tsk_assignment_id: %s\n",SC_STATE,h_tsk_assignment_id);//trace_flush();
         Else
            -- insert LEVEL_TASK_DAY_OF_YEAR
            BEGIN
               INSERT INTO LEVEL_TASK_DAY_OF_YEAR SELECT * FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND ACTUAL_FLAG = 0 AND DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 194+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(195) : cplan_fd.sql(1882) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If SubStr(h_res_type, 1, 1) = '0' Then
            If h_leveling = 0 Then
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  r.DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE r.ELEMENT_ID = h_res_id AND r.DAY_YEAR > TO_DATE(SubStr(h_FD, 1, 10)) AND r.DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10))
                  AND EXISTS (SELECT p.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR p
                  WHERE p.ELEMENT_ID = h_project_id AND p.DAY_YEAR = r.DAY_YEAR);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 239+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(240) : cplan_fd.sql(1927) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO LEVEL_TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  r.DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE r.ELEMENT_ID = h_res_id AND r.DAY_YEAR > TO_DATE(SubStr(h_FD, 1, 10)) AND r.DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10))
                  AND EXISTS (SELECT p.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR p
                  WHERE p.ELEMENT_ID = h_project_id AND p.DAY_YEAR = r.DAY_YEAR);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 281+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(282) : cplan_fd.sql(1969) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            If h_leveling = 0 Then
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  r.DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE r.ELEMENT_ID = h_project_id AND r.DAY_YEAR > TO_DATE(SubStr(h_FD, 1, 10)) AND r.DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 323+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(324) : cplan_fd.sql(2011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO LEVEL_TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  r.DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE r.ELEMENT_ID = h_project_id AND r.DAY_YEAR > TO_DATE(SubStr(h_FD, 1, 10)) AND r.DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 363+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(364) : cplan_fd.sql(2051) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
   Else
      If h_bitflag1 > 0 Then
         BEGIN
            SELECT COUNT(DAY_YEAR) INTO h_ins_cnt FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND ACTUAL_FLAG = 0 AND DAY_YEAR >= TO_DATE(SubStr(h_SD, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_start_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 382+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(383) : cplan_fd.sql(2070) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If h_leveling = 0 Then
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND ACTUAL_FLAG = 0 AND DAY_YEAR >= TO_DATE(SubStr(h_SD, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_start_DT, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 402+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(403) : cplan_fd.sql(2090) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO LEVEL_TASK_DAY_OF_YEAR SELECT * FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND ACTUAL_FLAG = 0 AND DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 421+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(422) : cplan_fd.sql(2109) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If SubStr(h_res_type, 1, 1) = '0' Then
            BEGIN
               SELECT COUNT(r.DAY_YEAR) INTO h_ins_cnt FROM CALENDAR_DAY_OF_YEAR r JOIN CALENDAR_DAY_OF_YEAR p
               ON r.DAY_YEAR = p.DAY_YEAR
               WHERE p.ELEMENT_ID = h_project_id AND r.ELEMENT_ID = h_res_id
               AND p.DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND p.DAY_YEAR < TO_DATE(SubStr(h_SD, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 439+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(440) : cplan_fd.sql(2127) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("C.1====> SC_STATE: %d, h_res_id: %s\n",SC_STATE,h_res_id);//trace_flush();
            
            If h_leveling = 0 Then
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  r.DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE r.ELEMENT_ID = h_res_id AND r.DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND r.DAY_YEAR < TO_DATE(SubStr(h_SD, 1, 10))
                  AND EXISTS (SELECT p.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR p
                  WHERE p.ELEMENT_ID = h_project_id AND p.DAY_YEAR = r.DAY_YEAR);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 483+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(484) : cplan_fd.sql(2171) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO LEVEL_TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  r.DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE r.ELEMENT_ID = h_res_id AND r.DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND r.DAY_YEAR < TO_DATE(SubStr(h_SD, 1, 10))
                  AND EXISTS (SELECT p.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR p
                  WHERE p.ELEMENT_ID = h_project_id AND p.DAY_YEAR = r.DAY_YEAR);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 525+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(526) : cplan_fd.sql(2213) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("C.2====> SC_STATE: %d, h_project_id: %s, h_res_id: %s\n",SC_STATE,h_project_id,h_res_id);//trace_flush();
         Else
            BEGIN
               SELECT COUNT(DAY_YEAR) INTO h_ins_cnt FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_SD, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 541+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(542) : cplan_fd.sql(2229) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("C.2====> SC_STATE: %d, h_ins_cnt: %d\n",SC_STATE,h_ins_cnt);//trace_flush();
            
            If h_leveling = 0 Then
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_SD, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 583+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(584) : cplan_fd.sql(2271) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO LEVEL_TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_SD, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 623+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(624) : cplan_fd.sql(2311) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA AND (Coalesce(h_ins_cnt, 0) = 0) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_CTD_TDY', 632+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_FD_CTD_TDY(633) : cplan_fd.sql(2320) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN -1;
         End If;
         
         OPEN C_FDD1;
         
         WHILE Coalesce(h_ins_cnt, 0) > 0 LOOP
            FETCH C_FDD1 INTO h_dayofyear;
            IF C_FDD1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            If h_leveling = 0 Then
               BEGIN
                  DELETE FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                  AND ACTUAL_FLAG = 0 AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 665+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(666) : cplan_fd.sql(2353) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("D.1====> SC_STATE: %d, h_dayofyear: %s, h_ins_cnt: %d\n",SC_STATE,h_dayofyear,h_ins_cnt);//trace_flush();
                     h_ins_cnt := Coalesce(h_ins_cnt, 0) - 1;
         END LOOP;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            FETCH C_FDD1 INTO h_dayofyear;
            IF C_FDD1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("D.2====> SC_STATE: %d, h_dayofyear: %s, h_ins_cnt: %d\n",SC_STATE,h_dayofyear,h_ins_cnt);//trace_flush();
         End If;
         
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_finish_DT)) INTO h_finish_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 693+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(694) : cplan_fd.sql(2381) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            CLOSE C_FDD1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 706+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(707) : cplan_fd.sql(2394) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   --trace_printf("1.0====> h_ins_cnt: %d, h_start_DT: %s, h_finish_DT: %s\n",h_ins_cnt,h_start_DT,h_finish_DT);//trace_flush();
   If Coalesce(h_ins_cnt, 0) <> 0 Then
      If PMO_ORASTD.strncmp(h_FD, h_start_DT, 10) < 0 Then
         BEGIN
            SELECT TO_DATE(SubStr(h_start_DT, 1, 10)) - 1      INTO h_start_D
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 725+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(726) : cplan_fd.sql(2413) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_start_D := SubStr(h_FD, 1, 10);
      End If;
      --trace_printf("1.1====> h_start_D: %s\n",h_start_D);//trace_flush();
      
      If SubStr(h_res_type, 1, 1) = '0' Then
         stmt := 
               'SELECT r.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR r JOIN CALENDAR_DAY_OF_YEAR p ON p.DAY_YEAR = r.DAY_YEAR WHERE p.ELEMENT_ID = ''' 
               || h_project_id || ''' '
                || 'AND r.ELEMENT_ID = ''' || h_res_id || ''' AND p.DAY_YEAR > TO_DATE(SubStr(''' || h_start_D 
                     || ''', 1, 10))  AND  ROWNUM <= ' || TO_CHAR(Coalesce(h_ins_cnt, 0));
         --trace_printf("RESOURCE: h_ins_cnt: %d\n%s\n", h_ins_cnt,(char *) stmt);
         
         OPEN C_FDI1 FOR stmt;
         
         WHILE Coalesce(h_ins_cnt, 0) > 0 LOOP
            FETCH C_FDI1 INTO h_dayofyear;
            IF C_FDI1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("1.i====> h_dayofyear: %s\n",h_dayofyear);//trace_flush();
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            If h_leveling = 0 Then
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID )
                  VALUES (
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  h_dayofyear,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 790+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(791) : cplan_fd.sql(2478) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO LEVEL_TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID )
                  VALUES (
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  h_dayofyear,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 828+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(829) : cplan_fd.sql(2516) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("1.1====> SC_STATE: %d, h_ins_cnt: %d, h_dayofyear: %s\n",SC_STATE,h_ins_cnt,h_dayofyear);//trace_flush();
            End If;
                     h_ins_cnt := Coalesce(h_ins_cnt, 0) - 1;
         END LOOP;
         
         BEGIN
            CLOSE C_FDI1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 845+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(846) : cplan_fd.sql(2533) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         stmt := 'SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = ''' || h_project_id || ''' '
                || 'AND DAY_YEAR > TO_DATE(SubStr(''' || h_start_D || ''', 1, 10))  AND  ROWNUM <= ' 
                     || TO_CHAR(Coalesce(h_ins_cnt, 0));
         --trace_printf("PROFILE:\n%s\n", (char *) stmt);
         
         OPEN C_FDI2 FOR stmt;
         
         WHILE Coalesce(h_ins_cnt, 0) > 0 LOOP
            FETCH C_FDI2 INTO h_dayofyear;
            IF C_FDI2%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            If h_leveling = 0 Then
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID )
                  VALUES (
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  h_dayofyear,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 902+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(903) : cplan_fd.sql(2590) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO LEVEL_TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID )
                  VALUES (
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  h_dayofyear,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 940+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(941) : cplan_fd.sql(2628) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("1.2====> SC_STATE: %d, h_ins_cnt: %d, h_dayofyear: %s\n",SC_STATE,h_ins_cnt,h_dayofyear);//trace_flush();
                     h_ins_cnt := Coalesce(h_ins_cnt, 0) - 1;
         END LOOP;
         
         BEGIN
            CLOSE C_FDI2 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 957+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(958) : cplan_fd.sql(2645) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --trace_printf("1.3====> h_finish_DT: %s, h_dayofyear: %s\n",h_finish_DT,h_dayofyear);//trace_flush();
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_finish_DT)) INTO h_finish_DT
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 973+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(974) : cplan_fd.sql(2661) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --trace_printf("h_work_type: %d, h_minutes: %d, h_start_DT: %s, h_finish_DT: %s\n",h_work_type,h_minutes,h_start_DT,h_finish_DT);//trace_flush();
   
   If h_leveling = 0 Then
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP(MIN(DAY_YEAR),PMO_ORASTD.TIME(h_start_DT)), PMO_ORASTD.DB2TIMESTAMP(MAX(DAY_YEAR),PMO_ORASTD.TIME(h_finish_DT))
         INTO h_start_DT, h_finish_DT
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 992+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(993) : cplan_fd.sql(2680) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_work_type, 0) = 2 Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            SCHED_START = h_start_DT,
            SCHED_FINISH = h_finish_DT
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 1013+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(1014) : cplan_fd.sql(2701) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            PLANNED_START = h_start_DT,
            PLANNED_FINISH = h_finish_DT
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 1033+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(1034) : cplan_fd.sql(2721) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      h_temp_var := FC_INI_ASSIGN_STR();
      h_assign_string := '';
      h_total := 0;
      
      OPEN C_FSTR;
      
      LOOP
         FETCH C_FSTR INTO h_julian_day, h_minutes;
         IF C_FSTR%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
         h_temp_var := FC_ASSIGN_STRING(h_julian_day, h_minutes, h_assign_string);
      END LOOP;
      
      BEGIN
         CLOSE C_FSTR ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 1068+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(1069) : cplan_fd.sql(2756) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_temp_var := FC_GET_ASSIGN_STR(h_assign_string);
      
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET
         LEVEL_START         = h_start_DT,
         LEVEL_FINISH        = h_finish_DT,
         LEVEL_EFFORT        = h_total,
         LEVEL_ASSIGN_STRING = h_assign_string
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 1092+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(1093) : cplan_fd.sql(2780) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_total: %d, h_assign_string: %s\n", SC_STATE,h_total,h_assign_string);
      --trace_printf("h_start_DT: %d %s, h_finish_DT: %d %s\n", h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);
   End If;
   --trace_printf("2.====> SC_STATE: %d, h_tsk_assignment_id: %s\n",SC_STATE,h_tsk_assignment_id);//trace_flush();
   
   h_temp_var := PMO_FUNC.BIT_OR(Coalesce(h_percent_units, 0), PMO_CNST.SC_BIT_30);
   h_temp_var := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_CTD_TDY', 1105+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FD_CTD_TDY(1106) : cplan_fd.sql(2793) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FD_CTD_TDY;
ALTER FUNCTION FC_FD_CTD_TDY COMPILE DEBUG;

/***************************************************************************************************************************/

-- h_start_DT = task new start  date

-- h_start_DT = task new finish date

-- h_bitflag  = 1 Start Date has been changed

--            = 2 Finish Date has been changed

--            = 3 Start Date  Finish Date have been changed

/***************************************************************************************************************************/

Prompt == FUNCTION FC_CHANGE_TA_DATES ==
CREATE OR REPLACE FUNCTION FC_CHANGE_TA_DATES
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_task_start_DT IN OUT NOCOPY TIMESTAMP
   , h_task_finish_DT IN OUT NOCOPY TIMESTAMP
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_per_day PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_task_type PLS_INTEGER
   , sh_h_leveling PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_per_day PLS_INTEGER := sh_h_per_day;
   h_task_type PLS_INTEGER := Coalesce(sh_h_task_type, 0);
   h_leveling PLS_INTEGER := Coalesce(sh_h_leveling, 0);
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_dayofyear DATE;
   h_SD DATE;
   h_FD DATE;
   h_ta_start_DT TIMESTAMP;
   h_ta_finish_DT TIMESTAMP;
   h_start_D DATE;
   h_finish_D DATE;
   h_last_submitted DATE;
   h_res_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_bit_flag VARCHAR2(1);
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_delay PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_bitflag1 PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_reg_sale NUMBER := 0;
   h_reg_cost NUMBER := 0;
   stmt VARCHAR2(7999);
   i_bitflag PLS_INTEGER NOT NULL := 0;
   i PLS_INTEGER;
   i001 PLS_INTEGER;
-- Declare Cursors
      -- Dynamic Cursor Declaration
      C_FDD2 PMO_DB2M.REF_CURSOR;

      CURSOR C_CHGD2 IS
         SELECT TSK_ASSIGNMENT_ID, ETC, ASSIGNMENT_DELAY, RESOURCE_ID, RESOURCE_TYPE,
         UNITS_MSP, Trunc(BIT_FLAG), UNRESOLVED_START, LAST_SCHED_MIN, LAST_SUBMITTED,
         TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id
         AND TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)) BETWEEN h_start_D AND h_finish_D
         ORDER BY UNRESOLVED_START;

      CURSOR C_CHGD1 IS
         SELECT TSK_ASSIGNMENT_ID, ETC, ASSIGNMENT_DELAY, RESOURCE_ID, RESOURCE_TYPE,
         UNITS_MSP, BIT_FLAG, C_CONTOUR, PROJECT_ASSIGNMENT_ID, LAST_SCHED_MIN, LAST_SUBMITTED,
         TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND ACTUAL_START IS NuLL
         ORDER BY UNRESOLVED_START;

      CURSOR C_P_DELAY IS
         SELECT * FROM (SELECT p.DAY_YEAR
         FROM CALENDAR_DAY_OF_YEAR p
         WHERE p.ELEMENT_ID = h_project_id
         AND p.DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10))
         ORDER BY DAY_YEAR
         ) WHERE ROWNUM<=300;

      CURSOR C_R_DELAY IS
         SELECT * FROM (SELECT p.DAY_YEAR
         FROM CALENDAR_DAY_OF_YEAR p
         JOIN CALENDAR_DAY_OF_YEAR r ON p.DAY_YEAR = r.DAY_YEAR
         WHERE p.ELEMENT_ID = h_project_id AND r.ELEMENT_ID = h_res_id
         AND r.DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10))
         ORDER BY DAY_YEAR
         ) WHERE ROWNUM<=300;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CHANGE_TA_DATES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CHANGE_TA_DATES'), '-- ' || SYSTIMESTAMP, ':r := FC_CHANGE_TA_DATES(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_task_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_task_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_task_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_leveling), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHANGE_TA_DATES', 101+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CHANGE_TA_DATES(102) : cplan_fd.sql(2917) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_leveling: %d, h_start_DT: %d %s, h_finish_DT: %d %s\n",h_leveling,h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);//trace_flush();
   --trace_printf("h_task_start_DT: %d %s, h_task_finish_task_DT: %d %s\n",h_task_start_DT_Ind,h_task_start_DT,h_task_finish_DT_Ind,h_task_finish_DT);//trace_flush();
   
   If (h_task_type = PMO_CNST.SC_FD_EFFORT_DRIVEN) AND (h_leveling > 0) Then
      BEGIN
         SELECT MAX(DAY_YEAR) INTO h_SD FROM LEVEL_TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND RESOURCE_ID IN (
         SELECT RESOURCE_ID FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id);
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 118+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(119) : cplan_fd.sql(2934) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_SD: %d %s, h_task_id: %s\n",SC_STATE,h_SD_Ind,h_SD,h_task_id);//trace_flush();
      
      If (h_SD IS NOT NULL) AND (PMO_ORASTD.strncmp(h_SD, h_start_DT, 10) >= 0) Then
         If PMO_ORASTD.strncmp(h_SD, h_start_DT, 10) = 0 Then
            h_SD := SubStr(h_start_DT, 1, 10);
         End If;
         
         stmt := 'SELECT DAY_YEAR FROM PMO_TMP_CDY WHERE DAY_YEAR > TO_DATE(SubStr(''' || h_SD 
               || ''', 1, 10)) '
                || ' AND  ROWNUM <= ' || TO_CHAR(h_leveling);
         --trace_printf("DURATION: h_leveling: %d\n%s\n",h_leveling,(char *) stmt);//trace_flush();
         
         OPEN C_FDD2 FOR stmt;
         
         i := h_leveling;
         WHILE i > 0 LOOP
            FETCH C_FDD2 INTO h_FD;
            IF C_FDD2%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHANGE_TA_DATES', 146+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_CHANGE_TA_DATES(147) : cplan_fd.sql(2962) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN -1;
            End If;
            
            If i = h_leveling Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_FD, 1, 10)), PMO_ORASTD.TIME(h_start_DT)) INTO h_start_DT FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 160+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(161) : cplan_fd.sql(2976) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
                     i := i - 1;
         END LOOP;
         
         BEGIN
            CLOSE C_FDD2 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 176+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(177) : cplan_fd.sql(2992) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_FD, 1, 10)), PMO_ORASTD.TIME(h_finish_DT)) INTO h_finish_DT FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 189+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(190) : cplan_fd.sql(3005) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("h_start_DT: %d %s, h_finish_DT: %d %s\n",h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);//trace_flush();
      End If;
   End If;
   
   If PMO_ORASTD.strncmp(h_start_DT, h_task_start_DT, 10) <> 0 Then
      i_bitflag := 1;
   End If;
   If PMO_ORASTD.strncmp(h_finish_DT, h_task_finish_DT, 10) <> 0 Then
      i_bitflag := i_bitflag + 2;
   End If;
   
   --trace_printf("i_bitflag: %d, h_task_id: %d %s\n",i_bitflag,h_task_id_Ind,h_task_id);//trace_flush();
   If i_bitflag = 0 Then
      -- insert LEVEL_TASK_DAY_OF_YEAR
      BEGIN
         INSERT INTO LEVEL_TASK_DAY_OF_YEAR SELECT * FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND ACTUAL_FLAG = 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 220+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(221) : cplan_fd.sql(3036) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("==> INSERT INTO LEVEL_TASK_DAY_OF_YEAR SC_STATE: %d\n",SC_STATE);//trace_flush();
      
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHANGE_TA_DATES', 227+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_CHANGE_TA_DATES(228) : cplan_fd.sql(3043) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If i_bitflag = 2 Then
      h_bitflag1 := PMO_ORASTD.strncmp(h_finish_DT, h_task_finish_DT, 10);
      -- new task date , old task date
      
      If Coalesce(h_bitflag1, 0) >= 0 Then
         h_start_D := SubStr(h_task_finish_DT, 1, 10);
         h_finish_D := SubStr(h_finish_DT, 1, 10);
      Else
         h_start_D := SubStr(h_finish_DT, 1, 10);
         h_finish_D := SubStr(h_task_finish_DT, 1, 10);
      End If;
      
      OPEN C_CHGD2;
      
      LOOP
         FETCH C_CHGD2 INTO h_tsk_assignment_id, h_remaining, h_delay, h_res_id, h_res_type,
         h_percent_units, h_work_type, h_ta_start_DT, h_last_day, h_last_submitted,
         h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale;
         IF C_CHGD2%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("FD_** SC_STATE: %d, h_tsk_assignment_id: %s\n",SC_STATE,h_tsk_assignment_id);//trace_flush();
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         --trace_printf("h_tsk_assignment_id: %s, h_remaining: %d, h_delay: %d\n",h_tsk_assignment_id,h_remaining,h_delay);//trace_flush();
         --trace_printf("h_SD: %s, h_FD: %s\n",h_SD,h_FD);//trace_flush();
         
         If h_leveling = 0 Then
            h_tph_delete := -1;
            h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                     , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                     , h_billable);
            
            If (Coalesce(h_last_day, 0) > 0) AND (h_last_submitted IS NOT NULL) Then
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 282+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(283) : cplan_fd.sql(3098) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
         h_ta_finish_DT := h_finish_DT;
         
         h_temp_var := FC_FD_CTD_TDY(h_project_id, h_task_id, h_tsk_assignment_id, h_res_id, h_res_type, h_SD
                            , h_FD, h_ta_start_DT, h_ta_finish_DT, h_work_type, h_per_day
                            , h_percent_units, h_rec_user, i_bitflag, Coalesce(h_bitflag1, 0)
                            , h_leveling);
         
         If h_leveling = 0 Then
            BEGIN
               SELECT TO_DATE(SubStr(h_start_DT, 1, 10)), TO_DATE(SubStr(h_finish_DT, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
               INTO h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
               FROM TASK_ASSIGNMENTS
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 308+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(309) : cplan_fd.sql(3124) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_tph_delete := 0;
            h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                     , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                     , h_billable);
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_CHGD2 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 328+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(329) : cplan_fd.sql(3144) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      OPEN C_CHGD1;
      
      LOOP
         FETCH C_CHGD1 INTO h_tsk_assignment_id, h_remaining, h_delay, h_res_id, h_res_type,
         h_percent_units, h_bit_flag, h_contour, h_prj_assign_id, h_last_day, h_last_submitted,
         h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale;
         IF C_CHGD1%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("SD_** SC_STATE: %d\n",SC_STATE);//trace_flush();
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         --trace_printf("h_tsk_assignment_id: %s, h_remaining: %d, h_delay: %d\n",h_tsk_assignment_id,h_remaining,h_delay);//trace_flush();
         --trace_printf("h_SD: %s, h_FD: %s\n",h_SD,h_FD);//trace_flush();
         --trace_printf("*** h_res_type: %s, h_res_id: %s, h_start_DT: %d %s\n",h_res_type,h_res_id,h_start_DT_Ind,h_start_DT);//trace_flush();
         --trace_printf("h_project_id: %s, h_start_DT: %s\n",h_project_id,h_start_DT);//trace_flush();
         
         If Coalesce(h_remaining, 0) > 0 Then
            h_ta_finish_DT := h_finish_DT;
            If Coalesce(h_delay, 0) = 0 Then
               h_ta_start_DT := h_start_DT;
            Else
               If SubStr(h_res_type, 1, 1) = '1' Then
                  OPEN C_P_DELAY;
               Else
                  OPEN C_R_DELAY;
               End If;
               i001 := Coalesce(h_delay, 0);
               WHILE i001 > 0 LOOP
                  If SubStr(h_res_type, 1, 1) = '1' Then
                     FETCH C_P_DELAY INTO h_dayofyear;
                     IF C_P_DELAY%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  Else
                     FETCH C_R_DELAY INTO h_dayofyear;
                     IF C_R_DELAY%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  End If;
                  --trace_printf("*** SC_STATE: %d, h_dayofyear: %d %s\n",SC_STATE,h_dayofyear_Ind,h_dayofyear);//trace_flush();
                  
                  If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                     BEGIN
                        CLOSE C_CHGD1 ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 391+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(392) : cplan_fd.sql(3207) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If SubStr(h_res_type, 1, 1) = '1' Then
                        BEGIN
                           CLOSE C_P_DELAY ;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 405+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(406) : cplan_fd.sql(3221) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           CLOSE C_R_DELAY ;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 418+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(419) : cplan_fd.sql(3234) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHANGE_TA_DATES', 425+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: FC_CHANGE_TA_DATES(426) : cplan_fd.sql(3241) : ' || SYSTIMESTAMP);
                     END IF; 
                     RETURN PMO_CNST.RECORD_NOT_FOUND;
                  End If;
                                 i001 := i001 - 1;
               END LOOP;
               
               If SubStr(h_res_type, 1, 1) = '1' Then
                  BEGIN
                     CLOSE C_P_DELAY ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 441+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(442) : cplan_fd.sql(3257) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     CLOSE C_R_DELAY ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 454+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(455) : cplan_fd.sql(3270) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_start_DT))
                  INTO h_ta_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 470+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(471) : cplan_fd.sql(3286) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_bitflag1 := PMO_ORASTD.strncmp(h_ta_start_DT, h_SD, 10);
            -- assignment: new date, old date
            --trace_printf("**** h_delay: %d, h_bitflag1: %d, h_ta_start_DT: %d, %s\n", h_delay, h_bitflag1, h_ta_start_DT_Ind, h_ta_start_DT);
            --trace_printf("**** h_ta_start_DT: %s, h_start_DT: %s\n", h_ta_start_DT, h_start_DT);
            If (Coalesce(h_bitflag1, 0) < 0) OR (Coalesce(h_delay, 0) = 0) OR (PMO_ORASTD.strncmp(h_ta_start_DT
                  , h_start_DT, 10) > 0) Then
               If h_leveling = 0 Then
                  h_tph_delete := -1;
                  h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                     , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                     , h_billable);
                  
                  If (Coalesce(h_last_day, 0) > 0) AND (h_last_submitted IS NOT NULL) Then
                     BEGIN
                        UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN
                        WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 502+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(503) : cplan_fd.sql(3318) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
               
               If (h_task_type = PMO_CNST.SC_FD_EFFORT_DRIVEN) OR (h_task_type = PMO_CNST.SC_FIXED_DURATION) Then
                  If SubStr(h_bit_flag, 1, 1) = '2' Then
                     h_work_type := 2;
                  Else
                     h_work_type := 0;
                  End If;
                  
                  h_temp_var := FC_FD_CTD_TDY(h_project_id, h_task_id, h_tsk_assignment_id, h_res_id, h_res_type, h_SD
                            , h_FD, h_ta_start_DT, h_ta_finish_DT, h_work_type, h_per_day
                            , h_percent_units, h_rec_user, i_bitflag, Coalesce(h_bitflag1, 0)
                            , h_leveling);
                  
                  If h_leveling = 0 Then
                     BEGIN
                        SELECT TO_DATE(SubStr(h_start_DT, 1, 10)), TO_DATE(SubStr(h_finish_DT, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                        INTO h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
                        FROM TASK_ASSIGNMENTS
                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 533+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(534) : cplan_fd.sql(3349) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     h_tph_delete := 0;
                     h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                     , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                     , h_billable);
                  End If;
               Else
                  If SubStr(h_bit_flag, 1, 1) = '2' Then
                     h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 0);
                  ElsIf h_prj_assign_id IS NULL Then
                     h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 1);
                  Else
                     h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 2);
                  End If;
                  
                  h_temp_var := 2;
                  h_temp_var := FC_RESCHEDULE(h_tsk_assignment_id, h_ta_start_DT, h_remaining, h_temp_var, h_contour
                            , h_rec_user, 0, Coalesce(h_percent_units, 0));
                  
                  If Coalesce(h_temp_var, 0) <> 0 Then
                     BEGIN
                        CLOSE C_CHGD1 ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 565+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(566) : cplan_fd.sql(3381) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHANGE_TA_DATES', 571+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: FC_CHANGE_TA_DATES(572) : cplan_fd.sql(3387) : ' || SYSTIMESTAMP);
                     END IF; 
                     RETURN Coalesce(h_temp_var, 0);
                  End If;
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT))
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 589+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(590) : cplan_fd.sql(3405) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_CHGD1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 606+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(607) : cplan_fd.sql(3422) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHANGE_TA_DATES', 613+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CHANGE_TA_DATES(614) : cplan_fd.sql(3429) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CHANGE_TA_DATES;
ALTER FUNCTION FC_CHANGE_TA_DATES COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_FD_U_TDY ==
CREATE OR REPLACE FUNCTION FC_FD_U_TDY
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_ta_start_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_per_day PLS_INTEGER
   , sh_h_minutes PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_per_day PLS_INTEGER := sh_h_per_day;
   h_minutes PLS_INTEGER := sh_h_minutes;
   h_dayofyear DATE;
   h_first_day PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
-- Declare Cursors
      CURSOR C_FDT IS
         SELECT * FROM (SELECT COALESCE(AVAILABLE_MIN,480), COALESCE(BIT_SCHED,PMO_CNST.SC_BIT_SCHED8H)
         FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_project_id AND AVAILABLE_MIN = h_per_day
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FD_U_TDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FD_U_TDY'), '-- ' || SYSTIMESTAMP, ':r := FC_FD_U_TDY(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_ta_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_minutes), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_U_TDY', 33+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FD_U_TDY(34) : cplan_fd.sql(3473) : ' || SYSTIMESTAMP);
   END IF; 

   -- get first day datetime
   
   OPEN C_FDT;
   
   FETCH C_FDT INTO h_first_day, h_bitID;
   IF C_FDT%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   BEGIN
      CLOSE C_FDT ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_U_TDY', 55+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_U_TDY(56) : cplan_fd.sql(3495) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_first_day, 0) > Coalesce(h_minutes, 0) Then
      PMO_STPU.clearFinishDayStruct(Coalesce(h_minutes, 0), h_bitID);
   End If;
   
   BEGIN
      UPDATE TASK_DAY_OF_YEAR SET
      SCHED_MIN = h_minutes,
      BIT_SCHED = h_bitID
      WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND ACTUAL_FLAG = 0;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_U_TDY', 79+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_U_TDY(80) : cplan_fd.sql(3519) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_minutes: %d\n",SC_STATE,h_minutes);//trace_flush();
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_U_TDY', 86+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FD_U_TDY(87) : cplan_fd.sql(3526) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FD_U_TDY;
ALTER FUNCTION FC_FD_U_TDY COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_FD_EFFORT_DRIVEN ==
CREATE OR REPLACE FUNCTION FC_FD_EFFORT_DRIVEN
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_res_type IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_remaining PLS_INTEGER
   , sh_h_work_todate PLS_INTEGER
   , sh_h_cnt PLS_INTEGER
   , sh_h_per_day PLS_INTEGER
   , sh_add_remove PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_remaining PLS_INTEGER := sh_h_remaining;
   h_work_todate PLS_INTEGER := sh_h_work_todate;
   h_cnt PLS_INTEGER := sh_h_cnt;
   h_per_day PLS_INTEGER := sh_h_per_day;
   add_remove PLS_INTEGER := Coalesce(sh_add_remove, 0);
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_ta_start_DT TIMESTAMP;
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_available PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_new_task_rem PLS_INTEGER := 0;
   h_old_res_rem PLS_INTEGER := 0;
   h_new_res_rem PLS_INTEGER := 0;
   h_sum_res_rem PLS_INTEGER := 0;
   h_sum_units PLS_INTEGER := 0;
   h_tsk_units PLS_INTEGER := 0;
   h_new_units PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_etc_diff PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_total_h NUMBER := 0;
   ratio_rem NUMBER;
-- Declare Cursors
      CURSOR C_NEW_REM IS
         SELECT TSK_ASSIGNMENT_ID, COALESCE(RESUME_DT,UNRESOLVED_START), OLD_REMAINING, DURATION
         FROM PMO_TMP_FIXDUR WHERE RESOURCE_ID <> h_res_id;

      CURSOR C_DEL_RES IS
         SELECT TSK_ASSIGNMENT_ID, COALESCE(RESUME_DT,UNRESOLVED_START), OLD_REMAINING
         FROM PMO_TMP_FIXDUR
         WHERE RESOURCE_ID <> h_res_id;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FD_EFFORT_DRIVEN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FD_EFFORT_DRIVEN'), '-- ' || SYSTIMESTAMP, ':r := FC_FD_EFFORT_DRIVEN(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_res_type || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_remaining), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work_todate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_cnt), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_add_remove), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_EFFORT_DRIVEN', 69+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FD_EFFORT_DRIVEN(70) : cplan_fd.sql(3606) : ' || SYSTIMESTAMP);
   END IF; 

   If (Coalesce(h_remaining, 0) = 0) OR (Coalesce(h_cnt, 0) = 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_EFFORT_DRIVEN', 74+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_FD_EFFORT_DRIVEN(75) : cplan_fd.sql(3611) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.ASSN_FIXED_DURATION;
   End If;
   --trace_printf("add_remove: %d, h_res_id: %s, h_res_type: %s, h_remaining: %d, h_work_todate: %d\n",add_remove,h_res_id,h_res_type,h_remaining,h_work_todate);//trace_flush();
   --trace_printf("h_start_DT: %d %s, h_finish_DT: %d %s\n",h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);//trace_flush();
   
   h_tph_delete := -1;
   h_temp_var := FC_TPHASE_FD_SCHED(h_project_id, h_task_id, h_tph_delete);
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_FIXDUR';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_FIXDUR
      SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, RESOURCE_TYPE,
      UNRESOLVED_START, PMO_ORASTD.DB2TIMESTAMP(LAST_SUBMITTED,'00:00') + 1     ,
      WORK_TODATE, ETC, CASE WHEN UNITS_MSP = 0 THEN 10000 ELSE UNITS_MSP END, 0, 0, 0
      FROM TASK_ASSIGNMENTS
      WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id
      AND (ACTUAL_FINISH IS NuLL OR RESOURCE_ID = h_res_id);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 106+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(107) : cplan_fd.sql(3643) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_project_id: %s, h_task_id: %s\n",SC_STATE,h_project_id,h_task_id);//trace_flush();
   
   --  if (SC_NOSUCCESS) return SC_STATE;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         UPDATE PMO_TMP_FIXDUR t SET t.DURATION = (
         SELECT COUNT(tdy.DAY_YEAR)
         FROM TASK_DAY_OF_YEAR tdy
         WHERE tdy.TSK_ASSIGNMENT_ID = t.TSK_ASSIGNMENT_ID AND tdy.ACTUAL_FLAG = 0);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 130+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(131) : cplan_fd.sql(3667) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_tsk_units := 0;
   Else
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(MIN(ACTUAL_FINISH), 1, 10)),'00:00') + 1      INTO h_ta_start_DT
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 147+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(148) : cplan_fd.sql(3684) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_ta_start_DT: %d %s\n",SC_STATE,h_ta_start_DT_Ind,h_ta_start_DT);//trace_flush();
      
      h_tsk_units := 10000;
   End If;
   
   h_new_task_rem := 0;
   h_sum_res_rem := 0;
   h_sum_units := 0;
   h_minutes := 0;
   h_available := 0;
   h_count := 0;
   h_etc_diff := 0;
   h_temp_var := 0;
   
   If add_remove = 0 Then
      If Coalesce(h_tsk_units, 0) = 0 Then
         BEGIN
            SELECT MIN(RESUME_DT), SUM(OLD_UNITS) INTO h_ta_start_DT, h_tsk_units
            FROM PMO_TMP_FIXDUR;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 176+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(177) : cplan_fd.sql(3713) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_tsk_units: %d, h_ta_start_DT: %d %s\n",SC_STATE,h_tsk_units,h_ta_start_DT_Ind,h_ta_start_DT);//trace_flush();
      End If;
      
      If h_ta_start_DT IS NOT NULL Then
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_ta_start_DT, 1, 10)),PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_start_DT
            FROM PMO_TMP_CDY
            WHERE DAY_YEAR = TO_DATE(SubStr(h_ta_start_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 194+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(195) : cplan_fd.sql(3731) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_work_todate, 0) = 0 Then
         BEGIN
            SELECT SUM(DURATION) INTO h_duration
            FROM PMO_TMP_FIXDUR;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 210+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(211) : cplan_fd.sql(3747) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            BEGIN
               SELECT COUNT(*) INTO h_count
               FROM PMO_TMP_CDY
               WHERE DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 226+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(227) : cplan_fd.sql(3763) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT COUNT(r.DAY_YEAR) INTO h_count
               FROM PMO_TMP_CDY p JOIN CALENDAR_DAY_OF_YEAR r ON p.DAY_YEAR = r.DAY_YEAR
               WHERE r.ELEMENT_ID = h_res_id AND r.DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 241+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(242) : cplan_fd.sql(3778) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         h_available := Coalesce(h_duration, 0) + Coalesce(h_count, 0);
      Else
         If SubStr(h_res_type, 1, 1) = '1' Then
            BEGIN
               SELECT SUM(AVAILABLE_MIN) INTO h_available
               FROM PMO_TMP_CDY
               WHERE DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 260+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(261) : cplan_fd.sql(3797) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT SUM(PMO_STPU.TOTAL_DAY_2(r.BIT_SCHED,p.BIT_SCHED)) INTO h_available
               FROM PMO_TMP_CDY p JOIN CALENDAR_DAY_OF_YEAR r ON p.DAY_YEAR = r.DAY_YEAR
               WHERE r.ELEMENT_ID = h_res_id AND r.DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 275+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(276) : cplan_fd.sql(3812) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      --trace_printf("SC_STATE: %d, h_start_DT: %d %s, h_finish_DT: %d %s\n",SC_STATE,h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);//trace_flush();
      --trace_printf("h_res_id: %s, h_available: %d\n",h_res_id,h_available);//trace_flush();
      
      If Coalesce(h_available, 0) > 0 Then
         BEGIN
            INSERT INTO PMO_TMP_FIXDUR
            VALUES (NULL, h_res_id, h_res_type, NULL, h_start_DT, 0, h_available, 0, 0, 0, 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 298+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(299) : cplan_fd.sql(3835) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_temp_var := 0;
         h_total_h := 0.00;
         h_new_res_rem := 0;
         h_new_units := 0;
         ratio_rem := 0.00;
         
         --trace_printf("h_work_todate: %d, h_remaining: %d\n",h_work_todate,h_remaining);//trace_flush();
         If Coalesce(h_work_todate, 0) = 0 Then
            h_minutes := Trunc((Coalesce(h_remaining, 0) / Coalesce(h_available, 0)) + 0.4);
            h_sum_res_rem := Coalesce(h_minutes, 0) * Coalesce(h_duration, 0);
            h_new_units := Trunc((Coalesce(h_minutes, 0) / Coalesce(h_per_day, 0)) * 10000.00);
            --trace_printf("1.1*** h_minutes: %d, h_new_units: %d, h_sum_res_rem: %d\n",h_minutes,h_new_units,h_sum_res_rem);//trace_flush();
         Else
            h_new_task_rem := Coalesce(h_remaining, 0) + Coalesce(h_available, 0);
            ratio_rem := (Coalesce(h_remaining, 0) / Coalesce(h_new_task_rem, 0));
            --trace_printf("1.2*** ratio_rem: %.4f, h_new_task_rem: %d\n",ratio_rem,h_new_task_rem);//trace_flush();
         End If;
         
         OPEN C_NEW_REM;
         
         LOOP
            FETCH C_NEW_REM INTO h_tsk_assignment_id, h_ta_start_DT, h_old_res_rem, h_duration;
            IF C_NEW_REM%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            --trace_printf("h_tsk_assignment_id: %s, h_old_res_rem: %d, h_duration: %d\n",h_tsk_assignment_id,h_old_res_rem,h_duration);//trace_flush();
            
            If Coalesce(h_work_todate, 0) > 0 Then
               h_new_res_rem := Trunc(Coalesce(h_old_res_rem, 0) * ratio_rem);
               
               h_new_res_rem := Coalesce(h_new_res_rem, 0) + Coalesce(h_etc_diff, 0);
               
               h_minutes := Trunc((Coalesce(h_new_res_rem, 0) / Coalesce(h_duration, 0)) + 0.4);
               h_temp_var := Coalesce(h_minutes, 0) MOD 5;
               If Coalesce(h_temp_var, 0) <> 0 Then
                  If Coalesce(h_temp_var, 0) < 3 Then
                     h_minutes := Coalesce(h_minutes, 0) + Coalesce(h_temp_var, 0);
                  Else
                     h_minutes := Coalesce(h_minutes, 0) - 5 - Coalesce(h_temp_var, 0);
                  End If;
               End If;
               
               h_new_units := Trunc((Coalesce(h_minutes, 0) / Coalesce(h_per_day, 0)) * 10000.00);
               h_temp_var := Coalesce(h_minutes, 0) * Coalesce(h_duration, 0);
               h_new_res_rem := Coalesce(h_temp_var, 0);
               h_sum_res_rem := Coalesce(h_sum_res_rem, 0) + Coalesce(h_temp_var, 0);
               h_etc_diff := Coalesce(h_new_res_rem, 0) - Coalesce(h_temp_var, 0);
               --trace_printf("h_new_units: %d, h_new_res_rem: %d, h_etc_diff: %d\n",h_new_units,h_new_res_rem,h_etc_diff);//trace_flush();
            Else
               h_new_res_rem := Coalesce(h_minutes, 0) * Coalesce(h_duration, 0);
            End If;
            
            h_sum_units := Coalesce(h_sum_units, 0) + Coalesce(h_new_units, 0);
            
            h_temp_var := FC_FD_U_TDY(h_project_id, h_tsk_assignment_id, h_ta_start_DT, h_per_day, h_minutes);
            
            -- fix assign string
            h_temp_var := Coalesce(h_new_units, 0) + PMO_CNST.SC_BIT_30;
            h_temp_var := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, Coalesce(h_per_day, 0)
                              , Coalesce(h_new_res_rem, 0));
         END LOOP;
         
         BEGIN
            CLOSE C_NEW_REM ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 378+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(379) : cplan_fd.sql(3915) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("h_new_units: %d, h_tsk_units: %d, h_sum_units: %d\n",h_new_units,h_tsk_units,h_sum_units);//trace_flush();
         --trace_printf("h_etc_diff: %d, h_sum_res_rem: %d\n",h_etc_diff,h_sum_res_rem);//trace_flush();
         
         If Coalesce(h_work_todate, 0) > 0 Then
            h_new_units := Coalesce(h_tsk_units, 0) - Coalesce(h_sum_units, 0);
         End If;
         
         h_new_res_rem := Coalesce(h_remaining, 0) - Coalesce(h_sum_res_rem, 0);
         
         BEGIN
            UPDATE PMO_TMP_FIXDUR SET
            NEW_REMAINING = h_new_res_rem,
            NEW_UNITS = h_new_units
            WHERE RESOURCE_ID = h_res_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 406+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(407) : cplan_fd.sql(3943) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("***END_0: SC_STATE: %d, h_new_res_rem: %d, h_new_units: %d\n",SC_STATE,h_new_res_rem,h_new_units);//trace_flush();
      End If;
   Else
      BEGIN
         SELECT SUM(OLD_UNITS) INTO h_tsk_units
         FROM PMO_TMP_FIXDUR;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 422+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(423) : cplan_fd.sql(3959) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_tsk_units: %d\n",SC_STATE,h_tsk_units);//trace_flush();
      
      BEGIN
         SELECT OLD_REMAINING, WORK_TODATE INTO h_temp_var, h_work_todate
         FROM PMO_TMP_FIXDUR
         WHERE RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 438+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(439) : cplan_fd.sql(3975) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_res_id: %s, h_temp_var: %d, h_available: %d\n",SC_STATE,h_res_id,h_temp_var,h_available);//trace_flush();
      
      h_new_task_rem := Coalesce(h_remaining, 0) - Coalesce(h_temp_var, 0);
      h_available := Coalesce(h_work_todate, 0) + Coalesce(h_temp_var, 0);
      --trace_printf("h_new_task_rem: %d, h_remaining: %d\n",h_new_task_rem,h_remaining);//trace_flush();
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_EFFORT_DRIVEN', 450+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_FD_EFFORT_DRIVEN(451) : cplan_fd.sql(3987) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_DB2M.SC_STATE;
      End If;
      
      OPEN C_DEL_RES;
      
      LOOP
         FETCH C_DEL_RES INTO h_tsk_assignment_id, h_ta_start_DT, h_old_res_rem;
         IF C_DEL_RES%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         --trace_printf("h_tsk_assignment_id: %s, h_old_res_rem: %d\n",h_tsk_assignment_id,h_old_res_rem);//trace_flush();
         
         BEGIN
            SELECT COUNT(*) INTO h_duration
            FROM TASK_DAY_OF_YEAR
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND ACTUAL_FLAG = 0;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 479+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(480) : cplan_fd.sql(4016) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_duration: %d, h_etc_diff: %d\n",SC_STATE,h_duration,h_etc_diff);//trace_flush();
         
         If Coalesce(h_cnt, 0) = 2 Then
            h_new_res_rem := Coalesce(h_remaining, 0) + Coalesce(h_work_todate, 0) - Coalesce(h_sum_res_rem, 0);
            
            --h_new_res_rem = h_remaining - h_sum_res_rem;
            --h_new_units       = h_tsk_units - h_sum_units;
            h_minutes := Trunc((Coalesce(h_new_res_rem, 0) / Coalesce(h_duration, 0)) + 0.4);
         Else
            h_total_h := Coalesce(h_old_res_rem, 0) / Coalesce(h_new_task_rem, 0);
            h_new_res_rem := Trunc(Coalesce(h_old_res_rem, 0) + Coalesce(h_etc_diff, 0) + (Coalesce(
                                h_available, 0) * Coalesce(h_total_h, 0)));
            h_minutes := Trunc((Coalesce(h_new_res_rem, 0) / Coalesce(h_duration, 0)) + 0.4);
            h_temp_var := Coalesce(h_minutes, 0) MOD 5;
            If Coalesce(h_temp_var, 0) <> 0 Then
               If Coalesce(h_temp_var, 0) < 3 Then
                  h_minutes := Coalesce(h_minutes, 0) + Coalesce(h_temp_var, 0);
               Else
                  h_minutes := Coalesce(h_minutes, 0) - 5 - Coalesce(h_temp_var, 0);
               End If;
            End If;
         End If;
         
         h_new_units := Trunc((Coalesce(h_minutes, 0) / Coalesce(h_per_day, 0)) * 10000.00);
         --trace_printf("h_total_h: %.4f, h_new_units: %d, h_new_res_rem: %d\n",h_total_h,h_new_units,h_new_res_rem);//trace_flush();
         --trace_printf("h_cnt: %d, h_sum_res_rem: %d, h_sum_units: %d\n",h_cnt,h_sum_res_rem,h_sum_units);//trace_flush();
         
         h_temp_var := FC_FD_U_TDY(h_project_id, h_tsk_assignment_id, h_ta_start_DT, h_per_day, h_minutes);
         
         If Coalesce(h_cnt, 0) > 2 Then
            h_temp_var := Coalesce(h_minutes, 0) * Coalesce(h_duration, 0);
            h_etc_diff := Coalesce(h_new_res_rem, 0) - Coalesce(h_temp_var, 0);
            
            h_new_res_rem := Coalesce(h_temp_var, 0);
            h_sum_res_rem := Coalesce(h_sum_res_rem, 0) + Coalesce(h_temp_var, 0);
            h_sum_units := Coalesce(h_sum_units, 0) + Coalesce(h_new_units, 0);
            h_cnt := Coalesce(h_cnt, 0) - 1;
         End If;
         
         -- fix assign string
         h_temp_var := Coalesce(h_new_units, 0) + PMO_CNST.SC_BIT_30;
         h_temp_var := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, Coalesce(h_per_day, 0)
                              , Coalesce(h_new_res_rem, 0));
      END LOOP;
      
      BEGIN
         CLOSE C_DEL_RES ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_EFFORT_DRIVEN', 536+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_EFFORT_DRIVEN(537) : cplan_fd.sql(4073) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("***END_1: SC_STATE: %d, h_new_res_rem: %d, h_new_units: %d\n",SC_STATE,h_new_res_rem,h_new_units);//trace_flush();
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_EFFORT_DRIVEN', 544+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FD_EFFORT_DRIVEN(545) : cplan_fd.sql(4081) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FD_EFFORT_DRIVEN;
ALTER FUNCTION FC_FD_EFFORT_DRIVEN COMPILE DEBUG;

/*******************************************************************************
********************************************************************************/

Prompt == FUNCTION FC_TPHASE_SCHED_ASSIGN ==
CREATE OR REPLACE FUNCTION FC_TPHASE_SCHED_ASSIGN
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_reg_cost NUMBER
   , sh_h_reg_sale NUMBER
   , sh_h_update PLS_INTEGER
   , sh_h_capital_expense PLS_INTEGER
   , sh_h_billable PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_reg_cost NUMBER := sh_h_reg_cost;
   h_reg_sale NUMBER := sh_h_reg_sale;
   h_update PLS_INTEGER := sh_h_update;
   h_capital_expense PLS_INTEGER := sh_h_capital_expense;
   h_billable PLS_INTEGER := sh_h_billable;
   h_cc_id VARCHAR2(32);
   h_FD DATE;
   h_SD DATE;
   h_pa_timephase VARCHAR2(1);
   h_type PLS_INTEGER := 0;
   h_cur_year PLS_INTEGER := 0;
   h_end_year PLS_INTEGER := 0;
   h_start_month PLS_INTEGER := 0;
   h_end_month PLS_INTEGER := 0;
   h_budget_type PLS_INTEGER := 0;
   h_rollup_bits PLS_INTEGER := 0;
   h_01 NUMBER := 0;
   h_02 NUMBER := 0;
   h_03 NUMBER := 0;
   h_04 NUMBER := 0;
   h_05 NUMBER := 0;
   h_06 NUMBER := 0;
   h_07 NUMBER := 0;
   h_08 NUMBER := 0;
   h_09 NUMBER := 0;
   h_10 NUMBER := 0;
   h_11 NUMBER := 0;
   h_12 NUMBER := 0;
   h_total_h NUMBER := 0;
   h_d_total NUMBER := 0;
   h_d_01 NUMBER := 0;
   h_d_02 NUMBER := 0;
   h_d_03 NUMBER := 0;
   h_d_04 NUMBER := 0;
   h_d_05 NUMBER := 0;
   h_d_06 NUMBER := 0;
   h_d_07 NUMBER := 0;
   h_d_08 NUMBER := 0;
   h_d_09 NUMBER := 0;
   h_d_10 NUMBER := 0;
   h_d_11 NUMBER := 0;
   h_d_12 NUMBER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   stmt VARCHAR2(7999);
   l_actual_flag PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      -- Dynamic Cursor Declaration
      CTPH01 PMO_DB2M.REF_CURSOR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_TPHASE_SCHED_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_TPHASE_SCHED_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := FC_TPHASE_SCHED_ASSIGN(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_reg_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_reg_sale), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_update), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_capital_expense), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_billable), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_TPHASE_SCHED_ASSIGN', 79+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_TPHASE_SCHED_ASSIGN(80) : cplan_fd.sql(4172) : ' || SYSTIMESTAMP);
   END IF; 

   h_SD := SubStr(h_start_DT, 1, 10);
   h_FD := SubStr(h_finish_DT, 1, 10);
   --trace_printf("h_update: %d, h_billable: %d, h_capital_expense: %d, h_reg_cost: %.2f, h_reg_sale: %.2f, h_SD: %s, h_FD: %s\n",h_update,h_billable,h_capital_expense,h_reg_cost,h_reg_sale,h_SD,h_FD);
   --trace_printf("h_project_id: %d:%s, h_task_id: %d:%s\n",h_project_id_Ind,h_project_id,h_task_id_Ind,h_task_id);
   
   If (Coalesce(Length(h_start_DT), 0) = 0) OR (h_start_DT IS NULL) 
         OR (Coalesce(Length(h_finish_DT), 0) = 0) OR (h_finish_DT IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_TPHASE_SCHED_ASSIGN', 90+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_TPHASE_SCHED_ASSIGN(91) : cplan_fd.sql(4183) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(h_billable, 0) >= PMO_CNST.SC_BIT_30 Then
      l_actual_flag := 1;
      h_billable := (BITAND(Coalesce(h_billable, 0), PMO_CNST.SC_BIT_NOT_30));
      -- remove actual flag
   End If;
   
   h_rollup_bits := PMO_CNST.SC_SCHED_DELTA_BIT;
   If PMO_ORASTD.strcmp(h_project_id, h_task_id) = 0 Then
      BEGIN
         SELECT TIMEPHASE_PROJECT_ASSIGNMENT INTO h_pa_timephase FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 111+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(112) : cplan_fd.sql(4204) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("1_* h_pa_timephase: %s\n", h_pa_timephase);
      
      If SubStr(h_pa_timephase, 1, 1) = 'Y' Then
         h_budget_type := PMO_CNST.SC_ESTIMATED_BUDGET;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_TPHASE_SCHED_ASSIGN', 121+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_TPHASE_SCHED_ASSIGN(122) : cplan_fd.sql(4214) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
   ElsIf l_actual_flag = 1 Then
      h_budget_type := PMO_CNST.SC_ACTUAL_BUDGET;
      --      h_rollup_bits = h_rollup_bits | SC_ACTUAL_DELTA_BIT;
      h_rollup_bits := PMO_CNST.SC_ACTUAL_DELTA_BIT;
   Else
      h_budget_type := PMO_CNST.SC_SCHED_BUDGET;
   End If;
   
   h_temp_sc_state := FC_FISCAL_DATE(h_SD, h_cur_year, h_start_month);
   h_temp_sc_state := FC_FISCAL_DATE(h_FD, h_end_year, h_end_month);
   
   --trace_printf("h_tsk_assignment_id: %s, l_actual_flag: %d, h_cur_year: %d, h_end_year: %d, h_budget_type: %d\n", h_tsk_assignment_id,l_actual_flag, h_cur_year, h_end_year, h_budget_type);
   
   stmt := '';
   If Coalesce(h_budget_type, 0) = PMO_CNST.SC_ESTIMATED_BUDGET Then
      stmt := 'SELECT FY.FISCAL_YEAR, '
             || 'SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 1)))), '
             || 'SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 2)))), '
             || 'SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 3)))), '
             || 'SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 4)))), '
             || 'SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 5)))), '
             || 'SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 6)))), '
             || 'SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 7)))), '
             || 'SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 8)))), '
             || 'SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 9)))), '
             || 'SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 10)))),'
             || 'SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 11)))),'
             || 'SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 12)))) '
             || 'FROM TASK_DAY_OF_YEAR tdy JOIN FISCAL_CALENDAR fy '
             || 'ON tdy.DAY_YEAR BETWEEN fy.START_DATE AND fy.FINISH_DATE '
             || 'WHERE tdy.PROJECT_ID = ''' || h_project_id || ''' AND tdy.PARENT_ID = ''' 
                  || h_tsk_assignment_id || ''' '
             || 'GROUP BY FY.FISCAL_YEAR ';
   ElsIf l_actual_flag = 1 Then
      stmt := 'SELECT FY.FISCAL_YEAR, '
             || 'SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 1)))), '
             || 'SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 2)))), '
             || 'SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 3)))), '
             || 'SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 4)))), '
             || 'SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 5)))), '
             || 'SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 6)))), '
             || 'SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 7)))), '
             || 'SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 8)))), '
             || 'SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 9)))), '
             || 'SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 10)))),'
             || 'SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 11)))),'
             || 'SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 12)))) '
             || 'FROM TASK_DAY_OF_YEAR tdy JOIN FISCAL_CALENDAR fy '
             || 'ON tdy.DAY_YEAR BETWEEN fy.START_DATE AND fy.FINISH_DATE '
             || 'WHERE tdy.PROJECT_ID = ''' || h_project_id 
                  || ''' AND tdy.ACTUAL_FLAG = 1 AND tdy.TSK_ASSIGNMENT_ID = ''' 
                  || h_tsk_assignment_id || ''' '
             || 'GROUP BY FY.FISCAL_YEAR ';
   Else
      stmt := 'SELECT FY.FISCAL_YEAR, '
             || 'SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 1)))), '
             || 'SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 2)))), '
             || 'SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 3)))), '
             || 'SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 4)))),  '
             || 'SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 5)))),  '
             || 'SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 6)))),  '
             || 'SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 7)))),  '
             || 'SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 8)))),  '
             || 'SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 9)))),  '
             || 'SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 10)))),'
             || 'SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 11)))),'
             || 'SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 12))))  '
             || 'FROM TASK_DAY_OF_YEAR tdy JOIN FISCAL_CALENDAR fy '
             || 'ON tdy.DAY_YEAR BETWEEN fy.START_DATE AND fy.FINISH_DATE '
             || 'WHERE tdy.PROJECT_ID = ''' || h_project_id || ''' AND tdy.TSK_ASSIGNMENT_ID = ''' 
                  || h_tsk_assignment_id || ''' '
             || 'GROUP BY FY.FISCAL_YEAR ';
   End If;
   --trace_printf("%s\n",(char *) stmt); //trace_flush();
   
   OPEN CTPH01 FOR stmt;
   
   LOOP
      FETCH CTPH01 INTO h_cur_year, h_01, h_02, h_03, h_04, h_05, h_06, h_07, h_08, h_09, h_10, h_11, h_12;
      IF CTPH01%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("FETCH: SC_STATE: %d, h_cur_year: %d, h_01: %d:%.2f\n", SC_STATE,h_cur_year,h_01_Ind,h_01);
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS OR (h_01 IS NULL) Then
         EXIT;
      End If;
      
      If Coalesce(h_update, 0) < 0 Then
         h_01 := -(Coalesce(h_01, 0) / 60.00);
         h_02 := -(Coalesce(h_02, 0) / 60.00);
         h_03 := -(Coalesce(h_03, 0) / 60.00);
         h_04 := -(Coalesce(h_04, 0) / 60.00);
         h_05 := -(Coalesce(h_05, 0) / 60.00);
         h_06 := -(Coalesce(h_06, 0) / 60.00);
         h_07 := -(Coalesce(h_07, 0) / 60.00);
         h_08 := -(Coalesce(h_08, 0) / 60.00);
         h_09 := -(Coalesce(h_09, 0) / 60.00);
         h_10 := -(Coalesce(h_10, 0) / 60.00);
         h_11 := -(Coalesce(h_11, 0) / 60.00);
         h_12 := -(Coalesce(h_12, 0) / 60.00);
      Else
         h_01 := (Coalesce(h_01, 0) / 60.00);
         h_02 := (Coalesce(h_02, 0) / 60.00);
         h_03 := (Coalesce(h_03, 0) / 60.00);
         h_04 := (Coalesce(h_04, 0) / 60.00);
         h_05 := (Coalesce(h_05, 0) / 60.00);
         h_06 := (Coalesce(h_06, 0) / 60.00);
         h_07 := (Coalesce(h_07, 0) / 60.00);
         h_08 := (Coalesce(h_08, 0) / 60.00);
         h_09 := (Coalesce(h_09, 0) / 60.00);
         h_10 := (Coalesce(h_10, 0) / 60.00);
         h_11 := (Coalesce(h_11, 0) / 60.00);
         h_12 := (Coalesce(h_12, 0) / 60.00);
      End If;
      
      h_total_h := Coalesce(h_01, 0) + Coalesce(h_02, 0) + Coalesce(h_03, 0) + Coalesce(h_04, 0) 
                      + Coalesce(h_05, 0) + Coalesce(h_06, 0) + Coalesce(h_07, 0) + Coalesce(
                      h_08, 0) + Coalesce(h_09, 0) + Coalesce(h_10, 0) + Coalesce(h_11, 0) 
                      + Coalesce(h_12, 0);
      --trace_printf("h_total_h: %.2f, h_01: %.2f, h_02: %.2f, h_03: %.2f, h_04: %.2f, h_05: %.2f, h_06: %.2f, h_07: %.2f, h_08: %.2f, h_09: %.2f, h_10: %.2f, h_11: %.2f, h_12: %.2f\n", h_total_h,h_01,h_02,h_03,h_04,h_05,h_06,h_07,h_08,h_09,h_10,h_12,h_12);
      
      If Coalesce(h_total_h, 0) <> 0.0 Then
         If Coalesce(h_capital_expense, 0) = 0 Then
            h_cc_id := 'EXPENSE_HOURS';
         Else
            h_cc_id := 'CAPITAL_HOURS';
         End If;
         
         BEGIN
            INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
            MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
            MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
            VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_total_h,
            h_01, h_02, h_03, h_04, h_05, h_06,
            h_07, h_08, h_09, h_10, h_11, h_12);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 273+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(274) : cplan_fd.sql(4366) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("EXPENSE_CAPITAL_HOURS: SC_STATE: %d, h_cc_id: %s, h_cur_year: %d, h_total_h: %.2f\n", SC_STATE,h_cc_id,h_cur_year,h_total_h);
         
         If l_actual_flag = 1 Then
            BEGIN
               INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
               MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
               MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
               VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, -1 * h_total_h,
               -1 * h_01, -1 * h_02, -1 * h_03, -1 * h_04, -1 * h_05, -1 * h_06,
               -1 * h_07, -1 * h_08, -1 * h_09, -1 * h_10, -1 * h_11, -1 * h_12);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 297+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(298) : cplan_fd.sql(4390) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         BEGIN
            UPDATE BUDGETS
            SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS,h_rollup_bits)
            WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
            AND CHARGECODE_ID = h_cc_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 318+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(319) : cplan_fd.sql(4411) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_reg_cost, 0) <> 0.00 Then
            If Coalesce(h_capital_expense, 0) = 0 Then
               h_cc_id := 'EXPENSE_LABOR';
            Else
               h_cc_id := 'CAPITAL_LABOR';
            End If;
            
            h_d_01 := Coalesce(h_01, 0) * Coalesce(h_reg_cost, 0);
            h_d_02 := Coalesce(h_02, 0) * Coalesce(h_reg_cost, 0);
            h_d_03 := Coalesce(h_03, 0) * Coalesce(h_reg_cost, 0);
            h_d_04 := Coalesce(h_04, 0) * Coalesce(h_reg_cost, 0);
            h_d_05 := Coalesce(h_05, 0) * Coalesce(h_reg_cost, 0);
            h_d_06 := Coalesce(h_06, 0) * Coalesce(h_reg_cost, 0);
            h_d_07 := Coalesce(h_07, 0) * Coalesce(h_reg_cost, 0);
            h_d_08 := Coalesce(h_08, 0) * Coalesce(h_reg_cost, 0);
            h_d_09 := Coalesce(h_09, 0) * Coalesce(h_reg_cost, 0);
            h_d_10 := Coalesce(h_10, 0) * Coalesce(h_reg_cost, 0);
            h_d_11 := Coalesce(h_11, 0) * Coalesce(h_reg_cost, 0);
            h_d_12 := Coalesce(h_12, 0) * Coalesce(h_reg_cost, 0);
            
            h_d_total := Coalesce(h_d_01, 0) + Coalesce(h_d_02, 0) + Coalesce(h_d_03, 0) + Coalesce(h_d_04, 0) 
                      + Coalesce(h_d_05, 0) + Coalesce(h_d_06, 0) + Coalesce(h_d_07, 0) + Coalesce(
                      h_d_08, 0) + Coalesce(h_d_09, 0) + Coalesce(h_d_10, 0) + Coalesce(h_d_11, 0) 
                      + Coalesce(h_d_12, 0);
            
            BEGIN
               INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
               MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
               MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
               VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_d_total,
               h_d_01, h_d_02, h_d_03, h_d_04, h_d_05, h_d_06,
               h_d_07, h_d_08, h_d_09, h_d_10, h_d_11, h_d_12);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 365+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(366) : cplan_fd.sql(4458) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("LABOR_COST: SC_STATE: %d, h_cur_year: %d, h_d_total: %.2f\n", SC_STATE,h_cur_year,h_d_total);
            
            If l_actual_flag = 1 Then
               BEGIN
                  INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                  MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                  MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                  VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, -1 * h_d_total,
                  -1 * h_d_01, -1 * h_d_02, -1 * h_d_03, -1 * h_d_04, -1 * h_d_05, -1 * h_d_06,
                  -1 * h_d_07, -1 * h_d_08, -1 * h_d_09, -1 * h_d_10, -1 * h_d_11, -1 * h_d_12);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 389+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(390) : cplan_fd.sql(4482) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               UPDATE BUDGETS
               SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS,h_rollup_bits)
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
               AND CHARGECODE_ID = h_cc_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 410+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(411) : cplan_fd.sql(4503) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If Coalesce(h_billable, 0) = 1 Then
            h_cc_id := 'BENEFIT_HOURS';
            
            BEGIN
               INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
               MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
               MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
               VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_total_h,
               h_01, h_02, h_03, h_04, h_05, h_06,
               h_07, h_08, h_09, h_10, h_11, h_12);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 436+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(437) : cplan_fd.sql(4529) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("BILLABLE_HOURS: SC_STATE: %d, h_cur_year: %d, h_total_h: %.2f\n", SC_STATE,h_cur_year,h_total_h);
            
            If l_actual_flag = 1 Then
               BEGIN
                  INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                  MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                  MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                  VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, -1 * h_total_h,
                  -1 * h_01, -1 * h_02, -1 * h_03, -1 * h_04, -1 * h_05, -1 * h_06,
                  -1 * h_07, -1 * h_08, -1 * h_09, -1 * h_10, -1 * h_11, -1 * h_12);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 460+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(461) : cplan_fd.sql(4553) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               UPDATE BUDGETS
               SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS,h_rollup_bits)
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
               AND CHARGECODE_ID = h_cc_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 481+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(482) : cplan_fd.sql(4574) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_reg_sale, 0) <> 0.00 Then
               h_d_01 := Coalesce(h_01, 0) * Coalesce(h_reg_sale, 0);
               h_d_02 := Coalesce(h_02, 0) * Coalesce(h_reg_sale, 0);
               h_d_03 := Coalesce(h_03, 0) * Coalesce(h_reg_sale, 0);
               h_d_04 := Coalesce(h_04, 0) * Coalesce(h_reg_sale, 0);
               h_d_05 := Coalesce(h_05, 0) * Coalesce(h_reg_sale, 0);
               h_d_06 := Coalesce(h_06, 0) * Coalesce(h_reg_sale, 0);
               h_d_07 := Coalesce(h_07, 0) * Coalesce(h_reg_sale, 0);
               h_d_08 := Coalesce(h_08, 0) * Coalesce(h_reg_sale, 0);
               h_d_09 := Coalesce(h_09, 0) * Coalesce(h_reg_sale, 0);
               h_d_10 := Coalesce(h_10, 0) * Coalesce(h_reg_sale, 0);
               h_d_11 := Coalesce(h_11, 0) * Coalesce(h_reg_sale, 0);
               h_d_12 := Coalesce(h_12, 0) * Coalesce(h_reg_sale, 0);
               
               h_d_total := Coalesce(h_d_01, 0) + Coalesce(h_d_02, 0) + Coalesce(h_d_03, 0) + Coalesce(h_d_04, 0) 
                      + Coalesce(h_d_05, 0) + Coalesce(h_d_06, 0) + Coalesce(h_d_07, 0) + Coalesce(
                      h_d_08, 0) + Coalesce(h_d_09, 0) + Coalesce(h_d_10, 0) + Coalesce(h_d_11, 0) 
                      + Coalesce(h_d_12, 0);
               
               h_cc_id := 'BILLABLE_BENEFIT_LABOR';
               BEGIN
                  INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                  MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                  MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                  VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_d_total,
                  h_d_01, h_d_02, h_d_03, h_d_04, h_d_05, h_d_06,
                  h_d_07, h_d_08, h_d_09, h_d_10, h_d_11, h_d_12);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 523+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(524) : cplan_fd.sql(4616) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("LABOR_REVENU: SC_STATE: %d, h_cur_year: %d, h_d_total: %.2f\n", SC_STATE,h_cur_year,h_d_total);
               
               If l_actual_flag = 1 Then
                  BEGIN
                     INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                     MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                     MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                     VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, -1 * h_d_total,
                     -1 * h_d_01, -1 * h_d_02, -1 * h_d_03, -1 * h_d_04, -1 * h_d_05, -1 * h_d_06,
                     -1 * h_d_07, -1 * h_d_08, -1 * h_d_09, -1 * h_d_10, -1 * h_d_11, -1 * h_d_12);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 547+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(548) : cplan_fd.sql(4640) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               BEGIN
                  UPDATE BUDGETS
                  SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS,h_rollup_bits)
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
                  AND CHARGECODE_ID = h_cc_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 568+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(569) : cplan_fd.sql(4661) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
   END LOOP;
   BEGIN
      CLOSE CTPH01 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 584+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(585) : cplan_fd.sql(4677) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_WBS SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS,h_rollup_bits) WHERE ELEMENT_ID = h_task_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 601+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(602) : cplan_fd.sql(4694) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("TMT_WBS: SC_STATE: %d, h_rollup_bits: %d\n", SC_STATE,h_rollup_bits);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      BEGIN
         UPDATE TMT_CRI SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS,h_rollup_bits) WHERE ELEMENT_ID = h_task_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_SCHED_ASSIGN', 620+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_SCHED_ASSIGN(621) : cplan_fd.sql(4713) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_TPHASE_SCHED_ASSIGN', 627+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_TPHASE_SCHED_ASSIGN(628) : cplan_fd.sql(4720) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_TPHASE_SCHED_ASSIGN;
ALTER FUNCTION FC_TPHASE_SCHED_ASSIGN COMPILE DEBUG;

/*******************************************************************************
********************************************************************************/

Prompt == FUNCTION FC_TPHASE_FD_SCHED ==
CREATE OR REPLACE FUNCTION FC_TPHASE_FD_SCHED
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , sh_h_update PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_update PLS_INTEGER := sh_h_update;
   h_tsk_assignment_id VARCHAR2(32);
   h_cc_id VARCHAR2(32);
   h_type PLS_INTEGER := 0;
   h_cur_year PLS_INTEGER := 0;
   h_end_year PLS_INTEGER := 0;
   h_start_month PLS_INTEGER := 0;
   h_end_month PLS_INTEGER := 0;
   h_budget_type PLS_INTEGER := 0;
   h_rollup_bits PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_01 NUMBER := 0;
   h_02 NUMBER := 0;
   h_03 NUMBER := 0;
   h_04 NUMBER := 0;
   h_05 NUMBER := 0;
   h_06 NUMBER := 0;
   h_07 NUMBER := 0;
   h_08 NUMBER := 0;
   h_09 NUMBER := 0;
   h_10 NUMBER := 0;
   h_11 NUMBER := 0;
   h_12 NUMBER := 0;
   h_total_h NUMBER := 0;
   h_d_total NUMBER := 0;
   h_d_01 NUMBER := 0;
   h_d_02 NUMBER := 0;
   h_d_03 NUMBER := 0;
   h_d_04 NUMBER := 0;
   h_d_05 NUMBER := 0;
   h_d_06 NUMBER := 0;
   h_d_07 NUMBER := 0;
   h_d_08 NUMBER := 0;
   h_d_09 NUMBER := 0;
   h_d_10 NUMBER := 0;
   h_d_11 NUMBER := 0;
   h_d_12 NUMBER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   stmt VARCHAR2(7999);
-- Declare Cursors
      CURSOR C_FD_TA IS
         SELECT TSK_ASSIGNMENT_ID, BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id;

      CURSOR CTPHFD IS
         SELECT FY.FISCAL_YEAR,
         SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 1)))),
         SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 2)))),
         SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 3)))),
         SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 4)))),
         SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 5)))),
         SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 6)))),
         SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 7)))),
         SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 8)))),
         SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 9)))),
         SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 10)))),
         SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 11)))),
         SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 12))))
         FROM TASK_DAY_OF_YEAR tdy JOIN FISCAL_CALENDAR fy
         ON tdy.DAY_YEAR BETWEEN fy.START_DATE AND fy.FINISH_DATE
         WHERE tdy.PROJECT_ID = h_project_id AND tdy.TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         GROUP BY FY.FISCAL_YEAR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_TPHASE_FD_SCHED') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_TPHASE_FD_SCHED'), '-- ' || SYSTIMESTAMP, ':r := FC_TPHASE_FD_SCHED(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_update), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_TPHASE_FD_SCHED', 83+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_TPHASE_FD_SCHED(84) : cplan_fd.sql(4815) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_update: %d, h_project_id: %d:%s, h_task_id: %d:%s\n",h_update,h_project_id_Ind,h_project_id,h_task_id_Ind,h_task_id);
   
   h_rollup_bits := PMO_CNST.SC_SCHED_DELTA_BIT;
   h_budget_type := PMO_CNST.SC_SCHED_BUDGET;
   
   OPEN C_FD_TA;
   
   LOOP
      FETCH C_FD_TA INTO h_tsk_assignment_id, h_billable, h_capital_expense, h_reg_cost, h_reg_sale;
      IF C_FD_TA%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      --trace_printf("FETCH_TA: SC_STATE: %d, h_tsk_assignment_id: %d %s\n", SC_STATE,h_tsk_assignment_id_Ind,h_tsk_assignment_id);
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS OR (h_01 IS NULL) Then
         EXIT;
      End If;
      
      OPEN CTPHFD;
      
      LOOP
         FETCH CTPHFD INTO h_cur_year, h_01, h_02, h_03, h_04, h_05, h_06, h_07, h_08, h_09, h_10, h_11, h_12;
         IF CTPHFD%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("FETCH: SC_STATE: %d, h_cur_year: %d, h_01: %d:%.2f\n", SC_STATE,h_cur_year,h_01_Ind,h_01);
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS OR (h_01 IS NULL) Then
            EXIT;
         End If;
         
         If Coalesce(h_update, 0) < 0 Then
            h_01 := -(Coalesce(h_01, 0) / 60.00);
            h_02 := -(Coalesce(h_02, 0) / 60.00);
            h_03 := -(Coalesce(h_03, 0) / 60.00);
            h_04 := -(Coalesce(h_04, 0) / 60.00);
            h_05 := -(Coalesce(h_05, 0) / 60.00);
            h_06 := -(Coalesce(h_06, 0) / 60.00);
            h_07 := -(Coalesce(h_07, 0) / 60.00);
            h_08 := -(Coalesce(h_08, 0) / 60.00);
            h_09 := -(Coalesce(h_09, 0) / 60.00);
            h_10 := -(Coalesce(h_10, 0) / 60.00);
            h_11 := -(Coalesce(h_11, 0) / 60.00);
            h_12 := -(Coalesce(h_12, 0) / 60.00);
         Else
            h_01 := (Coalesce(h_01, 0) / 60.00);
            h_02 := (Coalesce(h_02, 0) / 60.00);
            h_03 := (Coalesce(h_03, 0) / 60.00);
            h_04 := (Coalesce(h_04, 0) / 60.00);
            h_05 := (Coalesce(h_05, 0) / 60.00);
            h_06 := (Coalesce(h_06, 0) / 60.00);
            h_07 := (Coalesce(h_07, 0) / 60.00);
            h_08 := (Coalesce(h_08, 0) / 60.00);
            h_09 := (Coalesce(h_09, 0) / 60.00);
            h_10 := (Coalesce(h_10, 0) / 60.00);
            h_11 := (Coalesce(h_11, 0) / 60.00);
            h_12 := (Coalesce(h_12, 0) / 60.00);
         End If;
         
         h_total_h := Coalesce(h_01, 0) + Coalesce(h_02, 0) + Coalesce(h_03, 0) + Coalesce(h_04, 0) 
                      + Coalesce(h_05, 0) + Coalesce(h_06, 0) + Coalesce(h_07, 0) + Coalesce(
                      h_08, 0) + Coalesce(h_09, 0) + Coalesce(h_10, 0) + Coalesce(h_11, 0) 
                      + Coalesce(h_12, 0);
         --trace_printf("h_total_h: %.2f, h_01: %.2f, h_02: %.2f, h_03: %.2f, h_04: %.2f, h_05: %.2f, h_06: %.2f, h_07: %.2f, h_08: %.2f, h_09: %.2f, h_10: %.2f, h_11: %.2f, h_12: %.2f\n", h_total_h,h_01,h_02,h_03,h_04,h_05,h_06,h_07,h_08,h_09,h_10,h_12,h_12);
         
         If Coalesce(h_total_h, 0) <> 0.0 Then
            If Coalesce(h_capital_expense, 0) = 0 Then
               h_cc_id := 'EXPENSE_HOURS';
            Else
               h_cc_id := 'CAPITAL_HOURS';
            End If;
            
            BEGIN
               INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
               MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
               MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
               VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_total_h,
               h_01, h_02, h_03, h_04, h_05, h_06,
               h_07, h_08, h_09, h_10, h_11, h_12);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_FD_SCHED', 180+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_FD_SCHED(181) : cplan_fd.sql(4912) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("EXPENSE_CAPITAL_HOURS: SC_STATE: %d, h_cc_id: %s, h_cur_year: %d, h_total_h: %.2f\n", SC_STATE,h_cc_id,h_cur_year,h_total_h);
            
            BEGIN
               UPDATE BUDGETS
               SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS,h_rollup_bits)
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
               AND CHARGECODE_ID = h_cc_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_FD_SCHED', 201+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_FD_SCHED(202) : cplan_fd.sql(4933) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_reg_cost, 0) <> 0.00 Then
               If Coalesce(h_capital_expense, 0) = 0 Then
                  h_cc_id := 'EXPENSE_LABOR';
               Else
                  h_cc_id := 'CAPITAL_LABOR';
               End If;
               
               h_d_01 := Coalesce(h_01, 0) * Coalesce(h_reg_cost, 0);
               h_d_02 := Coalesce(h_02, 0) * Coalesce(h_reg_cost, 0);
               h_d_03 := Coalesce(h_03, 0) * Coalesce(h_reg_cost, 0);
               h_d_04 := Coalesce(h_04, 0) * Coalesce(h_reg_cost, 0);
               h_d_05 := Coalesce(h_05, 0) * Coalesce(h_reg_cost, 0);
               h_d_06 := Coalesce(h_06, 0) * Coalesce(h_reg_cost, 0);
               h_d_07 := Coalesce(h_07, 0) * Coalesce(h_reg_cost, 0);
               h_d_08 := Coalesce(h_08, 0) * Coalesce(h_reg_cost, 0);
               h_d_09 := Coalesce(h_09, 0) * Coalesce(h_reg_cost, 0);
               h_d_10 := Coalesce(h_10, 0) * Coalesce(h_reg_cost, 0);
               h_d_11 := Coalesce(h_11, 0) * Coalesce(h_reg_cost, 0);
               h_d_12 := Coalesce(h_12, 0) * Coalesce(h_reg_cost, 0);
               
               h_d_total := Coalesce(h_d_01, 0) + Coalesce(h_d_02, 0) + Coalesce(h_d_03, 0) + Coalesce(h_d_04, 0) 
                      + Coalesce(h_d_05, 0) + Coalesce(h_d_06, 0) + Coalesce(h_d_07, 0) + Coalesce(
                      h_d_08, 0) + Coalesce(h_d_09, 0) + Coalesce(h_d_10, 0) + Coalesce(h_d_11, 0) 
                      + Coalesce(h_d_12, 0);
               
               BEGIN
                  INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                  MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                  MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                  VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_d_total,
                  h_d_01, h_d_02, h_d_03, h_d_04, h_d_05, h_d_06,
                  h_d_07, h_d_08, h_d_09, h_d_10, h_d_11, h_d_12);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_FD_SCHED', 248+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_FD_SCHED(249) : cplan_fd.sql(4980) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("LABOR_COST: SC_STATE: %d, h_cur_year: %d, h_d_total: %.2f\n", SC_STATE,h_cur_year,h_d_total);
               
               BEGIN
                  UPDATE BUDGETS
                  SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS,h_rollup_bits)
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
                  AND CHARGECODE_ID = h_cc_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_FD_SCHED', 269+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_FD_SCHED(270) : cplan_fd.sql(5001) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If Coalesce(h_billable, 0) = 1 Then
               h_cc_id := 'BENEFIT_HOURS';
               
               BEGIN
                  INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                  MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                  MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                  VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_total_h,
                  h_01, h_02, h_03, h_04, h_05, h_06,
                  h_07, h_08, h_09, h_10, h_11, h_12);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_FD_SCHED', 295+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_FD_SCHED(296) : cplan_fd.sql(5027) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("BILLABLE_HOURS: SC_STATE: %d, h_cur_year: %d, h_total_h: %.2f\n", SC_STATE,h_cur_year,h_total_h);
               
               BEGIN
                  UPDATE BUDGETS
                  SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS,h_rollup_bits)
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
                  AND CHARGECODE_ID = h_cc_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_FD_SCHED', 316+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_FD_SCHED(317) : cplan_fd.sql(5048) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_reg_sale, 0) <> 0.00 Then
                  h_d_01 := Coalesce(h_01, 0) * Coalesce(h_reg_sale, 0);
                  h_d_02 := Coalesce(h_02, 0) * Coalesce(h_reg_sale, 0);
                  h_d_03 := Coalesce(h_03, 0) * Coalesce(h_reg_sale, 0);
                  h_d_04 := Coalesce(h_04, 0) * Coalesce(h_reg_sale, 0);
                  h_d_05 := Coalesce(h_05, 0) * Coalesce(h_reg_sale, 0);
                  h_d_06 := Coalesce(h_06, 0) * Coalesce(h_reg_sale, 0);
                  h_d_07 := Coalesce(h_07, 0) * Coalesce(h_reg_sale, 0);
                  h_d_08 := Coalesce(h_08, 0) * Coalesce(h_reg_sale, 0);
                  h_d_09 := Coalesce(h_09, 0) * Coalesce(h_reg_sale, 0);
                  h_d_10 := Coalesce(h_10, 0) * Coalesce(h_reg_sale, 0);
                  h_d_11 := Coalesce(h_11, 0) * Coalesce(h_reg_sale, 0);
                  h_d_12 := Coalesce(h_12, 0) * Coalesce(h_reg_sale, 0);
                  
                  h_d_total := Coalesce(h_d_01, 0) + Coalesce(h_d_02, 0) + Coalesce(h_d_03, 0) + Coalesce(h_d_04, 0) 
                        + Coalesce(h_d_05, 0) + Coalesce(h_d_06, 0) + Coalesce(h_d_07, 0) 
                        + Coalesce(h_d_08, 0) + Coalesce(h_d_09, 0) + Coalesce(h_d_10, 0) 
                        + Coalesce(h_d_11, 0) + Coalesce(h_d_12, 0);
                  
                  h_cc_id := 'BILLABLE_BENEFIT_LABOR';
                  BEGIN
                     INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                     MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                     MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                     VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_d_total,
                     h_d_01, h_d_02, h_d_03, h_d_04, h_d_05, h_d_06,
                     h_d_07, h_d_08, h_d_09, h_d_10, h_d_11, h_d_12);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_FD_SCHED', 358+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_FD_SCHED(359) : cplan_fd.sql(5090) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("LABOR_REVENU: SC_STATE: %d, h_cur_year: %d, h_d_total: %.2f\n", SC_STATE,h_cur_year,h_d_total);
                  
                  BEGIN
                     UPDATE BUDGETS
                     SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS,h_rollup_bits)
                     WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
                     AND CHARGECODE_ID = h_cc_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_FD_SCHED', 379+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_FD_SCHED(380) : cplan_fd.sql(5111) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE CTPHFD ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_FD_SCHED', 396+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_FD_SCHED(397) : cplan_fd.sql(5128) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE C_FD_TA ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_FD_SCHED', 410+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_FD_SCHED(411) : cplan_fd.sql(5142) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_WBS SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS,h_rollup_bits) WHERE ELEMENT_ID = h_task_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TPHASE_FD_SCHED', 427+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TPHASE_FD_SCHED(428) : cplan_fd.sql(5159) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("TMT_WBS: SC_STATE: %d, h_rollup_bits: %d\n", SC_STATE,h_rollup_bits);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_TPHASE_FD_SCHED', 434+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_TPHASE_FD_SCHED(435) : cplan_fd.sql(5166) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_TPHASE_FD_SCHED;
ALTER FUNCTION FC_TPHASE_FD_SCHED COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_GET_DELAY ==
CREATE OR REPLACE FUNCTION FC_GET_DELAY
(
     h_res_id IN OUT NOCOPY VARCHAR2
   , h_res_type IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , p_delay IN OUT NOCOPY PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_contour PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_DELAY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_DELAY'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_DELAY(' || 
       '''' || h_res_id || ''''
          || ', ' || '''' || h_res_type || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_delay), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_DELAY', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_DELAY(23) : cplan_fd.sql(5199) : ' || SYSTIMESTAMP);
   END IF; 

   If SubStr(h_res_type, 1, 1) = '0' Then
      -- Extend resource calendar
      h_duration := 0;
      If EXTEND_PRCAL(h_res_id, h_start_DT, h_finish_DT, h_duration) = 0 Then
         BEGIN
            SELECT COUNT(DAY_YEAR) INTO h_contour
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_res_id AND DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_DELAY', 39+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_DELAY(40) : cplan_fd.sql(5216) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      BEGIN
         SELECT COUNT(DAY_YEAR) INTO h_contour
         FROM PMO_TMP_CDY
         WHERE DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_DELAY', 55+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_DELAY(56) : cplan_fd.sql(5232) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   p_delay := Coalesce(h_contour, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_DELAY', 64+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_DELAY(65) : cplan_fd.sql(5241) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_DELAY;
ALTER FUNCTION FC_GET_DELAY COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_U_TASK_PERCENT ==
CREATE OR REPLACE FUNCTION FC_U_TASK_PERCENT
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , sh_h_new_actual PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_close_task PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_new_actual PLS_INTEGER := sh_h_new_actual;
   h_close_task PLS_INTEGER := Coalesce(sh_h_close_task, 0);
   h_tsk_assignment_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_dayofyear DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_last_submitted DATE;
   h_FD DATE;
   h_SD DATE;
   h_bit_flag VARCHAR2(1);
   h_last_day PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_work_percent PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_SF PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_sale NUMBER := 0;
   h_b_reg_cost NUMBER := 0;
   h_b_reg_sale NUMBER := 0;
   h_b_spec_cost NUMBER := 0;
   h_b_spec_sale NUMBER := 0;
   i_prev_min PLS_INTEGER NOT NULL := 0;
   i_r_rem PLS_INTEGER NOT NULL := 0;
   i_t_rem PLS_INTEGER NOT NULL := 0;
   ratio_rem NUMBER NOT NULL := 0.00;
   i_TWP9 PLS_INTEGER;
   l_count PLS_INTEGER;
-- Declare Cursors
      CURSOR C_TWP00 IS
         SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, BIT_FLAG, LAST_SUBMITTED, WORK_TODATE, SCHED_WORK, LAST_SCHED_MIN,
         TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
         REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE,
         B_REGULAR_COST, B_REGULAR_RATE, B_SPECIAL_COST, B_SPECIAL_RATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND BIT_FLAG = '2';

      CURSOR C_TWP01 IS
         SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, BIT_FLAG, LAST_SUBMITTED, WORK_TODATE, SCHED_WORK, LAST_SCHED_MIN,
         TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
         REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE,
         B_REGULAR_COST, B_REGULAR_RATE, B_SPECIAL_COST, B_SPECIAL_RATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND BIT_FLAG = '2' AND MILESTONE_PERCENT <> 10000;

      CURSOR C_TWP0 IS
         SELECT DAY_YEAR, SUM(SCHED_MIN), COUNT(DAY_YEAR)
         FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND BIT_FLAG = '2'
         GROUP BY DAY_YEAR;

      CURSOR C_TWP9 IS
         SELECT TSK_ASSIGNMENT_ID, SUM(SCHED_MIN)
         FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id
         AND BIT_FLAG = '2' AND DAY_YEAR >= h_start_D AND DAY_YEAR <= h_finish_D
         GROUP BY TSK_ASSIGNMENT_ID;

      CURSOR C_TWP9i IS
         SELECT tdy.TSK_ASSIGNMENT_ID, SUM(tdy.SCHED_MIN)
         FROM TASK_DAY_OF_YEAR tdy
         WHERE tdy.PROJECT_ID = h_project_id AND tdy.TASK_ID = h_task_id
         AND tdy.BIT_FLAG = '2' AND tdy.DAY_YEAR > h_finish_D AND tdy.ACTUAL_FLAG = 1 AND
         NOT EXISTS (SELECT t.TSK_ASSIGNMENT_ID from PMO_TMP_TWP9 t
         WHERE tdy.TSK_ASSIGNMENT_ID = t.TSK_ASSIGNMENT_ID)
         GROUP BY TSK_ASSIGNMENT_ID;

      CURSOR C_TWP10 IS
         SELECT TSK_ASSIGNMENT_ID, SCHED_MIN
         FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id
         AND BIT_FLAG = '2' AND DAY_YEAR = h_finish_D;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_TASK_PERCENT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_TASK_PERCENT'), '-- ' || SYSTIMESTAMP, ':r := FC_U_TASK_PERCENT(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_new_actual), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_close_task), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TASK_PERCENT', 98+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_TASK_PERCENT(99) : cplan_fd.sql(5350) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_project_id: %s, h_task_id: %s\n", h_project_id,h_task_id);
   --trace_printf("h_work_todate: %d, h_new_actual = %d\n", h_work_todate,h_new_actual);
   
   If Coalesce(h_new_actual, 0) = 0 Then
      h_work_percent := 0;
      
      OPEN C_TWP00;
      
      LOOP
         FETCH C_TWP00 INTO h_tsk_assignment_id, h_res_id, h_bit_flag, h_last_submitted, h_work_todate, h_effort, h_last_day,
         h_SD, h_FD, h_billable, h_capital_expense,
         h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale,
         h_b_reg_cost, h_b_reg_sale, h_b_spec_cost, h_b_spec_sale;
         IF C_TWP00%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("i00_* SC_STATE: %d,h_tsk_assignment_id = %s, h_new_actual = %d\n", SC_STATE, h_tsk_assignment_id,h_new_actual);
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id
                                  , h_last_submitted, h_bit_flag, h_effort, h_new_actual, h_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, 0);
      END LOOP;
      
      BEGIN
         CLOSE C_TWP00;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 140+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(141) : cplan_fd.sql(5392) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf h_close_task = 1 Then
      h_work_percent := 10000;
      
      OPEN C_TWP01;
      
      LOOP
         FETCH C_TWP01 INTO h_tsk_assignment_id, h_res_id, h_bit_flag, h_last_submitted, h_work_todate, h_effort, h_last_day,
         h_SD, h_FD, h_billable, h_capital_expense,
         h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale,
         h_b_reg_cost, h_b_reg_sale, h_b_spec_cost, h_b_spec_sale;
         IF C_TWP01%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("i00_* SC_STATE: %d,h_tsk_assignment_id = %s, h_new_actual = %d\n", SC_STATE, h_tsk_assignment_id,h_new_actual);
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id
                                  , h_last_submitted, h_bit_flag, h_effort, h_effort, h_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, 0);
      END LOOP;
      
      BEGIN
         CLOSE C_TWP01;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 180+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(181) : cplan_fd.sql(5432) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_total := 0;
      h_start_D := NULL;
      
      OPEN C_TWP0;
      
      LOOP
         FETCH C_TWP0 INTO h_dayofyear, h_minutes, h_count;
         IF C_TWP0%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("h_dayofyear = %s, h_minutes = %d\n", h_dayofyear,h_minutes);
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         If Coalesce(h_total, 0) = 0 Then
            h_start_D := h_dayofyear;
         End If;
         
         i_prev_min := Coalesce(h_total, 0);
         h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
         --trace_printf("*** i_prev_min: %d, h_total = %d, h_minutes = %d\n", i_prev_min,h_total,h_minutes);
         If Coalesce(h_total, 0) <= Coalesce(h_new_actual, 0) Then
            If Coalesce(h_total, 0) = Coalesce(h_new_actual, 0) Then
               h_last_day := Coalesce(h_minutes, 0);
               EXIT;
            End If;
         Else
            h_last_day := Coalesce(h_minutes, 0);
            
            If Coalesce(h_new_actual, 0) > i_prev_min Then
               h_minutes := Coalesce(h_new_actual, 0) - i_prev_min;
            Else
               h_minutes := Coalesce(h_new_actual, 0);
            End If;
            
            /*if (h_new_actual <= h_minutes)
                            {
                                h_minutes = h_new_actual;
                            }
                            else
                            {
                                h_minutes = h_new_actual - i_prev_min;
                            }*/
            --trace_printf("### h_minutes: %d, h_new_actual: %d, i_prev_min: %d\n", h_minutes,h_new_actual,i_prev_min);
            
            EXIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_TWP0 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 245+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(246) : cplan_fd.sql(5497) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_finish_D := h_dayofyear;
      ratio_rem := Coalesce(h_minutes, 0) / Coalesce(h_last_day, 0);
      --trace_printf("h_count: %d, h_last_day: %d, h_minutes: %d, ratio_rem: %.4f, h_start_D = %s, h_finish_D = %s\n", h_count,h_last_day,h_minutes,ratio_rem,h_start_D,h_finish_D);
      --trace_printf("====================================================================================\n");
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TWP9';
      PMO_DB2M.SC_STATE := 0;
      
      i_TWP9 := 0;
      l_count := 0;
      h_new_actual := 0;
      If PMO_ORASTD.strcmp(h_start_D, h_finish_D) <> 0 Then
         OPEN C_TWP9;
         
         LOOP
            FETCH C_TWP9 INTO h_tsk_assignment_id,h_new_actual;
            IF C_TWP9%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("\n===> i1_* SC_STATE: %d, h_tsk_assignment_id = %s, h_new_actual = %d\n", SC_STATE, h_tsk_assignment_id,h_new_actual);
            
            If PMO_DB2M.SC_STATE <> 0 Then
               --trace_printf("==========> i_TWP9: %d <===================\n", i_TWP9); //trace_flush();
               If i_TWP9 <> 0 Then
                  EXIT;
               End If;
               
               i_TWP9 := 1;
               
               OPEN C_TWP9i;
               
               LOOP
                  FETCH C_TWP9i INTO h_tsk_assignment_id,h_new_actual;
                  IF C_TWP9i%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  --trace_printf("i0_* SC_STATE: %d,h_tsk_assignment_id = %s, h_new_actual = %d\n", SC_STATE, h_tsk_assignment_id,h_new_actual);
                  If PMO_DB2M.SC_STATE <> 0 Then
                     h_new_actual := 0;
                     EXIT;
                  End If;
                  
                  If Coalesce(h_new_actual, 0) <> 0 Then
                     h_new_actual := 0;
                  End If;
                  BEGIN
                     SELECT RESOURCE_ID, BIT_FLAG, LAST_SUBMITTED, WORK_TODATE, SCHED_WORK, LAST_SCHED_MIN,
                     TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
                     REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE,
                     B_REGULAR_COST, B_REGULAR_RATE, B_SPECIAL_COST, B_SPECIAL_RATE
                     INTO h_res_id, h_bit_flag, h_last_submitted, h_work_todate, h_effort, h_last_day,
                     h_SD, h_FD, h_billable, h_capital_expense,
                     h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale,
                     h_b_reg_cost, h_b_reg_sale, h_b_spec_cost, h_b_spec_sale
                     FROM TASK_ASSIGNMENTS
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 316+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(317) : cplan_fd.sql(5568) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_work_percent := 0;
                  -- new % Work
                  
                  h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id
                                  , h_last_submitted, h_bit_flag, h_effort, h_new_actual, h_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, 0);
               END LOOP;
               
               BEGIN
                  CLOSE C_TWP9i ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 340+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(341) : cplan_fd.sql(5592) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If Coalesce(h_new_actual, 0) > 0 Then
               BEGIN
                  INSERT INTO PMO_TMP_TWP9 VALUES (h_tsk_assignment_id);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 359+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(360) : cplan_fd.sql(5611) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               l_count := l_count + 1;
               BEGIN
                  SELECT SCHED_MIN INTO h_total
                  FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_finish_D;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 375+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(376) : cplan_fd.sql(5627) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("** SC_STATE: %d, h_finish_D: %s, h_total = %d, h_new_actual = %d\n", SC_STATE, h_finish_D,h_total,h_new_actual);
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  h_new_actual := Coalesce(h_new_actual, 0) - Coalesce(h_total, 0);
                  If l_count = Coalesce(h_count, 0) Then
                     i_r_rem := Coalesce(h_minutes, 0) - i_t_rem;
                  Else
                     i_r_rem := Trunc((Coalesce(h_total, 0) * ratio_rem) + 0.4);
                     i_t_rem := i_t_rem + i_r_rem;
                  End If;
                  
                  h_new_actual := Coalesce(h_new_actual, 0) + i_r_rem;
               End If;
               --trace_printf("1_* h_new_actual: %d, h_total: %d, ratio_rem: %.4f\n", h_new_actual,h_total,ratio_rem);
               --trace_printf("l_count: %d, i_t_rem: %d, i_r_rem: %d\n", l_count,i_t_rem,i_r_rem);
               
               BEGIN
                  SELECT RESOURCE_ID, BIT_FLAG, LAST_SUBMITTED, WORK_TODATE, SCHED_WORK, LAST_SCHED_MIN,
                  TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
                  REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE,
                  B_REGULAR_COST, B_REGULAR_RATE, B_SPECIAL_COST, B_SPECIAL_RATE
                  INTO h_res_id, h_bit_flag, h_last_submitted, h_work_todate, h_effort, h_last_day,
                  h_SD, h_FD, h_billable, h_capital_expense,
                  h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale,
                  h_b_reg_cost, h_b_reg_sale, h_b_spec_cost, h_b_spec_sale
                  FROM TASK_ASSIGNMENTS
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 412+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(413) : cplan_fd.sql(5664) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_work_percent := Trunc(((Coalesce(h_new_actual, 0) / Coalesce(h_effort, 0)) * 10000.00) + 0.4);
               -- new % Work
               --trace_printf("h_work_percent: %d, h_work_todate: %d, h_effort = %d\n", h_work_percent,h_work_todate,h_effort);
               --trace_printf("=====> h_SD: %d %s, h_FD: %d %s, h_new_actual: %d\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD,h_new_actual);
               
               h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id
                                  , h_last_submitted, h_bit_flag, h_effort, h_new_actual, h_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, 0);
               
               h_new_actual := 0;
               i_prev_min := 0;
               --i_r_rem  = 0.00;
               --i_t_rem  = 0.00;
            End If;
         END LOOP;
         
         BEGIN
            CLOSE C_TWP9 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 444+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(445) : cplan_fd.sql(5696) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         OPEN C_TWP10;
         
         LOOP
            FETCH C_TWP10 INTO h_tsk_assignment_id,h_new_actual;
            IF C_TWP10%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("i2_* SC_STATE: %d,h_tsk_assignment_id = %s, h_new_actual = %d\n", SC_STATE, h_tsk_assignment_id,h_new_actual);
            If PMO_DB2M.SC_STATE <> 0 Then
               EXIT;
            End If;
            
            l_count := l_count + 1;
            If l_count = Coalesce(h_count, 0) Then
               i_r_rem := Coalesce(h_minutes, 0) - i_t_rem;
            Else
               i_r_rem := Trunc((Coalesce(h_new_actual, 0) * ratio_rem) + 0.4);
               i_t_rem := i_t_rem + i_r_rem;
            End If;
            
            h_new_actual := i_r_rem;
            --trace_printf("2_* h_new_actual: %d, h_total: %d, ratio_rem: %.4f\n", h_new_actual,h_total,ratio_rem);
            --trace_printf("l_count: %d, i_t_rem: %d, i_r_rem: %d\n", l_count,i_t_rem,i_r_rem);
            
            BEGIN
               SELECT RESOURCE_ID, BIT_FLAG, LAST_SUBMITTED, WORK_TODATE, SCHED_WORK, LAST_SCHED_MIN,
               TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
               REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE,
               B_REGULAR_COST, B_REGULAR_RATE, B_SPECIAL_COST, B_SPECIAL_RATE
               INTO h_res_id, h_bit_flag, h_last_submitted, h_work_todate, h_effort, h_last_day,
               h_SD, h_FD, h_billable, h_capital_expense,
               h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale,
               h_b_reg_cost, h_b_reg_sale, h_b_spec_cost, h_b_spec_sale
               FROM TASK_ASSIGNMENTS
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 492+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(493) : cplan_fd.sql(5744) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_work_percent := Trunc(((Coalesce(h_new_actual, 0) / Coalesce(h_effort, 0)) * 10000.00) + 0.4);
            -- new actual work
            --trace_printf("h_work_percent: %d, h_work_todate: %d, h_effort = %d\n", h_work_percent,h_work_todate,h_effort);
            --trace_printf("=====> h_SD: %d %s, h_FD: %d %s\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD);
            
            h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id
                                  , h_last_submitted, h_bit_flag, h_effort, h_new_actual, h_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, 0);
         END LOOP;
         
         BEGIN
            CLOSE C_TWP10 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 518+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(519) : cplan_fd.sql(5770) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TASK_PERCENT', 526+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_TASK_PERCENT(527) : cplan_fd.sql(5778) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_TASK_PERCENT;
ALTER FUNCTION FC_U_TASK_PERCENT COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_U_ASSIGN_PERCENT ==
CREATE OR REPLACE FUNCTION FC_U_ASSIGN_PERCENT
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_last_submitted IN OUT NOCOPY DATE
   , h_bit_flag IN OUT NOCOPY VARCHAR2
   , sh_h_effort PLS_INTEGER
   , sh_h_new_actual PLS_INTEGER
   , sh_h_sched_work PLS_INTEGER
   , sh_h_work_todate PLS_INTEGER
   , sh_h_work_percent PLS_INTEGER
   , sh_h_actual_regular_cost NUMBER
   , sh_h_actual_special_cost NUMBER
   , sh_h_actual_regular_rate NUMBER
   , sh_h_actual_special_rate NUMBER
   , sh_h_baseline_regular_cost NUMBER
   , sh_h_baseline_special_cost NUMBER
   , sh_h_baseline_regular_rate NUMBER
   , sh_h_baseline_special_rate NUMBER
   , sh_h_capital_expense001 PLS_INTEGER
   , sh_h_billable001 PLS_INTEGER
   , sh_h_last_day PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_per_day PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_effort PLS_INTEGER := sh_h_effort;
   h_new_actual PLS_INTEGER := sh_h_new_actual;
   h_sched_work PLS_INTEGER := sh_h_sched_work;
   h_work_todate PLS_INTEGER := sh_h_work_todate;
   h_work_percent PLS_INTEGER := sh_h_work_percent;
   h_actual_regular_cost NUMBER := sh_h_actual_regular_cost;
   h_actual_special_cost NUMBER := sh_h_actual_special_cost;
   h_actual_regular_rate NUMBER := sh_h_actual_regular_rate;
   h_actual_special_rate NUMBER := sh_h_actual_special_rate;
   h_baseline_regular_cost NUMBER := sh_h_baseline_regular_cost;
   h_baseline_special_cost NUMBER := sh_h_baseline_special_cost;
   h_baseline_regular_rate NUMBER := sh_h_baseline_regular_rate;
   h_baseline_special_rate NUMBER := sh_h_baseline_special_rate;
   h_capital_expense001 PLS_INTEGER := sh_h_capital_expense001;
   h_billable001 PLS_INTEGER := sh_h_billable001;
   h_last_day PLS_INTEGER := sh_h_last_day;
   h_per_day PLS_INTEGER := Coalesce(sh_h_per_day, 0);
   h_chargecode_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_dayofyear DATE;
   h_week_apv DATE;
   h_total PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_milestone_percent PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_week_of DATE;
   h_next_week_of DATE;
   h_start_D DATE;
   h_null_str VARCHAR2(1);
   h_exchange_rate NUMBER := 0.00;
   h_prj_exchange_rate NUMBER := 0.00;
   h_res_exchange_rate NUMBER := 0.00;
   h_total_h NUMBER := 0;
   h_d_total NUMBER := 0;
   h_call_type PLS_INTEGER := 0;
   h_last_week PLS_INTEGER := 0;
   h_prj_currency_id PLS_INTEGER := 0;
   h_res_currency_id PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_SF PLS_INTEGER := 0;
   h_activity_count PLS_INTEGER := 0;
   h_baseline1_work PLS_INTEGER := 0;
   h_baseline_effort_week PLS_INTEGER := 0;
   h_friday_regular PLS_INTEGER := 0;
   h_friday_special PLS_INTEGER := 0;
   h_monday_regular PLS_INTEGER := 0;
   h_monday_special PLS_INTEGER := 0;
   h_saturday_regular PLS_INTEGER := 0;
   h_saturday_special PLS_INTEGER := 0;
   h_sunday_regular PLS_INTEGER := 0;
   h_sunday_special PLS_INTEGER := 0;
   h_thursday_regular PLS_INTEGER := 0;
   h_thursday_special PLS_INTEGER := 0;
   h_tuesday_regular PLS_INTEGER := 0;
   h_tuesday_special PLS_INTEGER := 0;
   h_wednesday_regular PLS_INTEGER := 0;
   h_wednesday_special PLS_INTEGER := 0;
   h_weekly_regular_total PLS_INTEGER := 0;
   h_weekly_special_total PLS_INTEGER := 0;
   h_temp_consider PLS_INTEGER := 0;
   h_temp_work_type PLS_INTEGER := 0;
   h_temp_contour_type PLS_INTEGER := 0;
   h_temp_occurrence PLS_INTEGER := 0;
   h_ta_ms PLS_INTEGER := 0;
   h_ta_etc PLS_INTEGER := 0;
   h_SD DATE;
   h_FD DATE;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_res_type VARCHAR2(1);
   h_work PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   h_current_weekday PLS_INTEGER := 0;
   h_db_day_of_week PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_assign_string VARCHAR2(8192);
   i_keep_etc PLS_INTEGER NOT NULL := 0;
   h_per_day_Ind PLS_INTEGER NOT NULL := 0;
   i_prev_min PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR CFW_AWP0 IS
         SELECT SCHED_MIN, DAY_YEAR, BIT_SCHED, PMO_ORASTD.DAYOFWEEK(DAY_YEAR)
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         ORDER BY DAY_YEAR;

      CURSOR CFW_AWP IS
         SELECT SCHED_MIN, DAY_YEAR, BIT_SCHED, PMO_ORASTD.DAYOFWEEK(DAY_YEAR)
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR >= h_last_submitted
         ORDER BY DAY_YEAR;

      CURSOR CBW_AWP IS
         SELECT SCHED_MIN, DAY_YEAR, BIT_SCHED, PMO_ORASTD.DAYOFWEEK(DAY_YEAR)
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND ACTUAL_FLAG = 1 AND DAY_YEAR <= h_last_submitted
         ORDER BY DAY_YEAR DESC;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_ASSIGN_PERCENT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_ASSIGN_PERCENT'), '-- ' || SYSTIMESTAMP, ':r := FC_U_ASSIGN_PERCENT(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_last_submitted || '''',''''''), 'NULL')
          || ', ' || '''' || h_bit_flag || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_new_actual), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_sched_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work_todate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work_percent), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_actual_regular_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_actual_special_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_actual_regular_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_actual_special_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_baseline_regular_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_baseline_special_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_baseline_regular_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_baseline_special_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_capital_expense001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_billable001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_last_day), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_ASSIGN_PERCENT', 171+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_ASSIGN_PERCENT(172) : cplan_fd.sql(5960) : ' || SYSTIMESTAMP);
   END IF; 

   --  SC_LONG(h_sched_min);
   --  SC_LONG_INIT(h_sched_work, 0);
   
   h_monday_regular := 0;
   h_tuesday_regular := 0;
   h_wednesday_regular := 0;
   h_thursday_regular := 0;
   h_friday_regular := 0;
   h_saturday_regular := 0;
   h_sunday_regular := 0;
   h_monday_special := 0;
   h_tuesday_special := 0;
   h_wednesday_special := 0;
   h_thursday_special := 0;
   h_friday_special := 0;
   h_saturday_special := 0;
   h_sunday_special := 0;
   
   h_null_str := NULL;
   h_week_of := NULL;
   h_next_week_of := NULL;
   h_dayofyear := NULL;
   h_week_apv := NULL;
   h_remaining := 0;
   h_weekly_regular_total := 0;
   h_weekly_special_total := 0;
   h_current_weekday := 0;
   
   --trace_printf("\nh_project_id: %s, h_task_id: %s\n", h_project_id,h_task_id);
   --trace_printf("h_tsk_assignment_id = %s, h_effort: %d, h_sched_work: %d\n", h_tsk_assignment_id,h_effort,h_sched_work);
   --trace_printf("h_last_submitted: %d:%s, h_work_todate: %d, h_new_actual = %d\n", h_last_submitted_Ind,h_last_submitted,h_work_todate,h_new_actual);
   If Coalesce(h_work_todate, 0) = Coalesce(h_new_actual, 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_ASSIGN_PERCENT', 207+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_U_ASSIGN_PERCENT(208) : cplan_fd.sql(5996) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN -1;
   End If;
   
   If Coalesce(h_work_percent, 0) > PMO_CNST.SC_BIT_30 Then
      i_keep_etc := 1;
      h_work_percent := (BITAND(Coalesce(h_work_percent, 0), PMO_CNST.SC_BIT_NOT_30));
      -- remove etc flag
   End If;
   
   -- Fetch currency exchnage info ************************************************************************************** 2
   --trace_printf("RATES+++> h_actual_regular_cost: %.4f, h_actual_regular_rate: %.4f\n", h_actual_regular_cost, h_actual_regular_rate);
   If (Coalesce(h_actual_regular_rate, 0) + Coalesce(h_actual_regular_cost, 0)) > 0.00 Then
      --project currency
      BEGIN
         SELECT CURRENCY_ID INTO h_prj_currency_id
         FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 231+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(232) : cplan_fd.sql(6020) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT R.CURRENCY_ID
         INTO h_res_currency_id
         FROM RESOURCES R
         WHERE R.RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 247+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(248) : cplan_fd.sql(6036) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --exchage rate
      If Coalesce(h_prj_currency_id, 0) <> Coalesce(h_res_currency_id, 0) Then
         BEGIN
            SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME, 18, 3)
            INTO h_prj_exchange_rate
            FROM TMT_DATAFIELDS
            WHERE ELEMENT_ID = h_prj_currency_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 265+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(266) : cplan_fd.sql(6054) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME, 18, 3)
            INTO h_res_exchange_rate
            FROM TMT_DATAFIELDS
            WHERE ELEMENT_ID = h_res_currency_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 281+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(282) : cplan_fd.sql(6070) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If (Coalesce(h_prj_exchange_rate, 0) > 0.00) AND (Coalesce(h_res_exchange_rate, 0) > 0.00) Then
         BEGIN
            SELECT PMO_ORASTD.TODECIMAL(h_prj_exchange_rate / h_res_exchange_rate, 18, 3 )
            INTO h_exchange_rate
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 298+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(299) : cplan_fd.sql(6087) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_exchange_rate := 0.000;
      End If;
   End If;
   --trace_printf("RATES+++> h_exchange_rate: %.4f\n", h_exchange_rate);
   
   h_call_type := 2;
   h_last_week := 0;
   h_week := 0;
   
   BEGIN
      SELECT SETTING_VALUE INTO h_db_day_of_week
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'TIMESHEET_START_WEEKDAY';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 322+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(323) : cplan_fd.sql(6111) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("+++> SC_STATE: %d, h_db_day_of_week: %d %d\n", SC_STATE,h_db_day_of_week_Ind,h_db_day_of_week);
   
   --h_billable = (h_billable | SC_BIT_30);
   h_ta_ms := Trunc((Coalesce(h_work_todate, 0) / Coalesce(h_sched_work, 0)) * 10000.00);
   h_ta_etc := Coalesce(h_sched_work, 0) - Coalesce(h_work_todate, 0);
   
   If Coalesce(h_work_todate, 0) = 0 Then
      --timeLog (file,"1_* FC_U_ASSIGN_PERCENT Old Actual = 0:");
      --trace_printf("***** h_work_todate == 0 ******************************************************\n");
      If SubStr(h_bit_flag, 1, 1) = '0' Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            SCHED_START         = PLANNED_START,
            SCHED_FINISH        = PLANNED_FINISH,
            SCHED_WORK          = PLANNED_WORK,
            BIT_FLAG            = '2'
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 353+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(354) : cplan_fd.sql(6142) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_milestone_percent := 0;
      h_etc := h_effort;
      h_total := Coalesce(h_work_todate, 0);
      
      OPEN CFW_AWP0;
      
      LOOP
         FETCH CFW_AWP0 INTO h_minutes, h_dayofyear, h_bitID, h_current_weekday;
         IF CFW_AWP0%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("h_dayofyear = %s, h_minutes = %d, h_total: %d\n", h_dayofyear,h_minutes,h_total);
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         --h_sched_min = h_minutes;
         If Coalesce(h_week, 0) = 0 Then
            h_week := Coalesce(h_db_day_of_week, 0) + 1;
            If Coalesce(h_current_weekday, 0) >= Coalesce(h_week, 0) Then
               h_temp_var := Coalesce(h_current_weekday, 0) - Coalesce(h_week, 0);
            Else
               h_temp_var := 7 + Coalesce(h_current_weekday, 0) - Coalesce(h_week, 0);
            End If;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) - h_temp_var     
               INTO h_week_of
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 396+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(397) : cplan_fd.sql(6185) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --this is for to keep the next week and if the week changes then call FC_U_Smmary_timesheets
            BEGIN
               SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_next_week_of FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 410+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(411) : cplan_fd.sql(6199) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf PMO_ORASTD.strcmp(h_dayofyear, h_next_week_of) >= 0 Then
            h_weekly_regular_total := Coalesce(h_monday_regular, 0) + Coalesce(h_tuesday_regular, 0) + Coalesce(
                                   h_wednesday_regular, 0) + Coalesce(h_thursday_regular, 0) 
                                   + Coalesce(h_friday_regular, 0) + Coalesce(
                                   h_saturday_regular, 0) + Coalesce(h_sunday_regular, 0);
            If Coalesce(h_effort, 0) > Coalesce(h_sched_work, 0) Then
               h_etc := 0;
               h_milestone_percent := 10000;
            Else
               h_etc := Coalesce(h_etc, 0) - Coalesce(h_weekly_regular_total, 0);
               h_milestone_percent := Trunc((1 - (Coalesce(h_etc, 0) / Coalesce(h_effort, 0))) * 10000.00);
            End If;
            --trace_printf("===> h_week_of: %s, h_weekly_regular_total: %d, h_milestone_percent: %d, h_ta_ms: %d, h_etc: %d\n", h_week_of,h_weekly_regular_total,h_milestone_percent,h_ta_ms,h_etc);
            
            PMO_DB2M.SC_STATE := FC_U_SUMMARY_TIMESHEETS(h_call_type, h_last_week, h_project_id, h_res_id
                                             , h_tsk_assignment_id, h_week_of, h_null_str
                                             , h_null_str, h_tsk_assignment_id, h_rec_user
                                             , h_billable, h_capital_expense, h_monday_regular
                                             , h_monday_special, h_tuesday_regular
                                             , h_tuesday_special, h_wednesday_regular
                                             , h_wednesday_special, h_thursday_regular
                                             , h_thursday_special, h_friday_regular
                                             , h_friday_special, h_saturday_regular
                                             , h_saturday_special, h_sunday_regular
                                             , h_sunday_special, h_weekly_regular_total
                                             , h_weekly_special_total, h_sched_work
                                             , h_baseline1_work, h_baseline_effort_week, h_ta_etc
                                             , h_ta_ms, h_etc, h_milestone_percent, h_exchange_rate
                                             , h_actual_regular_cost, h_actual_regular_rate
                                             , h_actual_special_cost, h_actual_special_rate
                                             , h_baseline_regular_cost, h_baseline_regular_rate
                                             , h_baseline_special_cost, h_baseline_special_rate);
            
            h_monday_regular := 0;
            h_tuesday_regular := 0;
            h_wednesday_regular := 0;
            h_thursday_regular := 0;
            h_friday_regular := 0;
            h_saturday_regular := 0;
            h_sunday_regular := 0;
            
            h_ta_ms := Coalesce(h_milestone_percent, 0);
            h_ta_etc := Coalesce(h_etc, 0);
            
            --the week changed, reinitialize the week of  next_week
            h_week_of := h_next_week_of;
            BEGIN
               SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_next_week_of FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 468+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(469) : cplan_fd.sql(6257) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("*** h_week_of: %s, h_next_week_of: %s\n", h_week_of,h_next_week_of);
         
         i_prev_min := Coalesce(h_total, 0);
         h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
         --trace_printf("*** i_prev_min: %d, h_total = %d, h_minutes = %d\n", i_prev_min,h_total,h_minutes);
         If Coalesce(h_total, 0) <= Coalesce(h_new_actual, 0) Then
            If Coalesce(h_total, 0) = Coalesce(h_new_actual, 0) Then
               EXIT;
            End If;
         Else
            /*if (h_new_actual <= h_minutes)
                            {
                                h_minutes = h_new_actual;
                            }
                            else
                            {
                                h_minutes = h_new_actual - i_prev_min;
                            }*/
            h_minutes := Coalesce(h_new_actual, 0) - i_prev_min;
            --trace_printf("### h_total = %d, i_prev_min: %d, h_temp_var: %d, h_minutes = %d\n", h_total,i_prev_min,h_temp_var,h_minutes);
            
            EXIT;
         End If;
         
         CASE h_current_weekday
            
            WHEN 1 THEN
               h_sunday_regular := Coalesce(h_minutes, 0);
            
            WHEN 2 THEN
               h_monday_regular := Coalesce(h_minutes, 0);
            
            WHEN 3 THEN
               h_tuesday_regular := Coalesce(h_minutes, 0);
            
            WHEN 4 THEN
               h_wednesday_regular := Coalesce(h_minutes, 0);
            
            WHEN 5 THEN
               h_thursday_regular := Coalesce(h_minutes, 0);
            
            WHEN 6 THEN
               h_friday_regular := Coalesce(h_minutes, 0);
            
            WHEN 7 THEN
               h_saturday_regular := Coalesce(h_minutes, 0);
            ELSE
               NULL;
         END CASE;
      END LOOP;
      
      BEGIN
         CLOSE CFW_AWP0 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 532+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(533) : cplan_fd.sql(6321) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_work_todate, 0) < Coalesce(h_new_actual, 0) Then
      --trace_printf("***** h_work_todate < h_new_actual *** h_last_day: %d\n",h_last_day);
      --timeLog (file,"2_* FC_U_ASSIGN_PERCENT Old Actual < New Actual");
      
      BEGIN
         SELECT MAX(WEEK_OF) INTO h_week_apv FROM TIMESHEET_APPROVAL_STATUS
         WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id
         AND CURRENT_STATE IN (PMO_CNST.SC_APV_STATE_SUBMITTED,PMO_CNST.SC_APV_STATE_RM_APPROVED,PMO_CNST.SC_APV_STATE_PM_APPROVED);
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 550+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(551) : cplan_fd.sql(6339) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("+++> SC_STATE: %d, h_week_apv: %d %s, h_res_id: %d %s\n", SC_STATE,h_week_apv_Ind,h_week_apv,h_res_id_Ind,h_res_id);
      
      If h_week_apv IS NOT NULL Then
         BEGIN
            SELECT COUNT('X') INTO h_temp_var FROM DUAL WHERE EXISTS (SELECT 'X' FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND ACTUAL_FLAG = 1 AND DAY_YEAR BETWEEN h_week_apv AND TO_DATE(SubStr(h_week_apv, 1, 10)) + 7     );
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 567+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(568) : cplan_fd.sql(6356) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("1_* SC_STATE: %d, h_temp_var: %d\n", SC_STATE,h_temp_var);
         
         If Coalesce(h_temp_var, 0) > 0 Then
            BEGIN
               SELECT PROJECT_ASSIGNMENT_ID, C_CONTOUR, UNITS_MSP
               INTO h_prj_assignment_id, h_contour, h_percent_units
               FROM TASK_ASSIGNMENTS
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 585+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(586) : cplan_fd.sql(6374) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_prj_assignment_id: %d %s\n", SC_STATE,h_prj_assignment_id_Ind,h_prj_assignment_id);//trace_flush();
            
            If h_prj_assignment_id IS NOT NULL Then
               h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 2);
            Else
               If SubStr(h_bit_flag, 1, 1) = '2' Then
                  h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 0);
               Else
                  h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 1);
               End If;
            End If;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_week_apv, 1, 10)) + 7     
               INTO h_start_date
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 611+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(612) : cplan_fd.sql(6400) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --trace_printf("WP_FC_DEL_NOACTUAL: h_start_date: %s, h_contour: %d, h_percent_units: %d\n", h_start_date,h_contour,h_percent_units);
            
            h_total := Coalesce(h_effort, 0) - Coalesce(h_work_todate, 0);
            h_temp_var := 2;
            h_temp_var := FC_RESCHEDULE(h_tsk_assignment_id, h_start_date, h_total, h_temp_var, h_contour
                            , h_rec_user, 0, Coalesce(h_percent_units, 0));
            --trace_printf("WP_FC_RESCHEDULE: h_temp_var: %d\n",h_temp_var); //trace_flush();
            
            If Coalesce(h_temp_var, 0) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_ASSIGN_PERCENT', 626+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_U_ASSIGN_PERCENT(627) : cplan_fd.sql(6415) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
            End If;
         End If;
      End If;
      
      If Coalesce(h_last_day, 0) = 0 Then
         BEGIN
            SELECT TO_DATE(SubStr(h_last_submitted, 1, 10)) + 1      INTO h_last_submitted FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 642+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(643) : cplan_fd.sql(6431) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_total := Coalesce(h_work_todate, 0);
      Else
         h_total := Coalesce(h_work_todate, 0) - Coalesce(h_last_day, 0);
      End If;
      
      OPEN CFW_AWP;
      
      LOOP
         FETCH CFW_AWP INTO h_minutes, h_dayofyear, h_bitID, h_current_weekday;
         IF CFW_AWP%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         --h_sched_min = h_minutes;
         --trace_printf("*** h_dayofyear: %s, h_minutes: %d, h_week: %d, h_current_weekday: %d\n",h_dayofyear,h_minutes,h_week,h_current_weekday);
         
         If Coalesce(h_week, 0) = 0 Then
            h_week := Coalesce(h_db_day_of_week, 0) + 1;
            If Coalesce(h_current_weekday, 0) >= Coalesce(h_week, 0) Then
               h_temp_var := Coalesce(h_current_weekday, 0) - Coalesce(h_week, 0);
            Else
               h_temp_var := 7 + Coalesce(h_current_weekday, 0) - Coalesce(h_week, 0);
            End If;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) - h_temp_var     
               INTO h_week_of
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 686+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(687) : cplan_fd.sql(6475) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_next_week_of FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 699+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(700) : cplan_fd.sql(6488) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT
               SUNDAY_REGULAR,
               SUNDAY_SPECIAL,
               MONDAY_REGULAR,
               MONDAY_SPECIAL,
               TUESDAY_REGULAR,
               TUESDAY_SPECIAL,
               WEDNESDAY_REGULAR,
               WEDNESDAY_SPECIAL,
               THURSDAY_REGULAR,
               THURSDAY_SPECIAL,
               FRIDAY_REGULAR,
               FRIDAY_SPECIAL,
               SATURDAY_REGULAR,
               SATURDAY_SPECIAL
               INTO
               h_sunday_regular,
               h_sunday_special,
               h_monday_regular,
               h_monday_special,
               h_tuesday_regular,
               h_tuesday_special,
               h_wednesday_regular,
               h_wednesday_special,
               h_thursday_regular,
               h_thursday_special,
               h_friday_regular,
               h_friday_special,
               h_saturday_regular,
               h_saturday_special
               FROM SUMMARY_TIMESHEETS
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND WEEK_OF = TO_DATE(SubStr(h_week_of, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 744+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(745) : cplan_fd.sql(6533) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("1.*** SC_STATE: %d, h_week_of: %s,h_current_weekday: %d\n",SC_STATE,h_week_of,h_current_weekday);
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               /*BEGIN_SQL
                                       CLOSE CFW_AWP WITH RELEASE;
                                   END_SQL_CHECK;
                                   return SC_STATE; */
               h_monday_regular := 0;
               h_tuesday_regular := 0;
               h_wednesday_regular := 0;
               h_thursday_regular := 0;
               h_friday_regular := 0;
               h_saturday_regular := 0;
               h_sunday_regular := 0;
            End If;
         ElsIf PMO_ORASTD.strcmp(h_dayofyear, h_next_week_of) >= 0 Then
            h_weekly_regular_total := Coalesce(h_monday_regular, 0) + Coalesce(h_tuesday_regular, 0) + Coalesce(
                                   h_wednesday_regular, 0) + Coalesce(h_thursday_regular, 0) 
                                   + Coalesce(h_friday_regular, 0) + Coalesce(
                                   h_saturday_regular, 0) + Coalesce(h_sunday_regular, 0);
            If Coalesce(h_effort, 0) > Coalesce(h_sched_work, 0) Then
               h_etc := 0;
               h_milestone_percent := 10000;
            Else
               h_etc := Coalesce(h_etc, 0) - Coalesce(h_weekly_regular_total, 0);
               h_milestone_percent := Trunc((1 - (Coalesce(h_etc, 0) / Coalesce(h_effort, 0))) * 10000.00);
            End If;
            
            PMO_DB2M.SC_STATE := FC_U_SUMMARY_TIMESHEETS(h_call_type, h_last_week, h_project_id, h_res_id
                                             , h_tsk_assignment_id, h_week_of, h_null_str
                                             , h_null_str, h_tsk_assignment_id, h_rec_user
                                             , h_billable, h_capital_expense, h_monday_regular
                                             , h_monday_special, h_tuesday_regular
                                             , h_tuesday_special, h_wednesday_regular
                                             , h_wednesday_special, h_thursday_regular
                                             , h_thursday_special, h_friday_regular
                                             , h_friday_special, h_saturday_regular
                                             , h_saturday_special, h_sunday_regular
                                             , h_sunday_special, h_weekly_regular_total
                                             , h_weekly_special_total, h_sched_work
                                             , h_baseline1_work, h_baseline_effort_week, h_ta_etc
                                             , h_ta_ms, h_etc, h_work_percent, h_exchange_rate
                                             , h_actual_regular_cost, h_actual_regular_rate
                                             , h_actual_special_cost, h_actual_special_rate
                                             , h_baseline_regular_cost, h_baseline_regular_rate
                                             , h_baseline_special_cost, h_baseline_special_rate);
            
            h_ta_ms := Coalesce(h_milestone_percent, 0);
            h_ta_etc := Coalesce(h_etc, 0);
            
            h_monday_regular := 0;
            h_tuesday_regular := 0;
            h_wednesday_regular := 0;
            h_thursday_regular := 0;
            h_friday_regular := 0;
            h_saturday_regular := 0;
            h_sunday_regular := 0;
            
            --the week changed, reinitialize the week_of  next_week
            h_week_of := h_next_week_of;
            BEGIN
               SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_next_week_of FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 815+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(816) : cplan_fd.sql(6604) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("1.1** h_dayofyear = %s, h_minutes = %d, h_total: %d\n", h_dayofyear,h_minutes,h_total);
         
         i_prev_min := Coalesce(h_total, 0);
         h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
         If Coalesce(h_total, 0) <= Coalesce(h_new_actual, 0) Then
            If Coalesce(h_total, 0) = Coalesce(h_new_actual, 0) Then
               EXIT;
            End If;
         Else
            /*if (h_new_actual <= h_minutes)
                            {
                                h_minutes = h_new_actual;
                            }
                            else
                            {
                                h_minutes = h_new_actual - i_prev_min;
                            }*/
            h_minutes := Coalesce(h_new_actual, 0) - i_prev_min;
            --trace_printf("### h_total = %d, i_prev_min: %d, h_temp_var: %d, h_minutes = %d\n", h_total,i_prev_min,h_temp_var,h_minutes);
            
            EXIT;
         End If;
         --trace_printf("1.2** h_dayofyear = %s, h_minutes = %d, h_total = %d\n", h_dayofyear,h_minutes,h_total);
         
         CASE h_current_weekday
            
            WHEN 1 THEN
               h_sunday_regular := Coalesce(h_minutes, 0);
            
            WHEN 2 THEN
               h_monday_regular := Coalesce(h_minutes, 0);
            
            WHEN 3 THEN
               h_tuesday_regular := Coalesce(h_minutes, 0);
            
            WHEN 4 THEN
               h_wednesday_regular := Coalesce(h_minutes, 0);
            
            WHEN 5 THEN
               h_thursday_regular := Coalesce(h_minutes, 0);
            
            WHEN 6 THEN
               h_friday_regular := Coalesce(h_minutes, 0);
            
            WHEN 7 THEN
               h_saturday_regular := Coalesce(h_minutes, 0);
            ELSE
               NULL;
         END CASE;
      END LOOP;
      
      BEGIN
         CLOSE CFW_AWP ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 879+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(880) : cplan_fd.sql(6668) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      --trace_printf("***** h_work_todate > h_new_actual *** h_last_submitted: %d %s\n",h_last_submitted_Ind,h_last_submitted);
      --timeLog (file,"3_* FC_U_ASSIGN_PERCENT Old Actual >= New Actual");
      
      BEGIN
         SELECT MAX(WEEK_OF) INTO h_week_apv FROM TIMESHEET_APPROVAL_STATUS
         WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id
         AND CURRENT_STATE IN (PMO_CNST.SC_APV_STATE_SUBMITTED,PMO_CNST.SC_APV_STATE_RM_APPROVED,PMO_CNST.SC_APV_STATE_PM_APPROVED);
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 897+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(898) : cplan_fd.sql(6686) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("+++> SC_STATE: %d, h_week_apv: %d %s, h_res_id: %d %s\n", SC_STATE,h_week_apv_Ind,h_week_apv,h_res_id_Ind,h_res_id);
      
      h_total := Coalesce(h_work_todate, 0);
      
      OPEN CBW_AWP;
      
      LOOP
         FETCH CBW_AWP INTO h_minutes, h_dayofyear, h_bitID, h_current_weekday;
         IF CBW_AWP%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         --h_sched_min = h_minutes;
         --trace_printf("h_dayofyear: %s, h_minutes: %d, h_week: %d, h_current_weekday: %d\n", h_dayofyear,h_minutes,h_week,h_current_weekday);
         
         If Coalesce(h_week, 0) = 0 Then
            If Coalesce(h_last_day, 0) > 0 Then
               h_minutes := Coalesce(h_last_day, 0);
            End If;
            
            h_week := Coalesce(h_db_day_of_week, 0) + 1;
            If Coalesce(h_current_weekday, 0) >= Coalesce(h_week, 0) Then
               h_temp_var := Coalesce(h_current_weekday, 0) - Coalesce(h_week, 0);
            Else
               h_temp_var := 7 + Coalesce(h_current_weekday, 0) - Coalesce(h_week, 0);
            End If;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) - h_temp_var     
               INTO h_week_of
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 943+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(944) : cplan_fd.sql(6732) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_week_of, 1, 10)) - 7      INTO h_next_week_of FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 956+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(957) : cplan_fd.sql(6745) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("1_* SC_STATE: %d, h_week_of: %s, h_next_week_of: %s\n", SC_STATE,h_week_of,h_next_week_of);
            
            If PMO_ORASTD.strcmp(h_week_apv, h_week_of) >= 0 Then
               BEGIN
                  CLOSE CBW_AWP ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 971+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(972) : cplan_fd.sql(6760) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_ASSIGN_PERCENT', 976+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_U_ASSIGN_PERCENT(977) : cplan_fd.sql(6765) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN 0;
            Else
               BEGIN
                  SELECT
                  SUNDAY_REGULAR,
                  SUNDAY_SPECIAL,
                  MONDAY_REGULAR,
                  MONDAY_SPECIAL,
                  TUESDAY_REGULAR,
                  TUESDAY_SPECIAL,
                  WEDNESDAY_REGULAR,
                  WEDNESDAY_SPECIAL,
                  THURSDAY_REGULAR,
                  THURSDAY_SPECIAL,
                  FRIDAY_REGULAR,
                  FRIDAY_SPECIAL,
                  SATURDAY_REGULAR,
                  SATURDAY_SPECIAL
                  INTO
                  h_sunday_regular,
                  h_sunday_special,
                  h_monday_regular,
                  h_monday_special,
                  h_tuesday_regular,
                  h_tuesday_special,
                  h_wednesday_regular,
                  h_wednesday_special,
                  h_thursday_regular,
                  h_thursday_special,
                  h_friday_regular,
                  h_friday_special,
                  h_saturday_regular,
                  h_saturday_special
                  FROM SUMMARY_TIMESHEETS
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                  AND WEEK_OF = TO_DATE(SubStr(h_week_of, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1020+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1021) : cplan_fd.sql(6809) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                  /*BEGIN_SQL
                                              CLOSE CBW_AWP WITH RELEASE;
                                          END_SQL_CHECK;
                                          return SC_STATE;*/
                  h_monday_regular := 0;
                  h_tuesday_regular := 0;
                  h_wednesday_regular := 0;
                  h_thursday_regular := 0;
                  h_friday_regular := 0;
                  h_saturday_regular := 0;
                  h_sunday_regular := 0;
               End If;
            End If;
         ElsIf PMO_ORASTD.strcmp(h_dayofyear, h_week_of) < 0 Then
            If PMO_ORASTD.strcmp(h_week_apv, h_next_week_of) = 0 Then
               h_new_actual := Coalesce(h_total, 0);
               h_work_percent := Trunc(Coalesce(h_new_actual, 0) / Coalesce(h_effort, 0) * 10000.00 + 0.5);
               h_minutes := 0;
               --trace_printf("3_* SC_STATE: %d, h_week_of: %s, h_week_apv: %s, h_new_actual: %d, h_work_percent: %d\n", SC_STATE,h_week_of,h_week_apv,h_new_actual,h_work_percent);
               EXIT;
            Else
               h_weekly_regular_total := Coalesce(h_monday_regular, 0) + Coalesce(h_tuesday_regular, 0) + Coalesce(
                                   h_wednesday_regular, 0) + Coalesce(h_thursday_regular, 0) 
                                   + Coalesce(h_friday_regular, 0) + Coalesce(
                                   h_saturday_regular, 0) + Coalesce(h_sunday_regular, 0);
               If Coalesce(h_effort, 0) > Coalesce(h_sched_work, 0) Then
                  h_etc := 0;
                  h_milestone_percent := 10000;
               Else
                  h_etc := Coalesce(h_etc, 0) - Coalesce(h_weekly_regular_total, 0);
                  h_milestone_percent := Trunc((1 - (Coalesce(h_etc, 0) / Coalesce(h_effort, 0))) * 10000.00);
               End If;
               
               PMO_DB2M.SC_STATE := FC_U_SUMMARY_TIMESHEETS(h_call_type, h_last_week, h_project_id, h_res_id
                                             , h_tsk_assignment_id, h_week_of, h_null_str
                                             , h_null_str, h_tsk_assignment_id, h_rec_user
                                             , h_billable, h_capital_expense, h_monday_regular
                                             , h_monday_special, h_tuesday_regular
                                             , h_tuesday_special, h_wednesday_regular
                                             , h_wednesday_special, h_thursday_regular
                                             , h_thursday_special, h_friday_regular
                                             , h_friday_special, h_saturday_regular
                                             , h_saturday_special, h_sunday_regular
                                             , h_sunday_special, h_weekly_regular_total
                                             , h_weekly_special_total, h_sched_work
                                             , h_baseline1_work, h_baseline_effort_week, h_ta_etc
                                             , h_ta_ms, h_etc, h_work_percent, h_exchange_rate
                                             , h_actual_regular_cost, h_actual_regular_rate
                                             , h_actual_special_cost, h_actual_special_rate
                                             , h_baseline_regular_cost, h_baseline_regular_rate
                                             , h_baseline_special_cost, h_baseline_special_rate);
               
               h_ta_ms := Coalesce(h_milestone_percent, 0);
               h_ta_etc := Coalesce(h_etc, 0);
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  BEGIN
                     SELECT
                     SUNDAY_REGULAR,
                     SUNDAY_SPECIAL,
                     MONDAY_REGULAR,
                     MONDAY_SPECIAL,
                     TUESDAY_REGULAR,
                     TUESDAY_SPECIAL,
                     WEDNESDAY_REGULAR,
                     WEDNESDAY_SPECIAL,
                     THURSDAY_REGULAR,
                     THURSDAY_SPECIAL,
                     FRIDAY_REGULAR,
                     FRIDAY_SPECIAL,
                     SATURDAY_REGULAR,
                     SATURDAY_SPECIAL
                     INTO
                     h_sunday_regular,
                     h_sunday_special,
                     h_monday_regular,
                     h_monday_special,
                     h_tuesday_regular,
                     h_tuesday_special,
                     h_wednesday_regular,
                     h_wednesday_special,
                     h_thursday_regular,
                     h_thursday_special,
                     h_friday_regular,
                     h_friday_special,
                     h_saturday_regular,
                     h_saturday_special
                     FROM SUMMARY_TIMESHEETS
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                     AND WEEK_OF = TO_DATE(SubStr(h_next_week_of, 1, 10));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1121+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1122) : cplan_fd.sql(6910) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                     /*BEGIN_SQL
                                                     CLOSE CBW_AWP WITH RELEASE;
                                                 END_SQL_CHECK;
                                                 return SC_STATE;*/
                     h_monday_regular := 0;
                     h_tuesday_regular := 0;
                     h_wednesday_regular := 0;
                     h_thursday_regular := 0;
                     h_friday_regular := 0;
                     h_saturday_regular := 0;
                     h_sunday_regular := 0;
                  End If;
               End If;
               
               --the week changed, reinitialize the week_of  next_week
               h_week_of := h_next_week_of;
               
               BEGIN
                  SELECT TO_DATE(SubStr(h_week_of, 1, 10)) - 7      INTO h_next_week_of FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1152+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1153) : cplan_fd.sql(6941) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("2_* SC_STATE: %d, h_week_of: %s, h_next_week_of: %s\n", SC_STATE,h_week_of,h_next_week_of);
            End If;
         End If;
         
         --          i_prev_min = h_total;
         h_total := Coalesce(h_total, 0) - Coalesce(h_minutes, 0);
         --trace_printf("h_dayofyear = %s, h_total = %d, h_minutes = %d\n", h_dayofyear,h_total,h_minutes);
         If Coalesce(h_total, 0) >= Coalesce(h_new_actual, 0) Then
            h_minutes := 0;
            If Coalesce(h_total, 0) = Coalesce(h_new_actual, 0) Then
               EXIT;
            End If;
         Else
            --              h_temp_var  = i_prev_min - h_new_actual;
            h_temp_var := Coalesce(h_new_actual, 0) - Coalesce(h_total, 0);
            If Coalesce(h_temp_var, 0) < Coalesce(h_minutes, 0) Then
               h_minutes := Coalesce(h_temp_var, 0);
            Else
               h_minutes := 0;
            End If;
            
            EXIT;
         End If;
         --trace_printf("h_current_weekday: %d, h_temp_var = %d, h_total = %d, h_minutes = %d\n", h_current_weekday,h_temp_var,h_total,h_minutes);
         
         CASE h_current_weekday
            
            WHEN 1 THEN
               h_sunday_regular := Coalesce(h_minutes, 0);
            
            WHEN 2 THEN
               h_monday_regular := Coalesce(h_minutes, 0);
            
            WHEN 3 THEN
               h_tuesday_regular := Coalesce(h_minutes, 0);
            
            WHEN 4 THEN
               h_wednesday_regular := Coalesce(h_minutes, 0);
            
            WHEN 5 THEN
               h_thursday_regular := Coalesce(h_minutes, 0);
            
            WHEN 6 THEN
               h_friday_regular := Coalesce(h_minutes, 0);
            
            WHEN 7 THEN
               h_saturday_regular := Coalesce(h_minutes, 0);
            ELSE
               NULL;
         END CASE;
      END LOOP;
      
      BEGIN
         CLOSE CBW_AWP ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1216+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1217) : cplan_fd.sql(7005) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --trace_printf("h_current_weekday: %d, h_minutes: %d, h_dayofyear = %d:%s, h_tsk_assignment_id = %d:%s\n",h_current_weekday,h_minutes,h_dayofyear_Ind,h_dayofyear,h_tsk_assignment_id_Ind,h_tsk_assignment_id);
   --trace_printf("h_work_percent: %d, h_etc: %d, h_total = %d, h_new_actual: %d, h_week_of: %s\n", h_work_percent,h_etc,h_total,h_new_actual,h_week_of);
   --timeLog (file,"4_* FC_U_ASSIGN_PERCENT End of While Loop");
   
   If h_dayofyear IS NOT NULL Then
      CASE h_current_weekday
         WHEN 1 THEN
            h_sunday_regular := Coalesce(h_minutes, 0);
         
         WHEN 2 THEN
            h_monday_regular := Coalesce(h_minutes, 0);
         
         WHEN 3 THEN
            h_tuesday_regular := Coalesce(h_minutes, 0);
         
         WHEN 4 THEN
            h_wednesday_regular := Coalesce(h_minutes, 0);
         
         WHEN 5 THEN
            h_thursday_regular := Coalesce(h_minutes, 0);
         
         WHEN 6 THEN
            h_friday_regular := Coalesce(h_minutes, 0);
         
         WHEN 7 THEN
            h_saturday_regular := Coalesce(h_minutes, 0);
         ELSE
            NULL;
      END CASE;
      
      h_SF := PMO_CNST.SC_SF;
      If Coalesce(h_work_percent, 0) = 10000 Then
         h_etc := 0;
         h_effort := Coalesce(h_new_actual, 0);
         h_SF := PMO_CNST.SC_AF;
      Else
         h_etc := Coalesce(h_effort, 0) - Coalesce(h_new_actual, 0);
      End If;
      
      If Coalesce(h_etc, 0) = 0 Then
         h_work_percent := 10000;
         h_SF := PMO_CNST.SC_AF;
      End If;
      
      h_weekly_regular_total := Coalesce(h_monday_regular, 0) + Coalesce(h_tuesday_regular, 0) + Coalesce(
                                   h_wednesday_regular, 0) + Coalesce(h_thursday_regular, 0) 
                                   + Coalesce(h_friday_regular, 0) + Coalesce(
                                   h_saturday_regular, 0) + Coalesce(h_sunday_regular, 0);
      If Coalesce(h_effort, 0) > Coalesce(h_sched_work, 0) Then
         h_etc := 0;
         h_milestone_percent := 10000;
      Else
         h_milestone_percent := Trunc((1 - (Coalesce(h_etc, 0) / Coalesce(h_effort, 0))) * 10000.00);
      End If;
      --trace_printf("+++> h_week_of: %s, h_weekly_regular_total: %d, h_milestone_percent: %d, h_ta_ms: %d, h_etc: %d\n", h_week_of,h_weekly_regular_total,h_milestone_percent,h_ta_ms,h_etc);
      
      PMO_DB2M.SC_STATE := FC_U_SUMMARY_TIMESHEETS(h_call_type, h_last_week, h_project_id, h_res_id
                                             , h_tsk_assignment_id, h_week_of, h_null_str
                                             , h_null_str, h_tsk_assignment_id, h_rec_user
                                             , h_billable, h_capital_expense, h_monday_regular
                                             , h_monday_special, h_tuesday_regular
                                             , h_tuesday_special, h_wednesday_regular
                                             , h_wednesday_special, h_thursday_regular
                                             , h_thursday_special, h_friday_regular
                                             , h_friday_special, h_saturday_regular
                                             , h_saturday_special, h_sunday_regular
                                             , h_sunday_special, h_weekly_regular_total
                                             , h_weekly_special_total, h_sched_work
                                             , h_baseline1_work, h_baseline_effort_week, h_ta_etc
                                             , h_ta_ms, h_etc, h_work_percent, h_exchange_rate
                                             , h_actual_regular_cost, h_actual_regular_rate
                                             , h_actual_special_cost, h_actual_special_rate
                                             , h_baseline_regular_cost, h_baseline_regular_rate
                                             , h_baseline_special_cost, h_baseline_special_rate);
      
      --trace_printf("*** h_new_actual %d, h_last_day: %d\n",h_new_actual,h_last_day); //trace_flush();
      If Coalesce(h_new_actual, 0) = 0 Then
         --timeLog (file,"5_* FC_U_ASSIGN_PERCENT UPDATE TASK_ASSIGNMENTS  run U_TSKRES2");
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7,
            LAST_SCHED_MIN      = 0,
            WORK_TODATE         = 0,
            MILESTONE_PERCENT   = 0,
            ACTUAL_START        = NuLL,
            ACTUAL_FINISH       = NuLL,
            ACTUAL_DURATION     = 0,
            LAST_SUBMITTED      = NuLL,
            ETC                 = SCHED_WORK,
            ROLLUP_BITS         = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
            SD                  = PMO_CNST.SC_SS,
            FD                  = PMO_CNST.SC_SF
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1323+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1324) : cplan_fd.sql(7112) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         /*BEGIN_SQL
                         SELECT PROJECT_ASSIGNMENT_ID, BAK_START_DT, BAK_FINISH_DT, BAK_WORK, BAK_ASSIGN_STRING,
                             C_EFFORT, T_UNIT, C_UNIT, RESOURCE_TYPE,
                             DATE(SCHED_START), DATE(SCHED_FINISH), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                         INTO HOST(h_prj_assignment_id), HOST(h_start_date), HOST(h_finish_date), HOST(h_effort), HOST(h_assign_string),
                             HOST(h_work), HOST(h_eunit), HOST(h_unit), HOST(h_res_type),
                             HOST(h_SD), HOST(h_FD), HOST(h_billable), HOST(h_capital_expense), HOST(h_reg_cost), HOST(h_reg_sale)
                         FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
                     END_SQL;
         //trace_printf("*** h_effort %d, h_per_day: %d, h_start_date %s, h_finish_date %s\n",h_effort, h_per_day,h_start_date, h_finish_date);
         
                     InitializeHost(SC_PARAM(h_tph_delete), -1);
                     SC_STATE = FC_TPHASE_SCHED_ASSIGN(
                                 SC_PARAM(h_project_id),
                                 SC_PARAM(h_task_id),
                                 SC_PARAM(h_tsk_assignment_id),
                                 SC_PARAM(h_SD),
                                 SC_PARAM(h_FD),
                                 SC_PARAM(h_reg_cost),
                                 SC_PARAM(h_reg_sale),
                                 SC_PARAM(h_tph_delete),
                                 SC_PARAM(h_capital_expense),
                                 SC_PARAM(h_billable));
         
                     h_temp_var = FC_DEL_NOACTUAL(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id));
         
                     if ( (isNotNull(SC_NULL_INDICATOR(h_assign_string)))  (strlen(h_assign_string) > 0) )
                     {
                         if ( isNotNull(SC_NULL_INDICATOR(h_prj_assignment_id) ))
                             h_temp_var = (1 | SC_BIT_29_30);
                         else
                             h_temp_var = (1 | SC_BIT_28);
         
                         InitializeHost(SC_PARAM(h_temp_work_type), 2);
                         InitializeHost(SC_PARAM(h_temp_contour_type), 0);
                         InitializeHost(SC_PARAM(h_temp_occurrence), 0);
                         InitializeHost(SC_PARAM(h_temp_consider), 7);
         
                         // Update TASK_ASSIGNMENTS  TASK_DAY_OF_YEAR with the original schedule
                         SC_STATE = U_TSKRES2(SC_PARAM(h_tsk_assignment_id),    SC_PARAM(h_project_id),         SC_PARAM(h_task_id),
                                              SC_PARAM(h_res_id),               SC_PARAM(h_null_str),           SC_PARAM(h_null_str),
                                              SC_PARAM(h_start_date),           SC_PARAM(h_finish_date),        SC_PARAM(h_effort),
                                              SC_PARAM(h_work),                 SC_PARAM(h_eunit),              SC_PARAM(h_unit),
                                              SC_PARAM(h_res_type),             SC_PARAM(h_temp_work_type),     SC_PARAM(h_temp_var),
                                              SC_PARAM(h_contour),              SC_PARAM(h_temp_contour_type),  SC_PARAM(h_temp_occurrence),
                                              SC_PARAM(h_temp_consider),        SC_PARAM(h_assign_string),      SC_PARAM(h_rec_user),
                                              SC_PARAM(h_per_day));
         //trace_printf("*** h_per_day: %d\n", h_per_day);
         
                         InitializeHost(SC_PARAM(h_temp_var), 1);
                         h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var), h_per_day, 0);    // fix ACTUAL_START  SCHED
                     }*/
      Else
         --timeLog (file,"6_* FC_U_ASSIGN_PERCENT UPDATE TASK_ASSIGNMENTS  run FC_I_TIMEPHASED_DELTA");
         If (Coalesce(h_work_todate, 0) > Coalesce(h_new_actual, 0)) AND (Coalesce(h_minutes, 0) = 0) Then
            BEGIN
               SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) - 1      INTO h_dayofyear FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1391+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1392) : cplan_fd.sql(7180) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         --trace_printf("***** i_keep_etc = %d, h_minutes: %d, h_etc: %d, h_work_percent: %d, h_new_actual: %d\n", i_keep_etc,h_minutes,h_etc,h_work_percent,h_new_actual);
         BEGIN
            SELECT MAX(DAY_YEAR) INTO h_dayofyear
            FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND ACTUAL_FLAG = 1;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1409+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1410) : cplan_fd.sql(7198) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("TDY_* SC_STATE: %d, h_dayofyear: %d %s\n", SC_STATE,h_dayofyear_Ind,h_dayofyear);
         
         If i_keep_etc = 1 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7,
               LAST_SCHED_MIN      = h_minutes,
               WORK_TODATE         = h_new_actual,
               MILESTONE_PERCENT   = h_work_percent,
               LAST_SUBMITTED      = h_dayofyear,
               FORECAST_START      = NuLL,
               FD                  = h_SF,
               UNITS_MSP           = CASE WHEN h_percent_units = 0 THEN UNITS_MSP ELSE h_percent_units END,
               ROLLUP_BITS         = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT))
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1437+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1438) : cplan_fd.sql(7226) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf (Coalesce(h_etc, 0) = 0) OR (Coalesce(h_work_percent, 0) = 10000) Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7,
               LAST_SCHED_MIN      = 0,
               SCHED_WORK          = h_new_actual,
               WORK_TODATE         = h_new_actual,
               MILESTONE_PERCENT   = 10000,
               LAST_SUBMITTED      = h_dayofyear,
               ETC                 = 0,
               FORECAST_START      = NuLL,
               FORECAST_FINISH     = NuLL,
               FORECAST_DURATION   = 0,
               FD                  = h_SF,
               UNITS_MSP           = CASE WHEN h_percent_units = 0 THEN UNITS_MSP ELSE h_percent_units END,
               ROLLUP_BITS         = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT))
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1467+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1468) : cplan_fd.sql(7256) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7,
               LAST_SCHED_MIN      = h_minutes,
               WORK_TODATE         = h_new_actual,
               MILESTONE_PERCENT   = h_work_percent,
               LAST_SUBMITTED      = h_dayofyear,
               ETC                 = h_etc,
               FORECAST_START      = NuLL,
               FD                  = h_SF,
               UNITS_MSP           = CASE WHEN h_percent_units = 0 THEN UNITS_MSP ELSE h_percent_units END,
               ROLLUP_BITS         = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT))
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1494+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1495) : cplan_fd.sql(7283) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         --timeLog (file,"6.1_* FC_U_ASSIGN_PERCENT run FC_I_TIMEPHASED_DELTA");
         --trace_printf("A_0_* SC_STATE: %d, h_percent_units: %d\n", SC_STATE,h_percent_units);
         /*if ( (SC_SUCCESS)  (h_minutes > 0) )
                     {
                         BEGIN_SQL
                             SELECT SCHED_MIN - ACTUAL_MIN INTO HOST(h_minutes)
                             FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
                                 AND DAY_YEAR = HOST(h_dayofyear);
                         END_SQL_CHECK;
         //trace_printf("A_1_* SC_STATE: %d, h_dayofyear: %s, h_minutes: %d\n", SC_STATE,h_dayofyear,h_minutes);
         
                         if ( (SC_SUCCESS)  (h_minutes> 0) )
                         {
                             InitializeHost(SC_PARAM(h_temp_var), SC_SCHED_BUDGET);
                             InitializeHost(SC_PARAM(h_total_h), h_minutes / 60.0);
         
                             if ( h_capital_expense == 0 )
                             {
                                 InitializeHost(SC_PARAM(h_chargecode_id), "EXPENSE_HOURS");
                             }
                             else
                             {
                                 InitializeHost(SC_PARAM(h_chargecode_id), "CAPITAL_HOURS");
                             }
         
                             SC_STATE = FC_I_TIMEPHASED_DELTA(   SC_PARAM(h_project_id),
                                                 SC_PARAM(h_task_id),
                                                 SC_PARAM(h_chargecode_id),
                                                 SC_PARAM(h_temp_var),
                                                 SC_PARAM(h_total_h),
                                                 SC_PARAM(h_dayofyear) );
         //trace_printf("H_E_* SC_STATE: %d, h_total_h: %.2f\n", SC_STATE,h_total_h);
         
                             if (h_actual_regular_rate > 0.00)
                             {
                                 if (h_capital_expense == 0)
                                     InitializeHost(SC_PARAM(h_chargecode_id), "EXPENSE_LABOR");
                                 else
                                     InitializeHost(SC_PARAM(h_chargecode_id), "CAPITAL_LABOR");
         
                                 InitializeHost(SC_PARAM(h_d_total), h_total_h * h_actual_regular_cost);
                                 SC_STATE = FC_I_TIMEPHASED_DELTA(   SC_PARAM(h_project_id),
                                                     SC_PARAM(h_task_id),
                                                     SC_PARAM(h_chargecode_id),
                                                     SC_PARAM(h_temp_var),
                                                     SC_PARAM(h_d_total),
                                                     SC_PARAM(h_dayofyear) );
                             }
         
                             if ( h_billable == 1 )
                             {
                                 InitializeHost(SC_PARAM(h_chargecode_id), "BENEFIT_HOURS");
                                 SC_STATE = FC_I_TIMEPHASED_DELTA(   SC_PARAM(h_project_id),
                                                     SC_PARAM(h_task_id),
                                                     SC_PARAM(h_chargecode_id),
                                                     SC_PARAM(h_temp_var),
                                                     SC_PARAM(h_total_h),
                                                     SC_PARAM(h_dayofyear) );
         //trace_printf("H_B_* SC_STATE: %d, h_total_h: %.2f\n", SC_STATE,h_total_h);
         
                                 if (h_actual_regular_rate > 0.00)
                                 {
                                     InitializeHost(SC_PARAM(h_chargecode_id), "BILLABLE_BENEFIT_LABOR");
                                     InitializeHost(SC_PARAM(h_d_total), h_total_h * h_actual_regular_rate);
                                     SC_STATE = FC_I_TIMEPHASED_DELTA(   SC_PARAM(h_project_id),
                                                     SC_PARAM(h_task_id),
                                                     SC_PARAM(h_chargecode_id),
                                                     SC_PARAM(h_temp_var),
                                                     SC_PARAM(h_d_total),
                                                     SC_PARAM(h_dayofyear) );
                                 }
                             }
         
                         }
                     }*/
         --timeLog (file,"6.2_* FC_U_ASSIGN_PERCENT end FC_I_TIMEPHASED_DELTA");
      End If;
      --trace_printf("SC_STATE: %d, h_SF: %d, h_dayofyear: %s, h_work_percent: %d, h_minutes: %d, h_etc: %d\n",SC_STATE,h_SF,h_dayofyear,h_work_percent,h_minutes,h_etc);
      
      h_temp_var := FC_Rollup(h_project_id);
   End If;
   --trace_printf("**_FC_U_ASSIGN_PERCENT_** SC_STATE: %d\n",SC_STATE);
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_ASSIGN_PERCENT', 1588+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_ASSIGN_PERCENT(1589) : cplan_fd.sql(7377) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_ASSIGN_PERCENT;
ALTER FUNCTION FC_U_ASSIGN_PERCENT COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_GET_DT ==
CREATE OR REPLACE FUNCTION FC_GET_DT
(
     h_start_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_bitflag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_dayofyear DATE;
   h_temp_sc_state PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_DT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_DT'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_DT(' || 
       COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_DT', 17+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_DT(18) : cplan_fd.sql(7405) : ' || SYSTIMESTAMP);
   END IF; 

   --short h_start_DT_Ind = 0;
   
   h_dayofyear := SubStr(h_start_DT, 1, 10);
   --h_dayofyear[10] = '\0';
   --trace_printf("h_dayofyear: %s, h_start_DT = %s, h_bitflag = %d\n",h_dayofyear,h_start_DT,h_bitflag);
   
   If Coalesce(h_bitflag, 0) = 0 Then
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_start_DT
         FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_DT', 36+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_DT(37) : cplan_fd.sql(7424) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED))) INTO h_start_DT
         FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_DT', 50+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_DT(51) : cplan_fd.sql(7438) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      If Coalesce(h_bitflag, 0) = 0 Then
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME('08:00') ) INTO h_start_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_DT', 67+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_DT(68) : cplan_fd.sql(7455) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME('17:00') ) INTO h_start_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_DT', 81+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_DT(82) : cplan_fd.sql(7469) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   --trace_printf("h_start_DT = %d;%s\n",h_start_DT_Ind,h_start_DT);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_DT', 91+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_DT(92) : cplan_fd.sql(7479) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_DT;
ALTER FUNCTION FC_GET_DT COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION U_LEVEL_STRING ==
CREATE OR REPLACE FUNCTION U_LEVEL_STRING
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , assign_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , sh_hrs_per_day PLS_INTEGER := 0
   , sh_work PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   hrs_per_day PLS_INTEGER := Coalesce(sh_hrs_per_day, 0);
   work PLS_INTEGER := Coalesce(sh_work, 0);
   h_prj_assignment_id VARCHAR2(32);
   h_start_D DATE;
   h_finish_D DATE;
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_min VARCHAR2(19);
   h_julian_day PLS_INTEGER := 0;
   h_jdy_start PLS_INTEGER := 0;
   h_jdy_finish PLS_INTEGER := 0;
   h_julian_start PLS_INTEGER := 0;
   h_julian_finish PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   h_tmp_string VARCHAR2(8192);
-- Declare Cursors
      CURSOR CC24c IS
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR), SCHED_MIN
         FROM LEVEL_TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id
         ORDER BY DAY_YEAR;

      CURSOR C_SDa IS
         SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_start_D, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED)) )
         FROM LEVEL_TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_start_D, 1, 10))
         ORDER BY BIT_SCHED DESC;

      CURSOR C_FDb IS
         SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_finish_D, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED)) )
         FROM LEVEL_TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_finish_D, 1, 10))
         ORDER BY BIT_SCHED;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('U_LEVEL_STRING') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('U_LEVEL_STRING'), '-- ' || SYSTIMESTAMP, ':r := U_LEVEL_STRING(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || assign_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_hrs_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_work), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_LEVEL_STRING', 59+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: U_LEVEL_STRING(60) : cplan_fd.sql(7549) : ' || SYSTIMESTAMP);
   END IF; 

   h_prj_assignment_id := assign_id;
   
   /*int ii = 0;*/
   h_assign_string := '';
   h_finish_D := '';
   h_finish_D := '';
   
   h_julian_day := 0;
   h_jdy_start := 0;
   h_jdy_finish := 0;
   h_minutes := 0;
   h_total := 0;
   
   --trace_printf("h_project_id = %s, h_prj_assignment_id = %s\n", h_project_id,h_prj_assignment_id);
   h_temp_sc_state := FC_INI_ASSIGN_STR();
   h_tmp_string := '';
   
   BEGIN
      SELECT MIN(DAY_YEAR), MAX(DAY_YEAR) INTO h_start_D, h_finish_D
      FROM LEVEL_TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_LEVEL_STRING', 88+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_LEVEL_STRING(89) : cplan_fd.sql(7578) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_start_D: %d %s, h_finish_D: %d %s\n", SC_STATE, h_start_D_Ind, h_start_D, h_finish_D_Ind,h_finish_D);
   
   BEGIN
      SELECT PMO_ORASTD.JULIAN_DAY(h_start_D), PMO_ORASTD.JULIAN_DAY(h_finish_D)
      INTO h_julian_start, h_julian_finish
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_LEVEL_STRING', 104+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_LEVEL_STRING(105) : cplan_fd.sql(7594) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN CC24c;
   
   LOOP
      FETCH CC24c INTO h_julian_day, h_minutes;
      IF CC24c%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE = %d, h_minutes = %d\n", SC_STATE,h_minutes);
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      h_temp_sc_state := FC_ASSIGN_STRING(h_julian_day, h_minutes, h_tmp_string);
      
      If (Coalesce(h_total, 0) = 0) AND (Coalesce(h_julian_start, 0) < Coalesce(h_julian_day, 0)) Then
         h_jdy_start := Coalesce(h_julian_day, 0) - Coalesce(h_julian_start, 0);
      End If;
      
      h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
   END LOOP;
   
   BEGIN
      CLOSE CC24c ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_LEVEL_STRING', 141+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_LEVEL_STRING(142) : cplan_fd.sql(7631) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_julian_day, 0) > 0 Then
      If Coalesce(h_jdy_start, 0) = 0 Then
         h_temp_sc_state := FC_GET_ASSIGN_STR(h_tmp_string);
         
         h_assign_string := h_tmp_string;
      Else
         h_assign_string := TO_CHAR(Coalesce(h_jdy_start, 0)) || '*0,';
         
         h_temp_sc_state := FC_GET_ASSIGN_STR(h_tmp_string);
         
         h_assign_string := h_assign_string || h_tmp_string;
      End If;
      
      If Coalesce(h_julian_day, 0) < Coalesce(h_julian_finish, 0) Then
         h_min := ',' || TO_CHAR(Coalesce(h_julian_finish, 0) - Coalesce(h_julian_day, 0)) || '*0';
         h_assign_string := h_assign_string || h_min;
      End If;
   Else
      h_assign_string := TO_CHAR(Coalesce(h_julian_finish, 0) - Coalesce(h_julian_start, 0) + 1) || '*0,';
   End If;
   
   If (Coalesce(Length(h_start_D), 0) > 0) AND (Coalesce(Length(h_finish_D), 0) > 0) Then
      OPEN C_SDa;
      
      FETCH C_SDa INTO h_start_date;
      IF C_SDa%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE: %d, h_start_date: %d %s\n", SC_STATE,h_start_date_Ind,h_start_date);
      
      BEGIN
         CLOSE C_SDa ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_LEVEL_STRING', 186+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_LEVEL_STRING(187) : cplan_fd.sql(7676) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C_FDb;
      
      FETCH C_FDb INTO h_finish_date;
      IF C_FDb%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE: %d, h_finish_date: %d %s\n", SC_STATE,h_finish_date_Ind,h_finish_date);
      
      BEGIN
         CLOSE C_FDb ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_LEVEL_STRING', 209+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_LEVEL_STRING(210) : cplan_fd.sql(7699) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("h_prj_assignment_id: %s, h_bitflag: %d, h_assign_string: %s\n", h_prj_assignment_id, h_bitflag, h_assign_string);
      --trace_printf("h_start_date: %d %s, h_finish_date: %d %s, h_total: %d\n", h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date,h_total);
      
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET LEVEL_ASSIGN_STRING = h_assign_string,
         LEVEL_START = h_start_date, LEVEL_FINISH = h_finish_date, LEVEL_EFFORT = h_total
         WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_LEVEL_STRING', 231+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_LEVEL_STRING(232) : cplan_fd.sql(7721) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --trace_printf("SC_STATE: %d, h_prj_assignment_id: %d:%s\n", SC_STATE,h_prj_assignment_id_Ind,h_prj_assignment_id); //trace_flush();
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_LEVEL_STRING', 239+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: U_LEVEL_STRING(240) : cplan_fd.sql(7729) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION U_LEVEL_STRING;
ALTER FUNCTION U_LEVEL_STRING COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION U_ASSIGN_STRING ==
CREATE OR REPLACE FUNCTION U_ASSIGN_STRING
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , assign_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , sh_hrs_per_day PLS_INTEGER := 0
   , sh_work PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   hrs_per_day PLS_INTEGER := Coalesce(sh_hrs_per_day, 0);
   work PLS_INTEGER := Coalesce(sh_work, 0);
   h_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_dayofyear DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_min VARCHAR2(19);
   h_julian_day PLS_INTEGER := 0;
   h_jdy_start PLS_INTEGER := 0;
   h_jdy_finish PLS_INTEGER := 0;
   h_julian_start PLS_INTEGER := 0;
   h_julian_finish PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_actual_min PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_tsk_prj PLS_INTEGER := 0;
   h_available PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_actual_flag PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_u_units PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   h_tmp_string VARCHAR2(8192);
   i_max_act PLS_INTEGER NOT NULL := 0;
   i_max_sched PLS_INTEGER NOT NULL := 0;
   i_u_dates PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR CC24b IS
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR), DAY_YEAR, SCHED_MIN, ACTUAL_MIN, ACTUAL_FLAG
         FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID=h_prj_assignment_id
         ORDER BY DAY_YEAR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('U_ASSIGN_STRING') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('U_ASSIGN_STRING'), '-- ' || SYSTIMESTAMP, ':r := U_ASSIGN_STRING(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || assign_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_hrs_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_work), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_ASSIGN_STRING', 68+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: U_ASSIGN_STRING(69) : cplan_fd.sql(7808) : ' || SYSTIMESTAMP);
   END IF; 

   h_prj_assignment_id := assign_id;
   
   /*int ii = 0;*/
   h_assign_string := '';
   h_dayofyear := '';
   h_finish_D := '';
   h_work := 0;
   h_etc := work;
   h_temp_var := 0;
   h_percent_units := 0;
   h_u_units := 0;
   
   h_jdy_start := 0;
   h_jdy_finish := 0;
   h_minutes := 0;
   h_count := 0;
   h_bit_minutes := 0;
   
   --trace_printf("STR_0_* h_bitflag: %d\n", h_bitflag);
   If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_29 Then
      If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_30 Then
         h_percent_units := BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_30);
      Else
         i_u_dates := 1;
         h_percent_units := BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_29);
      End If;
      
      h_bitflag := 4;
      If hrs_per_day = 0 Then
         hrs_per_day := 480;
      End If;
      
      h_u_units := 1;
   ElsIf Coalesce(h_bitflag, 0) > PMO_CNST.SC_0000FFFF Then
      h_percent_units := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_FFFF0000)), 15);
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_0000FFFF));
      h_u_units := 1;
   End If;
   --trace_printf("STR_1_* h_percent_units: %d, h_bitflag: %d, h_u_units: %d\n", h_percent_units,h_bitflag,h_u_units);
   
   BEGIN
      SELECT Trunc(BIT_FLAG), CASE WHEN PROJECT_ID = TASK_ID THEN 0 ELSE 1 END, BEST, WORK_TODATE, ETC, SCHED_WORK, C_EFFORT,
      CASE WHEN h_percent_units = 0 THEN UNITS_MSP ELSE h_percent_units END, PMO_ORASTD.JULIAN_DAY(UNRESOLVED_START), PMO_ORASTD.JULIAN_DAY(UNRESOLVED_FINISH)
      INTO h_type, h_tsk_prj, h_count, h_work_todate, h_remaining, h_sched_work, h_work,
      h_percent_units, h_julian_start, h_julian_finish
      FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 123+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(124) : cplan_fd.sql(7863) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_prj_assignment_id = %s, h_work_todate = %d, h_remaining = %d\nh_dayofyear = %d %s, h_tsk_prj = %d, h_count = %d\n", SC_STATE,h_prj_assignment_id,h_work_todate,h_remaining,h_dayofyear_Ind,h_dayofyear,h_tsk_prj,h_count);
   --trace_printf("STR_A_* h_percent_units: %d\n", h_percent_units);
   
   If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_ASSIGN_STRING', 132+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: U_ASSIGN_STRING(133) : cplan_fd.sql(7872) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_DB2M.SC_STATE;
   End If;
   
   If (Coalesce(h_u_units, 0) = 1) AND (Coalesce(h_work, 0) > 0) AND (hrs_per_day > 0) AND ((Coalesce(
             h_percent_units, 0) = 0) OR (Coalesce(h_percent_units, 0) = 10000)) Then
      h_percent_units := Trunc((Coalesce(h_work, 0) / hrs_per_day) * 10000.00);
      --trace_printf("STR_2_* h_percent_units: %d, h_work: %d, hrs_per_day: %d\n", h_percent_units,h_minutes,hrs_per_day);
   End If;
   
   If Coalesce(h_tsk_prj, 0) = 1 Then
      BEGIN
         SELECT MIN(DAY_YEAR),MAX(DAY_YEAR) INTO h_dayofyear,h_finish_D
         FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 153+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(154) : cplan_fd.sql(7893) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS OR (h_dayofyear IS NULL) 
            OR (h_finish_D IS NULL) Then
         h_assign_string := ',' || TO_CHAR(Coalesce(h_julian_finish, 0) - Coalesce(h_julian_start, 0)) 
                   || '*0';
         
         If Coalesce(h_type, 0) = 2 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET SCHED_WORK = 0, ASSIGN_STRING = h_assign_string
               WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 177+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(178) : cplan_fd.sql(7917) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET PLANNED_WORK = 0, ASSIGN_STRING = h_assign_string
               WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 195+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(196) : cplan_fd.sql(7935) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_ASSIGN_STRING', 202+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: U_ASSIGN_STRING(203) : cplan_fd.sql(7942) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_DB2M.SC_STATE;
      End If;
   Else
      BEGIN
         SELECT MIN(DAY_YEAR),MAX(DAY_YEAR),SUM(SCHED_MIN) INTO h_dayofyear,h_finish_D,h_count
         FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 216+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(217) : cplan_fd.sql(7956) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS OR (h_dayofyear IS NULL) 
            OR (h_finish_D IS NULL) Then
         BEGIN
            SELECT TO_DATE(SubStr(SCHED_START, 1, 10)),TO_DATE(SubStr(SCHED_FINISH, 1, 10)) INTO h_dayofyear,h_finish_D
            FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 232+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(233) : cplan_fd.sql(7972) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   --trace_printf("h_dayofyear_Ind = %d, h_dayofyear = %s, h_finish_D_Ind = %d, h_finish_D = %s\n", h_dayofyear_Ind,h_dayofyear,h_finish_D_Ind,h_finish_D);
   
   If h_count IS NULL Then
      h_count := 0;
   End If;
   
   h_temp_sc_state := FC_INI_ASSIGN_STR();
   h_tmp_string := '';
   
   BEGIN
      SELECT PMO_ORASTD.JULIAN_DAY(h_dayofyear), PMO_ORASTD.JULIAN_DAY(h_finish_D)
      INTO h_julian_start, h_julian_finish
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 258+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(259) : cplan_fd.sql(7998) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("SC_STATE = %d,h_dayofyear_Ind = %d, h_dayofyear = %s, h_finish_D_Ind = %d, h_finish_D = %s\n", SC_STATE,h_dayofyear_Ind,h_dayofyear,h_finish_D_Ind,h_finish_D);
   
   h_julian_day := 0;
   h_total := 0;
   h_work := 0;
   
   OPEN CC24b;
   
   LOOP
      FETCH CC24b INTO h_julian_day, h_start_D, h_minutes, h_actual_min, h_actual_flag;
      IF CC24b%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE = %d, h_minutes = %d, h_actual_flag = %d\n", SC_STATE,h_minutes,h_actual_flag);
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      If Coalesce(h_actual_flag, 0) = 1 Then
         h_work := Coalesce(h_work, 0) + Coalesce(h_actual_min, 0);
         If Coalesce(h_actual_min, 0) > i_max_act Then
            i_max_act := Coalesce(h_actual_min, 0);
         End If;
      End If;
      
      h_temp_sc_state := FC_ASSIGN_STRING(h_julian_day, h_minutes, h_tmp_string);
      
      If Coalesce(h_total, 0) = 0 Then
         If Coalesce(h_julian_start, 0) < Coalesce(h_julian_day, 0) Then
            h_jdy_start := Coalesce(h_julian_day, 0) - Coalesce(h_julian_start, 0);
         ElsIf Coalesce(h_julian_start, 0) > Coalesce(h_julian_day, 0) Then
            h_dayofyear := h_start_D;
         End If;
      End If;
      
      h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
      
      If Coalesce(h_minutes, 0) > i_max_sched Then
         i_max_sched := Coalesce(h_minutes, 0);
      End If;
   END LOOP;
   
   BEGIN
      CLOSE CC24b ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 316+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(317) : cplan_fd.sql(8056) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_julian_day, 0) > 0 Then
      If Coalesce(h_jdy_start, 0) = 0 Then
         h_temp_sc_state := FC_GET_ASSIGN_STR(h_tmp_string);
         
         h_assign_string := h_tmp_string;
      Else
         h_assign_string := TO_CHAR(Coalesce(h_jdy_start, 0)) || '*0,';
         
         h_temp_sc_state := FC_GET_ASSIGN_STR(h_tmp_string);
         
         h_assign_string := h_assign_string || h_tmp_string;
      End If;
      
      If Coalesce(h_julian_day, 0) < Coalesce(h_julian_finish, 0) Then
         h_min := ',' || TO_CHAR(Coalesce(h_julian_finish, 0) - Coalesce(h_julian_day, 0)) || '*0';
         h_assign_string := h_assign_string || h_min;
      End If;
   ElsIf Coalesce(h_tsk_prj, 0) = 0 Then
      h_assign_string := TO_CHAR(Coalesce(h_julian_finish, 0) - Coalesce(h_julian_start, 0) + 1) || '*0,';
      
      If Coalesce(h_type, 0) = 2 Then
         BEGIN
            SELECT MIN(SCHED_START), MAX(SCHED_FINISH) INTO h_start_date, h_finish_date
            FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND TASK_ID <> PROJECT_ID AND PROJECT_ASSIGNMENT_ID = h_prj_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 352+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(353) : cplan_fd.sql(8092) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT MIN(PLANNED_START), MAX(PLANNED_FINISH) INTO h_start_date, h_finish_date
            FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND TASK_ID <> PROJECT_ID AND PROJECT_ASSIGNMENT_ID = h_prj_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 367+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(368) : cplan_fd.sql(8107) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("SC_STATE: %d, h_type: %d, h_start_date: %d %s, h_finish_date: %d %s\n", SC_STATE,h_type,h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date);
   End If;
   
   --trace_printf("h_julian_day: %d, h_dayofyear = %s, h_finish_D = %s\n", h_julian_day,h_dayofyear,h_finish_D);
   If (Coalesce(Length(h_dayofyear), 0) > 0) AND (Coalesce(Length(h_finish_D), 0) > 0) Then
      If Coalesce(h_julian_day, 0) > 0 Then
         h_start_time := '08:00:00';
         h_finish_time := '17:00:00';
         
         BEGIN
            SELECT PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED) INTO h_start_time FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 390+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(391) : cplan_fd.sql(8130) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED) INTO h_finish_time
            FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR=TO_DATE(SubStr(h_finish_D, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 405+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(406) : cplan_fd.sql(8145) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("h_start_time = %s, h_finish_time = %s\n", h_start_time,h_finish_time);
         
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_start_time)), PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)), PMO_ORASTD.TIME(h_finish_time))
            INTO h_start_date, h_finish_date
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 421+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(422) : cplan_fd.sql(8161) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --trace_printf("h_prj_assignment_id: %s, h_bitflag: %d, h_type: %d, h_tsk_prj: %d\n", h_prj_assignment_id,h_bitflag, h_type,h_tsk_prj);
      --trace_printf("work: %d, h_start_date: %d %s, h_finish_date: %d %s, h_sched_work: %d\n", work,h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date,h_sched_work);
      --trace_printf("h_total: %d, h_etc: %d, h_count: %d, h_work_todate: %d, h_remaining: %d, h_assign_string: %s\n", h_total,h_etc,h_count,h_work_todate,h_remaining,h_assign_string);
      --trace_printf("i_u_dates: %d, h_u_units: %d, h_percent_units: %d, i_max_act: %d, i_max_sched: %d, hrs_per_day: %d\n", i_u_dates,h_u_units,h_percent_units,i_max_act,i_max_sched,hrs_per_day);
      
      If Coalesce(h_type, 0) = 2 Then
         If Coalesce(h_etc, 0) > 0 Then
            h_total := Coalesce(h_etc, 0) + Coalesce(h_work, 0);
         Else
            h_etc := Coalesce(h_total, 0) - Coalesce(h_work, 0);
         End If;
         
         If Coalesce(h_tsk_prj, 0) = 1 Then
            If Coalesce(h_bitflag, 0) = 4 Then
               If i_max_act > 0 Then
                  h_temp_var := Trunc((i_max_act / hrs_per_day) * 10000.00);
               ElsIf (i_max_sched > 0) AND (Coalesce(h_u_units, 0) = 1) Then
                  h_percent_units := Trunc((i_max_sched / hrs_per_day) * 10000.00);
                  h_temp_var := 0;
               End If;
               
               If i_u_dates = 1 Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7,
                     ASSIGN_STRING   = h_assign_string,
                     BAK_ASSIGN_STRING = CASE WHEN h_temp_var = 0 THEN h_assign_string ELSE BAK_ASSIGN_STRING END,
                     SCHED_WORK      = h_total,
                     SCHED_START     = h_start_date,
                     SCHED_FINISH    = h_finish_date,
                     PROPOSED_WORK   = h_total,
                     ETC             = h_etc,
                     ASSIGN_UNITS    = h_temp_var,
                     UNITS_MSP       = h_percent_units
                     WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 471+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(472) : cplan_fd.sql(8211) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7,
                     ASSIGN_STRING   = h_assign_string,
                     BAK_ASSIGN_STRING = CASE WHEN h_temp_var = 0 THEN h_assign_string ELSE BAK_ASSIGN_STRING END,
                     SCHED_WORK      = h_total,
                     PROPOSED_WORK   = h_total,
                     ETC             = h_etc,
                     ASSIGN_UNITS    = h_temp_var,
                     UNITS_MSP       = h_percent_units
                     WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 496+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(497) : cplan_fd.sql(8236) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("SC_STATE: %d, h_percent_units: %d, h_temp_var: %d\n", SC_STATE,h_percent_units,h_temp_var);
            ElsIf Coalesce(h_bitflag, 0) = 1 Then
               If Coalesce(h_work_todate, 0) > 0 Then
                  If Coalesce(h_remaining, 0) = 0 Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = 10000,
                        SCHED_START = h_start_date, SCHED_FINISH = h_finish_date, SCHED_WORK = h_work_todate,
                        ACTUAL_START = h_start_date, ACTUAL_FINISH = h_finish_date, ASSIGN_STRING = h_assign_string
                        WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 520+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(521) : cplan_fd.sql(8260) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET
                        SCHED_START = h_start_date, SCHED_FINISH = h_finish_date,
                        SCHED_WORK = CASE WHEN h_total >= WORK_TODATE THEN h_total ELSE BAK_WORK END,
                        ACTUAL_START = h_start_date, ACTUAL_FINISH = NuLL, FD = PMO_CNST.SC_SF,
                        ASSIGN_STRING = h_assign_string
                        WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 542+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(543) : cplan_fd.sql(8282) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     NULL;
                  End If;
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET
                     SCHED_START = h_start_date, SCHED_FINISH = h_finish_date, SCHED_WORK = h_total,
                     ACTUAL_START = NuLL, ACTUAL_FINISH = NuLL, ACTUAL_DURATION = 0, WORK_TODATE = 0,
                     SD = PMO_CNST.SC_SS, FD = PMO_CNST.SC_SF,
                     MILESTONE_PERCENT = 0,
                     ASSIGN_STRING = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 567+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(568) : cplan_fd.sql(8307) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            ElsIf Coalesce(h_bitflag, 0) = 2 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET ASSIGN_STRING = h_assign_string, BAK_ASSIGN_STRING = h_assign_string,
                  PROPOSED_START = h_start_date, PROPOSED_FINISH = h_finish_date, PROPOSED_WORK = h_total,
                  SCHED_START = h_start_date, SCHED_FINISH = h_finish_date, SCHED_DURATION = 0,
                  SCHED_WORK = CASE WHEN h_etc = 0 THEN h_work_todate ELSE h_total END,
                  FORECAST_START = NULL, FORECAST_FINISH = NULL, FORECAST_DURATION = 0, ETC = h_etc,
                  BAK_START_DT = h_start_date, BAK_FINISH_DT = h_finish_date, BAK_WORK = h_total, BEST = h_total
                  WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 591+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(592) : cplan_fd.sql(8331) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf Coalesce(h_bitflag, 0) = 3 Then
               If Coalesce(h_work_todate, 0) > 0 Then
                  If Coalesce(h_remaining, 0) = 0 Then
                     If Coalesce(h_percent_units, 0) = 10000 Then
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET SCHED_START = h_start_date, SCHED_FINISH = h_finish_date,
                           SCHED_WORK = h_total, WORK_TODATE = h_total,
                           ACTUAL_START = h_start_date, ACTUAL_FINISH = h_finish_date,
                           BAK_START_DT = h_start_date, BAK_FINISH_DT = h_finish_date, BAK_WORK = h_total,
                           BEST = h_count, ASSIGN_STRING = h_assign_string, BAK_ASSIGN_STRING = h_assign_string
                           WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 616+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(617) : cplan_fd.sql(8356) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET SCHED_START = h_start_date, SCHED_FINISH = h_finish_date,
                           SCHED_WORK = CASE WHEN h_total >= WORK_TODATE THEN h_total ELSE BAK_WORK END,
                           ACTUAL_START = h_start_date, ACTUAL_FINISH = h_finish_date,
                           BAK_START_DT = h_start_date, BAK_FINISH_DT = h_finish_date, BAK_WORK = h_total,
                           BEST = h_count, ASSIGN_STRING = h_assign_string, BAK_ASSIGN_STRING = h_assign_string
                           WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 638+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(639) : cplan_fd.sql(8378) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET
                        SCHED_START = h_start_date, SCHED_FINISH = h_finish_date,
                        SCHED_WORK = CASE WHEN h_total >= WORK_TODATE THEN h_total ELSE BAK_WORK END,
                        ACTUAL_START = h_start_date, ACTUAL_FINISH = NuLL, FD = PMO_CNST.SC_SF,
                        BAK_START_DT = h_start_date, BAK_FINISH_DT = h_finish_date, BAK_WORK = h_total,
                        BEST = h_count, ASSIGN_STRING = h_assign_string, BAK_ASSIGN_STRING = h_assign_string
                        WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 662+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(663) : cplan_fd.sql(8402) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     NULL;
                  End If;
                  --trace_printf("b_* SC_STATE: %d, h_remaining: %d\n", SC_STATE,h_remaining);
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET SD = PMO_CNST.SC_SS, FD = PMO_CNST.SC_SF, START_DT = h_start_date, FINISH_DT = h_finish_date,
                     SCHED_START = h_start_date, SCHED_FINISH = h_finish_date, SCHED_WORK = h_total,
                     ACTUAL_START = NuLL, ACTUAL_FINISH = NuLL, ACTUAL_DURATION = 0, WORK_TODATE = 0,
                     MILESTONE_PERCENT = 0,
                     BAK_START_DT = h_start_date, BAK_FINISH_DT = h_finish_date, BAK_WORK = h_total,
                     BEST = h_count, ASSIGN_STRING = h_assign_string, BAK_ASSIGN_STRING = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 688+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(689) : cplan_fd.sql(8428) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("SC_STATE: %d, ,h_prj_assignment_id: %d:%s\n", SC_STATE,h_prj_assignment_id_Ind,h_prj_assignment_id);
               End If;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET START_DT = h_start_date, FINISH_DT = h_finish_date,
                  SCHED_START = h_start_date, SCHED_FINISH = h_finish_date,
                  SCHED_WORK = CASE WHEN h_total >= WORK_TODATE THEN h_total ELSE BAK_WORK END,
                  ASSIGN_STRING = h_assign_string
                  WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 711+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(712) : cplan_fd.sql(8451) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            If Coalesce(h_bitflag, 0) = 3 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET SD = PMO_CNST.SC_SS, FD = PMO_CNST.SC_SF, START_DT = h_start_date, FINISH_DT = h_finish_date,
                  SCHED_START = h_start_date, SCHED_FINISH = h_finish_date, SCHED_WORK = h_total,
                  BAK_START_DT = h_start_date, BAK_FINISH_DT = h_finish_date, BAK_WORK = h_count,
                  BEST = h_count, ESTIMATION_LABOR_COST = h_count * REGULAR_COST / 60,
                  ESTIMATION_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE h_count * REGULAR_RATE / 60 END,
                  ETC = h_total, ASSIGN_STRING = h_assign_string, BAK_ASSIGN_STRING = h_assign_string
                  WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 736+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(737) : cplan_fd.sql(8476) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET START_DT = h_start_date, FINISH_DT = h_finish_date,
                  SCHED_START = CASE WHEN SCHED_START IS NuLL THEN h_start_date ELSE SCHED_START END,
                  SCHED_FINISH = CASE WHEN SCHED_FINISH IS NuLL THEN h_finish_date ELSE SCHED_FINISH END,
                  SCHED_WORK = h_total, ETC = h_total, ASSIGN_STRING = h_assign_string
                  WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 757+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(758) : cplan_fd.sql(8497) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("a_* SC_STATE: %d, h_bitflag: %d\n", SC_STATE,h_bitflag);
         End If;
      Else
         If Coalesce(h_etc, 0) > 0 Then
            h_total := Coalesce(h_etc, 0);
         End If;
         
         h_temp_var := 0;
         If (i_max_sched > 0) AND (hrs_per_day > 0) Then
            h_temp_var := Trunc((i_max_sched / hrs_per_day) * 10000.00);
            
            --trace_printf("a_* h_temp_var: %d, h_percent_units: %d\n", h_temp_var,h_percent_units);
            --if ( (h_percent_units >= h_temp_var) || (abs(h_percent_units - h_temp_var) < 100) ) //AIX does not like abs
            If Coalesce(h_percent_units, 0) >= Coalesce(h_temp_var, 0) Then
               h_temp_var := 0;
            ElsIf (Coalesce(h_temp_var, 0) > Coalesce(h_percent_units, 0)) AND ((Coalesce(h_temp_var, 0) 
                                          - Coalesce(h_percent_units, 0)) < 100) Then
               h_temp_var := 0;
            End If;
         End If;
         
         If Coalesce(h_bitflag, 0) = 4 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PROPOSED_START = h_start_date, PROPOSED_FINISH = h_finish_date, PROPOSED_WORK = h_total,
               PLANNED_START = h_start_date, PLANNED_FINISH = h_finish_date, PLANNED_WORK = h_total,
               BAK_START_DT = h_start_date, BAK_FINISH_DT = h_finish_date, BAK_WORK = h_total, BAK_ASSIGN_STRING = h_assign_string,
               BEST = h_count, ASSIGN_STRING = h_assign_string, ETC = h_total, UNITS_MSP = h_percent_units
               WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 801+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(802) : cplan_fd.sql(8541) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_tsk_prj, 0) = 1 Then
            If (Coalesce(h_temp_var, 0) > 0) AND (Coalesce(h_temp_var, 0) <> 10000) AND (Coalesce(
                  h_temp_var, 0) <> Coalesce(h_percent_units, 0)) Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET START_DT = h_start_date, FINISH_DT = h_finish_date,
                  PLANNED_START = h_start_date, PLANNED_FINISH = h_finish_date, PLANNED_WORK = h_total,
                  BAK_START_DT = h_start_date, BAK_FINISH_DT = h_finish_date, BAK_WORK=h_total,
                  UNITS_MSP = h_temp_var, T_UNIT = 0, C_EFFORT = 0, C_UNIT = 0, C_CONTOUR = 0, CONTOUR_TYPE = 0,
                  BEST = h_count, BAK_ASSIGN_STRING = h_assign_string, ASSIGN_STRING = h_assign_string
                  WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 825+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(826) : cplan_fd.sql(8565) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET START_DT = h_start_date, FINISH_DT = h_finish_date,
                  PLANNED_START = h_start_date, PLANNED_FINISH = h_finish_date, PLANNED_WORK = h_total, ETC = h_total,
                  BAK_START_DT = h_start_date, BAK_FINISH_DT = h_finish_date, BAK_WORK=h_total,
                  BEST = h_count, BAK_ASSIGN_STRING = h_assign_string, ASSIGN_STRING = h_assign_string
                  WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 846+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(847) : cplan_fd.sql(8586) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            If Coalesce(h_bitflag, 0) = 3 Then
               If (Coalesce(h_temp_var, 0) > 0) AND (Coalesce(h_temp_var, 0) <> 10000) AND (Coalesce(
                     h_temp_var, 0) <> Coalesce(h_percent_units, 0)) Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET SD = PMO_CNST.SC_NS, FD = PMO_CNST.SC_NF, START_DT = h_start_date, FINISH_DT = h_finish_date,
                     PLANNED_START = h_start_date, PLANNED_FINISH = h_finish_date, PLANNED_WORK = h_total,
                     BAK_START_DT = h_start_date, BAK_FINISH_DT = h_finish_date, BAK_WORK = h_count,
                     ETC = h_total, BEST = h_count, ESTIMATION_LABOR_COST = h_count * REGULAR_COST / 60,
                     ESTIMATION_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE h_count * REGULAR_RATE / 60 END,
                     UNITS_MSP = h_temp_var, T_UNIT = 0, C_EFFORT = 0, C_UNIT = 0, C_CONTOUR = 0, CONTOUR_TYPE = 0,
                     ASSIGN_STRING = h_assign_string, BAK_ASSIGN_STRING = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 874+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(875) : cplan_fd.sql(8614) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET SD = PMO_CNST.SC_NS, FD = PMO_CNST.SC_NF, START_DT = h_start_date, FINISH_DT = h_finish_date,
                     PLANNED_START = h_start_date, PLANNED_FINISH = h_finish_date, PLANNED_WORK = h_total,
                     BAK_START_DT = h_start_date, BAK_FINISH_DT = h_finish_date, BAK_WORK = h_count,
                     ETC = h_total, BEST = h_count, ESTIMATION_LABOR_COST = h_count * REGULAR_COST / 60,
                     ESTIMATION_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE h_count * REGULAR_RATE / 60 END,
                     ASSIGN_STRING = h_assign_string, BAK_ASSIGN_STRING = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 897+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(898) : cplan_fd.sql(8637) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET SD = PMO_CNST.SC_NS, FD = PMO_CNST.SC_NF, START_DT = h_start_date, FINISH_DT = h_finish_date,
                  PLANNED_START = h_start_date, PLANNED_FINISH = h_finish_date, PLANNED_WORK = h_total,
                  BEST = h_count, ESTIMATION_LABOR_COST = h_count * REGULAR_COST / 60,
                  ESTIMATION_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE h_count * REGULAR_RATE / 60 END,
                  UNITS_MSP = CASE WHEN h_temp_var > 0 THEN h_temp_var ELSE UNITS_MSP END,
                  ASSIGN_STRING = h_assign_string
                  WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 921+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(922) : cplan_fd.sql(8661) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         --trace_printf("a_* SC_STATE: %d, h_tsk_prj: %d, h_temp_var: %d, h_percent_units: %d\n", SC_STATE,h_tsk_prj,h_temp_var,h_percent_units);
      End If;
   End If;
   --trace_printf("SC_STATE: %d, h_prj_assignment_id: %d:%s\n", SC_STATE,h_prj_assignment_id_Ind,h_prj_assignment_id); //trace_flush();
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_ASSIGN_STRING', 933+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: U_ASSIGN_STRING(934) : cplan_fd.sql(8673) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION U_ASSIGN_STRING;
ALTER FUNCTION U_ASSIGN_STRING COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_U_TDY_LAST_ACTUAL ==
CREATE OR REPLACE FUNCTION FC_U_TDY_LAST_ACTUAL
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_prj_assign_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_last_submitted IN OUT NOCOPY TIMESTAMP
   , sh_h_last_day PLS_INTEGER
   , sh_h_add_tp_delta_flag PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_last_day PLS_INTEGER := sh_h_last_day;
   h_add_tp_delta_flag PLS_INTEGER := Coalesce(sh_h_add_tp_delta_flag, 0);
   h_dayofyear DATE;
   h_temp_var PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   tempStr VARCHAR2(255);
   h_sched_work PLS_INTEGER := 0;
   h_01 NUMBER := 0;
   h_02 NUMBER := 0;
   h_03 NUMBER := 0;
   h_04 NUMBER := 0;
   h_05 NUMBER := 0;
   h_06 NUMBER := 0;
   h_07 NUMBER := 0;
   h_08 NUMBER := 0;
   h_09 NUMBER := 0;
   h_10 NUMBER := 0;
   h_11 NUMBER := 0;
   h_12 NUMBER := 0;
   h_total_h NUMBER := 0;
   h_d_total NUMBER := 0;
   h_d_01 NUMBER := 0;
   h_d_02 NUMBER := 0;
   h_d_03 NUMBER := 0;
   h_d_04 NUMBER := 0;
   h_d_05 NUMBER := 0;
   h_d_06 NUMBER := 0;
   h_d_07 NUMBER := 0;
   h_d_08 NUMBER := 0;
   h_d_09 NUMBER := 0;
   h_d_10 NUMBER := 0;
   h_d_11 NUMBER := 0;
   h_d_12 NUMBER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_budget_type PLS_INTEGER := 0;
   h_cc_id VARCHAR2(32);
   h_cur_year PLS_INTEGER := 0;
   h_start_month PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_TDY_LAST_ACTUAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_TDY_LAST_ACTUAL'), '-- ' || SYSTIMESTAMP, ':r := FC_U_TDY_LAST_ACTUAL(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_prj_assign_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_last_submitted || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_last_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_add_tp_delta_flag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TDY_LAST_ACTUAL', 68+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_TDY_LAST_ACTUAL(69) : cplan_fd.sql(8752) : ' || SYSTIMESTAMP);
   END IF; 

   If Coalesce(h_last_day, 0) = 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TDY_LAST_ACTUAL', 73+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_U_TDY_LAST_ACTUAL(74) : cplan_fd.sql(8757) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If h_prj_assign_id IS NOT NULL Then
      BEGIN
         SELECT SCHED_MIN - ACTUAL_MIN, BIT_SCHED INTO h_temp_var, h_bitID
         FROM TASK_DAY_OF_YEAR
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 89+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(90) : cplan_fd.sql(8773) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TDY_LAST_ACTUAL', 96+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_TDY_LAST_ACTUAL(97) : cplan_fd.sql(8780) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
      
      BEGIN
         UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_temp_var
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assign_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 114+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(115) : cplan_fd.sql(8798) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         h_bitID1 := h_bitID;
         --h_bitID1_Ind = 0;
         PMO_STPU.clearStartDayStruct(Coalesce(h_last_day, 0), h_bitID1);
         BEGIN
            INSERT INTO TASK_DAY_OF_YEAR (PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG)
            VALUES(h_project_id, h_task_id, h_res_id, h_prj_assign_id, h_prj_assign_id, h_last_submitted, h_temp_var, h_bitID1, '2');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 136+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(137) : cplan_fd.sql(8820) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("INSERT: SC_STATE = %d, h_last_submitted: %s, h_last_day: %d\n", SC_STATE,h_last_submitted,h_last_day);
      End If;
   Else
      BEGIN
         SELECT SCHED_MIN - ACTUAL_MIN, BIT_SCHED INTO h_temp_var, h_bitID
         FROM TASK_DAY_OF_YEAR
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 153+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(154) : cplan_fd.sql(8837) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TDY_LAST_ACTUAL', 160+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_TDY_LAST_ACTUAL(161) : cplan_fd.sql(8844) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
   End If;
   
   --strcpy(tempStr,"");
   --printTimes((unsigned char *)h_bitID.h_value,tempStr);
   --trace_printf("0_*** h_last_day: %d, tempStr = %s\n",h_last_day,tempStr); //trace_flush();
   PMO_STPU.clearFinishDayStruct(Coalesce(h_last_day, 0), h_bitID);
   
   If h_add_tp_delta_flag = 1 Then
      BEGIN
         SELECT SCHED_MIN - ACTUAL_MIN INTO h_sched_work
         FROM TASK_DAY_OF_YEAR
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 183+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(184) : cplan_fd.sql(8867) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_sched_work := 0;
   End If;
   
   If Coalesce(h_sched_work, 0) > 0 Then
      h_sched_work := 0 - Coalesce(h_sched_work, 0);
      
      BEGIN
         SELECT FISCAL_YEAR, FISCAL_MONTH INTO h_cur_year, h_start_month
         FROM FISCAL_CALENDAR
         WHERE TO_DATE(SubStr(h_last_submitted, 1, 10)) BETWEEN START_DATE AND FINISH_DATE;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 204+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(205) : cplan_fd.sql(8888) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         BEGIN
            SELECT BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
            INTO h_billable, h_capital_expense, h_reg_cost, h_reg_sale
            FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 221+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(222) : cplan_fd.sql(8905) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_01 := 0.00;
         h_02 := 0.00;
         h_03 := 0.00;
         h_04 := 0.00;
         h_05 := 0.00;
         h_06 := 0.00;
         h_07 := 0.00;
         h_08 := 0.00;
         h_09 := 0.00;
         h_10 := 0.00;
         h_11 := 0.00;
         h_12 := 0.00;
         h_total_h := 0.00;
         
         CASE h_start_month
            
            WHEN 1 THEN
               h_01 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_01, 0);
               h_d_01 := Coalesce(h_01, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_01, 0);
            
            WHEN 2 THEN
               h_02 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_02, 0);
               h_d_02 := Coalesce(h_02, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_02, 0);
            
            WHEN 3 THEN
               h_03 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_03, 0);
               h_d_03 := Coalesce(h_03, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_03, 0);
            
            WHEN 4 THEN
               h_04 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_04, 0);
               h_d_04 := Coalesce(h_04, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_04, 0);
            
            WHEN 5 THEN
               h_05 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_05, 0);
               h_d_05 := Coalesce(h_05, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_05, 0);
            
            WHEN 6 THEN
               h_06 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_06, 0);
               h_d_06 := Coalesce(h_06, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_06, 0);
            
            WHEN 7 THEN
               h_07 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_07, 0);
               h_d_07 := Coalesce(h_07, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_07, 0);
            
            WHEN 8 THEN
               h_08 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_08, 0);
               h_d_08 := Coalesce(h_08, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_08, 0);
            
            WHEN 9 THEN
               h_09 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_09, 0);
               h_d_09 := Coalesce(h_09, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_09, 0);
            
            WHEN 10 THEN
               h_10 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_10, 0);
               h_d_10 := Coalesce(h_10, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_10, 0);
            
            WHEN 11 THEN
               h_11 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_11, 0);
               h_d_11 := Coalesce(h_11, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_11, 0);
            
            WHEN 12 THEN
               h_12 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_12, 0);
               h_d_12 := Coalesce(h_12, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_12, 0);
            ELSE
               NULL;
         END CASE;
         
         h_budget_type := PMO_CNST.SC_SCHED_BUDGET;
         If Coalesce(h_capital_expense, 0) = 0 Then
            h_cc_id := 'EXPENSE_HOURS';
         Else
            h_cc_id := 'CAPITAL_HOURS';
         End If;
         
         BEGIN
            INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
            MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
            MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
            VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_total_h,
            h_01, h_02, h_03, h_04, h_05, h_06,
            h_07, h_08, h_09, h_10, h_11, h_12);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 341+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(342) : cplan_fd.sql(9025) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE BUDGETS
            SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS, PMO_CNST.SC_SCHED_DELTA_BIT)
            WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
            AND CHARGECODE_ID = h_cc_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 361+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(362) : cplan_fd.sql(9045) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_reg_cost, 0) <> 0.00 Then
            If Coalesce(h_capital_expense, 0) = 0 Then
               h_cc_id := 'EXPENSE_LABOR';
            Else
               h_cc_id := 'CAPITAL_LABOR';
            End If;
            
            BEGIN
               INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
               MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
               MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
               VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_d_total,
               h_d_01, h_d_02, h_d_03, h_d_04, h_d_05, h_d_06,
               h_d_07, h_d_08, h_d_09, h_d_10, h_d_11, h_d_12);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 390+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(391) : cplan_fd.sql(9074) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE BUDGETS
               SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS, PMO_CNST.SC_SCHED_DELTA_BIT)
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
               AND CHARGECODE_ID = h_cc_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 410+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(411) : cplan_fd.sql(9094) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If Coalesce(h_billable, 0) = 1 Then
            h_cc_id := 'BENEFIT_HOURS';
            
            BEGIN
               INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
               MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
               MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
               VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_total_h,
               h_01, h_02, h_03, h_04, h_05, h_06,
               h_07, h_08, h_09, h_10, h_11, h_12);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 436+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(437) : cplan_fd.sql(9120) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE BUDGETS
               SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS, PMO_CNST.SC_SCHED_DELTA_BIT)
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
               AND CHARGECODE_ID = h_cc_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 456+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(457) : cplan_fd.sql(9140) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_reg_sale, 0) <> 0.00 Then
               h_d_01 := Coalesce(h_01, 0) * Coalesce(h_reg_sale, 0);
               h_d_02 := Coalesce(h_02, 0) * Coalesce(h_reg_sale, 0);
               h_d_03 := Coalesce(h_03, 0) * Coalesce(h_reg_sale, 0);
               h_d_04 := Coalesce(h_04, 0) * Coalesce(h_reg_sale, 0);
               h_d_05 := Coalesce(h_05, 0) * Coalesce(h_reg_sale, 0);
               h_d_06 := Coalesce(h_06, 0) * Coalesce(h_reg_sale, 0);
               h_d_07 := Coalesce(h_07, 0) * Coalesce(h_reg_sale, 0);
               h_d_08 := Coalesce(h_08, 0) * Coalesce(h_reg_sale, 0);
               h_d_09 := Coalesce(h_09, 0) * Coalesce(h_reg_sale, 0);
               h_d_10 := Coalesce(h_10, 0) * Coalesce(h_reg_sale, 0);
               h_d_11 := Coalesce(h_11, 0) * Coalesce(h_reg_sale, 0);
               h_d_12 := Coalesce(h_12, 0) * Coalesce(h_reg_sale, 0);
               
               h_d_total := Coalesce(h_d_01, 0) + Coalesce(h_d_02, 0) + Coalesce(h_d_03, 0) + Coalesce(h_d_04, 0) 
                      + Coalesce(h_d_05, 0) + Coalesce(h_d_06, 0) + Coalesce(h_d_07, 0) + Coalesce(
                      h_d_08, 0) + Coalesce(h_d_09, 0) + Coalesce(h_d_10, 0) + Coalesce(h_d_11, 0) 
                      + Coalesce(h_d_12, 0);
               
               h_cc_id := 'BILLABLE_BENEFIT_LABOR';
               BEGIN
                  INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                  MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                  MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                  VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_d_total,
                  h_d_01, h_d_02, h_d_03, h_d_04, h_d_05, h_d_06,
                  h_d_07, h_d_08, h_d_09, h_d_10, h_d_11, h_d_12);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 498+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(499) : cplan_fd.sql(9182) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE BUDGETS
                  SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS, PMO_CNST.SC_SCHED_DELTA_BIT)
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
                  AND CHARGECODE_ID = h_cc_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 518+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(519) : cplan_fd.sql(9202) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
   End If;
   
   BEGIN
      UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN, BIT_SCHED = h_bitID
      WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 540+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(541) : cplan_fd.sql(9224) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --strcpy(tempStr,"");
   --printTimes((unsigned char *)h_bitID.h_value,tempStr);
   --trace_printf("0_*** SC_STATE: %d, tempStr = %s\n",SC_STATE,tempStr); //trace_flush();
   
   BEGIN
      UPDATE TASK_ASSIGNMENTS SET LAST_SCHED_MIN = 0 WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 560+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(561) : cplan_fd.sql(9244) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TDY_LAST_ACTUAL', 566+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_TDY_LAST_ACTUAL(567) : cplan_fd.sql(9250) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_TDY_LAST_ACTUAL;
ALTER FUNCTION FC_U_TDY_LAST_ACTUAL COMPILE DEBUG;

/********************************************************************************************************************/

Prompt == FUNCTION FC_ACPT_ETC_ASSIGN ==
CREATE OR REPLACE FUNCTION FC_ACPT_ETC_ASSIGN
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , h_finish_date IN OUT NOCOPY TIMESTAMP
   , h_last_submitted IN OUT NOCOPY TIMESTAMP
   , sh_h_effort PLS_INTEGER
   , sh_h_remaining PLS_INTEGER
   , sh_h_sched_work PLS_INTEGER
   , sh_h_etc PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_FF_flag PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , h_SD IN OUT NOCOPY DATE
   , h_FD IN OUT NOCOPY DATE
   , sh_h_reg_cost NUMBER
   , sh_h_reg_sale NUMBER
   , sh_h_billable PLS_INTEGER
   , sh_h_capital_expense PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_update PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_effort PLS_INTEGER := sh_h_effort;
   h_remaining PLS_INTEGER := sh_h_remaining;
   h_sched_work PLS_INTEGER := sh_h_sched_work;
   h_etc PLS_INTEGER := sh_h_etc;
   h_contour PLS_INTEGER := sh_h_contour;
   h_FF_flag PLS_INTEGER := sh_h_FF_flag;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_reg_cost NUMBER := sh_h_reg_cost;
   h_reg_sale NUMBER := sh_h_reg_sale;
   h_billable PLS_INTEGER := sh_h_billable;
   h_capital_expense PLS_INTEGER := sh_h_capital_expense;
   h_update PLS_INTEGER := Coalesce(sh_h_update, 0);
   h_res_id VARCHAR2(32);
   h_constr_DT TIMESTAMP;
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_week_of DATE;
   h_finish_D DATE;
   h_dayofyear DATE;
   h_finish_time VARCHAR2(8);
   h_minute PLS_INTEGER := 0;
   h_constr_type PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_state VARCHAR2(1);
   h_available PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_FF_B IS
         SELECT DAY_YEAR, SCHED_MIN, BIT_FLAG FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR > TO_DATE(SubStr(h_finish_D, 1, 10)) AND ACTUAL_FLAG = 0;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ACPT_ETC_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ACPT_ETC_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := FC_ACPT_ETC_ASSIGN(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_last_submitted || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_remaining), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_sched_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_etc), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_FF_flag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_SD || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_FD || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_reg_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_reg_sale), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_billable), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_capital_expense), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_update), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_ETC_ASSIGN', 89+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ACPT_ETC_ASSIGN(90) : cplan_fd.sql(9350) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_FF_flag = %d, h_finish_date = %d, %s\n", h_FF_flag,h_finish_date_Ind,h_finish_date);
   --trace_printf("h_start_date = %d, %s, h_sched_work: %d, h_effort: %d, h_etc: %d\n", h_start_date_Ind,h_start_date,h_sched_work,h_effort,h_etc);//trace_flush();
   
   h_finish_D := SubStr(h_finish_date, 1, 10);
   If (h_last_submitted IS NOT NULL) AND (h_finish_date IS NOT NULL) AND (PMO_ORASTD.strncmp(
         h_last_submitted, h_finish_D, 10) > 0) Then
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET 
         FORECAST_START = NULL,
         FORECAST_FINISH = NULL,
         FORECAST_DURATION = 0
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 114+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(115) : cplan_fd.sql(9375) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_ETC_ASSIGN', 120+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ACPT_ETC_ASSIGN(121) : cplan_fd.sql(9381) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   h_tph_delete := -1;
   h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
   
   h_count := 0;
   h_dayofyear := NULL;
   
   BEGIN
      UPDATE ACTIVITIES
      SET ETC_MODIFIED = 'N'
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
      AND TSK_ASSIGNMENT_ID <> ELEMENT_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 148+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(149) : cplan_fd.sql(9409) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_FF_flag, 0) = 1 Then
      h_effort := 0;
      
      OPEN C_FF_B;
      
      LOOP
         FETCH C_FF_B INTO h_dayofyear, h_minute, h_state;
         IF C_FF_B%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            EXIT;
         End If;
         
         h_effort := Coalesce(h_effort, 0) + Coalesce(h_minute, 0);
         
         h_temp_var := 0;
         h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
      END LOOP;
      
      BEGIN
         CLOSE C_FF_B ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 184+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(185) : cplan_fd.sql(9445) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT COUNT(DAY_YEAR) INTO h_count
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND ACTUAL_FLAG = 0;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 200+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(201) : cplan_fd.sql(9461) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT MAX(DAY_YEAR) INTO h_dayofyear
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR <= h_finish_D AND ACTUAL_FLAG = 0;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 216+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(217) : cplan_fd.sql(9477) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("h_dayofyear: %d %s, h_finish_D: %s, h_count: %d, h_available: %d\n", h_dayofyear_Ind,h_dayofyear,h_finish_D,h_count,h_effort,h_available);//trace_flush();
      If (h_dayofyear IS NOT NULL) AND (PMO_ORASTD.strncmp(h_dayofyear, h_finish_D, 10) < 0) Then
         If Coalesce(h_count, 0) = 1 Then
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET DAY_YEAR = h_finish_D
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_FD;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 237+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(238) : cplan_fd.sql(9498) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If SubStr(h_state, 1, 1) = '2' Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  SCHED_FINISH = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)),PMO_ORASTD.TIME(SCHED_FINISH))
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 257+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(258) : cplan_fd.sql(9518) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  PLANNED_FINISH = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)),PMO_ORASTD.TIME(PLANNED_FINISH))
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 276+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(277) : cplan_fd.sql(9537) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
      
      BEGIN
         SELECT PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED) INTO h_finish_time FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_finish_D;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 293+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(294) : cplan_fd.sql(9554) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)), PMO_ORASTD.TIME(h_finish_date)) INTO h_finish_DT FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 307+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(308) : cplan_fd.sql(9568) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)), PMO_ORASTD.TIME(h_finish_time)) INTO h_finish_DT FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 320+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(321) : cplan_fd.sql(9581) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --trace_printf("h_sched_work: %d, h_effort = %d, h_etc: %d, h_finish_DT: %s, h_finish_time: %s\n", h_sched_work,h_effort,h_etc,h_finish_DT,h_finish_time);//trace_flush();
      If Coalesce(h_sched_work, 0) > Coalesce(h_effort, 0) Then
         If SubStr(h_state, 1, 1) = '2' Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               SCHED_START = COALESCE(FORECAST_START,SCHED_START),
               SCHED_WORK = h_sched_work - h_effort, ETC = h_etc - h_effort,
               SCHED_FINISH = h_finish_DT,
               PROPOSED_WORK = h_sched_work - h_effort,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 349+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(350) : cplan_fd.sql(9610) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_START = COALESCE(FORECAST_START,PLANNED_START),
               PLANNED_WORK = h_sched_work - h_effort, ETC = h_etc - h_effort,
               PLANNED_FINISH = h_finish_DT,
               PROPOSED_WORK = h_sched_work - h_effort,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 374+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(375) : cplan_fd.sql(9635) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If SubStr(h_state, 1, 1) = '2' Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               SCHED_START = COALESCE(FORECAST_START,SCHED_START),
               SCHED_WORK = 0, ETC = 0,
               SCHED_FINISH = h_finish_DT,
               PROPOSED_WORK = 0,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 401+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(402) : cplan_fd.sql(9662) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_START = COALESCE(FORECAST_START,PLANNED_START),
               PLANNED_WORK = 0, ETC = 0,
               PLANNED_FINISH = h_finish_DT,
               PROPOSED_WORK = 0,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 426+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(427) : cplan_fd.sql(9687) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   ElsIf PMO_ORASTD.strncmp(h_start_date, h_SD, 10) <> 0 Then
      --get the bit_flag to determine the state of the project in order to update the correct info
      BEGIN
         SELECT RESOURCE_ID, BIT_FLAG INTO h_res_id, h_state
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 444+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(445) : cplan_fd.sql(9705) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("h_bitflag: %d, h_remaining = %d, h_etc: %d\n", h_bitflag,h_remaining,h_etc);//trace_flush();
      --check if FS  FF fall into weekend then exit
      If h_finish_date IS NOT NULL Then
         BEGIN
            SELECT SUM(AVAILABLE_MIN) INTO h_available
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_res_id
            AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 462+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(463) : cplan_fd.sql(9723) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_available, 0) = 0 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET 
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 484+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(485) : cplan_fd.sql(9745) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_ETC_ASSIGN', 490+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ACPT_ETC_ASSIGN(491) : cplan_fd.sql(9751) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN 0;
         End If;
         
         h_effort := Coalesce(h_available, 0);
      End If;
      
      If (Coalesce(h_remaining, 0) = 0) AND (Coalesce(h_etc, 0) > 0) AND (Coalesce(h_bitflag, 0) = 2) Then
         h_bitflag := 3;
         h_effort := Coalesce(h_etc, 0);
      End If;
      
      If Coalesce(h_bitflag, 0) = 2 Then
         If SubStr(h_state, 1, 1) = '2' Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               SCHED_START = COALESCE(FORECAST_START,SCHED_START),
               SCHED_FINISH = COALESCE(FORECAST_FINISH,SCHED_FINISH) ,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 523+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(524) : cplan_fd.sql(9784) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_START = COALESCE(FORECAST_START,PLANNED_START),
               PLANNED_FINISH = COALESCE(FORECAST_FINISH,PLANNED_FINISH) ,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 546+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(547) : cplan_fd.sql(9807) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If Coalesce(h_remaining, 0) = 0 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               SCHED_START = COALESCE(FORECAST_START,SCHED_START),
               SCHED_FINISH = COALESCE(FORECAST_FINISH,SCHED_FINISH),
               SCHED_WORK = WORK_TODATE + ETC,
               ACTUAL_FINISH = NULL,
               FD = PMO_CNST.SC_SF,
               PROPOSED_WORK = WORK_TODATE + ETC,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 575+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(576) : cplan_fd.sql(9836) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            If SubStr(h_state, 1, 1) = '2' Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  SCHED_START = COALESCE(FORECAST_START,SCHED_START),
                  SCHED_FINISH = COALESCE(FORECAST_FINISH,SCHED_FINISH),
                  SCHED_WORK = WORK_TODATE + ETC,
                  PROPOSED_WORK = WORK_TODATE + ETC,
                  FORECAST_START = NULL,
                  FORECAST_FINISH = NULL,
                  FORECAST_DURATION = 0
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 601+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(602) : cplan_fd.sql(9862) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  PLANNED_START = COALESCE(FORECAST_START,PLANNED_START),
                  PLANNED_FINISH = COALESCE(FORECAST_FINISH,PLANNED_FINISH),
                  PLANNED_WORK = WORK_TODATE + ETC,
                  PROPOSED_WORK = WORK_TODATE + ETC,
                  FORECAST_START = NULL,
                  FORECAST_FINISH = NULL,
                  FORECAST_DURATION = 0
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 626+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(627) : cplan_fd.sql(9887) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
      
      --trace_flush();
      If (h_start_date IS NOT NULL) AND (Coalesce(h_effort, 0) > 0) Then
         If h_last_submitted IS NOT NULL Then
            h_week_of := NULL;
            BEGIN
               SELECT MAX(WEEK_OF) INTO h_week_of FROM TIMESHEET_APPROVAL_STATUS
               WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 647+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(648) : cplan_fd.sql(9908) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If h_week_of IS NOT NULL Then
               BEGIN
                  SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_week_of FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 661+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(662) : cplan_fd.sql(9922) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_ORASTD.strncmp(h_start_date, h_week_of, 10) < 0 Then
                  h_start_date := h_week_of;
               End If;
            ElsIf Coalesce(h_contour, 0) > 0 Then
               --if the resource is assigned with contour then pass last submitted and FC_search_2 will handle
               h_contour := (PMO_FUNC.BIT_OR(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_30));
               
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date))
                  INTO h_start_date
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 683+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(684) : cplan_fd.sql(9944) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date)) + 1     
                  INTO h_start_date
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 698+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(699) : cplan_fd.sql(9959) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         --trace_flush();
         
         If h_prj_assignment_id IS NOT NULL Then
            h_temp_var := 2;
         ElsIf Coalesce(h_sched_work, 0) = 0 Then
            h_temp_var := 1;
         Else
            h_temp_var := 0;
         End If;
         
         h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, Coalesce(h_temp_var, 0));
         
         h_temp_var := 0;
         h_temp_sc_state := FC_RESCHEDULE(h_tsk_assignment_id, h_start_date, h_effort, h_temp_var, h_contour
                                 , h_rec_user, 0, 0);
      End If;
   End If;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   --trace_printf("h_update: %d, h_finish_date: %d %s\n", h_update,h_finish_date_Ind,h_finish_date); //trace_flush();
   If (h_update = 0) AND (h_finish_date IS NOT NULL) Then
      h_finish_D := SubStr(h_finish_date, 1, 10);
      
      If Coalesce(h_count, 0) <> 1 Then
         h_temp_sc_state := FC_FIX_FF(h_project_id, h_tsk_assignment_id, h_finish_D);
      End If;
      
      -- fix constraint date, forecast finish
      BEGIN
         SELECT CONSTRAINT_DT, CONSTRAINT_TYPE INTO h_constr_DT, h_constr_type
         FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 744+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(745) : cplan_fd.sql(10005) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         If h_constr_DT IS NOT NULL Then
            BEGIN
               UPDATE TMT_WBS SET
               CONSTRAINT_DT =
               CASE
               WHEN h_constr_type IN (1,2,3,4) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) < TO_DATE(SubStr(h_finish_date, 1, 10)) THEN h_finish_date
               WHEN h_constr_type IN (0,5,6,7) THEN h_finish_date
               ELSE h_constr_DT
               END,
               CONSTRAINT_TYPE =
               CASE
               WHEN h_constr_type IN (1,2,3,4) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) < TO_DATE(SubStr(h_finish_date, 1, 10)) THEN 2
               WHEN h_constr_type IN (0,5,6,7) THEN 2
               ELSE h_constr_type
               END, ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS
               WHERE ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 776+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(777) : cplan_fd.sql(10037) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_WBS SET CONSTRAINT_DT = h_finish_date, CONSTRAINT_TYPE = 2
               WHERE ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 794+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(795) : cplan_fd.sql(10055) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   ElsIf (h_start_date IS NOT NULL) AND (h_finish_date IS NULL) Then
      -- fix constraint date, forecast start
      BEGIN
         SELECT CONSTRAINT_DT, CONSTRAINT_TYPE INTO h_constr_DT, h_constr_type
         FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 811+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(812) : cplan_fd.sql(10072) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         If h_constr_DT IS NOT NULL Then
            BEGIN
               UPDATE TMT_WBS SET
               CONSTRAINT_DT =
               CASE
               WHEN h_constr_type IN (0,5,6,7) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) > TO_DATE(SubStr(h_start_date, 1, 10)) THEN h_start_date
               WHEN h_constr_type IN (1,2,3,4) THEN h_start_date
               ELSE h_constr_DT
               END,
               CONSTRAINT_TYPE =
               CASE
               WHEN h_constr_type IN (0,5,6,7) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) > TO_DATE(SubStr(h_start_date, 1, 10)) THEN 6
               WHEN h_constr_type IN (1,2,3,4) THEN 6
               ELSE h_constr_type
               END
               WHERE ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 843+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(844) : cplan_fd.sql(10104) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_WBS SET CONSTRAINT_DT = h_start_date, CONSTRAINT_TYPE = 6
               WHERE ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 861+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(862) : cplan_fd.sql(10122) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_ETC_ASSIGN', 870+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ACPT_ETC_ASSIGN(871) : cplan_fd.sql(10131) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ACPT_ETC_ASSIGN;
ALTER FUNCTION FC_ACPT_ETC_ASSIGN COMPILE DEBUG;

/********************************************************************************************************************/

Prompt == FUNCTION FC_CALC_DURATION ==
CREATE OR REPLACE FUNCTION FC_CALC_DURATION
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_effort PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , sh_h_eunit PLS_INTEGER
   , sh_h_unit PLS_INTEGER
   , sh_h_consider PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_occurrence PLS_INTEGER
   , sh_h_per_day PLS_INTEGER
   , sh_h_per_week PLS_INTEGER
   , work IN OUT NOCOPY PLS_INTEGER
   , effort IN OUT NOCOPY PLS_INTEGER
   , duration IN OUT NOCOPY PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_effort PLS_INTEGER := sh_h_effort;
   h_work PLS_INTEGER := sh_h_work;
   h_eunit PLS_INTEGER := sh_h_eunit;
   h_unit PLS_INTEGER := sh_h_unit;
   h_consider PLS_INTEGER := sh_h_consider;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_contour PLS_INTEGER := sh_h_contour;
   h_occurrence PLS_INTEGER := sh_h_occurrence;
   h_per_day PLS_INTEGER := sh_h_per_day;
   h_per_week PLS_INTEGER := sh_h_per_week;
   h_duration PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   i_work PLS_INTEGER NOT NULL := 0;
   d_percent NUMBER NOT NULL := 0.0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CALC_DURATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CALC_DURATION'), '-- ' || SYSTIMESTAMP, ':r := FC_CALC_DURATION(' || 
       '''' || h_project_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_week), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(duration), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 57+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CALC_DURATION(58) : cplan_fd.sql(10199) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_unit: %d, h_work: %d, h_start_DT: %d %s, h_finish_DT: %d %s, h_effort: %d\n", h_unit,h_work,h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT,h_effort);//trace_flush();
   
   If Coalesce(h_unit, 0) = 100 Then
      If Coalesce(h_occurrence, 0) = 0 Then
         d_percent := 10000.00 / Coalesce(h_work, 0);
         i_work := Trunc(Coalesce(h_per_day, 0) * d_percent);
      Else
         --          h_work = int(((h_per_day * h_work) / 10000.00) + 0.4);
         --          i_work = h_work;
         i_work := Trunc(((Coalesce(h_per_day, 0) * Coalesce(h_work, 0)) / 10000.00) + 0.4);
         --          h_unit = 0;
      End If;
   Else
      If (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) > 0) Then
         If (Coalesce(h_unit, 0) = 1) AND (Coalesce(h_unit, 0) = 2) Then
            h_work := Coalesce(h_work, 0) * Coalesce(h_per_day, 0);
         ElsIf Coalesce(h_unit, 0) = 5 Then
            h_work := Coalesce(h_work, 0) * Coalesce(h_per_week, 0);
         End If;
      End If;
      
      i_work := Coalesce(h_work, 0);
   End If;
   
   If Coalesce(h_effort, 0) = 0 Then
      If h_finish_DT IS NULL Then
         If Coalesce(h_occurrence, 0) = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 88+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(89) : cplan_fd.sql(10230) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.INVALID_PARAMS;
         ElsIf Coalesce(h_work, 0) = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 93+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(94) : cplan_fd.sql(10235) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.INVALID_PARAMS;
         Else
            h_effort := Coalesce(h_occurrence, 0) * i_work;
         End If;
      End If;
      --trace_printf("h_occurrence: %d, h_work: %d, i_work: %d, h_effort: %d\n", h_occurrence,h_work,i_work,h_effort);//trace_flush();
   ElsIf Coalesce(h_eunit, 0) > 0 Then
      If Coalesce(h_eunit, 0) = 1 Then
         h_effort := Coalesce(h_effort, 0) * 60;
      ElsIf Coalesce(h_eunit, 0) = 2 Then
         h_effort := Coalesce(h_per_day, 0) * Coalesce(h_effort, 0);
      ElsIf Coalesce(h_eunit, 0) = 3 Then
         h_effort := Coalesce(h_per_week, 0) * Coalesce(h_effort, 0);
      End If;
   End If;
   
   --trace_printf("h_per_day = %d, h_unit = %d, i_work = %d, h_work: %d, h_effort = %d\n", h_per_day,h_unit,i_work,h_work,h_effort);//trace_flush();
   If i_work > 0 Then
      If (Coalesce(h_unit, 0) = 1) AND (Coalesce(h_unit, 0) = 2) Then
         h_work := Coalesce(h_work, 0) * Coalesce(h_per_day, 0);
      ElsIf Coalesce(h_unit, 0) = 5 Then
         h_work := Coalesce(h_work, 0) * Coalesce(h_per_week, 0);
      End If;
      
      h_duration := Trunc(Coalesce(h_effort, 0) / i_work);
   ElsIf Coalesce(h_effort, 0) > Coalesce(h_per_day, 0) Then
      h_duration := Trunc(Coalesce(h_effort, 0) / Coalesce(h_per_day, 0));
   Else
      h_duration := 1;
   End If;
   --trace_printf("h_bitflag = %d, h_duration = %d, h_effort = %d, h_work: %d, h_per_day = %d\n", h_bitflag,h_duration,h_effort,h_work,h_per_day);//trace_flush();
   
   If ((h_finish_DT IS NULL) OR (Coalesce(Length(h_finish_DT), 0) = 0)) AND (i_work > 0) Then
      h_every := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0001F000)), 12);
      If Coalesce(h_bitflag, 0) = 3 Then
         If (Coalesce(h_duration, 0) * Coalesce(h_every, 0)) > 60 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 132+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(133) : cplan_fd.sql(10274) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.CANNOT_SEARCH_ASSIGN;
         End If;
         
         h_week := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_00000F80)), 7);
         If Coalesce(h_week, 0) = 0 Then
            h_week := 1;
         End If;
         h_duration := Coalesce(h_duration, 0) + 1;
         h_duration := Coalesce(h_duration, 0) * Coalesce(h_every, 0);
         --trace_printf("h_duration = %d, h_every = %d, h_week = %d\n", h_duration,h_every,h_week);//trace_flush();
         
         BEGIN
            SELECT DAY_YEAR INTO h_finish_DT FROM (
            SELECT DAY_YEAR, ROW_NUMBER() OVER (ORDER BY DAY_YEAR) AS CNT
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_project_id AND
            DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10)) AND
            PMO_ORASTD.DAY(DAY_YEAR) = h_week )  WHERE CNT = h_duration;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CALC_DURATION', 158+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CALC_DURATION(159) : cplan_fd.sql(10300) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("1.c__* SC_STATE: %d ,h_finish_DT: %d %s\n",SC_STATE,h_finish_DT_Ind,h_finish_DT);//trace_flush();
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               SELECT Add_Months(TO_DATE(SubStr(h_start_DT, 1, 10)), 5 * 12) INTO h_finish_DT FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CALC_DURATION', 173+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CALC_DURATION(174) : cplan_fd.sql(10315) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("1.d__* h_start_DT = %s\n",h_start_DT);//trace_flush();
         
         h_duration := 0;
      Else
         If (Coalesce(h_bitflag, 0) = 5) AND (i_work > 0) Then
            If Coalesce(h_duration, 0) > 60 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_CALC_DURATION', 185+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_CALC_DURATION(186) : cplan_fd.sql(10327) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_SEARCH_ASSIGN), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.CANNOT_SEARCH_ASSIGN, h_project_id);
            Else
               h_duration := 1200;
            End If;
         Else
            h_duration := FC_SRCH_DURATION(h_bitflag, h_contour, h_every, h_duration, h_occurrence, h_unit);
         End If;
      End If;
      
      If i_work > 0 Then
         If (Coalesce(h_unit, 0) = 1) OR (Coalesce(h_unit, 0) = 2) Then
            h_eunit := 2;
         ElsIf (Coalesce(h_unit, 0) = 0) OR (Coalesce(h_unit, 0) = 3) OR (Coalesce(h_unit, 0) = 4) Then
            h_eunit := 1;
         ElsIf Coalesce(h_unit, 0) = 5 Then
            h_eunit := 3;
         End If;
      End If;
   End If;
   --trace_printf("h_effort = %d, h_duration = %d, i_work = %d, d_percent: %.4f\nh_start_DT = %s, h_finish_DT = %s\n", h_effort,h_duration,i_work,d_percent,h_start_DT,h_finish_DT);//trace_flush();
   
   If Coalesce(h_effort, 0) = 0 Then
      BEGIN
         SELECT PMO_ORASTD.DAYS(h_finish_DT) - PMO_ORASTD.DAYS(h_start_DT) INTO h_duration FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CALC_DURATION', 217+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CALC_DURATION(218) : cplan_fd.sql(10359) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_duration, 0) > 5000) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 224+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(225) : cplan_fd.sql(10366) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.CANNOT_SEARCH_ASSIGN;
      End If;
      h_duration := 0;
   ElsIf (Coalesce(h_effort, 0) > 0) AND (Coalesce(h_duration, 0) > 0) Then
      If Coalesce(h_duration, 0) > 2000 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 232+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(233) : cplan_fd.sql(10374) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.CANNOT_SEARCH_ASSIGN;
      ElsIf Coalesce(h_duration, 0) < 1200 Then
         If Coalesce(h_duration, 0) < 30 Then
            h_duration := 50;
         Else
            h_duration := Trunc(Coalesce(h_duration, 0) * 1.5);
         End If;
         
         If d_percent > 0.00 Then
            h_duration := Trunc(Coalesce(h_duration, 0) * d_percent);
         End If;
      End If;
   End If;
   
   If Coalesce(h_unit, 0) <> 100 Then
      h_work := i_work;
   End If;
   
   --trace_printf("*** h_effort: %d, h_duration = %d, h_unit: %d, h_work: %d\n", h_effort,h_duration,h_unit,h_work);//trace_flush();
   duration := Coalesce(h_duration, 0);
   effort := Coalesce(h_effort, 0);
   work := Coalesce(h_work, 0);
   
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 260+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(261) : cplan_fd.sql(10402) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CALC_DURATION;
ALTER FUNCTION FC_CALC_DURATION COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_AD_TASK ==
CREATE OR REPLACE FUNCTION FC_AD_TASK
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_element_id IN OUT NOCOPY VARCHAR2
   , sh_h_work_todate PLS_INTEGER
   , sh_h_per_day PLS_INTEGER
   , g_h_new_actual_duration IN OUT NOCOPY PLS_INTEGER
   , g_h_new_sched_duration IN OUT NOCOPY PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_work_todate PLS_INTEGER := sh_h_work_todate;
   h_per_day PLS_INTEGER := sh_h_per_day;
   h_dayofyear DATE;
   h_actual_duration PLS_INTEGER := 0;
   h_sched_duration PLS_INTEGER := 0;
   h_minute PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_totpr PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_TAD IS
         SELECT DAY_YEAR, SCHED_MIN
         FROM PMO_TMP_TAD
         ORDER BY DAY_YEAR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_AD_TASK') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_AD_TASK'), '-- ' || SYSTIMESTAMP, ':r := FC_AD_TASK(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_element_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_work_todate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(g_h_new_actual_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(g_h_new_sched_duration), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_AD_TASK', 38+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_AD_TASK(39) : cplan_fd.sql(10451) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TAD';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_TAD SELECT DAY_YEAR, SUM(SCHED_MIN)
      FROM TASK_DAY_OF_YEAR
      WHERE PROJECT_ID = h_project_id AND TASK_ID = h_element_id
      GROUP BY DAY_YEAR;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AD_TASK', 60+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AD_TASK(61) : cplan_fd.sql(10473) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         SELECT COUNT(DAY_YEAR) INTO h_sched_duration FROM PMO_TMP_TAD;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AD_TASK', 74+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AD_TASK(75) : cplan_fd.sql(10487) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      g_h_new_sched_duration := Coalesce(h_sched_duration, 0) * Coalesce(h_per_day, 0);
      
      h_total := 0;
      h_count := 0;
      
      OPEN C_TAD;
      
      LOOP
         FETCH C_TAD INTO h_dayofyear, h_minute;
         IF C_TAD%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            EXIT;
         End If;
         
         If (Coalesce(h_total, 0) = 0) AND (Coalesce(h_minute, 0) > Coalesce(h_work_todate, 0)) Then
            If Coalesce(h_minute, 0) < Coalesce(h_per_day, 0) Then
               g_h_new_actual_duration := Coalesce(h_work_todate, 0);
            Else
               g_h_new_actual_duration := Trunc((Coalesce(h_work_todate, 0) / Coalesce(h_minute, 0)) * Coalesce(
                                           h_per_day, 0));
            End If;
            
            EXIT;
         Else
            h_total := Coalesce(h_total, 0) + Coalesce(h_minute, 0);
            If Coalesce(h_total, 0) > Coalesce(h_work_todate, 0) Then
               h_temp_var := Coalesce(h_work_todate, 0) - Coalesce(h_totpr, 0);
               If Coalesce(h_minute, 0) < Coalesce(h_per_day, 0) Then
                  g_h_new_actual_duration := Coalesce(h_count, 0) * Coalesce(h_per_day, 0) + Coalesce(h_temp_var, 0);
               Else
                  g_h_new_actual_duration := Trunc((Coalesce(h_count, 0) + (Coalesce(h_temp_var, 0) / Coalesce(
                                           h_minute, 0))) * Coalesce(h_per_day, 0));
               End If;
               
               EXIT;
            ElsIf Coalesce(h_total, 0) = Coalesce(h_work_todate, 0) Then
               h_count := Coalesce(h_count, 0) + 1;
               g_h_new_actual_duration := Coalesce(h_count, 0) * Coalesce(h_per_day, 0);
               
               EXIT;
            End If;
            
            h_totpr := Coalesce(h_total, 0);
            h_count := Coalesce(h_count, 0) + 1;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_TAD ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AD_TASK', 139+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AD_TASK(140) : cplan_fd.sql(10552) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_AD_TASK', 146+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_AD_TASK(147) : cplan_fd.sql(10559) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_AD_TASK;
ALTER FUNCTION FC_AD_TASK COMPILE DEBUG;

