/*****************************************************************************************/

Prompt == FUNCTION RollupEV ==
CREATE OR REPLACE FUNCTION RollupEV
(
     h_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_c_rank VARCHAR2(200);
   h_current_datetime TIMESTAMP;
   h_current_week DATE;
   h_cursor_str VARCHAR2(9999);
   h_e_c_id VARCHAR2(32);
   h_e_c_project_id VARCHAR2(32);
   h_e_c_tsk_assignment_id VARCHAR2(32);
   h_e_last_ev_c_week DATE;
   h_e_last_ev_update TIMESTAMP;
   h_e_last_ev_week DATE;
   h_rank VARCHAR2(200);
   h_e_c_acwp NUMBER := 0.00;
   h_e_c_bac NUMBER := 0.00;
   h_e_c_bcwp NUMBER := 0.00;
   h_e_c_bcws NUMBER := 0.00;
   h_e_c_eac NUMBER := 0.00;
   h_e_c_eac_forecast NUMBER := 0.00;
   h_e_c_etc NUMBER := 0.00;
   h_e_c_etc_forecast NUMBER := 0.00;
   h_count PLS_INTEGER := 0;
   h_e_c_type_id PLS_INTEGER := 0;
   h_e_day_of_week PLS_INTEGER := 0;
   h_e_db_day_of_week PLS_INTEGER := 0;
   h_e_diff PLS_INTEGER := 0;
   h_e_ev_delay_days PLS_INTEGER := 0;
-- Declare Cursors
      -- Dynamic Cursor Declaration
      C_EV_PRJ PMO_DB2M.REF_CURSOR;

      CURSOR C_EV_WBS IS
         SELECT RTRIM(RANK), PROJECT_ID, ELEMENT_ID, TYPE_ID
         FROM TMT_WBS
         WHERE RANK LIKE h_rank || '%'
         ORDER BY RANK DESC;

      CURSOR C_EV_TA IS
         SELECT TA.TSK_ASSIGNMENT_ID, MAX(ST.WEEK_OF)
         FROM TASK_ASSIGNMENTS TA
         JOIN SUMMARY_TIMESHEETS ST
         ON ST.PROJECT_ID = TA.PROJECT_ID
         AND ST.TSK_ASSIGNMENT_ID = TA.TSK_ASSIGNMENT_ID
         AND WEEK_OF <= TO_DATE(SubStr(h_e_last_ev_week, 1, 10))
         WHERE TA.PROJECT_ID = h_e_c_project_id
         AND TA.TASK_ID = h_e_c_id
         GROUP BY TA.PROJECT_ID, TA.TASK_ID, TA.TSK_ASSIGNMENT_ID;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('RollupEV') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('RollupEV'), '-- ' || SYSTIMESTAMP, ':r := RollupEV(' || 
       '''' || h_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'RollupEV', 61+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: RollupEV(62) : evalue.sql(65) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT SYSTIMESTAMP
      INTO h_current_datetime
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 74+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(75) : evalue.sql(78) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   /*
       DB2 WEEKOFDAY( SUN: 1, MON: 2, TUE: 3, WED: 4, THU: 5, FRI: 6, SAT: 7 )
       PMO WEEKOFDAY( SUN: 0, MON: 1, TUE: 2, WED: 3, THU: 4, FRI: 5, SAT: 6 )
       */
   --get repository week start day setting (+1 to match the db2 setting)
   BEGIN
      SELECT SETTING_VALUE + 1
      INTO h_e_db_day_of_week
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'TIMESHEET_START_WEEKDAY';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 95+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(96) : evalue.sql(99) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --get last ev week (date)
   BEGIN
      SELECT SETTING_VALUE
      INTO h_e_ev_delay_days
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'EV_DELAY_DAYS';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 112+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(113) : evalue.sql(116) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --  if( h_e_ev_delay_days == 0 )
   --      h_e_ev_delay_days = 1;
   --  else
   --      h_e_ev_delay_days = h_e_ev_delay_days * 7;
   
   h_e_ev_delay_days := Coalesce(h_e_ev_delay_days, 0) * 7;
   
   --get last ev week (date)
   BEGIN
      SELECT TO_DATE(SubStr(REC_DATETIME, 1, 10))
      INTO h_e_last_ev_week
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'LAST_EV_WEEK';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 136+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(137) : evalue.sql(140) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --get weekday value of the parameter h_weekof
   BEGIN
      SELECT PMO_ORASTD.DAYOFWEEK(TO_DATE(SYSDATE))
      INTO h_e_day_of_week
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 152+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(153) : evalue.sql(156) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --get diff
   If Coalesce(h_e_db_day_of_week, 0) < Coalesce(h_e_day_of_week, 0) Then
      h_e_diff := Coalesce(h_e_day_of_week, 0) - Coalesce(h_e_db_day_of_week, 0);
   Else
      h_e_diff := 7 - (Coalesce(h_e_db_day_of_week, 0) - Coalesce(h_e_day_of_week, 0));
   End If;
   
   If Coalesce(h_e_diff, 0) = 7 Then
      h_e_diff := 0;
   End If;
   
   --get the current timesheet weekOf
   BEGIN
      SELECT TO_DATE(SYSDATE) - h_e_diff      - h_e_ev_delay_days     
      INTO h_current_week
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 179+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(180) : evalue.sql(183) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --test for update last ev week
   BEGIN
      SELECT CASE WHEN TO_DATE(SubStr(h_e_last_ev_week, 1, 10)) < h_current_week THEN 1 ELSE 0 END
      INTO h_e_diff
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 195+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(196) : evalue.sql(199) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_e_diff, 0) = 1 Then
      h_e_last_ev_week := h_current_week;
      
      BEGIN
         UPDATE TMT_SETTINGS
         SET REC_DATETIME = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_e_last_ev_week, 1, 10)), PMO_ORASTD.TIME(SYSTIMESTAMP))
         WHERE ELEMENT_ID = 'LAST_EV_WEEK';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 217+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(218) : evalue.sql(221) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --get last ev update timestamp
   BEGIN
      SELECT REC_DATETIME
      INTO h_e_last_ev_update
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'LAST_EV_UPDATE';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 235+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(236) : evalue.sql(239) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_EVALUE_0001';
   PMO_DB2M.SC_STATE := 0;
   
   If Coalesce(h_bitflag, 0) = 1 Then
      BEGIN
         UPDATE TMT_WBS SET
         EV_FLAG = '1'
         WHERE PROJECT_ID = h_id
         AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
         AND BAC > 0.00
         AND ((BCWS < BAC) OR (BCWP <> PMO_ORASTD.TODECIMAL(((BAC/10000) * MILESTONE_PERCENT),18,2)));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 262+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(263) : evalue.sql(266) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_TMP_EVALUE_0001
         SELECT P.ELEMENT_ID, P.RANK
         FROM TMT_WBS P
         JOIN TMT_WBS T
         ON T.PROJECT_ID = P.ELEMENT_ID
         AND T.EV_FLAG = '1'
         WHERE P.ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 285+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(286) : evalue.sql(289) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_cursor_str := 'SELECT SUBSTR(RANK,1,8) AS RANK FROM PMO_TMP_EVALUE_0001 ORDER BY RANK ';
   ElsIf Coalesce(h_bitflag, 0) = 2 Then
      BEGIN
         UPDATE TMT_WBS SET
         EV_FLAG = '1'
         WHERE PROJECT_ID = h_id
         AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
         AND BAC > 0.00
         AND ((BCWS < BAC) OR (BCWP <> PMO_ORASTD.TODECIMAL(((BAC/10000) * MILESTONE_PERCENT),18,2)));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 309+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(310) : evalue.sql(313) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_TMP_EVALUE_0001
         SELECT P.ELEMENT_ID, P.RANK
         FROM TMT_WBS P
         JOIN TMT_WBS T
         ON T.PROJECT_ID = P.ELEMENT_ID
         AND T.EV_FLAG = '1'
         WHERE P.ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 332+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(333) : evalue.sql(336) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_cursor_str := 'SELECT RTRIM(RANK) AS RANK FROM PMO_TMP_EVALUE_0001 ORDER BY RANK ';
   Else
      --flag dirty tasks
      BEGIN
         UPDATE TMT_WBS SET
         EV_FLAG = '1'
         WHERE RANK > '0000'
         AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
         AND BAC > 0.00
         AND ((BCWS < BAC) OR (BCWP <> PMO_ORASTD.TODECIMAL(((BAC/10000) * MILESTONE_PERCENT),18,2)));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 357+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(358) : evalue.sql(361) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_TMP_EVALUE_0001
         SELECT P.ELEMENT_ID, SUBSTR(P.RANK,1,8) AS RANK
         FROM TMT_WBS P
         JOIN TMT_WBS T
         ON T.PROJECT_ID = P.ELEMENT_ID
         AND T.EV_FLAG = '1'
         WHERE P.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 380+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(381) : evalue.sql(384) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_cursor_str := 'SELECT DISTINCT RANK AS RANK FROM PMO_TMP_EVALUE_0001 ORDER BY RANK ';
   End If;
   
   h_cursor_str := h_cursor_str || ' ';
   
   --loop through the affected projects
   
   OPEN C_EV_PRJ FOR h_cursor_str;
   
   LOOP
      FETCH C_EV_PRJ INTO h_rank;
      IF C_EV_PRJ%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      OPEN C_EV_WBS;
      
      LOOP
         FETCH C_EV_WBS INTO     h_c_rank,
         h_e_c_project_id,
         h_e_c_id,
         h_e_c_type_id;
         IF C_EV_WBS%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         --trace_printf("******************************\n");
         --trace_printf("rank: %s (%d)\n",h_c_rank,h_c_rank_Ind);
         --trace_printf("prj : %s (%d)\n",h_e_c_project_id,h_e_c_project_id_Ind);
         --trace_printf("id  : %s (%d)\n",h_e_c_id,h_e_c_id_Ind);
         --trace_printf("type: %d (%d)\n",h_e_c_type_id,h_e_c_type_id_Ind);
         --trace_flush();
         
         --Re-initialize the EV variables
         h_count := 0;
         h_e_c_bac := 0.00;
         h_e_c_bcws := 0.00;
         h_e_c_bcwp := 0.00;
         h_e_c_acwp := 0.00;
         h_e_c_eac := 0.00;
         h_e_c_etc := 0.00;
         h_e_c_eac_forecast := 0.00;
         h_e_c_etc_forecast := 0.00;
         
         If (Coalesce(h_e_c_type_id, 0) = PMO_CNST.SC_TASK) OR (Coalesce(
               h_e_c_type_id, 0) = PMO_CNST.SC_MILESTONE) OR (Coalesce(
               h_e_c_type_id, 0) = PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK) OR (Coalesce(
               h_e_c_type_id, 0) = PMO_CNST.SC_WBS_ACTION_ITEM_TASK) OR (Coalesce(
               h_e_c_type_id, 0) = PMO_CNST.SC_WBS_RISK_TASK) OR (Coalesce(
               h_e_c_type_id, 0) = PMO_CNST.SC_WBS_ISSUE_TASK) OR (Coalesce(
               h_e_c_type_id, 0) = PMO_CNST.SC_WBS_REQUIREMENT_TASK) OR (Coalesce(
               h_e_c_type_id, 0) = PMO_CNST.SC_WBS_DEFECT_TASK) OR (Coalesce(
               h_e_c_type_id, 0) = PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) Then
            --open a cursor to fetch
            --tsk_assignment_id
            --max(week_of) before h_e_last_ev_week
            BEGIN
               SELECT COUNT(TA.TSK_ASSIGNMENT_ID)
               INTO h_count
               FROM TASK_ASSIGNMENTS TA
               JOIN SUMMARY_TIMESHEETS ST
               ON ST.PROJECT_ID = TA.PROJECT_ID
               AND ST.TSK_ASSIGNMENT_ID = TA.TSK_ASSIGNMENT_ID
               AND WEEK_OF <= TO_DATE(SubStr(h_e_last_ev_week, 1, 10))
               WHERE TA.PROJECT_ID = h_e_c_project_id
               AND TA.TASK_ID = h_e_c_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 466+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(467) : evalue.sql(470) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_count, 0) > 0 Then
               OPEN C_EV_TA;
               
               LOOP
                  FETCH C_EV_TA INTO h_e_c_tsk_assignment_id, h_e_last_ev_c_week;
                  IF C_EV_TA%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                     EXIT;
                  End If;
                  
                  BEGIN
                     SELECT  COALESCE(BAC, 0.00) + h_e_c_bac,
                     COALESCE(BCWS, 0.00) + h_e_c_bcws,
                     COALESCE(BCWP, 0.00) + h_e_c_bcwp,
                     COALESCE(ACWP, 0.00) + h_e_c_acwp,
                     COALESCE(EAC, 0.00) + h_e_c_eac,
                     COALESCE(EV_ETC, 0.00) + h_e_c_etc,
                     COALESCE(EAC_FORECAST, 0.00) + h_e_c_eac_forecast,
                     COALESCE(EV_ETC_FORECAST, 0.00) + h_e_c_etc_forecast
                     INTO    h_e_c_bac,
                     h_e_c_bcws,
                     h_e_c_bcwp,
                     h_e_c_acwp,
                     h_e_c_eac,
                     h_e_c_etc,
                     h_e_c_eac_forecast,
                     h_e_c_etc_forecast
                     FROM SUMMARY_TIMESHEETS
                     WHERE PROJECT_ID = h_e_c_project_id
                     AND TSK_ASSIGNMENT_ID = h_e_c_tsk_assignment_id
                     AND WEEK_OF = TO_DATE(SubStr(h_e_last_ev_c_week, 1, 10));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 512+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(513) : evalue.sql(516) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               END LOOP;
               
               BEGIN
                  CLOSE C_EV_TA ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 526+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(527) : evalue.sql(530) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TMT_WBS SET
                  BCWS        = h_e_c_bcws,
                  EV_FLAG     = '0'
                  WHERE ELEMENT_ID = h_e_c_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 546+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(547) : evalue.sql(550) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --else
            --{
            --  BEGIN_SQL
            --      UPDATE TMT_WBS SET
            --          BCWP    = (BAC/10000) * MILESTONE_PERCENT,
            --          EV_FLAG = '0'
            --      WHERE ELEMENT_ID = HOST(h_e_c_id);
            --  END_SQL_CHECK;
            --}
            
            BEGIN
               UPDATE TMT_WBS SET
               BCWS    =   CASE
               WHEN TO_DATE(SubStr(BASELINE1_START, 1, 10)) > TO_DATE(SubStr(h_e_last_ev_week, 1, 10)) + 6     
               THEN 0
               WHEN TO_DATE(SubStr(BASELINE1_FINISH, 1, 10)) < TO_DATE(SubStr(h_e_last_ev_week, 1, 10)) + 6     
               THEN BAC
               ELSE BCWS
               END,
               BCWP    = (BAC/10000) * WORK_PERCENT,
               EV_FLAG = '0'
               WHERE ELEMENT_ID = h_e_c_id
               AND BASELINE1_START IS NOT NULL
               AND BASELINE1_FINISH IS NOT NULL
               AND BAC > 0;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 586+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(587) : evalue.sql(590) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT  COUNT(TW.ELEMENT_ID),
               SUM(COALESCE(TW.BAC,0.00)),
               SUM(COALESCE(TW.BCWS,0.00)),
               SUM(COALESCE(TW.BCWP,0.00)),
               SUM(COALESCE(TW.ACWP,0.00)),
               SUM(COALESCE(TW.EAC,0.00)),
               SUM(COALESCE(TW.EV_ETC,0.00)),
               SUM(COALESCE(TW.EAC_FORECAST,0.00)),
               SUM(COALESCE(TW.EV_ETC_FORECAST,0.00))
               INTO    h_count,
               h_e_c_bac,
               h_e_c_bcws,
               h_e_c_bcwp,
               h_e_c_acwp,
               h_e_c_eac,
               h_e_c_etc,
               h_e_c_eac_forecast,
               h_e_c_etc_forecast
               FROM TMT_WBS TW
               WHERE RANK LIKE h_rank || '%'
               AND PMO_ORASTD.LEFT(TW.RANK, LENGTH(h_c_rank)) = h_c_rank
               AND LENGTH(TW.RANK) = (LENGTH(h_c_rank) + 4);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 620+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(621) : evalue.sql(624) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_count, 0) = 0 Then
               h_e_c_bac := 0.00;
               h_e_c_bcws := 0.00;
               h_e_c_bcwp := 0.00;
               h_e_c_acwp := 0.00;
               h_e_c_eac := 0.00;
               h_e_c_etc := 0.00;
               h_e_c_eac_forecast := 0.00;
               h_e_c_etc_forecast := 0.00;
            End If;
            
            BEGIN
               UPDATE TMT_WBS SET
               BAC         = h_e_c_bac,
               BCWS        = h_e_c_bcws,
               BCWP        = h_e_c_bcwp,
               ACWP        = h_e_c_acwp,
               EV_ETC      = h_e_c_etc,
               EAC         = h_e_c_eac,
               EV_ETC_FORECAST = h_e_c_etc_forecast,
               EAC_FORECAST    = h_e_c_eac_forecast,
               EV_FLAG     = '0'
               WHERE ELEMENT_ID = h_e_c_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 658+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(659) : evalue.sql(662) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      --end of while
      
      BEGIN
         CLOSE C_EV_WBS ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 674+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(675) : evalue.sql(678) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      COMMIT;
   END LOOP;
   --end of while
   
   BEGIN
      CLOSE C_EV_PRJ ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 690+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(691) : evalue.sql(694) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --reset BCWS/BCWP when BAC is reset to zero
   BEGIN
      UPDATE TMT_WBS SET
      BCWS = 0, BCWP = 0
      WHERE BAC = 0
      AND (BCWS+BCWP) > 0;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 711+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(712) : evalue.sql(715) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --get last ev update timestamp
   BEGIN
      UPDATE TMT_SETTINGS SET
      REC_DATETIME = PMO_ORASTD.DB2TIMESTAMP(h_current_datetime)
      WHERE ELEMENT_ID = 'LAST_EV_UPDATE';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'RollupEV', 731+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: RollupEV(732) : evalue.sql(735) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'RollupEV', 737+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: RollupEV(738) : evalue.sql(741) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION RollupEV;
ALTER FUNCTION RollupEV COMPILE DEBUG;

/*********************************************************************************/

Prompt == FUNCTION SP_ROLLUP_EV ==
CREATE OR REPLACE FUNCTION SP_ROLLUP_EV
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_ROLLUP_EV') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_ROLLUP_EV'), '-- ' || SYSTIMESTAMP, ':r := SP_ROLLUP_EV(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ROLLUP_EV', 18+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_ROLLUP_EV(19) : evalue.sql(770) : ' || SYSTIMESTAMP);
   END IF; 

   h_id := p_id;
   h_bitflag := p_bitflag;
   
   --getStringParam(id, h_id, 0);
   --getLongParam(bitflag, h_bitflag, 0);
   
   PMO_DB2M.SC_STATE := RollupEV(h_id, h_bitflag);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_ROLLUP_EV', 30+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_ROLLUP_EV(31) : evalue.sql(782) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_EV', 37+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_EV(38) : evalue.sql(789) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_EV', 49+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_EV(50) : evalue.sql(801) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_ROLLUP_EV;
ALTER FUNCTION SP_ROLLUP_EV COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION FC_STR_SECURITY ==
CREATE OR REPLACE FUNCTION FC_STR_SECURITY
(
     h_resource_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_id VARCHAR2(32);
   h_parent_rank VARCHAR2(200);
   h_rank VARCHAR2(200);
   h_security_id VARCHAR2(2048);
-- Declare Cursors
      CURSOR C20 IS
         SELECT SECURITY_ID, PARENT_RANK
         FROM PMO_TMP_SECURITY ORDER BY PARENT_RANK;

      CURSOR C19 IS
         SELECT SECURITY_ID FROM PMO_TMP_SECURITY
         WHERE (SUBSTR(h_rank, 1, CASE WHEN LENGTH(PARENT_RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(PARENT_RANK) END) = PARENT_RANK 
         AND (PARENT_RANK LIKE SUBSTR(h_rank, 1, 8) || '%'))
         ORDER BY PARENT_RANK DESC;

      CURSOR C22 IS
         SELECT a.SECURITY_ID FROM RESOURCE_ASSIGNMENTS a, TMT_WBS w
         WHERE (SUBSTR(h_rank, 1, CASE WHEN LENGTH(w.RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(w.RANK) END) = w.RANK 
         AND (w.RANK LIKE SUBSTR(h_rank, 1, 8) || '%'))
         AND w.ELEMENT_ID = a.ELEMENT_ID
         AND w.PROJECT_ID = a.PROJECT_ID
         AND a.RESOURCE_ID = h_resource_id
         AND a.RESOURCE_TYPE='0'
         ORDER BY w.RANK DESC;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_STR_SECURITY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_STR_SECURITY'), '-- ' || SYSTIMESTAMP, ':r := FC_STR_SECURITY(' || 
       '''' || h_resource_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_STR_SECURITY', 37+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_STR_SECURITY(38) : evalue.sql(855) : ' || SYSTIMESTAMP);
   END IF; 

   h_security_id := '';
   h_id := '';
   
   BEGIN
      UPDATE PMO_TMP_SECURITY S SET SECURITY_ID =
      (SELECT RA.SECURITY_ID FROM RESOURCE_ASSIGNMENTS RA WHERE RA.RESOURCE_TYPE = '0'
      AND RA.RESOURCE_ID = h_resource_id AND RA.ELEMENT_ID = S.PARENT_ID );
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_STR_SECURITY', 57+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_STR_SECURITY(58) : evalue.sql(875) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_resource_id: %s\n",SC_STATE,h_resource_id);
   
   OPEN C20;
   
   LOOP
      FETCH C20 INTO h_id, h_parent_rank;
      IF C20%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      --trace_printf("***** h_parent_rank: %s, h_id: %d:%s\n",h_parent_rank,h_id_Ind,h_id);
      
      If h_id IS NULL Then
         h_id := '';
      End If;
      
      If Coalesce(Length(h_parent_rank), 0) > 8 Then
         h_rank := SubStr(h_parent_rank, 1, Coalesce(Length(h_parent_rank), 0) - 4);
         
         OPEN C19;
         
         FETCH C19 INTO h_security_id;
         IF C19%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         --trace_printf("SC_STATE: %d, h_security_id: %d:%s, h_id: %d:%s\n",SC_STATE,h_security_id_Ind,h_security_id,h_id_Ind,h_id);
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            BEGIN
               CLOSE C19 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_STR_SECURITY', 103+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_STR_SECURITY(104) : evalue.sql(921) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If h_security_id IS NULL Then
               h_security_id := '';
            End If;
            
            If Coalesce(Length(h_id), 0) > 0 Then
               h_security_id := h_security_id || h_id;
               --                  InitializeHost(SC_PARAM(h_id),"");
            End If;
            --trace_printf("1_* h_security_id: %d:%s\n",h_security_id_Ind);
         Else
            --strcpy(h_security_id, h_id);
            h_security_id := h_id;
            
            BEGIN
               CLOSE C19 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_STR_SECURITY', 129+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_STR_SECURITY(130) : evalue.sql(947) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            OPEN C22;
            
            LOOP
               FETCH C22 INTO h_id;
               IF C22%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
               
               If Coalesce(Length(h_id), 0) > 0 Then
                  h_security_id := h_security_id || h_id;
                  h_id := '';
               End If;
               --trace_printf("2_* h_security_id: %s,h_id: %d:%s\n",h_security_id,h_id_Ind,h_id);
            END LOOP;
            
            BEGIN
               CLOSE C22 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_STR_SECURITY', 162+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_STR_SECURITY(163) : evalue.sql(980) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         BEGIN
            UPDATE PMO_TMP_SECURITY SET
            SECURITY_ID = h_security_id
            WHERE PARENT_RANK = h_parent_rank;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_STR_SECURITY', 182+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_STR_SECURITY(183) : evalue.sql(1000) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_parent_rank: %s, h_security_id: %d:%s\n",SC_STATE,h_parent_rank,h_security_id_Ind,h_security_id);
      End If;
   END LOOP;
   --end of while
   
   BEGIN
      CLOSE C20 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_STR_SECURITY', 199+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_STR_SECURITY(200) : evalue.sql(1017) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_STR_SECURITY', 205+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_STR_SECURITY(206) : evalue.sql(1023) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_STR_SECURITY;
ALTER FUNCTION FC_STR_SECURITY COMPILE DEBUG;

/*********************************************************************************/

Prompt == FUNCTION FC_TMP_SECURITY ==
CREATE OR REPLACE FUNCTION FC_TMP_SECURITY
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_resource_id IN OUT NOCOPY VARCHAR2
   , sh_h_type_id PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := Coalesce(sh_h_type_id, 0);
   h_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_c_id VARCHAR2(32);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_TMP_SECURITY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_TMP_SECURITY'), '-- ' || SYSTIMESTAMP, ':r := FC_TMP_SECURITY(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_resource_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_type_id), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_TMP_SECURITY', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_TMP_SECURITY(21) : evalue.sql(1054) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("*** h_type_id: %d, h_resource_id: %d:%s, h_project_id: %d:%s\n", h_type_id,h_resource_id_Ind,h_resource_id,h_project_id_Ind,h_project_id);
   
   If h_type_id = PMO_CNST.SC_PROJECT Then
      BEGIN
         INSERT INTO PMO_TMP_SECURITY( PARENT_ID, PARENT_RANK )
         SELECT ELEMENT_ID, RANK FROM TMT_WBS
         WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TMP_SECURITY', 40+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TMP_SECURITY(41) : evalue.sql(1074) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf h_type_id = PMO_CNST.SC_DELIVERABLE Then
      BEGIN
         INSERT INTO PMO_TMP_SECURITY( PARENT_ID, PARENT_RANK )
         SELECT TW.ELEMENT_ID, TW.RANK
         FROM PMO_TMP_PRJ_DLV D JOIN TMT_WBS TW ON TW.PROJECT_ID = h_project_id AND TW.ELEMENT_ID = D.ELEMENT_ID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TMP_SECURITY', 59+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TMP_SECURITY(60) : evalue.sql(1093) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf h_type_id = PMO_CNST.SC_TASK Then
      BEGIN
         INSERT INTO PMO_TMP_SECURITY( PARENT_ID, PARENT_RANK )
         SELECT DISTINCT TW.PARENT_ID, SUBSTR(TW.RANK,1,LENGTH(TW.RANK) - 4)
         FROM RESOURCE_ASSIGNMENTS RA JOIN TMT_WBS TW
         ON TW.PROJECT_ID = RA.PROJECT_ID AND TW.ELEMENT_ID = RA.ELEMENT_ID AND TW.PARENT_ID IS NOT NuLL
         WHERE RA.PROJECT_ID = h_project_id AND RA.RESOURCE_ID = h_resource_id
         AND RA.ELEMENT_TYPE IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TMP_SECURITY', 81+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TMP_SECURITY(82) : evalue.sql(1115) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      /*      BEGIN_SQL
                  SELECT PARENT_ID INTO HOST(h_c_id) FROM session.PMO_TMP_SECURITY
                  WHERE PARENT_ID = HOST(h_project_id);
              END_SQL_CHECK;
      
              if ( SC_NODATA )
              {
                  BEGIN_SQL
                      INSERT INTO session.PMO_TMP_SECURITY( PARENT_ID, PARENT_RANK )
                      SELECT TW.ELEMENT_ID, TW.RANK
                      FROM RESOURCE_ASSIGNMENTS RA JOIN TMT_WBS TW
                          ON TW.PROJECT_ID = RA.PROJECT_ID AND TW.ELEMENT_ID = RA.ELEMENT_ID
                      WHERE RA.PROJECT_ID = HOST(h_project_id) AND RA.RESOURCE_ID = HOST(h_resource_id)
                          AND RA.ELEMENT_ID = HOST(h_project_id);
                  END_SQL_CHECK;
              }*/
   ElsIf h_type_id = 0 Then
      BEGIN
         INSERT INTO PMO_TMP_SECURITY( PARENT_ID, PARENT_RANK )
         SELECT DISTINCT TW.PARENT_ID, SUBSTR(TW.RANK,1,LENGTH(TW.RANK) - 4)
         FROM RESOURCE_ASSIGNMENTS RA
         JOIN PMO_TMP_PROJECTS_01 P ON RA.PROJECT_ID = P.ELEMENT_ID AND RA.RESOURCE_ID = h_resource_id
         JOIN TMT_WBS TW  ON TW.PROJECT_ID = RA.PROJECT_ID AND TW.ELEMENT_ID = RA.ELEMENT_ID AND TW.PARENT_ID IS NOT NuLL
         AND RA.ELEMENT_TYPE IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_TMP_SECURITY', 120+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_TMP_SECURITY(121) : evalue.sql(1154) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      /*      BEGIN_SQL
                  INSERT INTO session.PMO_TMP_SECURITY( PARENT_ID, PARENT_RANK )
                  SELECT ELEMENT_ID, RANK FROM session.PMO_TMP_PROJECTS_01
                  WHERE ELEMENT_ID NOT IN (SELECT PARENT_ID FROM session.PMO_TMP_SECURITY);
              END_SQL_CHECK;*/
   End If;
   
   /**haleh    the following section generates error in oracle*/
   /*  BEGIN_SQL
           DECLARE C_TMP_SECURITY CURSOR WITH HOLD FOR
           SELECT DISTINCT TW.ELEMENT_ID, TW.RANK
           FROM RESOURCE_ASSIGNMENTS RA
               JOIN TMT_WBS TW2
                   ON TW2.PROJECT_ID = RA.PROJECT_ID AND TW2.ELEMENT_ID = RA.ELEMENT_ID AND TW2.PARENT_ID IS NOT NuLL
               JOIN TMT_WBS TW
                   ON TW.PROJECT_ID = TW2.PROJECT_ID AND TW.ELEMENT_ID = TW2.PARENT_ID
           WHERE RA.PROJECT_ID = HOST(h_project_id) AND RA.RESOURCE_ID = HOST(h_resource_id)
           ORDER BY RANK;
       END_SQL;
   
       BEGIN_SQL
           OPEN C_TMP_SECURITY;
       END_SQL_CHECK;
   
       while( 1==1 )
       {
           BEGIN_SQL
               FETCH C_TMP_SECURITY INTO HOST(h_id), HOST(h_rank) ;
           END_SQL_CHECK;
           if( !SC_SUCCESS) break;
   
           BEGIN_SQL
               SELECT PARENT_ID INTO HOST(h_c_id) FROM session.PMO_TMP_SECURITY WHERE PARENT_RANK = HOST(h_rank);
           END_SQL_CHECK;
   
           if ( SC_NODATA )
           {
               BEGIN_SQL
                   INSERT INTO session.PMO_TMP_SECURITY( PROJECT_ID, PARENT_ID, PARENT_RANK, SECURITY_ID)
                   VALUES( HOST(h_project_id), HOST(h_id), HOST(h_rank), '');
               END_SQL_CHECK;
   //trace_printf("SC_STATE: %d, h_project_id: %s, h_id: %s, h_rank: %s\n", SC_STATE,h_project_id,h_id,h_rank);
           }
       }
   
       BEGIN_SQL
           CLOSE C_TMP_SECURITY WITH RELEASE;
       END_SQL_CHECK;*/
   
   PMO_DB2M.SC_STATE := FC_STR_SECURITY(h_resource_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_TMP_SECURITY', 177+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_TMP_SECURITY(178) : evalue.sql(1211) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_TMP_SECURITY;
ALTER FUNCTION FC_TMP_SECURITY COMPILE DEBUG;

/*****************************************************************************************/

--BOOL_RETURN_CODE FC_WKF_SECUR_ROLLUP

--(

--  SC_CHAR_PARAM   h_project_id[SC_H_ID],

--  short       h_project_id_Ind

--)

--{

--  PMO_BEGIN_FUNCTION(FC_WKF_SECUR_ROLLUP);

--

--  SC_CHAR(h_id, SC_H_ID);

--  SC_CHAR(h_parent_rank, SC_H_FULL_RANK);

--  SC_CHAR(h_rank, SC_H_FULL_RANK);

--  SC_CHAR(h_security_id, 2049);

--  SC_LONG(h_count);

--

--  BEGIN_SQL

--      DECLARE CZZZ CURSOR WITH HOLD FOR

--      SELECT RANK, SUBSTR(RANK, 1, LENGTH(RANK) - 4)

--      FROM session.PMO_WKF_SECURITY

--      WHERE ELEMENT_ID <> HOST(h_project_id)

--      ORDER BY RANK;

--  END_SQL;

--

--  BEGIN_SQL

--      OPEN CZZZ;

--  END_SQL_CHECK;

--

--  while( 1 == 1 )

--  {

--

--      BEGIN_SQL

--          FETCH CZZZ INTO HOST(h_rank), HOST(h_parent_rank);

--      END_SQL_CHECK;

--      if (SC_NOSUCCESS) break;

--

--      BEGIN_SQL

--          SELECT COALESCE(SECURITY_ID, '') INTO HOST(h_security_id)

--          FROM session.PMO_WKF_SECURITY

--          WHERE RANK = HOST(h_parent_rank);

--      END_SQL_CHECK;

--

--      BEGIN_SQL

--          UPDATE session.PMO_WKF_SECURITY SET

--              SECURITY_ID = HOST(h_security_id) || COALESCE(SECURITY_ID, '')

--          WHERE RANK = HOST(h_rank);

--      END_SQL_CHECK;

--

--  }

--

--  BEGIN_SQL

--      CLOSE CZZZ WITH RELEASE;

--  END_SQL_CHECK;

--

--  PMO_END_FUNCTION;

--}

/*********************************************************************************/

Prompt == FUNCTION FC_WKF_SECURITY ==
CREATE OR REPLACE FUNCTION FC_WKF_SECURITY
(
     sh_h_table_type PLS_INTEGER
   , h_id IN OUT NOCOPY VARCHAR2
   , h_resource_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_table_type PLS_INTEGER := sh_h_table_type;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_WKF_SECURITY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_WKF_SECURITY'), '-- ' || SYSTIMESTAMP, ':r := FC_WKF_SECURITY(' || 
       COALESCE(TO_CHAR(sh_h_table_type), 'NULL')
          || ', ' || '''' || h_id || ''''
          || ', ' || '''' || h_resource_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_WKF_SECURITY', 17+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_WKF_SECURITY(18) : evalue.sql(1349) : ' || SYSTIMESTAMP);
   END IF; 

   /*
       SC_CHAR(h_parent_rank, SC_H_FULL_RANK);
       SC_CHAR(h_project_id, SC_H_ID);
       SC_CHAR(h_rank, SC_H_FULL_RANK);
       SC_LONG(h_parent_table_type);
   
       SC_CHAR(h_wbs_rank, SC_H_FULL_RANK);
       SC_CHAR(h_cri_rank, SC_H_FULL_RANK);
       SC_CHAR(h_doc_rank, SC_H_FULL_RANK);
   
       BEGIN_SQL
           DECLARE GLOBAL TEMPORARY TABLE PMO_WKF_SECURITY (
               WBS_RANK    VARCHAR(192),
               CRI_RANK    VARCHAR(192),
               DOC_RANK    VARCHAR(192),
               RANK        VARCHAR(576),
               PROJECT_ID  CHAR(32),
               ELEMENT_ID  CHAR(32),
               SECURITY_ID     VARCHAR(2048)
           ) WITH REPLACE ON COMMIT PRESERVE ROWS NOT LOGGED IN USRTS32K;
       END_SQL;
   
       if( h_table_type == SC_TABLE_SCD )
       {
           BEGIN_SQL
               SELECT  TS.PROJECT_ID,
                   CASE WHEN SUBSTR(RTRIM(TS.PARENT_RANK), 1, 3) = 'WBS' THEN SC_TABLE_WBS WHEN SUBSTR(RTRIM(TS.PARENT_RANK), 1, 3) = 'CRI' THEN SC_TABLE_CRI ELSE 0 END,
                   SUBSTR(RTRIM(TS.PARENT_RANK), 5, LENGTH(TS.PARENT_RANK) - 4),
                   RTRIM(TS.RANK)
               INTO    HOST(h_project_id),
                   HOST(h_parent_table_type),
                   HOST(h_parent_rank),
                   HOST(h_rank)
               FROM TMT_ASSIGNED_SCORECARDS TS
               WHERE TS.ELEMENT_ID = HOST(h_id);
           END_SQL_CHECK;
   
           if( (SC_NODATA) || (h_parent_table_type == 0) )
               return RECORD_NOT_FOUND;
   
           InitializeHost(SC_PARAM(h_doc_rank), SC_PARAM(h_rank));
   
           if( h_parent_table_type == SC_TABLE_CRI )
           {
               InitializeHost(SC_PARAM(h_cri_rank), SC_PARAM(h_parent_rank));
   
               BEGIN_SQL
                   SELECT  SUBSTR(RTRIM(TC.PARENT_RANK), 5, LENGTH(TC.PARENT_RANK) - 4)
                   INTO    HOST(h_wbs_rank)
                   FROM TMT_CRI TC
                   WHERE TC.RANK = HOST(h_cri_rank);
               END_SQL_CHECK;
   
               BEGIN_SQL
                   INSERT INTO session.PMO_WKF_SECURITY (
                       WBS_RANK,
                       CRI_RANK,
                       DOC_RANK,
                       PROJECT_ID,
                       ELEMENT_ID,
                       SECURITY_ID )
                   SELECT  HOST(h_wbs_rank),
                       RTRIM(TC.RANK),
                       '',
                       TC.PROJECT_ID,
                       TC.ELEMENT_ID,
                       COALESCE(RA.SECURITY_ID, '')
                   FROM TMT_CRI TC
                   LEFT JOIN RESOURCE_ASSIGNMENTS RA
                       ON RA.PROJECT_ID = TC.PROJECT_ID
                       AND RA.ELEMENT_ID = TC.ELEMENT_ID
                       AND RA.RESOURCE_ID = HOST(h_resource_id)
                   WHERE TC.PROJECT_ID = HOST(h_project_id)
                       AND PMO_FAST_LOCATE(TC.RANK, HOST(h_cri_rank), 8);
               END_SQL_CHECK;
           }
           else
           {
               InitializeHost(SC_PARAM(h_wbs_rank), SC_PARAM(h_parent_rank));
               InitializeHost(SC_PARAM(h_cri_rank), "");
           }
   
           BEGIN_SQL
               INSERT INTO session.PMO_WKF_SECURITY (
                   WBS_RANK,
                   CRI_RANK,
                   DOC_RANK,
                   PROJECT_ID,
                   ELEMENT_ID,
                   SECURITY_ID )
               SELECT  HOST(h_wbs_rank),
                   HOST(h_cri_rank),
                   RTRIM(TS.RANK),
                   TS.PROJECT_ID,
                   TS.ELEMENT_ID,
                   COALESCE(RA.SECURITY_ID, '')
               FROM TMT_ASSIGNED_SCORECARDS TS
               LEFT JOIN RESOURCE_ASSIGNMENTS RA
                   ON RA.PROJECT_ID = TS.PROJECT_ID
                   AND RA.ELEMENT_ID = TS.ELEMENT_ID
                   AND RA.RESOURCE_ID = HOST(h_resource_id)
               WHERE TS.PROJECT_ID = HOST(h_project_id)
                   AND PMO_FAST_LOCATE(TS.RANK, HOST(h_doc_rank), 8);
           END_SQL_CHECK;
       }
       else if( h_table_type == SC_TABLE_DOC )
       {
           BEGIN_SQL
               SELECT  TD.PROJECT_ID,
                   CASE WHEN SUBSTR(RTRIM(TD.PARENT_RANK), 1, 3) = 'WBS' THEN SC_TABLE_WBS WHEN SUBSTR(RTRIM(TD.PARENT_RANK), 1, 3) = 'CRI' THEN SC_TABLE_CRI ELSE 0 END,
                   SUBSTR(RTRIM(TD.PARENT_RANK), 5, LENGTH(TD.PARENT_RANK) - 4),
                   RTRIM(TD.RANK)
               INTO    HOST(h_project_id),
                   HOST(h_parent_table_type),
                   HOST(h_parent_rank),
                   HOST(h_rank)
               FROM TMT_DOCUMENTS TD
               WHERE TD.ELEMENT_ID = HOST(h_id);
           END_SQL_CHECK;
   
           if( (SC_NODATA) || (h_parent_table_type == 0) )
               return RECORD_NOT_FOUND;
   
           InitializeHost(SC_PARAM(h_doc_rank), SC_PARAM(h_rank));
   
           if( h_parent_table_type == SC_TABLE_CRI )
           {
               InitializeHost(SC_PARAM(h_cri_rank), SC_PARAM(h_parent_rank));
   
               BEGIN_SQL
                   SELECT  SUBSTR(RTRIM(TC.PARENT_RANK), 5, LENGTH(TC.PARENT_RANK) - 4)
                   INTO    HOST(h_wbs_rank)
                   FROM TMT_CRI TC
                   WHERE TC.RANK = HOST(h_cri_rank);
               END_SQL_CHECK;
   
               BEGIN_SQL
                   INSERT INTO session.PMO_WKF_SECURITY (
                       WBS_RANK,
                       CRI_RANK,
                       DOC_RANK,
                       PROJECT_ID,
                       ELEMENT_ID,
                       SECURITY_ID )
                   SELECT  HOST(h_wbs_rank),
                       RTRIM(TC.RANK),
                       '',
                       TC.PROJECT_ID,
                       TC.ELEMENT_ID,
                       COALESCE(RA.SECURITY_ID, '')
                   FROM TMT_CRI TC
                   LEFT JOIN RESOURCE_ASSIGNMENTS RA
                       ON RA.PROJECT_ID = TC.PROJECT_ID
                       AND RA.ELEMENT_ID = TC.ELEMENT_ID
                       AND RA.RESOURCE_ID = HOST(h_resource_id)
                   WHERE TC.PROJECT_ID = HOST(h_project_id)
                       AND PMO_FAST_LOCATE(TC.RANK, HOST(h_cri_rank), 8);
               END_SQL_CHECK;
           }
           else
           {
               InitializeHost(SC_PARAM(h_wbs_rank), SC_PARAM(h_parent_rank));
               InitializeHost(SC_PARAM(h_cri_rank), "");
           }
   
           BEGIN_SQL
               INSERT INTO session.PMO_WKF_SECURITY (
                   WBS_RANK,
                   CRI_RANK,
                   DOC_RANK,
                   PROJECT_ID,
                   ELEMENT_ID,
                   SECURITY_ID )
               SELECT  HOST(h_wbs_rank),
                   HOST(h_cri_rank),
                   RTRIM(TD.RANK),
                   TD.PROJECT_ID,
                   TD.ELEMENT_ID,
                   COALESCE(RA.SECURITY_ID, '')
               FROM TMT_DOCUMENTS TD
               LEFT JOIN RESOURCE_ASSIGNMENTS RA
                   ON RA.PROJECT_ID = TD.PROJECT_ID
                   AND RA.ELEMENT_ID = TD.ELEMENT_ID
                   AND RA.RESOURCE_ID = HOST(h_resource_id)
               WHERE TD.PROJECT_ID = HOST(h_project_id)
                   AND PMO_FAST_LOCATE(TD.RANK, HOST(h_doc_rank), 8);
           END_SQL_CHECK;
       }
       else if( h_table_type == SC_TABLE_CRI )
       {
   
           BEGIN_SQL
               SELECT  TC.PROJECT_ID,
                   CASE WHEN SUBSTR(RTRIM(TC.PARENT_RANK), 1, 3) = 'WBS' THEN SC_TABLE_WBS WHEN SUBSTR(RTRIM(TC.PARENT_RANK), 1, 3) = 'CRI' THEN SC_TABLE_CRI ELSE 0 END,
                   SUBSTR(RTRIM(TC.PARENT_RANK), 5, LENGTH(TC.PARENT_RANK) - 4),
                   RTRIM(TC.RANK)
               INTO    HOST(h_project_id),
                   HOST(h_parent_table_type),
                   HOST(h_parent_rank),
                   HOST(h_rank)
               FROM TMT_CRI TC
               WHERE TC.ELEMENT_ID = HOST(h_id);
           END_SQL_CHECK;
   
           if( (SC_NODATA) || (h_parent_table_type == 0) )
               return RECORD_NOT_FOUND;
   
           InitializeHost(SC_PARAM(h_wbs_rank), SC_PARAM(h_parent_rank));
           InitializeHost(SC_PARAM(h_cri_rank), SC_PARAM(h_rank));
   
           BEGIN_SQL
               INSERT INTO session.PMO_WKF_SECURITY (
                   WBS_RANK,
                   CRI_RANK,
                   DOC_RANK,
                   PROJECT_ID,
                   ELEMENT_ID,
                   SECURITY_ID )
               SELECT  HOST(h_wbs_rank),
                   RTRIM(TC.RANK),
                   '',
                   TC.PROJECT_ID,
                   TC.ELEMENT_ID,
                   COALESCE(RA.SECURITY_ID, '')
               FROM TMT_CRI TC
               LEFT JOIN RESOURCE_ASSIGNMENTS RA
                   ON RA.PROJECT_ID = TC.PROJECT_ID
                   AND RA.ELEMENT_ID = TC.ELEMENT_ID
                   AND RA.RESOURCE_ID = HOST(h_resource_id)
               WHERE TC.PROJECT_ID = HOST(h_project_id)
                   AND PMO_FAST_LOCATE(TC.RANK, HOST(h_cri_rank), 8);
           END_SQL_CHECK;
       }
       else
       {
           return RECORD_NOT_FOUND;
       }
   
       BEGIN_SQL
           INSERT INTO session.PMO_WKF_SECURITY (
               WBS_RANK,
               CRI_RANK,
               DOC_RANK,
               PROJECT_ID,
               ELEMENT_ID,
               SECURITY_ID )
           SELECT  RTRIM(TW.RANK),
               '',
               '',
               TW.PROJECT_ID,
               TW.ELEMENT_ID,
               COALESCE(RA.SECURITY_ID, '')
           FROM TMT_WBS TW
           LEFT JOIN RESOURCE_ASSIGNMENTS RA
               ON RA.PROJECT_ID = TW.PROJECT_ID
               AND RA.ELEMENT_ID = TW.ELEMENT_ID
               AND RA.RESOURCE_ID = HOST(h_resource_id)
           WHERE TW.PROJECT_ID = HOST(h_project_id)
               AND PMO_FAST_LOCATE(TW.RANK, HOST(h_wbs_rank), 8);
       END_SQL_CHECK;
   
       BEGIN_SQL
           UPDATE session.PMO_WKF_SECURITY SET
               RANK = RTRIM(WBS_RANK) || RTRIM(CRI_RANK) || RTRIM(DOC_RANK)
           WHERE PROJECT_ID = HOST(h_project_id);
       END_SQL_CHECK;
   
       SC_STATE = FC_WKF_SECUR_ROLLUP(SC_PARAM(h_project_id));
   */
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_WKF_SECURITY', 291+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_WKF_SECURITY(292) : evalue.sql(1623) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_WKF_SECURITY;
ALTER FUNCTION FC_WKF_SECURITY COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION FC_LoadCalendar2 ==
CREATE OR REPLACE FUNCTION FC_LoadCalendar2
(
     h_project_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_LoadCalendar2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_LoadCalendar2'), '-- ' || SYSTIMESTAMP, ':r := FC_LoadCalendar2(' || 
       '''' || h_project_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_LoadCalendar2', 12+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_LoadCalendar2(13) : evalue.sql(1646) : ' || SYSTIMESTAMP);
   END IF; 

   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CDY';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_CDY
      SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
      FROM CALENDAR_DAY_OF_YEAR
      WHERE ELEMENT_ID = h_project_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LoadCalendar2', 36+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LoadCalendar2(37) : evalue.sql(1670) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_LoadCalendar2', 42+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_LoadCalendar2(43) : evalue.sql(1676) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_LoadCalendar2;
ALTER FUNCTION FC_LoadCalendar2 COMPILE DEBUG;

/*********************************************************************************/

Prompt == FUNCTION SP_FIX_ASSIGNMENT_DELAY ==
CREATE OR REPLACE FUNCTION SP_FIX_ASSIGNMENT_DELAY
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   e_c_project_id VARCHAR2(32);
   e_c_tsk_assignment_id VARCHAR2(32);
   h_assignment_start_dt TIMESTAMP;
   h_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_resource_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_task_start_dt TIMESTAMP;
   h_bitflag PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_ASSIGN_DELAY IS
         SELECT TA.PROJECT_ID, TA.TSK_ASSIGNMENT_ID, TA.RESOURCE_ID, TA.RESOURCE_TYPE, T.UNRESOLVED_START, TA.START_DT
         FROM PMO_FAD_PROJECTS P
         JOIN TMT_WBS T
         ON T.PROJECT_ID = P.ELEMENT_ID
         AND T.TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
         JOIN TASK_ASSIGNMENTS TA
         ON TA.PROJECT_ID = T.PROJECT_ID
         AND TA.TASK_ID = T.ELEMENT_ID
         ORDER BY T.RANK;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_FIX_ASSIGNMENT_DELAY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_FIX_ASSIGNMENT_DELAY'), '-- ' || SYSTIMESTAMP, ':r := SP_FIX_ASSIGNMENT_DELAY(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_FIX_ASSIGNMENT_DELAY', 38+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_FIX_ASSIGNMENT_DELAY(39) : evalue.sql(1725) : ' || SYSTIMESTAMP);
   END IF; 

   h_id := p_id;
   h_bitflag := p_bitflag;
   h_project_id := '';
   
   /*
        * This procedure only support project id, not folder id
        */
   
   --=====================================================================
   -- Declare Temporary Table
   --=====================================================================
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_FAD_PROJECTS';
   PMO_DB2M.SC_STATE := 0;
   --=====================================================================
   
   If (p_id IS NULL) OR (Coalesce(Length(h_id), 0) < 32) Then
      BEGIN
         INSERT INTO PMO_FAD_PROJECTS
         SELECT ELEMENT_ID
         FROM TMT_WBS
         WHERE RANK > '0000'
         AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_ASSIGNMENT_DELAY', 74+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_ASSIGNMENT_DELAY(75) : evalue.sql(1761) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT RTRIM(RANK) INTO h_rank
         FROM TMT_WBS
         WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_ASSIGNMENT_DELAY', 89+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_ASSIGNMENT_DELAY(90) : evalue.sql(1776) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_FAD_PROJECTS
         SELECT ELEMENT_ID
         FROM TMT_WBS
         WHERE RANK LIKE h_rank || '%'
         AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_ASSIGNMENT_DELAY', 110+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_ASSIGNMENT_DELAY(111) : evalue.sql(1797) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   OPEN C_ASSIGN_DELAY;
   
   LOOP
      FETCH C_ASSIGN_DELAY INTO e_c_project_id, e_c_tsk_assignment_id, h_resource_id, h_res_type, h_task_start_dt, h_assignment_start_dt;
      IF C_ASSIGN_DELAY%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      If PMO_ORASTD.strcmp(h_project_id, e_c_project_id) <> 0 Then
         h_project_id := e_c_project_id;
         PMO_DB2M.SC_STATE := FC_LoadCalendar2(h_project_id);
      End If;
      
      PMO_DB2M.SC_STATE := FC_GET_DELAY(h_resource_id, h_res_type, h_task_start_dt, h_assignment_start_dt
                                  , h_count);
      
      BEGIN
         UPDATE TASK_ASSIGNMENTS
         SET ASSIGNMENT_DELAY = h_count
         WHERE PROJECT_ID = e_c_project_id
         AND TSK_ASSIGNMENT_ID = e_c_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_ASSIGNMENT_DELAY', 152+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_ASSIGNMENT_DELAY(153) : evalue.sql(1839) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE C_ASSIGN_DELAY ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_ASSIGNMENT_DELAY', 166+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_ASSIGNMENT_DELAY(167) : evalue.sql(1853) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_FIX_ASSIGNMENT_DELAY', 172+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_FIX_ASSIGNMENT_DELAY(173) : evalue.sql(1859) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_ASSIGNMENT_DELAY', 179+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_ASSIGNMENT_DELAY(180) : evalue.sql(1866) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_ASSIGNMENT_DELAY', 191+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_ASSIGNMENT_DELAY(192) : evalue.sql(1878) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_FIX_ASSIGNMENT_DELAY;
ALTER FUNCTION SP_FIX_ASSIGNMENT_DELAY COMPILE DEBUG;

/*********************************************************************************/

Prompt == FUNCTION SP_FIX_ASSIGNMENT_WORK ==
CREATE OR REPLACE FUNCTION SP_FIX_ASSIGNMENT_WORK
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_project_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_ta_id VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_ASSIGN_WORK IS
         SELECT TA.TSK_ASSIGNMENT_ID, TA.RESOURCE_ID
         FROM TASK_ASSIGNMENTS TA
         WHERE PROJECT_ID = h_project_id
         AND SCHED_WORK <> (WORK_TODATE+ETC);

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_FIX_ASSIGNMENT_WORK') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_FIX_ASSIGNMENT_WORK'), '-- ' || SYSTIMESTAMP, ':r := SP_FIX_ASSIGNMENT_WORK(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_FIX_ASSIGNMENT_WORK', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_FIX_ASSIGNMENT_WORK(27) : evalue.sql(1921) : ' || SYSTIMESTAMP);
   END IF; 

   h_project_id := p_id;
   h_bitflag := p_bitflag;
   
   OPEN C_ASSIGN_WORK;
   
   LOOP
      FETCH C_ASSIGN_WORK INTO h_ta_id, h_resource_id;
      IF C_ASSIGN_WORK%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      BEGIN
         UPDATE TASK_ASSIGNMENTS
         SET SCHED_WORK = (WORK_TODATE+ETC)
         WHERE PROJECT_ID = h_project_id
         AND TSK_ASSIGNMENT_ID = h_ta_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_ASSIGNMENT_WORK', 60+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_ASSIGNMENT_WORK(61) : evalue.sql(1955) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --Triggers updates ROLLUP_BITS in TASK_ASSIGNMENTS, BUDGETS, TMT_WBS
      
      PMO_DB2M.SC_STATE := FC_ETC_SCHEDULE(h_ta_id, h_bitflag, h_resource_id);
   END LOOP;
   
   BEGIN
      CLOSE C_ASSIGN_WORK ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_ASSIGNMENT_WORK', 78+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_ASSIGNMENT_WORK(79) : evalue.sql(1973) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_DB2M.SC_STATE := FC_Rollup(h_project_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_FIX_ASSIGNMENT_WORK', 86+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_FIX_ASSIGNMENT_WORK(87) : evalue.sql(1981) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_project_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_ASSIGNMENT_WORK', 93+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_ASSIGNMENT_WORK(94) : evalue.sql(1988) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_ASSIGNMENT_WORK', 105+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_ASSIGNMENT_WORK(106) : evalue.sql(2000) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_FIX_ASSIGNMENT_WORK;
ALTER FUNCTION SP_FIX_ASSIGNMENT_WORK COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION FC_POPULATE_RFS_TDY ==
CREATE OR REPLACE FUNCTION FC_POPULATE_RFS_TDY RETURN PLS_INTEGER
AS
   h_db_day_of_week PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_POPULATE_RFS_TDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_POPULATE_RFS_TDY'), '-- ' || SYSTIMESTAMP, ':r := FC_POPULATE_RFS_TDY();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_POPULATE_RFS_TDY', 9+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_POPULATE_RFS_TDY(10) : evalue.sql(2026) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT SETTING_VALUE
      INTO h_db_day_of_week
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'TIMESHEET_START_WEEKDAY';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY', 23+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY(24) : evalue.sql(2040) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_db_day_of_week, 0) = 6 Then
      BEGIN
         INSERT INTO PMO_RFS_TDY
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF, (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID,  sts.WEEK_OF + 1     , (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 1     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 2     ,(sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 2     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 3     , (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) , tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 3     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 4     , (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 4     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 5     , (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 5     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 6     , (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 6     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL) > 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY', 101+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY(102) : evalue.sql(2118) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_db_day_of_week, 0) = 0 Then
      BEGIN
         INSERT INTO PMO_RFS_TDY
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 6     , (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 6     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID,  sts.WEEK_OF, (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 1     ,(sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 1     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 2     , (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) , tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 2     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 3     , (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 3     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 4     , (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 4     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 5     , (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 5     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL) > 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY', 178+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY(179) : evalue.sql(2195) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_db_day_of_week, 0) = 1 Then
      BEGIN
         INSERT INTO PMO_RFS_TDY
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 5     , (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 5     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID,  sts.WEEK_OF + 6     , (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 6     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF,(sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 1     , (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) , tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 1     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 2     , (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 2     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 3     , (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 3     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 4     , (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 4     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL) > 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY', 255+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY(256) : evalue.sql(2272) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_db_day_of_week, 0) = 2 Then
      BEGIN
         INSERT INTO PMO_RFS_TDY
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 4     , (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 4     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID,  sts.WEEK_OF + 5     , (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 5     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 6     ,(sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 6     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF, (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) , tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 1     , (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 1     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 2     , (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 2     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 3     , (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 3     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL) > 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY', 332+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY(333) : evalue.sql(2349) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_db_day_of_week, 0) = 3 Then
      BEGIN
         INSERT INTO PMO_RFS_TDY
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 3     , (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 3     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID,  sts.WEEK_OF + 4     , (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 4     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 5     ,(sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 5     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 6     , (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) , tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 6     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF, (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 1     , (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 1     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 2     , (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 2     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL) > 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY', 409+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY(410) : evalue.sql(2426) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_db_day_of_week, 0) = 4 Then
      BEGIN
         INSERT INTO PMO_RFS_TDY
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 2     , (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 2     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID,  sts.WEEK_OF + 3     , (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 3     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 4     ,(sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 4     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 5     , (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) , tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 5     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 6     , (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 6     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF, (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 1     , (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 1     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL) > 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY', 486+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY(487) : evalue.sql(2503) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_db_day_of_week, 0) = 5 Then
      BEGIN
         INSERT INTO PMO_RFS_TDY
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 1     , (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 1     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SATURDAY_REGULAR + sts.SATURDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID,  sts.WEEK_OF + 2     , (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 2     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.SUNDAY_REGULAR + sts.SUNDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 3     ,(sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 3     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.MONDAY_REGULAR + sts.MONDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 4     , (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) , tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 4     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.TUESDAY_REGULAR + sts.TUESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 5     , (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF + 5     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.WEDNESDAY_REGULAR + sts.WEDNESDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF + 6     , (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.DAY_YEAR = sts.WEEK_OF + 6     
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.THURSDAY_REGULAR + sts.THURSDAY_SPECIAL) > 0
         
         UNION ALL
         
         select sts.PROJECT_ID, sts.TSK_ASSIGNMENT_ID, sts.WEEK_OF, (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL), tdy.SCHED_MIN
         from SUMMARY_TIMESHEETS sts LEFT JOIN TASK_DAY_OF_YEAR tdy
         ON sts.PROJECT_ID = tdy.PROJECT_ID AND sts.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND DAY_YEAR = WEEK_OF
         where sts.PROJECT_ID IN
         (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND (sts.FRIDAY_REGULAR + sts.FRIDAY_SPECIAL) > 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY', 563+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY(564) : evalue.sql(2580) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --trace_printf("SC_STATE: %d, h_db_day_of_week: %d\n", SC_STATE,h_db_day_of_week);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_POPULATE_RFS_TDY', 571+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_POPULATE_RFS_TDY(572) : evalue.sql(2588) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_POPULATE_RFS_TDY;
ALTER FUNCTION FC_POPULATE_RFS_TDY COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION FC_POPULATE_RFS_MM ==
CREATE OR REPLACE FUNCTION FC_POPULATE_RFS_MM RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_POPULATE_RFS_MM') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_POPULATE_RFS_MM'), '-- ' || SYSTIMESTAMP, ':r := FC_POPULATE_RFS_MM();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_POPULATE_RFS_MM', 8+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_POPULATE_RFS_MM(9) : evalue.sql(2607) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      INSERT INTO PMO_RFS_MM (
      PROJECT_ID,
      TSK_ASSIGNMENT_ID,
      WORK_TODATE,
      START_DATE,
      FINISH )
      SELECT  PROJECT_ID,
      TSK_ASSIGNMENT_ID,
      SUM(EFFORT),
      MIN(DAY_YEAR),
      MAX(DAY_YEAR)
      FROM PMO_RFS_TDY
      GROUP BY PROJECT_ID, TSK_ASSIGNMENT_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_MM', 35+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_MM(36) : evalue.sql(2634) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_POPULATE_RFS_MM', 41+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_POPULATE_RFS_MM(42) : evalue.sql(2640) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_POPULATE_RFS_MM;
ALTER FUNCTION FC_POPULATE_RFS_MM COMPILE DEBUG;

/*****************************************************************************************/

-- Assuming ACTUAL_START in TASK_ASSIGNMENTS is good

/*****************************************************************************************/

Prompt == FUNCTION FC_POPULATE_RFS_TDY3 ==
CREATE OR REPLACE FUNCTION FC_POPULATE_RFS_TDY3 RETURN PLS_INTEGER
AS
   h_project_id VARCHAR2(32);
   h_ta_id VARCHAR2(32);
   h_SD DATE;
   h_FD DATE;
   h_d_week_of DATE;
   h_c_work_todate PLS_INTEGER := 0;
   h_c_dayweek PLS_INTEGER := 0;
   h_db_day_of_week PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_FTS_03 IS
         SELECT PROJECT_ID, TSK_ASSIGNMENT_ID, TO_DATE(SubStr(ACTUAL_START, 1, 10)), PMO_ORASTD.DAYOFWEEK(ACTUAL_START)
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS);

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_POPULATE_RFS_TDY3') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_POPULATE_RFS_TDY3'), '-- ' || SYSTIMESTAMP, ':r := FC_POPULATE_RFS_TDY3();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_POPULATE_RFS_TDY3', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_POPULATE_RFS_TDY3(23) : evalue.sql(2677) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT SETTING_VALUE INTO h_db_day_of_week
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'TIMESHEET_START_WEEKDAY';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY3', 35+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY3(36) : evalue.sql(2690) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("+++> SC_STATE: %d, h_db_day_of_week: %d %d\n", SC_STATE,h_db_day_of_week_Ind,h_db_day_of_week);
   
   OPEN C_FTS_03;
   
   LOOP
      FETCH C_FTS_03 INTO h_project_id, h_ta_id, h_SD, h_c_dayweek;
      IF C_FTS_03%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      If h_SD IS NULL Then
         BEGIN
            SELECT MIN(DAY_YEAR) INTO h_SD
            FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id AND ACTUAL_FLAG = 1;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY3', 65+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY3(66) : evalue.sql(2720) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_SD, 1, 10)), case WHEN BIT_SCHED = PMO_CNST.SC_ZERO_BIT THEN PMO_ORASTD.TIME('08:00')
               ELSE PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED)) END)
               INTO h_SD
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id AND DAY_YEAR = h_SD;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY3', 83+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY3(84) : evalue.sql(2738) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      If (Coalesce(h_db_day_of_week, 0) + 1) <> Coalesce(h_c_dayweek, 0) Then
         BEGIN
            SELECT TO_DATE(SubStr(h_SD, 1, 10)) + (h_db_day_of_week - h_c_dayweek - 6)     
            INTO h_d_week_of
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY3', 101+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY3(102) : evalue.sql(2756) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_d_week_of := h_SD;
      End If;
      
      BEGIN
         SELECT MAX(DAY_YEAR) INTO h_FD
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id AND ACTUAL_FLAG = 1;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY3', 119+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY3(120) : evalue.sql(2774) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("h_d_week_of: %s, h_FD: %s\n",h_d_week_of,h_FD); //trace_flush();
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         WHILE h_d_week_of IS NOT NULL AND PMO_ORASTD.strcmp(h_d_week_of, h_FD) <= 0 LOOP
            BEGIN
               INSERT INTO PMO_RFS_TDY2
               SELECT h_project_id, h_ta_id, h_d_week_of,
               COALESCE(SUM(SCHED_MIN * (1 - ABS(SIGN(PMO_ORASTD.DAYOFWEEK(DAY_YEAR) - 2)))),0),COALESCE(SUM(SCHED_MIN * (1 - ABS(SIGN(PMO_ORASTD.DAYOFWEEK(DAY_YEAR) - 3)))),0),
               COALESCE(SUM(SCHED_MIN * (1 - ABS(SIGN(PMO_ORASTD.DAYOFWEEK(DAY_YEAR) - 4)))),0),COALESCE(SUM(SCHED_MIN * (1 - ABS(SIGN(PMO_ORASTD.DAYOFWEEK(DAY_YEAR) - 5)))),0),
               COALESCE(SUM(SCHED_MIN * (1 - ABS(SIGN(PMO_ORASTD.DAYOFWEEK(DAY_YEAR) - 6)))),0),COALESCE(SUM(SCHED_MIN * (1 - ABS(SIGN(PMO_ORASTD.DAYOFWEEK(DAY_YEAR) - 7)))),0),
               COALESCE(SUM(SCHED_MIN * (1 - ABS(SIGN(PMO_ORASTD.DAYOFWEEK(DAY_YEAR) - 1)))),0)
               from TASK_DAY_OF_YEAR where PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id AND ACTUAL_FLAG = 1
               AND DAY_YEAR BETWEEN h_d_week_of AND TO_DATE(SubStr(h_d_week_of, 1, 10)) + 6     ;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY3', 146+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY3(147) : evalue.sql(2801) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --if(!SC_SUCCESS) break;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_d_week_of, 1, 10)) + 7      INTO h_d_week_of FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY3', 160+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY3(161) : evalue.sql(2815) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         END LOOP;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_FTS_03 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RFS_TDY3', 176+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RFS_TDY3(177) : evalue.sql(2831) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_POPULATE_RFS_TDY3', 182+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_POPULATE_RFS_TDY3(183) : evalue.sql(2837) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_POPULATE_RFS_TDY3;
ALTER FUNCTION FC_POPULATE_RFS_TDY3 COMPILE DEBUG;

/*********************************************************************************/

Prompt == FUNCTION SP_ROLLUP_FROM_ST ==
CREATE OR REPLACE FUNCTION SP_ROLLUP_FROM_ST
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_F TIMESTAMP;
   h_FD DATE;
   h_S TIMESTAMP;
   h_SD DATE;
   h_bit_flag VARCHAR2(1);
   h_c_actual_start TIMESTAMP;
   h_current_datetime TIMESTAMP;
   h_d_week_of DATE;
   h_id VARCHAR2(32);
   h_null_str VARCHAR2(1);
   h_parent_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_user VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_resource_id VARCHAR2(32);
   h_ta_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_actual_regular_cost NUMBER := 0.00;
   h_actual_regular_rate NUMBER := 0.00;
   h_actual_special_cost NUMBER := 0.00;
   h_actual_special_rate NUMBER := 0.00;
   h_billable PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_c_dayweek PLS_INTEGER := 0;
   h_c_planned_work PLS_INTEGER := 0;
   h_c_work_todate PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_db_day_of_week PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_flag PLS_INTEGER := 0;
   h_friday PLS_INTEGER := 0;
   h_milestone_percent PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_monday PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_saturday PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_sunday PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_thursday PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_tuesday PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_wednesday PLS_INTEGER := 0;
   h_week_of PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_year_of PLS_INTEGER := 0;
   h_zero PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   i_first_row PLS_INTEGER;
   null_return LONG;
-- Declare Cursors
      CURSOR C_FTA_01 IS
         SELECT PROJECT_ID, TASK_ID, RES_ASSIGNMENT_ID
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         GROUP BY PROJECT_ID, TASK_ID, RES_ASSIGNMENT_ID
         HAVING COUNT(TSK_ASSIGNMENT_ID) > 1;

      CURSOR C_FTA_02 IS
         SELECT PROJECT_ASSIGNMENT_ID, TSK_ASSIGNMENT_ID, ACTUAL_START, ETC, WORK_TODATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND RES_ASSIGNMENT_ID = h_id;

      CURSOR C_FTDY_3S IS
         SELECT ta.PROJECT_ID, ta.TASK_ID, ta.TSK_ASSIGNMENT_ID, ta.ACTUAL_START, ta.WORK_TODATE,
         ta.SCHED_WORK, ta.SCHED_START, ta.SCHED_FINISH, C_EFFORT, T_UNIT, C_UNIT, C_CONTOUR,
         ta.RESOURCE_ID, ta.RESOURCE_TYPE, ta.MILESTONE_PERCENT, ta.ASSIGN_STRING
         FROM TASK_ASSIGNMENTS ta
         where ta.PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS) AND ta.BIT_FLAG = '2' and (
         ta.WORK_TODATE <> (select COALESCE(sum(tdy.sched_min),0) from task_day_of_year tdy where tdy.project_id = ta.project_id and tdy.tsk_assignment_id = ta.tsk_assignment_id and actual_flag = 1 AND tdy.BIT_FLAG = '2')
         or
         ta.SCHED_WORK <> (select SUM(tdy0.SCHED_MIN) from TASK_DAY_OF_YEAR tdy0 where ta.PROJECT_ID = tdy0.PROJECT_ID AND ta.TSK_ASSIGNMENT_ID = tdy0.TSK_ASSIGNMENT_ID AND tdy0.BIT_FLAG = '2')
         or
         TO_DATE(SubStr(ta.SCHED_START, 1, 10)) <> (select MIN(tdy1.DAY_YEAR) from TASK_DAY_OF_YEAR tdy1 where ta.PROJECT_ID = tdy1.PROJECT_ID AND ta.TSK_ASSIGNMENT_ID = tdy1.TSK_ASSIGNMENT_ID AND tdy1.BIT_FLAG = '2')
         or
         TO_DATE(SubStr(ta.SCHED_FINISH, 1, 10)) <> (select MAX(tdy2.DAY_YEAR) from TASK_DAY_OF_YEAR tdy2 where ta.PROJECT_ID = tdy2.PROJECT_ID AND ta.TSK_ASSIGNMENT_ID = tdy2.TSK_ASSIGNMENT_ID AND tdy2.BIT_FLAG = '2')
         );

      CURSOR CAF IS
         SELECT SCHED_MIN, DAY_YEAR
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id
         FOR UPDATE OF ACTUAL_FLAG;

      CURSOR C_FTDY_3P IS
         SELECT ta.PROJECT_ID, ta.TASK_ID, ta.TSK_ASSIGNMENT_ID, ta.PLANNED_WORK, ta.PLANNED_START, ta.PLANNED_FINISH,
         C_EFFORT, T_UNIT, C_UNIT, C_CONTOUR, ta.RESOURCE_ID, ta.RESOURCE_TYPE, ta.ASSIGN_STRING
         FROM TASK_ASSIGNMENTS ta
         where ta.PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS) AND ta.BIT_FLAG = '0' and (
         ta.PLANNED_WORK <> (select SUM(tdy0.SCHED_MIN) from TASK_DAY_OF_YEAR tdy0 where ta.PROJECT_ID = tdy0.PROJECT_ID AND ta.TSK_ASSIGNMENT_ID = tdy0.TSK_ASSIGNMENT_ID AND tdy0.BIT_FLAG = '0')
         or
         TO_DATE(SubStr(ta.PLANNED_START, 1, 10)) <> (select MIN(tdy1.DAY_YEAR) from TASK_DAY_OF_YEAR tdy1 where ta.PROJECT_ID = tdy1.PROJECT_ID AND ta.TSK_ASSIGNMENT_ID = tdy1.TSK_ASSIGNMENT_ID AND tdy1.BIT_FLAG = '0')
         or
         TO_DATE(SubStr(ta.PLANNED_FINISH, 1, 10)) <> (select MAX(tdy2.DAY_YEAR) from TASK_DAY_OF_YEAR tdy2 where ta.PROJECT_ID = tdy2.PROJECT_ID AND ta.TSK_ASSIGNMENT_ID = tdy2.TSK_ASSIGNMENT_ID AND tdy2.BIT_FLAG = '0')
         );

      CURSOR C_FTS_1 IS
         SELECT  PROJECT_ID, TSK_ASSIGNMENT_ID, WEEK_OF,
         MONDAY_REGULAR, TUESDAY_REGULAR, WEDNESDAY_REGULAR,
         THURSDAY_REGULAR, FRIDAY_REGULAR, SATURDAY_REGULAR, SUNDAY_REGULAR
         FROM PMO_RFS_TDY2
         ORDER BY PROJECT_ID, TSK_ASSIGNMENT_ID, WEEK_OF;

      CURSOR CFSCH0 IS
         SELECT ta.PROJECT_ID, ta.TSK_ASSIGNMENT_ID, ta.TASK_ID, ta.RESOURCE_ID, ta.RESOURCE_TYPE,
         ta.UNRESOLVED_START, ta.UNRESOLVED_FINISH, ta.PLANNED_WORK, ta.SCHED_WORK, ta.ASSIGN_STRING, tdy.SCHED_WORK, tdy.START_DATE, tdy.FINISH_DATE
         from TASK_ASSIGNMENTS ta JOIN PMO_TMP_TDY1 tdy ON ta.PROJECT_ID = tdy.PROJECT_ID AND ta.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID
         and ta.MILESTONE_PERCENT <> 10000 and
         (ta.SCHED_WORK <> tdy.SCHED_WORK
         or
         TO_DATE(SubStr(ta.UNRESOLVED_START, 1, 10)) <> tdy.START_DATE
         or
         TO_DATE(SubStr(ta.UNRESOLVED_FINISH, 1, 10)) <> tdy.FINISH_DATE);

      CURSOR CFSCH1 IS
         SELECT ta.PROJECT_ID, ta.TSK_ASSIGNMENT_ID, ta.BIT_FLAG
         from TASK_ASSIGNMENTS ta
         WHERE ta.MILESTONE_PERCENT <> 10000 and ta.PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS) and
         EXISTS(select tdy.BIT_FLAG from task_day_of_year tdy where tdy.project_id = ta.project_id and tdy.tsk_assignment_id = ta.tsk_assignment_id and tdy.BIT_FLAG <> ta.BIT_FLAG);

      CURSOR C_RFS_1 IS
         SELECT  PROJECT_ID,
         TSK_ASSIGNMENT_ID,
         START_DATE,
         FINISH
         FROM PMO_RFS_MM
         ORDER BY PROJECT_ID;

      CURSOR CFACT IS
         SELECT ta.PROJECT_ID, ta.TSK_ASSIGNMENT_ID
         FROM TASK_ASSIGNMENTS ta LEFT JOIN PMO_RFS_TDY t on
         t.PROJECT_ID = ta.PROJECT_ID AND t.TSK_ASSIGNMENT_ID = ta.TSK_ASSIGNMENT_ID
         WHERE ta.PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS) AND
         (ta.ACTUAL_START IS NOT NuLL OR ta.WORK_TODATE > 0) AND t.TSK_ASSIGNMENT_ID IS NuLL;

      CURSOR CFACTF IS
         SELECT ta.PROJECT_ID,ta.TSK_ASSIGNMENT_ID, t.WORK_TODATE, t.FINISH
         FROM TASK_ASSIGNMENTS ta JOIN PMO_RFS_MM t ON
         t.PROJECT_ID = ta.PROJECT_ID AND t.TSK_ASSIGNMENT_ID = ta.TSK_ASSIGNMENT_ID ; --AND t.FLAG = 1

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_ROLLUP_FROM_ST') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_ROLLUP_FROM_ST'), '-- ' || SYSTIMESTAMP, ':r := SP_ROLLUP_FROM_ST(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ROLLUP_FROM_ST', 162+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_ROLLUP_FROM_ST(163) : evalue.sql(3010) : ' || SYSTIMESTAMP);
   END IF; 

   h_id := p_id;
   h_bitflag := p_bitflag;
   
   h_rec_user := 'PMO_SUPERVISOR';
   h_zero := 0;
   h_null_str := NULL;
   
   /*
        * This procedure only support project id, not folder id
        */
   
   --=====================================================================
   -- Declare Temporary Table
   --=====================================================================
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_RFS_PROJECTS';
   PMO_DB2M.SC_STATE := 0;
   
   If (p_id IS NULL) OR (Coalesce(Length(h_id), 0) < 32) Then
      BEGIN
         INSERT INTO PMO_RFS_PROJECTS
         SELECT ELEMENT_ID, RANK
         FROM TMT_WBS
         WHERE RANK > '0000'
         AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 200+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(201) : evalue.sql(3048) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT RTRIM(RANK) INTO h_rank
         FROM TMT_WBS
         WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 215+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(216) : evalue.sql(3063) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_RFS_PROJECTS
         SELECT ELEMENT_ID, RANK
         FROM TMT_WBS
         WHERE RANK LIKE h_rank || '%'
         AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 236+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(237) : evalue.sql(3084) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If Coalesce(h_bitflag, 0) = 4 Then
      i_first_row := 0;
      
      OPEN C_FTA_01;
      
      LOOP
         FETCH C_FTA_01 INTO h_project_id, h_task_id, h_id;
         IF C_FTA_01%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         i_first_row := 0;
         h_uniqueID := '';
         
         OPEN C_FTA_02;
         
         LOOP
            FETCH C_FTA_02 INTO h_parent_id, h_ta_id, h_c_actual_start, h_etc, h_work_todate;
            IF C_FTA_02%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            If i_first_row = 0 Then
               h_uniqueID := h_ta_id;
               
               If h_parent_id IS NULL Then
                  --Task Assignment
                  h_parent_id := h_ta_id;
                  i_first_row := 1;
               Else
                  --Project Assignment
                  i_first_row := 2;
               End If;
            Else
               If h_parent_id IS NULL Then
                  --Task Assignment
                  h_parent_id := h_ta_id;
               End If;
               
               --trace_printf("h_project_id: %s (%d)\n",h_project_id,h_project_id_Ind);
               --trace_printf("h_task_id: %s (%d)\n",h_task_id,h_task_id_Ind);
               --trace_printf("h_res_assignment_id: %s (%d)\n",h_id,h_id_Ind);
               --trace_printf("h_prj_assignment_id: %s (%d)\n",h_parent_id,h_parent_id_Ind);
               --trace_printf("h_tsk_assignment_id: %s (%d)\n",h_ta_id,h_ta_id_Ind);
               --trace_printf("h_c_actual_start: %s (%d)\n",h_c_actual_start,h_c_actual_start_Ind);
               --trace_flush();
               
               BEGIN
                  DELETE FROM TASK_ASSIGNMENTS
                  WHERE PROJECT_ID = h_project_id
                  AND TSK_ASSIGNMENT_ID = h_ta_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 313+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(314) : evalue.sql(3161) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  ETC = ETC + h_etc,
                  WORK_TODATE = WORK_TODATE + h_work_todate,
                  MILESTONE_PERCENT = CASE WHEN WORK_TODATE+ETC = 0 THEN 0 ELSE (WORK_TODATE*10000/(WORK_TODATE+ETC)) END
                  WHERE PROJECT_ID = h_project_id
                  AND TSK_ASSIGNMENT_ID = h_uniqueID;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 335+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(336) : evalue.sql(3183) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  MILESTONE_PERCENT = CASE WHEN WORK_TODATE+ETC = 0 THEN 0 ELSE (WORK_TODATE*10000/(WORK_TODATE+ETC)) END
                  WHERE PROJECT_ID = h_project_id
                  AND TSK_ASSIGNMENT_ID = h_uniqueID;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 355+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(356) : evalue.sql(3203) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET
                  PARENT_ID = h_parent_id,
                  TSK_ASSIGNMENT_ID = h_uniqueID
                  WHERE PROJECT_ID = h_project_id
                  AND TSK_ASSIGNMENT_ID = h_ta_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 376+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(377) : evalue.sql(3224) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE SUMMARY_TIMESHEETS SET
                  TSK_ASSIGNMENT_ID = h_uniqueID
                  WHERE PROJECT_ID = h_project_id
                  AND TSK_ASSIGNMENT_ID = h_ta_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 396+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(397) : evalue.sql(3244) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If h_c_actual_start IS NOT NULL Then
               h_temp_var := 3;
            Else
               h_temp_var := 0;
            End If;
            
            h_temp_var := U_ASSIGN_STRING(h_project_id, h_uniqueID, h_temp_var, 0, 0);
            
            If i_first_row = 2 Then
               h_temp_var := 3;
               h_temp_var := U_ASSIGN_STRING(h_project_id, h_parent_id, h_temp_var, 0, 0);
            End If;
         END LOOP;
         
         BEGIN
            CLOSE C_FTA_02 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 424+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(425) : evalue.sql(3272) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      END LOOP;
      
      BEGIN
         CLOSE C_FTA_01 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 438+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(439) : evalue.sql(3286) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_bitflag, 0) = 3 Then
      OPEN C_FTDY_3S;
      --trace_printf("*** SCHEDULE *****************************************************************\n");
      
      LOOP
         FETCH C_FTDY_3S INTO h_project_id, h_task_id, h_ta_id, h_c_actual_start, h_work_todate,
         h_sched_work, h_S, h_F, h_work, h_eunit, h_unit, h_contour,
         h_resource_id, h_res_type, h_milestone_percent, h_assign_string;
         IF C_FTDY_3S%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         BEGIN
            DELETE FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 472+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(473) : evalue.sql(3320) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("DELETE FROM TASK_DAY_OF_YEAR: SC_STATE: %d, h_sched_work: %d, h_work_todate: %d, h_milestone_percent: %d\n", SC_STATE,h_sched_work,h_work_todate,h_milestone_percent);
         
         h_count := 2;
         h_temp_var := 4;
         h_consider := 7;
         
         --trace_printf("h_work: %d, h_assign_string: %s\n", h_work, h_assign_string);
         -- Update TASK_DAY_OF_YEAR based on assign string
         h_temp_var := U_TSKRES2(h_ta_id, h_project_id, h_task_id, h_resource_id, h_null_str, h_null_str, h_S
                        , h_F, h_sched_work, h_work, h_eunit, h_unit, h_res_type, h_count
                        , h_temp_var, h_contour, h_zero, h_zero, h_consider, h_assign_string
                        , h_rec_user, h_per_day, 0.00, 0.00);
         --trace_printf("h_project_id: %s, h_ta_id: %s, h_temp_var: %d, h_c_actual_start: %d %s\n", h_project_id,h_ta_id,h_temp_var,h_c_actual_start_Ind,h_c_actual_start);
         
         If h_c_actual_start IS NOT NULL Then
            If (Coalesce(h_work_todate, 0) = Coalesce(h_sched_work, 0)) AND (Coalesce(
                                          h_milestone_percent, 0) = 10000) Then
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET ACTUAL_FLAG = 1
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 506+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(507) : evalue.sql(3354) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               OPEN CAF;
               
               LOOP
                  FETCH CAF INTO h_minutes, h_d_week_of;
                  IF CAF%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  If PMO_DB2M.SC_STATE <> 0 Then
                     EXIT;
                  End If;
                  
                  --trace_printf("h_d_week_of = %s, h_total = %d,h_minutes = %d\n", h_d_week_of,h_total,h_minutes);
                  h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
                  
                  If Coalesce(h_total, 0) > Coalesce(h_work_todate, 0) Then
                     If Coalesce(h_minutes, 0) >= Coalesce(h_work_todate, 0) Then
                        h_minutes := Coalesce(h_work_todate, 0);
                     Else
                        h_minutes := Coalesce(h_work_todate, 0) - Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
                     End If;
                  End If;
                  
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET ACTUAL_FLAG = 1 WHERE CURRENT OF CAF;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 547+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(548) : evalue.sql(3395) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If Coalesce(h_total, 0) >= Coalesce(h_work_todate, 0) Then
                     EXIT;
                  End If;
               END LOOP;
               
               BEGIN
                  CLOSE CAF ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 565+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(566) : evalue.sql(3413) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            h_temp_var := 1;
         Else
            h_temp_var := 0;
         End If;
         
         h_temp_var := U_ASSIGN_STRING(h_project_id, h_ta_id, h_temp_var, 0, 0);
      END LOOP;
      
      BEGIN
         CLOSE C_FTDY_3S ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 586+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(587) : evalue.sql(3434) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("*** PLAN *****************************************************************\n");
      
      OPEN C_FTDY_3P;
      
      LOOP
         FETCH C_FTDY_3P INTO h_project_id, h_task_id, h_ta_id, h_sched_work, h_S, h_F,
         h_work, h_eunit, h_unit, h_contour, h_resource_id, h_res_type, h_assign_string;
         IF C_FTDY_3P%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         BEGIN
            DELETE FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 620+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(621) : evalue.sql(3468) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_count := 0;
         h_temp_var := 4;
         h_consider := 7;
         
         -- Update TASK_DAY_OF_YEAR based on assign string
         h_temp_var := U_TSKRES2(h_ta_id, h_project_id, h_task_id, h_resource_id, h_null_str, h_null_str, h_S
                        , h_F, h_sched_work, h_work, h_eunit, h_unit, h_res_type, h_count
                        , h_temp_var, h_contour, h_zero, h_zero, h_consider, h_assign_string
                        , h_rec_user, h_per_day, 0.00, 0.00);
         --trace_printf("h_project_id: %s, h_ta_id: %s, h_temp_var: %d\n", h_project_id,h_ta_id,h_temp_var);
         
         h_temp_var := 0;
         h_temp_var := U_ASSIGN_STRING(h_project_id, h_ta_id, h_temp_var, 0, 0);
      END LOOP;
      
      BEGIN
         CLOSE C_FTDY_3P ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 648+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(649) : evalue.sql(3496) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_bitflag, 0) = 1 Then
      --=====================================================================
      -- Declare Temporary Table
      --=====================================================================
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_RFS_TDY2';
      PMO_DB2M.SC_STATE := 0;
      --=====================================================================
      
      PMO_DB2M.SC_STATE := FC_POPULATE_RFS_TDY3();
      
      h_monday := 0;
      h_tuesday := 0;
      h_wednesday := 0;
      h_thursday := 0;
      h_friday := 0;
      h_saturday := 0;
      h_sunday := 0;
      h_year_of := 0;
      h_week_of := 0;
      h_total := 0;
      h_bitflag := 0;
      h_c_work_todate := 0;
      h_sched_work := 0;
      h_etc := 0;
      h_milestone_percent := 0;
      h_uniqueID := '';
      
      BEGIN
         SELECT SYSTIMESTAMP
         INTO h_current_datetime
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 690+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(691) : evalue.sql(3538) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C_FTS_1;
      
      LOOP
         FETCH C_FTS_1 INTO h_project_id, h_ta_id, h_d_week_of,
         h_monday, h_tuesday, h_wednesday,
         h_thursday, h_friday, h_saturday, h_sunday;
         IF C_FTS_1%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         h_total := Coalesce(h_monday, 0) + Coalesce(h_tuesday, 0) + Coalesce(h_wednesday, 0) + Coalesce(
                    h_thursday, 0) + Coalesce(h_friday, 0) + Coalesce(h_saturday, 0) + Coalesce(
                    h_sunday, 0);
         --trace_printf("**************** h_total: %d\n", h_total); //trace_flush();
         If (Coalesce(Length(h_uniqueID), 0) = 0) OR (PMO_ORASTD.strcmp(h_uniqueID, h_ta_id) <> 0) Then
            --trace_printf("h_project_id: %s (%d)\n", h_project_id, h_project_id_Ind);
            --trace_printf("h_ta_id: %s (%d)\n", h_ta_id, h_ta_id_Ind);
            --trace_flush();
            COMMIT;
            
            BEGIN
               SELECT RESOURCE_ID, BILLABLE, COALESCE(REGULAR_RATE, 0.00), COALESCE(REGULAR_COST, 0.00), COALESCE(SPECIAL_RATE, 0.00), COALESCE(SPECIAL_COST, 0.00)
               INTO h_resource_id, h_billable, h_actual_regular_rate, h_actual_regular_cost, h_actual_special_rate, h_actual_special_cost
               from TASK_ASSIGNMENTS where TSK_ASSIGNMENT_ID = h_ta_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 730+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(731) : evalue.sql(3578) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d\n", SC_STATE);
            --trace_printf("h_c_dayweek: %d (%d)\n", h_c_dayweek, h_c_dayweek_Ind);
            --trace_flush();
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               --trace_printf("*** ERROR: Assignment record missing from TASK_ASSIGNMENTS: %s\n", h_ta_id); //trace_flush();
               GOTO Continue0000;
            End If;
            
            h_count := 0;
            BEGIN
               SELECT SUM(SCHED_MIN), COUNT(DAY_YEAR)
               INTO h_work_todate, h_count
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id AND ACTUAL_FLAG = 1;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 754+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(755) : evalue.sql(3602) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If Coalesce(h_count, 0) = 0 Then
               h_work_todate := 0;
            End If;
            
            h_count := 0;
            BEGIN
               SELECT SUM(SCHED_MIN), COUNT(DAY_YEAR)
               INTO h_sched_work, h_count
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 774+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(775) : evalue.sql(3622) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If Coalesce(h_count, 0) = 0 Then
               h_sched_work := 0;
            End If;
            
            If Coalesce(h_sched_work, 0) > 0 Then
               h_milestone_percent := Trunc((Coalesce(h_work_todate, 0) / Coalesce(h_sched_work, 0)) * 10000.00);
            Else
               h_milestone_percent := 0;
            End If;
            
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               WORK_TODATE         = h_work_todate,
               SCHED_WORK          = h_sched_work,
               ETC             = CASE WHEN (SCHED_WORK-WORK_TODATE) >= ETC THEN h_sched_work-h_work_todate ELSE ETC END,
               MILESTONE_PERCENT   = h_milestone_percent,
               ROLLUP_BITS         = 1
               WHERE TSK_ASSIGNMENT_ID = h_ta_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 806+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(807) : evalue.sql(3654) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            -- CALL UPDATE assign string function
            h_flag := 3;
            null_return := U_ASSIGN_STRING(h_project_id, h_ta_id, h_flag, 0, 0);
            
            h_uniqueID := h_ta_id;
            h_c_work_todate := Coalesce(h_total, 0);
         Else
            h_c_work_todate := Coalesce(h_c_work_todate, 0) + Coalesce(h_total, 0);
         End If;
         
         --trace_printf("\th_d_week_of: %s (%d)\n", h_d_week_of, h_d_week_of_Ind);
         --trace_printf("\th_total: %d (%d)\n", h_total, h_total_Ind);
         --trace_printf("\th_c_work_todate: %d (%d)\n", h_c_work_todate, h_c_work_todate_Ind);
         --trace_flush();
         
         --trace_printf("\t\th_sched_work: %d (%d)\n", h_sched_work, h_sched_work_Ind);
         --trace_printf("\t\th_work_todate: %d (%d)\n", h_work_todate, h_work_todate_Ind);
         --trace_printf("\t\th_c_work_todate: %d (%d)\n", h_c_work_todate, h_c_work_todate_Ind);
         --trace_flush();
         If (Coalesce(h_work_todate, 0) >= Coalesce(h_c_work_todate, 0)) AND (Coalesce(
                                           h_sched_work, 0) >= Coalesce(h_c_work_todate, 0)) Then
            h_etc := Coalesce(h_sched_work, 0) - Coalesce(h_c_work_todate, 0);
            h_milestone_percent := Trunc((Coalesce(h_c_work_todate, 0) / Coalesce(h_sched_work, 0)) * 10000.00);
            
            BEGIN
               UPDATE SUMMARY_TIMESHEETS SET
               MONDAY_REGULAR    = h_monday,     MONDAY_SPECIAL    = 0,
               TUESDAY_REGULAR   = h_tuesday,    TUESDAY_SPECIAL   = 0,
               WEDNESDAY_REGULAR = h_wednesday,  WEDNESDAY_SPECIAL = 0,
               THURSDAY_REGULAR  = h_thursday,   THURSDAY_SPECIAL  = 0,
               FRIDAY_REGULAR    = h_friday,     FRIDAY_SPECIAL    = 0,
               SATURDAY_REGULAR  = h_saturday,   SATURDAY_SPECIAL  = 0,
               SUNDAY_REGULAR    = h_sunday,     SUNDAY_SPECIAL    = 0,
               WEEKLY_REGULAR_TOTAL = h_total,       WEEKLY_SPECIAL_TOTAL = 0,
               TOTAL_SCHED_EFFORT   = h_sched_work,  WORK_TODATE          = h_c_work_todate,
               ETC                  = h_etc,         MILESTONE_PERCENT    = h_milestone_percent
               where PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id
               AND WEEK_OF = TO_DATE(SubStr(h_d_week_of, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 859+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(860) : evalue.sql(3707) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               --trace_printf("\t\t\th_project_id: %s (%d)\n", h_project_id, h_project_id_Ind);
               --trace_printf("\t\t\th_ta_id: %s (%d)\n", h_ta_id, h_ta_id_Ind);
               --trace_printf("\t\t\th_resource_id: %s (%d)\n", h_resource_id, h_resource_id_Ind);
               --trace_printf("\t\t\th_d_week_of: %s (%d)\n", h_d_week_of, h_d_week_of_Ind);
               --trace_printf("\t\t\th_monday: %d (%d)\n", h_monday, h_monday_Ind);
               --trace_printf("\t\t\th_tuesday: %d (%d)\n", h_tuesday, h_tuesday_Ind);
               --trace_printf("\t\t\th_wednesday: %d (%d)\n", h_wednesday, h_wednesday_Ind);
               --trace_printf("\t\t\th_thursday: %d (%d)\n", h_thursday, h_thursday_Ind);
               --trace_printf("\t\t\th_friday: %d (%d)\n", h_friday, h_friday_Ind);
               --trace_printf("\t\t\th_saturday: %d (%d)\n", h_saturday, h_saturday_Ind);
               --trace_printf("\t\t\th_sunday: %d (%d)\n", h_sunday, h_sunday_Ind);
               --trace_printf("\t\t\th_total: %d (%d)\n", h_total, h_total_Ind);
               --trace_printf("\t\t\th_sched_work: %d (%d)\n", h_sched_work, h_sched_work_Ind);
               --trace_printf("\t\t\th_c_work_todate: %d (%d)\n", h_c_work_todate, h_c_work_todate_Ind);
               --trace_printf("\t\t\th_billable: %d (%d)\n", h_billable, h_billable_Ind);
               --trace_printf("\t\t\th_etc: %d (%d)\n", h_etc, h_etc_Ind);
               --trace_printf("\t\t\th_milestone_percent: %d (%d)\n", h_milestone_percent, h_milestone_percent_Ind);
               --trace_printf("\t\t\th_actual_regular_cost: %.2f (%d)\n", h_actual_regular_cost, h_actual_regular_cost_Ind);
               --trace_printf("\t\t\th_actual_special_cost: %.2f (%d)\n", h_actual_special_cost, h_actual_special_cost_Ind);
               --trace_printf("\t\t\th_actual_regular_rate: %.2f (%d)\n", h_actual_regular_rate, h_actual_regular_rate_Ind);
               --trace_printf("\t\t\th_actual_special_rate: %.2f (%d)\n", h_actual_special_rate, h_actual_special_rate_Ind);
               --trace_printf("\t\t\th_current_datetime: %s (%d)\n", h_current_datetime, h_current_datetime_Ind);
               --trace_flush();
               BEGIN
                  INSERT INTO SUMMARY_TIMESHEETS (PROJECT_ID, TSK_ASSIGNMENT_ID, RESOURCE_ID, WEEK_OF, REC_USER,
                  MONDAY_REGULAR, MONDAY_SPECIAL, TUESDAY_REGULAR, TUESDAY_SPECIAL, WEDNESDAY_REGULAR, WEDNESDAY_SPECIAL,
                  THURSDAY_REGULAR, THURSDAY_SPECIAL, FRIDAY_REGULAR, FRIDAY_SPECIAL, SATURDAY_REGULAR, SATURDAY_SPECIAL,
                  SUNDAY_REGULAR, SUNDAY_SPECIAL, WEEKLY_REGULAR_TOTAL, WEEKLY_SPECIAL_TOTAL,
                  TOTAL_SCHED_EFFORT, WORK_TODATE, BILLABLE_WORK_TODATE, ETC, MILESTONE_PERCENT,
                  ACTUAL_REGULAR_COST, ACTUAL_SPECIAL_COST, ACTUAL_REGULAR_RATE, ACTUAL_SPECIAL_RATE,
                  SYSTEM_EXCHANGE_RATE, REC_DATETIME )
                  VALUES (h_project_id, h_ta_id, h_resource_id, h_d_week_of, h_resource_id,
                  h_monday,     0,
                  h_tuesday,    0,
                  h_wednesday,  0,
                  h_thursday,   0,
                  h_friday,     0,
                  h_saturday,   0,
                  h_sunday,     0,
                  h_total,              0,
                  h_sched_work, h_c_work_todate, CASE WHEN h_billable = 1 THEN h_c_work_todate ELSE 0 END,
                  h_etc,        h_milestone_percent,
                  h_actual_regular_cost, h_actual_special_cost,
                  h_actual_regular_rate, h_actual_special_rate,
                  1.000, h_current_datetime);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 920+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(921) : evalue.sql(3768) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("1.* SC_STATE: %d, h_week_of: %d %d\n", SC_STATE,h_week_of_Ind,h_week_of);
            End If;
         Else
            --trace_printf("*** ERROR: SC_STATE: %d, h_ta_id: %s, h_d_week_of: %s\nh_sched_work: %d, h_work_todate: %d, h_c_work_todate: %d, h_etc: %d\n",SC_STATE,h_ta_id,h_d_week_of,h_sched_work,h_work_todate,h_c_work_todate,h_etc); //trace_flush();
            NULL;
         End If;
         <<Continue0000>>
         NULL;
      END LOOP;
      
      BEGIN
         CLOSE C_FTS_1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 942+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(943) : evalue.sql(3790) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_bitflag, 0) = 2 Then
      /*
              int i = 0;
              BEGIN_SQL
                  DECLARE CFTA_70 CURSOR WITH HOLD FOR SELECT ta.PROJECT_ID, ta.TSK_ASSIGNMENT_ID
                  FROM TASK_ASSIGNMENTS ta JOIN TASK_ASSIGNMENTS_70 ta70 on
                      ta.PROJECT_ID = ta70.PROJECT_ID AND ta.TSK_ASSIGNMENT_ID = ta70.TSK_ASSIGNMENT_ID
                  WHERE ta.PROJECT_ID IN (SELECT ELEMENT_ID FROM session.PMO_RFS_PROJECTS)
                      AND (ta.ACTUAL_START IS NOT NuLL OR TA.WORK_TODATE > 0)
                  FOR READ ONLY;
              END_SQL;
      
              BEGIN_SQL
                  OPEN CFTA_70;
              END_SQL_CHECK;
      
              while(1==1)
              {
                  BEGIN_SQL
                      FETCH CFTA_70 INTO HOST(h_project_id), HOST(h_ta_id);
                  END_SQL_CHECK;
                  if (SC_NOSUCCESS) break;
      
                  BEGIN_SQL
                      INSERT INTO TASK_DAY_OF_YEAR
                      SELECT * FROM TASK_DAY_OF_YEAR_70 tdy70
                      WHERE tdy70.PROJECT_ID = HOST(h_project_id) AND tdy70.TSK_ASSIGNMENT_ID = HOST(h_ta_id)
                      AND tdy70.ACTUAL_FLAG = 1 AND tdy70.DAY_YEAR NOT IN (
                          SELECT tdy.DAY_YEAR FROM TASK_DAY_OF_YEAR tdy WHERE
                          tdy.PROJECT_ID = HOST(h_project_id) AND tdy.TSK_ASSIGNMENT_ID = HOST(h_ta_id)
                          AND tdy.DAY_YEAR = tdy70.DAY_YEAR);
                  END_SQL_CHECK;
      
                  // CALL UPDATE assign string function
                  InitializeHost(SC_PARAM(h_total), 3);
                  U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_ta_id), SC_PARAM(h_total));
      
                  i ++;
              }
      
              BEGIN_SQL
                  CLOSE CFTA_70 WITH RELEASE;
              END_SQL_CHECK;
      //trace_printf(" %d records have been processed", i);
      
          */
      NULL;
   ElsIf Coalesce(h_bitflag, 0) = 0 Then
      --Fix timesheet
      /*
              BEGIN_SQL
                  DECLARE GLOBAL TEMPORARY TABLE TMP_BAD_ST
                  LIKE SUMMARY_TIMESHEETS
                  WITH REPLACE ON COMMIT PRESERVE ROWS NOT LOGGED IN USRTS32K;
              END_SQL;
      
              BEGIN_SQL
                  DECLARE GLOBAL TEMPORARY TABLE TMP_GOOD_ST
                  LIKE SUMMARY_TIMESHEETS
                  WITH REPLACE ON COMMIT PRESERVE ROWS NOT LOGGED IN USRTS32K;
              END_SQL;
      
              BEGIN_SQL
                  INSERT INTO session.TMP_BAD_ST
                  SELECT ST.*
                  FROM session.PMO_RFS_PROJECTS P
                  JOIN TASK_ASSIGNMENTS TA
                      ON TA.PROJECT_ID = P.ELEMENT_ID
                      AND TA.BAC > 0
                  JOIN SUMMARY_TIMESHEETS ST
                      ON ST.PROJECT_ID = TA.PROJECT_ID
                      AND TA.TSK_ASSIGNMENT_ID = TA.TSK_ASSIGNMENT_ID
                      AND ST.BAC > 0
                      AND ST.WEEK_OF < DATE(TA.BASELINE1_START)
                      AND (ST.WEEKLY_REGULAR_TOTAL+ST.WEEKLY_SPECIAL_TOTAL) = 0;
              END_SQL_CHECK;
      
              BEGIN_SQL
                  INSERT INTO session.TMP_BAD_ST
                  SELECT ST.*
                  FROM session.PMO_RFS_PROJECTS P
                  JOIN TASK_ASSIGNMENTS TA
                      ON TA.PROJECT_ID = P.ELEMENT_ID
                      AND TA.BAC > 0
                  JOIN SUMMARY_TIMESHEETS ST
                      ON ST.PROJECT_ID = TA.PROJECT_ID
                      AND TA.TSK_ASSIGNMENT_ID = TA.TSK_ASSIGNMENT_ID
                      AND ST.BAC > 0
                      AND ST.WEEK_OF > DATE(TA.BASELINE1_FINISH)
                      AND (ST.WEEKLY_REGULAR_TOTAL+ST.WEEKLY_SPECIAL_TOTAL) = 0;
              END_SQL_CHECK;
      
              BEGIN_SQL
                  INSERT INTO session.TMP_GOOD_ST
                  SELECT *
                  FROM SUMMARY_TIMESHEETS
                  EXCEPT ALL
                  SELECT *
                  FROM session.TMP_BAD_ST;
              END_SQL_CHECK;
      
              BEGIN_SQL
                  SELECT COUNT(*)
                  INTO HOST(h_count)
                  FROM session.TMP_GOOD_ST;
              END_SQL_CHECK;
      
              if (h_count > 0)
              {
                  BEGIN_SQL
                      DELETE FROM SUMMARY_TIMESHEETS
                      WHERE 1 = 1;
                  END_SQL_CHECK;
      
                  BEGIN_SQL
                      INSERT INTO SUMMARY_TIMESHEETS
                      SELECT * FROM session.TMP_GOOD_ST;
                  END_SQL_CHECK;
      
                  SQL_COMMIT;
              }
      */
      BEGIN
         update summary_timesheets st
         set st.weekly_regular_total = (st.monday_regular+st.tuesday_regular+st.wednesday_regular+st.thursday_regular+st.friday_regular+st.saturday_regular+st.sunday_regular),
         st.weekly_special_total = (st.monday_special+st.tuesday_special+st.wednesday_special+st.thursday_special+st.friday_special+st.saturday_special+st.sunday_special)
         where st.PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS) AND
         (st.weekly_regular_total <> (st.monday_regular+st.tuesday_regular+st.wednesday_regular+st.thursday_regular+st.friday_regular+st.saturday_regular+st.sunday_regular)
         or
         st.weekly_special_total <> (st.monday_special+st.tuesday_special+st.wednesday_special+st.thursday_special+st.friday_special+st.saturday_special+st.sunday_special));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1086+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1087) : evalue.sql(3934) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         update summary_timesheets st1
         set st1.work_todate = (select sum(st.monday_regular+st.tuesday_regular+st.wednesday_regular+st.thursday_regular+st.friday_regular+st.saturday_regular+st.sunday_regular
         +st.monday_special+st.tuesday_special+st.wednesday_special+st.thursday_special+st.friday_special+st.saturday_special+st.sunday_special)
         from summary_timesheets st where st.project_id = st1.project_id and st.tsk_assignment_id = st1.tsk_assignment_id and st.week_of <= st1.week_of)
         where st1.PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS) AND
         st1.work_todate <> (select sum(st.monday_regular+st.tuesday_regular+st.wednesday_regular+st.thursday_regular+st.friday_regular+st.saturday_regular+st.sunday_regular
         +st.monday_special+st.tuesday_special+st.wednesday_special+st.thursday_special+st.friday_special+st.saturday_special+st.sunday_special)
         from summary_timesheets st where st.project_id = st1.project_id and st.tsk_assignment_id = st1.tsk_assignment_id and st.week_of <= st1.week_of);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1110+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1111) : evalue.sql(3958) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE SUMMARY_TIMESHEETS
         SET ETC =   CASE
         WHEN (TOTAL_SCHED_EFFORT > WORK_TODATE)
         THEN TOTAL_SCHED_EFFORT - WORK_TODATE
         ELSE 0
         END
         WHERE PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND ETC < 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1134+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1135) : evalue.sql(3982) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE SUMMARY_TIMESHEETS
         SET MILESTONE_PERCENT  = Trunc((PMO_ORASTD.TODECIMAL(WORK_TODATE,18,2)/(WORK_TODATE+ETC))*10000)
         WHERE PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND MILESTONE_PERCENT > 10000;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1154+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1155) : evalue.sql(4002) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE SUMMARY_TIMESHEETS
         SET MILESTONE_PERCENT  = Trunc((PMO_ORASTD.TODECIMAL(WORK_TODATE,18,2)/(WORK_TODATE+ETC))*10000)
         WHERE PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND WORK_TODATE > 0
         AND MILESTONE_PERCENT = 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1175+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1176) : evalue.sql(4023) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE SUMMARY_TIMESHEETS
         SET EV_ETC = (ETC/60.00)*ACTUAL_REGULAR_COST
         WHERE PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND BAC > 0
         AND EAC <> (EV_ETC+ACWP);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1196+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1197) : evalue.sql(4044) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE SUMMARY_TIMESHEETS
         SET EAC = (EV_ETC+ACWP)
         WHERE PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND BAC > 0
         AND EAC <> (EV_ETC+ACWP);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1217+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1218) : evalue.sql(4065) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE SUMMARY_TIMESHEETS ST
         SET MILESTONE_PERCENT = 0
         WHERE PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND WORK_TODATE = 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1237+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1238) : evalue.sql(4085) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE SUMMARY_TIMESHEETS ST
         SET BCWP = (BAC/10000) * MILESTONE_PERCENT
         WHERE PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND BAC > 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1257+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1258) : evalue.sql(4105) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE SUMMARY_TIMESHEETS ST
         SET BCWS = BAC
         WHERE PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         AND BAC > 0
         AND BCWS > BAC;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1278+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1279) : evalue.sql(4126) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      COMMIT;
      
      /************************************************************************/
      --trace_printf("*** Fix Schedule TDY from TA assign string *********************\n");
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDY1';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_TDY1
         select PROJECT_ID, TSK_ASSIGNMENT_ID, MIN(DAY_YEAR), MAX(DAY_YEAR), SUM(SCHED_MIN)
         from TASK_DAY_OF_YEAR WHERE PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_RFS_PROJECTS)
         group by PROJECT_ID,TSK_ASSIGNMENT_ID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1307+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1308) : evalue.sql(4155) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDY';
      PMO_DB2M.SC_STATE := 0;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLNF_0001';
      PMO_DB2M.SC_STATE := 0;
      
      h_uniqueID := '';
      
      OPEN CFSCH0;
      
      LOOP
         FETCH CFSCH0 INTO h_project_id, h_ta_id, h_task_id, h_resource_id, h_res_type,
         h_S, h_F, h_c_planned_work, h_sched_work, h_assign_string, h_total, h_SD, h_FD;
         IF CFSCH0%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         If PMO_ORASTD.strcmp(h_uniqueID, h_project_id) <> 0 Then
            BEGIN
               SELECT COALESCE(MINUTE_PER_DAY, 480) INTO h_per_day FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1345+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1346) : evalue.sql(4193) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_uniqueID := h_project_id;
            COMMIT;
         End If;
         
         BEGIN
            DELETE FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1367+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1368) : evalue.sql(4215) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_sched_work, 0) > 0 Then
            h_count := 2;
         Else
            h_sched_work := h_c_planned_work;
            h_count := 0;
         End If;
         
         h_temp_var := 4;
         h_consider := 7;
         
         -- Update TASK_DAY_OF_YEAR based on assign string
         h_temp_var := U_TSKRES2(h_ta_id, h_project_id, h_task_id, h_resource_id, h_null_str, h_null_str, h_S
                        , h_F, h_sched_work, h_zero, h_zero, h_zero, h_res_type, h_count, h_temp_var
                        , h_zero, h_zero, h_zero, h_consider, h_assign_string, h_rec_user, h_per_day
                        , 0.00, 0.00);
         
         h_temp_var := 0;
         h_temp_var := U_ASSIGN_STRING(h_project_id, h_ta_id, h_temp_var, 0, 0);
      END LOOP;
      
      BEGIN
         CLOSE CFSCH0 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1400+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1401) : evalue.sql(4248) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      /************************************************************************/
      
      OPEN CFSCH1;
      
      LOOP
         FETCH CFSCH1 INTO h_project_id, h_ta_id,h_bit_flag;
         IF CFSCH1%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         --update TDY
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET BIT_FLAG = h_bit_flag
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ta_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1434+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1435) : evalue.sql(4282) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      END LOOP;
      
      BEGIN
         CLOSE CFSCH1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1448+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1449) : evalue.sql(4296) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      /************************************************************************/
      
      --=====================================================================
      -- Declare Temporary Table
      --=====================================================================
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_RFS_TDY';
      PMO_DB2M.SC_STATE := 0;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_RFS_MM';
      PMO_DB2M.SC_STATE := 0;
      --=====================================================================
      
      PMO_DB2M.SC_STATE := FC_POPULATE_RFS_TDY();
      
      PMO_DB2M.SC_STATE := FC_POPULATE_RFS_MM();
      
      --trace_printf("111111111111111111111111111111111111111111111111111111111111111111111111111111111\n");
      h_uniqueID := '';
      
      OPEN C_RFS_1;
      
      LOOP
         FETCH C_RFS_1 INTO h_project_id,
         h_ta_id,
         h_S,
         h_F;
         IF C_RFS_1%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         If PMO_ORASTD.strcmp(h_uniqueID, h_project_id) <> 0 Then
            h_uniqueID := h_project_id;
            COMMIT;
         End If;
         
         h_flag := 0;
         
         --delete TDY 1
         BEGIN
            DELETE FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id
            AND TSK_ASSIGNMENT_ID = h_ta_id
            AND DAY_YEAR < TO_DATE(SubStr(h_S, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1513+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1514) : evalue.sql(4361) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            h_flag := 1;
         End If;
         
         --update TDY 1
         BEGIN
            UPDATE TASK_DAY_OF_YEAR
            SET ACTUAL_FLAG = 0, BIT_FLAG = '2'
            WHERE PROJECT_ID = h_project_id
            AND TSK_ASSIGNMENT_ID = h_ta_id
            AND DAY_YEAR > TO_DATE(SubStr(h_F, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1539+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1540) : evalue.sql(4387) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            h_flag := 1;
         End If;
         
         --delete TDY 2
         BEGIN
            DELETE FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id
            AND TSK_ASSIGNMENT_ID = h_ta_id
            AND DAY_YEAR NOT IN (   SELECT DAY_YEAR
            FROM PMO_RFS_TDY
            WHERE PROJECT_ID = h_project_id
            AND TSK_ASSIGNMENT_ID = h_ta_id )
            AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_S, 1, 10)) AND TO_DATE(SubStr(h_F, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1568+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1569) : evalue.sql(4416) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            h_flag := 1;
         End If;
         
         --update TDY 2
         BEGIN
            UPDATE TASK_DAY_OF_YEAR TDY SET
            SCHED_MIN = (   SELECT T.EFFORT FROM PMO_RFS_TDY T
            WHERE T.PROJECT_ID = h_project_id
            AND T.TSK_ASSIGNMENT_ID = h_ta_id
            AND T.DAY_YEAR = TDY.DAY_YEAR ),
            ACTUAL_FLAG = 1, BIT_FLAG = '2'
            WHERE PROJECT_ID = h_project_id
            AND TSK_ASSIGNMENT_ID = h_ta_id
            AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_S, 1, 10)) AND TO_DATE(SubStr(h_F, 1, 10))
            AND (DAY_YEAR IN (  SELECT T.DAY_YEAR FROM PMO_RFS_TDY T
            WHERE T.PROJECT_ID = h_project_id
            AND T.TSK_ASSIGNMENT_ID = h_ta_id
            AND T.DAY_YEAR = TDY.DAY_YEAR
            AND T.SCHED_MIN IS NOT NuLL
            AND T.EFFORT <> T.SCHED_MIN) OR ACTUAL_FLAG = 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1604+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1605) : evalue.sql(4452) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            h_flag := 1;
         End If;
         
         --insert TDY
         BEGIN
            INSERT INTO TASK_DAY_OF_YEAR (
            PROJECT_ID,
            RESOURCE_ID,
            TASK_ID,
            DAY_YEAR,
            BIT_FLAG,
            PARENT_ID,
            TSK_ASSIGNMENT_ID,
            SCHED_MIN,
            ACTUAL_FLAG )
            SELECT  t.PROJECT_ID,
            ta.RESOURCE_ID,
            ta.TASK_ID,
            t.DAY_YEAR,
            '2',
            COALESCE(ta.PROJECT_ASSIGNMENT_ID,ta.TSK_ASSIGNMENT_ID),
            t.TSK_ASSIGNMENT_ID,
            t.EFFORT,
            1
            FROM PMO_RFS_TDY t
            JOIN TASK_ASSIGNMENTS ta ON ta.PROJECT_ID = t.PROJECT_ID AND
            ta.TSK_ASSIGNMENT_ID = t.TSK_ASSIGNMENT_ID
            WHERE t.PROJECT_ID = h_project_id
            AND t.TSK_ASSIGNMENT_ID = h_ta_id
            AND t.SCHED_MIN IS NULL;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1650+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1651) : evalue.sql(4498) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            h_flag := 1;
         End If;
         
         BEGIN
            UPDATE PMO_RFS_MM
            SET FLAG = h_flag
            WHERE PROJECT_ID = h_project_id
            AND TSK_ASSIGNMENT_ID = h_ta_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1674+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1675) : evalue.sql(4522) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      END LOOP;
      
      BEGIN
         CLOSE C_RFS_1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1688+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1689) : evalue.sql(4536) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      /************************************************************************/
      --trace_printf("22222222222222222222222222222222222222222222222222222222222222222222222222222222222\n");
      h_uniqueID := '';
      
      OPEN CFACT;
      
      LOOP
         FETCH CFACT INTO h_project_id, h_ta_id;
         IF CFACT%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         If PMO_ORASTD.strcmp(h_uniqueID, h_project_id) <> 0 Then
            h_uniqueID := h_project_id;
            COMMIT;
         End If;
         
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET ACTUAL_FLAG = 0, BIT_FLAG = '2'
            WHERE PROJECT_ID = h_project_id
            AND TSK_ASSIGNMENT_ID = h_ta_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1728+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1729) : evalue.sql(4576) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            WORK_TODATE = 0,
            ACTUAL_START = Null,
            ACTUAL_FINISH = Null,
            ETC = SCHED_WORK,
            LAST_SUBMITTED = NuLL
            WHERE TSK_ASSIGNMENT_ID = h_ta_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1751+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1752) : evalue.sql(4599) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --          SQL_COMMIT;
      END LOOP;
      
      BEGIN
         CLOSE CFACT ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1767+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1768) : evalue.sql(4615) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      /************************************************************************/
      --trace_printf("3333333333333333333333333333333333333333333333333333333333333333333333\n");
      h_uniqueID := '';
      
      OPEN CFACTF;
      
      LOOP
         FETCH CFACTF INTO h_project_id, h_ta_id, h_c_work_todate, h_F;
         IF CFACTF%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         If PMO_ORASTD.strcmp(h_uniqueID, h_project_id) <> 0 Then
            --trace_printf("h_project_id: %s\n",h_project_id);
            h_uniqueID := h_project_id;
            COMMIT;
         End If;
         
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            WORK_TODATE = h_c_work_todate,
            LAST_SUBMITTED = h_F
            WHERE TSK_ASSIGNMENT_ID = h_ta_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1810+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1811) : evalue.sql(4658) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         -- CALL UPDATE assign string function
         h_bitflag := 3;
         null_return := U_ASSIGN_STRING(h_project_id, h_ta_id, h_bitflag, 0, 0);
         
         --          BEGIN_SQL
         --              UPDATE TASK_ASSIGNMENTS SET
         --                  ETC = CASE WHEN (SCHED_WORK - HOST(h_c_work_todate)) > ETC THEN ETC ELSE (SCHED_WORK - HOST(h_c_work_todate)) END,
         --                  MILESTONE_PERCENT =  HOST(h_c_work_todate) * 10000 / SCHED_WORK
         --              WHERE TSK_ASSIGNMENT_ID = HOST(h_ta_id)
         --              AND SCHED_WORK>=HOST(h_c_work_todate);
         --  //          trace_printf("\nh_c_work_todate=%d,h_ta_id=%s",h_c_work_todate,h_ta_id);
         --  //          //trace_flush();
         --          END_SQL_CHECK;
      END LOOP;
      
      BEGIN
         CLOSE CFACTF ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1838+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1839) : evalue.sql(4686) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE TASK_ASSIGNMENTS ta SET ta.LAST_SUBMITTED  = (select MAX(tdy.DAY_YEAR) from TASK_DAY_OF_YEAR tdy
         where ta.PROJECT_ID = tdy.PROJECT_ID AND ta.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.ACTUAL_FLAG = 1)
         WHERE ta.ACTUAL_START IS NOT NULL and ta.BIT_FLAG = '2' AND ta.LAST_SUBMITTED <> (select MAX(tdy.DAY_YEAR) from TASK_DAY_OF_YEAR tdy
         where ta.PROJECT_ID = tdy.PROJECT_ID AND ta.TSK_ASSIGNMENT_ID = tdy.TSK_ASSIGNMENT_ID AND tdy.ACTUAL_FLAG = 1);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1858+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1859) : evalue.sql(4706) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("******************** END ********************\n");
      
      /*************************************************************************/
      
      /***********************************************************************
              BEGIN_SQL
                  DECLARE CFACTF2 CURSOR WITH HOLD FOR SELECT ta.PROJECT_ID,ta.TSK_ASSIGNMENT_ID, ta.WORK_TODATE
                  FROM TASK_ASSIGNMENTS ta
                  where ta.work_todate > ta.sched_work
                  FOR READ ONLY;
              END_SQL;
      
              BEGIN_SQL
                  OPEN CFACTF2;
              END_SQL_CHECK;
      
              while(1==1)
              {
                  BEGIN_SQL
                      FETCH CFACTF2 INTO HOST(h_project_id), HOST(h_ta_id), HOST(h_c_work_todate);
                  END_SQL_CHECK;
                  if (SC_NOSUCCESS) break;
      //  trace_printf("2. h_project_id: %s, h_ta_id: %s\n",h_project_id,h_ta_id);
      
                  BEGIN_SQL
                      UPDATE TASK_ASSIGNMENTS SET
                          SCHED_WORK = HOST(h_c_work_todate)
                      WHERE TSK_ASSIGNMENT_ID = HOST(h_ta_id) ;
                  END_SQL_CHECK;
      
                  // CALL UPDATE assign string function
                  InitializeHost(SC_PARAM(h_bitflag), 3);
                  U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_ta_id), SC_PARAM(h_bitflag));
      
                  BEGIN_SQL
                      UPDATE TASK_ASSIGNMENTS SET
                          ETC = 0,
                          MILESTONE_PERCENT =10000
                      WHERE TSK_ASSIGNMENT_ID = HOST(h_ta_id) ;
                  END_SQL_CHECK;
              }
      
              BEGIN_SQL
                  CLOSE CFACTF2 WITH RELEASE;
              END_SQL_CHECK;
      
      *************************************************************************/
   End If;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_ROLLUP_FROM_ST', 1917+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_ROLLUP_FROM_ST(1918) : evalue.sql(4765) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1924+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1925) : evalue.sql(4772) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_FROM_ST', 1936+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_FROM_ST(1937) : evalue.sql(4784) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_ROLLUP_FROM_ST;
ALTER FUNCTION SP_ROLLUP_FROM_ST COMPILE DEBUG;

/*********************************************************************************/

Prompt == FUNCTION SP_TA_REC_DATETIME ==
CREATE OR REPLACE FUNCTION SP_TA_REC_DATETIME
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_S TIMESTAMP;
   h_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_REC_DT IS
         SELECT ELEMENT_ID, REC_DATETIME
         FROM PMO_LOCK_PROJECTS;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_TA_REC_DATETIME') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_TA_REC_DATETIME'), '-- ' || SYSTIMESTAMP, ':r := SP_TA_REC_DATETIME(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_TA_REC_DATETIME', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_TA_REC_DATETIME(26) : evalue.sql(4826) : ' || SYSTIMESTAMP);
   END IF; 

   h_id := p_id;
   h_bitflag := p_bitflag;
   
   --=====================================================================
   -- Declare Temporary Table
   --=====================================================================
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_LOCK_PROJECTS';
   PMO_DB2M.SC_STATE := 0;
   
   If (p_id IS NULL) OR (Coalesce(Length(h_id), 0) < 32) Then
      BEGIN
         INSERT INTO PMO_LOCK_PROJECTS
         SELECT ELEMENT_ID, REC_DATETIME, RANK
         FROM TMT_WBS
         WHERE RANK > '0000'
         AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
         AND REC_STATUS = 'L';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TA_REC_DATETIME', 56+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TA_REC_DATETIME(57) : evalue.sql(4857) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT RTRIM(RANK) INTO h_rank
         FROM TMT_WBS
         WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TA_REC_DATETIME', 71+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TA_REC_DATETIME(72) : evalue.sql(4872) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_LOCK_PROJECTS
         SELECT ELEMENT_ID, REC_DATETIME, RANK
         FROM TMT_WBS
         WHERE RANK LIKE h_rank || '%'
         AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
         AND REC_STATUS = 'L';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TA_REC_DATETIME', 93+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TA_REC_DATETIME(94) : evalue.sql(4894) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   OPEN C_REC_DT;
   
   LOOP
      FETCH C_REC_DT INTO h_project_id, h_S;
      IF C_REC_DT%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET
         REC_DATETIME = h_S
         WHERE PROJECT_ID = h_project_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TA_REC_DATETIME', 126+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TA_REC_DATETIME(127) : evalue.sql(4927) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE C_REC_DT ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TA_REC_DATETIME', 140+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TA_REC_DATETIME(141) : evalue.sql(4941) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_TA_REC_DATETIME', 146+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_TA_REC_DATETIME(147) : evalue.sql(4947) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TA_REC_DATETIME', 153+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TA_REC_DATETIME(154) : evalue.sql(4954) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TA_REC_DATETIME', 165+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TA_REC_DATETIME(166) : evalue.sql(4966) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_TA_REC_DATETIME;
ALTER FUNCTION SP_TA_REC_DATETIME COMPILE DEBUG;

/*********************************************************************************/

Prompt == FUNCTION SP_RE_POPULATE_CAL ==
CREATE OR REPLACE FUNCTION SP_RE_POPULATE_CAL
(
     p_table_type PLS_INTEGER
   , p_id VARCHAR2
   , p_calendar_id VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_FD DATE;
   h_SD DATE;
   h_c_calendar_id VARCHAR2(32);
   h_c_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_output PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_RPC_A IS
         SELECT ELEMENT_ID, MIN(DAY_YEAR), MAX(DAY_YEAR)
         FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID IN (   SELECT RESOURCE_ID
         FROM RESOURCES
         WHERE CALENDAR_ID IS NOT NULL )
         GROUP BY ELEMENT_ID;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_RE_POPULATE_CAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_RE_POPULATE_CAL'), '-- ' || SYSTIMESTAMP, ':r := SP_RE_POPULATE_CAL(' || 
       COALESCE(TO_CHAR(p_table_type), 'NULL')
          || ', ' || '''' || p_id || ''''
          || ', ' || '''' || p_calendar_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_RE_POPULATE_CAL', 32+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_RE_POPULATE_CAL(33) : evalue.sql(5015) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_RPC_LOG';
   PMO_DB2M.SC_STATE := 0;
   
   If Coalesce(p_table_type, 0) = PMO_CNST.SC_RESOURCE Then
      If p_id IS NULL AND p_calendar_id IS NULL Then
         OPEN C_RPC_A;
         
         LOOP
            FETCH C_RPC_A INTO h_c_id, h_SD, h_FD;
            IF C_RPC_A%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            BEGIN
               SELECT PARENT_ID
               INTO h_c_calendar_id
               FROM TMT_CALENDARS
               WHERE ELEMENT_ID = h_c_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RE_POPULATE_CAL', 67+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RE_POPULATE_CAL(68) : evalue.sql(5050) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID = h_c_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RE_POPULATE_CAL', 85+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RE_POPULATE_CAL(86) : evalue.sql(5068) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_output := 0;
            PMO_DB2M.SC_STATE := PROJECT_CALENDAR(h_c_id, h_c_calendar_id, h_SD, h_FD, h_output);
            
            If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               BEGIN
                  INSERT INTO PMO_RPC_LOG (
                  ELEMENT_ID,
                  CALENDAR_ID,
                  START_DATE,
                  FINISH_DATE )
                  VALUES (
                  COALESCE(h_c_id,'bad id'),
                  COALESCE(h_c_calendar_id,'bad calendar id'),
                  COALESCE(h_SD,TO_DATE('1899-01-01')),
                  COALESCE(h_FD,TO_DATE('1899-01-01'))  );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RE_POPULATE_CAL', 115+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RE_POPULATE_CAL(116) : evalue.sql(5098) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            COMMIT;
         END LOOP;
         
         BEGIN
            CLOSE C_RPC_A ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RE_POPULATE_CAL', 132+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RE_POPULATE_CAL(133) : evalue.sql(5115) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_c_id := p_id;
         h_c_calendar_id := p_calendar_id;
         
         BEGIN
            SELECT MIN(DAY_YEAR), MAX(DAY_YEAR)
            INTO h_SD, h_FD
            FROM CALENDAR_DAY_OF_YEAR TDY
            WHERE ELEMENT_ID = h_c_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RE_POPULATE_CAL', 151+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RE_POPULATE_CAL(152) : evalue.sql(5134) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_c_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RE_POPULATE_CAL', 169+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RE_POPULATE_CAL(170) : evalue.sql(5152) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_output := 0;
         PMO_DB2M.SC_STATE := PROJECT_CALENDAR(h_c_id, h_c_calendar_id, h_SD, h_FD, h_output);
         
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            BEGIN
               INSERT INTO PMO_RPC_LOG (
               ELEMENT_ID,
               CALENDAR_ID,
               START_DATE,
               FINISH_DATE )
               VALUES (
               COALESCE(h_c_id,'bad id'),
               COALESCE(h_c_calendar_id,'bad calendar id'),
               COALESCE(h_SD,TO_DATE('1899-01-01')),
               COALESCE(h_FD,TO_DATE('1899-01-01'))  );
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RE_POPULATE_CAL', 199+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RE_POPULATE_CAL(200) : evalue.sql(5182) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_RE_POPULATE_CAL', 208+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_RE_POPULATE_CAL(209) : evalue.sql(5191) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RE_POPULATE_CAL', 215+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RE_POPULATE_CAL(216) : evalue.sql(5198) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RE_POPULATE_CAL', 227+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RE_POPULATE_CAL(228) : evalue.sql(5210) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_RE_POPULATE_CAL;
ALTER FUNCTION SP_RE_POPULATE_CAL COMPILE DEBUG;

/*********************************************************************************/

Prompt == FUNCTION SP_FIX_TDY_BITS ==
CREATE OR REPLACE FUNCTION SP_FIX_TDY_BITS
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_F TIMESTAMP;
   h_S TIMESTAMP;
   h_dayofyear DATE;
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_ta_id VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_flag PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_FTB_1 IS
         SELECT  PROJECT_ID, TASK_ID,
         TSK_ASSIGNMENT_ID,
         BIT_FLAG,
         DAY_YEAR,
         SCHED_MIN,
         FLAG,
         UNRESOLVED_START,
         UNRESOLVED_FINISH
         FROM PMO_TMP_FTB;

      CURSOR C_FTB_2 IS
         SELECT PROJECT_ID
         FROM PMO_TMP_FTB
         GROUP BY PROJECT_ID, RANK
         ORDER BY RANK DESC;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_FIX_TDY_BITS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_FIX_TDY_BITS'), '-- ' || SYSTIMESTAMP, ':r := SP_FIX_TDY_BITS(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_FIX_TDY_BITS', 44+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_FIX_TDY_BITS(45) : evalue.sql(5271) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_FTB';
   PMO_DB2M.SC_STATE := 0;
   
   If p_id IS NULL Then
      BEGIN
         INSERT INTO PMO_TMP_FTB
         SELECT  P.RANK,
         TA.PROJECT_ID,
         TA.TASK_ID,
         TA.TSK_ASSIGNMENT_ID,
         Trunc(TDY.BIT_FLAG),
         TDY.DAY_YEAR,
         TDY.SCHED_MIN,
         CASE WHEN TDY.DAY_YEAR = TO_DATE(SubStr(TA.UNRESOLVED_START, 1, 10)) THEN 1 WHEN TDY.DAY_YEAR = TO_DATE(SubStr(TA.UNRESOLVED_FINISH, 1, 10)) THEN 2 else 0 END,
         TA.UNRESOLVED_START,
         TA.UNRESOLVED_FINISH
         FROM TASK_DAY_OF_YEAR TDY
         JOIN TASK_ASSIGNMENTS TA
         ON TA.PROJECT_ID = TDY.PROJECT_ID
         AND TA.TSK_ASSIGNMENT_ID = TDY.TSK_ASSIGNMENT_ID
         JOIN TMT_WBS P
         ON P.ELEMENT_ID = TA.PROJECT_ID
         WHERE TDY.BIT_SCHED = PMO_CNST.SC_ZERO_BIT;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 81+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(82) : evalue.sql(5308) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         INSERT INTO PMO_TMP_FTB
         SELECT  P.RANK,
         TA.PROJECT_ID,
         TA.TASK_ID,
         TA.TSK_ASSIGNMENT_ID,
         Trunc(TDY.BIT_FLAG),
         TDY.DAY_YEAR,
         TDY.SCHED_MIN,
         CASE WHEN TDY.DAY_YEAR = TO_DATE(SubStr(TA.UNRESOLVED_START, 1, 10)) THEN 1 WHEN TDY.DAY_YEAR = TO_DATE(SubStr(TA.UNRESOLVED_FINISH, 1, 10)) THEN 2 ELSE 0 END,
         TA.UNRESOLVED_START,
         TA.UNRESOLVED_FINISH
         FROM TASK_DAY_OF_YEAR TDY
         JOIN TASK_ASSIGNMENTS TA
         ON TA.PROJECT_ID = TDY.PROJECT_ID
         AND TA.TSK_ASSIGNMENT_ID = TDY.TSK_ASSIGNMENT_ID
         JOIN TMT_WBS P
         ON P.ELEMENT_ID = TA.PROJECT_ID
         WHERE TDY.BIT_SCHED = PMO_CNST.SC_ZERO_BIT
         AND TDY.PROJECT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 116+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(117) : evalue.sql(5343) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   OPEN C_FTB_1;
   
   LOOP
      FETCH C_FTB_1 INTO h_project_id, h_task_id, h_ta_id, h_bitflag, h_dayofyear, h_minutes, h_flag, h_S, h_F;
      IF C_FTB_1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      BEGIN
         SELECT BIT_SCHED INTO h_bitID
         FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_project_id
         AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 146+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(147) : evalue.sql(5373) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         GOTO Continue0000;
      End If;
      
      If Coalesce(h_minutes, 0) < 5 Then
         h_minutes := 5;
      End If;
      -- can't assign less then 5 minutes
      h_temp_var := Coalesce(h_minutes, 0) MOD 5;
      If Coalesce(h_temp_var, 0) <> 0 Then
         h_minutes := Coalesce(h_minutes, 0) + 5 - Coalesce(h_temp_var, 0);
      End If;
      PMO_STPU.clearFinishDayStruct(Coalesce(h_minutes, 0), h_bitID);
      
      BEGIN
         UPDATE TASK_DAY_OF_YEAR
         SET BIT_SCHED = h_bitID
         WHERE PROJECT_ID = h_project_id
         AND TSK_ASSIGNMENT_ID = h_ta_id
         AND DAY_YEAR = h_dayofyear;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 180+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(181) : evalue.sql(5407) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_flag, 0) = 1 Then
         --Fix Start Time
         If Coalesce(h_bitflag, 0) = 2 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = 1,
               SCHED_START = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_S, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(h_bitID)))
               WHERE PROJECT_ID = h_project_id
               AND TSK_ASSIGNMENT_ID = h_ta_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 203+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(204) : evalue.sql(5430) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = 1,
               PLANNED_START = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_S, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(h_bitID)))
               WHERE PROJECT_ID = h_project_id
               AND TSK_ASSIGNMENT_ID = h_ta_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 223+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(224) : evalue.sql(5450) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      ElsIf Coalesce(h_flag, 0) = 2 Then
         --Fix Finish Time
         If Coalesce(h_bitflag, 0) = 2 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = 1,
               SCHED_FINISH = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_F, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(h_bitID)))
               WHERE PROJECT_ID = h_project_id
               AND TSK_ASSIGNMENT_ID = h_ta_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 246+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(247) : evalue.sql(5473) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = 1,
               PLANNED_FINISH = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_F, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(h_bitID)))
               WHERE PROJECT_ID = h_project_id
               AND TSK_ASSIGNMENT_ID = h_ta_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 266+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(267) : evalue.sql(5493) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      BEGIN
         UPDATE TMT_WBS SET ROLLUP_BITS = 1
         WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 286+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(287) : evalue.sql(5513) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE BUDGETS SET ROLLUP_BITS = 1
         WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 304+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(305) : evalue.sql(5531) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      <<Continue0000>>
      NULL;
   END LOOP;
   
   BEGIN
      CLOSE C_FTB_1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 320+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(321) : evalue.sql(5547) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_FTB_2;
   
   LOOP
      FETCH C_FTB_2 INTO h_project_id;
      IF C_FTB_2%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      h_flag := FC_Rollup(h_project_id);
   END LOOP;
   
   BEGIN
      CLOSE C_FTB_2 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 349+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(350) : evalue.sql(5576) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_FIX_TDY_BITS', 355+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_FIX_TDY_BITS(356) : evalue.sql(5582) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 362+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(363) : evalue.sql(5589) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FIX_TDY_BITS', 374+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FIX_TDY_BITS(375) : evalue.sql(5601) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_FIX_TDY_BITS;
ALTER FUNCTION SP_FIX_TDY_BITS COMPILE DEBUG;

