/*********************************************************************************************/

Prompt == FUNCTION FC_CLEANS_WBS_BY_NAME ==
CREATE OR REPLACE FUNCTION FC_CLEANS_WBS_BY_NAME
(
     sh_h_search_option PLS_INTEGER
   , h_current_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_search_option PLS_INTEGER := sh_h_search_option;
   h_element_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_element_name VARCHAR2(255);
   h_ref_code VARCHAR2(50);
   h_approved PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR all_wbs_curs IS
         SELECT PROJECT_ID, ELEMENT_ID, ELEMENT_NAME, REF_CODE
         FROM PMO_WBS_VIEW
         WHERE ELEMENT_NAME <> 'WBS';

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CLEANS_WBS_BY_NAME') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CLEANS_WBS_BY_NAME'), '-- ' || SYSTIMESTAMP, ':r := FC_CLEANS_WBS_BY_NAME(' || 
       COALESCE(TO_CHAR(sh_h_search_option), 'NULL')
          || ', ' || '''' || h_current_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CLEANS_WBS_BY_NAME', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CLEANS_WBS_BY_NAME(27) : views.sql(30) : ' || SYSTIMESTAMP);
   END IF; 

   OPEN all_wbs_curs;
   
   LOOP
      h_element_name := NULL;
      h_ref_code := NULL;
      
      FETCH all_wbs_curs INTO h_project_id, h_element_id, h_element_name, h_ref_code;
      IF all_wbs_curs%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      --trace_printf("\n h_current_id:%s", h_current_id);
      --trace_printf("\n h_element_id:%s", h_element_id);
      
      If h_current_id IS NULL OR (PMO_ORASTD.strcmp(h_current_id, h_element_id) <> 0) Then
         h_approved := 1;
         h_approved := 1;
         
         If FC_FIND_KEYWORD(h_element_name, h_search_option) = 0 Then
            If FC_FIND_KEYWORD(h_ref_code, h_search_option) = 0 Then
               h_approved := 0;
            End If;
         End If;
         
         --trace_printf("\n h_search_option:%d", h_search_option);
         --trace_printf("\n h_approved:%d", h_approved);
         
         If Coalesce(h_approved, 0) = 0 Then
            BEGIN
               DELETE FROM PMO_WBS_VIEW
               WHERE ELEMENT_ID = h_element_id
               AND PROJECT_ID = h_project_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CLEANS_WBS_BY_NAME', 77+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CLEANS_WBS_BY_NAME(78) : views.sql(81) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE all_wbs_curs;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CLEANS_WBS_BY_NAME', 93+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CLEANS_WBS_BY_NAME(94) : views.sql(97) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CLEANS_WBS_BY_NAME', 100+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CLEANS_WBS_BY_NAME(101) : views.sql(104) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CLEANS_WBS_BY_NAME;
ALTER FUNCTION FC_CLEANS_WBS_BY_NAME COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION FC_GET_PRJ_VAR_CONDITION ==
CREATE OR REPLACE FUNCTION FC_GET_PRJ_VAR_CONDITION
(
     sh_h_type_id PLS_INTEGER
   , h_prj_var_condition IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := sh_h_type_id;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_PRJ_VAR_CONDITION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_PRJ_VAR_CONDITION'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_PRJ_VAR_CONDITION(' || 
       COALESCE(TO_CHAR(sh_h_type_id), 'NULL')
          || ', ' || '''' || h_prj_var_condition || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 15+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_PRJ_VAR_CONDITION(16) : views.sql(130) : ' || SYSTIMESTAMP);
   END IF; 

   h_prj_var_condition := '';
   -- budget variance
   If Coalesce(h_type_id, 0) = 1100 Then
      h_prj_var_condition := '  ((T.BASELINE1_BUDGET < T.CURRENT_BUDGET) AND (T.BASELINE1_BUDGET > 0)) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 23+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(24) : views.sql(138) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1101 Then
      h_prj_var_condition := '  (T.BASELINE1_BUDGET < T.CURRENT_BUDGET AND T.BASELINE1_BUDGET > 0 AND 10 * T.CURRENT_BUDGET < 11 * T.BASELINE1_BUDGET) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 30+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(31) : views.sql(145) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1102) AND (Coalesce(h_type_id, 0) <= 1110) Then
      h_prj_var_condition := '  (T.BASELINE1_BUDGET < T.CURRENT_BUDGET AND T.BASELINE1_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1100) - 1) 
                       || '  * T.BASELINE1_BUDGET <= 10 * (T.CURRENT_BUDGET - T.BASELINE1_BUDGET) AND 10 * (T.CURRENT_BUDGET - T.BASELINE1_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1100)) || ' * T.BASELINE1_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 41+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(42) : views.sql(156) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1112) AND (Coalesce(h_type_id, 0) <= 1120) Then
      h_prj_var_condition := '  (T.BASELINE1_BUDGET < T.CURRENT_BUDGET AND T.BASELINE1_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1100) - 2) 
                       || '  * T.BASELINE1_BUDGET <= 10 * (T.CURRENT_BUDGET - T.BASELINE1_BUDGET) AND 10 * (T.CURRENT_BUDGET - T.BASELINE1_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1100)) || ' * T.BASELINE1_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 52+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(53) : views.sql(167) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1121 Then
      h_prj_var_condition := ' (T.BASELINE1_BUDGET < T.CURRENT_BUDGET AND T.BASELINE1_BUDGET > 0 AND T.CURRENT_BUDGET >= 3 * T.BASELINE1_BUDGET) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 59+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(60) : views.sql(174) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1200 Then
      h_prj_var_condition := '  (T.BASELINE3_BUDGET < T.CURRENT_BUDGET AND T.BASELINE3_BUDGET > 0)         ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 67+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(68) : views.sql(182) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1201 Then
      h_prj_var_condition := ' (T.BASELINE3_BUDGET < T.CURRENT_BUDGET AND T.BASELINE3_BUDGET > 0 AND 10 * T.CURRENT_BUDGET < 11 * T.BASELINE3_BUDGET)          ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 74+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(75) : views.sql(189) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If (Coalesce(h_type_id, 0) >= 1202) AND (Coalesce(h_type_id, 0) <= 1210) Then
      h_prj_var_condition := '  (T.BASELINE3_BUDGET < T.CURRENT_BUDGET AND T.BASELINE3_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1200) - 1) 
                       || '  * T.BASELINE3_BUDGET <= 10 * (T.CURRENT_BUDGET - T.BASELINE3_BUDGET) AND 10 * (T.CURRENT_BUDGET - T.BASELINE3_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1200)) || ' * T.BASELINE3_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 86+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(87) : views.sql(201) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1212) AND (Coalesce(h_type_id, 0) <= 1220) Then
      h_prj_var_condition := '  (T.BASELINE3_BUDGET < T.CURRENT_BUDGET AND T.BASELINE3_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1200) - 2) 
                       || '  * T.BASELINE3_BUDGET <= 10 * (T.CURRENT_BUDGET - T.BASELINE3_BUDGET) AND 10 * (T.CURRENT_BUDGET - T.BASELINE3_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1200)) || ' * T.BASELINE3_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 97+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(98) : views.sql(212) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1221 Then
      h_prj_var_condition := ' (T.BASELINE3_BUDGET < T.CURRENT_BUDGET AND T.BASELINE3_BUDGET > 0 AND T.CURRENT_BUDGET >= 3 * T.BASELINE3_BUDGET) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 104+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(105) : views.sql(219) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1300 Then
      h_prj_var_condition := ' (T.ESTIMATED_BASELINE1_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE1_BUDGET > 0)           ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 112+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(113) : views.sql(227) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1301 Then
      h_prj_var_condition := ' (T.ESTIMATED_BASELINE1_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE1_BUDGET > 0 AND 10 * T.ESTIMATION_BUDGET < 11 * T.ESTIMATED_BASELINE1_BUDGET)          ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 119+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(120) : views.sql(234) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1302) AND (Coalesce(h_type_id, 0) <= 1310) Then
      h_prj_var_condition := 
                       ' (T.ESTIMATED_BASELINE1_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE1_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1300) - 1) 
                       || '  * T.ESTIMATED_BASELINE1_BUDGET <= 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE1_BUDGET) AND 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE1_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1300)) 
                       || ' * T.ESTIMATED_BASELINE1_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 132+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(133) : views.sql(247) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1312) AND (Coalesce(h_type_id, 0) <= 1320) Then
      h_prj_var_condition := 
                       ' (T.ESTIMATED_BASELINE1_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE1_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1300) - 2) 
                       || '  * T.ESTIMATED_BASELINE1_BUDGET <= 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE1_BUDGET) AND 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE1_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1300)) 
                       || ' * T.ESTIMATED_BASELINE1_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 145+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(146) : views.sql(260) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1321 Then
      h_prj_var_condition := ' (T.ESTIMATED_BASELINE1_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE1_BUDGET > 0 AND T.ESTIMATION_BUDGET >= 3 * T.ESTIMATED_BASELINE1_BUDGET) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 152+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(153) : views.sql(267) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1400 Then
      h_prj_var_condition := '  (T.ESTIMATED_BASELINE3_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE3_BUDGET > 0)   ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 160+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(161) : views.sql(275) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1401 Then
      h_prj_var_condition := ' (T.ESTIMATED_BASELINE3_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE3_BUDGET > 0 AND 10 * T.ESTIMATION_BUDGET < 11 * T.ESTIMATED_BASELINE3_BUDGET)   ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 167+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(168) : views.sql(282) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1402) AND (Coalesce(h_type_id, 0) <= 1410) Then
      h_prj_var_condition := 
                       '  (T.ESTIMATED_BASELINE3_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE3_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1400) - 1) 
                       || '  * T.ESTIMATED_BASELINE3_BUDGET <= 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE3_BUDGET) AND 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE3_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1400)) 
                       || ' * T.ESTIMATED_BASELINE3_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 180+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(181) : views.sql(295) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1412) AND (Coalesce(h_type_id, 0) <= 1420) Then
      h_prj_var_condition := 
                       '  (T.ESTIMATED_BASELINE3_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE3_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1400) - 2) 
                       || '  * T.ESTIMATED_BASELINE3_BUDGET <= 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE3_BUDGET) AND 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE3_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1400)) 
                       || ' * T.ESTIMATED_BASELINE3_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 193+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(194) : views.sql(308) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1421 Then
      h_prj_var_condition := '  (T.ESTIMATED_BASELINE3_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE3_BUDGET > 0 AND T.ESTIMATION_BUDGET >= 3 * T.ESTIMATED_BASELINE3_BUDGET) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 200+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(201) : views.sql(315) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1500 Then
      h_prj_var_condition := '  (T.FINISH_DT > T.BASELINE1_FINISH AND T.BASELINE1_FINISH IS NOT NuLL) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 208+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(209) : views.sql(323) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1501 Then
      h_prj_var_condition := '  (T.FINISH_DT > T.BASELINE1_FINISH AND T.BASELINE1_FINISH IS NOT NuLL AND 10 * (DATE(T.FINISH_DT) - DATE(T.BASELINE1_FINISH)) <  1  * (DATE(T.BASELINE1_FINISH) - DATE(T.BASELINE1_START))) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 215+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(216) : views.sql(330) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1502) AND (Coalesce(h_type_id, 0) <= 1510) Then
      h_prj_var_condition := 
                       '  (T.FINISH_DT > T.BASELINE1_FINISH AND T.BASELINE1_FINISH IS NOT NuLL AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1500) - 1) 
                       || '  * (TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_START, 1, 10))) <= 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10))) AND 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10))) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1500)) 
                       || '  * (TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_START, 1, 10)))) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 228+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(229) : views.sql(343) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1512) AND (Coalesce(h_type_id, 0) <= 1520) Then
      h_prj_var_condition := 
                       '  (T.FINISH_DT > T.BASELINE1_FINISH AND T.BASELINE1_FINISH IS NOT NuLL AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1500) - 2) 
                       || '  * (TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_START, 1, 10))) <= 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10))) AND 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10))) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1500)) 
                       || '  * (TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_START, 1, 10)))) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 241+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(242) : views.sql(356) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1521 Then
      h_prj_var_condition := '  (T.FINISH_DT > T.BASELINE1_FINISH AND T.BASELINE1_FINISH IS NOT NuLL AND 10 * ((DATE(T.FINISH_DT) - DATE(T.BASELINE1_FINISH))) >= 20 * ((DATE(T.BASELINE1_FINISH) - DATE(T.BASELINE1_START)))) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 248+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(249) : views.sql(363) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1600 Then
      h_prj_var_condition := '  (T.FINISH_DT > T.BASELINE3_FINISH AND T.BASELINE3_FINISH IS NOT NuLL) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 256+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(257) : views.sql(371) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1601 Then
      h_prj_var_condition := '  (T.FINISH_DT > T.BASELINE3_FINISH AND T.BASELINE3_FINISH IS NOT NuLL AND 10 * (DATE(T.FINISH_DT) - DATE(T.BASELINE3_FINISH)) <  1  * (DATE(T.BASELINE3_FINISH) - DATE(T.BASELINE3_START))) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 263+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(264) : views.sql(378) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1602) AND (Coalesce(h_type_id, 0) <= 1610) Then
      h_prj_var_condition := 
                       '  (T.FINISH_DT > T.BASELINE3_FINISH AND T.BASELINE3_FINISH IS NOT NuLL AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1600) - 1) 
                       || '  * (TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_START, 1, 10))) <= 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10))) AND 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10))) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1600)) 
                       || '  * (TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_START, 1, 10)))) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 276+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(277) : views.sql(391) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1612) AND (Coalesce(h_type_id, 0) <= 1620) Then
      h_prj_var_condition := 
                       '  (T.FINISH_DT > T.BASELINE3_FINISH AND T.BASELINE3_FINISH IS NOT NuLL AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1600) - 2) 
                       || '  * (TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_START, 1, 10))) <= 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10))) AND 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10))) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1600)) 
                       || '  * (TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_START, 1, 10)))) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 289+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(290) : views.sql(404) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1621 Then
      h_prj_var_condition := '  (T.FINISH_DT > T.BASELINE3_FINISH AND T.BASELINE3_FINISH IS NOT NuLL AND 10 * ((DATE(T.FINISH_DT) - DATE(T.BASELINE3_FINISH))) >= 20 * ((DATE(T.BASELINE3_FINISH) - DATE(T.BASELINE3_START)))) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 296+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(297) : views.sql(411) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1700 Then
      h_prj_var_condition := '  ((T.PLANNED_EFFORT + T.SCHED_EFFORT) > T.BASELINE1_EFFORT AND T.BASELINE1_EFFORT > 0)            ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 304+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(305) : views.sql(419) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1701 Then
      h_prj_var_condition := '  (T.BASELINE1_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE1_EFFORT > 0 AND 10 * (T.PLANNED_EFFORT + T.SCHED_EFFORT) < 11 * T.BASELINE1_EFFORT) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 311+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(312) : views.sql(426) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1702) AND (Coalesce(h_type_id, 0) <= 1710) Then
      h_prj_var_condition := 
                       '  (T.BASELINE1_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE1_EFFORT > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1700) - 1) 
                       || '  * T.BASELINE1_EFFORT <= 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE1_EFFORT) AND 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE1_EFFORT) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1700)) || ' * T.BASELINE1_EFFORT) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 323+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(324) : views.sql(438) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1712) AND (Coalesce(h_type_id, 0) <= 1720) Then
      h_prj_var_condition := 
                       '  (T.BASELINE1_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE1_EFFORT > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1700) - 2) 
                       || '  * T.BASELINE1_EFFORT <= 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE1_EFFORT) AND 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE1_EFFORT) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1700)) || ' * T.BASELINE1_EFFORT) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 335+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(336) : views.sql(450) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1721 Then
      h_prj_var_condition := '  (T.BASELINE1_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE1_EFFORT > 0 AND (T.PLANNED_EFFORT + T.SCHED_EFFORT) >= 3 * T.BASELINE1_EFFORT) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 342+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(343) : views.sql(457) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1800 Then
      h_prj_var_condition := ' ((T.PLANNED_EFFORT + T.SCHED_EFFORT) > T.BASELINE3_EFFORT AND T.BASELINE3_EFFORT > 0)            ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 350+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(351) : views.sql(465) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1801 Then
      h_prj_var_condition := '  (T.BASELINE3_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE3_EFFORT > 0 AND 10 * (T.PLANNED_EFFORT + T.SCHED_EFFORT) < 11 * T.BASELINE3_EFFORT) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 357+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(358) : views.sql(472) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1802) AND (Coalesce(h_type_id, 0) <= 1810) Then
      h_prj_var_condition := 
                       '  (T.BASELINE3_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE3_EFFORT > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1800) - 1) 
                       || '  * T.BASELINE3_EFFORT <= 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE3_EFFORT) AND 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE3_EFFORT) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1800)) || ' * T.BASELINE3_EFFORT) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 369+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(370) : views.sql(484) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1812) AND (Coalesce(h_type_id, 0) <= 1820) Then
      h_prj_var_condition := 
                       '  (T.BASELINE3_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE3_EFFORT > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1800) - 2) 
                       || '  * T.BASELINE3_EFFORT <= 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE3_EFFORT) AND 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE3_EFFORT) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1800)) || ' * T.BASELINE3_EFFORT) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 381+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(382) : views.sql(496) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1821 Then
      h_prj_var_condition := '  (T.BASELINE3_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE3_EFFORT > 0 AND (T.PLANNED_EFFORT + T.SCHED_EFFORT) >= 3 * T.BASELINE3_EFFORT) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 388+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(389) : views.sql(503) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 396+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(397) : views.sql(511) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_PRJ_VAR_CONDITION;
ALTER FUNCTION FC_GET_PRJ_VAR_CONDITION COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION FC_APPLY_VARIANCE_SECURITY ==
CREATE OR REPLACE FUNCTION FC_APPLY_VARIANCE_SECURITY
(
     sh_h_security_value PLS_INTEGER
   , h_resource_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_security_value PLS_INTEGER := sh_h_security_value;
   h_security_num PLS_INTEGER := 0;
   h_security_bitflag PLS_INTEGER := 0;
   h_project_id VARCHAR2(32);
   h_pure_rank VARCHAR2(200);
   null_return LONG;
-- Declare Cursors
      CURSOR prj_list IS
         SELECT DISTINCT ELEMENT_ID, RANK
         FROM PMO_TMP_PROJECTS_01
         ORDER BY RANK;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_APPLY_VARIANCE_SECURITY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_APPLY_VARIANCE_SECURITY'), '-- ' || SYSTIMESTAMP, ':r := FC_APPLY_VARIANCE_SECURITY(' || 
       COALESCE(TO_CHAR(sh_h_security_value), 'NULL')
          || ', ' || '''' || h_resource_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_APPLY_VARIANCE_SECURITY', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_APPLY_VARIANCE_SECURITY(27) : views.sql(548) : ' || SYSTIMESTAMP);
   END IF; 

   --timeLog(file,"------------------>BEGIN FC_APPLY_VARIANCE_SECURITY ");
   
   OPEN prj_list;
   
   LOOP
      FETCH prj_list INTO h_project_id, h_pure_rank;
      IF prj_list%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      h_security_num := 0;
      h_security_bitflag := 1;
      null_return := FC_GET_PRJ_SECURITY_NUM(h_pure_rank, h_security_num, h_resource_id
                                       , h_security_bitflag);
      
      If (BITAND(Coalesce(h_security_num, 0), Coalesce(h_security_value, 0))) <> Coalesce(
                                                 h_security_value, 0) Then
         BEGIN
            DELETE FROM PMO_TMP_PROJECTS_01
            WHERE ELEMENT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_APPLY_VARIANCE_SECURITY', 65+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_APPLY_VARIANCE_SECURITY(66) : views.sql(587) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE prj_list ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_APPLY_VARIANCE_SECURITY', 80+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_APPLY_VARIANCE_SECURITY(81) : views.sql(602) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_DB2M.SC_STATE := 0;
   
   --timeLog(file,"------------------>END   FC_APPLY_VARIANCE_SECURITY ");
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_APPLY_VARIANCE_SECURITY', 90+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_APPLY_VARIANCE_SECURITY(91) : views.sql(612) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_APPLY_VARIANCE_SECURITY;
ALTER FUNCTION FC_APPLY_VARIANCE_SECURITY COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION FC_FIX_WBS_VIEW_SECURITY_ID ==
CREATE OR REPLACE FUNCTION FC_FIX_WBS_VIEW_SECURITY_ID
(
     sh_h_bitflag PLS_INTEGER
   , sh_h_type PLS_INTEGER
   , h_resource_id IN OUT NOCOPY VARCHAR2
   , h_executive_security IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_type PLS_INTEGER := sh_h_type;
   h_project_id VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_pure_rank VARCHAR2(200);
-- Declare Cursors
      CURSOR CRPTLA3 IS
         SELECT DISTINCT ELEMENT_ID, RANK
         FROM PMO_TMP_PROJECTS_01;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FIX_WBS_VIEW_SECURITY_ID') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FIX_WBS_VIEW_SECURITY_ID'), '-- ' || SYSTIMESTAMP, ':r := FC_FIX_WBS_VIEW_SECURITY_ID(' || 
       COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || '''' || h_resource_id || ''''
          || ', ' || '''' || h_executive_security || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_WBS_VIEW_SECURITY_ID', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FIX_WBS_VIEW_SECURITY_ID(29) : views.sql(651) : ' || SYSTIMESTAMP);
   END IF; 

   If (Coalesce(h_bitflag, 0) = 2) OR (Coalesce(h_bitflag, 0) = 302) Then
      BEGIN
         UPDATE PMO_WBS_VIEW W
         SET (W.PARTICIPANT, W.SECURITY_ID) = (SELECT R.PARTICIPANT, R.SECURITY_ID
         FROM RESOURCE_ASSIGNMENTS R
         WHERE R.ELEMENT_ID = W.ELEMENT_ID
         AND R.RESOURCE_ID = h_resource_id
         AND TABLE_TYPE = PMO_CNST.SC_TABLE_WBS)
         WHERE 1 = 1;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 50+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(51) : views.sql(673) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (Coalesce(h_type, 0) <> 0) AND (Coalesce(h_type, 0) <> 1) AND (Coalesce(h_type, 0) <> 145) Then
      OPEN CRPTLA3;
      
      LOOP
         FETCH CRPTLA3 INTO h_project_id, h_pure_rank;
         IF CRPTLA3%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM TMP_INHERIT_SECURITY';
         PMO_DB2M.SC_STATE := 0;
         
         BEGIN
            UPDATE PMO_WBS_VIEW W
            SET W.SECURITY_ID = (SELECT R.SECURITY_ID FROM RESOURCE_ASSIGNMENTS R
            WHERE R.PROJECT_ID = h_project_id AND R.ELEMENT_ID = W.ELEMENT_ID
            AND R.RESOURCE_ID = h_resource_id)
            WHERE W.PROJECT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 91+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(92) : views.sql(714) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE PMO_WBS_VIEW W
            SET W.ASSIGNED = 1
            WHERE W.PROJECT_ID = h_project_id
            AND W.SECURITY_ID IS NOT NuLL;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 111+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(112) : views.sql(734) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO TMP_INHERIT_SECURITY (PROJECT_ID, ELEMENT_ID, RANK, SECURITY_ID)
            SELECT PROJECT_ID, ELEMENT_ID, RANK, SECURITY_ID
            FROM PMO_WBS_VIEW
            WHERE PROJECT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 131+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(132) : views.sql(754) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            h_table_type := PMO_CNST.SC_TABLE_WBS;
            PMO_DB2M.SC_STATE := FC_INHERIT_SECURITY(h_table_type, h_project_id, h_project_id, h_resource_id);
            
            BEGIN
               UPDATE PMO_WBS_VIEW W
               SET W.SECURITY_ID = (SELECT S.SECURITY_IDS FROM TMP_INHERIT_SECURITY S
               WHERE S.PROJECT_ID = h_project_id AND S.ELEMENT_ID = W.ELEMENT_ID)
               WHERE W.PROJECT_ID = h_project_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 155+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(156) : views.sql(778) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      BEGIN
         CLOSE CRPTLA3 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 169+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(170) : views.sql(792) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   BEGIN
      UPDATE PMO_WBS_VIEW T
      SET T.TYPE_ID = PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT + T.TYPE_ID
      WHERE T.PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_01 P WHERE PROJECT_FLAG >= PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 189+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(190) : views.sql(812) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE PMO_WBS_VIEW T
      SET T.SECURITY_ID = case when T.TYPE_ID in (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) and (T.SECURITY_ID = '' OR T.SECURITY_ID is Null) then PMO_CNST.SCPROJECTVISITOR_DB_STR
      when T.TYPE_ID > PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT then h_executive_security || case when (T.SECURITY_ID = '' OR T.SECURITY_ID is Null) then PMO_CNST.SCPROJECTVISITOR_DB_STR else T.SECURITY_ID end
      else T.SECURITY_ID end
      WHERE T.PROJECT_ID IN (SELECT P.ELEMENT_ID FROM PMO_TMP_PROJECTS_01 P);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 210+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(211) : views.sql(833) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_WBS_VIEW_SECURITY_ID', 218+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_WBS_VIEW_SECURITY_ID(219) : views.sql(841) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FIX_WBS_VIEW_SECURITY_ID;
ALTER FUNCTION FC_FIX_WBS_VIEW_SECURITY_ID COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION FC_WBS_PORTFOLIO_VIEW ==
CREATE OR REPLACE FUNCTION FC_WBS_PORTFOLIO_VIEW
(
     sh_h_sp_id PLS_INTEGER
   , h_wkf_resource_id IN OUT NOCOPY VARCHAR2
   , h_resource_id IN OUT NOCOPY VARCHAR2
   , h_ids IN OUT NOCOPY VARCHAR2
   , sh_h_active_record PLS_INTEGER
   , h_full_name IN OUT NOCOPY VARCHAR2
   , h_column_set IN OUT NOCOPY VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_sp_id PLS_INTEGER := sh_h_sp_id;
   h_active_record PLS_INTEGER := sh_h_active_record;
   h_tree_type PLS_INTEGER := 0;
   h_res_row_rank VARCHAR2(200);
   h_table_type PLS_INTEGER := 0;
   h_executive_security VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_can_view_pivot PLS_INTEGER := 0;
   h_include_closed_items PLS_INTEGER := 0;
   h_include_completed_items PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_id VARCHAR2(32);
   h_level_id PLS_INTEGER := 0;
   h_rank VARCHAR2(200);
   h_element_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_tmp_security_ids VARCHAR2(4000);
   h_security_ids VARCHAR2(4000);
   h_type_id PLS_INTEGER := 0;
   idList VARCHAR2(3199);
   myStmt VARCHAR2(29999);
   null_return LONG;
-- Declare Cursors
      CURSOR PORT_SECURITY IS
         SELECT DISTINCT PARENT_ID, TYPE_ID, SECURITY_ID
         FROM PMO_TMP_VIEWS_ROOT
         WHERE PARENT_ID <> 'ROOT'
         ORDER BY PARENT_ID;

      CURSOR PORT_SECURITY1 IS
         SELECT V.PARENT_ID, P.LEVEL_ID, P.RANK, V.SECURITY_ID
         FROM PMO_TMP_VIEWS_LEVEL1 V, TMT_PORTFOLIOS P
         WHERE V.PARENT_ID = P.ELEMENT_ID
         ORDER BY P.RANK DESC;

      CURSOR PORT_SECURITY2 IS
         SELECT DISTINCT PARENT_ID, TYPE_ID, SECURITY_ID
         FROM PMO_TMP_VIEWS_ROOT
         WHERE PARENT_ID <> 'ROOT'
         ORDER BY PARENT_ID;

      CURSOR PORT_SECURITY3 IS
         SELECT V.PARENT_ID, P.RANK, COALESCE(V.SECURITY_ID, '')
         FROM PMO_TMP_VIEWS_LEVEL1 V, TMT_PORTFOLIOS P
         WHERE V.PARENT_ID = P.ELEMENT_ID
         ORDER BY P.RANK DESC;

      CURSOR PORT_SECURITY4 IS
         SELECT V.SECURITY_ID
         FROM PMO_TMP_VIEWS_LEVEL1 V, TMT_PORTFOLIOS P
         WHERE V.PARENT_ID = P.ELEMENT_ID
         AND SUBSTR(P.RANK, 1, LENGTH(P.RANK)-4) = h_rank;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_WBS_PORTFOLIO_VIEW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_WBS_PORTFOLIO_VIEW'), '-- ' || SYSTIMESTAMP, ':r := FC_WBS_PORTFOLIO_VIEW(' || 
       COALESCE(TO_CHAR(sh_h_sp_id), 'NULL')
          || ', ' || '''' || h_wkf_resource_id || ''''
          || ', ' || '''' || h_resource_id || ''''
          || ', ' || '''' || h_ids || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_active_record), 'NULL')
          || ', ' || '''' || h_full_name || ''''
          || ', ' || '''' || h_column_set || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_WBS_PORTFOLIO_VIEW', 80+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_WBS_PORTFOLIO_VIEW(81) : views.sql(932) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_VIEWS_ROOT';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_VIEWS_01';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_VIEWS_LEVEL1';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      SELECT RANK || '%'
      INTO h_res_row_rank
      FROM TMT_PORTFOLIOS
      WHERE ELEMENT_ID = h_resource_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 106+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(107) : views.sql(958) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO PMO_TMP_VIEWS_ROOT(PARENT_ID, ELEMENT_ID, PARTICIPANT, SECURITY_ID, RANK, MILESTONE_PERCENT, STAGE_ID, TYPE_ID)
      SELECT COALESCE(P.PARENT_ID,'ROOT') AS PARENT_ID, T.ELEMENT_ID, A.PARTICIPANT,
      COALESCE(A.SECURITY_ID, ''), T.RANK, T.MILESTONE_PERCENT, T.STAGE_ID , T.TYPE_ID
      FROM PROJECT_ASSIGNMENTS A
      JOIN TMT_WBS T ON T.PROJECT_ID = A.PROJECT_ID
      AND T.ELEMENT_ID = A.PROJECT_ID
      AND T.REC_STATUS <> 'D'
      AND T.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
      LEFT JOIN TMT_PORTFOLIOS P ON P.RANK LIKE h_res_row_rank
      AND P.ELEMENT_ID = T.ELEMENT_ID
      WHERE A.RESOURCE_ID = h_resource_id
      AND (BITAND(A.PARTICIPANT, 3) > 0 OR (BITAND(A.PARTICIPANT, 3) = 0 AND T.PUBLISHED ='1' ));
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 134+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(135) : views.sql(986) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- add resource executive projects to the list
   h_tree_type := PMO_CNST.SC_TABLE_ATTRIBUTE;
   
   h_table_type := PMO_CNST.SC_TABLE_WBS;
   h_executive_security := NULL;
   h_bitflag := FC_GET_ACTIVE_EXECUTIVE_SECURE(h_table_type, h_resource_id, h_executive_security
                                            , h_can_view_pivot);
   h_bitflag := Coalesce(h_bitflag, 0);
   
   If Coalesce(h_bitflag, 0) = 1 Then
      null_return := FC_GET_SELECTED_PROJECTS(h_tree_type, h_ids, h_resource_id, h_bitflag);
      
      -- insert pure executive projects
      BEGIN
         INSERT INTO PMO_TMP_VIEWS_ROOT (PARENT_ID, ELEMENT_ID, PARTICIPANT, SECURITY_ID, RANK, MILESTONE_PERCENT, STAGE_ID, TYPE_ID)
         SELECT COALESCE(P.PARENT_ID, 'ROOT'), RP.ELEMENT_ID, 0 AS PARTICIPANT, COALESCE(PA.SECURITY_ID, '') AS SECURITY_ID, RP.RANK, T.MILESTONE_PERCENT, T.STAGE_ID, RP.PROJECT_FLAG
         FROM PMO_TMP_PROJECTS_01 RP
         JOIN TMT_WBS T ON T.ELEMENT_ID = RP.ELEMENT_ID
         AND T.PROJECT_ID = RP.ELEMENT_ID
         LEFT JOIN PROJECT_ASSIGNMENTS PA ON PA.PROJECT_ID = RP.ELEMENT_ID AND PA.RESOURCE_ID = h_resource_id
         LEFT JOIN TMT_PORTFOLIOS P ON P.RANK LIKE h_res_row_rank
         AND P.ELEMENT_ID = RP.ELEMENT_ID
         WHERE RP.ELEMENT_ID NOT IN (SELECT ELEMENT_ID FROM PMO_TMP_VIEWS_ROOT);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 172+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(173) : views.sql(1024) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- update assigned_executive projects
      BEGIN
         UPDATE PMO_TMP_VIEWS_ROOT
         SET TYPE_ID = TYPE_ID + PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT
         WHERE ELEMENT_ID IN (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_01);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 192+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(193) : views.sql(1044) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   BEGIN
      DELETE FROM PMO_TMP_VIEWS_ROOT
      WHERE ELEMENT_ID IN (SELECT B.ELEMENT_ID
      FROM PMO_TMP_VIEWS_ROOT A, PMO_TMP_VIEWS_ROOT B
      WHERE A.RANK < B.RANK AND ((A.RANK >= SUBSTR(B.RANK, 1, 8) AND A.RANK <= B.RANK) 
      AND SUBSTR(B.RANK,1, CASE WHEN LENGTH(A.RANK) > LENGTH(B.RANK) THEN 1 ELSE LENGTH(A.RANK) END ) = A.RANK) );
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 214+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(215) : views.sql(1066) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (BITAND(Coalesce(
         h_active_record, 0), PMO_CNST.SC_INCLUDE_CLOSED)) = PMO_CNST.SC_INCLUDE_CLOSED Then
      h_include_closed_items := 1;
   End If;
   If (BITAND(Coalesce(
         h_active_record, 0), PMO_CNST.SC_INCLUDE_COMPLETED)) = PMO_CNST.SC_INCLUDE_COMPLETED Then
      h_include_completed_items := 1;
   End If;
   
   If (Coalesce(h_include_closed_items, 0) = 1) AND (Coalesce(h_include_completed_items, 0) = 0) Then
      BEGIN
         DELETE FROM PMO_TMP_VIEWS_ROOT
         WHERE (MILESTONE_PERCENT >= 10000 AND STAGE_ID IN (5490, 4948, 5520, 4909, 5416, 5385, 6116, 4938, 4919, 4929))
         AND STAGE_ID NOT IN (4949, 5491, 5521, 4910, 4920, 4930, 4939, 5386, 5417, 6117   );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 243+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(244) : views.sql(1095) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf (Coalesce(h_include_closed_items, 0) = 0) AND (Coalesce(
                h_include_completed_items, 0) = 1) Then
      BEGIN
         DELETE FROM PMO_TMP_VIEWS_ROOT
         WHERE STAGE_ID IN (4949, 5491, 5521, 4910, 4920, 4930, 4939, 5386, 5417, 6117   );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 262+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(263) : views.sql(1114) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf (Coalesce(h_include_closed_items, 0) = 0) AND (Coalesce(
                h_include_completed_items, 0) = 0) Then
      BEGIN
         DELETE FROM PMO_TMP_VIEWS_ROOT
         WHERE ((MILESTONE_PERCENT >= 10000 AND STAGE_ID IN (5490, 4948, 5520, 4909, 5416, 5385, 6116, 4938, 4919, 4929)) OR STAGE_ID IN (4949, 5491, 5521, 4910, 4920, 4930, 4939, 5386, 5417, 6117   ));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 281+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(282) : views.sql(1133) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If h_ids IS NULL Then
      BEGIN
         INSERT INTO PMO_TMP_VIEWS_01
         SELECT PARENT_ID
         FROM PMO_TMP_VIEWS_ROOT
         WHERE PARENT_ID <> 'ROOT'
         UNION ALL
         SELECT PARENT_ID FROM TMT_PORTFOLIOS
         WHERE RANK LIKE h_res_row_rank AND TYPE_ID = 64 AND REC_STATUS <> 'D' AND LEVEL_ID = 2;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 306+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(307) : views.sql(1158) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_TMP_VIEWS_LEVEL1 (PARENT_ID, CHILD_COUNT)
         SELECT PARENT_ID, COUNT(*) AS CHILD_COUNT
         FROM PMO_TMP_VIEWS_01
         GROUP BY PARENT_ID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 326+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(327) : views.sql(1178) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE PMO_TMP_VIEWS_ROOT
         SET SECURITY_ID = case when type_id in (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) and (SECURITY_ID = '' OR SECURITY_ID is Null)
         then PMO_CNST.SCPROJECTVISITOR_DB_STR
         when type_id > PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT
         then case when (SECURITY_ID = '' OR SECURITY_ID is Null)
         then PMO_CNST.SCPROJECTVISITOR_DB_STR
         else SECURITY_ID end || h_executive_security
         else SECURITY_ID end;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 350+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(351) : views.sql(1202) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN PORT_SECURITY;
      
      h_parent_id := 'YY';
      h_security_ids := '';
      LOOP
         FETCH PORT_SECURITY INTO h_element_id, h_type_id, h_tmp_security_ids;
         IF PORT_SECURITY%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         If PMO_ORASTD.strcmp(h_parent_id, 'YY') = 0 Then
            h_parent_id := h_element_id;
            h_security_ids := h_tmp_security_ids;
         ElsIf PMO_ORASTD.strcmp(h_parent_id, h_element_id) = 0 Then
            If InStr(h_security_ids, h_tmp_security_ids) = 0 Then
               h_security_ids := h_security_ids || ',';
               h_security_ids := h_security_ids || h_tmp_security_ids;
            End If;
         Else
            BEGIN
               UPDATE PMO_TMP_VIEWS_LEVEL1
               SET SECURITY_ID = h_security_ids
               WHERE PARENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 394+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(395) : views.sql(1246) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_parent_id := h_element_id;
            h_security_ids := h_tmp_security_ids;
         End If;
      END LOOP;
      BEGIN
         CLOSE PORT_SECURITY;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 411+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(412) : views.sql(1263) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_ORASTD.strcmp(h_parent_id, 'YY') <> 0 Then
         BEGIN
            UPDATE PMO_TMP_VIEWS_LEVEL1
            SET SECURITY_ID = h_security_ids
            WHERE PARENT_ID = h_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 431+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(432) : views.sql(1283) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      OPEN PORT_SECURITY1;
      
      h_security_ids := '';
      LOOP
         FETCH PORT_SECURITY1 INTO h_element_id, h_level_id, h_rank, h_tmp_security_ids;
         IF PORT_SECURITY1%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         If InStr(h_security_ids, h_tmp_security_ids) = 0 Then
            If Coalesce(Length(h_security_ids), 0) <> 0 Then
               h_security_ids := h_security_ids || ',';
            End If;
            h_security_ids := h_security_ids || h_tmp_security_ids;
         End If;
         
         If Coalesce(h_level_id, 0) = 1 Then
            BEGIN
               UPDATE PMO_TMP_VIEWS_LEVEL1
               SET SECURITY_ID = h_security_ids
               WHERE PARENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 474+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(475) : views.sql(1326) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_security_ids := '';
         End If;
      END LOOP;
      BEGIN
         CLOSE PORT_SECURITY1;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 490+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(491) : views.sql(1342) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      myStmt := 
            'SELECT 0 AS LEVEL_ID, T.TYPE_ID, CASE WHEN T.TYPE_ID = 1 THEN ''qqqqqqq1'' ELSE ''qqqqqqq2'' END || RTRIM(T.RANK) AS RANK, T.PARENT_ID, T.ELEMENT_ID, ';
      
      If Coalesce(h_sp_id, 0) = PMO_CNST.SC_PORTFOLIO_CONFIG_VIEW Then
         myStmt := myStmt || 
                    ' T.SUBPROJECT_COUNT AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
      Else
         myStmt := myStmt || ' T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
      End If;
      
      myStmt := myStmt || ' A.SECURITY_ID AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
      
      myStmt := myStmt || 'CASE WHEN InStr(T.ASSIGN_NAMES, ''';
      myStmt := myStmt || h_full_name;
      myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
      myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, '''' AS PROJECT_NAME, ';
      myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
      myStmt := myStmt || h_column_set;
      myStmt := myStmt || ' FROM PMO_TMP_VIEWS_ROOT A, TMT_WBS T ';
      myStmt := myStmt || 'WHERE A.PARENT_ID = ''ROOT'' AND T.RANK = A.RANK ';
      
      myStmt := myStmt || ' UNION ALL ';
      
      myStmt := myStmt || 
                    ' SELECT P.LEVEL_ID - 1 AS LEVEL_ID, P.TYPE_ID, P.RANK, P.PARENT_ID, P.ELEMENT_ID, ';
      myStmt := myStmt || 
                    'COALESCE(L1.CHILD_COUNT,0) AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ';
      myStmt := myStmt || 
                    '''SCFOLDERMANAGER'' AS SECURITY_ID, L1.SECURITY_ID AS PORTFOLIO_SECURITY, ''N'' AS ASSIGNED,'''' AS PROJECT_NAME, ';
      myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
      myStmt := myStmt || h_column_set;
      myStmt := myStmt || 
                    'FROM TMT_PORTFOLIOS P LEFT JOIN PMO_TMP_VIEWS_LEVEL1 L1 ON L1.PARENT_ID = P.ELEMENT_ID ';
      myStmt := myStmt || 
                    'LEFT JOIN TMT_WBS T ON T.PROJECT_ID = P.ELEMENT_ID AND T.ELEMENT_ID = P.ELEMENT_ID ';
      myStmt := myStmt || 'WHERE P.PARENT_ID = ''';
      myStmt := myStmt || h_resource_id;
      myStmt := myStmt || ''' ORDER BY RANK ASC';
      
      --trace_printf("\n  WBS VIEW-------> resultset 011   myStmt:%s", (char *) myStmt);
   Else
      BEGIN
         INSERT INTO PMO_TMP_VIEWS_01
         SELECT PARENT_ID
         FROM PMO_TMP_VIEWS_ROOT
         WHERE PARENT_ID <> 'ROOT';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 552+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(553) : views.sql(1404) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_total := Trunc((Coalesce(Length(h_ids), 0) + 1) / 32);
      h_count := 0;
      WHILE Coalesce(h_count, 0) < Coalesce(h_total, 0) LOOP
         h_id := SubStr(h_ids, Coalesce(h_count, 0) * 32 + 1, 32);
         h_id := h_id;
         
         BEGIN
            SELECT LEVEL_ID, RANK || '%'
            INTO h_level_id, h_rank
            FROM TMT_PORTFOLIOS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 574+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(575) : views.sql(1426) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO PMO_TMP_VIEWS_01
            SELECT PARENT_ID
            FROM TMT_PORTFOLIOS
            WHERE RANK LIKE h_rank
            AND REC_STATUS <> 'D'
            AND TYPE_ID = 64
            AND LEVEL_ID >= h_level_id
            AND LEVEL_ID <= h_level_id + 2;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 598+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(599) : views.sql(1450) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_count, 0) = 0 Then
            idList := '(''' || h_id || '''';
         Else
            idList := idList || ',''';
            idList := idList || h_id;
            idList := idList || '''';
         End If;
         h_count := Coalesce(h_count, 0) + 1;
      END LOOP;
      idList := idList || ')';
      
      BEGIN
         INSERT INTO PMO_TMP_VIEWS_LEVEL1 (PARENT_ID, CHILD_COUNT)
         SELECT PARENT_ID, COUNT(*) AS CHILD_COUNT
         FROM PMO_TMP_VIEWS_01
         GROUP BY PARENT_ID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 629+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(630) : views.sql(1481) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_ORASTD.strncmp(h_wkf_resource_id, h_resource_id, 32) <> 0 Then
         BEGIN
            UPDATE PMO_TMP_VIEWS_ROOT R
            SET SECURITY_ID = (
            SELECT A.SECURITY_ID
            FROM PROJECT_ASSIGNMENTS A
            WHERE R.ELEMENT_ID = A.PROJECT_ID
            AND A.RESOURCE_ID = h_wkf_resource_id
            AND (BITAND(A.PARTICIPANT, 3) > 0 OR (BITAND(A.PARTICIPANT, 3) = 0)));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 653+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(654) : views.sql(1505) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         -- delete all projects not in tmt_portfolios
         BEGIN
            DELETE FROM PMO_TMP_VIEWS_ROOT WHERE PARENT_ID = 'ROOT';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 671+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(672) : views.sql(1523) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE PMO_TMP_VIEWS_ROOT
            SET SECURITY_ID = case when type_id in (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) and (SECURITY_ID = '' OR SECURITY_ID is Null)
            then PMO_CNST.SCPROJECTVISITOR_DB_STR
            when type_id > PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT
            then case when (SECURITY_ID = '' OR SECURITY_ID is Null)
            then PMO_CNST.SCPROJECTVISITOR_DB_STR
            else SECURITY_ID end || h_executive_security
            else SECURITY_ID end;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 695+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(696) : views.sql(1547) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         OPEN PORT_SECURITY2;
         
         h_parent_id := 'YY';
         h_security_ids := '';
         LOOP
            FETCH PORT_SECURITY2 INTO h_element_id, h_type_id, h_tmp_security_ids;
            IF PORT_SECURITY2%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            --trace_printf("h_element_id; %s\n", h_element_id);
            If PMO_ORASTD.strcmp(h_parent_id, 'YY') = 0 Then
               h_parent_id := h_element_id;
               h_security_ids := h_tmp_security_ids;
            ElsIf PMO_ORASTD.strcmp(h_parent_id, h_element_id) = 0 Then
               If InStr(h_security_ids, h_tmp_security_ids) = 0 Then
                  h_security_ids := h_security_ids || ',';
                  h_security_ids := h_security_ids || h_tmp_security_ids;
               End If;
            Else
               --trace_printf("h_parent_id0; %s\n", h_parent_id);
               BEGIN
                  UPDATE PMO_TMP_VIEWS_LEVEL1
                  SET SECURITY_ID = h_security_ids
                  WHERE PARENT_ID = h_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 741+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(742) : views.sql(1593) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_parent_id := h_element_id;
               h_security_ids := h_tmp_security_ids;
            End If;
         END LOOP;
         BEGIN
            CLOSE PORT_SECURITY2;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 758+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(759) : views.sql(1610) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strcmp(h_parent_id, 'YY') <> 0 Then
            --trace_printf("h_parent_id1; %s\n", h_parent_id);
            BEGIN
               UPDATE PMO_TMP_VIEWS_LEVEL1
               SET SECURITY_ID = h_security_ids
               WHERE PARENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 779+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(780) : views.sql(1631) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         OPEN PORT_SECURITY3;
         
         LOOP
            FETCH PORT_SECURITY3 INTO h_element_id, h_rank, h_security_ids;
            IF PORT_SECURITY3%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            OPEN PORT_SECURITY4;
            
            h_count := 0;
            LOOP
               FETCH PORT_SECURITY4 INTO h_tmp_security_ids;
               IF PORT_SECURITY4%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
               
               h_count := Coalesce(h_count, 0) + 1;
               If Coalesce(Length(h_security_ids), 0) > 0 Then
                  h_security_ids := h_security_ids || ',';
               End If;
               h_security_ids := h_security_ids || h_tmp_security_ids;
            END LOOP;
            BEGIN
               CLOSE PORT_SECURITY4;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 828+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(829) : views.sql(1680) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_count, 0) > 0 Then
               BEGIN
                  UPDATE PMO_TMP_VIEWS_LEVEL1
                  SET SECURITY_ID = h_security_ids
                  WHERE PARENT_ID = h_element_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 848+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(849) : views.sql(1700) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         END LOOP;
         BEGIN
            CLOSE PORT_SECURITY3;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 862+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(863) : views.sql(1714) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      myStmt := 'SELECT P.LEVEL_ID, P.TYPE_ID , P.RANK, P.PARENT_ID, P.ELEMENT_ID, ';
      myStmt := myStmt || 
                    'COALESCE(L1.CHILD_COUNT,0) AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ';
      myStmt := myStmt || '''SCFOLDERMANAGER'' AS SECURITY_ID, L1.SECURITY_ID AS PORTFOLIO_SECURITY, ';
      myStmt := myStmt || '''N'' AS ASSIGNED, '''' AS PROJECT_NAME, ';
      myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
      myStmt := myStmt || h_column_set;
      myStmt := myStmt || 'FROM TMT_PORTFOLIOS P LEFT JOIN PMO_TMP_VIEWS_LEVEL1 L1 ';
      myStmt := myStmt || 'ON L1.PARENT_ID = P.ELEMENT_ID ';
      myStmt := myStmt || 
                    'LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.PROJECT_ID AND P.ELEMENT_ID = T.ELEMENT_ID ';
      myStmt := myStmt || 'WHERE (P.ELEMENT_ID IN ';
      myStmt := myStmt || idList;
      myStmt := myStmt || ' OR P.PARENT_ID IN ';
      myStmt := myStmt || idList;
      myStmt := myStmt || ') AND P.TYPE_ID = 64 ';
      
      myStmt := myStmt || ' UNION ALL ';
      
      myStmt := myStmt || 'SELECT P.LEVEL_ID, P.TYPE_ID, P.RANK, P.PARENT_ID, P.ELEMENT_ID, ';
      
      If Coalesce(h_sp_id, 0) = PMO_CNST.SC_PORTFOLIO_CONFIG_VIEW Then
         myStmt := myStmt || 
                    'T.SUBPROJECT_COUNT AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, ';
      Else
         myStmt := myStmt || 'T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, ';
      End If;
      
      myStmt := myStmt || 
                    'A.PARTICIPANT, COALESCE(A.SECURITY_ID,''SCPROJECTWORKFLOWVISITOR'') AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
      myStmt := myStmt || 'CASE WHEN InStr(T.ASSIGN_NAMES, ''';
      myStmt := myStmt || h_full_name;
      myStmt := myStmt || 
                    ''' || '';'') > 0 THEN ''M'' ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, '''' AS PROJECT_NAME, ';
      myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
      myStmt := myStmt || h_column_set;
      myStmt := myStmt || 'FROM TMT_PORTFOLIOS P JOIN TMT_WBS T ON P.RANK LIKE ''';
      myStmt := myStmt || h_res_row_rank;
      myStmt := myStmt || ''' AND P.ELEMENT_ID = T.PROJECT_ID AND P.ELEMENT_ID = T.ELEMENT_ID ';
      myStmt := myStmt || 'JOIN PMO_TMP_VIEWS_ROOT A ON P.ELEMENT_ID = A.ELEMENT_ID AND A.PARENT_ID = ''';
      myStmt := myStmt || h_id;
      myStmt := myStmt || ''' ORDER BY RANK';
      
      --trace_printf("\n  WBS VIEW-------> resultset 012");
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_WBS_PORTFOLIO_VIEW', 915+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_WBS_PORTFOLIO_VIEW(916) : views.sql(1767) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_WBS_PORTFOLIO_VIEW', 918+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_WBS_PORTFOLIO_VIEW(919) : views.sql(1770) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(myStmt);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 926+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(927) : views.sql(1778) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 938+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(939) : views.sql(1790) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION FC_WBS_PORTFOLIO_VIEW;
ALTER FUNCTION FC_WBS_PORTFOLIO_VIEW COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION FC_WBS_VIEW ==
CREATE OR REPLACE FUNCTION FC_WBS_VIEW
(
     sh_h_sp_id PLS_INTEGER
   , h_resource_id IN OUT NOCOPY VARCHAR2
   , h_name IN OUT NOCOPY VARCHAR2
   , h_filter_string IN OUT NOCOPY VARCHAR2
   , sh_h_filter_count PLS_INTEGER
   , sh_h_type PLS_INTEGER
   , sh_h_active_record PLS_INTEGER
   , sh_h_stage_id PLS_INTEGER
   , sh_h_is_single_project PLS_INTEGER
   , h_wbs_search_criteria IN OUT NOCOPY PLS_INTEGER
   , h_prj_var_condition IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_sp_id PLS_INTEGER := sh_h_sp_id;
   h_filter_count PLS_INTEGER := sh_h_filter_count;
   h_type PLS_INTEGER := sh_h_type;
   h_active_record PLS_INTEGER := sh_h_active_record;
   h_stage_id PLS_INTEGER := sh_h_stage_id;
   h_is_single_project PLS_INTEGER := sh_h_is_single_project;
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_security_id VARCHAR2(32);
   h_security_ids VARCHAR2(4000);
   h_table_type PLS_INTEGER := 0;
   h_project_name VARCHAR2(255);
   h_tpf_parent_id VARCHAR2(32);
   h_level VARCHAR2(4);
   h_str_filter_count VARCHAR2(4);
   h_start_finish_date DATE;
   h_effort_driven PLS_INTEGER := 0;
   h_minute_per_day PLS_INTEGER := 0;
   typeList VARCHAR2(99);
   myStmt VARCHAR2(29999);
   dynamicSql VARCHAR2(1999);
   h_filedName VARCHAR2(254);
   h_keyword_search1 VARCHAR2(8000);
   h_keyword_search2 VARCHAR2(8000);
   h_critical_element_filter VARCHAR2(255);
   h_critical_filter PLS_INTEGER := 0;
   h_exculde_sub_elements PLS_INTEGER := 0;
   h_is_critical_tasks PLS_INTEGER := 0;
   h_next_level_only PLS_INTEGER := 0;
   h_include_closed_items PLS_INTEGER := 0;
   h_include_completed_items PLS_INTEGER := 0;
   h_is_del_manager_counter PLS_INTEGER := 0;
   h_condition_flag PLS_INTEGER := 0;
   localType PLS_INTEGER;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_WBS_VIEW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_WBS_VIEW'), '-- ' || SYSTIMESTAMP, ':r := FC_WBS_VIEW(' || 
       COALESCE(TO_CHAR(sh_h_sp_id), 'NULL')
          || ', ' || '''' || h_resource_id || ''''
          || ', ' || '''' || h_name || ''''
          || ', ' || '''' || h_filter_string || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_filter_count), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_active_record), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_stage_id), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_is_single_project), 'NULL')
          || ', ' || COALESCE(TO_CHAR(h_wbs_search_criteria), 'NULL')
          || ', ' || '''' || h_prj_var_condition || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_WBS_VIEW', 67+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_WBS_VIEW(68) : views.sql(1874) : ' || SYSTIMESTAMP);
   END IF; 

   -- UPDATE FILTER_FLAG IN TEMPORARY TABLE
   
   If (BITAND(Coalesce(
         h_active_record, 0), PMO_CNST.SC_DELIVERABLE_MANAGER_COUNTER)) = PMO_CNST.SC_DELIVERABLE_MANAGER_COUNTER Then
      h_is_del_manager_counter := 1;
   End If;
   
   --timeLog(file,"------------------>BEGIN FC_WBS_VIEW ");
   
   If (BITAND(Coalesce(h_active_record, 0), 4)) = 0 Then
      h_next_level_only := 1;
   End If;
   
   If (Coalesce(h_sp_id, 0) = 2) OR (Coalesce(h_sp_id, 0) = 302) Then
      --trace_printf("\n insert 1 ");
      
      h_condition_flag := 1;
      
      myStmt := 'INSERT INTO PMO_WBS_VIEW (PROJECT_ID, PROJECT_NAME, ELEMENT_NAME, START_FINISH_DATE,';
      myStmt := myStmt || 
                    'MINUTE_PER_DAY, EFFORT_DRIVEN, RANK, ELEMENT_ID, PARENT_ID, LEVEL_ID, REF_CODE, TYPE_ID,';
      myStmt := myStmt || ' FILTER_FLAG, STAGE_ID, MILESTONE_PERCENT, T_RANK, BUBBLE_COLOR)';
      myStmt := myStmt || 
                    ' SELECT PL.ELEMENT_ID AS PROJECT_ID, PL.ELEMENT_NAME AS PROJECT_NAME, T.ELEMENT_NAME AS ELEMENT_NAME, T.START_FINISH_DATE,';
      myStmt := myStmt || ' T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, T.RANK, T.ELEMENT_ID, T.PARENT_ID, ';
      
      myStmt := myStmt || ' CASE WHEN ';
      myStmt := myStmt || TO_CHAR(h_next_level_only);
      myStmt := myStmt || ' = 0 then T.LEVEL_ID ';
      myStmt := myStmt || ' when ';
      myStmt := myStmt || TO_CHAR(h_next_level_only);
      myStmt := myStmt || ' = 1 and T.ELEMENT_ID = PL.ELEMENT_ID then 0 else 1 end AS LEVEL_ID';
      
      myStmt := myStmt || ', T.REF_CODE, T.TYPE_ID,';
      myStmt := myStmt || ' 0 AS FILTER_FLAG, T.STAGE_ID, T.MILESTONE_PERCENT, T.T_RANK, T.BUBBLE_COLOR';
      myStmt := myStmt || ' FROM TMT_WBS T, PMO_TMP_PROJECTS_01 PL';
      --myStmt.strcat(" WHERE SUBSTR(T.RANK, 1, CASE WHEN LENGTH(T.RANK) >= LENGTH(PL.RANK) THEN LENGTH(PL.RANK) ELSE 1 END) = PL.RANK ");
      myStmt := myStmt || 
                    ' WHERE PL.RANK <= T.RANK  AND LENGTH(PL.RANK) <= LENGTH(T.RANK) AND PMO_ORASTD.LEFT(T.RANK,LENGTH(PL.RANK)) = PL.RANK ';
   Else
      If (BITAND(Coalesce(h_active_record, 0), 2)) = 2 Then
         --trace_printf("\n insert 2 ");
         h_condition_flag := 1;
         
         myStmt := 'INSERT INTO PMO_WBS_VIEW (PROJECT_ID, PROJECT_NAME, ELEMENT_NAME, START_FINISH_DATE,';
         myStmt := myStmt || 
                    'MINUTE_PER_DAY, EFFORT_DRIVEN, RANK, ELEMENT_ID, PARENT_ID, LEVEL_ID, REF_CODE, TYPE_ID,';
         myStmt := myStmt || 'FILTER_FLAG, STAGE_ID, MILESTONE_PERCENT, T_RANK, BUBBLE_COLOR)';
         myStmt := myStmt || 
                    'SELECT PL.ELEMENT_ID AS PROJECT_ID, PL.ELEMENT_NAME AS PROJECT_NAME, T.ELEMENT_NAME AS ELEMENT_NAME, T.START_FINISH_DATE,';
         myStmt := myStmt || 'T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, T.RANK, T.ELEMENT_ID, T.PARENT_ID, ';
         
         myStmt := myStmt || ' CASE WHEN ';
         myStmt := myStmt || TO_CHAR(h_next_level_only);
         myStmt := myStmt || ' = 0 then T.LEVEL_ID ';
         myStmt := myStmt || ' when ';
         myStmt := myStmt || TO_CHAR(h_next_level_only);
         myStmt := myStmt || ' = 1 and T.ELEMENT_ID = PL.ELEMENT_ID then 0 else 1 end AS LEVEL_ID';
         
         myStmt := myStmt || ', T.REF_CODE, T.TYPE_ID,';
         myStmt := myStmt || '1  AS FILTER_FLAG, T.STAGE_ID, T.MILESTONE_PERCENT, T.T_RANK, T.BUBBLE_COLOR ';
         myStmt := myStmt || ' FROM TMT_WBS T, RESOURCE_ASSIGNMENTS RA, PMO_TMP_PROJECTS_01 PL ';
         myStmt := myStmt || ' WHERE RA.PROJECT_ID = T.PROJECT_ID ';
         myStmt := myStmt || ' AND RA.ELEMENT_ID = T.ELEMENT_ID ';
         myStmt := myStmt || ' AND RA.RESOURCE_ID = ''';
         myStmt := myStmt || h_resource_id;
         myStmt := myStmt || '''';
         myStmt := myStmt || ' AND PL.ELEMENT_ID = T.PROJECT_ID ';
         myStmt := myStmt || 
                    ' AND SUBSTR(T.RANK, 1, CASE WHEN LENGTH(T.RANK) >= LENGTH(PL.RANK) THEN LENGTH(PL.RANK) ELSE 1 END) = PL.RANK ';
      Else
         --trace_printf("\n insert 3 ");
         
         h_condition_flag := 0;
         
         myStmt := 'INSERT INTO PMO_WBS_VIEW (PROJECT_ID, PROJECT_NAME, ELEMENT_NAME, START_FINISH_DATE,';
         myStmt := myStmt || 
                    ' MINUTE_PER_DAY, EFFORT_DRIVEN, RANK, ELEMENT_ID, PARENT_ID, LEVEL_ID, REF_CODE, TYPE_ID,';
         myStmt := myStmt || ' FILTER_FLAG, STAGE_ID, MILESTONE_PERCENT, T_RANK, BUBBLE_COLOR)';
         myStmt := myStmt || 
                    ' SELECT PL.ELEMENT_ID AS PROJECT_ID, PL.ELEMENT_NAME AS PROJECT_NAME, T.ELEMENT_NAME AS ELEMENT_NAME, T.START_FINISH_DATE,';
         myStmt := myStmt || ' T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, T.RANK, T.ELEMENT_ID, T.PARENT_ID, ';
         
         If Coalesce(h_wbs_search_criteria, 0) <> PMO_CNST.SC_SEARCH_ALL_ITEMS Then
            myStmt := myStmt || ' CASE WHEN ';
            myStmt := myStmt || TO_CHAR(h_next_level_only);
            myStmt := myStmt || ' = 0 then T.LEVEL_ID ';
            myStmt := myStmt || ' when ';
            myStmt := myStmt || TO_CHAR(h_next_level_only);
            myStmt := myStmt || ' = 1 and T.ELEMENT_ID = PL.ELEMENT_ID then 0 else 1 end AS LEVEL_ID';
         Else
            myStmt := myStmt || ' 0 AS LEVEL_ID ';
         End If;
         
         myStmt := myStmt || ', T.REF_CODE, T.TYPE_ID,';
         myStmt := myStmt || ' 1 AS FILTER_FLAG, T.STAGE_ID, T.MILESTONE_PERCENT, T.T_RANK, T.BUBBLE_COLOR ';
         myStmt := myStmt || ' FROM TMT_WBS T JOIN PMO_TMP_PROJECTS_01 PL';
         myStmt := myStmt || 
                    ' ON SUBSTR(T.RANK, 1, CASE WHEN LENGTH(T.RANK) >= LENGTH(PL.RANK) THEN LENGTH(PL.RANK) ELSE 1 END) = PL.RANK ';
         myStmt := myStmt || ' AND T.PROJECT_ID = PL.ELEMENT_ID';
      End If;
   End If;
   
   --trace_printf("\n h_type:%d \n", h_type);
   
   h_critical_filter := 0;
   CASE h_type
      WHEN 1001 THEN
         -- All Milestone
         myStmt := myStmt || ' AND T.TYPE_ID = ';
         myStmt := myStmt || TO_CHAR(PMO_CNST.SC_MILESTONE);
         
      
      WHEN 1003 THEN
         -- All Late to start tasks
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || 
                    ') AND T.ACTUAL_START IS NuLL AND TO_DATE(SubStr(T.UNRESOLVED_START, 1, 10)) < TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10))';
         
      
      WHEN 1005 THEN
         -- All Late to finish tasks
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || 
                    ') AND T.ACTUAL_FINISH IS NuLL AND TO_DATE(SubStr(T.UNRESOLVED_FINISH, 1, 10)) < TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10))';
         
      
      WHEN 1007 THEN
         -- All task due within 1 week
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') AND T.ACTUAL_FINISH IS NuLL';
         myStmt := myStmt || 
                    ' AND TO_DATE(SubStr(T.UNRESOLVED_FINISH, 1, 10)) BETWEEN TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) AND (TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) + 6     )';
         
      
      WHEN 1009 THEN
         -- All task due within 4 weeks (28 days)
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') AND T.ACTUAL_FINISH IS NuLL';
         myStmt := myStmt || 
                    ' AND TO_DATE(SubStr(T.UNRESOLVED_FINISH, 1, 10)) BETWEEN TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) AND (TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) + 27     )';
         
      
      WHEN 1011 THEN
         -- All tasks starting within 1 week
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') AND T.ACTUAL_START IS NuLL';
         myStmt := myStmt || 
                    ' AND TO_DATE(SubStr(T.UNRESOLVED_START, 1, 10)) BETWEEN TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) AND (TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) + 6     )';
         
      
      WHEN 1013 THEN
         -- All tasks starting within 4 weeks (28 days)
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') AND T.ACTUAL_START IS NuLL';
         myStmt := myStmt || 
                    ' AND TO_DATE(SubStr(T.UNRESOLVED_START, 1, 10)) BETWEEN TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) AND (TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) + 27     )';
         
      
      WHEN 1015 THEN
         -- All critical elements (filter by attributes)
         h_critical_element_filter := '';
         h_critical_filter := 1;
         
      
      WHEN 1023 THEN
         -- All critical projects (filter by attributes)
         h_critical_element_filter := ' AND T.TYPE_ID IN (';
         h_critical_element_filter := h_critical_element_filter || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
         h_critical_element_filter := h_critical_element_filter || ') ';
         h_critical_filter := 1;
         
      
      WHEN 1025 THEN
         -- All critical deliverables (filter by attributes)
         h_critical_element_filter := ' AND T.TYPE_ID IN (';
         h_critical_element_filter := h_critical_element_filter || PMO_CNST.SC_STR_WBS_DELV_GROUP_TYPES;
         h_critical_element_filter := h_critical_element_filter || ') ';
         h_critical_filter := 1;
         
      
      WHEN 1027 THEN
         -- All critical tasks (filter by attributes)
         h_critical_element_filter := ' AND T.TYPE_ID IN (';
         h_critical_element_filter := h_critical_element_filter || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         h_critical_element_filter := h_critical_element_filter || ') ';
         h_critical_filter := 1;
         
      
      WHEN 1017 THEN
         -- All Ready to start task
         myStmt := myStmt || ' AND T.DEPEND_FLAG IN (''0'', ''2'') AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') AND T.ACTUAL_START IS NuLL';
         myStmt := myStmt || ' AND (T.CONSTRAINT_TYPE NOT IN (5, 6)';
         myStmt := myStmt || 
                    ' OR (T.CONSTRAINT_TYPE IN (5, 6) AND TO_DATE(SubStr(T.CONSTRAINT_DT, 1, 10)) <= TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10))))';
         
      
      WHEN 1019 THEN
         -- All Projects
         --trace_printf("\n case 1019 \n");
         If Coalesce(h_sp_id, 0) = 302 Then
            dynamicSql := 'UPDATE PMO_WBS_VIEW T SET T.FILTER_FLAG = 1 WHERE T.TYPE_ID IN (';
            dynamicSql := dynamicSql || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            dynamicSql := dynamicSql || ')';
            
            If h_stage_id IS NULL OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.BUBBLE_COLOR = ';
                  dynamicSql := dynamicSql || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.STAGE_ID = ';
                  dynamicSql := dynamicSql || typeList;
               End If;
            End If;
         Else
            If Coalesce(h_wbs_search_criteria, 0) <> PMO_CNST.SC_SEARCH_ALL_ITEMS Then
               myStmt := myStmt || ' AND T.ELEMENT_ID = PL.ELEMENT_ID';
               myStmt := myStmt || ' AND T.TYPE_ID IN (';
               myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
               myStmt := myStmt || ')';
               
               If h_stage_id IS NULL OR Coalesce(h_stage_id, 0) <> 0 Then
                  If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                        h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                     typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                     myStmt := myStmt || ' AND T.BUBBLE_COLOR = ';
                     myStmt := myStmt || typeList;
                  Else
                     typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                     myStmt := myStmt || ' AND T.STAGE_ID = ';
                     myStmt := myStmt || typeList;
                  End If;
               End If;
            End If;
         End If;
         
      
      WHEN 1021 THEN
         -- Critical Task
         myStmt := myStmt || ' AND T.LEVEL_INDEX <> -1 AND T.TOTAL_SLACK = 0 AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ')';
         
      
      WHEN 1029 THEN
         -- Projects
         If Coalesce(h_sp_id, 0) = 302 Then
            dynamicSql := 'UPDATE PMO_WBS_VIEW T SET T.FILTER_FLAG = 1 WHERE T.TYPE_ID = 1';
            
            If h_stage_id IS NULL OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.BUBBLE_COLOR = ';
                  dynamicSql := dynamicSql || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.STAGE_ID = ';
                  dynamicSql := dynamicSql || typeList;
               End If;
            End If;
         Else
            myStmt := myStmt || ' AND T.ELEMENT_ID = PL.ELEMENT_ID';
            myStmt := myStmt || ' AND T.TYPE_ID = 1';
            
            If h_stage_id IS NULL OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  myStmt := myStmt || ' AND T.BUBBLE_COLOR = ';
                  myStmt := myStmt || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  myStmt := myStmt || ' AND T.STAGE_ID = ';
                  myStmt := myStmt || typeList;
               End If;
            End If;
         End If;
         
      
      WHEN 1031 THEN
         -- Proposals
         If Coalesce(h_sp_id, 0) = 302 Then
            dynamicSql := 'UPDATE PMO_WBS_VIEW T SET T.FILTER_FLAG = 1 WHERE T.TYPE_ID = 154';
            
            If h_stage_id IS NULL OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.BUBBLE_COLOR = ';
                  dynamicSql := dynamicSql || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.STAGE_ID = ';
                  dynamicSql := dynamicSql || typeList;
               End If;
            End If;
         Else
            myStmt := myStmt || ' AND T.ELEMENT_ID = PL.ELEMENT_ID';
            myStmt := myStmt || ' AND T.TYPE_ID = 154';
            
            If h_stage_id IS NULL OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  myStmt := myStmt || ' AND T.BUBBLE_COLOR = ';
                  myStmt := myStmt || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  myStmt := myStmt || ' AND T.STAGE_ID = ';
                  myStmt := myStmt || typeList;
               End If;
            End If;
         End If;
         
      
      WHEN 1033 THEN
         -- Organization
         If Coalesce(h_sp_id, 0) = 302 Then
            dynamicSql := 'UPDATE PMO_WBS_VIEW T SET T.FILTER_FLAG = 1 WHERE T.TYPE_ID = 288';
            
            If h_stage_id IS NULL OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.BUBBLE_COLOR = ';
                  dynamicSql := dynamicSql || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.STAGE_ID = ';
                  dynamicSql := dynamicSql || typeList;
               End If;
            End If;
         Else
            myStmt := myStmt || ' AND T.ELEMENT_ID = PL.ELEMENT_ID';
            myStmt := myStmt || ' AND T.TYPE_ID = 288';
            
            If h_stage_id IS NULL OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  myStmt := myStmt || ' AND T.BUBBLE_COLOR = ';
                  myStmt := myStmt || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  myStmt := myStmt || ' AND T.STAGE_ID = ';
                  myStmt := myStmt || typeList;
               End If;
            End If;
         End If;
         
      
      WHEN 3001 THEN
         -- Critical Task after levelling
         myStmt := myStmt || ' AND (T.ELEMENT_ID = PL.ELEMENT_ID ';
         myStmt := myStmt || ' OR (T.LEVEL_INDEX <> -1 AND T.TOTAL_SLACK = 0 AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ')))';
         
      
      WHEN 1035 THEN
         -- Tasks with ETC Varinace
         myStmt := myStmt || 
                    ' AND (T.SCHED_EFFORT + T.PLANNED_EFFORT) - T.WORK_TODATE <> T.ETC AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ')';
         
         --trace_printf("\n Tasks with ETC Varinace \n");
         
      
      WHEN PMO_CNST.SC_WBS_IN_PROGRESS_TASKS THEN
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') ';
         myStmt := myStmt || ' AND T.STAGE_ID NOT IN (';
         myStmt := myStmt || PMO_CNST.SC_ALL_WBS_CLOSED_STATE_STR;
         myStmt := myStmt || ') ';
         myStmt := myStmt || ' AND T.ACTUAL_FINISH IS NULL ';
         myStmt := myStmt || ' AND T.ACTUAL_START IS NOT NULL ';
         
         
      
      WHEN PMO_CNST.SC_WBS_CLOSED_TASKS THEN
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') ';
         myStmt := myStmt || ' AND T.STAGE_ID  IN (';
         myStmt := myStmt || PMO_CNST.SC_ALL_WBS_CLOSED_STATE_STR;
         myStmt := myStmt || ') ';
         
         
      
      WHEN PMO_CNST.SC_WBS_COMPLETED_TASKS THEN
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') ';
         myStmt := myStmt || ' AND T.STAGE_ID NOT IN (';
         myStmt := myStmt || PMO_CNST.SC_ALL_WBS_CLOSED_STATE_STR;
         myStmt := myStmt || ') ';
         myStmt := myStmt || ' AND T.ACTUAL_FINISH IS NOT NULL AND T.MILESTONE_PERCENT >= 10000 ';
         myStmt := myStmt || ' AND T.STAGE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_ALL_WBS_COMPLETED_STATE_STR;
         myStmt := myStmt || ')';
         
         
      
      ELSE
         NULL;
   END CASE;
   
   If (h_name IS NOT NULL) Then
      myStmt := myStmt || ' AND ((';
      myStmt := myStmt || h_name;
      
      myStmt := myStmt || ')';
      
      If Coalesce(h_is_single_project, 0) = 1 Then
         myStmt := myStmt || ' OR T.ELEMENT_ID = PL.ELEMENT_ID';
      End If;
      myStmt := myStmt || ')';
   End If;
   
   If (Coalesce(h_sp_id, 0) = 2) OR (Coalesce(h_sp_id, 0) = 302) OR ((Coalesce(h_type, 0) > 5000) 
             AND (Coalesce(h_type, 0) < 6000)) Then
      If (Coalesce(h_type, 0) > 5000) AND (Coalesce(h_type, 0) < 6000) Then
         localType := Coalesce(h_type, 0) - 5000;
      Else
         localType := Coalesce(h_type, 0);
      End If;
      
      CASE localType
         
         WHEN 223 THEN
            -- deliverable and transferred cri
            typeList := '(' || PMO_CNST.SC_STR_WBS_DELV_GROUP_TYPES || ') ';
         
         WHEN 224 THEN
            -- tasks and milestone
            typeList := '(' || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES || ') ';
         
         WHEN 4 THEN
            -- transfer cris
            typeList := '(' || PMO_CNST.SC_STR_TRANS_WBSDELV_TYPES || ') ';
         
         WHEN 5 THEN
            -- transfer tasks
            typeList := '(' || PMO_CNST.SC_STR_TRANS_WBSTASK_TYPES || ') ';
         
         WHEN 0 THEN
            -- project and proposal
            typeList := '(' || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES || ') ';
         
         WHEN 3 THEN
            -- task (include duration > 0 milestones)
            typeList := '(' || TO_CHAR(PMO_CNST.SC_TASK) || ',' || TO_CHAR(PMO_CNST.SC_MILESTONE) 
                  || ') AND T.DURATION > 0 ';
         
         ELSE
            typeList := '(' || TO_CHAR(localType) || ') ';
      END CASE;
   End If;
   
   If (Coalesce(h_type, 0) > 5000) AND (Coalesce(h_type, 0) < 6000) Then
      If Coalesce(h_sp_id, 0) = 302 Then
         dynamicSql := 
               'UPDATE PMO_WBS_VIEW T SET T.FILTER_FLAG = 1 WHERE T.PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_01) ';
         dynamicSql := dynamicSql || ' AND T.TYPE_ID IN ';
         dynamicSql := dynamicSql || typeList;
      Else
         myStmt := myStmt || ' AND T.TYPE_ID IN ';
         myStmt := myStmt || typeList;
         
         If Coalesce(h_is_del_manager_counter, 0) = 1 Then
            myStmt := myStmt || ' JOIN RESOURCE_ASSIGNMENTS RA ON RA.RESOURCE_ID = ''';
            myStmt := myStmt || h_resource_id;
            myStmt := myStmt || ''' ';
            myStmt := myStmt || ' AND RA.PROJECT_ID = T.PROJECT_ID ';
            myStmt := myStmt || ' AND RA.ELEMENT_ID = T.ELEMENT_ID ';
            myStmt := myStmt || ' AND BITAND(RA.PARTICIPANT, ';
            myStmt := myStmt || TO_CHAR(PMO_CNST.SC_DLV_MANAGER);
            myStmt := myStmt || ') = ';
            myStmt := myStmt || TO_CHAR(PMO_CNST.SC_DLV_MANAGER);
         End If;
      End If;
   ElsIf (Coalesce(h_type, 0) >= 1100) AND (Coalesce(h_type, 0) <= 1821) Then
      If Coalesce(h_sp_id, 0) = 300 Then
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
         myStmt := myStmt || ')';
      End If;
   End If;
   
   If Coalesce(h_sp_id, 0) = 300 Then
      If h_filter_string IS NOT NULL Then
         h_critical_filter := 1;
      End If;
   End If;
   
   If Coalesce(h_critical_filter, 0) = 1 Then
      If Coalesce(h_sp_id, 0) = 302 Then
         If Length(dynamicSql) = 0 Then
            dynamicSql := 
                  'UPDATE PMO_WBS_VIEW T SET T.FILTER_FLAG = 1 WHERE T.PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_01) ';
         End If;
         
         dynamicSql := dynamicSql || h_critical_element_filter;
         If h_filter_string IS NOT NULL Then
            h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
            dynamicSql := dynamicSql || ' AND (SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM';
            dynamicSql := dynamicSql || ' TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
            dynamicSql := dynamicSql || ' WHERE CA.ELEMENT_ID = T.ELEMENT_ID AND PA.ELEMENT_ID IN (';
            dynamicSql := dynamicSql || h_filter_string;
            dynamicSql := dynamicSql || ') ';
            dynamicSql := dynamicSql || 
                            ' AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
            dynamicSql := dynamicSql || h_str_filter_count;
         End If;
      Else
         --myStmt.strcat(h_critical_element_filter);
         If h_filter_string IS NOT NULL Then
            If Coalesce(h_condition_flag, 0) = 0 Then
               myStmt := myStmt || ' WHERE ';
            Else
               myStmt := myStmt || ' AND ';
            End If;
            
            h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
            myStmt := myStmt || ' (SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM';
            myStmt := myStmt || ' TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
            myStmt := myStmt || ' WHERE CA.ELEMENT_ID = T.ELEMENT_ID AND PA.ELEMENT_ID IN (';
            myStmt := myStmt || h_filter_string;
            myStmt := myStmt || ') ';
            myStmt := myStmt || 
                    ' AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
            myStmt := myStmt || h_str_filter_count;
         End If;
      End If;
   End If;
   
   If (Coalesce(h_sp_id, 0) = 2) OR (Coalesce(h_sp_id, 0) = 302) Then
      If (Coalesce(h_type, 0) = 0) OR (Coalesce(h_type, 0) = 1) OR (Coalesce(h_type, 0) = 145) Then
         -- when type in (1, 145) and filter_string is not null, return next level only
         If ((BITAND(Coalesce(h_active_record, 0), 4)) = 0) AND ((Coalesce(h_type, 0) <> 0) OR ((Coalesce(
                     h_type, 0) = 0) AND (h_filter_string IS NULL) AND (h_name IS NULL))) Then
            myStmt := myStmt || ' AND T.LEVEL_ID < (PL.LEVEL_ID + 2) ';
         End If;
         
         If (h_filter_string IS NOT NULL) AND (Coalesce(h_type, 0) = 0) Then
            -- Filter by attribute list
            h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
            
            myStmt := myStmt || ' AND (((SELECT COUNT(DISTINCT(PA.ELEMENT_ID))';
            myStmt := myStmt || ' FROM TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
            myStmt := myStmt || ' WHERE CA.ELEMENT_ID = T.ELEMENT_ID AND PA.ELEMENT_ID IN (';
            myStmt := myStmt || h_filter_string;
            myStmt := myStmt || ') ';
            myStmt := myStmt || ' AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1';
            myStmt := myStmt || ' ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
            myStmt := myStmt || h_str_filter_count;
            myStmt := myStmt || ' AND T.TYPE_ID NOT IN (';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || ')) OR T.ELEMENT_ID = PL.ELEMENT_ID)';
         End If;
         myStmt := myStmt || ' AND T.REC_STATUS <> ''D''';
      ElsIf Coalesce(h_type, 0) <> 3001 Then
         If Coalesce(h_sp_id, 0) = 302 Then
            If (h_name IS NULL) Then
               myStmt := myStmt || ' AND (T.PARENT_ID = PL.ELEMENT_ID';
               myStmt := myStmt || ' OR T.ELEMENT_ID = PL.ELEMENT_ID';
               myStmt := myStmt || ')';
            End If;
         Else
            myStmt := myStmt || ' AND ((T.PROJECT_ID = PL.ELEMENT_ID';
            myStmt := myStmt || ' AND T.TYPE_ID IN ';
            myStmt := myStmt || typeList;
            
            If Coalesce(h_type, 0) = 213 Then
               myStmt := myStmt || ' AND T.TRANSFER_FLAG = ''R''';
            ElsIf Coalesce(h_type, 0) = 2 AND (h_filter_string IS NULL) Then
               myStmt := myStmt || ' AND T.TRANSFER_FLAG IS NuLL';
            End If;
            
            myStmt := myStmt || ') OR T.ELEMENT_ID = PL.ELEMENT_ID ';
            myStmt := myStmt || ' OR (T.TYPE_ID IN (';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || ') AND EXISTS(SELECT ELEMENT_ID FROM TMT_WBS WHERE TYPE_ID IN ';
            myStmt := myStmt || typeList;
            myStmt := myStmt || 
                    ' AND (RANK >= SUBSTR(T.RANK,1,8) AND SUBSTR(RANK, 1, CASE WHEN LENGTH(RANK) < LENGTH(T.RANK) THEN 1 ELSE LENGTH(T.RANK) END) = T.RANK))))';
            
            --trace_printf("\n APPLY FILTER STRING 1003\n");
            If h_filter_string IS NOT NULL Then
               -- Filter by attribute list
               h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
               
               myStmt := myStmt || 
                     ' EXISTS(SELECT W.ELEMENT_ID FROM TMT_WBS W WHERE (W.ELEMENT_ID = T.ELEMENT_ID OR W.PROJECT_ID = T.ELEMENT_ID)';
               myStmt := myStmt || ' AND (SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM';
               myStmt := myStmt || ' TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
               myStmt := myStmt || ' WHERE CA.ELEMENT_ID = W.ELEMENT_ID AND PA.ELEMENT_ID IN (';
               myStmt := myStmt || h_filter_string;
               myStmt := myStmt || 
                     ') AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
               myStmt := myStmt || h_str_filter_count;
               myStmt := myStmt || ')';
            End If;
         End If;
      End If;
   End If;
   
   If (Coalesce(h_type, 0) >= 1100) AND (Coalesce(h_type, 0) <= 1821) AND (Coalesce(
             h_sp_id, 0) = 300) Then
      myStmt := myStmt || ' AND ';
      myStmt := myStmt || h_prj_var_condition;
   End If;
   --trace_printf("\n myStmt: %s", (char *)myStmt);
   
   BEGIN
      EXECUTE IMMEDIATE myStmt;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 710+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(711) : views.sql(2517) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            LFILE.FPUT('AllTrace',                         'SQL : ' || myStmt);
         END IF; 
      END;
   END;
   
   If (Coalesce(h_sp_id, 0) = 302) AND (Length(dynamicSql) > 0) Then
      --trace_printf("\n dynamicSql: %s", (char *)dynamicSql);
      
      If Coalesce(h_wbs_search_criteria, 0) = PMO_CNST.SC_SEARCH_ALL_ITEMS Then
         BEGIN
            EXECUTE IMMEDIATE dynamicSql;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 732+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(733) : views.sql(2539) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || dynamicSql);
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If (BITAND(Coalesce(
         h_active_record, 0), PMO_CNST.SC_INCLUDE_CLOSED)) = PMO_CNST.SC_INCLUDE_CLOSED Then
      h_include_closed_items := 1;
   End If;
   If (BITAND(Coalesce(
         h_active_record, 0), PMO_CNST.SC_INCLUDE_COMPLETED)) = PMO_CNST.SC_INCLUDE_COMPLETED Then
      h_include_completed_items := 1;
   End If;
   
   --*****************************************************************************
   -- FOR ALL LEVEL WE APPLY CLOSED AND COMPLETED FILTER, BY REMOVEING THESE ITESM
   -- IT CAN CAUSE SOME DEAD PLUS SIGNS
   --*****************************************************************************
   --if (h_sp_id == 300)
   --{
   -- remove items that don't have Closed and Completed contition (first level)
   --if ((h_include_closed_items == 1)  (h_include_completed_items == 1))
   
   --trace_printf("\n before deleting closed h_sp_id:%d", h_sp_id);
   
   If (Coalesce(h_include_closed_items, 0) = 1) AND (Coalesce(h_include_completed_items, 0) = 0) Then
      BEGIN
         DELETE FROM PMO_WBS_VIEW V
         WHERE (V.MILESTONE_PERCENT >= 10000 AND V.STAGE_ID IN (5490, 4948, 5520, 4909, 5416, 5385, 6116, 4938, 4919, 4929))
         AND V.STAGE_ID NOT IN (4949, 5491, 5521, 4910, 4920, 4930, 4939, 5386, 5417, 6117   )
         AND (h_sp_id!=2 OR NOT EXISTS (SELECT P.element_id from PMO_TMP_PROJECTS_01 P where P.ELEMENT_ID = V.ELEMENT_ID));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 776+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(777) : views.sql(2583) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf (Coalesce(h_include_closed_items, 0) = 0) AND (Coalesce(
                h_include_completed_items, 0) = 1) Then
      BEGIN
         DELETE FROM PMO_WBS_VIEW V
         WHERE V.STAGE_ID IN (4949, 5491, 5521, 4910, 4920, 4930, 4939, 5386, 5417, 6117   )
         AND (h_sp_id!=2 OR NOT EXISTS (SELECT P.element_id from PMO_TMP_PROJECTS_01 P where P.ELEMENT_ID = V.ELEMENT_ID));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 796+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(797) : views.sql(2603) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf (Coalesce(h_include_closed_items, 0) = 0) AND (Coalesce(
                h_include_completed_items, 0) = 0) Then
      BEGIN
         DELETE FROM PMO_WBS_VIEW V
         WHERE ((V.MILESTONE_PERCENT >= 10000 AND V.STAGE_ID IN (5490, 4948, 5520, 4909, 5416, 5385, 6116, 4938, 4919, 4929)) OR V.STAGE_ID IN (4949, 5491, 5521, 4910, 4920, 4930, 4939, 5386, 5417, 6117   ))
         AND (h_sp_id!=2 OR NOT EXISTS (SELECT P.element_id from PMO_TMP_PROJECTS_01 P where P.ELEMENT_ID = V.ELEMENT_ID));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 816+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(817) : views.sql(2623) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --}
   
   If (BITAND(Coalesce(h_active_record, 0), 8)) = 8 Then
      If Coalesce(h_sp_id, 0) = 300 Then
         BEGIN
            DELETE FROM PMO_WBS_VIEW C
            WHERE EXISTS (SELECT P.ELEMENT_ID FROM PMO_WBS_VIEW P
            WHERE P.RANK < C.RANK AND ((P.RANK >= SUBSTR(C.RANK, 1, 8) AND P.RANK <= C.RANK) 
            AND SUBSTR(C.RANK,1, CASE WHEN LENGTH(P.RANK) > LENGTH(C.RANK) THEN 1 ELSE LENGTH(P.RANK) END ) = P.RANK));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 841+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(842) : views.sql(2648) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            DELETE FROM PMO_WBS_VIEW C
            WHERE EXISTS (SELECT P.ELEMENT_ID FROM PMO_WBS_VIEW P, PMO_TMP_PROJECTS_01 PL
            WHERE P.RANK < C.RANK AND ((P.RANK >= SUBSTR(C.RANK, 1, 8) AND P.RANK <= C.RANK) 
            AND SUBSTR(C.RANK,1, CASE WHEN LENGTH(P.RANK) > LENGTH(C.RANK) THEN 1 ELSE LENGTH(P.RANK) END ) = P.RANK)
            AND P.ELEMENT_ID <> PL.ELEMENT_ID);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 862+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(863) : views.sql(2669) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If Coalesce(h_sp_id, 0) = 302 Then
      If (BITAND(Coalesce(h_active_record, 0), 2)) = 2 Then
         BEGIN
            UPDATE PMO_WBS_VIEW
            SET FILTER_FLAG = 0
            WHERE ASSIGNED = 0;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 885+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(886) : views.sql(2692) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --*******************************************************************************
      -- FOR SECOND LEVEL IN VIEWS BEFORE WE HIGHLIGHT CLOSED AND COMPLETED ITEMS BUT
      -- IN CURRENT CODE WE JUST REMOVE THEM FROM RESULTSET, THE FOLLOWING CODE WAS FOR
      -- HIGHLIGHTING CLOSED AND COMPLETED ITEAMS
      --*******************************************************************************
      
      --if ((h_include_closed_items == 1)  (h_include_completed_items == 0))
      --{
      --  BEGIN_SQL
      --      UPDATE session.PMO_WBS_VIEW
      --          SET FILTER_FLAG = 0
      --      WHERE MILESTONE_PERCENT = 10000
      --          AND STAGE_ID NOT IN (SC_ALL_WBS_CLOSED_STATE);
      --  END_SQL_CHECK;
      --}
      --else if ((h_include_closed_items == 0)  (h_include_completed_items == 1))
      --{
      --  BEGIN_SQL
      --      UPDATE session.PMO_WBS_VIEW
      --          SET FILTER_FLAG = 0
      --      WHERE STAGE_ID IN (SC_ALL_WBS_CLOSED_STATE);
      --  END_SQL_CHECK;
      --}
      --else if ((h_include_closed_items == 0)  (h_include_completed_items == 0))
      --{
      --  BEGIN_SQL
      --      UPDATE session.PMO_WBS_VIEW
      --          SET FILTER_FLAG = 0
      --      WHERE (MILESTONE_PERCENT = 10000 OR STAGE_ID IN (SC_ALL_WBS_CLOSED_STATE));
      --  END_SQL_CHECK;
      --}
   ElsIf (Coalesce(h_sp_id, 0) = 300) AND (Coalesce(h_type, 0) = 1015) AND (Coalesce(
                h_critical_filter, 0) = 1) AND (h_filter_string IS NOT NULL) Then
      BEGIN
         UPDATE PMO_WBS_VIEW
         SET LEVEL_ID = 0
         WHERE LEVEL_ID > 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 938+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(939) : views.sql(2745) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --timeLog(file,"------------------>END   FC_WBS_VIEW ");
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_WBS_VIEW', 947+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_WBS_VIEW(948) : views.sql(2754) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_WBS_VIEW;
ALTER FUNCTION FC_WBS_VIEW COMPILE DEBUG;

/* ************************************************************************************************************************************************** */

/*
   sp_id identifies the type of the stored procedure and the result set depends on the type
 1 = Projects/Deliverables/Tasks  Work Packages - FIRST Level - Returns all projects where the resource_id is envolded
                if type = 0 and ID is NULL returns all projects in the root, where PARENT_ID iS NULL
                if type = ( SC_RESOURCE(13) or SC_RM_VIEW(118)) and ID is NULL returns Work Package, flat list of all projects for RESOURCE_ID
                else
                 IF filter_string (no attributes filter) is null then returns all roots projects for resource_id, and depends on type, ex:
                        CHILD_COUNT = 1 if there is at least one child of the TYPE_ID = type, else = 0
                 ELSE (attributes filter)
                        IF type == SC_PROJECT returns a flat list of all projects witch have attributes IN filter_string
                        ELSE returns a two levels resultset: in the root projects and in the next level a flat list of all children of TYPE_ID == type

 2 = Projects/Deliverables/Tasks  Work Packages - NEXT  Level
                if type = SC_PROJECT (1) - returns all WBS elements on the next level (PARENT_ID = id)
                          = SC_DELIVERABLE (2) - returns all deliverables under the project (identified by id),
                          = SC_CHANGE_REQUEST (4) - returns all changes under the project (identified by id),
                          = SC_RISK (5) - returns all deliverables under the project (identified by id),
                          = SC_ISSUE (6) - returns all deliverables under the project (identified by id),
                          = SC_SUMMARY_TASK (60) - returns all deliverables under the project (identified by id),
                          = SC_WORK_PRODUCT (58) - returns all deliverables under the project (identified by id)
                          = SC_ALL_DELIVERABLE (223) - returns all deliverables, summary task, work products,
                            and transferred changes, issues, risks, and requirements under the project (identified by id),
                          = SC_TASK (3) - returns all tasks under the project (identified by id)
                          = SC_MILESTONE (59) - returns all milestone under the project (identified by id)
                          = SC_TASK_MILESTONE (224) - returns all task and milestone under the project (identified by id)
 3 = Domain  Work Packages - NEXT Level

 4 = Project Manager/Team Member Work Packages
        h_id        = Project ID
        h_resource_id   = Project Manager or Team Resource ID

        when (h_id is null) and (h_resource_id is not null) then return project list
        when (h_id is not null) and (h_resource_id is not null)
            then return work packages if h_resource_id is participant
            else return resources  profiles if h_resource_id is project manager

 5 = returns only one row for this particuler ID
 10 = returns a list of project for responsible or (published + participants)
 */

/* **************************************************************************************************************************************************** */

/* Templets

 31 = Projects/Deliverables/Tasks View all levels   ( type = any value)*/

/* ***************************************************************************************************************************************************** */

/*
 CRI-s  *type = 4 - Chang Request, 5 - Risk, 6 - Issue
 33 = Projects witch have CRI of TYPE_ID = *type    ____________________ LEVEL 0
            {call SP_WBS_VIEW(33,null,'PMO_SUPERVISOR',null,null,0,4,0)}
 34 = CRI-s of TYPE_ID = *type, attached to PROJECT_ID ______________ LEVEL 1
            {call SP_WBS_VIEW(34,'PROJECT_ID',null,null,null,0,4,0)}
 89 (SC_WORKFLOW_ATTACHMENT) = select parent list of workflow's attachment
            {CALL SP_WBS_VIEW(89,'element_id','user_id',null,null,0,0,0)} for project view
            {CALL SP_WBS_VIEW(89,'element_id','user_id',null,null,0,2,0)} for deliverable view
 64 (SC_PORTFOLIO) = return only one row for the passed id (tmt_portfolios)
            {CALL SP_WBS_VIEW(64,'element_id','user_id',null,null,0,0,0)} select root portfolios only
 SC_WBS_PORTFOLIO_VIEW (SC_PORTFOLIO) = select portfolios for the login user
            {CALL SP_WBS_VIEW(SC_WBS_PORTFOLIO_VIEW,null,'user_id',null,null,0,0,0)} select root portfolios only
            {CALL SP_WBS_VIEW(SC_WBS_PORTFOLIO_VIEW,'element_id','user_id',null,null,0,0,0)} select portfolios under the element

 active_record parameter used to control the following filters options
            1st bit used to indicate the active record:
                0:  excludes deleted and locked rows
                1:  includes deleted and locked rows

            2nd bit used to indicate my rows or all rows:
                0:  all rows
                1:  my rows only

            3th bit used to indicate expand all:
                0:  return next level only
                1:  return all level under the current level

            4th bit used to control resultset for filters:
                0:  Flat list (include children)
                1:  Flat list (exclude children)
                ex. d1
                        d11
                            d111
                    d2
                        d21
                if the logon user has the right to see d1, d111 and d21, the resultset will be
                0:  d1
                    d111
                    d21

                1:  d1
                    d21

            5th bit NOT USED

            6th bit used to indicate include/exclude Closed items                               (32)
                0:  exclude Closed items
                1:  include Closed items

            7th bit used to indicate include/exclude Completed items                            (64)
                0:  exclude Completed items
                1:  include Completed items

            8th bit NOT USED

            9th bit used to indicate include/exclude Templated items                            (256)
                0:  exclude Templated items
                1:  include Templated items

            12th bit: Deliverable Manager Counter                                               (2048)

*/

/*******************************************************************************************************************************************************/

Prompt == FUNCTION SP_WBS_VIEW ==
CREATE OR REPLACE FUNCTION SP_WBS_VIEW
(
     p_sp_id PLS_INTEGER
   , p_ids VARCHAR2
   , p_resource_id VARCHAR2
   , p_name VARCHAR2
   , p_filter_string VARCHAR2
   , p_filter_count PLS_INTEGER
   , p_type PLS_INTEGER
   , p_active_record PLS_INTEGER
   , p_stage_id PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   WBS_RETURN_COLUMNS VARCHAR2(9999);
   h_search_option PLS_INTEGER := 0;
   h_filter_string VARCHAR2(1024);
   h_full_name VARCHAR2(255);
   h_id VARCHAR2(32);
   h_ids VARCHAR2(4000);
   h_level VARCHAR2(4);
   h_name VARCHAR2(8000);
   h_project_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_resource_id VARCHAR2(32);
   h_str_filter_count VARCHAR2(4);
   h_str_level_id VARCHAR2(1);
   participant001 VARCHAR2(19);
   h_active_record PLS_INTEGER := 0;
   h_filter_count PLS_INTEGER := 0;
   h_sp_id PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_xcount PLS_INTEGER := 0;
   h_prj_var_condition VARCHAR2(1024);
   h_stage_id PLS_INTEGER := 0;
   h_executive_security VARCHAR2(32);
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_type_id PLS_INTEGER := 0;
   h_security_id VARCHAR2(32);
   h_security_ids VARCHAR2(4000);
   h_res_row_rank VARCHAR2(200);
   h_parent_id VARCHAR2(32);
   h_tpf_current_rank VARCHAR2(200);
   h_project_name VARCHAR2(255);
   h_start_finish_date DATE;
   h_effort_driven PLS_INTEGER := 0;
   h_pure_rank VARCHAR2(200);
   h_security_num PLS_INTEGER := 0;
   h_int_x PLS_INTEGER := 0;
   h_minute_per_day PLS_INTEGER := 0;
   h_security_bitflag PLS_INTEGER := 0;
   h_filedName VARCHAR2(254);
   h_keyword_search1 VARCHAR2(8000);
   h_current_id VARCHAR2(32);
   myStmt VARCHAR2(29999);
   dynamicSql VARCHAR2(1999);
   tempStr VARCHAR2(99);
   typeList VARCHAR2(99);
   idList VARCHAR2(3199);
   h_include_closed_items PLS_INTEGER := 0;
   h_include_completed_items PLS_INTEGER := 0;
   h_tree_type PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_is_single_project PLS_INTEGER := 0;
   h_can_view_pivot PLS_INTEGER := 0;
   h_project_flag PLS_INTEGER := 0;
   h_portfolio_owner_resource_id VARCHAR2(32);
   h_element_id VARCHAR2(32);
   h_tmp_security_ids VARCHAR2(4000);
   h_type_id PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_wbs_search_criteria PLS_INTEGER := 0;
   h_total PLS_INTEGER;
   searchUnderFolder PLS_INTEGER NOT NULL := 0;
   searchInPortConfig PLS_INTEGER NOT NULL := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR CRITICAL_CHILD IS
         SELECT RANK, PARENT_ID
         FROM PMO_WBS_VIEW
         ORDER BY RANK;

      CURSOR REFRESH_SECURITY IS
         SELECT RPAD(R.SECURITY_ID, 32)
         FROM TMT_WBS W, RESOURCE_ASSIGNMENTS R
         WHERE (SUBSTR(h_rank, 1, CASE WHEN LENGTH(W.RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(W.RANK) END) = W.RANK 
         AND (W.RANK LIKE SUBSTR(h_rank, 1, 8) || '%'))
         AND W.PROJECT_ID = R.PROJECT_ID
         AND W.ELEMENT_ID = R.ELEMENT_ID
         AND R.RESOURCE_ID = h_resource_id
         ORDER BY RANK;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_WBS_VIEW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_WBS_VIEW'), '-- ' || SYSTIMESTAMP, ':r := SP_WBS_VIEW(' || 
       COALESCE(TO_CHAR(p_sp_id), 'NULL')
          || ', ' || '''' || p_ids || ''''
          || ', ' || '''' || p_resource_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || '''' || p_filter_string || ''''
          || ', ' || COALESCE(TO_CHAR(p_filter_count), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_active_record), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_stage_id), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 109+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_WBS_VIEW(110) : views.sql(2988) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   --timeLog(file,"------------------>BEGIN SP_WBS_VIEW ");
   
   h_sp_id := p_sp_id;
   h_ids := p_ids;
   h_resource_id := p_resource_id;
   h_name := p_name;
   h_filter_string := p_filter_string;
   h_filter_count := p_filter_count;
   h_type := p_type;
   h_active_record := p_active_record;
   h_stage_id := p_stage_id;
   
   tempStr := TO_CHAR(Coalesce(h_type, 0));
   
   BEGIN
      SELECT REPLACE( RTRIM(NICK_NAME), '''', ''''''), EXECUTIVE_SECURITY
      INTO h_full_name, h_executive_security
      FROM RESOURCES
      WHERE RESOURCE_ID = h_resource_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 139+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(140) : views.sql(3018) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   WBS_RETURN_COLUMNS := 
                      'T.REC_USER, (SELECT R.FULL_NAME FROM RESOURCES R WHERE R.RESOURCE_ID = T.REC_USER) AS REC_USER_NAME, '
          || 'T.REC_DATETIME, T.LAST_CHECKIN, T.CONSTRAINT_TYPE, T.CONSTRAINT_DT, '
          || 'T.SD, T.START_DT, T.FD, T.FINISH_DT, T.DURATION, T.VARIANCE, T.BASELINE1_VARIANCE, '
          || 'T.FORECAST_VARIANCE, T.ACTUAL_VARIANCE, '
          || 'T.BASELINE1_START, T.BASELINE1_FINISH, T.BASELINE1_DURATION, (T.BASELINE1_EFFORT) AS BASELINE1_EFFORT, '
          || 'T.BASELINE2_START, T.BASELINE2_FINISH, T.BASELINE2_DURATION, (T.BASELINE2_EFFORT) AS BASELINE2_EFFORT, '
          || 'T.BASELINE3_START, T.BASELINE3_FINISH, T.BASELINE3_DURATION, (T.BASELINE3_EFFORT) AS BASELINE3_EFFORT, '
          || 'T.BASELINE3_START_VARIANCE, T.BASELINE3_FINISH_VARIANCE, '
          || '(CASE WHEN LEVEL_INDEX = -1 THEN 0 ELSE T.TOTAL_SLACK-100 END) as LEVEL_TOTAL_SLACK,T.LEVEL_START,T.LEVEL_FINISH,T.LEVEL_DURATION, '
          || 'CASE WHEN LEVEL_STATE = 2 THEN 1 WHEN LEVEL_STATE = 1 THEN 2 WHEN LEVEL_START IS NOT NuLL THEN 3 ELSE -1 END AS LEVEL_FLAG, '
          || 'CASE WHEN T.LEVEL_CONFLICT IS Null THEN 0 ELSE 1 END as LEVEL_HAS_CONFLICT, T.LEVEL_CONFLICT, '
          || 'T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_DURATION, (T.PLANNED_EFFORT) AS PLANNED_EFFORT, 0 AS PLANNED_ROLLUP_BIT, '
          || 'T.PROPOSED_START, T.PROPOSED_FINISH, T.PROPOSED_DURATION, (T.PROPOSED_EFFORT) AS PROPOSED_EFFORT, '
          || 'T.SCHED_START, T.SCHED_FINISH, T.SCHED_DURATION, (T.SCHED_EFFORT) AS SCHED_EFFORT, '
          || 'T.FORECAST_START, T.FORECAST_FINISH, T.FORECAST_DURATION, T.FORECAST_EFFORT, T.ETC, '
          || 'T.ACTUAL_START, T.ACTUAL_FINISH, T.ACTUAL_DURATION, (T.WORK_TODATE) AS ACTUAL_EFFORT, '
          || '((T.SCHED_EFFORT+T.PLANNED_EFFORT)-T.WORK_TODATE) AS REMAINING_EFFORT, '
          || 'CASE WHEN ((T.SCHED_EFFORT+T.PLANNED_EFFORT)-T.WORK_TODATE) = T.ETC THEN 0 ELSE 1 END AS ETC_RED_FLAG, '
          || 'T.MILESTONE_PERCENT, T.WORK_PERCENT, CASE WHEN T.TASK_TYPE = 10 THEN 0 WHEN T.TASK_TYPE = 11 THEN 2 '
          || 'WHEN T.TASK_TYPE = 1 THEN 3 WHEN T.TASK_TYPE = 12 THEN 4 ELSE 1 END AS TASK_TYPE, T.REC_FLAGS, '
          || 'CASE WHEN T.SCHED_EFFORT > 0 THEN Trunc(ROUND(((PMO_ORASTD.TODECIMAL(T.WORK_TODATE, 18, 2)/(T.SCHED_EFFORT + T.PLANNED_EFFORT)) * 10000), 0)) ELSE 0 END AS PERCENT_COMPLETE, '
          || 'T.BASELINE1_BUDGET, T.BASELINE2_BUDGET, T.BASELINE3_BUDGET, T.ESTIMATION_BUDGET, T.ESTIMATION_TOTAL, '
          || 'T.CURRENT_BUDGET, T.FORECAST_BUDGET, (T.FORECAST_BUDGET - T.CURRENT_BUDGET) AS BUDGET_FORECAST_VARIANCE, '
          || 'CASE WHEN T.BASELINE1_BUDGET = 0.00 THEN 0.00 ELSE (T.CURRENT_BUDGET - T.BASELINE1_BUDGET) END AS BASELINE_VARIANCE, '
          || 'T.ACTUAL_BUDGET, (T.CURRENT_BUDGET - T.ACTUAL_BUDGET) AS REMAINING, '
          || 'T.R_BASELINE1_BUDGET, T.R_BASELINE2_BUDGET, T.R_BASELINE3_BUDGET, T.R_ESTIMATION_BUDGET, T.R_ESTIMATION_TOTAL, '
          || 'ESTIMATED_ACTUAL, ESTIMATED_ETC, R_ESTIMATED_ACTUAL, R_ESTIMATED_ETC, S_ESTIMATED_ACTUAL, S_ESTIMATED_ETC, '
          || 'ESTIMATED_HOURS, ESTIMATED_ACTUAL_HOURS, ESTIMATED_HOURS_ETC, R_ESTIMATED_HOURS, R_ESTIMATED_ACTUAL_HOURS, R_ESTIMATED_HOURS_ETC, '
          || 'ESTIMATED_HOURS_BASELINE1, ESTIMATED_HOURS_BASELINE2, ESTIMATED_HOURS_BASELINE3, '
          || 'R_ESTIMATED_HOURS_BASELINE1, R_ESTIMATED_HOURS_BASELINE2, R_ESTIMATED_HOURS_BASELINE3, '
          || 'T.R_CURRENT_BUDGET, T.R_FORECAST_BUDGET, (T.R_FORECAST_BUDGET - T.R_CURRENT_BUDGET) AS R_FORECAST_VARIANCE, '
          || 'CASE WHEN T.R_BASELINE1_BUDGET = 0.00 THEN 0.00 ELSE (T.R_CURRENT_BUDGET - T.R_BASELINE1_BUDGET) END AS R_BASELINE_VARIANCE, '
          || 'T.R_ACTUAL_BUDGET, (T.R_CURRENT_BUDGET - T.R_ACTUAL_BUDGET) AS R_REMAINING, '
          || 'T.S_BASELINE1_BUDGET, T.S_BASELINE2_BUDGET, '
          || 'T.S_BASELINE3_BUDGET, T.S_ESTIMATION_BUDGET, T.S_ESTIMATION_TOTAL, '
          || 'T.S_CURRENT_BUDGET, T.S_FORECAST_BUDGET, (T.S_FORECAST_BUDGET - T.S_CURRENT_BUDGET) AS S_FORECAST_VARIANCE, '
          || 'CASE WHEN T.S_BASELINE1_BUDGET = 0.00 THEN 0.00 ELSE (T.S_CURRENT_BUDGET - T.S_BASELINE1_BUDGET) END AS S_BASELINE_VARIANCE, '
          || 'T.S_ACTUAL_BUDGET, (T.S_CURRENT_BUDGET - T.S_ACTUAL_BUDGET) AS S_REMAINING, '
          || 'T.BAC, T.EAC, T.EV_ETC, T.EAC_FORECAST, T.EV_ETC_FORECAST, (T.BAC - T.EAC) AS VAC, T.BCWS, T.BCWP, T.ACWP, (T.BCWP - T.ACWP) AS CV, (T.BCWP - T.BCWS) AS SV, '
          || 'CASE WHEN T.ACWP = 0.00 THEN 0.00 ELSE PMO_ORASTD.TODECIMAL((T.BCWP / T.ACWP), 18, 2) END AS CPI, '
          || 'CASE WHEN T.BCWS = 0.00 THEN 0.00 ELSE PMO_ORASTD.TODECIMAL((T.BCWP / T.BCWS), 18, 2) END AS SPI, '
          || 'PMO_ORASTD.TODECIMAL((CASE WHEN T.ACWP = 0.00 THEN 0.00 ELSE PMO_ORASTD.TODECIMAL((T.BCWP / T.ACWP), 18, 2) END)*(CASE WHEN T.BCWS = 0.00 THEN 0.00 ELSE PMO_ORASTD.TODECIMAL((T.BCWP / T.BCWS), 18, 2) END), 18, 2) AS CRITICAL_RATIO, '
          || 'T.PUBLISHED, T.PROJECT_ID, T.REF_CODE, '
          || '(CASE WHEN PMO_ORASTD.LOCATE(''.'', T.WBS_CODE, 3) = 0 THEN ''1.0'' ELSE ''1'' || SUBSTR(T.WBS_CODE, PMO_ORASTD.LOCATE(''.'', T.WBS_CODE, 3)) END) AS WBS_CODE, '
          || 'T.REFERENCE_NUMBER, '
          || 'T.MSP_ID, T.PRIORITY, T.STAGE_ID, T.TOOLTIP_DESCR, T.RTF_BITS, '
          || 'CASE WHEN T.RTF_BITS > 0 THEN ''Y'' ELSE ''N'' END AS RTF, '
          || 'CASE WHEN T.CRI_COUNT > 0 THEN ''Y'' ELSE ''N'' END AS EXCEPTIONS, '
          || 'CASE WHEN T.DOC_COUNT > 0 THEN ''Y'' ELSE ''N'' END AS DOCUMENTS, '
          || 'CASE WHEN T.SCORECARD_COUNT > 0 THEN ''Y'' ELSE ''N'' END AS SCORECARDS, '
          || '''N'' AS REQUIREMENTS, '
          || 'T.DEPENDENCIES, T.CALENDAR_ID, '
          || '(SELECT C.ELEMENT_NAME FROM TMT_CALENDARS C WHERE C.ELEMENT_ID = T.CALENDAR_ID) AS CALENDAR_NAME, '
          || 'T.START_FINISH_FLAG, T.FINISH_START_DATE, T.START_FINISH_DURATION, T.UNRESOLVED_DURATION, T.FORECAST_CLOSED, '''' AS FIXED_DURATION, T.CURRENCY_ID, '
          || 'T.BUFFER_PERCENT, T.METHOD, T.PROBABILITY, T.OPPORTUNITY_NAME, T.PURSUIT_START, T.PURSUIT_FINISH, '
          || 'T.EXPECTED_COST, T.EXPECTED_REVENUE, T.EXPECTED_MARGIN, T.EXPECTED_PAYBACK_PERIOD, T.EXPECTED_PAYBACK_MONTH, '
          || '(RTRIM(TO_CHAR(T.MAJOR_REVISION)) || ''.'' || RTRIM(TO_CHAR(T.MINOR_REVISION))) AS REVISION, '
          || ''''' AS REVISION_HISTORY, T.REVISION_TYPE, T.REVISION_IMPACT, T.LAST_WF_TYPE, T.LAST_WF_BY, T.LAST_WF_DT, '
          || 'T.CREATED_DT, T.CLOSED_BY, T.ASSIGN_COUNT, T.ASSIGN_NAMES, T.RISK_EXPOSURE, T.CONSEQUENSE_COST, '
          || 'T.TRANSFER_FLAG, T.CUSTOM_INT_1, '
          || 'T.FS_CODE, T.FS_STATE, T.FS_LABOR_CODE, T.BUBBLE_COLOR, T.BUBBLE_PUBLISHED_DT,T.PREVIOUS_WKF_STEP, T.DEPEND_FLAG , '
          || 'T.ESTIMATED_BASELINE1_START, T.ESTIMATED_BASELINE1_FINISH, T.ESTIMATED_BASELINE1_DURATION, T.ESTIMATED_BASELINE2_START, T.ESTIMATED_BASELINE2_FINISH, T.ESTIMATED_BASELINE2_DURATION, T.ESTIMATED_BASELINE3_START, T.ESTIMATED_BASELINE3_FINISH, T.ESTIMATED_BASELINE3_DURATION, '
          || 'T.ESTIMATED_BASELINE1_BUDGET, T.ESTIMATED_BASELINE2_BUDGET, T.ESTIMATED_BASELINE3_BUDGET, T.R_ESTIMATED_BASELINE1_BUDGET, T.R_ESTIMATED_BASELINE2_BUDGET, T.R_ESTIMATED_BASELINE3_BUDGET, '
          || 'T.S_ESTIMATED_BASELINE1_BUDGET, T.S_ESTIMATED_BASELINE2_BUDGET, T.S_ESTIMATED_BASELINE3_BUDGET, T.SUBPROJECT_COUNT, '
          || 'T.BASELINE1_EST_DT, T.BASELINE1_SCH_DT, T.BASELINE2_EST_DT, T.BASELINE2_SCH_DT, T.BASELINE3_EST_DT, T.BASELINE3_SCH_DT, '
          || 'T.TIMECODE_1_ID, T.TIMECODE_2_ID, T.ASSET_ID, T.ASSET_NAME, T.ASSET_FLAG, COALESCE(T.FINANCIAL_CATEGORY, 6501) AS FINANCIAL_CATEGORY, '
          || 'COALESCE((SELECT ELEMENT_NAME FROM TMT_TIMECODES WHERE ELEMENT_ID = T.TIMECODE_1_ID),'''') AS TIMECODE_1_NAME, '
          || 'COALESCE((SELECT ELEMENT_NAME FROM TMT_TIMECODES WHERE ELEMENT_ID = T.TIMECODE_2_ID),'''') AS TIMECODE_2_NAME, '
          || 'T.MP_STATUS_DT, T.PUBLISHED_BY, '
          || 'CASE WHEN T.TYPE_ID IN (' || PMO_CNST.SC_STR_PROJECT_RESOURCE_TYPES 
               || ') AND (SELECT COUNT(TSK_ASSIGNMENT_ID) FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = T.ELEMENT_ID AND TASK_ID = T.ELEMENT_ID AND (LEVEL_START <> UNRESOLVED_START OR LEVEL_FINISH <> UNRESOLVED_FINISH)) > 0 THEN ''1'' ELSE ''0'' END AS LEVEL_AFFECTED_FLAG, T.ELEMENT_FLAG ';
   
   If (h_name IS NOT NULL) OR (h_filter_string IS NOT NULL) Then
      -- ignore include closed, include completed, and exclude sub element filter
      h_active_record := PMO_FUNC.BIT_OR(Coalesce(h_active_record, 0), PMO_CNST.SC_INCLUDE_CLOSED);
      -- include closed
      h_active_record := PMO_FUNC.BIT_OR(Coalesce(h_active_record, 0), PMO_CNST.SC_INCLUDE_COMPLETED);
      -- include completed
      If (BITAND(Coalesce(
            h_active_record, 0), PMO_CNST.SC_EXCLUDE_SUB_ELEMENT)) = PMO_CNST.SC_EXCLUDE_SUB_ELEMENT Then
         h_active_record := Coalesce(h_active_record, 0) - PMO_CNST.SC_EXCLUDE_SUB_ELEMENT;
         -- remove exclude sub element
      End If;
   End If;
   
   -- search projects by name from a list of project or a list of portfolios or a portfolios
   If h_name IS NOT NULL Then
      If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WBS_TEMPLATE_VIEW Then
         h_filedName := 'T.ELEMENT_NAME(C)';
      Else
         h_filedName := 'T.ELEMENT_NAME(C),T.REF_CODE(C)';
      End If;
      h_search_option := FC_PARSE_SEARCH_STR(h_name, h_filedName, h_keyword_search1);
      
      If Coalesce(h_search_option, 0) = PMO_CNST.SC_FIND_ATTRIBUTE_NAME Then
         h_name := NULL;
         
         --trace_printf("\n h_keyword_search1:%s", h_keyword_search1);
         --trace_printf("\n h_filter_string:%s", h_filter_string);
         
         null_return := FC_FIND_ATTR_BY_NAME(PMO_CNST.SC_TABLE_WBS, h_keyword_search1, h_filter_string
                                    , h_filter_count);
         
         If Coalesce(h_filter_count, 0) = 0 Then
            h_filter_count := 1;
            h_filter_string := PMO_CNST.SC_EMPTY_ATTRIBUTE_ID;
         End If;
      Else
         If Coalesce(Length(h_keyword_search1), 0) = 0 Then
            h_name := NULL;
         Else
            h_name := h_keyword_search1;
         End If;
      End If;
      
      If (Coalesce(h_sp_id, 0) = 1) AND (Coalesce(h_type, 0) = 0) Then
         h_type := 1019;
      ElsIf (Coalesce(h_sp_id, 0) = PMO_CNST.SC_WBS_PORTFOLIO_VIEW) OR (Coalesce(
                h_sp_id, 0) = PMO_CNST.SC_PORTFOLIO_CONFIG_VIEW) Then
         If Coalesce(h_sp_id, 0) = PMO_CNST.SC_PORTFOLIO_CONFIG_VIEW Then
            searchInPortConfig := 1;
         End If;
         
         If (h_ids IS NULL) AND (Coalesce(h_type, 0) = 0) Then
            h_type := 1019;
         ElsIf Coalesce(h_type, 0) = 1000 Then
            h_type := 2019;
         ElsIf Coalesce(h_type, 0) = 999 Then
            h_type := 2019;
            searchUnderFolder := 1;
         End If;
      End If;
   End If;
   
   If Coalesce(h_sp_id, 0) = 5 Then
      h_type := 0;
   End If;
   
   /*********************************************************************/
   -- the following section is for supporting search by name just at root node in wbs tree
   /*********************************************************************/
   -- BEGIN
   h_wbs_search_criteria := PMO_CNST.SC_SEARCH_CHILDREN_ITEMS;
   
   If (searchInPortConfig = 0) AND (Coalesce(h_sp_id, 0) <> PMO_CNST.SC_WBS_TEMPLATE_VIEW) Then
      If (h_filter_string IS NOT NULL) AND (PMO_ORASTD.strcmp(h_filter_string
            , 'EMPTY_TREE_FLAG') = 0) Then
         h_sp_id := 0;
         h_type := 0;
         
         myStmt := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID,';
         myStmt := myStmt || 'T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, ';
         myStmt := myStmt || 
                    '0 AS PARTICIPANT, '''' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         myStmt := myStmt || 'FROM TMT_WBS T';
         myStmt := myStmt || 
                    ' WHERE T.ELEMENT_ID = ''EMPTY_TREE_FLAG'' AND T.PROJECT_ID = ''empty_tree_flag'' ';
         myStmt := myStmt || ' ORDER BY T.RANK ';
      ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH Then
         h_type := 0;
         h_filter_string := NULL;
         h_stage_id := 0;
         h_name := NULL;
      ElsIf h_name IS NOT NULL Then
         h_wbs_search_criteria := PMO_CNST.SC_SEARCH_ALL_ITEMS;
         h_filter_string := NULL;
         h_filter_count := 0;
         h_sp_id := 141;
         h_type := 1019;
         h_ids := NULL;
         h_stage_id := 0;
      ElsIf h_filter_string IS NOT NULL Then
         h_wbs_search_criteria := PMO_CNST.SC_SEARCH_ALL_ITEMS;
         h_sp_id := 141;
         h_type := 1019;
         h_ids := NULL;
         h_stage_id := 0;
      ElsIf Coalesce(h_sp_id, 0) = 2 Then
         h_type := 0;
      End If;
   End If;
   -- END
   /*********************************************************************/
   
   --trace_printf("\n h_type:%d \n", h_type);
   --trace_printf("\n h_sp_id:%d \n", h_sp_id);
   --trace_printf("\n searchUnderFolder:%d \n", searchUnderFolder);
   --trace_printf("\n searchInPortConfig:%d \n", searchInPortConfig);
   --trace_printf("\n h_name:%s:%d \n", h_name, h_name_Ind);
   --trace_printf("\n h_ids:%s:%d \n", h_ids, h_ids_Ind);
   --trace_printf("\n h_filter_string:%s:%d \n", h_filter_string, h_filter_string_Ind);
   --trace_printf("\n h_wbs_search_criteria:%d \n", h_wbs_search_criteria);
   
   If (Coalesce(h_sp_id, 0) = 31) OR (Coalesce(h_sp_id, 0) = 1) OR (Coalesce(h_sp_id, 0) = 2) 
             OR ((Coalesce(h_type, 0) > 1000) AND (Coalesce(h_type, 0) < 5000)) Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_WBS_VIEW';
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   h_current_id := NULL;
   If ((Coalesce(h_type, 0) > 1000) AND (Coalesce(h_type, 0) < 3000)) OR ((Coalesce(h_type, 0) > 5000) 
              AND (Coalesce(h_type, 0) < 7000)) Then
      If Coalesce(h_sp_id, 0) = 2 Then
         If ((Coalesce(h_type, 0) > 2000) AND (Coalesce(h_type, 0) < 3000)) OR ((Coalesce(h_type, 0) > 6000) 
               AND (Coalesce(h_type, 0) < 7000)) Then
            h_type := Coalesce(h_type, 0) - 1000;
         End If;
         
         h_id := h_ids;
         h_current_id := h_id;
         
         h_tree_type := 0;
         h_bitflag := 2;
         null_return := FC_GET_SELECTED_PROJECTS(h_tree_type, h_id, h_resource_id, h_bitflag);
         
         h_is_single_project := 1;
      ElsIf ((Coalesce(h_type, 0) > 2000) AND (Coalesce(h_type, 0) < 3000)) OR ((Coalesce(
                 h_type, 0) > 6000) AND (Coalesce(h_type, 0) < 7000)) Then
         h_type := Coalesce(h_type, 0) - 1000;
         
         h_tree_type := PMO_CNST.SC_TABLE_PORTFOLIO;
         
         h_table_type := PMO_CNST.SC_TABLE_WBS;
         h_executive_security := NULL;
         h_bitflag := FC_GET_ACTIVE_EXECUTIVE_SECURE(h_table_type, h_resource_id, h_executive_security
                                            , h_can_view_pivot);
         
         h_bitflag := Coalesce(h_bitflag, 0);
         
         h_is_single_project := 0;
         null_return := FC_GET_SELECTED_PROJECTS(h_tree_type, h_ids, h_resource_id, h_bitflag);
      ElsIf Coalesce(h_type, 0) > 1000 Then
         h_tree_type := PMO_CNST.SC_TABLE_WBS;
         
         h_table_type := PMO_CNST.SC_TABLE_WBS;
         h_executive_security := NULL;
         h_bitflag := FC_GET_ACTIVE_EXECUTIVE_SECURE(h_table_type, h_resource_id, h_executive_security
                                            , h_can_view_pivot);
         h_bitflag := Coalesce(h_bitflag, 0);
         
         h_is_single_project := 0;
         null_return := FC_GET_SELECTED_PROJECTS(h_tree_type, h_ids, h_resource_id, h_bitflag);
      End If;
      
      If Coalesce(h_sp_id, 0) = 2 Then
         h_sp_id := 302;
         -- next level in portal page
      Else
         h_sp_id := 300;
         -- first level in portal page
      End If;
      
      If (Coalesce(h_type, 0) >= 1100) AND (Coalesce(h_type, 0) <= 1821) Then
         h_temp_sc_state := FC_GET_PRJ_VAR_CONDITION(h_type, h_prj_var_condition);
      End If;
      
      If (Coalesce(h_type, 0) >= 1100) AND (Coalesce(h_type, 0) <= 1821) Then
         If Coalesce(h_sp_id, 0) = 302 Then
            PMO_DB2M.SC_STATE := FC_WBS_VIEW(h_sp_id, h_resource_id, h_name, h_filter_string, h_filter_count
                                 , h_type, h_active_record, h_stage_id, h_is_single_project
                                 , h_wbs_search_criteria, h_prj_var_condition);
         Else
            If (Coalesce(h_type, 0) >= 1100) AND (Coalesce(h_type, 0) >= 1421) Then
               h_int_x := PMO_CNST.SC_BUDGET_VAR_SECURITY_INT;
            ElsIf (Coalesce(h_type, 0) >= 1500) AND (Coalesce(h_type, 0) >= 1821) Then
               h_int_x := PMO_CNST.SC_SCHED_VAR_SECURITY_INT;
            End If;
            -- security value for effort/sched variance
            
            null_return := FC_APPLY_VARIANCE_SECURITY(h_int_x, h_resource_id);
            
            PMO_DB2M.SC_STATE := FC_WBS_VIEW(h_sp_id, h_resource_id, h_name, h_filter_string, h_filter_count
                                 , h_type, h_active_record, h_stage_id, h_is_single_project
                                 , h_wbs_search_criteria, h_prj_var_condition);
         End If;
      Else
         PMO_DB2M.SC_STATE := FC_WBS_VIEW(h_sp_id, h_resource_id, h_name, h_filter_string, h_filter_count
                                 , h_type, h_active_record, h_stage_id, h_is_single_project
                                 , h_wbs_search_criteria, h_prj_var_condition);
      End If;
      
      If (h_name IS NOT NULL) AND (Coalesce(h_search_option, 0) <> PMO_CNST.SC_FIND_ATTRIBUTE_NAME) Then
         h_temp_sc_state := FC_CLEANS_WBS_BY_NAME(h_search_option, h_current_id);
      End If;
      
      null_return := FC_FIX_WBS_VIEW_SECURITY_ID(h_sp_id, h_type, h_resource_id, h_executive_security);
      
      If searchInPortConfig = 1 Then
         myStmt := 
               'SELECT A.LEVEL_ID, T.TYPE_ID, T.RANK, A.PARENT_ID, T.ELEMENT_ID, T.SUBPROJECT_COUNT AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
      Else
         myStmt := 
               'SELECT A.LEVEL_ID, T.TYPE_ID, T.RANK, A.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
      End If;
      
      myStmt := myStmt || ' A.SECURITY_ID AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
      
      myStmt := myStmt || ' CASE WHEN A.ASSIGNED = 1 THEN ''M'' ';
      myStmt := myStmt || 
                    ' ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, A.PROJECT_NAME AS PROJECT_NAME, ';
      
      myStmt := myStmt || 'A.START_FINISH_DATE, A.MINUTE_PER_DAY, A.EFFORT_DRIVEN, ';
      
      If (Coalesce(h_sp_id, 0) = 302) AND (Coalesce(h_type, 0) >= 1100) AND (Coalesce(
             h_type, 0) <= 1821) Then
         myStmt := myStmt || 'CASE WHEN ';
         myStmt := myStmt || h_prj_var_condition;
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_RESOURCE_TYPES;
         myStmt := myStmt || ') THEN 1 ELSE 0 END AS FILTER_FLAG, ';
      Else
         myStmt := myStmt || 'A.FILTER_FLAG, ';
      End If;
      myStmt := myStmt || WBS_RETURN_COLUMNS;
      myStmt := myStmt || ' FROM TMT_WBS T, PMO_WBS_VIEW A WHERE A.ELEMENT_ID = T.ELEMENT_ID';
      
      --      if ((h_sp_id == 300)  (h_type >= 1100)  (h_type <= 1821) )   // project variance filter
      --      {
      --          myStmt.strcat(" AND ");
      --          myStmt.strcat(h_prj_var_condition);
      --      }
      
      If searchUnderFolder = 1 Then
         BEGIN
            UPDATE PMO_WBS_VIEW
            SET LEVEL_ID = LEVEL_ID + 2,
            PARENT_ID = h_ids
            WHERE 1 = 1;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 486+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(487) : views.sql(3365) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         myStmt := myStmt || 
                    ' UNION ALL SELECT 1 AS LEVEL_ID, P.TYPE_ID, ''0000'' AS RANK, P.PARENT_ID, P.ELEMENT_ID, '
                || '1 AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ''SCFOLDERMANAGER'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ''N'' AS ASSIGNED,'''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         myStmt := myStmt || 'FROM TMT_PORTFOLIOS P '
                || 'LEFT JOIN TMT_WBS T ON T.PROJECT_ID = P.ELEMENT_ID AND T.ELEMENT_ID = P.ELEMENT_ID '
                || 'WHERE P.ELEMENT_ID = ''';
         myStmt := myStmt || h_ids;
         myStmt := myStmt || '''';
      End If;
      
      myStmt := myStmt || ' ORDER BY RANK';
      
      --trace_printf("\n  WBS VIEW-------> resultset 001 h_filter_string: %s", (char *)h_filter_string);
      --trace_printf("\n  WBS VIEW-------> resultset 001: %s", (char *)myStmt);
   End If;
   --  else
   --  {
   --      InitializeHost(SC_PARAM(h_id), SC_PARAM(h_ids));
   --  }
   
   --trace_printf("\n h_sp_id:%d", h_sp_id);
   
   CASE h_sp_id
      WHEN 1 THEN
         -- FIRST level
         If (Coalesce(h_type, 0) = 0) OR (Coalesce(h_type, 0) = 3001) Then
            myStmt := 
                  'SELECT 0 AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, A.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
            myStmt := myStmt || ' case when T.TYPE_ID IN (';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || 
                    ') then COALESCE(A.SECURITY_ID,''SCPROJECTVISITOR'') else A.SECURITY_ID end AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || 'CASE WHEN InStr(ASSIGN_NAMES, ''';
            myStmt := myStmt || h_full_name;
            myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
            myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, T.ELEMENT_NAME AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
         Else
            myStmt := 
                  'SELECT 0 AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
            myStmt := myStmt || ' case when T.TYPE_ID IN (';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || 
                    ') then COALESCE(A.SECURITY_ID,''SCPROJECTVISITOR'') else A.SECURITY_ID end AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || 'CASE WHEN InStr(ASSIGN_NAMES, ''';
            myStmt := myStmt || h_full_name;
            myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
            myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, T.ELEMENT_NAME AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
         End If;
         
         If Coalesce(h_type, 0) = 0 Then
            BEGIN
               INSERT INTO PMO_WBS_VIEW (RANK, ELEMENT_ID, ELEMENT_NAME, TRANSFER_FLAG, ELEMENT_TYPE_BITS, PARTICIPANT, SECURITY_ID,  REF_CODE, TYPE_ID, CHILD_COUNT, MILESTONE_PERCENT, T_RANK, BUBBLE_COLOR)
               SELECT T.RANK, T.ELEMENT_ID, T.ELEMENT_NAME, T.TRANSFER_FLAG, A.ELEMENT_TYPE_BITS, A.PARTICIPANT, A.SECURITY_ID, T.REF_CODE, T.TYPE_ID, T.CHILD_COUNT, T.MILESTONE_PERCENT, T.T_RANK, T.BUBBLE_COLOR
               FROM PROJECT_ASSIGNMENTS A, TMT_WBS T
               WHERE T.ELEMENT_ID = A.PROJECT_ID AND T.REC_STATUS <> 'D'
               AND A.RESOURCE_ID = h_resource_id AND T.TYPE_ID <> PMO_CNST.SC_RESOURCE AND
               (BITAND(A.PARTICIPANT, 3) > 0 OR (BITAND(A.PARTICIPANT, 3) = 0 AND T.PUBLISHED = '1'));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 566+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(567) : views.sql(3445) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If h_filter_string IS NULL Then
               BEGIN
                  DELETE FROM PMO_WBS_VIEW T
                  WHERE T.ELEMENT_ID IN (SELECT C.ELEMENT_ID
                  FROM PMO_WBS_VIEW C, PMO_WBS_VIEW P
                  WHERE C.ELEMENT_ID = T.ELEMENT_ID AND LENGTH(P.RANK) < LENGTH(C.RANK)
                  AND ((P.RANK >= SUBSTR(C.RANK, 1, 8) AND P.RANK <= C.RANK) 
                  AND SUBSTR(C.RANK,1, CASE WHEN LENGTH(P.RANK) > LENGTH(C.RANK) THEN 1 ELSE LENGTH(P.RANK) END ) = P.RANK));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 589+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(590) : views.sql(3468) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            myStmt := myStmt || 
                    'FROM TMT_WBS T, PMO_WBS_VIEW A WHERE A.ELEMENT_ID = T.ELEMENT_ID AND T.TYPE_ID <> 11 ';
            
            If h_filter_string IS NOT NULL Then
               h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
               myStmt := myStmt || 
                     ' AND EXISTS(SELECT W.ELEMENT_ID FROM TMT_WBS W WHERE W.PROJECT_ID = T.ELEMENT_ID ';
               myStmt := myStmt || ' AND (SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM';
               myStmt := myStmt || ' TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
               myStmt := myStmt || 
                     ' WHERE CA.ELEMENT_ID = W.ELEMENT_ID AND ((CA.TYPE_ID <> 1) OR (W.PROJECT_ID = W.ELEMENT_ID))';
               myStmt := myStmt || ' AND PA.ELEMENT_ID IN (';
               myStmt := myStmt || h_filter_string;
               myStmt := myStmt || ') ';
               myStmt := myStmt || 'AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ';
               myStmt := myStmt || 'ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
               myStmt := myStmt || h_str_filter_count;
               myStmt := myStmt || ')';
            End If;
            myStmt := myStmt || ' ORDER BY T.RANK ';
            
            --trace_printf("\n  WBS VIEW-------> resultset 002");
         ElsIf Coalesce(h_type, 0) = 3001 Then
            BEGIN
               INSERT INTO PMO_WBS_VIEW (RANK, ELEMENT_ID, PARENT_ID, ELEMENT_NAME, TRANSFER_FLAG, ELEMENT_TYPE_BITS, PARTICIPANT, SECURITY_ID, CHILD_COUNT, MILESTONE_PERCENT, T_RANK, BUBBLE_COLOR)
               SELECT T.RANK, T.ELEMENT_ID, T.PARENT_ID, T.ELEMENT_NAME, T.TRANSFER_FLAG, A.ELEMENT_TYPE_BITS, A.PARTICIPANT, A.SECURITY_ID, T.CHILD_COUNT, T.MILESTONE_PERCENT, T.T_RANK, T.BUBBLE_COLOR
               FROM PROJECT_ASSIGNMENTS A, TMT_WBS T
               WHERE T.ELEMENT_ID = A.PROJECT_ID AND T.REC_STATUS <> 'D'
               AND A.RESOURCE_ID = h_resource_id AND
               (BITAND(A.PARTICIPANT, 3) > 0 OR (BITAND(A.PARTICIPANT, 3) = 0 AND T.PUBLISHED = '1'));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 635+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(636) : views.sql(3514) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            OPEN CRITICAL_CHILD;
            
            LOOP
               FETCH CRITICAL_CHILD INTO h_rank, h_parent_id;
               IF CRITICAL_CHILD%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
               
               BEGIN
                  UPDATE PMO_WBS_VIEW
                  SET CHILD_COUNT = (SELECT COUNT(RANK) FROM TMT_WBS
                  WHERE RANK LIKE h_rank || '%'
                  AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
                  AND LEVEL_INDEX <> -1 AND TOTAL_SLACK = 0),
                  SECURITY_ID = COALESCE((SELECT SECURITY_ID FROM PMO_WBS_VIEW
                  WHERE ELEMENT_ID = h_parent_id), '') || COALESCE(SECURITY_ID, '')
                  WHERE RANK = h_rank;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 672+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(673) : views.sql(3551) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            END LOOP;
            BEGIN
               CLOSE CRITICAL_CHILD ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 685+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(686) : views.sql(3564) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            myStmt := myStmt || 'FROM TMT_WBS T, PMO_WBS_VIEW A WHERE A.ELEMENT_ID = T.ELEMENT_ID ';
            myStmt := myStmt || ' ORDER BY T.RANK ';
            
            --trace_printf("\n  WBS VIEW-------> resultset 003");
         Else
            -- filter by type
            myStmt := myStmt || 'FROM PROJECT_ASSIGNMENTS A, TMT_WBS T ';
            myStmt := myStmt || 
                    'WHERE T.ELEMENT_ID = A.PROJECT_ID AND T.REC_STATUS <> ''D'' AND A.RESOURCE_ID = ''';
            myStmt := myStmt || h_resource_id;
            myStmt := myStmt || ''' AND (BITAND(A.PARTICIPANT, 3) > 0 OR T.PUBLISHED IN (''1'')) ';
            
            CASE h_type
               
               WHEN 223 THEN
                  --deliverable and transferred cri
                  typeList := '(' || PMO_CNST.SC_STR_WBS_DELV_GROUP_TYPES || ') ';
               
               WHEN 224 THEN
                  --tasks and milestone
                  typeList := '(' || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES || ') ';
               
               WHEN 4 THEN
                  --transfer cris
                  typeList := '(36, 37, 39) ';
               
               WHEN 0 THEN
                  --project and proposal
                  typeList := '(' || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES || ') ';
               
               ELSE
                  typeList := '(' || TO_CHAR(Coalesce(h_type, 0)) || ') ';
            END CASE;
            
            h_keyword_search1 := NULL;
            
            If h_filter_string IS NULL Then
               myStmt := myStmt || ' AND EXISTS(SELECT ELEMENT_ID FROM TMT_WBS WHERE TYPE_ID IN ';
               myStmt := myStmt || typeList;
               
               myStmt := myStmt || 
                     ' AND (T.RANK >= SUBSTR(RANK,1,8) AND T.RANK <= RANK AND SUBSTR(RANK, 1, CASE WHEN LENGTH(RANK) < LENGTH(T.RANK) THEN 1 ELSE LENGTH(T.RANK) END) = T.RANK)';
               
               If (BITAND(Coalesce(h_active_record, 0), 2)) = 2 Then
                  tempStr := TO_CHAR(FC_BITS_VALUE(h_type));
                  myStmt := myStmt || ' AND BITAND(A.ELEMENT_TYPE_BITS, ';
                  myStmt := myStmt || tempStr;
                  myStmt := myStmt || ') > 0 ';
               End If;
               
               If Coalesce(h_type, 0) = PMO_CNST.SC_DELIVERABLE Then
                  myStmt := myStmt || ' AND TRANSFER_FLAG IS NuLL) ';
               Else
                  myStmt := myStmt || ') ';
               End If;
            Else
               -- Filter by attribute list
               h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
               
               myStmt := myStmt || 
                     ' AND EXISTS(SELECT W.ELEMENT_ID FROM TMT_WBS W WHERE W.PROJECT_ID = T.ELEMENT_ID ';
               
               myStmt := myStmt || ' AND W.TYPE_ID IN ';
               myStmt := myStmt || typeList;
               
               myStmt := myStmt || ' AND (SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM';
               
               myStmt := myStmt || ' TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
               
               myStmt := myStmt || 
                     ' WHERE CA.ELEMENT_ID = W.ELEMENT_ID AND ((CA.TYPE_ID <> 1) OR (CA.TYPE_ID = 1 AND W.TYPE_ID IN (';
               myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
               myStmt := myStmt || '))) AND PA.ELEMENT_ID IN (';
               myStmt := myStmt || h_filter_string;
               myStmt := myStmt || ') ';
               myStmt := myStmt || 'AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ';
               myStmt := myStmt || 'ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
               myStmt := myStmt || h_str_filter_count;
               myStmt := myStmt || ')';
               
               myStmt := myStmt || ' AND EXISTS(SELECT ELEMENT_ID FROM TMT_WBS WHERE TYPE_ID IN ';
               myStmt := myStmt || typeList;
               myStmt := myStmt || ' AND PROJECT_ID = A.PROJECT_ID) ';
            End If;
            myStmt := myStmt || ' ORDER BY T.RANK ';
            
            --trace_printf("\n  WBS VIEW-------> resultset 004");
         End If;
      
      WHEN 2 THEN
         -- NEXT level
         h_id := h_ids;
         
         If (h_id IS NULL) OR (h_resource_id IS NULL) OR (h_type IS NULL) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 785+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(786) : views.sql(3664) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.INVALID_PARAMS);
         End If;
         
         h_tree_type := 0;
         h_bitflag := 2;
         h_current_id := h_id;
         null_return := FC_GET_SELECTED_PROJECTS(h_tree_type, h_id, h_resource_id, h_bitflag);
         
         h_is_single_project := 1;
         PMO_DB2M.SC_STATE := FC_WBS_VIEW(h_sp_id, h_resource_id, h_name, h_filter_string, h_filter_count
                                 , h_type, h_active_record, h_stage_id, h_is_single_project
                                 , h_wbs_search_criteria, h_prj_var_condition);
         
         If (h_name IS NOT NULL) AND (Coalesce(h_search_option, 0) <> PMO_CNST.SC_FIND_ATTRIBUTE_NAME) Then
            h_temp_sc_state := FC_CLEANS_WBS_BY_NAME(h_search_option, h_current_id);
         End If;
         
         null_return := FC_FIX_WBS_VIEW_SECURITY_ID(h_sp_id, h_type, h_resource_id, h_executive_security);
         
         If (BITAND(Coalesce(h_active_record, 0), 4)) = 4 Then
            myStmt := 
                  'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
            myStmt := myStmt || ' A.SECURITY_ID AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || 'CASE WHEN InStr(ASSIGN_NAMES, ''';
            myStmt := myStmt || h_full_name;
            myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
            myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, A.PROJECT_NAME, ';
            
            --trace_printf("\n  WBS VIEW-------> resultset 005.1 -> ok");
         Else
            myStmt := 
                  'SELECT A.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
            myStmt := myStmt || ' A.SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || 'CASE WHEN InStr(ASSIGN_NAMES, ''';
            myStmt := myStmt || h_full_name;
            myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
            myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, A.PROJECT_NAME, ';
            
            --trace_printf("\n  WBS VIEW-------> resultset 005.2 -> ok");
         End If;
         
         myStmt := myStmt || 'A.START_FINISH_DATE, A.MINUTE_PER_DAY, A.EFFORT_DRIVEN, A.FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         myStmt := myStmt || ' FROM TMT_WBS T, PMO_WBS_VIEW A WHERE A.ELEMENT_ID = T.ELEMENT_ID';
         
         myStmt := myStmt || ' ORDER BY T.RANK';
         
         
      
      WHEN 5 THEN
         -- refresh
         -- All these input parameters are required
         h_id := h_ids;
         
         If (h_id IS NULL) OR (h_resource_id IS NULL) OR (h_type IS NULL) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 845+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(846) : views.sql(3724) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.INVALID_PARAMS);
         End If;
         
         BEGIN
            SELECT RANK, PROJECT_ID, TYPE_ID
            INTO h_rank, h_project_id, h_temp_type_id
            FROM TMT_WBS WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 860+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(861) : views.sql(3739) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_WBS_VIEW', 867+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_WBS_VIEW(868) : views.sql(3746) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_id || '''');
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_PARAMS, h_id);
         End If;
         
         BEGIN
            SELECT REPLACE(ELEMENT_NAME, '''', ''''''), START_FINISH_DATE, MINUTE_PER_DAY, EFFORT_DRIVEN
            INTO h_project_name, h_start_finish_date, h_minute_per_day, h_effort_driven
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 883+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(884) : views.sql(3762) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_security_ids := '';
         
         OPEN REFRESH_SECURITY;
         LOOP
            FETCH REFRESH_SECURITY INTO h_security_id;
            IF REFRESH_SECURITY%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            h_security_ids := h_security_ids || h_security_id;
         END LOOP;
         BEGIN
            CLOSE REFRESH_SECURITY ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 912+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(913) : views.sql(3791) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         If Coalesce(h_type, 0) = 3001 Then
            h_rank := h_rank || '%%';
            myStmt := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, '
                   || '(SELECT COUNT(*) FROM TMT_WBS WHERE RANK LIKE ''' || h_rank || ''' '
                   || 'AND LEVEL_INDEX <> -1 AND TOTAL_SLACK = 0 AND TYPE_ID IN (' 
                        || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES || ')) AS CHILD_COUNT, '
                   || 'T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ''' || h_security_ids 
                        || ''' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '
                   || 'CASE WHEN InStr(ASSIGN_NAMES, ''' || h_full_name || ''' || '';'') > 0 THEN ''M'' '
                   || 'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, ''';
         Else
            myStmt := 
                  'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ''';
            
            h_project_flag := FC_IS_EXECUTIVE_PROJECT(h_project_id, h_resource_id);
            
            If (Coalesce(Length(h_security_ids), 0) = 0) AND isProjectType(Coalesce(
                  h_temp_type_id, 0)) <> 0 Then
               myStmt := myStmt || PMO_CNST.SCPROJECTVISITOR_C_STR;
            Else
               myStmt := myStmt || h_security_ids;
            End If;
            
            If (Coalesce(h_project_flag, 0) > PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT) 
                  AND (h_executive_security IS NOT NULL) Then
               myStmt := myStmt || h_executive_security;
            End If;
            myStmt := myStmt || '''';
            
            myStmt := myStmt || ' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || 'CASE WHEN InStr(ASSIGN_NAMES, ''';
            myStmt := myStmt || h_full_name;
            myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
            myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, ''';
         End If;
         myStmt := myStmt || h_project_name;
         myStmt := myStmt || ''' AS PROJECT_NAME, T.START_FINISH_DATE, ';
         myStmt := myStmt || TO_CHAR(h_minute_per_day);
         myStmt := myStmt || ' AS MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         myStmt := myStmt || 'FROM TMT_WBS T '
                || 'LEFT JOIN RESOURCE_ASSIGNMENTS A '
                || 'ON A.ELEMENT_ID = T.ELEMENT_ID '
                || 'AND A.RESOURCE_ID = ''';
         myStmt := myStmt || h_resource_id;
         myStmt := myStmt || ''' WHERE T.ELEMENT_ID = ''';
         myStmt := myStmt || h_id;
         myStmt := myStmt || ''' AND T.REC_STATUS <> ''D'' ';
         myStmt := myStmt || ' ORDER BY T.RANK ';
         --trace_printf("\n  WBS VIEW-------> resultset 006 -> myStmt:%s", (char *)myStmt);
      WHEN PMO_CNST.SC_WBS_TEMPLATE_VIEW THEN
         -- TEMPLETED all Levels ( PUBLISHED = '2')
         h_id := h_ids;
         
         If (h_id IS NULL) Then
            myStmt := 
                  'SELECT T.LEVEL_ID, T.TYPE_ID, T.T_RANK AS RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, 0 AS PARTICIPANT, ''SCPROJECTWORKFLOWVISITOR'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    'FROM TMT_WBS T WHERE T.RANK = ''00000001'' OR T.RANK = ''00000002'' ORDER BY T.RANK ';
         Else
            BEGIN
               SELECT LEVEL_ID, T_RANK, ELEMENT_ID
               INTO h_level_id, h_rank, h_current_id
               FROM TMT_WBS
               WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 990+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(991) : views.sql(3869) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            myStmt := 'INSERT INTO PMO_WBS_VIEW(PROJECT_ID, ELEMENT_ID, RANK, ELEMENT_NAME)';
            myStmt := myStmt || ' SELECT T.PROJECT_ID, T.ELEMENT_ID, T.RANK, T.ELEMENT_NAME ';
            myStmt := myStmt || 'FROM TMT_WBS T ';
            
            If (h_name IS NOT NULL) Then
               h_rank := h_rank || '%%';
               
               myStmt := myStmt || 'WHERE T.T_RANK LIKE ''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''' AND T.REC_STATUS <> ''D'' ';
               --AND (T.ELEMENT_ID = '");
               --myStmt.strcat(h_id);
               --myStmt.strcat("' OR (");
               --myStmt.strcat(h_keyword_search1);
               --myStmt.strcat("))");
               
               --trace_printf("\n myStmt:%s",(char *) myStmt);
               
               BEGIN
                  EXECUTE IMMEDIATE myStmt;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1025+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1026) : views.sql(3904) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        LFILE.FPUT('AllTrace',                         'SQL : ' || myStmt);
                     END IF; 
                  END;
               END;
               
               myStmt := 'INSERT INTO PMO_WBS_VIEW(PROJECT_ID, ELEMENT_ID, RANK, ELEMENT_NAME)';
               myStmt := myStmt || ' SELECT T.PROJECT_ID, T.ELEMENT_ID, T.RANK, T.ELEMENT_NAME ';
               myStmt := myStmt || 'FROM PMO_WBS_VIEW P, TMT_WBS T ';
               
               -- insert templated item's children if any
               myStmt := myStmt || ' WHERE P.ELEMENT_ID <> ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' ';
               myStmt := myStmt || 
                     ' AND SUBSTR(T.RANK, 1, CASE WHEN LENGTH(T.RANK) > LENGTH(P.RANK) THEN LENGTH(P.RANK) ELSE 1 END ) = P.RANK ';
               myStmt := myStmt || ' AND (';
               myStmt := myStmt || h_keyword_search1;
               myStmt := myStmt || ') ';
               
               --trace_printf("\n myStmt:%s\n",(char *) myStmt);
               
               BEGIN
                  EXECUTE IMMEDIATE myStmt;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1059+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1060) : views.sql(3938) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        LFILE.FPUT('AllTrace',                         'SQL : ' || myStmt);
                     END IF; 
                  END;
               END;
               
               -- delete templated items that dont match in search by name condition
               myStmt := ' DELETE FROM PMO_WBS_VIEW ';
               myStmt := myStmt || ' WHERE ELEMENT_ID <> ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' ';
               myStmt := myStmt || ' AND ELEMENT_ID IN ';
               myStmt := myStmt || ' ( ';
               myStmt := myStmt || 'SELECT T.ELEMENT_ID ';
               myStmt := myStmt || ' FROM PMO_WBS_VIEW T ';
               myStmt := myStmt || ' WHERE ELEMENT_ID <> ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' ';
               myStmt := myStmt || ' AND NOT (';
               myStmt := myStmt || h_keyword_search1;
               myStmt := myStmt || '))';
               
               --trace_printf("\n myStmt:%s\n",(char *) myStmt);
               BEGIN
                  EXECUTE IMMEDIATE myStmt;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1094+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1095) : views.sql(3973) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        LFILE.FPUT('AllTrace',                         'SQL : ' || myStmt);
                     END IF; 
                  END;
               END;
            Else
               If h_filter_string IS NULL Then
                  --trace_printf("\n 1- h_filter_string:%s",h_filter_string);
                  myStmt := myStmt || 'WHERE T.REC_STATUS <> ''D''';
                  myStmt := myStmt || ' AND (T.ELEMENT_ID = ''';
                  myStmt := myStmt || h_id;
                  myStmt := myStmt || ''' OR T.T_PARENT_ID=''';
                  myStmt := myStmt || h_id;
                  myStmt := myStmt || ''')';
               Else
                  --trace_printf("\n 2- h_filter_string:%s",h_filter_string);
                  h_rank := h_rank || '%%';
                  
                  myStmt := myStmt || 'WHERE T.T_RANK LIKE ''';
                  myStmt := myStmt || h_rank;
                  myStmt := myStmt || ''' AND T.REC_STATUS <> ''D'' AND (T.ELEMENT_ID = ''';
                  myStmt := myStmt || h_id;
                  myStmt := myStmt || ''' OR (';
                  
                  h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
                  myStmt := myStmt || ' (SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM';
                  myStmt := myStmt || ' TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
                  myStmt := myStmt || ' WHERE CA.ELEMENT_ID = T.ELEMENT_ID AND PA.ELEMENT_ID IN (';
                  myStmt := myStmt || h_filter_string;
                  myStmt := myStmt || ') ';
                  myStmt := myStmt || 
                        ' AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
                  myStmt := myStmt || h_str_filter_count;
                  myStmt := myStmt || '))';
               End If;
               --trace_printf("\n myStmt:%s",(char *) myStmt);
               
               BEGIN
                  EXECUTE IMMEDIATE myStmt;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1143+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1144) : views.sql(4022) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        LFILE.FPUT('AllTrace',                         'SQL : ' || myStmt);
                     END IF; 
                  END;
               END;
            End If;
            
            If (h_name IS NOT NULL) AND (Coalesce(h_search_option, 0) <> PMO_CNST.SC_FIND_ATTRIBUTE_NAME) Then
               h_temp_sc_state := FC_CLEANS_WBS_BY_NAME(h_search_option, h_current_id);
            End If;
            
            myStmt := 'SELECT CASE WHEN T.ELEMENT_ID = ''' || h_id || ''' THEN T.LEVEL_ID ELSE (' 
                  || TO_CHAR(Coalesce(h_level_id, 0)) || ' + 1) END AS LEVEL_ID, '
                   || 'T.TYPE_ID, T.T_RANK AS RANK, '
                   || 'CASE WHEN T.PUBLISHED=''2'' THEN T.T_PARENT_ID ELSE T.PARENT_ID END AS PARENT_ID, '
                   || 'T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, '
                   || '0 AS PARTICIPANT, ''SCPROJECTWORKFLOWVISITOR'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '
                   || ''''' AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 'FROM TMT_WBS T,  PMO_WBS_VIEW P';
            myStmt := myStmt || ' WHERE T.ELEMENT_ID = P.ELEMENT_ID ';
            myStmt := myStmt || ' ORDER BY T.RANK ';
            
            --trace_printf("\n select myStmt:%s", (char *)myStmt);
         End If;
         --trace_printf("\n  WBS VIEW-------> resultset 007");
         
         
      
      WHEN 32 THEN
         h_id := h_ids;
         myStmt := 
               'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, 0 AS PARTICIPANT, ''SCPROJECTWORKFLOWVISITOR'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         myStmt := myStmt || 'FROM TMT_WBS T WHERE (T.ELEMENT_ID = ''';
         myStmt := myStmt || h_id;
         myStmt := myStmt || ''' OR T.PARENT_ID = ''';
         myStmt := myStmt || h_id;
         myStmt := myStmt || ''') AND T.REC_STATUS <> ''D'' ORDER BY T.RANK ';
         
         
      
      WHEN 33 THEN
         If h_resource_id IS NULL Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 1190+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(1191) : views.sql(4069) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.INVALID_PARAMS);
         End If;
         
         participant001 := TO_CHAR(PMO_CNST.SC_MANAGER);
         tempStr := TO_CHAR(Coalesce(h_type, 0));
         
         myStmt := 
               'SELECT 0 AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CRI_COUNT AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, B.PARTICIPANT, B.SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         myStmt := myStmt || 'FROM (((TMT_WBS T JOIN TMT_CRI I ON I.PROJECT_ID=T.ELEMENT_ID AND I.TYPE_ID=';
         myStmt := myStmt || tempStr;
         myStmt := myStmt || 
                    ')  LEFT JOIN RESOURCE_ASSIGNMENTS B ON B.ELEMENT_ID = T.ELEMENT_ID AND B.RESOURCE_ID=''';
         myStmt := myStmt || h_resource_id;
         myStmt := myStmt || 
                    ''') LEFT JOIN RESOURCE_ASSIGNMENTS A ON A.ELEMENT_ID = T.ELEMENT_ID AND BITAND(A.PARTICIPANT,';
         myStmt := myStmt || participant001;
         myStmt := myStmt || ')>0) ';
         myStmt := myStmt || 'WHERE T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
         myStmt := myStmt || ') AND ((EXISTS(SELECT C.RES_ASSIGNMENT_ID FROM RESOURCE_ASSIGNMENTS C '
                || 'WHERE C.PROJECT_ID=T.ELEMENT_ID AND C.RESOURCE_ID=''';
         myStmt := myStmt || h_resource_id;
         myStmt := myStmt || 
                    ''') AND T.PUBLISHED=''1''  AND T.REC_STATUS <> ''D'') OR B.RES_ASSIGNMENT_ID=A.RES_ASSIGNMENT_ID)';
         
         If h_filter_string IS NOT NULL Then
            h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
            
            myStmt := myStmt || 
                    ' AND ((SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
            myStmt := myStmt || ' WHERE CA.ELEMENT_ID = T.PROJECT_ID AND PA.ELEMENT_ID IN (';
            myStmt := myStmt || h_filter_string;
            myStmt := myStmt || 
                    ') AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
            myStmt := myStmt || h_str_filter_count;
            myStmt := myStmt || ') ';
         End If;
         
         If (BITAND(Coalesce(h_active_record, 0), 1)) = 1 Then
            myStmt := myStmt || ' AND T.REC_STATUS NOT IN (''D'', ''L'') ORDER BY RANK';
         End If;
         
         --trace_printf("\n  WBS VIEW-------> resultset 008");
         
         
      
      WHEN PMO_CNST.SC_WORKFLOW_ATTACHMENT THEN
         -- select parent list of workflow's attachment
         -- select parent list of workflow's attachment
         -- select parent list of workflow's attachment
         -- All these input parameters are required
         h_id := h_ids;
         If (h_id IS NULL) OR (h_resource_id IS NULL OR (p_type IS NULL)) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 1248+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(1249) : views.sql(4127) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.INVALID_PARAMS);
         End If;
         
         participant001 := TO_CHAR(PMO_CNST.SC_MANAGER_RESPONSIBLE);
         
         BEGIN
            SELECT PROJECT_ID, RANK, RTRIM(TO_CHAR(LEVEL_ID + 1))
            INTO h_project_id, h_rank, h_level
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1266+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1267) : views.sql(4145) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               SELECT RANK
               INTO h_rank
               FROM TMT_PORTFOLIOS
               WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1283+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1284) : views.sql(4162) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_type := PMO_CNST.SC_PORTFOLIO_WKF_ATTACHMENT;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_WBS_VIEW', 1292+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_WBS_VIEW(1293) : views.sql(4171) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_id || '''');
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_PARAMS, h_id);
            End If;
         Else
            h_type := PMO_CNST.SC_WBS_ITEM_WKF_ATTACHMENT;
         End If;
         
         If Coalesce(h_type, 0) = PMO_CNST.SC_PORTFOLIO_WKF_ATTACHMENT Then
            myStmt := ' SELECT P.LEVEL_ID, P.TYPE_ID , P.RANK, P.PARENT_ID, P.ELEMENT_ID,  ';
            myStmt := myStmt || 
                    ' P.CHILD_COUNT AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ''SCFOLDERMANAGER'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || ' ''N'' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || ' T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || ' FROM TMT_PORTFOLIOS P ';
            myStmt := myStmt || 
                    ' LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.PROJECT_ID AND P.ELEMENT_ID = T.ELEMENT_ID ';
            
            If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || ' WHERE (SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, 12) = P.RANK OR P.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''')';
               myStmt := myStmt || ' ORDER BY RANK ';
            ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || ' WHERE P.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || '''';
               myStmt := myStmt || ' ORDER BY RANK ';
            Else
               h_sp_id := PMO_CNST.SC_WBS_PORTFOLIO_VIEW;
               
               BEGIN
                  SELECT ELEMENT_ID
                  INTO h_portfolio_owner_resource_id
                  FROM TMT_PORTFOLIOS
                  WHERE RANK = SUBSTR(h_rank, 1, 8);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1337+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1338) : views.sql(4216) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1343+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1344) : views.sql(4222) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_portfolio_owner_resource_id, h_ids
                     , h_active_record, h_full_name, WBS_RETURN_COLUMNS);
            End If;
         ElsIf Coalesce(h_type, 0) = PMO_CNST.SC_WBS_ITEM_WKF_ATTACHMENT Then
            -- Project View
            myStmt := 
                  'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, B.PARTICIPANT, ';
            myStmt := myStmt || 'CASE WHEN ';
            myStmt := myStmt || ' T.ELEMENT_ID = ''';
            myStmt := myStmt || h_id;
            myStmt := myStmt || ''' AND B.SECURITY_ID IS NuLL THEN ';
            myStmt := myStmt || 'CASE WHEN T.TYPE_ID IN ( ';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || ' ) THEN ''SCPROJECTWORKFLOWVISITOR'' WHEN T.TYPE_ID IN ( ';
            myStmt := myStmt || PMO_CNST.SC_STR_WBS_DELV_GROUP_TYPES;
            myStmt := myStmt || ' ) THEN ''SCDELIVERABLEWORKFLOWVISITOR'' ELSE ''SCTASKWORKFLOWVISITOR'' END ';
            myStmt := myStmt || 
                    'ELSE B.SECURITY_ID END AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    'FROM ((TMT_WBS T LEFT JOIN RESOURCE_ASSIGNMENTS B ON B.ELEMENT_ID = T.ELEMENT_ID AND B.RESOURCE_ID = ''';
            myStmt := myStmt || h_resource_id;
            myStmt := myStmt || 
                    ''') LEFT JOIN RESOURCE_ASSIGNMENTS A ON A.ELEMENT_ID = T.ELEMENT_ID AND BITAND(A.PARTICIPANT,';
            myStmt := myStmt || participant001;
            myStmt := myStmt || ')>0) ';
            
            If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || 'WHERE (SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, CASE WHEN LENGTH(T.RANK)>LENGTH(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''') THEN 1 ';
               myStmt := myStmt || 'ELSE LENGTH(T.RANK) END) = T.RANK ';
               myStmt := myStmt || 'AND T.RANK LIKE SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, 8) || ''%'' ) ';
               myStmt := myStmt || ' AND LEVEL_ID > -1 ';
            ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || 'WHERE T.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' ';
            Else
               myStmt := myStmt || 'WHERE (T.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' OR T.PARENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''') ';
            End If;
            
            -- in workflow attachements if project is lock, we still should see the project tree
            --if( (h_active_record  1) == 1)
            --{
            --  myStmt.strcat( " AND T.REC_STATUS NOT IN ('D', 'L')");
            --}
            
            myStmt := myStmt || ' ORDER BY RANK ';
         End If;
         
         --trace_printf("\n  WBS VIEW-------> resultset 009");
      
      WHEN PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT THEN
         -- select parent list of workflow's attachment
         -- select parent list of workflow's attachment
         -- All these input parameters are required
         h_id := h_ids;
         If (h_id IS NULL) OR (h_resource_id IS NULL OR (p_type IS NULL)) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 1414+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(1415) : views.sql(4293) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.INVALID_PARAMS);
         End If;
         
         participant001 := TO_CHAR(PMO_CNST.SC_MANAGER_RESPONSIBLE);
         
         BEGIN
            SELECT PROJECT_ID, RANK, RTRIM(TO_CHAR(LEVEL_ID + 1))
            INTO h_project_id, h_rank, h_level
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1432+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1433) : views.sql(4311) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               SELECT RANK
               INTO h_rank
               FROM TMT_PORTFOLIOS
               WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1449+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1450) : views.sql(4328) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_type := PMO_CNST.SC_PORTFOLIO_WKF_ATTACHMENT;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_WBS_VIEW', 1458+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_WBS_VIEW(1459) : views.sql(4337) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_id || '''');
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_PARAMS, h_id);
            End If;
         Else
            h_type := PMO_CNST.SC_WBS_ITEM_WKF_ATTACHMENT;
         End If;
         
         If Coalesce(h_type, 0) = PMO_CNST.SC_PORTFOLIO_WKF_ATTACHMENT Then
            myStmt := ' SELECT P.LEVEL_ID, P.TYPE_ID , P.RANK, P.PARENT_ID, P.ELEMENT_ID,  ';
            myStmt := myStmt || 
                    ' P.CHILD_COUNT AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ''SCFOLDERMANAGER'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || ' ''N'' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || ' T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || ' FROM TMT_PORTFOLIOS P ';
            myStmt := myStmt || 
                    ' LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.PROJECT_ID AND P.ELEMENT_ID = T.ELEMENT_ID ';
            
            If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || ' WHERE (SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, 12) = P.RANK OR P.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''')';
               myStmt := myStmt || ' ORDER BY RANK ';
            ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || ' WHERE P.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || '''';
               myStmt := myStmt || ' ORDER BY RANK ';
            Else
               h_sp_id := PMO_CNST.SC_WBS_PORTFOLIO_VIEW;
               
               BEGIN
                  SELECT ELEMENT_ID
                  INTO h_portfolio_owner_resource_id
                  FROM TMT_PORTFOLIOS
                  WHERE RANK = SUBSTR(h_rank, 1, 8);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1503+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1504) : views.sql(4382) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1509+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1510) : views.sql(4388) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_portfolio_owner_resource_id, h_ids
                     , h_active_record, h_full_name, WBS_RETURN_COLUMNS);
            End If;
         ElsIf Coalesce(h_type, 0) = PMO_CNST.SC_WBS_ITEM_WKF_ATTACHMENT Then
            -- Project View
            myStmt := 
                  'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, B.PARTICIPANT, ';
            myStmt := myStmt || 'CASE WHEN ';
            myStmt := myStmt || ' T.ELEMENT_ID = ''';
            myStmt := myStmt || h_id;
            myStmt := myStmt || ''' AND B.SECURITY_ID IS NuLL THEN ';
            myStmt := myStmt || 'CASE WHEN T.TYPE_ID IN ( ';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || ' ) THEN ''SCPROJECTWORKFLOWVISITOR'' WHEN T.TYPE_ID IN ( ';
            myStmt := myStmt || PMO_CNST.SC_STR_WBS_DELV_GROUP_TYPES;
            myStmt := myStmt || ' ) THEN ''SCDELIVERABLEWORKFLOWVISITOR'' ELSE ''SCTASKWORKFLOWVISITOR'' END ';
            myStmt := myStmt || 
                    'ELSE B.SECURITY_ID END AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    'FROM ((TMT_WBS T LEFT JOIN RESOURCE_ASSIGNMENTS B ON B.ELEMENT_ID = T.ELEMENT_ID AND B.RESOURCE_ID = ''';
            myStmt := myStmt || h_resource_id;
            myStmt := myStmt || 
                    ''') LEFT JOIN RESOURCE_ASSIGNMENTS A ON A.ELEMENT_ID = T.ELEMENT_ID AND BITAND(A.PARTICIPANT,';
            myStmt := myStmt || participant001;
            myStmt := myStmt || ')>0) ';
            
            If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || 'WHERE (SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, CASE WHEN LENGTH(T.RANK)>LENGTH(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''') THEN 1 ';
               myStmt := myStmt || 'ELSE LENGTH(T.RANK) END) = T.RANK ';
               myStmt := myStmt || 'AND T.RANK LIKE SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, 8) || ''%'' ) ';
               myStmt := myStmt || ' AND LEVEL_ID > -1 ';
            ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || 'WHERE T.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' ';
            Else
               myStmt := myStmt || 'WHERE (T.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' OR T.PARENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''') ';
            End If;
            
            -- in workflow attachements if project is lock, we still should see the project tree
            --if( (h_active_record  1) == 1)
            --{
            --  myStmt.strcat( " AND T.REC_STATUS NOT IN ('D', 'L')");
            --}
            
            myStmt := myStmt || ' ORDER BY RANK ';
         End If;
         
         --trace_printf("\n  WBS VIEW-------> resultset 009");
      
      WHEN PMO_CNST.SC_EXPAND_WORKFLOW_ATTACHMENT THEN
         -- select parent list of workflow's attachment
         -- All these input parameters are required
         h_id := h_ids;
         If (h_id IS NULL) OR (h_resource_id IS NULL OR (p_type IS NULL)) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 1579+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(1580) : views.sql(4458) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.INVALID_PARAMS);
         End If;
         
         participant001 := TO_CHAR(PMO_CNST.SC_MANAGER_RESPONSIBLE);
         
         BEGIN
            SELECT PROJECT_ID, RANK, RTRIM(TO_CHAR(LEVEL_ID + 1))
            INTO h_project_id, h_rank, h_level
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1597+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1598) : views.sql(4476) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               SELECT RANK
               INTO h_rank
               FROM TMT_PORTFOLIOS
               WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1614+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1615) : views.sql(4493) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_type := PMO_CNST.SC_PORTFOLIO_WKF_ATTACHMENT;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_WBS_VIEW', 1623+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_WBS_VIEW(1624) : views.sql(4502) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_id || '''');
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_PARAMS, h_id);
            End If;
         Else
            h_type := PMO_CNST.SC_WBS_ITEM_WKF_ATTACHMENT;
         End If;
         
         If Coalesce(h_type, 0) = PMO_CNST.SC_PORTFOLIO_WKF_ATTACHMENT Then
            myStmt := ' SELECT P.LEVEL_ID, P.TYPE_ID , P.RANK, P.PARENT_ID, P.ELEMENT_ID,  ';
            myStmt := myStmt || 
                    ' P.CHILD_COUNT AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ''SCFOLDERMANAGER'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || ' ''N'' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || ' T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || ' FROM TMT_PORTFOLIOS P ';
            myStmt := myStmt || 
                    ' LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.PROJECT_ID AND P.ELEMENT_ID = T.ELEMENT_ID ';
            
            If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || ' WHERE (SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, 12) = P.RANK OR P.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''')';
               myStmt := myStmt || ' ORDER BY RANK ';
            ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || ' WHERE P.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || '''';
               myStmt := myStmt || ' ORDER BY RANK ';
            Else
               h_sp_id := PMO_CNST.SC_WBS_PORTFOLIO_VIEW;
               
               BEGIN
                  SELECT ELEMENT_ID
                  INTO h_portfolio_owner_resource_id
                  FROM TMT_PORTFOLIOS
                  WHERE RANK = SUBSTR(h_rank, 1, 8);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1668+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1669) : views.sql(4547) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1674+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1675) : views.sql(4553) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_portfolio_owner_resource_id, h_ids
                     , h_active_record, h_full_name, WBS_RETURN_COLUMNS);
            End If;
         ElsIf Coalesce(h_type, 0) = PMO_CNST.SC_WBS_ITEM_WKF_ATTACHMENT Then
            -- Project View
            myStmt := 
                  'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, B.PARTICIPANT, ';
            myStmt := myStmt || 'CASE WHEN ';
            myStmt := myStmt || ' T.ELEMENT_ID = ''';
            myStmt := myStmt || h_id;
            myStmt := myStmt || ''' AND B.SECURITY_ID IS NuLL THEN ';
            myStmt := myStmt || 'CASE WHEN T.TYPE_ID IN ( ';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || ' ) THEN ''SCPROJECTWORKFLOWVISITOR'' WHEN T.TYPE_ID IN ( ';
            myStmt := myStmt || PMO_CNST.SC_STR_WBS_DELV_GROUP_TYPES;
            myStmt := myStmt || ' ) THEN ''SCDELIVERABLEWORKFLOWVISITOR'' ELSE ''SCTASKWORKFLOWVISITOR'' END ';
            myStmt := myStmt || 
                    'ELSE B.SECURITY_ID END AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    'FROM ((TMT_WBS T LEFT JOIN RESOURCE_ASSIGNMENTS B ON B.ELEMENT_ID = T.ELEMENT_ID AND B.RESOURCE_ID = ''';
            myStmt := myStmt || h_resource_id;
            myStmt := myStmt || 
                    ''') LEFT JOIN RESOURCE_ASSIGNMENTS A ON A.ELEMENT_ID = T.ELEMENT_ID AND BITAND(A.PARTICIPANT,';
            myStmt := myStmt || participant001;
            myStmt := myStmt || ')>0) ';
            
            If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || 'WHERE (SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, CASE WHEN LENGTH(T.RANK)>LENGTH(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''') THEN 1 ';
               myStmt := myStmt || 'ELSE LENGTH(T.RANK) END) = T.RANK ';
               myStmt := myStmt || 'AND T.RANK LIKE SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, 8) || ''%'' ) ';
               myStmt := myStmt || ' AND LEVEL_ID > -1 ';
            ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || 'WHERE T.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' ';
            Else
               myStmt := myStmt || 'WHERE (T.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' OR T.PARENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''') ';
            End If;
            
            -- in workflow attachements if project is lock, we still should see the project tree
            --if( (h_active_record  1) == 1)
            --{
            --  myStmt.strcat( " AND T.REC_STATUS NOT IN ('D', 'L')");
            --}
            
            myStmt := myStmt || ' ORDER BY RANK ';
         End If;
         
         --trace_printf("\n  WBS VIEW-------> resultset 009");
      
      WHEN 64 THEN
         -- returns only one row for this particuler ID
         h_id := h_ids;
         myStmt := 
               'SELECT P.LEVEL_ID, P.TYPE_ID, P.RANK, P.PARENT_ID, P.ELEMENT_ID, P.CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ''SCFOLDERMANAGER'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         myStmt := myStmt || 'FROM TMT_PORTFOLIOS P LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.ELEMENT_ID ';
         myStmt := myStmt || 'WHERE P.ELEMENT_ID = ''';
         myStmt := myStmt || h_id;
         myStmt := myStmt || '''';
         
         --trace_printf("\n  WBS VIEW-------> resultset 010");
         
      
      WHEN PMO_CNST.SC_WBS_PORTFOLIO_VIEW THEN
         -- 141 select portfolio view
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1756+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1757) : views.sql(4635) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_resource_id, h_ids, h_active_record
               , h_full_name, WBS_RETURN_COLUMNS);
         
      
      WHEN PMO_CNST.SC_PORTFOLIO_CONFIG_VIEW THEN
         -- 142 select portfolio configuration view
         h_active_record := 96;
         If (h_ids IS NOT NULL) AND (Coalesce(Length(h_ids), 0) = 32) Then
            BEGIN
               SELECT RANK || '%'
               INTO h_res_row_rank
               FROM TMT_WBS
               WHERE ELEMENT_ID = h_ids;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1777+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1778) : views.sql(4656) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_sp_id := 2;
               myStmt := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, ';
               myStmt := myStmt || 
                     'T.SUBPROJECT_COUNT AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
               myStmt := myStmt || 
                     'COALESCE(A.SECURITY_ID, ''SCPROJECTVISITOR'') AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
               myStmt := myStmt || 'CASE WHEN InStr(T.ASSIGN_NAMES, ''';
               myStmt := myStmt || h_full_name;
               myStmt := myStmt || 
                     ''' || '';'') > 0 THEN ''M'' ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, '''' AS PROJECT_NAME, ';
               myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
               myStmt := myStmt || WBS_RETURN_COLUMNS;
               myStmt := myStmt || 
                     ' FROM TMT_WBS T LEFT JOIN PROJECT_ASSIGNMENTS A ON A.PROJECT_ID = T.ELEMENT_ID AND A.RESOURCE_ID = ''';
               myStmt := myStmt || h_resource_id;
               myStmt := myStmt || ''' WHERE (T.ELEMENT_ID = ''';
               myStmt := myStmt || h_ids;
               myStmt := myStmt || ''' OR T.PARENT_ID = ''';
               myStmt := myStmt || h_ids;
               myStmt := myStmt || ''') AND T.TYPE_ID IN (';
               myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
               myStmt := myStmt || ') ORDER BY RANK ';
            Else
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1807+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1808) : views.sql(4686) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_resource_id, h_ids, h_active_record
                     , h_full_name, WBS_RETURN_COLUMNS);
            End If;
         Else
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1814+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1815) : views.sql(4693) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_resource_id, h_ids, h_active_record
                  , h_full_name, WBS_RETURN_COLUMNS);
         End If;
         /*
                     BEGIN_SQL
                         DECLARE GLOBAL TEMPORARY TABLE PMO_TMP_VIEWS_PORT(
                             PARENT_ID   CHAR(32),
                             LEVEL_ID    INTEGER,
                             ELEMENT_ID  CHAR(32),
                             TYPE_ID INTEGER,
                             PARTICIPANT INTEGER,
                             SECURITY_ID VARCHAR(3200),
                             RANK_PORT   VARCHAR(192),
                             RANK        VARCHAR(192)
                         ) WITH REPLACE ON COMMIT PRESERVE ROWS NOT LOGGED IN USRTS32K;
                     END_SQL;
         
                     BEGIN_SQL
                         DECLARE GLOBAL TEMPORARY TABLE PMO_TMP_VIEWS_LEVEL1(
                             PARENT_ID   CHAR(32),
                             CHILD_COUNT INTEGER,
                             SECURITY_ID VARCHAR(3200)
                         ) WITH REPLACE ON COMMIT PRESERVE ROWS NOT LOGGED IN USRTS32K;
                     END_SQL;
         
                     // fetch RANK related to the resource
                     BEGIN_SQL
                         SELECT RANK || '%'
                         INTO HOST(h_res_row_rank)
                         FROM TMT_PORTFOLIOS
                         WHERE ELEMENT_ID = HOST(h_resource_id);
                     END_SQL_CHECK;
         
                     BEGIN_SQL
                         INSERT INTO session.PMO_TMP_VIEWS_PORT (PARENT_ID, LEVEL_ID, ELEMENT_ID, TYPE_ID, PARTICIPANT, SECURITY_ID, RANK_PORT, RANK)
                         SELECT COALESCE(P.PARENT_ID,'ROOT'), COALESCE(P.LEVEL_ID,0), T.ELEMENT_ID, COALESCE(P.TYPE_ID,T.TYPE_ID), A.PARTICIPANT, A.SECURITY_ID, P.RANK, T.RANK
                         FROM PROJECT_ASSIGNMENTS A
                         JOIN TMT_WBS T ON T.PROJECT_ID = A.PROJECT_ID
                             AND T.ELEMENT_ID = A.PROJECT_ID
                             AND T.REC_STATUS <> 'D'
                         LEFT JOIN TMT_PORTFOLIOS P ON P.RANK LIKE HOST(h_res_row_rank)
                             AND P.ELEMENT_ID = T.ELEMENT_ID
                         WHERE A.RESOURCE_ID = HOST(h_resource_id)
                             AND (BIT_AND(A.PARTICIPANT, 3) > 0 OR (BIT_AND(A.PARTICIPANT, 3) = 0 AND T.PUBLISHED ='1' ));
                     END_SQL_CHECK;
         
                     // add resource executive projects to the list
                     InitializeHost(SC_PARAM(h_tree_type), SC_TABLE_ATTRIBUTE);
         
                     InitializeHost(SC_PARAM(h_table_type), SC_TABLE_WBS);
                     Nullify(SC_PARAM(h_executive_security));
                     h_bitflag = FC_GET_ACTIVE_EXECUTIVE_SECURE(SC_PARAM(h_table_type), SC_PARAM(h_resource_id), SC_PARAM(h_executive_security), SC_PARAM(h_can_view_pivot));
                     InitializeHost(SC_PARAM(h_bitflag), h_bitflag);
         
                     if (h_bitflag == 1)
                     {
                         FC_GET_SELECTED_PROJECTS(SC_PARAM(h_tree_type), SC_PARAM(h_ids), SC_PARAM(h_resource_id), SC_PARAM(h_bitflag));
         
                         BEGIN_SQL
                             INSERT INTO session.PMO_TMP_VIEWS_PORT (PARENT_ID, LEVEL_ID, ELEMENT_ID, TYPE_ID, PARTICIPANT, SECURITY_ID, RANK_PORT, RANK)
                             SELECT COALESCE(P.PARENT_ID,'ROOT'), COALESCE(P.LEVEL_ID, 0), RP.ELEMENT_ID, RP.PROJECT_FLAG, 0 AS PARTICIPANT, '' AS SECURITY_ID, P.RANK, RP.RANK
                             FROM session.PMO_TMP_PROJECTS_01 RP
                             LEFT JOIN TMT_PORTFOLIOS P ON P.RANK LIKE HOST(h_res_row_rank)
                                 AND P.ELEMENT_ID = RP.ELEMENT_ID
                             WHERE RP.ELEMENT_ID NOT IN (SELECT ELEMENT_ID FROM session.PMO_TMP_VIEWS_PORT);
                         END_SQL_CHECK;
         
                         // update assigned_executive projects
                         BEGIN_SQL
                             UPDATE session.PMO_TMP_VIEWS_PORT
                             SET TYPE_ID = TYPE_ID + SC_EXECUTIVE_ASSIGNED_PROJECT
                             WHERE ELEMENT_ID IN (SELECT ELEMENT_ID FROM session.PMO_TMP_PROJECTS_01);
                         END_SQL_CHECK;
                     }
         
                     BEGIN_SQL
                         DELETE FROM session.PMO_TMP_VIEWS_PORT
                         WHERE ELEMENT_ID IN (SELECT B.ELEMENT_ID
                             FROM session.PMO_TMP_VIEWS_PORT A, session.PMO_TMP_VIEWS_PORT B
                             WHERE A.RANK < B.RANK AND PMO_LOCATE(A.RANK, B.RANK));
                     END_SQL_CHECK;
         
                     BEGIN_SQL
                         DECLARE C_PORT CURSOR WITH HOLD FOR SELECT PARENT_ID, LEVEL_ID, RANK_PORT, RANK
                         FROM session.PMO_TMP_VIEWS_PORT WHERE TYPE_ID <> 64;
                     END_SQL;
         
                     BEGIN_SQL
                         OPEN C_PORT;
                     END_SQL_CHECK;
         
                     while (1 == 1)
                     {
                         BEGIN_SQL
                             FETCH C_PORT INTO HOST(h_parent_id), HOST(h_level_id), HOST(h_tpf_current_rank), HOST(h_rank);
                         END_SQL_CHECK;
         
                         if (SC_NOSUCCESS) break;
         
                         if ( isNull(SC_NULL_INDICATOR(h_tpf_current_rank)) )
                         {
                             BEGIN_SQL
                                 INSERT INTO session.PMO_TMP_VIEWS_PORT (PARENT_ID, LEVEL_ID, ELEMENT_ID, TYPE_ID, PARTICIPANT, SECURITY_ID, RANK_PORT, RANK)
                                 SELECT HOST(h_parent_id), HOST(h_level_id), T.ELEMENT_ID, T.TYPE_ID, A.PARTICIPANT, A.SECURITY_ID, '', T.RANK
                                 FROM TMT_WBS T
                                 LEFT JOIN PROJECT_ASSIGNMENTS A ON T.ELEMENT_ID = A.PROJECT_ID
                                     AND A.RESOURCE_ID = HOST(h_resource_id)
                                 WHERE T.RANK LIKE HOST(h_rank) || '%'
                                     AND T.RANK > HOST(h_rank)
                                     AND TYPE_ID IN (SC_PROJECT_GROUP_TYPES) AND REC_STATUS <> 'D';
                             END_SQL_CHECK;
                         }
                         else
                         {
                             BEGIN_SQL
                                 INSERT INTO session.PMO_TMP_VIEWS_PORT (PARENT_ID, LEVEL_ID, ELEMENT_ID, TYPE_ID, PARTICIPANT, SECURITY_ID, RANK_PORT, RANK)
                                 SELECT HOST(h_parent_id), HOST(h_level_id), T.ELEMENT_ID, T.TYPE_ID, A.PARTICIPANT, A.SECURITY_ID, '', HOST(h_tpf_current_rank) || T.RANK
                                 FROM TMT_WBS T
                                 LEFT JOIN PROJECT_ASSIGNMENTS A ON T.ELEMENT_ID = A.PROJECT_ID
                                     AND A.RESOURCE_ID = HOST(h_resource_id)
                                 WHERE T.RANK LIKE HOST(h_rank) || '%'
                                     AND T.RANK > HOST(h_rank)
                                     AND TYPE_ID IN (SC_PROJECT_GROUP_TYPES) AND REC_STATUS <> 'D';
                             END_SQL_CHECK;
         
                             BEGIN_SQL
                                 UPDATE session.PMO_TMP_VIEWS_PORT
                                 SET RANK = HOST(h_tpf_current_rank)
                                 WHERE RANK = HOST(h_rank);
                             END_SQL_CHECK;
                         }
                     }
         
                     BEGIN_SQL
                         CLOSE C_PORT WITH RELEASE;
                     END_SQL_CHECK;
         
                     BEGIN_SQL
                         INSERT INTO session.PMO_TMP_VIEWS_PORT (PARENT_ID, LEVEL_ID, ELEMENT_ID, TYPE_ID, PARTICIPANT, SECURITY_ID, RANK_PORT, RANK)
                         SELECT PARENT_ID, LEVEL_ID, ELEMENT_ID, TYPE_ID, 0 AS PARTICIPANT, '' AS SECURITY_ID, RANK, RANK
                         FROM TMT_PORTFOLIOS
                         WHERE RANK LIKE HOST(h_res_row_rank) || '%'
                             AND RANK > HOST(h_res_row_rank)
                             AND TYPE_ID = 64
                             AND REC_STATUS <> 'D';
                     END_SQL_CHECK;
         
                     BEGIN_SQL
                         INSERT INTO session.PMO_TMP_VIEWS_LEVEL1 (PARENT_ID, CHILD_COUNT)
                         SELECT PARENT_ID, COUNT(*) AS CHILD_COUNT
                         FROM session.PMO_TMP_VIEWS_PORT
                         WHERE PARENT_ID <> 'ROOT'
                         GROUP BY PARENT_ID;
                     END_SQL_CHECK;
         
                     BEGIN_SQL
                         UPDATE session.PMO_TMP_VIEWS_PORT
                         SET SECURITY_ID = case when type_id in (SC_PROJECT_GROUP_TYPES) and (SECURITY_ID = '' OR SECURITY_ID is Null)
                             then SCPROJECTVISITOR_DB_STR
                             when type_id > SC_EXECUTIVE_ASSIGNED_PROJECT
                             then case when (SECURITY_ID = '' OR SECURITY_ID is Null)
                                 then SCPROJECTVISITOR_DB_STR
                                 else SECURITY_ID end || HOST(h_executive_security)
                             else SECURITY_ID end;
                     END_SQL_CHECK;
         
                     BEGIN_SQL
                         DECLARE PORT_SECURITY_A CURSOR WITH HOLD FOR
                         SELECT DISTINCT PARENT_ID, TYPE_ID, SECURITY_ID
                         FROM session.PMO_TMP_VIEWS_PORT
                         WHERE PARENT_ID <> 'ROOT'
                         ORDER BY PARENT_ID;
                     END_SQL_CHECK;
         
                     BEGIN_SQL
                         OPEN PORT_SECURITY_A;
                     END_SQL_CHECK;
         
                     InitializeHost(SC_PARAM(h_parent_id), "YY");
                     InitializeHost(SC_PARAM(h_security_ids), "");
                     while (1 == 1)
                     {
                         BEGIN_SQL
                             FETCH PORT_SECURITY_A INTO HOST(h_element_id), HOST(h_type_id), HOST(h_tmp_security_ids);
                         END_SQL_CHECK;
         
                         if (!(SC_SUCCESS)) break;
         
                         if (strcmp(h_parent_id, "YY") == 0)
                         {
                             InitializeHost(SC_PARAM(h_parent_id), SC_PARAM(h_element_id));
                             InitializeHost(SC_PARAM(h_security_ids), SC_PARAM(h_tmp_security_ids));
                         }
                         else if (strcmp(h_parent_id, h_element_id) == 0)
                         {
                             if (strstr(h_security_ids, h_tmp_security_ids) == 0)
                             {
                                 if (strlen(h_security_ids) > 0)
                                 {
                                     strcat(h_security_ids, ",");
                                 }
                                 strcat(h_security_ids, h_tmp_security_ids);
                             }
                         }
                         else
                         {
                             BEGIN_SQL
                                 UPDATE session.PMO_TMP_VIEWS_LEVEL1
                                 SET SECURITY_ID = HOST(h_security_ids)
                                 WHERE PARENT_ID = HOST(h_parent_id);
                             END_SQL_CHECK;
         
                             InitializeHost(SC_PARAM(h_parent_id), SC_PARAM(h_element_id));
                             InitializeHost(SC_PARAM(h_security_ids), SC_PARAM(h_tmp_security_ids));
                         }
                     }
                     BEGIN_SQL
                         CLOSE PORT_SECURITY_A;
                     END_SQL_CHECK;
         
                     if (strcmp(h_parent_id, "YY") != 0)
                     {
                         BEGIN_SQL
                             UPDATE session.PMO_TMP_VIEWS_LEVEL1
                             SET SECURITY_ID = HOST(h_security_ids)
                             WHERE PARENT_ID = HOST(h_parent_id);
                         END_SQL_CHECK;
                     }
         
                     BEGIN_SQL
                         DECLARE PORT_SECURITY_B CURSOR WITH HOLD FOR
                         SELECT V.PARENT_ID, P.RANK, COALESCE(V.SECURITY_ID, '')
                         FROM session.PMO_TMP_VIEWS_LEVEL1 V, TMT_PORTFOLIOS P
                         WHERE V.PARENT_ID = P.ELEMENT_ID
                         ORDER BY P.RANK DESC;
                     END_SQL_CHECK;
         
                     BEGIN_SQL
                         OPEN PORT_SECURITY_B;
                     END_SQL_CHECK;
         
                     while (1 == 1)
                     {
                         BEGIN_SQL
                             FETCH PORT_SECURITY_B INTO HOST(h_element_id), HOST(h_rank), HOST(h_security_ids);
                         END_SQL_CHECK;
         
                         if (!(SC_SUCCESS)) break;
         
                         BEGIN_SQL
                             DECLARE PORT_SECURITY_C CURSOR WITH HOLD FOR
                             SELECT V.SECURITY_ID
                             FROM session.PMO_TMP_VIEWS_LEVEL1 V, TMT_PORTFOLIOS P
                             WHERE V.PARENT_ID = P.ELEMENT_ID
                                 AND SUBSTR(P.RANK, 1, LENGTH(P.RANK)-4) = HOST(h_rank);
                         END_SQL_CHECK;
         
                         BEGIN_SQL
                             OPEN PORT_SECURITY_C;
                         END_SQL_CHECK;
         
                         InitializeHost(SC_PARAM(h_count), 0);
                         while (1 == 1)
                         {
                             BEGIN_SQL
                                 FETCH PORT_SECURITY_C INTO HOST(h_tmp_security_ids);
                             END_SQL_CHECK;
         
                             if (!(SC_SUCCESS)) break;
         
                             h_count++;
                             if (strlen(h_security_ids) > 0)
                             {
                                 strcat(h_security_ids, ",");
                             }
                             strcat(h_security_ids, h_tmp_security_ids);
                         }
                         BEGIN_SQL
                             CLOSE PORT_SECURITY_C;
                         END_SQL_CHECK;
         
                         if (h_count > 0)
                         {
                             BEGIN_SQL
                                 UPDATE session.PMO_TMP_VIEWS_LEVEL1
                                 SET SECURITY_ID = HOST(h_security_ids)
                                 WHERE PARENT_ID = HOST(h_element_id);
                             END_SQL_CHECK;
                         }
                     }
                     BEGIN_SQL
                         UPDATE session.PMO_TMP_VIEWS_PORT
                         SET TYPE_ID = TYPE_ID - SC_EXECUTIVE_ASSIGNED_PROJECT
                         WHERE TYPE_ID > SC_EXECUTIVE_ASSIGNED_PROJECT;
                     END_SQL_CHECK;
         
                     BEGIN_SQL
                         CLOSE PORT_SECURITY_B;
                     END_SQL_CHECK;
         
                     myStmt.strcpy("SELECT 0 AS LEVEL_ID, T.TYPE_ID, 'qqqqqqq2' || RTRIM(T.RANK) AS RANK, T.PARENT_ID, T.ELEMENT_ID, ");
                     myStmt.strcat("0 AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ");
                     myStmt.strcat(" case when T.TYPE_ID IN (");
                     myStmt.strcat(SC_STR_PROJECT_GROUP_TYPES);
                     myStmt.strcat(") then COALESCE(A.SECURITY_ID,'SCPROJECTVISITOR') else A.SECURITY_ID end AS SECURITY_ID, '' AS PORTFOLIO_SECURITY, ");
                     myStmt.strcat("CASE WHEN POSSTR(T.ASSIGN_NAMES, '");
                     myStmt.strcat(h_full_name);
                     myStmt.strcat("' || ';') > 0 THEN 'M' ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN 'A' ELSE 'N' END) END AS ASSIGNED, '' AS PROJECT_NAME, ");
                     myStmt.strcat("T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ");
                     myStmt.strcat(WBS_RETURN_COLUMNS);
                     myStmt.strcat(" FROM session.PMO_TMP_VIEWS_PORT A JOIN TMT_WBS T ON T.PROJECT_ID = A.ELEMENT_ID AND T.ELEMENT_ID = A.ELEMENT_ID ");
                     myStmt.strcat(" WHERE A.PARENT_ID = 'ROOT' ");
         
                     myStmt.strcat("UNION ALL ");
         
                     myStmt.strcat("SELECT tpf.LEVEL_ID, tpf.TYPE_ID, 'qqqqqqq1' || RTRIM(A.RANK) AS RANK, tpf.PARENT_ID, tpf.ELEMENT_ID, COALESCE(L1.CHILD_COUNT,0) AS CHILD_COUNT, ");
                     myStmt.strcat("tpf.ELEMENT_NAME, tpf.DELETED_COUNT, tpf.REC_STATUS, A.PARTICIPANT, ");
                     myStmt.strcat(" case when tpf.TYPE_ID IN (");
                     myStmt.strcat(SC_STR_PROJECT_GROUP_TYPES);
                     myStmt.strcat(") then COALESCE(A.SECURITY_ID,'SCPROJECTVISITOR') else A.SECURITY_ID end AS SECURITY_ID, L1.SECURITY_ID AS PORTFOLIO_SECURITY, ");
                     myStmt.strcat(" 'N' AS ASSIGNED, '' AS PROJECT_NAME, ");
                     myStmt.strcat("T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ");
                     myStmt.strcat(WBS_RETURN_COLUMNS);
                     myStmt.strcat(" FROM session.PMO_TMP_VIEWS_PORT A LEFT JOIN session.PMO_TMP_VIEWS_LEVEL1 L1 ON L1.PARENT_ID = A.ELEMENT_ID ");
                     myStmt.strcat("JOIN TMT_PORTFOLIOS tpf ON tpf.RANK LIKE '");
                     myStmt.strcat(h_res_row_rank);
                     myStmt.strcat("' AND tpf.ELEMENT_ID = A.ELEMENT_ID ");
                     myStmt.strcat("LEFT JOIN TMT_WBS T ON T.PROJECT_ID = A.ELEMENT_ID AND T.ELEMENT_ID = A.ELEMENT_ID ");
                     myStmt.strcat("WHERE A.TYPE_ID = 64 ");
         
                     myStmt.strcat("UNION ALL ");
         
                     myStmt.strcat("SELECT A.LEVEL_ID, A.TYPE_ID, 'qqqqqqq1' || RTRIM(A.RANK) AS RANK, A.PARENT_ID, T.ELEMENT_ID, ");
                     myStmt.strcat("L1.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, ");
                     myStmt.strcat("A.PARTICIPANT, ");
         
                     myStmt.strcat(" case when A.TYPE_ID IN (");
                     myStmt.strcat(SC_STR_PROJECT_GROUP_TYPES);
                     myStmt.strcat(") then COALESCE(A.SECURITY_ID,'SCPROJECTVISITOR') else A.SECURITY_ID end AS SECURITY_ID, '' AS PORTFOLIO_SECURITY, ");
         
                     myStmt.strcat("CASE WHEN POSSTR(T.ASSIGN_NAMES, '");
                     myStmt.strcat(h_full_name);
                     myStmt.strcat("' || ';') > 0 ");
                     myStmt.strcat("THEN 'M' ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN 'A' ELSE 'N' END) END AS ASSIGNED, '' AS PROJECT_NAME, ");
                     myStmt.strcat("T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ");
                     myStmt.strcat(WBS_RETURN_COLUMNS);
                     myStmt.strcat(" FROM session.PMO_TMP_VIEWS_PORT A JOIN session.PMO_TMP_VIEWS_LEVEL1 L1 ON L1.PARENT_ID = A.PARENT_ID ");
                     myStmt.strcat("JOIN TMT_WBS T ON T.PROJECT_ID = A.ELEMENT_ID AND T.ELEMENT_ID = A.ELEMENT_ID ");
                     myStmt.strcat(" WHERE A.PARENT_ID <> 'ROOT' AND A.TYPE_ID <> 64");
                     myStmt.strcat(" ORDER BY RANK ASC ");
         */
         --trace_printf("\n  WBS VIEW-------> resultset 014");
      
      WHEN 300 THEN
         -- first level of portal page
         NULL;
      
      WHEN 302 THEN
         -- next level of portal page
         NULL;
      
      WHEN 0 THEN
         -- return empty tree
         NULL;
      
      ELSE
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 2183+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(2184) : views.sql(5062) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.INVALID_PARAMS);
   END CASE;
   
   --timeLog(file,"------------------>BEGIN LAST SELECT STMT IN SP_WBS_VIEW ");
   
   If (Coalesce(h_sp_id, 0) <> PMO_CNST.SC_WBS_PORTFOLIO_VIEW) AND (Coalesce(
             h_sp_id, 0) <> PMO_CNST.SC_WBS_PORTFOLIO_VIEW) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 2193+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(2194) : views.sql(5072) : ' || SYSTIMESTAMP);
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 2196+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(2197) : views.sql(5075) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMODEFINES.PMO_SELECT_READ(myStmt);
   End If;
   
   --timeLog(file,"------------------>END   LAST SELECT STMT IN SP_WBS_VIEW ");
   
   --timeLog(file,"------------------>END   SP_WBS_VIEW ");
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 2209+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(2210) : views.sql(5088) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 2221+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(2222) : views.sql(5100) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_WBS_VIEW;
ALTER FUNCTION SP_WBS_VIEW COMPILE DEBUG;

/****************************************************************************************
db2 drop procedure SP_CRITERIA_SRCH
db2 create procedure SP_CRITERIA_SRCH(in type_id integer, in parent_id varchar(32), in filter_string varchar(50), in filter_type intger) external name '%procdir%pmoffice!SP_CRITERIA_SRCH' language c deterministic parameter style general with nulls

filter_type
    0: LIKE '%h_filter_string%'
    1: SOUNDEX '%h_filter_string%'
***************************************************************************************/

Prompt == FUNCTION SP_CRITERIA_SRCH ==
CREATE OR REPLACE FUNCTION SP_CRITERIA_SRCH
(
     p_type_id PLS_INTEGER
   , p_parent_id VARCHAR2
   , p_short_filter_string VARCHAR2
   , p_filter_type PLS_INTEGER
   , p_resource_id VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rank_like VARCHAR2(200);
   h_short_filter_string VARCHAR2(100);
   h_str_level_id VARCHAR2(1);
   h_table_name VARCHAR2(100);
   h_filter_type PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_sql VARCHAR2(2048);
-- Declare Cursors
      -- Resultset Cursor Declaration
      SCP661e PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661a PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661b PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661f PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661c PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661d PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_CRITERIA_SRCH') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_CRITERIA_SRCH'), '-- ' || SYSTIMESTAMP, ':r := SP_CRITERIA_SRCH(' || 
       COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_short_filter_string || ''''
          || ', ' || COALESCE(TO_CHAR(p_filter_type), 'NULL')
          || ', ' || '''' || p_resource_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 50+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_CRITERIA_SRCH(51) : views.sql(5174) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If p_resource_id IS NULL Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_CRITERIA_SRCH', 57+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_CRITERIA_SRCH(58) : views.sql(5181) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.INVALID_PARAMS);
   End If;
   
   h_type_id := p_type_id;
   h_filter_type := p_filter_type;
   h_parent_id := p_parent_id;
   h_short_filter_string := p_short_filter_string;
   
   If h_short_filter_string IS NULL Then
      h_short_filter_string := '';
   End If;
   
   CASE h_type_id
      WHEN PMO_CNST.SC_ORGANIZATION THEN
         --in case of updating this case update also SC_ORGANIZATION: case SC_GROUP: case SC_DEPARTMENT:case SC_OFFICE
         h_table_name := 'TMT_ORGANIZATIONS';
         
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 85+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(86) : views.sql(5209) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 98+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(99) : views.sql(5222) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_GROUP THEN
         --in case of updating this case update also SC_ORGANIZATION: case SC_GROUP: case SC_DEPARTMENT:case SC_OFFICE
         h_table_name := 'TMT_ORGANIZATIONS';
         
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 118+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(119) : views.sql(5242) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 131+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(132) : views.sql(5255) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_DEPARTMENT THEN
         --in case of updating this case update also SC_ORGANIZATION: case SC_GROUP: case SC_DEPARTMENT:case SC_OFFICE
         h_table_name := 'TMT_ORGANIZATIONS';
         
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 151+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(152) : views.sql(5275) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 164+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(165) : views.sql(5288) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      
      WHEN PMO_CNST.SC_OFFICE THEN
         --in case of updating this case update also SC_ORGANIZATION: case SC_GROUP: case SC_DEPARTMENT:case SC_OFFICE
         h_table_name := 'TMT_ORGANIZATIONS';
         
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 183+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(184) : views.sql(5307) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 196+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(197) : views.sql(5320) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_GEOGRAPHY THEN
         --in case of updating this case update also SC_GEOGRAPHY: SC_REGION: SC_COUNTRY: SC_STATE_PROVINCE: SC_CITY: SC_LOCATION:
         h_table_name := 'TMT_GEOGRAPHIES';
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 215+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(216) : views.sql(5339) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 228+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(229) : views.sql(5352) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_REGION THEN
         --in case of updating this case update also SC_GEOGRAPHY: SC_REGION: SC_COUNTRY: SC_STATE_PROVINCE: SC_CITY: SC_LOCATION:
         h_table_name := 'TMT_GEOGRAPHIES';
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 247+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(248) : views.sql(5371) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 260+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(261) : views.sql(5384) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_COUNTRY THEN
         --in case of updating this case update also SC_GEOGRAPHY: SC_REGION: SC_COUNTRY: SC_STATE_PROVINCE: SC_CITY: SC_LOCATION:
         h_table_name := 'TMT_GEOGRAPHIES';
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 279+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(280) : views.sql(5403) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 292+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(293) : views.sql(5416) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_STATE_PROVINCE THEN
         --in case of updating this case update also SC_GEOGRAPHY: SC_REGION: SC_COUNTRY: SC_STATE_PROVINCE: SC_CITY: SC_LOCATION:
         h_table_name := 'TMT_GEOGRAPHIES';
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 311+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(312) : views.sql(5435) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 324+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(325) : views.sql(5448) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_CITY THEN
         --in case of updating this case update also SC_GEOGRAPHY: SC_REGION: SC_COUNTRY: SC_STATE_PROVINCE: SC_CITY: SC_LOCATION:
         h_table_name := 'TMT_GEOGRAPHIES';
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 343+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(344) : views.sql(5467) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 356+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(357) : views.sql(5480) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_LOCATION THEN
         --in case of updating this case update also SC_GEOGRAPHY: SC_REGION: SC_COUNTRY: SC_STATE_PROVINCE: SC_CITY: SC_LOCATION:
         h_table_name := 'TMT_GEOGRAPHIES';
         h_table_name := 'TMT_GEOGRAPHIES';
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 376+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(377) : views.sql(5500) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 389+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(390) : views.sql(5513) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_POOLS THEN
         If p_parent_id IS NULL Then
            If Coalesce(h_filter_type, 0) = 2 Then
               h_short_filter_string := h_short_filter_string || '%%';
               
               OPEN SCP661e FOR SELECT
               Trunc(h_level_id) AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
               T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               T.DELETED_COUNT, T.REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS T
               WHERE T.TYPE_ID NOT IN (0, 13)
               AND T.REC_STATUS <> 'D'
               AND UPPER(T.ELEMENT_NAME) LIKE UPPER(h_short_filter_string)
               ORDER BY RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 418+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(419) : views.sql(5542) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN SCP661e;
            ElsIf Coalesce(h_filter_type, 0) = 1 Then
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_VIEWS_0002';
               PMO_DB2M.SC_STATE := 0;
               
               BEGIN
                  INSERT INTO PMO_TMP_VIEWS_0002
                  SELECT ELEMENT_ID
                  FROM TMT_POOLS
                  WHERE RANK LIKE '0000' || '%'
                  AND PUBLIC_FLAG = '1'
                  UNION ALL
                  SELECT TP.ELEMENT_ID
                  FROM RESOURCE_ASSIGNMENTS RA, TMT_POOLS TP
                  WHERE TP.ELEMENT_ID = RA.ELEMENT_ID
                  AND RA.RESOURCE_ID = p_resource_id
                  AND RA.ELEMENT_TYPE = 11
                  AND TP.RANK LIKE '0000' || '%'
                  AND TP.TYPE_ID = 11;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 450+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(451) : views.sql(5574) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               OPEN SCP661a FOR SELECT
               Trunc(h_level_id) AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
               T.CHILD_COUNT - (   SELECT COUNT(ELEMENT_ID)
               FROM TMT_POOLS
               WHERE RANK LIKE '0000' || '%'
               AND PARENT_ID = T.ELEMENT_ID
               AND TYPE_ID = 13) AS CHILD_COUNT,
               T.DELETED_COUNT, T.REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS T
               JOIN PMO_TMP_VIEWS_0002 ON PMO_TMP_VIEWS_0002.ELEMENT_ID=T.ELEMENT_ID
               WHERE T.RANK LIKE '0000' || '%'
               AND T.TYPE_ID NOT IN (0, 13)
               AND T.REC_STATUS <> 'D'
               AND SOUNDEX(T.ELEMENT_NAME) = SOUNDEX(h_short_filter_string)
               ORDER BY T.RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 478+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(479) : views.sql(5602) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN SCP661a;
            Else
               h_short_filter_string := h_short_filter_string || '%%';
               
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_VIEWS_0002';
               PMO_DB2M.SC_STATE := 0;
               
               BEGIN
                  INSERT INTO PMO_TMP_VIEWS_0002
                  SELECT ELEMENT_ID
                  FROM TMT_POOLS
                  WHERE PUBLIC_FLAG = '1'
                  UNION ALL
                  SELECT TP.ELEMENT_ID
                  FROM TMT_POOLS TP
                  JOIN RESOURCE_ASSIGNMENTS RA
                  ON TP.ELEMENT_ID=RA.ELEMENT_ID
                  AND RA.ELEMENT_TYPE = 11
                  AND RA.RESOURCE_ID = p_resource_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 510+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(511) : views.sql(5634) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               OPEN SCP661b FOR SELECT
               Trunc(h_level_id) AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
               T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               T.DELETED_COUNT, T.REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS T
               JOIN PMO_TMP_VIEWS_0002 ON PMO_TMP_VIEWS_0002.ELEMENT_ID=T.ELEMENT_ID
               WHERE T.TYPE_ID NOT IN (0, 13)
               AND T.REC_STATUS <> 'D'
               AND UPPER(T.ELEMENT_NAME) LIKE UPPER(h_short_filter_string)
               ORDER BY RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 533+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(534) : views.sql(5657) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN SCP661b;
            End If;
         Else
            If Coalesce(h_filter_type, 0) = 2 Then
               h_short_filter_string := h_short_filter_string || '%%';
               
               OPEN SCP661f FOR SELECT
               LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
               CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               DELETED_COUNT, REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS
               WHERE ELEMENT_ID = h_parent_id
               UNION ALL
               SELECT
               LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
               CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               DELETED_COUNT, REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS
               WHERE TYPE_ID NOT IN (0, 13)
               AND PARENT_ID = h_parent_id
               AND REC_STATUS <> 'D'
               AND UPPER(ELEMENT_NAME) LIKE UPPER(h_short_filter_string)
               ORDER BY RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 573+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(574) : views.sql(5697) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN SCP661f;
            ElsIf Coalesce(h_filter_type, 0) = 1 Then
               OPEN SCP661c FOR SELECT
               LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
               CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               DELETED_COUNT, REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS
               WHERE ELEMENT_ID = h_parent_id
               UNION ALL
               SELECT
               LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
               CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               DELETED_COUNT, REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS
               WHERE TYPE_ID NOT IN (0, 13)
               AND PARENT_ID = h_parent_id
               AND REC_STATUS <> 'D'
               AND SOUNDEX(ELEMENT_NAME) = SOUNDEX(h_short_filter_string)
               ORDER BY RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 609+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(610) : views.sql(5733) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN SCP661c;
            Else
               h_short_filter_string := h_short_filter_string || '%%';
               
               OPEN SCP661d FOR SELECT
               LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
               CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               DELETED_COUNT, REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS
               WHERE ELEMENT_ID = h_parent_id
               UNION ALL
               SELECT
               LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
               CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               DELETED_COUNT, REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS
               WHERE TYPE_ID NOT IN (0, 13)
               AND PARENT_ID = h_parent_id
               AND REC_STATUS <> 'D'
               AND UPPER(ELEMENT_NAME) LIKE UPPER(h_short_filter_string)
               ORDER BY RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 647+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(648) : views.sql(5771) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN SCP661d;
            End If;
         End If;
         
      
      WHEN PMO_CNST.SC_ROLE_CATEGORY THEN
         --in case of updating this case update also SC_ROLE: SC_ROLE_CATEGORY: SC_ROLE_CLASSIFICATION: SC_ROLE_LEVEL:
         h_table_name := 'TMT_COMPETENCIES';
         
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 667+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(668) : views.sql(5791) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 680+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(681) : views.sql(5804) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_ROLE_CLASSIFICATION THEN
         --in case of updating this case update also SC_ROLE: SC_ROLE_CATEGORY: SC_ROLE_CLASSIFICATION: SC_ROLE_LEVEL:
         h_table_name := 'TMT_COMPETENCIES';
         
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 700+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(701) : views.sql(5824) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 713+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(714) : views.sql(5837) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_ROLE THEN
         --in case of updating this case update also SC_ROLE: SC_ROLE_CATEGORY: SC_ROLE_CLASSIFICATION: SC_ROLE_LEVEL:
         h_table_name := 'TMT_COMPETENCIES';
         
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 733+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(734) : views.sql(5857) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 746+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(747) : views.sql(5870) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_ROLE_LEVEL THEN
         --in case of updating this case update also SC_ROLE: SC_ROLE_CATEGORY: SC_ROLE_CLASSIFICATION: SC_ROLE_LEVEL:
         h_table_name := 'TMT_COMPETENCIES';
         
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 766+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(767) : views.sql(5890) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 779+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(780) : views.sql(5903) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_SKILL_CATEGORY THEN
         --in case of updating this case update also SC_SKILL_CATEGORY: case SC_SKILL_CLASSIFICATION: case SC_SKILL:case SC_SKILL_LEVEL
         h_table_name := 'TMT_COMPETENCIES';
         
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 799+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(800) : views.sql(5923) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 812+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(813) : views.sql(5936) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_SKILL_CLASSIFICATION THEN
         --in case of updating this case update also SC_SKILL_CATEGORY: case SC_SKILL_CLASSIFICATION: case SC_SKILL:case SC_SKILL_LEVEL
         h_table_name := 'TMT_COMPETENCIES';
         
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 832+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(833) : views.sql(5956) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 845+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(846) : views.sql(5969) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_SKILL THEN
         --in case of updating this case update also SC_SKILL_CATEGORY: case SC_SKILL_CLASSIFICATION: case SC_SKILL:case SC_SKILL_LEVEL
         h_table_name := 'TMT_COMPETENCIES';
         
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 865+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(866) : views.sql(5989) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 878+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(879) : views.sql(6002) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_SKILL_LEVEL THEN
         --in case of updating this case update also SC_SKILL_CATEGORY: case SC_SKILL_CLASSIFICATION: case SC_SKILL:case SC_SKILL_LEVEL
         h_table_name := 'TMT_COMPETENCIES';
         
         If p_parent_id IS NULL Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 898+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(899) : views.sql(6022) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 911+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(912) : views.sql(6035) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      ELSE
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_CRITERIA_SRCH', 920+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_CRITERIA_SRCH(921) : views.sql(6044) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.INVALID_PARAMS);
   END CASE;
   
   If Coalesce(h_type_id, 0) <> PMO_CNST.SC_POOLS Then
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CRITERIA_SRCH', 928+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CRITERIA_SRCH(929) : views.sql(6052) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_parent_id || '''');
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, p_parent_id);
      End If;
      
      h_rank_like := h_rank;
      h_rank_like := h_rank_like || '%%';
      
      If p_parent_id IS NOT NULL Then
         h_sql := 
               'SELECT T.LEVEL_ID AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, ';
         --h_sql.strcat("T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM ");
         --h_sql.strcat(h_table_name);
         --h_sql.strcat(" WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID IN (13)) AS CHILD_COUNT, ");
         
         h_sql := h_sql || '(SELECT COUNT(C.ELEMENT_ID) FROM ';
         h_sql := h_sql || h_table_name;
         h_sql := h_sql || ' C WHERE C.PARENT_ID = T.ELEMENT_ID AND C.TYPE_ID NOT IN (';
         h_sql := h_sql || TO_CHAR(PMO_CNST.SC_RESOURCE);
         h_sql := h_sql || ',';
         h_sql := h_sql || TO_CHAR(PMO_CNST.SC_ASSETS);
         h_sql := h_sql || ')) AS CHILD_COUNT, ';
         
         h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, ';
         h_sql := h_sql || ''''' AS DESCRIPTION, '''' AS CREATED_BY, '''' AS CREATED_BY_NAME, ';
         h_sql := h_sql || ''''' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID, ';
         h_sql := h_sql || 
                  '0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,';
         h_sql := h_sql || '0 AS EXPERIENCE_ID, '''' AS LAST_USED_DATETIME, '''' AS LAST_USED_LOCATION, ';
         h_sql := h_sql || '0 AS DEFAULT_FLAG FROM ';
         
         h_sql := h_sql || h_table_name;
         
         If p_parent_id IS NOT NULL Then
            h_sql := h_sql || ' T WHERE T.ELEMENT_ID = ''';
            h_sql := h_sql || h_parent_id;
            h_sql := h_sql || ''' UNION ALL ';
         Else
            h_sql := h_sql || ' T WHERE T.ELEMENT_ID IS NULL UNION ALL ';
         End If;
      End If;
      
      If p_parent_id IS NULL Then
         h_sql := 'SELECT Trunc(RTRIM(''';
      Else
         h_sql := h_sql || 'SELECT Trunc(RTRIM(''';
      End If;
      
      h_str_level_id := TO_CHAR(Coalesce(h_level_id, 0));
      h_sql := h_sql || h_str_level_id;
      h_sql := h_sql || ''')) AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, ';
      
      --h_sql.strcat("T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM ");
      --h_sql.strcat(h_table_name);
      --h_sql.strcat(" WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID IN (13)) AS CHILD_COUNT, ");
      
      h_sql := h_sql || '(SELECT COUNT(C.ELEMENT_ID) FROM ';
      h_sql := h_sql || h_table_name;
      h_sql := h_sql || ' C WHERE C.PARENT_ID = T.ELEMENT_ID AND C.TYPE_ID NOT IN (';
      h_sql := h_sql || TO_CHAR(PMO_CNST.SC_RESOURCE);
      h_sql := h_sql || ',';
      h_sql := h_sql || TO_CHAR(PMO_CNST.SC_ASSETS);
      h_sql := h_sql || ')) AS CHILD_COUNT, ';
      
      h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, ';
      h_sql := h_sql || ''''' AS DESCRIPTION, '''' AS CREATED_BY, '''' AS CREATED_BY_NAME, ';
      h_sql := h_sql || ''''' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID, ';
      h_sql := h_sql || 
                  '0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,';
      h_sql := h_sql || '0 AS EXPERIENCE_ID, '''' AS LAST_USED_DATETIME, '''' AS LAST_USED_LOCATION, ';
      h_sql := h_sql || '0 AS DEFAULT_FLAG FROM ';
      
      h_sql := h_sql || h_table_name;
      h_sql := h_sql || ' T WHERE T.RANK LIKE ''';
      h_sql := h_sql || h_rank_like;
      h_sql := h_sql || ''' AND T.RANK <> ''';
      h_sql := h_sql || h_rank;
      h_sql := h_sql || ''' ';
      
      If Coalesce(h_filter_type, 0) = 1 Then
         h_sql := h_sql || 'AND SOUNDEX(T.ELEMENT_NAME) = SOUNDEX(''';
         h_sql := h_sql || h_short_filter_string;
         h_sql := h_sql || ''') ';
      Else
         h_sql := h_sql || 'AND UPPER(T.ELEMENT_NAME) LIKE UPPER(''';
         h_short_filter_string := h_short_filter_string || '%%';
         h_sql := h_sql || h_short_filter_string;
         h_sql := h_sql || ''') ';
      End If;
      
      h_sql := h_sql || 'AND T.REC_STATUS <> ''D'' ';
      
      If Coalesce(h_type_id, 0) = PMO_CNST.SC_ROLE Then
         h_sql := h_sql || 'AND T.TYPE_ID IN (501, 71, 72, 30, 73) ';
      ElsIf Coalesce(h_type_id, 0) = PMO_CNST.SC_SKILL Then
         h_sql := h_sql || 'AND T.TYPE_ID IN (502, 74, 75, 29, 76) ';
      End If;
      
      h_sql := h_sql || 'AND TYPE_ID NOT IN (';
      h_sql := h_sql || TO_CHAR(PMO_CNST.SC_RESOURCE);
      h_sql := h_sql || ',';
      h_sql := h_sql || TO_CHAR(PMO_CNST.SC_ASSETS);
      h_sql := h_sql || ') ';
      h_sql := h_sql || 'ORDER BY RANK ';
      
      --trace_printf("\n h_sql:%s", (char *)h_sql);
      
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 1036+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(1037) : views.sql(6160) : ' || SYSTIMESTAMP);
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 1039+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(1040) : views.sql(6163) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMODEFINES.PMO_SELECT_READ(h_sql);
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 1048+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(1049) : views.sql(6172) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 1060+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(1061) : views.sql(6184) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_CRITERIA_SRCH;
ALTER FUNCTION SP_CRITERIA_SRCH COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_ARCH_PROJ ==
CREATE OR REPLACE FUNCTION SP_S_ARCH_PROJ
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rec_user VARCHAR2(100);
-- Declare Cursors
      -- Resultset Cursor Declaration
      C_ARCH_PROJ PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_ARCH_PROJ') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_ARCH_PROJ'), '-- ' || SYSTIMESTAMP, ':r := SP_S_ARCH_PROJ(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ARCH_PROJ', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_ARCH_PROJ(21) : views.sql(6221) : ' || SYSTIMESTAMP);
   END IF; 

   OPEN C_ARCH_PROJ FOR SELECT 0 AS LEVEL_ID, TW.TYPE_ID, TW.RANK, TW.PARENT_ID, TW.ELEMENT_ID, TW.ELEMENT_NAME,
   0 AS CHILD_COUNT, 0 AS DELETED_COUNT, TW.REC_STATUS, RSC.FULL_NAME AS PROJECT_MANAGER_NAME,
   (SELECT S.FULL_NAME  FROM RESOURCES S WHERE S.RESOURCE_ID = TW.REC_USER) AS REC_USER ,
   TW.SUBPROJECT_COUNT, TW.REC_DATETIME
   FROM TMT_WBS TW JOIN RESOURCE_ASSIGNMENTS RA
   ON TW.PROJECT_ID = RA.PROJECT_ID
   AND TW.ELEMENT_ID = RA.ELEMENT_ID
   AND BITAND( RA.PARTICIPANT, 1 ) = 1
   JOIN RESOURCES RSC ON RSC.RESOURCE_ID = RA.RESOURCE_ID
   WHERE TW.RANK LIKE '0000' || '%'
   AND TW.PROJECT_ID = TW.ELEMENT_ID
   AND TW.REC_STATUS = 'D'
   AND (((SELECT REC_STATUS FROM TMT_WBS WHERE ELEMENT_ID = TW.PARENT_ID) <> 'D') OR (TW.PARENT_ID IS NuLL))
   ORDER BY TW.RANK;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ARCH_PROJ', 38+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_ARCH_PROJ(39) : views.sql(6239) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN C_ARCH_PROJ;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ARCH_PROJ', 46+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ARCH_PROJ(47) : views.sql(6247) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ARCH_PROJ', 58+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ARCH_PROJ(59) : views.sql(6259) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_ARCH_PROJ;
ALTER FUNCTION SP_S_ARCH_PROJ COMPILE DEBUG;

/**********************************************************/

-- this procedure is just for migration

Prompt == FUNCTION SP_MIGRATE_TPF ==
CREATE OR REPLACE FUNCTION SP_MIGRATE_TPF RETURN PMO_DB2M.REF_CURSOR
AS
   h_resource_id VARCHAR2(32);
   h_child_element_id VARCHAR2(32);
   h_rankBase64 VARCHAR2(8);
   h_tpf_current_rank VARCHAR2(200);
   h_max_rank PLS_INTEGER := 0;
   h_tpf_parent_id VARCHAR2(32);
   h_tpf_element_id VARCHAR2(32);
   h_new_rank VARCHAR2(200);
-- Declare Cursors
      CURSOR res_set IS
         SELECT RESOURCE_ID
         FROM RESOURCES;

      CURSOR ptf_set IS
         SELECT tpf.RANK, tpf.PARENT_ID
         FROM TMT_PORTFOLIOS tpf, RESOURCES res
         WHERE res.RESOURCE_ID = tpf.PARENT_ID;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_MIGRATE_TPF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_MIGRATE_TPF'), '-- ' || SYSTIMESTAMP, ':r := SP_MIGRATE_TPF();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_MIGRATE_TPF', 27+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_MIGRATE_TPF(28) : views.sql(6305) : ' || SYSTIMESTAMP);
   END IF; 

   -- step 1: insert resource records
   
   -- select all the resources in RESOURCES table
   
   OPEN res_set;
   
   -- for each resource record add a new record in TMT_PORTFOLIOS,
   -- this record will be the parent node of portfolios record generated by the user
   LOOP
      FETCH res_set INTO h_resource_id;
      IF res_set%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_PORTFOLIOS', h_max_rank);
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      BEGIN
         INSERT INTO TMT_PORTFOLIOS (RANK,TYPE_ID,LEVEL_ID,ELEMENT_ID,REC_STATUS,CHILD_COUNT)
         VALUES ('0000' || h_rankBase64,PMO_CNST.SC_RESOURCE,0, h_resource_id,'I',0);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 66+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(67) : views.sql(6344) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE res_set ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 80+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(81) : views.sql(6358) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- step 2: change rank of existing portfolios
   
   -- select RANK field of all the portfolios record that its PARENT_ID is in RESOURCES table
   
   OPEN ptf_set;
   
   LOOP
      FETCH ptf_set INTO h_tpf_current_rank, h_tpf_parent_id;
      IF ptf_set%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      -- create a new RANK for selected record with using new parent record
      PMODEFINES.PMO_GET_MAX_RANK('TMT_PORTFOLIOS', h_tpf_parent_id, h_max_rank);
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      -- create new rank for selected record
      BEGIN
         SELECT RANK || h_rankBase64
         INTO h_new_rank
         FROM TMT_PORTFOLIOS
         WHERE ELEMENT_ID = h_tpf_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 119+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(120) : views.sql(6397) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- update RANK and LEVEL_ID of children
      PMODEFINES.PMO_PMO_VERIFY_RANK(h_tpf_current_rank, 0);
      BEGIN
         UPDATE TMT_PORTFOLIOS
         SET LEVEL_ID = LEVEL_ID + 1,
         RANK = case when RANK = h_tpf_current_rank then h_new_rank
         else h_new_rank || SUBSTR(RANK, 9, LENGTH(RANK) - 8) end
         WHERE RANK LIKE h_tpf_current_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 142+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(143) : views.sql(6420) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE ptf_set ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 156+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(157) : views.sql(6434) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 165+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(166) : views.sql(6443) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 177+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(178) : views.sql(6455) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_MIGRATE_TPF;
ALTER FUNCTION SP_MIGRATE_TPF COMPILE DEBUG;

